#include <unistd.h>
#include <string.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <features.h>
#if __GLIBC__ >=2 && __GLIBC_MINOR >= 1
#include <netpacket/packet.h>
#include <net/ethernet.h>
#else
#include <asm/types.h>
#include <linux/if_packet.h>
#include <linux/if_ether.h>
#endif
#include <errno.h>

#include "packet.h"
#include "debug.h"
#include "dhcpd.h"
#include "options.h"

/* read a packet from socket fd */
int get_packet(struct dhcpMessage *packet, int fd)
{
	int bytes;
	int i;
	char broken_vendors[][8] = {
		"MSFT 98",
		""
	};
	char *vendor;

	memset(packet, 0, sizeof(struct dhcpMessage));
	bytes = read(fd, packet, sizeof(struct dhcpMessage));
	if (bytes < 0) {
		DEBUG(LOG_INFO, "couldn't read on listening socket -- ignoring");
		return -1;
	}

	if (ntohl(packet->cookie) != DHCP_MAGIC) {
		LOG(LOG_ERR, "received bogus message -- ignoring");
		return -1;
	}
	DEBUG(LOG_INFO, "oooooh!!! got some!");
	
	if (packet->op == BOOTREQUEST && (vendor = get_option(packet, DHCP_VENDOR))) {
		for (i = 0; broken_vendors[i][0]; i++) {
			if (vendor[OPT_LEN - 2] == (signed) strlen(broken_vendors[i]) &&
			    !strncmp(vendor, broken_vendors[i], vendor[OPT_LEN - 2]) &&
			    !(ntohs(packet->flags) & BROADCAST_FLAG)) {
			    	DEBUG(LOG_INFO, "broken client (%s), forcing broadcast",
			    		broken_vendors[i]);
			    	packet->flags |= htons(BROADCAST_FLAG);
			}
		}
	}
			    	

	return bytes;
}


/* Calculate the length of a packet, and make sure its a multiple of 2 */
static int calc_length(struct dhcpMessage *payload)
{
	int payload_length;
	
	payload_length = sizeof(struct dhcpMessage) - 308;
	payload_length += end_option(payload->options) + 1;
	if (payload_length % 2) {
		payload_length++;
		*((char *) payload + payload_length - 1) = '\0';
	}

	//DEBUG(LOG_INFO, "payload length is %d bytes", payload_length);
	DPRINTF("payload length is %d bytes", payload_length);

	return payload_length;
}


u_int16_t checksum(void *addr, int count)
{
	/* Compute Internet Checksum for "count" bytes
	 *         beginning at location "addr".
	 */
	register int32_t sum = 0;
	u_int16_t *source = (u_int16_t *) addr;

	while( count > 1 )  {
		/*  This is the inner loop */
		sum += *source++;
		count -= 2;
	}

	/*  Add left-over byte, if any */
	if( count > 0 )
		sum += * (unsigned char *) source;

	/*  Fold 32-bit sum to 16 bits */
	while (sum>>16)
		sum = (sum & 0xffff) + (sum >> 16);

	return ~sum;
}


/* Constuct a ip/udp header for a packet, and specify the source and dest hardware address */
int raw_packet(struct dhcpMessage *payload, u_int32_t source_ip, int source_port,
		   u_int32_t dest_ip, int dest_port, char *dest_arp, int ifindex)
{
	int fd;
	int result=0, payload_length = calc_length(payload);
	struct sockaddr_ll dest;
	struct udp_dhcp_packet packet;
	int i;

	if ((fd = socket(PF_PACKET, SOCK_DGRAM, htons(ETH_P_IP))) < 0) {
		DEBUG(LOG_ERR, "socket call failed: %s", __FUNCTION__);
		return -1;
	}
	
	memset(&dest, 0, sizeof(dest));
	memset(&packet, 0, sizeof(packet));
	
	dest.sll_family = AF_PACKET;
	dest.sll_protocol = htons(ETH_P_IP);
	dest.sll_ifindex = ifindex;
	dest.sll_halen = 6;
	memcpy(dest.sll_addr, dest_arp, 6);
	//DPRINTF("dest_arp = %02x:%02x:%02x:%02x:%02x:%02x\n", arp_dest_mac[0], arp_dest_mac[1],arp_dest_mac[2],arp_dest_mac[3],arp_dest_mac[4],arp_dest_mac[5]);
	if (bind(fd, (struct sockaddr *)&dest, sizeof(struct sockaddr_ll)) < 0) {
		DEBUG(LOG_ERR, "bind call failed: %s", __FUNCTION__);
		close(fd);
		return -1;
	}

	packet.ip.protocol = IPPROTO_UDP;
	packet.ip.saddr = source_ip;
	packet.ip.daddr = dest_ip;
	packet.ip.tot_len = htons(sizeof(packet.udp) + payload_length); /* cheat on the psuedo-header */
	packet.udp.source = htons(source_port);
	packet.udp.dest = htons(dest_port);
	packet.udp.len = htons(sizeof(packet.udp) + payload_length);
	memcpy(&(packet.data), payload, payload_length);
	packet.udp.check = checksum(&packet, sizeof(packet.ip) + sizeof(packet.udp) + payload_length);
	packet.ip.tot_len = htons(sizeof(packet.ip) + sizeof(packet.udp) + payload_length);
	packet.ip.ihl = sizeof(packet.ip) >> 2;
	packet.ip.version = IPVERSION;
	packet.ip.ttl = IPDEFTTL;
	packet.ip.check = checksum(&(packet.ip), sizeof(packet.ip));

	result = sendto(fd, &packet, ntohs(packet.ip.tot_len), 0, (struct sockaddr *) &dest, sizeof(dest));
	
	if (result <= 0) {
		DEBUG(LOG_ERR, "write on socket failed: %s", __FUNCTION__);
		return -1;
	}
	close(fd);
	
	/* write and send */
	  for (i = 0; i < sizeof(packet); i++) {
	  	  unsigned char *ptr;

		  ptr = (unsigned char *)&packet;
		  DPRINTF("%02x ", ptr[i]);
	  }
	  DPRINTF("\n");
	  FPRINTF(stderr, "%d The DHCP DISCOVERY packet is send. The matrix is around you!\n", sizeof(packet));
	
	return result;
}


int ttpt_packet()		   
{
	int fd;
	int result;//, payload_length = 0;//calc_length(payload);
	u_char *dst_a_ip;
	//struct sockaddr_ll dest;
	//struct tcp_packet packet;
	struct sockaddr_in dest_addr;
	//int i, send_cnt=0;
	//u_char *src_a_ip, *dst_a_ip;
	//src_a_ip = "10.0.0.137";
	//dst_a_ip = "10.0.0.138";
	//unsigned int len, iphdr_len, tcphdr_len, tcpdata_len;
	//tcp_PseudoHeader ph;
	
	if ((fd = socket(AF_INET, SOCK_STREAM, 0/*htons(ETH_P_IP)*/)) < 0) {
		DEBUG(LOG_ERR, "socket call failed: %s ", __FUNCTION__);
		return -1;
	}

	//strcpy(arp_dest_ip, dst_a_ip);
	dest_addr.sin_family = AF_INET;
	dest_addr.sin_port = htons(1723);
	printf("arp_dest_ip = %s\n", arp_dest_ip);
	//dest_addr.sin_addr.s_addr = inet_addr(arp_dest_ip);
	//system("ifconfig eth0 10.0.0.137");
	inet_aton(arp_dest_ip, &(dest_addr.sin_addr));
	//printf("dst_a_ip = %d.%d.%d.%d\n", dest_addr.sin_addr.s_addr[0], dest_addr.sin_addr.s_addr[1],dest_addr.sin_addr.s_addr[2],dest_addr.sin_addr.s_addr[3]);
	//printf("dst_a_ip = %s\n", dest_addr.sin_addr);
	memset(&(dest_addr.sin_zero), '\0', 8);
	result = connect(fd, (struct sockaddr *)&dest_addr, sizeof(struct sockaddr));
	if(result == -1 ) {
		perror("connect");
		//exit(0);
	}
	close(fd);
#if 0
	memset(&dest, 0, sizeof(dest));
	memset(&packet, 0, sizeof(packet));
	
	dest.sll_family = AF_PACKET;
	dest.sll_protocol = htons(ETH_P_IP);
	dest.sll_ifindex = 0;//client_config.ifindex;//ifindex;
	dest.sll_halen = 6;
	memcpy(dest.sll_addr, arp_sr_mac, 6);
	DPRINTF("destMac = %02x:%02x:%02x:%02x:%02x:%02x\n", arp_dest_mac[0], arp_dest_mac[1],arp_dest_mac[2],arp_dest_mac[3],arp_dest_mac[4],arp_dest_mac[5]);

	/*if (bind(fd, (struct sockaddr *)&dest, sizeof(struct sockaddr_ll)) < 0) {
		DEBUG(LOG_ERR, "bind call failed: %s", __FUNCTION__);
		close(fd);
		return -1;
	}*/
	
        //iphdr_len = (packet.ip.ihl) << 2;
	//tp = (struct tcphdr *)((unsigned char *)packet.ip + iphdr_len);	/* tcp frame pointer */	
	packet.ip.protocol = IPPROTO_TCP;
	//strip((u_char *)&packet.ip.saddr, src_a_ip);
	//strip((u_char *)&packet.ip.daddr, dst_a_ip);
	packet.ip.saddr = inet_addr("192.168.1.2");
	packet.ip.daddr = inet_addr("192.168.1.100");
	packet.ip.tot_len = htons(sizeof(packet.ip) + sizeof(packet.tcp) + payload_length);
	packet.ip.ihl = sizeof(packet.ip) >> 2;
	packet.ip.version = IPVERSION;
	packet.ip.ttl = IPDEFTTL;
	
	
	//packet.tcp.tot_len = htons(sizeof(packet.ip) + htons(sizeof(packet.tcp) + payload_length); /* cheat on the psuedo-header */
	packet.tcp.source = htons(0x4096);//htons(source_port);
	packet.tcp.dest = htons(0x1723);//htons(dest_port);
	packet.tcp.seq = 0;
	//packet.tcp.ihl = sizeof(packet.tcp) >> 2;
	/* send tcp SYN and SEQ to ethx interface through raw socket */
	packet.tcp.syn = 1;//TCP_SYN_SENT;
	//packet.tcp.len = htons(sizeof(packet.tcp));
	//memcpy(&(packet.data), payload, payload_length);

	/* tcp checksum */
	//ph.src = packet.ip.saddr;
	//ph.dst = packet.ip.daddr;
	//ph.mbz = 0;
	//ph.protocol = 0x06;		/* TCP */
	//ph.length = htons(packet.tcp.ihl+packet.tcp.len);
	//packet.tcp.checksum = 0;
	//ph.checksum  = checksum(&packet.tcp, sizeof(packet.ip) + sizeof(packet.tcp) + payload_length);
	packet.tcp.check = checksum(&packet, sizeof(packet.ip) + sizeof(packet.tcp) + payload_length);
	packet.ip.check = 0;
	packet.ip.check = checksum(&(packet.ip), sizeof(packet.ip));

	result = sendto(fd, &packet, sizeof(packet)/*ntohs(packet.ip.tot_len)*/, 0, (struct sockaddr *) &dest, sizeof(dest));
	DPRINTF("result = %d\n", result);
	if (result <= 0) {
		DEBUG(LOG_ERR, "write on socket failed: %s", __FUNCTION__);
		return -1;
	}
	close(fd);
	
	/* write and send */
	  for (i = 0; i < sizeof(packet); i++) {
	  	  unsigned char *ptr;

		  ptr = (unsigned char *)&packet;
		  DPRINTF("%02x ", ptr[i]);
	  }
	  DPRINTF("\n");
	  fprintf(stderr, "%d The packet is send. The matrix is around you!\n", sizeof(packet));
#endif	
	return result;
}

/* Let the kernel do all the work for packet generation */
int kernel_packet(struct dhcpMessage *payload, u_int32_t source_ip, int source_port,
		   u_int32_t dest_ip, int dest_port)
{
	int n = 1;
	int fd, result, payload_length = calc_length(payload);
	struct sockaddr_in client;
	
	if ((fd = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0)
		return -1;
	
	if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (char *) &n, sizeof(n)) == -1)
		return -1;

	memset(&client, 0, sizeof(client));
	client.sin_family = AF_INET;
	client.sin_port = htons(source_port);
	client.sin_addr.s_addr = source_ip;

	if (bind(fd, (struct sockaddr *)&client, sizeof(struct sockaddr)) == -1)
		return -1;

	memset(&client, 0, sizeof(client));
	client.sin_family = AF_INET;
	client.sin_port = htons(dest_port);
	client.sin_addr.s_addr = dest_ip; 

	if (connect(fd, (struct sockaddr *)&client, sizeof(struct sockaddr)) == -1)
		return -1;

	result = write(fd, payload, payload_length);
	close(fd);
	return result;
}	

