/*
* Copyright c                  Realtek Semiconductor Corporation, 2002  
* All rights reserved.
* 
* Program : Switch core table driver rtl8651_tblDrv.cf
* Abstract : 
* Author : Edward Jin-Ru Chen (jzchen@realtek.com.tw)               
* $Id: rtl8651_tblDrv.c,v 1.3 2008/04/22 06:01:31 qjj_qin Exp $
*/
/* Implementation note
1. All information are implemented by host order
2. Expect tblAsicDrv to process the order issue
3. 
*/

/*	@doc RTL8651_TABLEDRIVER_API

	@module rtl8651_tblDrv.c - RTL8651 Home gateway controller table driver API documentation	|
	This document explains the internal and external API interface of the table driver module. Functions with _rtl8651 prefix
	are internal functions and functions with rtl8651_ prefix are external functions.
	@normal Chun-Feng Liu (cfliu@realtek.com.tw) <date>

	Copyright <cp>2003 Realtek<tm> Semiconductor Cooperation, All Rights Reserved.

 	@head3 List of Symbols |
 	Here is a list of all functions and variables in this module.

 	@index | RTL8651_TABLEDRIVER_API
*/

#include "rtl_types.h"
#include "assert.h"
#include "rtl8651_layer2.h"
#include "rtl8651_tblDrv.h"
#include "rtl8651_tblAsicDrv.h"
#include "rtl8651_tblDrvFwdLocal.h"
#include "rtl8651_tblDrvLocal.h"
#include "rtl8651_tblDrvProto.h"
#include "rtl8651_tblDrvPatch.h"
#include "rtl8651_tblDrvStatistic.h"
#include "rtl8651_dos_local.h"
#include "rtl8651_alg_init.h"
#include "assert.h"
#include "asicRegs.h"
#include "asicTabs.h"
#include "rtl_queue.h"
#include "rtl_utils.h"
#ifdef CONFIG_RTL865X_ROMEPERF
#include "../romeperf.h"
#endif
#ifdef CONFIG_RTL865X_IPSEC
#include "rtl8651_ipsec_local.h"
#endif
#ifdef RTL865X_TEST
#include <string.h>
#endif


#define RTL865X_EXP_MAC_ACCESS_CONTROL


// Software specification part
#define RTL8651_ARPARRANGE_NUM		64
#define RTL8651_POLLING_THRESHOLD	5
#define RTL8651_ALGSLOT_THRESHOLD 	3
#if 0  
#define DLOG(level,fmt,args...) if (level==2) rtlglue_printf("%s,%d "fmt,__FILE__,__LINE__,##args);
#define LOGERR rtlglue_printf("BUS TIMEOUT  %s,%d\n",__FILE__,__LINE__);
#define BUS_CHECK\
	if (*(int*)(0xbd012004)&0x200000) {\
		LOGERR\
		*(int*)0xbd012004=0x200000;\
		}
#else
#define DLOG(level,fmt,args...) 
#define BUS_CHECK
#endif

#define DRV_INIT_CHECK(expr) do {\
	if((expr)!=SUCCESS){\
		rtlglue_printf("Error >>> initialize failed at line %d!!!\n", __LINE__);\
			return FAILED;\
	}\
}while(0)


int8 rtl8651_drvProtocolBasedNatEnable		= FALSE;		// the flag indicating whether protocol-based NAT is enabled


#ifdef DROP_NEW_CONNECTION_WHEN_NAT_FULL
 static int32 rtl8651_dropNewConnWhenNatFull=FALSE;
#endif

/*
Store the arrangement status of current ARP table. This information does not write to ASIC but for driver reference
*/

const static uint32 *tcpAgeTimer[TCP_CLOSED_FLOW+1]={ 
	NULL, /* NULL state*/
	NULL, 	/*TCP_LISTEN_FLOW*/
	&rtl8651GlobalControl.tcpFastTimeout, /*TCP_OUTBOUND_SYN_RCVD_FLOW*/
	&rtl8651GlobalControl.tcpFastTimeout, /*TCP_INBOUND_SYN_RCVD_FLOW*/
	NULL, /* No such state */
	 &rtl8651GlobalControl.tcpLongTimeout, /*TCP_ESTABLISHED_FLOW*/
	 &rtl8651GlobalControl.tcpMediumTimeout, /*TCP_OUTBOUND_FIN_RCVD_FLOW*/
	 &rtl8651GlobalControl.tcpMediumTimeout, /*TCP_INBOUND_FIN_RCVD_FLOW*/
	 &rtl8651GlobalControl.tcpFastTimeout /*TCP_CLOSED_FLOW*/
};	

#ifdef CACHE_FIND_NAPT_RESULT
int8 cachedIsTcp;
ipaddr_t cachedIntIp, cachedExtIp, cachedRemIp;
uint16 cachedIntPort, cachedExtPort, cachedRemPort;
struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *cachedPEntry;

void INVALID_FIND_CACHE( void )
{
	cachedIsTcp = 0;
	cachedIntIp = 0;
	cachedIntPort = 0;
	cachedExtIp = 0;
	cachedExtPort = 0;
	cachedRemIp = 0;
	cachedRemPort = 0;
	cachedPEntry = NULL;
}
#endif/*CACHE_FIND_NAPT_RESULT*/

#ifdef CACHE_DOS_COUNT_NAPT_RESULT
uint32 cachedDosTcpCnt, cachedDosUdpCnt;
ipaddr_t cachedDosCntIp;

inline void INVALID_DOS_COUNT_CACHE( void )
{
	cachedDosTcpCnt = 0;
	cachedDosUdpCnt = 0;
	cachedDosCntIp = 0;
}

inline void SET_DOS_COUNT_CACHE( ipaddr_t ip, uint32 tcpCnt, uint32 udpCnt )
{
	cachedDosTcpCnt = tcpCnt;
	cachedDosUdpCnt = udpCnt;
	cachedDosCntIp = ip;
}
#endif /* CACHE_COUNT_NAPT_RESULT */

#ifdef FLUSH_NAPT_OLDEST_CONNECTION2
struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *candidateToBeFreed;
#endif/*FLUSH_NAPT_OLDEST_CONNECTION2*/

#define PPPOE_TIME_TO_HANGUP	30 /* 30 seconds to hang up PPPoE */

int8 rtl8651_tblDrv_Id[] = "$Id: rtl8651_tblDrv.c,v 1.3 2008/04/22 06:01:31 qjj_qin Exp $";
//Public defined data structure data area 

int8 aclMode;		/* ACL Mode=> TRUE: use push/pop; FALSE: use Indexing */

int pppoeDefaultGlobalIpAddr;


//Hardware independent structure data area
rtl8651_tblDrv_global_t	rtl8651GlobalControl;
rtl8651_tblDrv_global_t	rtl8651AsicAge; /* Only age fields are valid. Others are invalid. */
static int32 _gidxRegister[RTL8651_IPTABLE_SIZE];

uint16 * rtl8651NaptIcmpBackwardCollisionCountTable; //Whether Flow Table flows are kept in software due to backward ID collision 
uint16 * rtl8651NaptIcmpFlowNumberTable, *rtl8651NaptIcmpFlowNonAsicNumberTable;
uint32 rtl8651ExistedNaptIcmpFlows;//For avoid interface configuration while flow existed

_rtl8651_sessionHangUp_t rtl8651_sessionHangUp[RTL8651_MAX_DIALSESSION];

rtl8651_tblDrvNaptCounter_t	rtl8651_tblDrvNaptCounter;
rtl8651_tblDrvIcmpCounter_t rtl8651_tblDrvIcmpCounter;
static uint32 naptIpCount, natIpCount, localServerCount;

/* Age out time for ARP */

static uint32 rtEntry;		/* the number of added routing entry */
static int8 naptAutoLearn;	/* TRUE: auto-learn enable; FALSE: disable */ 

/* For wan status				 			 */
int32 *rtl8651_wanStatus;				 			 
int16 _rtl8651_dsid;

urlFiltering_t _urlFilter;
#ifdef RTL865XB_URL_FILTER
int32 _urlFilterAclRuleNum;
#endif

#ifdef RTL865XB_URLFILTER_UNKNOWNURLTYPE_SUPPORT
/* 
     When url keyword string is unknown type (ie. no knowledge of host string or path string), 
     this switch determines the case-sensitive comparison or not of path part of the packet.
*/
static int8 rtl8651_urlStringUnknownType_caseSensitive = 0;
#endif

uint16 rtl8651_gVlanIDforPassthru = RTL8651_PASSTHRU_VLANID_IS_NOT_SET;   /* the VLAN created for passthru */
int32 rtl8651_gRefCountofVlanforPassthru = 0; /* reference count of this VLAN */

/* CONFIG_RTL865XB_EXP_PPPOE_PASSTHRU */
uint8 rtl8651_gPPPoEExternalVlanIndex = RTL8651_VLAN_NUMBER;
ether_addr_t rtl8651_gPPPoEExternalGatewayMAC;
uint16 rtl8651_gPPPoEPassthruVlanID = RTL8651_PPPOEPASSTHRU_VLANID_IS_NOT_SET;


/*CONFIG_RTL865XB_EXP_IPV6_PASSTHRU*/
uint8 rtl8651_gIPv6ExternalVlanIndex = RTL8651_VLAN_NUMBER;
ether_addr_t rtl8651_gIPv6ExternalGatewayMAC;
uint16 rtl8651_gIPv6PassthruVlanID = RTL8651_IPV6PASSTHRU_VLANID_IS_NOT_SET;
uint32 rtl8651_pbv_ipv6passthru = RTL8651_IPV6PASSTHRU_PBVID_IS_NOT_SET;


/*CONFIG_RTL865XB_EXP_IPX_PASSTHRU*/
uint8 rtl8651_gIPXExternalVlanIndex = RTL8651_VLAN_NUMBER;
ether_addr_t rtl8651_gIPXExternalGatewayMAC;
uint16 rtl8651_gIPXPassthruVlanID = RTL8651_IPXPASSTHRU_VLANID_IS_NOT_SET;
uint32 rtl8651_pbv_ipxpassthru = RTL8651_IPXPASSTHRU_PBVID_IS_NOT_SET;


/*CONFIG_RTL865XB_EXP_NETBIOS_PASSTHRU*/
uint8 rtl8651_gNetbiosExternalVlanIndex = RTL8651_VLAN_NUMBER;
ether_addr_t rtl8651_gNetbiosExternalGatewayMAC;
uint16 rtl8651_gNetbiosPassthruVlanID = RTL8651_NETBIOSPASSTHRU_VLANID_IS_NOT_SET;
uint32 rtl8651_pbv_netbiospassthru = RTL8651_NETBIOSPASSTHRU_PBVID_IS_NOT_SET;


/* For Soft Policy Route*/
#define STATICPOLICYSIZE	32
#define DYNAMICPOLICYSIZE	32
#define SOFTICMPTABLE		64
static uint16 global_port=3200;
static uint16 icmp_global_port=3200;
uint32 _rtl8651_DefaultDialSessionId=0;
_rtl8651_tblDrvPolicyRoute_t rtStaticPolicy[STATICPOLICYSIZE];
_rtl8651_tblDrvPolicyRoute_t rtDynamicPolicy[DYNAMICPOLICYSIZE];
static _rtl8651_tblDrvPolicyRoute_t *rtHashTable[DYNAMICPOLICYSIZE];
static _rtl8651_tblDrvPolicyRoute_t *rtDynamicPolicyHeadPtr;
#if 0 // chenyl
static _rtl8651_tblDrvSoftIcmpTable_t softIcmpNaptTable[SOFTICMPTABLE];
#endif
static rtl8651_tblDrvDemandRoute_t rtDemandRouteTable[STATICPOLICYSIZE];

//for Server Port
static int8 extIfName[8]="\0";

#ifdef _RTL_LOGGING
// keep the installed logging function pointer here
rtl8651_loggingFuncPtr_t _pUserLoggingFunc = NULL;
uint32 *_loggingModule; /* changed to a pointer, the buffer will be initialzed in tblDrvInit() */
#endif /* _RTL_LOGGING */

static uint32 _userConfigFastTimeoutValue, _adaptiveTimeoutThreshold; 
static int8 _enableAdaptiveNAPTfastTimeout;
static int8 _enableSmartNaptSwappingByLoad;
static uint32 _smartNaptSwappingThresholdTcp;
static uint32 _smartNaptSwappingThresholdUdp;
static uint32 _smartNaptSwappingThresholdCdrouter;
//static rtl8651_tblDrv_naptTcpUdpFlowEntry_t  *_naptFlowPool;

#ifdef CONFIG_RTL865XB_EXP_INVALID
static int32 _smartNaptCreate;
#endif


static rtl8651_tblDrv_routeTable_t *multiNxtHop;
static rtl8651_tblDrv_networkIntfTable_t *lanSideIpUnnumberedWan;
static rtl8651_tblDrv_networkIntfTable_t *lanSideIpUnnumberedLan;
static rtl8651_ipUnnumbered_ctrlBlock_t rtl8651_ipUnnumberGlobalCtrlBlock;
static rtl8651_tblDrv_natEntry_t lanSideNatMapping;
rtl8651DmzHostStruct_t *rtl8651DmzHost;
extern uint16 *_rtl8651_udpSizeThreshValue;
#ifdef RTL8651_DSID_SUPPORT
uint8 *_rtl8651_acl_drop_nolog=NULL;
static uint32 _rtl8651_pppoe_count=0;
static uint32 _rtl8651_multiple_dsid=0;
#endif
//Function pointer installation variable space

uint32 rtl8651_externalMulticastPortMask = 0;

static uint32 enable4WayHash;
static rtl8651_tblDrv_natEntry_t unnumNatEntry;

//static uint32 qosTotalBandwidth;
rtl8651_tblDrv_mnqueue_sys_t mnqueue_ctrl;
static uint8 mnQ_enable;

static uint32 ipClassCount;
static rtl8651_tblDrv_routeTable_t *cur_rt_t;
ipClassABMonitorTable_t ipClassABMonitorTable[IP_CLASS_MAX_ENTRY];	

/* for Rate Limit Counter Registration */
static rtl8651_tblDrv_rateLimitCounter_t rateLimitCounterRegisterTable[RATELIMIT_COUNTER_RESV_SIZE];

rtl8651_tblDrv_miiTunneling_t tunnel;
ipaddr_t softNaptIp;
int32 (*p_mtuHandler) (struct rtl_pktHdr *);
rtl8651_tblDrv_ipEntry_t *def_napt_t;
int32 _rtl8651_createProtocolBasedVLANforIPv6Passthru(void);
int32 _rtl8651_createProtocolBasedVLANforPPPoEPassthru(void);
int32 _rtl8651_destroyProtocolBasedVLANforPPPoEPassthru(void);
int32 _rtl8651_destroyProtocolBasedVLANforIPv6Passthru(void);


/*==========================================================================================
 * Static Function Declaration
 *==========================================================================================*/
#define STATIC_FUNCTION_DEFINITION
//------------------------------- Phy port -------------------------------------------------
//static void _rtl8651_updateAsicPHY(uint32 port);
//static uint32 _rtl8651_returnPortMaskPortNumber(uint32 portMask);
//------------------------------- L2 Entry -------------------------------------------------
 void _rtl8651_syncL2Entrychange(void *l2entry_t, void *arg);
//static void _rtl8651_arrangeAsicL2Entry(uint16 fid, uint32 l2Index);
rtl8651_tblDrv_filterDbTableEntry_t * _rtl8651_getVlanFilterDatabaseEntry(uint16 vid, ether_addr_t * macAddr);
//------------------------------- ACL Rule -------------------------------------------------
static void _rtl8651_setAsicAclDefaultRule(uint32 aclIdx, int8 actionType);
static uint32 _rtl8651_tryAddAlgToAcl(uint16 vid, uint32 aclStart, uint32 aclEnd);
//static uint32 _rtl8651_tryAddServerPortToAcl(uint16 vid, uint32 aclStart, uint32 aclEnd);
static int8 _rtl8651_sameAclRule(_rtl8651_tblDrvAclRule_t *rule1, rtl8651_tblDrvAclRule_t *rule2);
//------------------------------- Vlan -----------------------------------------------------
//static void _rtl8651_updateAsicAllVlan(void);
/*------------------------------- Idle Timeout ----------------------------------------------------*/
static void _rtl8651_timeUpdateSessionHangUp(uint32 secPassed);
//------------------------------- PPPoE ----------------------------------------------------
//static void _rtl8651_unbindPppoeSession(rtl8651_tblDrv_ipIntfIpAddrEntry_t *ipAddr, rtl8651_tblDrv_pppTable_t *pppoe_t);
//------------------------------- IP Table -------------------------------------------------
//static int8 _rtl8651_getPppoeTblIndex(uint32 pppoeId, uint32 *tblIdx);
//------------------------------- IP Interface ---------------------------------------------
static void _rtl8651_updateGidxRegister(void);
//------------------------------- ARP ------------------------------------------------------
static void _rtl8651_arrangeAsicArpEntry(rtl8651_tblDrv_ipIntfEntry_t *ipIntf, rtl8651_tblDrv_arpEntry_t *arp_t);
int32 _rtl8651_delLocalArp(int8 fromDrv, ipaddr_t ipAddr, rtl8651_tblDrv_networkIntfTable_t *netIfPtr);
static void _rtl8651_timeUpdateArp(uint32 secPassed);
//------------------------------- Routing --------------------------------------------------
//static void _rtl8651_arrangeAsicRoutingEntry(uint32 rtArgIdx);
static void _rtl8651_updateDefaultRoute(void);
//static void _rtl8651_refillAsicRoute(uint32 rtArgIdx);
//------------------------------- NAPT TCP/UDP/ICMP ---------------------------------------
static int32 _rtl8651_selectNaptPortOffset(uint32 tblIdx, uint32 isTcp, ipaddr_t wantedIp, uint16 *offset);

int32 _rtl8651_delNaptConnection( rtl8651_tblDrv_naptTcpUdpFlowEntry_t *delConn);
static void _rtl8651_removeNaptConnectionBySrcIP(ipaddr_t sip);
static void _rtl8651_removeNaptConnectionByExtIP(ipaddr_t);
static int32 _rtl8651_flushNaptConnection(void);
static void _rtl8651_removeNaptConnectionBySrcIP(ipaddr_t sip);
static void _rtl8651_softwareNaptAging(uint32 secPassed);
static int32 _rtl8651_selectNaptIcmpFlowOffset(uint32 flowTblIdx, uint32 HashIdx, ipaddr_t ExtIp, uint16 *offset);
static void _rtl8651_removeIcmpFlowBySrcIP(ipaddr_t sip);
static rtl8651_tblDrv_naptIcmpFlowEntry_t * _rtl8651_getNaptIcmpFlowEntry(uint32 flowTblIdx, ipaddr_t insideLocalIpAddr, uint16 insideLocalId);
static int32 _rtl8651_delNaptIcmpFlowEntry(uint32 flowTblIdx, rtl8651_tblDrv_naptIcmpFlowEntry_t * deletedEntry);
//static rtl8651_tblDrv_naptIcmpFlowEntry_t * _rtl8651_getAsicNaptIcmpDynamicFlowToTable(uint32 flowTblIdx);
int32 _rtl8651_addNaptIcmpNonCandidateFlow(int8 fromDrv, ipaddr_t insideLocalIpAddr, uint16 insideLocalId, 
			ipaddr_t insideGlobalIpAddr, uint16 insideGlobalId, ipaddr_t dstIpAddr);
//static void _rtl8651_refillNaptIcmpFlowOccupiedEntry(uint32 flowTblIdx, rtl8651_tblDrv_naptIcmpFlowEntry_t *delEntry) ;
static int32 _rtl8651_delNaptIcmpFlow(uint32 flowTblIdx, rtl8651_tblDrv_naptIcmpFlowEntry_t *delEntry);
static void _rtl8651_timeUpdateNaptIcmpTable(uint32 secPassed);
#if 0
static int32 _rtl8651_refreshNaptConnectionAge(uint32 maxage);
#endif/*0*/
#ifdef SUPPORT_HASH3
static int32 _rtl8651_changeNaptFlowHashType( struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s * thisFlow,
                                              uint32 newHashType );
#endif/*SUPPORT_HASH3*/
int32 _rtl8651_changeNaptFlowListType( struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s * thisFlow,
                                              uint8 inListType, uint8 outListType );
static int32 _rtl8651_NaptEntryIsEmpty( uint16 tblIndex );
static void _rtl8651_setNaptAsicAge( void );		/* Set TCP/UDP/ICMP NAPT table's HW age */
/*
	We MUST suppress ASIC age of NAPT table being 8 sec.
	Or the age sync between HW/SW will be fail (due to granularity problem, please refer to 865x sepc "TEATCR, L4 Table Entry Aging Time Control Register").
*/
#define RTL8651_MAX_NAPT_ASIC_AGE		8

//--------------------------------- NAT ----------------------------------------------------
static int8 _rtl8651_isNaptMapping(ipaddr_t extip);
static int8 _rtl8651_isNatMapping(ipaddr_t *extip, ipaddr_t *intip);
static int32 _rtl8651_delNaptMapping(ipaddr_t extIpAddr);
//------------------------------- ALG ------------------------------------------------------
static uint32 _rtl8651_algToAclCount(void);
static int32 _rtl8651_delAlgEntryFromAsic(rtl8651_tblDrv_algEntry_t *algPtr);
static int32 _rtl8651_addAlgEntryToAsic(rtl8651_tblDrv_algEntry_t *algPtr);
static void _rtl8651_arrangeAsicAlg(void);
//------------------------------- Server Port ----------------------------------------------
//static uint32 _rtl8651_servPortToAclCount(void);
//static void _rtl8651_setServerPortExternalIp(ipaddr_t orgNaptIp, ipaddr_t newNaptIp);
//static void _rtl8651_flushNaptServerPortTable(rtl8651_tblDrv_networkIntfTable_t *netif_t);
//-------------------------------- Lan Side IP-Unnumbered ----------------------------------
 int8 _rtl8651_isLanSideIpUnnumberedIpAddr(ipaddr_t ipAddr);
//------------------------------- DoS ------------------------------------------------------
static int32 _rtl8651_DoS_tcpSynFlood(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr);
static int32 _rtl8651_DoS_tcpFinFlood(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr);
static int32 _rtl8651_DoS_icmpFlood(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr);
static int32 _rtl8651_DoS_udpFlood(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr);
static int32 _rtl8651_DoS_ipSpoofing(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr);
static int32 _rtl8651_DoS_udpEchoChargen(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr);
static int32 _rtl8651_DoS_fullXmasScan(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr);
static int32 _rtl8651_DoS_fullSapuScan(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr);
static int32 _rtl8651_DoS_finScan(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr);
static int32 _rtl8651_DoS_synFinScan(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr);
static int32 _rtl8651_DoS_nullScan(uint16 vid, uint32 inAclStart, uint32 inAclEnd, int8 wr);
static int32 _rtl8651_DoS_nmapXmasScan(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr);
static int32 _rtl8651_DoS_vecnaScan(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr);
static int32 _rtl8651_DoS_portScan(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr);
static int32 _rtl8651_addAclRuleForDoS(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr);
//------------------------------- Misc. ----------------------------------------------------
#ifdef TRAP_TCP_SYN_FIN_RST
static uint32 _rtl8651_tryAddAclToTrapTcpSynFinRst(uint32 aclStart, uint32 aclEnd);
#else
static uint32 _rtl8651_tryAddAclForWanSynTrap(uint32 aclStart, uint32 aclEnd);
#endif
static uint32 _rtl8651_tryAddAclForMultipleNextHop(int8 wrt, uint16 vid, uint32 aclStart, uint32 aclEnd);

static int32 _rtl8651_delDmzHost(uint32 dsid,ipaddr_t dmzHostIp);
static int32 _rtl8651_initURLfilter(void);

static uint32 *urlFilterTrustedUsers = ((void *)0);
static int _rtl8651_initUrlFilterTrustedUserDb(void);
static void _rtl8651_ReinitUrlFilterTrustedUserDb(void);
static int _rtl8651_isUrlFilterTrustedUser(uint32 ipaddr);

static uint32 _rtl8651_tryAddAclForPolicyBasedQos(rtl8651_tblDrv_networkIntfTable_t *netif_t, uint32 aclStart, uint32 aclEnd);

static int32 _rtl8651_addInterfaceRoute(rtl8651_tblDrv_networkIntfTable_t *netif_t, rtl8651_tblDrv_ipIntfEntry_t *ipintf_t, uint32 arpsta, uint32 arpend);
static int32 _rtl8651_delInterfaceRoute(ipaddr_t ipAddr, ipaddr_t ipMask);
static void _rtl8651_updateL3Entry(rtl8651_tblDrv_routeTable_t *rt_t);
static void _rtl8651_unbindSession(rtl8651_tblDrv_ipIntfIpAddrEntry_t *ipAddr, rtl8651_tblDrv_pppTable_t *pppEntry_t) ;

void _rtl8651_ipClassTimeUpdate(void);
void _rtl8651_arrangeNATbyMacAccessControl(void);
void _rtl8651_arrangeSPbyMacAccessControl(void);
uint32 _rtl8651_acl_PPPoE_Passthru( uint32 aclStart, uint32 aclEnd, rtl8651_tblDrv_networkIntfTable_t *netIfPtr );
uint32 _rtl8651_acl_IPv6_Passthru( uint32 aclStart, uint32 aclEnd, rtl8651_tblDrv_networkIntfTable_t *netIfPtr );

rtl8651_tblDrv_ipEntry_t *_rtl8651_addIpEntry(ipaddr_t extip, ipaddr_t intip, int8 attr, rtl8651_tblDrv_networkIntfTable_t *netif_t);
static int32 _rtl8651_delIpEntry(int8 attr, ipaddr_t extip, ipaddr_t intip) ;
int32 _rtl8651_setExternalInterfaceforPPPoEPassthru( ether_addr_t* gmac, uint16 ExtVid );
int32 _rtl8651_setExternalInterfaceforIPv6Passthru( ether_addr_t* gmac, uint16 ExtVid );
int32 _rtl8651_setExternalInterfaceforIPXPassthru( ether_addr_t* gmac, uint16 ExtVid );
int32 _rtl8651_setExternalInterfaceforNetbiosPassthru( ether_addr_t* gmac, uint16 ExtVid );

rtl8651_tblDrv_pppTable_t *_rtl8651_getPPPInfo(uint32 sessionId, ipaddr_t ipaddr);

/* policy route */
static void _rtl8651_dynamicPolicyRouteTimeupdate(uint32 secpassed);

/* next hop table */
static int32 _rtl8651_init_nexthopEntry(uint32 entryIndex);
static void _rtl8651_updateNextHopEntry(ipaddr_t nexthop, rtl8651_tblDrv_nextHopEntry_t *entry_t);
static void  _rtl8651_arrangeNextHopTable(ipaddr_t nexthop, rtl8651_tblDrv_filterDbTableEntry_t *macInfo);
static int32 _rtl8651_addNextHopEntry(int8 attr, void *ptr_t, rtl8651_tblDrv_networkIntfTable_t *netif_t, ipaddr_t nexthop, ipaddr_t naptip, rtl8651_tblDrv_pppTable_t *session_t);
static int32 _rtl8651_delNextHopEntry(int8 attr, void *ptr_t);
static void _rtl8651_updateNextHop_NHIDX(rtl8651_tblDrv_arpEntry_t *arp_t,  rtl8651_tblDrv_pppTable_t *session_t);
static void _rtl8651_ipTbl_setDefNexthop(rtl8651_tblDrv_routeTable_t *rt_t);
int32 _rtl8651_addHash2ToNexthop(ipaddr_t nexthop, ipaddr_t naptip);
int32 _rtl8651_delHash2FromNexthop(ipaddr_t nexthop, ipaddr_t naptip);

#define RTL8651_TABLE_DRIVER_MACRO_DEFINITION
/*=======================================================================================
 * RTL8651 Table Driver Macro Definition:
 *	. RTL8651_SETASICVLAN()
 *
 *=======================================================================================*/



/* Constant value definition for ARP maniuplation. */
#define ARP_FORCE_DEL		0x03




#define RTL8651_TABLE_DRIVER_COMMON_FUNCTION
/*=======================================================================================
 * RTL8651 Common Functions:
 *	
 *
 *=======================================================================================*/



rtl8651_tblDrv_networkIntfTable_t *GET_NETWORK_INTERFACE(int8 *ifname, uint32 *ifIdx) {
	rtl8651_tblDrv_networkIntfTable_t *netif_t=&DrvTbl.netIface[0];
	uint32 netIdx;

	for(netIdx=0; netIdx<rtl8651_tblDrvPara.networkIntfTableSize; netIdx++, netif_t++)
		if((netif_t->valid==1)&&(_strncmp(netif_t->ifName, ifname, RTL8651_NETINTF_NAME_LEN)==0)) {
			if (ifIdx != (uint32 *)NULL)
				*ifIdx = netIdx;
			return (rtl8651_tblDrv_networkIntfTable_t *)netif_t;
		}
	return (rtl8651_tblDrv_networkIntfTable_t *) NULL;
}




#define GENERIC_APIS
 
int32 rtl8651_enableNaptAutoAdd(int8 enable){
	naptAutoLearn=enable;
	rtl8651_setAsicNaptAutoAddDelete(enable,TRUE);
	return SUCCESS;
}




//was _rtl8651_syncL2L3Arp
//Called by L2 driver, to notify L3/4 driver that the specified L2 entry just added/changed by user/driver.
//The 'arg2' (2nd parm) is a reserved field.
void _rtl8651_syncL2Entrychange( void *arg1, void *arg2) {
	rtl8651_tblDrv_filterDbTableEntry_t *l2entry_t=(rtl8651_tblDrv_filterDbTableEntry_t *)arg1;
	if(l2entry_t){
		rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
		rtl8651_tblDrv_ipIntfEntry_t *ipIf_t;
		rtl8651_tblDrv_arpEntry_t *arp_t;
		uint32 i;

		for(i=0,netIfPtr=&DrvTbl.netIface[0]; i<rtl8651_tblDrvPara.networkIntfTableSize; i++,netIfPtr++) {
			if (netIfPtr->valid == 0)
				continue;
			if (netIfPtr->linkLayerType == RTL8651_LL_PPPOE)
				continue;
			SLIST_FOREACH(ipIf_t, &(netIfPtr->IpHead), nextIp) {
				SLIST_FOREACH(arp_t, &(ipIf_t->nextArp), next){ 
					if (arp_t->macInfo == l2entry_t) 						
						_rtl8651_arrangeAsicArpEntry(ipIf_t, arp_t);
				}
			}
		}
	}
}
	
#ifdef CONFIG_RTL865XB_EXP_INVALID
/*
@func int32	| rtl8651_enableSmartNaptCreate	|  Enable/Disable smart ASIC NAPT arrangement.
@parm int8	| enable		| TRUE to turn on smart NAPT arrangement, FALSE to turn it off.
@comm 
If <p enable> is TRUE, driver tries its best to allocate an ASIC NAPT entry for both downlink and, if possible, the uplink flow. Enabling this option 
reduces ASIC table collision rate and improves system performance significantly when ASIC table utilization is not high.
If <p enable> is FALSE, driver allocates & arranges ASIC NAPT entries normally.

Note: If <p rtl8651_enableSmartNaptCreate()> has been called to disable ASIC acceleration, it automatically turns of this feature.
*/
int32 rtl8651_enableSmartNaptCreate(int8 enable) {
	rtlglue_drvMutexLock();//Lock resource
	_smartNaptCreate= enable;
	rtlglue_drvMutexUnlock();//Unlock resource
	return SUCCESS;
}
#endif


#define RTL8651_MULTIPLE_PPPoE_NEXTHOP
/*=======================================================================================
 * RTL8651 Multiple NextHop/PPPoE Patching APIs:
 *	1. Static Functions
 *	2. Internal APIs
 *	3. External APIs
 *
 *=======================================================================================*/


#define RTL8651_IP_ACL_RULE(ipRule, action, sip, sipm, dip, dipm, iptos, iptosm, iproto, iprotom, ipflag, ipflagm)\
{\
	(ipRule)->srcIpAddr_ = sip;\
	(ipRule)->srcIpAddrMask_ = sipm;\
	(ipRule)->dstIpAddr_ = dip;\
	(ipRule)->dstIpAddrMask_ = dipm;\
	(ipRule)->tos_ = iptos;\
	(ipRule)->tosMask_ = iptosm;\
	(ipRule)->ipProto_ = iproto;\
	(ipRule)->ipProtoMask_ = iprotom;\
	(ipRule)->ipFlag_ = ipflag;\
	(ipRule)->ipFlagMask_ = ipflagm;\
	(ipRule)->ruleType_ = RTL8651_ACL_IP;\
	(ipRule)->actionType_ = action;\
}

#define RTL8651_MAC_ACL_RULE(macRule, action, smac, smacm, dmac, dmacm, type, typem)\
{\
	memcpy(&((macRule)->srcMac_), smac, sizeof(ether_addr_t));\
	memcpy(&((macRule)->srcMacMask_), smacm, sizeof(ether_addr_t));\
	memcpy(&((macRule)->dstMac_), dmac, sizeof(ether_addr_t));\
	memcpy(&((macRule)->dstMacMask_), dmacm, sizeof(ether_addr_t));\
	(macRule)->typeLen_ = type;\
	(macRule)->typeLenMask_ = typem;\
	(macRule)->ruleType_ = RTL8651_ACL_MAC;\
	(macRule)->actionType_ = action;\
}

#define RTL8651_IFSEL_ACL_RULE(ifRule, action, g, dv, nh, ppid)\
{\
	(ifRule)->gidxSel_ = g;\
	(ifRule)->dvid_ = dv;\
	(ifRule)->nextHop_ = nh;\
	(ifRule)->pppoeIdx_ = ppid;\
	(ifRule)->ruleType_ = RTL8651_ACL_IFSEL;\
	(ifRule)->actionType_ = action;\
}
#define RTL8651_REDIRECT_IP_RULE(rRule, action, sip, sipm, dip, dipm, iptos, iptosm, iproto, iprotom, ipflag, ipflagm, dv, nh, pppid)\
{\
	(rRule)->srcIpAddr_ = sip;\
	(rRule)->srcIpAddrMask_ = sipm;\
	(rRule)->dstIpAddr_ = dip;\
	(rRule)->dstIpAddrMask_ = dipm;\
	(rRule)->tos_ = iptos;\
	(rRule)->tosMask_ = iptosm;\
	(rRule)->ipProto_ = iproto;\
	(rRule)->ipProtoMask_ = iprotom;\
	(rRule)->ipFlag_ = ipflag;\
	(rRule)->ipFlagMask_ = ipflagm;\
	(rRule)->dvid_ = dv;\
	(rRule)->nextHop_ = nh;\
	(rRule)->pppoeIdx_ = pppid;\
	(rRule)->ruleType_ = RTL8651_ACL_IP;\
	(rRule)->actionType_ = action;\
}


#if 0
static int32 _rtl8651_addRedirectForNat(int8 wrt, rtl8651_tblDrvSession_t *session, uint32 *aclIndex) {

#if 0 //chhuang:
	_rtl8651_tblDrvAclRule_t aclRule;
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_ipIntfEntry_t *ipIntf;
	rtl8651_tblDrv_pppoeTable_t *pppoe_t;
	rtl8651_tblDrv_filterDbTableEntry_t *macInfo;
	uint32 hashPos, pppidx, i, count=0;
	ipaddr_t intIp = 0;
	int8 retval;

	macInfo = session->pppoe_t->macInfo;
	if (session->pppoe_t->initiated == 0 || macInfo->configToAsic == 0 || !session->ip_t)
		return FAILED;
	/* The nexthop is PPPoE Session, use PPPoE Session IP to get NAT
	 * mapping. If the NAT mapping exists, add following rules:
	 * 		. permit Src=NAT_intIP and Dst=Local_subnet
	 *		. redirect Src=NAT_intIP
	 */
	if (_rtl8651_isNatMapping(&session->ip_t->ipAddr, &intIp) == FALSE)
		return FAILED;
	hashPos = (rtl8651_filterDbIndex(&macInfo->macAddr)<<2) + macInfo->asicPos;
	pppoe_t = &DrvTbl.pppoe[0];
	retval = _rtl8651_getPppoeTblIndex(session->pppoe_t->objectId, &pppidx);
	assert(retval == SUCCESS);
	
	/* add PERMIT rule for each local subnet */
	netif_t = &DrvTbl.netIface[0];
	for(i=0; i<rtl8651_tblDrvPara.networkIntfTableSize; i++, netif_t++) {
		if (netif_t->valid == 0 || netif_t->linkLayerType == 0 ||
			DrvTbl.vlan[rtl8651_vlanTableIndex(netif_t->vid)].internal == 0)
			continue;
		SLIST_FOREACH(ipIntf, &netif_t->IpHead, nextIp) {
			if (ipIntf->lanSideExternal == 1)
				continue;
			if (wrt == TRUE) {
				/* If the specified external IP address, which is mapped to a internal IP
 			  	 * (NAT mapping), is not in the ASIC routing's nexthop, a permit rule
 			 	 * (PERMIT src=NAT_intIP dst=Local_Subnet) and a redirect rule should be added.
 			 	 */
				RTL8651_IP_ACL_RULE(&aclRule, RTL8651_ACL_PERMIT, intIp, 0xffffffff, 
					(ipIntf->ipMask&ipIntf->ipAddr.slh_first->ipAddr), ipIntf->ipMask, 
					0, 0, 0, 0, 0, 0
				);
				rtl8651_setAsicAclRule(*aclIndex, &aclRule);
				(*aclIndex)++; count++;
				if (*aclIndex == RTL8651_ACLTBL_SIZE)
					return (int32)count;
			} else count++;
		}
	}
	if (wrt == TRUE) {
		/* add a Redirect rule to redirect NAT mapping */	
		RTL8651_REDIRECT_IP_RULE(&aclRule, RTL8651_ACL_REDIRECT_PPPOE, intIp, 0xffffffff, 0, 0, 0, 0, 0, 0, 0, 0, pppoe_t->vid, hashPos, pppidx);
		rtl8651_setAsicAclRule(*aclIndex, &aclRule);
		(*aclIndex)++; count++;
	} else count++;
	return (int32)count;
#endif
	return 0;
}
#endif


#ifdef TRAP_TCP_SYN_FIN_RST
static uint32 _rtl8651_tryAddAclToTrapTcpSynFinRst(uint32 aclStart, uint32 aclEnd){
	_rtl8651_tblDrvAclRule_t rule;
	int32 ret;
	int32 addCount  =0 ;

	if (aclStart > aclEnd)
		return addCount;
	memset(&rule, 0, sizeof(_rtl8651_tblDrvAclRule_t));
	rule.ruleType_     = RTL8651_ACL_TCP;
 	rule.actionType_   = RTL8651_ACL_CPU;
 	rule.tcpSYN_       = 0x01;
 	rule.tcpFlagMask_  = rule.tcpFlag_; /* SYN bit set */
 	rule.tcpSrcPortUB_ = 0xffff;
 	rule.tcpSrcPortLB_ = 0x00;
 	rule.tcpDstPortUB_ = 0xffff;
 	rule.tcpDstPortLB_ = 0x00;
 	rule.pktOpApp = RTL8651_ACLTBL_ONLY_L4;
 	ret = rtl8651_setAsicAclRule(aclStart, &rule);
 	assert( ret == SUCCESS );
	addCount++;
	aclStart++;

	if (aclStart > aclEnd)
		return addCount;
	memset(&rule, 0, sizeof(_rtl8651_tblDrvAclRule_t));
	rule.ruleType_     = RTL8651_ACL_TCP;
 	rule.actionType_   = RTL8651_ACL_CPU;
 	rule.tcpFIN_       = 0x01;
 	rule.tcpFlagMask_  = rule.tcpFlag_; /* SYN bit set */
 	rule.tcpSrcPortUB_ = 0xffff;
 	rule.tcpSrcPortLB_ = 0x00;
 	rule.tcpDstPortUB_ = 0xffff;
 	rule.tcpDstPortLB_ = 0x00;
 	rule.pktOpApp = RTL8651_ACLTBL_ONLY_L4;
 	ret = rtl8651_setAsicAclRule(aclStart, &rule);
 	assert( ret == SUCCESS );
	addCount++;
	aclStart++;

	if (aclStart > aclEnd)
		return addCount;
	memset(&rule, 0, sizeof(_rtl8651_tblDrvAclRule_t));
	rule.ruleType_     = RTL8651_ACL_TCP;
 	rule.actionType_   = RTL8651_ACL_CPU;
 	rule.tcpRST_       = 0x01;
 	rule.tcpFlagMask_  = rule.tcpFlag_; /* SYN bit set */
 	rule.tcpSrcPortUB_ = 0xffff;
 	rule.tcpSrcPortLB_ = 0x00;
 	rule.tcpDstPortUB_ = 0xffff;
 	rule.tcpDstPortLB_ = 0x00;
  	rule.pktOpApp = RTL8651_ACLTBL_ONLY_L4;
	ret = rtl8651_setAsicAclRule(aclStart, &rule);
 	assert( ret == SUCCESS );
	addCount++;
	aclStart++;
	
	return addCount;
} /* end _rtl8651_tryAddAclForWanSynTrap */

#else/*TRAP_TCP_SYN_FIN_RST*/

static uint32 _rtl8651_tryAddAclForWanSynTrap(uint32 aclStart, uint32 aclEnd){
	_rtl8651_tblDrvAclRule_t rule;

	if (aclStart > aclEnd)
		return 0;
	memset(&rule, 0, sizeof(_rtl8651_tblDrvAclRule_t));
	rule.ruleType_     = RTL8651_ACL_TCP;
 	rule.actionType_   = RTL8651_ACL_CPU;
 	rule.tcpSYN_       = 0x01;
 	rule.tcpFlagMask_  = rule.tcpFlag_; /* SYN bit set */
 	rule.tcpSrcPortUB_ = 0xffff;
 	rule.tcpSrcPortLB_ = 0x00;
 	rule.tcpDstPortUB_ = 0xffff;
 	rule.tcpDstPortLB_ = 0x00;
 	rule.pktOpApp = RTL8651_ACLTBL_ONLY_L4;
 	rtl8651_setAsicAclRule(aclStart, &rule);
	return 1;
} /* end _rtl8651_tryAddAclForWanSynTrap */

#endif/*TRAP_TCP_SYN_FIN_RST*/

static uint32 _rtl8651_tryAddAclForMultipleNextHop(int8 wrt, uint16 vid, uint32 aclStart, uint32 aclEnd) {

#if 0 //chhuang:
	rtl8651_tblDrv_networkIntfTable_t *netif_t = &DrvTbl.netIface[0];
	rtl8651_tblDrv_networkIntfTable_t *netif_array[RTL8651_ROUTINGTBL_SIZE] = { NULL };
#if 0
	rtl8651_tblDrv_routeAsicArrangementEntry_t *asic_rt;
#else
	rtl8651_tblDrv_routeTable_t *rt_t;
#endif
	//rtl8651_tblDrv_pppoeSession_t *session;
	rtl8651_tblDrvSession_t*session;
	rtl8651_tblDrv_vlanTable_t *vlan_t;
	uint32 rtidx, i, j, count = 0;
	ipaddr_t intIp;
	int32 addCount;

	/*
	 * Use ACL rules to patch RTL8651 Multiple PPPoE Session supporting:
	 * 	1. For each LAN, only internal VLAN should add patched ACL rules 
	 *	   in its Ingress rule.
	 *	2. If a route, which is in the ASIC routing table, to the Mulitple 
	 *	   PPPoE network interface, the patched ACL rules should be added.
	 *	3. In the mulitple PPPoE Session environment, if a route(in the ASIC)
	 *	   uses one of the multiple PPPoE Session, we don't add patched ACL
	 *	   rules for the session.
	 *
	 * Limitation:
	 *	1. Multiple PPPoE Session is only workable when the multiple PPPoE
	 *	   network interface is an external interface.
	 *	2. Only support 1-NAPT + n-NAT.
	 *	 
	 */


	if (aclStart > aclEnd || aclEnd>=RTL8651_ACLTBL_SIZE)	
		return 0;
	vlan_t = &DrvTbl.vlan[rtl8651_vlanTableIndex(vid)];
	assert(vlan_t->valid == 1 && vlan_t->vid == vid);
	if (vlan_t->internal == 0)
		return 0;

#if 0
	asic_rt = &AsicTableUsage.route[0];
	for(rtidx=0; rtidx<RTL8651_ROUTINGTBL_SIZE; rtidx++, asic_rt++) {
		if (asic_rt->rt_entry == NULL)	continue;
		if (asic_rt->rt_entry->nextHopType != RTL8651_LL_PPPOE)	continue;
		for(i=0; i<rtl8651_tblDrvPara.networkIntfTableSize; i++, netif_t++)
			if (netif_t->valid == 1 && netif_t->vid == asic_rt->rt_entry->dvid)
				break;
		if (i == rtl8651_tblDrvPara.networkIntfTableSize)
			continue;
		for(i=0; i<RTL8651_ROUTINGTBL_SIZE; i++) {
			if (netif_array[i] == NULL) 
                                j = i;
			if (netif_array[i] == netif_t)	
                                break;
		}
		if (i < RTL8651_ROUTINGTBL_SIZE)
			continue; /* already process */
		netif_array[j] = netif_t;
		SLIST_FOREACH(session, &netif_t->psidHead, nextSession) {
			/* If the PPPoE Session is written into ASIC routing entry, no 
			 * patched ACL rule should be added for it.
			 */
			if (asic_rt->rt_entry->pppoe_t == session->pppoe_t)
				continue;
			intIp = 0;
			if (_rtl8651_isNatMapping(&session->ip_t->ipAddr, &intIp) == FALSE)
				continue;
			{	
				/* If the internal IP address of NAT mapping is not in the specified
				 * network interface, no needed to add ACL rule for the netwrok 
	 		 	 * interface.
				 */
				rtl8651_tblDrv_networkIntfTable_t *net_t;
				rtl8651_tblDrv_ipIntfEntry_t *ipintf_t;
				GET_NETIF_BY_VID(net_t, vid);
				SLIST_FOREACH(ipintf_t, &net_t->IpHead, nextIp) {
					if ((ipintf_t->ipMask&ipintf_t->ipAddr.slh_first->ipAddr) ==
						(ipintf_t->ipMask&intIp))
						break;
				}
				if (ipintf_t == NULL)
					continue;
			}
			if ((addCount=_rtl8651_addRedirectForNat(wrt, session, &aclStart)) == FAILED)
				continue;
			count += addCount;
			if (aclStart > aclEnd) 
				return count;
		}		
	}
#else

	rt_t = DrvTbl.route;
	for(rtidx=0; rtidx<RTL8651_ROUTINGTBL_SIZE; rtidx++, rt_t++) {
		if (rt_t->valid == 0)	continue;
		if (rt_t->nextHopType != RTL8651_LL_PPPOE)	continue;
		for(i=0; i<rtl8651_tblDrvPara.networkIntfTableSize; i++, netif_t++)
			if (netif_t->valid == 1 && netif_t->vid == rt_t->dvid)
				break;
		if (i == rtl8651_tblDrvPara.networkIntfTableSize)
			continue;
		for(i=0; i<RTL8651_ROUTINGTBL_SIZE; i++) {
			if (netif_array[i] == NULL) j = i;
			if (netif_array[i] == netif_t)	break;
		}
		if (i < RTL8651_ROUTINGTBL_SIZE)
			continue; /* already process */
		netif_array[j] = netif_t;
		SLIST_FOREACH(session, &netif_t->psidHead, nextSession) {
			/* If the PPPoE Session is written into ASIC routing entry, no 
			 * patched ACL rule should be added for it.
			 */
			if (rt_t->rt_nhpppoe_t == session->pppoe_t)
				continue;
			intIp = 0;
			if (_rtl8651_isNatMapping(&session->ip_t->ipAddr, &intIp) == FALSE)
				continue;
			{	
				/* If the internal IP address of NAT mapping is not in the specified
				 * network interface, no needed to add ACL rule for the netwrok 
	 		 	 * interface.
				 */
				rtl8651_tblDrv_networkIntfTable_t *net_t;
				rtl8651_tblDrv_ipIntfEntry_t *ipintf_t;
				GET_NETIF_BY_VID(net_t, vid);
				SLIST_FOREACH(ipintf_t, &net_t->IpHead, nextIp) {
					if ((ipintf_t->ipMask&ipintf_t->ipAddr.slh_first->ipAddr) ==
						(ipintf_t->ipMask&intIp))
						break;
				}
				if (ipintf_t == NULL)
					continue;
			}
			if ((addCount=_rtl8651_addRedirectForNat(wrt, session, &aclStart)) == FAILED)
				continue;
			count += addCount;
			if (aclStart > aclEnd) 
				return count;
		}		
	}

#endif
	return count;	
#endif
	return 0;
	
}

#define RTL8651_SESSIONHANGUP_APIs
/*=======================================================================================
 * RTL8651 PPPoE APIs:
 *	1. Static Functions
 *		. _rtl8651_timeUpdateSessionHangUp()
 *		  Check idle timeout of each session and call the callBack hangup function is ilde-timer is up.
 *	2. Internal APIs
 *	3. External APIs
 *=======================================================================================*/
 /* ------------------------------------------------------------------------
 	Time update function to check the idle-timer of each registrated session.
 	If the idle-timer is up, this module will call the call back function to alert upper layer.
      ------------------------------------------------------------------------ */
static void _rtl8651_timeUpdateSessionHangUp(uint32 secPassed)
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t = NULL;
	_rtl8651_sessionHangUp_t *hangUp_p;
	uint32 idx, syncSetting = FALSE;

	/* --------------------------------------------------------------------------
			(1)
				For HW forwarding traffic counting, we update idle timer if
				they have traffic forwarded via HW.
	    -------------------------------------------------------------------------- */

	for ( idx = 0, netif_t = DrvTbl.netIface ; idx < rtl8651_tblDrvPara.networkIntfTableSize ; idx ++, netif_t++ )
	{
		if (	(netif_t->valid) &&
			_rtl8651_getRateLimitCounter(	netif_t,
											RATELIMIT_COUNTER_TYPE_WANTRAFFICCOUNTER,
											TRUE ) > 0)
		{
			if (netif_t->linkLayerType == RTL8651_LL_VLAN)
			{
				/*
					For VLAN type wan.
					=> Assumption: we assume there would only ONE external VLAN-type network interface.
				*/
				RTL_DEBUG(	RTL_MSG_SESSION_HANGUP,
							"Record traffic for (%s) - Dsid (%d)",
							netif_t->ifName,
							0);

				_rtl8651_recordTraffic( 0 /* update default session*/, TRUE /* to WAN */);
			} else
			{
				/* For session-based LL types: For Multiple Session case, only default session would be written into HW, so we only update its timer */
				rtl8651_tblDrvSession_t *psession_t;

				CTAILQ_FOREACH(psession_t, &netif_t->sessionHead, nextSession)
				{
					if (	(psession_t) &&
						(psession_t->session_t) &&
						(psession_t->session_t->valid == TRUE))
					{
						uint32 updateTraffic = FALSE;

						if (netif_t->linkLayerType == RTL8651_LL_LIPUNNUMBERED)
						{
							/* for Unnumber WAN case, we always update its sessions */
							updateTraffic = TRUE;
						} else
						{
							rtl8651_tblDrv_ipIntfIpAddrEntry_t *ip_t;

							/*
								We start to trace session table. If its IP address in the session is in HW,
								we can assume the traffic is belong to it.
								(FIXME if HW multiple session forwarding is realized)
							*/
							ip_t = psession_t->ip_t;
							while (ip_t)
							{
								if (_rtl8651_getIPTableEntry(ip_t->ipAddr, 0, IP_ATTR_ALL))
									break;
								ip_t = SLIST_NEXT(ip_t, nextIpAddr);
							}
							if (ip_t)
							{
								updateTraffic = TRUE;
							}
						}

						if (updateTraffic == TRUE)
						{
							
							RTL_DEBUG(	RTL_MSG_SESSION_HANGUP,
										"Record traffic for (%s) - Dsid (%d)",
										netif_t->ifName,
										_rtl8651_SessionIDToDSID(psession_t->session_t->sessionId));

							_rtl8651_recordTraffic(_rtl8651_SessionIDToDSID(psession_t->session_t->sessionId), TRUE /* to WAN */);
						}
					}
				}
			}
		}
	}

	/* --------------------------------------------------------------------------
			(2)
				CallBack for the session entries which has been timed-up.
	    -------------------------------------------------------------------------- */

	for (	idx = 0, hangUp_p = rtl8651_sessionHangUp ;
			idx < RTL8651_MAX_DIALSESSION ;
			idx ++, hangUp_p ++)
	{
		/* empty entry ? ignore */
		if (hangUp_p->p_callBack == NULL) continue;

		/* idle-time is up (no TX/RX) ! --- call Back */
		if (	((_sysUpSeconds - hangUp_p->last_received) > hangUp_p->idletime_limit) &&
			((_sysUpSeconds - hangUp_p->last_transmit) > hangUp_p->idletime_limit))
		{
			RTL_NOTICE(	RTL_MSG_SESSION_HANGUP,
							"Hang Up callBack (%p): session (%d)",
							hangUp_p->p_callBack,
							hangUp_p->sessionID);

			hangUp_p->p_callBack(hangUp_p->sessionID);				/* call callBack function */
			memset(hangUp_p, 0, sizeof(_rtl8651_sessionHangUp_t));	/* clear this entry */
			syncSetting = TRUE;									/* data-structure is modified, need to sync setting */
		}
	}

	if ( syncSetting == TRUE )
	{
		_rtl8651_RearrangeHWTrafficCounter();	/* Maybe we need to diable HW Counter */
	}

}

/*
@func int32 | rtl8651_setSessionHangUp | Enable/Disable Session idle-timeout hang up callBack.
@parm rtl8651_sessionHangUp_t* | param | parameters to set the idle-timeout callBack.
@rvalue SUCCESS | Success.
@rvalue TBLDRV_EINVALIDINPUT | Invalid input parameter.
@rvalue TBLDRV_EINVALIDSESSIONID | Invalid Session ID.
*/
int32 rtl8651_setSessionHangUp(rtl8651_sessionHangUp_t *param)
{
	int32 retval = FAILED;

	UNLOCK_ERROR_CHECK((param == NULL), TBLDRV_EINVALIDINPUT);

	rtlglue_drvMutexLock();
	retval = _rtl8651_setSessionHangUp(param);
	rtlglue_drvMutexUnlock();

	return retval;
}

/*
	Central function to set Idle-timeout to each session.
*/
int32 _rtl8651_setSessionHangUp(rtl8651_sessionHangUp_t *param)
{
	uint32 dsid;
	assert(param);

	UNLOCK_ERROR_CHECK(	((param == NULL) ||
							 ((param->enable == TRUE) && (param->idleTimeout_callBack == NULL))),
							TBLDRV_EINVALIDINPUT);

	RTL_NOTICE(	RTL_MSG_SESSION_HANGUP,
					"Session Hangup Enable/Disable function is involved: %s - session %d",
					(param->enable == TRUE)?"ENABLE":"DISABLE",
					param->sessionID);

	/*
		get the mapping of Session ID <-> dsid

		chenyl (2005/11/1) :

			For PPTP/L2TP/VLAN ... link layer types, dsid would be 0.
	*/
	dsid = _rtl8651_SessionIDToDSID(param->sessionID);

	UNLOCK_ERROR_CHECK((dsid >= RTL8651_MAX_DIALSESSION), TBLDRV_EINVALIDSESSIONID);

	if (param->enable == TRUE)
	{
			RTL_NOTICE(	RTL_MSG_SESSION_HANGUP,
							"Register IdleTime callBack function: %p session(%d) limit(%d)",
							param->idleTimeout_callBack,
							param->sessionID,
							param->idleTimeout_sec);

		rtl8651_sessionHangUp[dsid].p_callBack = param->idleTimeout_callBack;
		rtl8651_sessionHangUp[dsid].sessionID = param->sessionID;
		rtl8651_sessionHangUp[dsid].idletime_limit = param->idleTimeout_sec;
		rtl8651_sessionHangUp[dsid].last_received = _sysUpSeconds;
		rtl8651_sessionHangUp[dsid].last_transmit = _sysUpSeconds;
	} else
	{
		RTL_NOTICE(	RTL_MSG_SESSION_HANGUP,
						"Remove idle-timeout mechanism for session %d",
						param->sessionID);

		memset(	&(rtl8651_sessionHangUp[dsid]),
					0,
					sizeof(_rtl8651_sessionHangUp_t));		
	}

	_rtl8651_RearrangeHWTrafficCounter();
	_rtl8651_reArrangeLanSideExternalIpInterfacePatch();

	return SUCCESS;
}

/*
	Update time-record for specific dsid
*/
void _rtl8651_recordTraffic(int32 dsid, int32 isXmit)
{
	assert(dsid < RTL8651_MAX_DIALSESSION);

	/* idle callBack function exist, update its time */
	if (rtl8651_sessionHangUp[dsid].p_callBack)
	{
		if (isXmit == TRUE)
		{
			rtl8651_sessionHangUp[dsid].last_transmit = _sysUpSeconds;
		} else
		{
			rtl8651_sessionHangUp[dsid].last_received = _sysUpSeconds;
		}
	}
}

int32 _rtl8651_RearrangeHWTrafficCounter(void)
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t = NULL;
	rtl8651_tblDrv_ipEntry_t *track_t = NULL;
	int32 idx, retval = SUCCESS;

	RTL_INFO(	RTL_MSG_SESSION_HANGUP,
				"=> Rearrange HW Traffic Counter");

	/* ================ flush Rate Limit Counter before Rearrange ================ */
	RTL_INFO(	RTL_MSG_SESSION_HANGUP,
				"Flush HW Traffic Counter");

	for ( idx = 0, netif_t = DrvTbl.netIface ; idx < rtl8651_tblDrvPara.networkIntfTableSize ; idx ++, netif_t++ )
	{
		if (netif_t->valid == TRUE)
		{
			RTL_DEBUG(	RTL_MSG_SESSION_HANGUP,
						"Try to delete Rate Limit counter for NetIF (%s)",
						netif_t->ifName);

			_rtl8651_delRateLimitCounter(netif_t, RATELIMIT_COUNTER_TYPE_WANTRAFFICCOUNTER);
		}
	}

	/* If there is no any Idle Timeout traffic counting issue, we DONT add Rate Limit Counter */
	for ( idx = 0 ; idx < RTL8651_MAX_DIALSESSION ; idx ++ )
	{
		if (rtl8651_sessionHangUp[idx].p_callBack)
		{
			break;
		}
	}
	if ( idx == RTL8651_MAX_DIALSESSION)
	{
		goto out;
	}

	RTL_INFO(	RTL_MSG_SESSION_HANGUP,
				"Try to add HW Traffic Counter");

	/* ================ Trace IP Entry to add HW Counter at egress interface ================ */
	for( idx = 0, track_t = DrvTbl.iptbl ; idx < RTL8651_IPTABLE_SIZE ; idx ++, track_t ++ )
	{
		if ( track_t->valid == FALSE )
		{
			continue;
		}

		assert(track_t->extip);

		switch (track_t->type)
		{
			case IP_ATTR_NAPT:
			case IP_ATTR_NAT:
			{
				/* create HW counters an add rule into it */
				/* ---------- Always add HW counters, redundancy check is done in "_rtl8651_addRateLimitCounter()" ---------- */
				if ((netif_t = _rtl8651_getIpInfo(	track_t->extip,
												NULL,
												NULL,
												NULL)) != NULL )
				{
					rtl8651_tblDrv_vlanTable_t *vlan_t;

					if (	(netif_t->linkLayerType == RTL8651_LL_LIPUNNUMBERED) &&
						(netif_t == lanSideIpUnnumberedLan) &&
						(lanSideIpUnnumberedWan) &&
						(lanSideIpUnnumberedWan->valid))
					{
						if (	(DrvTbl.vlan[rtl8651_vlanTableIndex(lanSideIpUnnumberedWan->vid)].valid) &&
							(DrvTbl.vlan[rtl8651_vlanTableIndex(lanSideIpUnnumberedWan->vid)].internal == FALSE))
						{
							netif_t = lanSideIpUnnumberedWan;
						}
					}

					assert(netif_t->valid);
					vlan_t = &DrvTbl.vlan[rtl8651_vlanTableIndex(netif_t->vid)];
					assert(vlan_t->valid);

					if (vlan_t->internal == FALSE)
					{
						RTL_DEBUG(	RTL_MSG_SESSION_HANGUP,
									"Add RateLimitCounter for HW counting at (%s) vid [%d]",
									netif_t->ifName,
									vlan_t->vid);
						if ((retval = _rtl8651_addRateLimitCounter(netif_t, RATELIMIT_COUNTER_TYPE_WANTRAFFICCOUNTER)) != SUCCESS)
						{
							RTL_ERR(	RTL_MSG_SESSION_HANGUP,
										"Rate Limit Counter addition FAILED (retval %d)",
										retval);
						} else
						{
							/* ---------- Add Rate limit counter Rule for this IP entry ---------- */
							rtl8651_tblDrvAclRule_t rl_rule;
							memset(&rl_rule, 0, sizeof(rtl8651_tblDrvAclRule_t));
							rl_rule.ruleType_		= RTL8651_ACL_IP;
						 	rl_rule.actionType_		= RTL8651_ACL_DROP_LOG;	/* we MUST forward it via SW */
							rl_rule.srcIpAddr_		= track_t->extip;
							rl_rule.srcIpAddrMask_	= 0xffffffff;

							RTL_DEBUG(	RTL_MSG_SESSION_HANGUP,
										"-> Add Rate Limit Rule at (%s) for IP (%u.%u.%u.%u)",
										netif_t->ifName,
										NIPQUAD(track_t->extip));

							if ((	retval = _rtl8651_addRateLimitCounterRule(&rl_rule,
										netif_t,
										RATELIMIT_COUNTER_TYPE_WANTRAFFICCOUNTER))
								!= SUCCESS)
							{	/* remove this entry and break */
								RTL_ERR(	RTL_MSG_SESSION_HANGUP,
											"Add Rate Limit Rule at (%s) for IP (%u.%u.%u.%u) FAILED (return %d)",
											netif_t->ifName,
											NIPQUAD(track_t->extip),
											retval);
							}
						}
					}
				}
				break;
			}
			case IP_ATTR_LS:
			{
				/* create HW counters to ALL external Network Interface/VLAN and add rule into it */
				for (idx = 0, netif_t = DrvTbl.netIface ; idx < rtl8651_tblDrvPara.networkIntfTableSize ; idx ++, netif_t++)
				{
					if (netif_t->valid == TRUE)
					{
						rtl8651_tblDrv_vlanTable_t *vlan_t;
						vlan_t = &DrvTbl.vlan[rtl8651_vlanTableIndex(netif_t->vid)];
						assert(vlan_t->valid);

						if (vlan_t->internal == FALSE)
						{
							if ((retval = _rtl8651_addRateLimitCounter(netif_t, RATELIMIT_COUNTER_TYPE_WANTRAFFICCOUNTER)) != SUCCESS)
							{
								RTL_ERR(	RTL_MSG_SESSION_HANGUP,
											"Rate Limit Counter addition FAILED (retval %d)",
											retval);
							} else
							{
								/* ---------- Add Rate limit counter Rule for this IP entry ---------- */
								rtl8651_tblDrvAclRule_t rl_rule;
								memset(&rl_rule, 0, sizeof(rtl8651_tblDrvAclRule_t));
								rl_rule.ruleType_		= RTL8651_ACL_IP;
							 	rl_rule.actionType_		= RTL8651_ACL_DROP_LOG;	/* we MUST forward it via SW */
								rl_rule.srcIpAddr_		= track_t->extip;
								rl_rule.srcIpAddrMask_	= 0xffffffff;

								RTL_DEBUG(	RTL_MSG_SESSION_HANGUP,
											"-> Add Rate Limit Rule at (%s) for IP (%u.%u.%u.%u)",
											netif_t->ifName,
											NIPQUAD(track_t->extip));

								if ((	retval = _rtl8651_addRateLimitCounterRule(&rl_rule,
											netif_t,
											RATELIMIT_COUNTER_TYPE_WANTRAFFICCOUNTER))
									!= SUCCESS)
								{	/* remove this entry and break */
									RTL_ERR(	RTL_MSG_SESSION_HANGUP,
												"Add Rate Limit Rule at (%s) for IP (%u.%u.%u.%u) FAILED (return %d)",
												netif_t->ifName,
												NIPQUAD(track_t->extip),
												retval);
								}
							}
						}
					}
				}
				break;
			}
		}
	}

out:

	RTL_INFO(	RTL_MSG_SESSION_HANGUP,
				"==> Rearrange HW Traffic Counter (End)");

	return retval;
}

#define RTL8651_PPPoE_APIs
/*=======================================================================================
 * RTL8651 PPPoE APIs:
 *	1. Static Functions
 *		. _rtl8651_unbindPppoeSession()
 *		  Unbind the PPPoE Session and IP address. 
 *		. _rtl8651_arrangePppoeSession()
 *		  Select a proper PPPoE Session to bind with routing enry. If a session is 
 *		  used as NAPT, the session is first select; otherwise, NAT session will 
 *		  be selected to bind with rouitng entry.
 *	2. Internal APIs
 *		. _rtl8651_getPppoeTblIndex()
 *		  Use PPPoE ID to find the PPPoE Table Index
 *		. _rtl8651_getPppoeSessionByIpAddr()
 *		  Query the binding realtion between PPPoE Session & IP address
 *	3. External APIs
 *		rtl8651_setPppoeSessionHangUp()
 *		rtl8651_addPppoeSession()
 *		rtl8651_delPppoeSession()
 *		rtl8651_setPppoeSessionProperty()
 *		rtl8651_resetPppoeSessionProperty()
 *		rtl8651_bindPppoeSession()
 *		rtl8651_getPppoeSessionMac()
 *=======================================================================================*/

#if 0
static void _rtl8651_unbindPppoeSession(rtl8651_tblDrv_ipIntfIpAddrEntry_t *ipAddr, rtl8651_tblDrv_pppTable_t *pppoe_t) {
	rtl8651_tblDrvSession_t *pppoes_t;
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	rtl8651_tblDrv_pppTable_t *pppoePtr = NULL;
	uint32 i;

	netIfPtr = &DrvTbl.netIface[0];
	for(i=0; i<rtl8651_tblDrvPara.networkIntfTableSize; i++, netIfPtr++) {
		if (netIfPtr->valid == 1) { 
			CTAILQ_FOREACH(pppoes_t, &netIfPtr->sessionHead, nextSession) {
				if (ipAddr && pppoes_t->ip_t == ipAddr) {
					if (!pppoePtr)  pppoePtr = pppoes_t->session_t;
					pppoes_t->ip_t = NULL;
					break;
				}
				if (pppoe_t && pppoes_t->session_t == pppoe_t) {
					if (!pppoePtr)  pppoePtr = pppoes_t->session_t;
					pppoes_t->ip_t = NULL;
					break;
				}
			}
	        }
       } 
}
#endif

		

static rtl8651_tblDrv_pppTable_t *
_rtl8651_getPppoeBySessionId(uint32 sessionId)
{
	rtl8651_tblDrv_pppTable_t *pppoe_t = DrvTbl.pppoe;
	uint32 tblidx;

	for(tblidx=0; tblidx<RTL8651_PPPOETBL_SIZE; tblidx++, pppoe_t++) {
		if (pppoe_t->valid && pppoe_t->sessionId==sessionId)
			return pppoe_t;
	}
	return (rtl8651_tblDrv_pppTable_t *)NULL;
}

#if 0
static int8 _rtl8651_getPppoeTblIndex(uint32 pppoeId, uint32 *tblIdx) {
	rtl8651_tblDrv_pppTable_t *pppoePtr;

	for(*tblIdx=0, pppoePtr=&DrvTbl.pppoe[0]; *tblIdx<RTL8651_PPPOE_NUMBER; (*tblIdx)++, pppoePtr++)
		if (pppoePtr->valid == 1 && pppoePtr->sessionId == pppoeId)
			return SUCCESS;
 	return FAILED;
}
#endif


int32 _rtl8651_getPppoeTblPtr(uint32 pppoeId,rtl8651_tblDrv_pppTable_t **pppoePtr_P){
	uint32 tblIdx;
	rtl8651_tblDrv_pppTable_t *pppoePtr;

	for(tblIdx=0, pppoePtr=&DrvTbl.pppoe[0]; tblIdx<RTL8651_PPPOE_NUMBER; (tblIdx)++, pppoePtr++)
		if (pppoePtr->valid == 1 && pppoePtr->sessionId == pppoeId){
			*pppoePtr_P=pppoePtr;
			return SUCCESS;
		}
 	return FAILED;
}

rtl8651_tblDrv_pppTable_t *
_rtl8651_getPppoeSessionByIpAddr(ipaddr_t ipaddr) {
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrvSession_t *psession_t;
	rtl8651_tblDrv_pppTable_t *useSession_t = NULL;
	uint32 i;
	//assert(ipaddr != 0);
	if (ipaddr == 0){
		//rtlglue_printf("_rtl8651_getPppoeSessionByIpAddr: 1 ipaddr %x\n",ipaddr);
		return NULL;
	}
	netif_t = &DrvTbl.netIface[0];
	#if 0
	// this part only supports unnumber-ip with napt
	for(i=0; i<rtl8651_tblDrvPara.networkIntfTableSize; i++, netif_t++) {
		if (netif_t->valid == 0 ||netif_t->linkLayerType == RTL8651_LL_VLAN)
			continue;
		SLIST_FOREACH(psession_t, &netif_t->psidHead, nextSession){
			assert(IS4BYTEALIGNED((uint32)&psession_t->ip_t->ipAddr));
			if (psession_t->ip_t&&_rtl8651_isNaptMapping(psession_t->ip_t->ipAddr) == TRUE)
				useSession_t = psession_t->pppoe_t; /* for Lan Side IP-Unnumbered */
			if (psession_t->ip_t && psession_t->ip_t->ipAddr == ipaddr)
				return psession_t->pppoe_t;
		}
	}
	if (useSession_t && _rtl8651_isLanSideIpUnnumberedIpAddr(ipaddr) == TRUE){
		//rtlglue_printf("_rtl8651_getPppoeSessionByIpAddr: 2 ipaddr %x\n",ipaddr);
		return useSession_t;/* for Lan Side IP-Unnumbered */
	}
	//rtlglue_printf("_rtl8651_getPppoeSessionByIpAddr: 3 ipaddr %x\n",ipaddr);
	return (rtl8651_tblDrv_pppoeTable_t *)NULL;
	#else
	// this part supports unnumber-ip with/without napt, by orlando 2003/12/12
	for(i=0; i<rtl8651_tblDrvPara.networkIntfTableSize; i++, netif_t++) {
		if (netif_t->valid == 0 ||netif_t->linkLayerType == RTL8651_LL_VLAN)
			continue;
		CTAILQ_FOREACH(psession_t, &netif_t->sessionHead, nextSession){
			assert(IS4BYTEALIGNED((uint32)&psession_t->ip_t->ipAddr));			
			if (psession_t->ip_t&&_rtl8651_isNaptMapping(psession_t->ip_t->ipAddr) == TRUE){
				useSession_t = psession_t->session_t; /* for Lan Side IP-Unnumbered */
				if (useSession_t && _rtl8651_isLanSideIpUnnumberedIpAddr(ipaddr) == TRUE){
					//rtlglue_printf("_rtl8651_getPppoeSessionByIpAddr: 2 ipaddr %x\n",ipaddr);
					/* ipaddr is lan side global ip address in unnumbered-ip/napt case */
					return useSession_t;/* for Lan Side IP-Unnumbered */
				}			
			}else if (psession_t->ip_t&&_rtl8651_isLanSideIpUnnumberedIpAddr(psession_t->ip_t->ipAddr) == TRUE){
				useSession_t = psession_t->session_t; /* for Lan Side IP-Unnumbered */
				if (useSession_t){
					//rtlglue_printf("_rtl8651_getPppoeSessionByIpAddr: 3 ipaddr %x\n",ipaddr);
					/* ipaddr is lan side global ip address in unnumbered-ip/no-napt case */
					return useSession_t;/* for Lan Side IP-Unnumbered */
				}
			}
			if (psession_t->ip_t && psession_t->ip_t->ipAddr == ipaddr){
				/* ipaddr is gw wan ip addr assigned by pppoe */			
				//rtlglue_printf("_rtl8651_getPppoeSessionByIpAddr: 4 ipaddr %x\n",ipaddr);			
				assert(psession_t->session_t);				
				return psession_t->session_t;
			}			
		}
	}
	//rtlglue_printf("_rtl8651_getPppoeSessionByIpAddr: 5 ipaddr %x\n",ipaddr);
	return (rtl8651_tblDrv_pppTable_t *)NULL;
	#endif
}

/*
@func uint32 * | rtl8651_getNextPppoeSession | Get the next session ID followed by current session ID
@parm uint32 * | curSessionId | The pointer indicate the current session ID, if the value is NULL. This means to get the first session ID.
@parm uint32 * | retSessionId | The pointer indicate the session ID to return, if the next session do exist.
@rvalue uint32 * | The pointer indicate the next session ID value.
@comm
	retSessionId must point to a valid uint32 variable or function will reurn NULL. 
	When curSessionId is NULL, the returned pointer point to the variable store the first session ID.
	If curSessionId is specified, the function will find the session with session ID equal to *curSessionId and return the entry following 
the corresponding one.
	If next session not exist, return NULL.
	
*/
uint32 * rtl8651_getNextPppoeSession(uint32 * curSessionId, uint32 *retSessionId) {
	uint32 tblIdx, returnThis;
	rtl8651_tblDrv_pppTable_t *pppoePtr;

	returnThis = 0;
	if(retSessionId == NULL)
		return NULL;//When no place to store the found session ID, return NULL.
	for(tblIdx=0, pppoePtr=&DrvTbl.pppoe[0]; tblIdx<RTL8651_PPPOE_NUMBER; (tblIdx)++, pppoePtr++)
		if (pppoePtr->valid == 1){
			if(curSessionId == NULL || returnThis) {
				*retSessionId = pppoePtr->sessionId;
				return retSessionId;
			}
			else if(*curSessionId == pppoePtr->sessionId)
				returnThis = 1;
		}
 	return NULL;
}

/*
@func int32 | rtl8651_getPppoeCounter | Get the counter of specified PPPoE session
@parm uint32 | curSessionId | The session ID of PPPoE to get statistics value.
@parm rtl865x_tblDrv_basicCounterParam_t * | counter | The pointer point to the statistic value storage
@rvalue SUCCESS | Session found and statistic values are stored in the counter
@rvalue FAILED | Session found and statistic values are stored in the counter
@comm
	retSessionId must point to a valid uint32 variable or function will reurn NULL. 
	When curSessionId is NULL, the returned pointer point to the variable store the first session ID.
	If curSessionId is specified, the function will find the session with session ID equal to *curSessionId and return the entry following 
the corresponding one.
	If next session not exist, return NULL.
	
*/
int32 rtl8651_getPppoeCounter(uint32 curSessionId, rtl865x_tblDrv_basicCounterParam_t * counter) {
	uint32 tblIdx;
	rtl8651_tblDrv_pppTable_t *pppoePtr;

	if(counter == NULL)
		return FAILED;
	for(tblIdx=0, pppoePtr=&DrvTbl.pppoe[0]; tblIdx<RTL8651_PPPOE_NUMBER; (tblIdx)++, pppoePtr++)
		if (pppoePtr->valid == 1 && curSessionId == pppoePtr->sessionId) {
			counter->rxBytes = pppoePtr->rxBytes;
			counter->rxPackets = pppoePtr->rxPackets;
			counter->rxErrors = pppoePtr->rxErrors;
			counter->txBytes = pppoePtr->txBytes;
			counter->txPackets = pppoePtr->txPackets;
			counter->drops = pppoePtr->drops;
			return SUCCESS;
		}
 	return SUCCESS;
}

/*
@func rtl8651_tblDrv_pppTable_t * | _rtl8651_getPppoeSessionByPppoeSessionId | Get the PPPoE session data structure by session ID stored in PPPoE header
@parm uint16 | pppoeSessionId | The session ID stored in PPPoE header
@rvalue rtl8651_tblDrv_pppTable_t * | The pointer point to the data structure store the session information
@comm
	
*/
rtl8651_tblDrv_pppTable_t * _rtl8651_getPppoeSessionByPppoeSessionId(uint16 pppoeSessionId) {
	uint32 tblIdx;
	rtl8651_tblDrv_pppTable_t *pppoePtr;

	for(tblIdx=0, pppoePtr=&DrvTbl.pppoe[0]; tblIdx<RTL8651_PPPOE_NUMBER; (tblIdx)++, pppoePtr++)
		if (pppoePtr->valid == 1 && pppoePtr->initiated ==1 && pppoeSessionId == pppoePtr->pppoeSessionId_) 
			return pppoePtr;
	return NULL;
}

ipaddr_t _rtl8651_getExtIpAddrByDsid(uint32 dsid) {

	if(_rtl8651_multiple_dsid==1)
	{
		uint32 i;
		rtl8651_tblDrvSession_t *psession_t;
		rtl8651_tblDrv_networkIntfTable_t *netif_t;
		
		netif_t = &DrvTbl.netIface[0];

		for(i=0; i<rtl8651_tblDrvPara.networkIntfTableSize; i++, netif_t++) {
			if (netif_t->valid == 0 ||netif_t->linkLayerType == RTL8651_LL_VLAN)
				continue;
			CTAILQ_FOREACH(psession_t, &netif_t->sessionHead, nextSession){
				if(psession_t->ip_t==NULL) continue;
				assert(IS4BYTEALIGNED((uint32)&psession_t->ip_t->ipAddr));			
				if(psession_t->session_t->dsid == dsid) return psession_t->ip_t->ipAddr;				
			}
		}
	}
	else
	{
		rtl8651_tblDrv_ipEntry_t *ipEntry_t = DrvTbl.iptbl;
		ipaddr_t tempip=0;
		int32 ipIdx;
		
		for(ipIdx = 0; ipIdx<RTL8651_IPTABLE_SIZE; ipIdx++, ipEntry_t++) {
			if (ipEntry_t->valid == 0)
				break;
			else if (ipEntry_t->extip&& ipEntry_t->type==IP_ATTR_NAPT)
			{			
				tempip=ipEntry_t->extip; 
				return tempip;
			}
		}		
	}
	return 0;	
}


int32 _rtl8651_isExtIpAddr(ipaddr_t extip) {
	
	if(_rtl8651_multiple_dsid==1)
	{
		uint32 i;
		rtl8651_tblDrvSession_t *psession_t;
		rtl8651_tblDrv_networkIntfTable_t *netif_t;	
		netif_t = &DrvTbl.netIface[0];

		for(i=0; i<rtl8651_tblDrvPara.networkIntfTableSize; i++, netif_t++) {
			if (netif_t->valid == 0 ||netif_t->linkLayerType == RTL8651_LL_VLAN)
				continue;
			CTAILQ_FOREACH(psession_t, &netif_t->sessionHead, nextSession){
				if(psession_t->ip_t==NULL) continue;
				assert(IS4BYTEALIGNED((uint32)&psession_t->ip_t->ipAddr));			
				if(psession_t->ip_t->ipAddr == extip) return TRUE;				
			}
		}
	}
	else
	{
		rtl8651_tblDrv_ipEntry_t *ipEntry_t=DrvTbl.iptbl;
		int32 ipIdx;	

		for(ipIdx = 0; ipIdx<RTL8651_IPTABLE_SIZE; ipIdx++, ipEntry_t++) 
		{
			if (ipEntry_t->valid == 0)
				break;
			else if (ipEntry_t->extip==extip && ipEntry_t->type==IP_ATTR_NAPT)
			{			
				return TRUE;
			}
		}
	}
	return FALSE;
	
}

int32 rtl8651_setSoftPppoeSessionHangUp(uint32 SessionID, int32 enable, uint32 sec, int32 (*p_callBack)(uint32)) {
	rtl8651_sessionHangUp_t param;
	int32 retval = FAILED;

	param.enable = enable;
	param.sessionID = SessionID;
	param.idleTimeout_sec = sec;
	param.idleTimeout_callBack = p_callBack;

	rtlglue_drvMutexLock();
	retval = _rtl8651_setSessionHangUp(&param);
	rtlglue_drvMutexUnlock();

	return retval;
}
/*
@func int32 | rtl8651_setPppoeSessionHangUp | Enable/Disable PPPoE hang up.
@parm uint32 | PPPoEID | PPPoE ID.
@parm int32 | enable | TRUE to enable auto hangup. FALSE to disable auto hangup.
@parm uint32 | sec | Time in second to auto hangup.
@parm int32 (*)(uint32) | p_callBack | Call back function.
@rvalue SUCCESS | Success.
@rvalue TBLDRV_EINVALIDINPUT | Invalid input parameter.
*/
int32 rtl8651_setPppoeSessionHangUp(uint32 SessionID, int32 enable, uint32 sec, int32 (*p_callBack)(uint32)) {
	rtl8651_sessionHangUp_t param;
	int32 retval = FAILED;

	param.enable = enable;
	param.sessionID = SessionID;
	param.idleTimeout_sec = sec;
	param.idleTimeout_callBack = p_callBack;

	rtlglue_drvMutexLock();
	retval = _rtl8651_setSessionHangUp(&param);
	rtlglue_drvMutexUnlock();

	return retval;
}

#define RTL8651_NETWORK_INTERFACE_APIs
/*=======================================================================================
 * RTL8651 Network Interface APIs:
 *	1. Internal APIs
 *		. _rtl8651_getNetworkIntfEntryByAddr()
 *		. _rtl8651_getNetworkIntfEntryByVid()
 *		. _rtl8651_getNetworkIntfLinkLayerType()
 *	2. External APIs
 *		. rtl8651_addNetworkIntf()
 *		. rtl8651_delNetworkIntf()
 *		. rtl8651_removeNetworkIntfLinkLayerType()
 *		. rtl8651_addExtNetworkInterface()
 *		. rtl8651_delExtNetworkInterface()
 *=======================================================================================*/
rtl8651_tblDrv_networkIntfTable_t * _rtl8651_getNetworkIntfEntryByAddr(ipaddr_t ipAddr) {
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	rtl8651_tblDrv_ipIntfEntry_t * ipIfPtr;
	uint32 ifIndex;

	//Request local network host, try to generate arp query
	for(ifIndex=0,netIfPtr=&DrvTbl.netIface[0]; ifIndex< rtl8651_tblDrvPara.networkIntfTableSize; ifIndex++, netIfPtr++){
		//if(netIfPtr->valid == 1 && (netIfPtr->linkLayerType == RTL8651_LL_VLAN || netIfPtr->linkLayerType == RTL8651_LL_WIPUNNUMBERED)) {//Entry match
		if(netIfPtr->valid == 1) {//Entry match
			SLIST_FOREACH(ipIfPtr, &(netIfPtr->IpHead), nextIp) {
				assert(ipIfPtr->ipAddr.slh_first);
				//rtlglue_printf("ipAddr=%x ipMask=%x in_ipAddr=%x\n",ipIfPtr->ipAddr.slh_first->ipAddr,ipIfPtr->ipMask,ipAddr );
				if((ipIfPtr->ipAddr.slh_first->ipAddr & ipIfPtr->ipMask) == (ipAddr & ipIfPtr->ipMask)) 
					{												
						return netIfPtr;//ipAddr belonged network interface is found
					}
			}
		}
	}
	return NULL;//Entry not found
}

rtl8651_tblDrv_networkIntfTable_t * _rtl8651_getNetworkIntfEntryByVid(uint16 vid) {
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	uint32 ifIndex;

#ifdef RTL865X_PSEUDO_VLAN
	{
		rtl8651_tblDrv_vlanTable_t *vlan_t = &DrvTbl.vlan[rtl8651_vlanTableIndex(vid)];
		if (vlan_t->valid && vlan_t->fvlan_t)
			vid = vlan_t->fvlan_t->vid;
	}
#endif

	for(ifIndex=0, netIfPtr=&DrvTbl.netIface[0];ifIndex< rtl8651_tblDrvPara.networkIntfTableSize; ifIndex++,netIfPtr++) {
		//if (netIfPtr->valid == 1 && netIfPtr->linkLayerType == RTL8651_LL_VLAN && netIfPtr->vid == vid)
		if (netIfPtr->valid == 1 &&  netIfPtr->vid == vid)
			return netIfPtr;
	}
	return NULL;//Entry not found
}


int8 _rtl8651_getNetworkIntfLinkLayerType(int8 *ifName, uint32 *llType, uint32 *llIndex) {
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	uint32 netIdx;

	for(netIdx=0, netIfPtr=&DrvTbl.netIface[0]; netIdx<rtl8651_tblDrvPara.networkIntfTableSize; netIdx++,netIfPtr++) {
		if (netIfPtr->valid == 1 &&	_strncmp(netIfPtr->ifName, ifName, RTL8651_NETINTF_NAME_LEN) == 0) {
			if (llType != NULL)
				*llType = netIfPtr->linkLayerType;
			if (llIndex != NULL)
				*llIndex = netIfPtr->vid;
			return SUCCESS;
		}
	}
	return FAILED;
}




static rtl8651_tblDrv_networkIntfTable_t * _rtl8651_addNetworkInterface(int8 *ifName) 
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	uint32 netIdx;

	if (GET_NETWORK_INTERFACE(ifName, NULL) != NULL)
		return (rtl8651_tblDrv_networkIntfTable_t *)NULL;
	for(netIdx=0,netif_t=DrvTbl.netIface; netIdx< rtl8651_tblDrvPara.networkIntfTableSize; netIdx++, netif_t++) 
		if(netif_t->valid == 0) {//Empty entry found and create Network interface
			_strncpy(netif_t->ifName, ifName, RTL8651_NETINTF_NAME_LEN);
			netif_t->linkLayerType 	= 0;
			netif_t->valid 			= 1;
			netif_t->vid 			= 0;
			netif_t->netMask 		= 0;
			netif_t->pppoeAge		= 0;
			netif_t->defAclAction	= RTL8651_ACL_PERMIT;
			netif_t->mtu			= 1500;
			SLIST_INIT(&(netif_t->IpHead));
			SLIST_INIT(&(netif_t->nextNat));
			SLIST_INIT(&(netif_t->nextNaptServerPort));
			CTAILQ_INIT(&netif_t->sessionHead);
			CTAILQ_INIT(&netif_t->inAclHead);
			CTAILQ_INIT(&netif_t->outAclHead);
			CTAILQ_INIT(&netif_t->policyQosHead);
			CTAILQ_INIT(&netif_t->inRateLimitHead);
			CTAILQ_INIT(&netif_t->outRateLimitHead);
			CTAILQ_INIT(&netif_t->mnqHead);
			_rtl8651_arrangeAllACL();
			return (rtl8651_tblDrv_networkIntfTable_t *)netif_t;
		}
	return (rtl8651_tblDrv_networkIntfTable_t *)NULL;
}




/*
@func int32 | rtl8651_addNetworkIntf | Add a network interface.
@parm int8 * | ifName | Network interface name.
@rvalue SUCCESS | A netwrok interface was successfully added.
@rvalue TBLDRV_EINVNETIFNAME | Invalud network interface name.
@rvalue TBLDRV_ENOFREEBUFFER | Of of memory buffer.
@comm
At table driver initial time, the networkIntfTableSize parameter decides the number of allowable network interfaces. If exceeds the capability,
an error number, TBLDRV_ENOFREEBUFFER will be returned. Note that the VLAN and Network interface are one-to-one maping. It is not allowed that
more than one network interface attach to a VLAN.
*/
int32 rtl8651_addNetworkIntf(int8 *ifName) {
	int32 retval = SUCCESS;
	rtlglue_drvMutexLock();//Lock resource
	if (_rtl8651_addNetworkInterface(ifName) == NULL)
		retval = TBLDRV_ENOFREEBUFFER;
	rtlglue_drvMutexUnlock();//Unlock resource
	return retval;
}

/*
@func int32 | rtl8651_delNetworkIntf | Remove a network interface.
@parm int8 * | ifName | Network interface name.
@rvalue SUCCESS | The specified network interface was successfully removed.
@rvalue TBLDRV_EINVNETIFNAME | Invalid network interface name.
@rvalue TBLDRV_ENETIFREFERENCEDBYIPIF | The specified network interface is being referenced by IP interface.
@rvalue TBLDRV_ENETIFREFBYROUTE | The specified network interface is being referenced by a routing entry.
@rvalue TBLDRV_ELINKTYPESHOULDBERESET | The link layer type of the specified network interface should be reset before removing.
@rvalue TBLDRV_ENETIFREFERENCEDBYACL | The specified network interface is referenced by ACL rule.
@comm
Remove an existing network interface. Note that a removing network interface should not be referenced.
*/
int32 rtl8651_delNetworkIntf(int8 *ifName) {
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	rtl8651_tblDrv_routeTable_t *rt;
	uint32 i, j;

	rtlglue_drvMutexLock();//Lock resource
	netIfPtr = GET_NETWORK_INTERFACE(ifName, &i);
	LOCK_ERROR_CHECK((netIfPtr==NULL), TBLDRV_EINVNETIFNAME);
	/* Not allow Network interface with IP interface to be removed */
	LOCK_ERROR_CHECK((netIfPtr->IpHead.slh_first), TBLDRV_ENETIFREFERENCEDBYIPIF);
#if 0
	/* Not allowd Network interface referenced by routing table to be removed */
	for(j=0,rt=&DrvTbl.route[0]; j<rtl8651_tblDrvPara.routingTableSize-1; j++, rt++) {
		LOCK_ERROR_CHECK((rt->valid == 1 && rt->networkIntfIndex == i), TBLDRV_ENETIFREFBYROUTE);
		}
#else
	/* Not allowed Network interface referenced by routing table to be removed */
	for(j=0, rt=DrvTbl.route; j<rtl8651_tblDrvPara.routingTableSize; j++, rt++) {
		if (rt->valid == 1 && rt->netif_t == netIfPtr) {
			rtlglue_drvMutexUnlock();
			return TBLDRV_ENETIFREFBYROUTE;
		}
	}

#endif
	/* The Link layer type should be reset */
	LOCK_ERROR_CHECK((netIfPtr->linkLayerType != 0), TBLDRV_ELINKTYPESHOULDBERESET);
	/* Acl rule should be empty */
	LOCK_ERROR_CHECK(
		(CTAILQ_FIRST(&netIfPtr->inAclHead) || CTAILQ_FIRST(&netIfPtr->outAclHead)),
		TBLDRV_ENETIFREFERENCEDBYACL
	);
	/* Ok! Invalidate the Network interface */
	netIfPtr->valid = 0;
	_rtl8651_arrangeAllACL();
	rtlglue_drvMutexUnlock();//Unlock resource
	return SUCCESS;
}





/*
@func int32 | rtl8651_addExtNetworkInterface | Change an internal network interface to external network interface.
@parm int8 * | ifName | Network interface name.
@rvalue SUCCESS | The specified network interface was successfully changed to external interface.
@rvalue TBLDRV_ENAPTSTART | Can not add the specified network itnerface to external network itnerface because the NAPT is running.
@rvalue TBLDRV_EINVNETIFNAME | Invalid network interface name.
@rvalue TBLDRV_ENETIFALREADYEXTERNAL | The specified network interface already is an external interface.
@rvalue TBLDRV_ENOFREEBUFFER | Out of memory buffer.
*/
int32 rtl8651_addExtNetworkInterface(int8 *ifName) {
	rtl8651_tblDrv_ipIntfEntry_t * thisIpEntry;
	rtl8651_tblDrv_ipIntfIpAddrEntry_t *localServer;
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	rtl8651_tblDrv_vlanTable_t *vlanPtr;
	uint32 netIdx, ipCount, freeIpCount;
	uint16 vid;

	rtlglue_drvMutexLock();

	/* When NAPT start to operate, forbid new NAT configuration */
	LOCK_ERROR_CHECK(
		( (CTAILQ_TOTAL(&DrvTbl.inuseList.tcpudp)>0) ||
		  (rtl8651ExistedNaptIcmpFlows>0)
		), TBLDRV_ENAPTSTART
	);
	/* Find network interface index according to interface name */
	for(netIdx=0,netIfPtr=&DrvTbl.netIface[0], ipCount=0; netIdx<rtl8651_tblDrvPara.networkIntfTableSize; netIdx++,netIfPtr++)
		if(netIfPtr->valid == 1 && _strncmp(netIfPtr->ifName, ifName, RTL8651_NETINTF_NAME_LEN) == 0 && netIfPtr->linkLayerType != 0) {
			/*
			 * If Specified interface is found, try to count those IPs which
			 * will occupy IP Table.
			 */
			SLIST_FOREACH(thisIpEntry, &(netIfPtr->IpHead), nextIp) {
				assert(SLIST_FIRST(&thisIpEntry->ipAddr));
				SLIST_FOREACH(localServer, &(thisIpEntry->localServer), nextIpAddr) 
					ipCount++;
			}
			break; /* Entry found */
			}
	LOCK_ERROR_CHECK((netIdx==rtl8651_tblDrvPara.networkIntfTableSize), TBLDRV_EINVNETIFNAME);
	vid = netIfPtr->vid;
	vlanPtr = &DrvTbl.vlan[rtl8651_vlanTableIndex(vid)];
	
	/* Check to see if the specified network interface is assigned before */
	LOCK_ERROR_CHECK((vlanPtr->internal == 0), TBLDRV_ENETIFALREADYEXTERNAL);
	LOCK_ERROR_CHECK(
		(netIfPtr->linkLayerType == RTL8651_LL_LIPUNNUMBERED), 
		TBLDRV_ELIPUNNUMBEREDNOTALLOW
	);	
	/* Count free ASIC IP Slot */
	freeIpCount = RTL8651_IPTABLE_SIZE - (naptIpCount + natIpCount + localServerCount);
	/* If ASIC has not enough IP Table slot for the rquest, just reject it */
	LOCK_ERROR_CHECK((freeIpCount < ipCount), TBLDRV_ENOFREEBUFFER);
	assert(vlanPtr->valid == 1);
	vlanPtr->internal = 0;
	RTL8651_SETASICVLAN(vlanPtr);

#if 1//CONFIG_RTL865XB_EXP_PPPOE_PASSTHRU
	// Create PPPoE Passthru protocol-based VLAN
	_rtl8651_setExternalInterfaceforPPPoEPassthru( &vlanPtr->macAddr, vid );
	if ( rtl8651_l2protoPassThrough & PASSTHRU_PPPOE ) _rtl8651_createProtocolBasedVLANforPPPoEPassthru();
#else //CONFIG_RTL865XB
#endif//CONFIG_RTL865XB

#if 1//CONFIG_RTL865XB_EXP_IPV6_PASSTHRU
	// Create IPv6 Passthru protocol-based VLAN
	_rtl8651_setExternalInterfaceforIPv6Passthru( &vlanPtr->macAddr, vid );
	if ( rtl8651_l2protoPassThrough & PASSTHRU_IPV6 ) _rtl8651_createProtocolBasedVLANforIPv6Passthru();
#else //CONFIG_RTL865XB
#endif//CONFIG_RTL865XB

#if 1//CONFIG_RTL865XB_EXP_IPX_PASSTHRU
	// Create IPX Passthru protocol-based VLAN
	_rtl8651_setExternalInterfaceforIPXPassthru( &vlanPtr->macAddr, vid );
	if ( rtl8651_l2protoPassThrough & PASSTHRU_IPX ) _rtl8651_createProtocolBasedVLANforIPXPassthru();
#else //CONFIG_RTL865XB
#endif//CONFIG_RTL865XB

#if 1//CONFIG_RTL865XB_EXP_NETBIOS_PASSTHRU
	// Create NETBIOS Passthru protocol-based VLAN
	_rtl8651_setExternalInterfaceforNetbiosPassthru( &vlanPtr->macAddr, vid );
	if ( rtl8651_l2protoPassThrough & PASSTHRU_NETBIOS ) _rtl8651_createProtocolBasedVLANforNetbiosPassthru();
#else //CONFIG_RTL865XB
#endif//CONFIG_RTL865XB

	/* Put all Local Server over this external IP interface into IP table */
	SLIST_FOREACH(thisIpEntry, &(netIfPtr->IpHead), nextIp) {
		SLIST_FOREACH(localServer, &(thisIpEntry->localServer), nextIpAddr) {
			rtl8651_tblDrv_ipEntry_t *retval;
			retval = _rtl8651_addIpEntry(localServer->ipAddr, localServer->ipAddr, IP_ATTR_LS, NULL);
			assert(retval != NULL);
		 }
	}
	/* Update ASIC operation layer */
	if(freeIpCount == RTL8651_IPTABLE_SIZE && ipCount > 0)
		rtl8651_setAsicOperationLayer(4);
	rtl8651_memcpy(extIfName,ifName,8);
	_rtl8651_arrangeAllACL();

	/* rearrange multicast port status */
	_rtl8651_arrangeMulticastPortStatus();
	
	rtlglue_drvMutexUnlock();	
	return SUCCESS;
}

/*
@func int32 | rtl8651_delExtNetworkInterface | Change an External network interface to internal interface.
@parm int8 * | ifName | Network interface name.
@rvalue SUCCESS | The specified external network itnerface was successfully changed to internal interface.
@rvalue TBLDRV_ENAPTSTART | Can not change the specified network itnerface to internal network itnerface because the NAPT is running.
@rvalue TBLDRV_EINVNETIFNAME | Invalid network interface name.
@rvalue TBLDRV_ESHOULDBEEXTIF | The network interface should be an external network interface.
@rvalue TBLDRV_ESHOULDNOTDMZEXT | The network interface should not be a DMZ netwrok interface.
@rvalue TBLDRV_ELIPUNNUMBEREDNOTALLOW | The specified network itnerface cannot be IP Unnumbered network.
@rvalue TBLDRV_ENETIFREFBYNATNAPT | The network interface is referenced by NAT or NAPT.
*/
int32 rtl8651_delExtNetworkInterface(int8 * ifName) {
	rtl8651_tblDrv_ipIntfEntry_t * ipintf_t;
	rtl8651_tblDrv_ipIntfIpAddrEntry_t *	ipAddr;
	rtl8651_tblDrv_ipIntfIpAddrEntry_t *	localServer;
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	rtl8651_tblDrv_vlanTable_t *vlanPtr;
	uint32 netIdx;
	uint16 vid;

	rtlglue_drvMutexLock();		
	/* When NAPT start to operate, forbid new NAT configuration */
	LOCK_ERROR_CHECK(
		( (CTAILQ_TOTAL(&DrvTbl.inuseList.tcpudp)>0) ||
		  (rtl8651ExistedNaptIcmpFlows>0)
		), TBLDRV_ENAPTSTART
	);
	/* Find out the specified network interface */
	netIfPtr = GET_NETWORK_INTERFACE(ifName, &netIdx);
	LOCK_ERROR_CHECK((netIfPtr==NULL), TBLDRV_EINVNETIFNAME);
	vid = netIfPtr->vid;
	vlanPtr = &DrvTbl.vlan[rtl8651_vlanTableIndex(vid)];
	/* Make sure the network interface is an external interface */
	LOCK_ERROR_CHECK((vlanPtr->internal == 1), TBLDRV_ESHOULDBEEXTIF);
	/* For LanSide IP-Unnumber checking */
	LOCK_ERROR_CHECK(
		(netIfPtr->linkLayerType == RTL8651_LL_LIPUNNUMBERED),
		TBLDRV_ELIPUNNUMBEREDNOTALLOW
	)
	/* When NAT list is not empty, stop external IP interface deletion */
	LOCK_ERROR_CHECK(
		( (netIfPtr->nextNat.slh_first ) ||
		  (netIfPtr->nextNaptServerPort.slh_first)
		), TBLDRV_ENETIFREFBYNATNAPT
	);
	/* If NAPT address is present, forbid the request */
	SLIST_FOREACH(ipintf_t, &(netIfPtr->IpHead), nextIp) {
		SLIST_FOREACH(ipAddr, &(ipintf_t->ipAddr), nextIpAddr) {
			LOCK_ERROR_CHECK(
				(_rtl8651_isNaptMapping(ipAddr->ipAddr) == TRUE), 
				TBLDRV_ENETIFREFBYNATNAPT
			);
		}
	}
	assert(netIfPtr->linkLayerType);
	assert(vlanPtr->valid == 1);
	vlanPtr->internal = 1;
	RTL8651_SETASICVLAN(vlanPtr);
	/* Remove IP address over this network from ASIC IP Table */
	SLIST_FOREACH(ipintf_t, &(netIfPtr->IpHead), nextIp) {		
		SLIST_FOREACH(localServer, &(ipintf_t->localServer), nextIpAddr) {
			_rtl8651_delIpEntry(IP_ATTR_LS, localServer->ipAddr, localServer->ipAddr);
		}
	}
	if(naptIpCount == 0 && natIpCount ==0 && localServerCount == 0)
		rtl8651_setAsicOperationLayer(3);
	_rtl8651_arrangeAllACL();

	/* rearrange multicast port status */
	_rtl8651_arrangeMulticastPortStatus();

	rtlglue_drvMutexUnlock();
	return SUCCESS;
}

/*
@func int32 | rtl8651_addIpUnnumbered | Set a network itnerface to be IP Unnumbered network.
@parm int8 * | wanIfName | WAN network itnerface name.
@parm int8 * | lanIfName | LAN network interface name.
@parm ipaddr_t | netMask | Network Mask.
@rvalue TBLDRV_EINVALIDINPUT | Invalid input parameter. Either <p wanIfName> is NULL or netmask=0xffffffff.
@rvalue TBLDRV_EONLYONEIPUNNUMBERISALLOWED | The whole system only allows one IP Unnumbered network.
@rvalue TBLDRV_NETIFTYPENOTMATCH | The external network interface should be PPPoE type and internal network itnerface should be VLAN type.
@rvalue TBLDRV_ENETIFINVALID | <p wanIfName> should be an external interface and <p lanIfName> should be an internal interface.
@rvalue TBLDRV_EWANIFCANNOTHAVEIPINTF | When set IP Unnumebred network, the external network interface cannot have IP interface attached.
@rvalue TBLDRV_EINVNETIFNAME | Invalid network interface name.
@rvalue TBLDRV_EEXTINTFSHOULDBEPPPOE | The specified external network interface should be PPPoE type.
@rvalue TBLDRV_ESHOULDBEEXTNETIF | The <p wanIfName> should be external network interface.
@comm
When configure IP Unnumbered network, an external network interface and an internal network interface should be prepared. The external network
interface type should be PPPoE and the internal network interface type should be VLAN. Meanwhile, the external network interface cannot have any
IP interface attached.
*/
int32 rtl8651_addIpUnnumbered(int8 *wanIfName, int8 *lanIfName, ipaddr_t netMask) {
	rtl8651_tblDrv_networkIntfTable_t *wnetif_t, *lnetif_t;
	rtl8651_tblDrv_vlanTable_t *vlan_t;

	rtlglue_drvMutexLock();
	LOCK_ERROR_CHECK(
		(!wanIfName || netMask == 0 || netMask == 0xffffffff),
		TBLDRV_EINVALIDINPUT
	);
	wnetif_t = GET_NETWORK_INTERFACE(wanIfName, NULL);
	lnetif_t = GET_NETWORK_INTERFACE(lanIfName, NULL);
	
	if (wnetif_t && lnetif_t) {
		/* Lan Side IP Unnumbered */
		LOCK_ERROR_CHECK(
			(lanSideIpUnnumberedLan || lanSideIpUnnumberedWan), 
			TBLDRV_EONLYONEIPUNNUMBERISALLOWED
		);
		LOCK_ERROR_CHECK(
			( (wnetif_t->linkLayerType != RTL8651_LL_PPPOE) ||
			  (lnetif_t->linkLayerType != RTL8651_LL_VLAN)
			), TBLDRV_NETIFTYPENOTMATCH
		);
		vlan_t = &DrvTbl.vlan[rtl8651_vlanTableIndex(wnetif_t->vid)];
		LOCK_ERROR_CHECK(
			( (DrvTbl.vlan[rtl8651_vlanTableIndex(lnetif_t->vid)].internal != 1) ||
			  (vlan_t->internal != 0)
			), TBLDRV_ENETIFINVALID
		);
		LOCK_ERROR_CHECK((SLIST_FIRST(&wnetif_t->IpHead)), TBLDRV_EWANIFCANNOTHAVEIPINTF);
		wnetif_t->linkLayerType = RTL8651_LL_LIPUNNUMBERED;
		lnetif_t->linkLayerType = RTL8651_LL_LIPUNNUMBERED;
		wnetif_t->netMask = netMask;
		lanSideIpUnnumberedWan = wnetif_t;
		lanSideIpUnnumberedLan = lnetif_t;
		vlan_t->ipAttached = 1;
		RTL8651_SETASICVLAN(vlan_t);
		rtlglue_drvMutexUnlock();
		return SUCCESS;	
	} else if (wnetif_t && !lnetif_t) {
		/* Wan Side IP Unnumbered */
		LOCK_ERROR_CHECK((wnetif_t->linkLayerType != RTL8651_LL_PPPOE), TBLDRV_EEXTINTFSHOULDBEPPPOE);
		LOCK_ERROR_CHECK(
			(DrvTbl.vlan[rtl8651_vlanTableIndex(wnetif_t->vid)].internal != 0), 
			TBLDRV_ESHOULDBEEXTNETIF
		);
		wnetif_t->linkLayerType = RTL8651_LL_WIPUNNUMBERED;
		rtlglue_drvMutexUnlock();
		return SUCCESS;	
	}
	rtlglue_drvMutexUnlock();	
	return TBLDRV_EINVNETIFNAME;
}

/*
@func int32 | rtl8651_delIpUnnumbered | Remove IP Unnumbered network.
@parm int8 * | wanIfName | External Network interface of IP Unnumbered.
@parm int8 * | lanIfName | Internal Netoork interface of IP Unnumbered.
@rvalue SUCCESS | Teh IP Unnumbered network was successfully removed.
@rvalue TBLDRV_EINVALIDINPUT | The <p wanIfName> cannot be NULL.
@rvalue TBLDRV_ENETIFNEEDIPUNNUMBERED | The specified network interface should be IP Unnumbered.
@rvalue tBLDRV_EIPUNNUMBEREDHASHIPINTF | The IP Unnumebred stil has external IP interface attached. Hence forbid to remove it.
*/
int32 rtl8651_delIpUnnumbered(int8 *wanIfName, int8 *lanIfName) {
	rtl8651_tblDrv_networkIntfTable_t *wnetif_t, *lnetif_t;
	rtl8651_tblDrv_ipIntfEntry_t *ipintf_t;
	rtl8651_tblDrv_vlanTable_t *vlan_t;
	
	rtlglue_drvMutexLock();
	LOCK_ERROR_CHECK((!wanIfName), TBLDRV_EINVALIDINPUT);
	wnetif_t = GET_NETWORK_INTERFACE(wanIfName, NULL);
	lnetif_t = GET_NETWORK_INTERFACE(lanIfName, NULL);
	
	if (wnetif_t && lnetif_t) {
		/* Lan Side IP Unnumbered */
		LOCK_ERROR_CHECK(
			(!lanSideIpUnnumberedLan || !lanSideIpUnnumberedWan),
			TBLDRV_ENETIFNEEDIPUNNUMBERED
		);
		LOCK_ERROR_CHECK(
			( (wnetif_t->linkLayerType != RTL8651_LL_LIPUNNUMBERED) ||
			  (lnetif_t->linkLayerType != RTL8651_LL_LIPUNNUMBERED)
			), TBLDRV_ENETIFNEEDIPUNNUMBERED
		);
		vlan_t = &DrvTbl.vlan[rtl8651_vlanTableIndex(wnetif_t->vid)];
		LOCK_ERROR_CHECK(
			( (DrvTbl.vlan[rtl8651_vlanTableIndex(lnetif_t->vid)].internal != 1) ||
			  (vlan_t->internal != 0)
			), TBLDRV_ENETIFNEEDIPUNNUMBERED
		);
		SLIST_FOREACH(ipintf_t, &lnetif_t->IpHead, nextIp) {
			LOCK_ERROR_CHECK((ipintf_t->lanSideExternal == 1), TBLDRV_EIPUNNUMBEREDHASHIPINTF);
			}
		wnetif_t->linkLayerType = RTL8651_LL_PPPOE;
		lnetif_t->linkLayerType = RTL8651_LL_VLAN;
		wnetif_t->netMask = 0;
		lanSideIpUnnumberedWan = NULL;
		lanSideIpUnnumberedLan = NULL;
		vlan_t->ipAttached = 0;
		RTL8651_SETASICVLAN(vlan_t);
		rtlglue_drvMutexUnlock();	
		return SUCCESS;

	} else if (wnetif_t && !lnetif_t) {
		/* Wan Side IP Unnumbered */
		LOCK_ERROR_CHECK(
			(wnetif_t->linkLayerType != RTL8651_LL_WIPUNNUMBERED),
			TBLDRV_EINVALIDINPUT
		);
		wnetif_t->linkLayerType = RTL8651_LL_PPPOE;
		rtlglue_drvMutexUnlock();	
		return SUCCESS;
	}
	rtlglue_drvMutexUnlock();	
	return TBLDRV_EINVNETIFNAME;
}

/*
@func int32 | rtl8651_getIpUnnumbered | Get IP Unnumbered Netmask and External network interface name.
@parm int8 * | wanIfName | External Network interface name of IP Unnumbered.
@parm int8 * | ifName | Point to IP Unnumbered external network itnerface name after the API call.
@parm ipaddr_t * | netMask | IP Unnumbered netmask after the API call.
@rvalue SUCCESS | Success.
@rvalue TBLDRV_EINVALIDINPUT | <p ifName> and <p netMask> parameters cannot be NULL.
@rvalue TBLDRV_EINVNETIFNAME | Invlaid network interface name.
@rvalue TBLDRV_ENETIFNOTIPUNNUMBERED | The specified network interface is not an IP Unnumbered Network interface.
@comm
For an IP-Unnumbered network, it must have a netmask and the netmask must not be 0xffffffff. The API <p rtl8651_getIpUnnumbered()>
returns the netmask information and the external network interface name of IP Unnumbered.
*/
int32 rtl8651_getIpUnnumbered(int8 *wanIfName, int8 *ifName, ipaddr_t *netMask) {
	rtl8651_tblDrv_networkIntfTable_t *wnetif_t;

	UNLOCK_ERROR_CHECK((!ifName || !netMask), TBLDRV_EINVALIDINPUT);
	rtlglue_drvMutexLock();
	wnetif_t = GET_NETWORK_INTERFACE(wanIfName, NULL);
	LOCK_ERROR_CHECK((wnetif_t==NULL), TBLDRV_EINVNETIFNAME);
	LOCK_ERROR_CHECK(
		(wnetif_t->linkLayerType < RTL8651_LL_WIPUNNUMBERED), 
		TBLDRV_ENETIFNOTIPUNNUMBERED
	);
	if (wnetif_t->linkLayerType == RTL8651_LL_WIPUNNUMBERED)
		_strncpy(ifName, wnetif_t->ifName, RTL8651_NETINTF_NAME_LEN);
	else _strncpy(ifName, lanSideIpUnnumberedLan->ifName, RTL8651_NETINTF_NAME_LEN);
	*netMask = wnetif_t->netMask;
	rtlglue_drvMutexUnlock();
	return SUCCESS;
}




/*==========================================
  * GROUP: URL Filter API
  *==========================================*/
#define RTL865X_URL_FILTER_APIs

#ifdef RTL865XB_URLFILTER_UNKNOWNURLTYPE_SUPPORT
int32 rtl8651_urlUnknownTypeCaseSensitive(int8 enable)
{
	rtlglue_drvMutexLock();
	rtl8651_urlStringUnknownType_caseSensitive = enable;
	rtlglue_drvMutexUnlock();
	return SUCCESS;
}
#endif


/*
	Given a url filter data structure, 
	this internal API will lookup url filter database to find a matched entry.

	If no entry found, return -1.
*/
/*static*/ int32 _rtl8651_findUrlFiltering(rtl8651_tblDrv_urlFilter_t *urlFilter)
{	
	uint32 i, entry = -1;
	uint32 dsid = _rtl8651_SessionIDToDSID(urlFilter->sessionId);

	if ((urlFilter->urlFilterString == NULL || urlFilter->urlFilterStrLen == 0) && (urlFilter->pathFilterString == NULL || urlFilter->pathFilterStrLen == 0))	/* Invalid input */
		return entry;

	for (i = 0; i < rtl8651_tblDrvPara.urlfilterDBSize * rtl8651_tblDrvPara.dialSessionNumber; i++)
	{
		if (_urlFilter.urlFilterStrLen[i] == 0 && _urlFilter.pathFilterStrLen[i] == 0)	/* Note: skip empty entry. The following lookup algorithm bases on this assumption. */
			continue;
		
		if (dsid != _urlFilter.sessionId[i])	/* skip mis-matched dsid */
			continue;

#ifdef RTL865XB_URLFILTER_UNKNOWNURLTYPE_SUPPORT
		if (urlFilter->ruleType != _urlFilter.ruleType[i])	/* skip mis-matched ruleType */
			continue;
#endif

		/* (1) url string search */
		if (urlFilter->urlFilterString && urlFilter->urlFilterStrLen != 0)	/* url string is specified */
		{
			if ((_urlFilter.urlFilterDB[i] && _urlFilter.urlFilterStrLen[i] != 0) && _urlFilter.urlFilterStrLen[i] == urlFilter->urlFilterStrLen && 
				_urlFilter.urlExactlyMatch[i] == urlFilter->urlExactlyMatch)
			{
#ifdef RTL865XB_URLFILTER_UNKNOWNURLTYPE_SUPPORT
				if (urlFilter->ruleType == RTL8651_URLSTRING_UNKNOWNTYPE)
				{
					if ( (rtl8651_urlStringUnknownType_caseSensitive && _strncmp(_urlFilter.urlFilterDB[i], urlFilter->urlFilterString, urlFilter->urlFilterStrLen) == 0) ||
						( rtl8651_urlStringUnknownType_caseSensitive == 0 && _strncasecmp(_urlFilter.urlFilterDB[i], urlFilter->urlFilterString, urlFilter->urlFilterStrLen) == 0) )
						goto match_pathString;
					else
						continue;
				}
				else
#endif
				{
					/* host should be case-insensitive. */
					if ( _strncasecmp(_urlFilter.urlFilterDB[i], urlFilter->urlFilterString, urlFilter->urlFilterStrLen) == 0 )
						goto match_pathString;
					else
						continue;
				}
			}
			else	 
				continue;
		}
		else		/* url string is NOT specified */
		{
			if (_urlFilter.urlFilterDB[i] && _urlFilter.urlFilterStrLen[i] != 0)	/* Apparently, mis-match!! */
				continue;
			else
				goto match_pathString;
		}

match_pathString:	/* Only when url string is matched or don't-care, the program flow comes here. */

		/* (2) path string search */
		if (urlFilter->pathFilterString && urlFilter->pathFilterStrLen != 0)	/* path string is specified */
		{
			if ((_urlFilter.pathFilterDB[i] && _urlFilter.pathFilterStrLen[i] != 0) && _urlFilter.pathFilterStrLen[i] == urlFilter->pathFilterStrLen &&
				_urlFilter.pathExactlyMatch[i] == urlFilter->pathExactlyMatch &&
				_strncmp(_urlFilter.pathFilterDB[i], urlFilter->pathFilterString, urlFilter->pathFilterStrLen) == 0)
			{
				entry = i;		/* Got it! */
				goto out;
			}
			else	
				continue;
		}
		else		/* path string is NOT specified */
		{
			if (_urlFilter.pathFilterDB[i] && _urlFilter.pathFilterStrLen[i] != 0)	/* Apparently, mis-match!! */
				continue;
			else 
			{
				entry = i;		/* Got it! */
				goto out;
			}
		}
		
	}

out:
	return entry;

}

#if 0	/* Erroneous function */
static int32 _rtl8651_findUrlFiltering(rtl8651_tblDrv_urlFilter_t *urlFilter)
{
	uint32 entry;
	uint32 dsid = _rtl8651_SessionIDToDSID(urlFilter->sessionId);

	if (urlFilter->urlFilterString && urlFilter->pathFilterString) {		/* If both urlFilterString & pathFilterString are specified. */
		for (entry = 0; entry < rtl8651_tblDrvPara.urlfilterDBSize * rtl8651_tblDrvPara.dialSessionNumber; entry++) {
			if (_urlFilter.urlFilterStrLen[entry] == 0 || _urlFilter.pathFilterStrLen[entry] == 0)
				continue;
			if (_urlFilter.urlFilterStrLen[entry] != urlFilter->urlFilterStrLen || _urlFilter.pathFilterStrLen[entry] != urlFilter->pathFilterStrLen)
				continue;
			if (_urlFilter.urlExactlyMatch[entry] != urlFilter->urlExactlyMatch || _urlFilter.pathExactlyMatch[entry] != urlFilter->pathExactlyMatch)
				continue;
			if (_urlFilter.sessionId[entry] != dsid)
				continue;			
			if (_strncmp(_urlFilter.urlFilterDB[entry], urlFilter->urlFilterString, urlFilter->urlFilterStrLen) == 0 &&
				_strncmp(_urlFilter.pathFilterDB[entry], urlFilter->pathFilterString, urlFilter->pathFilterStrLen) == 0)	// path string should be the same 
				return (int32)entry;	// We find the string
		}
	}
	else if (urlFilter->urlFilterString) {		/* If only urlFilterString is specified. */
		for (entry = 0; entry < rtl8651_tblDrvPara.urlfilterDBSize * rtl8651_tblDrvPara.dialSessionNumber; entry++) {
			if (_urlFilter.urlFilterStrLen[entry] == 0)
				continue;
			if (_urlFilter.urlFilterStrLen[entry] != urlFilter->urlFilterStrLen)
				continue;
			if (_urlFilter.urlExactlyMatch[entry] != urlFilter->urlExactlyMatch)
				continue;
			if (_urlFilter.sessionId[entry] != dsid)
				continue;
			if (_strncmp(_urlFilter.urlFilterDB[entry], urlFilter->urlFilterString, urlFilter->urlFilterStrLen) == 0)
				return (int32)entry;	// We find the string 
		}
	}
	else if (urlFilter->pathFilterString) {		/* If only pathFilterString is specified. */
		for (entry = 0; entry < rtl8651_tblDrvPara.urlfilterDBSize * rtl8651_tblDrvPara.dialSessionNumber; entry++) {
			if (_urlFilter.pathFilterStrLen[entry] == 0)
				continue;
			if (_urlFilter.pathFilterStrLen[entry] != urlFilter->pathFilterStrLen)
				continue;
			if (_urlFilter.pathExactlyMatch[entry] != urlFilter->pathExactlyMatch)
				continue;
			if (_urlFilter.sessionId[entry] != dsid)
				continue;
			if (_strncmp(_urlFilter.pathFilterDB[entry], urlFilter->pathFilterString, urlFilter->pathFilterStrLen) == 0)
				return (int32)entry;	// We find the string 
		}
	}
	return -1;
}
#endif

static urlPktFiltering_t *_rtl8651_findUrlPktFiltering(int32 entry, rtl8651_tblDrv_urlPktFilter_t *filter, uint8 isExactlyMatch)
{
	urlPktFiltering_t *url_pf_t = NULL;

	if ((entry < 0) || filter == NULL)
		goto out;

	SLIST_FOREACH(url_pf_t, &(_urlFilter.pktFilter[entry]), nextUse)
	{
		if (filter->rule_type != url_pf_t->rule_type)
			continue;
	
		switch (url_pf_t->rule_type)
		{
			case RTL8651_URLPKTFILTER_TYPE_SIP:
				if (isExactlyMatch == TRUE)
				{
					if ((url_pf_t->sip_start==filter->sip_start) && (url_pf_t->sip_end==filter->sip_end))
						goto out;
				}else
				{	/* range match */
					if ((url_pf_t->sip_start<=filter->sip_start) && (url_pf_t->sip_end<=filter->sip_end))
						goto out;
				}
				break;
			case RTL8651_URLPKTFILTER_TYPE_SMAC:
				if (memcmp(&(url_pf_t->smac), &(filter->smac), sizeof(ether_addr_t)) == 0)
					goto out;
				break;
		}
	}


out:
	return url_pf_t;
}

static int32 _rtl8651_delUrlFiltering(rtl8651_tblDrv_urlFilter_t *urlFilter, rtl8651_tblDrv_urlPktFilter_t *pktFilter)
{
	int32 entry;
	rtl8651_tblDrv_urlPktFilter_t fake;

	if((!urlFilter->urlFilterString) && (!urlFilter->pathFilterString))
		return TBLDRV_EINVALIDINPUT;
	if ((urlFilter->urlFilterString && urlFilter->urlFilterStrLen == 0) || (urlFilter->pathFilterString && urlFilter->pathFilterStrLen == 0))
		return TBLDRV_EINVALIDINPUT;	
	if (urlFilter->urlFilterStrLen > rtl8651_tblDrvPara.urlfilterStringLen - 1)
		return TBLDRV_EURLFILTER_URLSTRINGLEN_EXCEEND;
	if (urlFilter->pathFilterStrLen > rtl8651_tblDrvPara.pathfilterStringLen - 1)	/* If path filter is specified, make sure its length is valid. */
		return TBLDRV_EURLFILTER_PATHSTRINGLEN_EXCEEND;
	if (pktFilter && pktFilter->rule_type == 0)
		return TBLDRV_EINVALIDINPUT;
#ifdef RTL865XB_URLFILTER_UNKNOWNURLTYPE_SUPPORT
	if (urlFilter->ruleType != RTL8651_URLSTRING_KNOWNTYPE && urlFilter->ruleType != RTL8651_URLSTRING_UNKNOWNTYPE)
		return TBLDRV_EURLFILTER_INVALIDRULETYPE;
	/* when unknown string type is specified, pathFilterString should be NULL. */
	if (urlFilter->ruleType == RTL8651_URLSTRING_UNKNOWNTYPE && urlFilter->pathFilterString != NULL)
		return TBLDRV_EURLFILTER_INVALIDRULETYPE;
#endif

	if ((entry = _rtl8651_findUrlFiltering(urlFilter)) >= 0)
	{
		urlPktFiltering_t *url_pf_t;

		if (pktFilter == NULL)
		{	/* fake a pseudo packet filter entry (corresponding for add API) */

			/* we fake a SIP route, from 0 ~ 0xfffffffff */
			fake.rule_type = RTL8651_URLPKTFILTER_TYPE_SIP;
			fake.sip_start = 0;
			fake.sip_end = 0xffffffff;
			pktFilter = &fake;
		}

		url_pf_t = _rtl8651_findUrlPktFiltering(entry, pktFilter, TRUE);

		if (url_pf_t)
		{	/* free this packet filter */
			SLIST_REMOVE(&(_urlFilter.pktFilter[entry]), url_pf_t, urlPktFiltering_s, nextUse);
			SLIST_REMOVE(&(DrvTbl.inuseList.urlPktFilter), url_pf_t, urlPktFiltering_s, next);
			SLIST_INSERT_HEAD(&(DrvTbl.freeList.freeUrlPktFilter), url_pf_t, next);
		} else
		{
			goto not_found;
		}

		if (SLIST_FIRST(&(_urlFilter.pktFilter[entry])) == NULL)
		{	/* no any packet filter, remove the url filter*/
			memset(_urlFilter.urlFilterDB[entry], 0, rtl8651_tblDrvPara.urlfilterStringLen);
			memset(_urlFilter.pathFilterDB[entry], 0, rtl8651_tblDrvPara.pathfilterStringLen);
			memset(_urlFilter.urlFilterJmpTbl[entry], 0, 256);
			memset(_urlFilter.pathFilterJmpTbl[entry], 0, 256);
			_urlFilter.urlFilterCount --;
			_urlFilter.urlFilterStrLen[entry] = 0;
			_urlFilter.pathFilterStrLen[entry] = 0;
			_urlFilter.urlExactlyMatch[entry] = 0;
			_urlFilter.pathExactlyMatch[entry] = 0;
			_urlFilter.sessionId[entry] = 0;
#ifdef RTL865XB_URLFILTER_UNKNOWNURLTYPE_SUPPORT
			_urlFilter.ruleType[entry] = 0;
#endif
		}
	}else
	{
		goto not_found;
	}

	if (_urlFilter.urlFilterCount == 0)
	{
#ifdef RTL865XB_URL_FILTER
		_rtl8651_arrangeAllACL();
#else
	//	if (_urlFilter.urlFilterCount == 0) {
	//		for(entry=0; entry<RTL8651_PORT_NUMBER; entry++) {
	//			retval = rtl8651_setAsicPortPatternMatch(entry,0, 0, 0);
	//			assert(retval == SUCCESS);
	//		}
	//	}
#endif
	}
	return SUCCESS;

not_found:
	return TBLDRV_EENTRYNOTFOUND;

}


static int32 _rtl8651_addUrlFiltering(rtl8651_tblDrv_urlFilter_t *urlFilter, rtl8651_tblDrv_urlPktFilter_t *pktFilter)
{
	int32 entry;
	urlPktFiltering_t *url_pf_t;
	rtl8651_tblDrv_urlPktFilter_t fake;

	if((!urlFilter->urlFilterString) && (!urlFilter->pathFilterString))
		return TBLDRV_EINVALIDINPUT;
	if ((urlFilter->urlFilterString && urlFilter->urlFilterStrLen == 0) || (urlFilter->pathFilterString && urlFilter->pathFilterStrLen == 0))
		return TBLDRV_EINVALIDINPUT;	
	if (urlFilter->urlFilterStrLen > rtl8651_tblDrvPara.urlfilterStringLen - 1)
		return TBLDRV_EURLFILTER_URLSTRINGLEN_EXCEEND;
	if (urlFilter->pathFilterStrLen > rtl8651_tblDrvPara.pathfilterStringLen - 1)	/* If path filter is specified, make sure its length is valid. */
		return TBLDRV_EURLFILTER_PATHSTRINGLEN_EXCEEND;
	if (pktFilter && pktFilter->rule_type == 0)
		return TBLDRV_EINVALIDINPUT;
#ifdef RTL865XB_URLFILTER_UNKNOWNURLTYPE_SUPPORT
	if (urlFilter->ruleType != RTL8651_URLSTRING_KNOWNTYPE && urlFilter->ruleType != RTL8651_URLSTRING_UNKNOWNTYPE)
		return TBLDRV_EURLFILTER_INVALIDRULETYPE;
	/* when unknown string type is specified, pathFilterString should be NULL. */
	if (urlFilter->ruleType == RTL8651_URLSTRING_UNKNOWNTYPE && urlFilter->pathFilterString != NULL)
		return TBLDRV_EURLFILTER_INVALIDRULETYPE;
#endif

	/* find exist url filter string */
	if ((entry = _rtl8651_findUrlFiltering(urlFilter)) >= 0)
	{
		/* string exist, we find url pkt filtering */
		if (pktFilter && ((url_pf_t = _rtl8651_findUrlPktFiltering(entry, pktFilter, TRUE)) != NULL))
		{	/* duplicate entry found, return SCCESS directly */
			return SUCCESS;
		}

		/* filter not found, we add new packet filter for this entry */
		goto add_pktFilter;
	}

	/* add new url filter string */
	for(entry = 0; entry < rtl8651_tblDrvPara.urlfilterDBSize * rtl8651_tblDrvPara.dialSessionNumber; entry++)
	{
		if (_urlFilter.urlFilterStrLen[entry] == 0 && _urlFilter.pathFilterStrLen[entry] == 0)
		{
			/* before add url filter string, we check if we can allocate new packet filter for this entry or not */
			if (SLIST_FIRST(&(DrvTbl.freeList.freeUrlPktFilter)) == NULL)
				return TBLDRV_ENOFREEBUFFER;
		
			goto add_urlFilter_and_pktFilter;
		}
	}
	/* no enough entry */
	return TBLDRV_ENOFREEBUFFER;



add_urlFilter_and_pktFilter:
	/*
		we have gotten the 'entry' of free url filter, we add it.
	*/
	if (urlFilter->urlFilterString && urlFilter->urlFilterStrLen != 0) {


#ifdef RTL865XB_URLFILTER_UNKNOWNURLTYPE_SUPPORT
		if (urlFilter->ruleType == RTL8651_URLSTRING_UNKNOWNTYPE && rtl8651_urlStringUnknownType_caseSensitive)
		_strncpy(_urlFilter.urlFilterDB[entry], urlFilter->urlFilterString, urlFilter->urlFilterStrLen);
		else
			__strlowerncpy(_urlFilter.urlFilterDB[entry], urlFilter->urlFilterString, urlFilter->urlFilterStrLen);
#else
		/* change host string to lower-case and save to url database */
		__strlowerncpy(_urlFilter.urlFilterDB[entry], urlFilter->urlFilterString, urlFilter->urlFilterStrLen);
		#endif

		_urlFilter.urlFilterDB[entry][urlFilter->urlFilterStrLen]	= '\0';
		_urlFilter.urlFilterStrLen[entry]		= urlFilter->urlFilterStrLen;
		_urlFilter.urlExactlyMatch[entry]		= urlFilter->urlExactlyMatch;
#ifdef RTL865XB_URLFILTER_UNKNOWNURLTYPE_SUPPORT
		if (urlFilter->ruleType == RTL8651_URLSTRING_UNKNOWNTYPE)
		{
			mBuf_getBMjmpTable(	(uint8 *)_urlFilter.urlFilterDB[entry], 
									(uint16 *)_urlFilter.urlFilterJmpTbl[entry], 
									urlFilter->urlFilterStrLen, 
									rtl8651_urlStringUnknownType_caseSensitive? TRUE: FALSE);
	}
		else
#endif
		{
			mBuf_getBMjmpTable((uint8 *)_urlFilter.urlFilterDB[entry], (uint16 *)_urlFilter.urlFilterJmpTbl[entry], urlFilter->urlFilterStrLen, FALSE);
		}
		}

	if (urlFilter->pathFilterString && urlFilter->pathFilterStrLen != 0) {
		_strncpy(_urlFilter.pathFilterDB[entry], urlFilter->pathFilterString, urlFilter->pathFilterStrLen);
		_urlFilter.pathFilterDB[entry][urlFilter->pathFilterStrLen] = '\0';
		_urlFilter.pathFilterStrLen[entry]		= urlFilter->pathFilterStrLen;
		_urlFilter.pathExactlyMatch[entry]	= urlFilter->pathExactlyMatch;
		mBuf_getBMjmpTable((uint8 *)_urlFilter.pathFilterDB[entry], (uint16 *)_urlFilter.pathFilterJmpTbl[entry], urlFilter->pathFilterStrLen, TRUE);
	}
	_urlFilter.sessionId[entry]		= _rtl8651_SessionIDToDSID(urlFilter->sessionId);
	_urlFilter.urlFilterCount++;	
#ifdef RTL865XB_URLFILTER_UNKNOWNURLTYPE_SUPPORT
	_urlFilter.ruleType[entry]		= urlFilter->ruleType;
#endif


	/*
		update other database for this modification
	*/
	if (_urlFilter.urlFilterCount == 1)
	{
#ifdef RTL865XB_URL_FILTER
		_rtl8651_arrangeAllACL();
#else
		union {
			char pat[4];
			uint32 pattern;
		}u;

		u.pat[0]='T';
		u.pat[1]='T';
		u.pat[2]='P';
		u.pat[3]='/';
		for(entry=0; entry<RTL8651_PORT_NUMBER; entry++) {
			int32 retval;
			retval = rtl8651_setAsicPortPatternMatch(entry,u.pattern, 0xffffffff, 0x2);
			assert(retval == SUCCESS);
		}
		/* Note: We always disables ASIC auto-learn */
#endif
	}

add_pktFilter:

	if (pktFilter == NULL)
	{	
		memset(&fake, 0, sizeof(rtl8651_tblDrv_urlPktFilter_t));
		
		/* we fake a SIP route, from 0 ~ 0xfffffffff */
		fake.rule_type = RTL8651_URLPKTFILTER_TYPE_SIP;
		fake.sip_start = 0;
		fake.sip_end = 0xffffffff;
		pktFilter = &fake;
		if ((url_pf_t = _rtl8651_findUrlPktFiltering(entry, pktFilter, TRUE)) != NULL)
			return SUCCESS;
	}

	if ((url_pf_t = SLIST_FIRST(&(DrvTbl.freeList.freeUrlPktFilter))) == NULL)
	{
		return TBLDRV_ENOFREEBUFFER;
	}

	SLIST_REMOVE(&(DrvTbl.freeList.freeUrlPktFilter), url_pf_t, urlPktFiltering_s, next);
	SLIST_INSERT_HEAD(&(DrvTbl.inuseList.urlPktFilter), url_pf_t, next);
	SLIST_INSERT_HEAD(&(_urlFilter.pktFilter[entry]), url_pf_t, nextUse);

	/* init entry */
	url_pf_t->rule_type = pktFilter->rule_type;
	url_pf_t->sip_start = pktFilter->sip_start;
	url_pf_t->sip_end = pktFilter->sip_end;
	memcpy(&(url_pf_t->smac), &(pktFilter->smac) ,sizeof(ether_addr_t));

	return SUCCESS;


}



/*
@func int32 | rtl8651_initURLfilter | Init pattern match string for url filtering feature.
@rvalue SUCCESS | Success.
@rvalue FAILED | Not success.
@comm
*/
static int32 _rtl8651_initURLfilter(void){

#ifdef RTL865XB_URL_FILTER
	_urlFilterAclRuleNum=-1;
	_rtl8651_arrangeAllACL();
#else

	union {
		char pat[4];
		uint32 pattern;
	}u;
	int32 i;
	
	u.pat[0]='T';
	u.pat[1]='T';
	u.pat[2]='P';
	u.pat[3]='/';
	for(i=0;i<RTL8651_PORT_NUMBER;i++){
		if(rtl8651_setAsicPortPatternMatch(i,u.pattern, 0xffffffff, 0x2 /* fwd to CPU */)!=SUCCESS)
			return FAILED;
	}
#endif
	return SUCCESS;
}




/*
@func int32 | rtl8651a_addURLfilterString | Add filter string to filter database.
@parm uint32 | sessionId | Session Id to add filter string.
@parm int8 * | string | String to be filtered.
@parm int32 | strlen | String length of string-to-be-filtered.
@rvalue SUCCESS | Success.
@rvalue FAILED | Not success.
@comm
strlen: NULL char NOT counted.
*/
int32 rtl8651a_addURLfilterString(uint32 sessionId, int8 *string, int32 strlen) 
{
	int32 retval;
	rtl8651_tblDrv_urlFilter_t urlFilter;
	
	bzero(&urlFilter, sizeof(rtl8651_tblDrv_urlFilter_t));
	urlFilter.urlFilterString = (int8 *)rtlglue_malloc(rtl8651_tblDrvPara.urlfilterStringLen);
	if (urlFilter.urlFilterString == NULL)
	{
		retval = TBLDRV_EMEMALLOCATEFAILEDFORURLFILTER;
		goto out;
	}
	_strncpy(urlFilter.urlFilterString, string, strlen);
	urlFilter.urlFilterStrLen = strlen;
	urlFilter.urlExactlyMatch = FALSE;
	urlFilter.sessionId = sessionId;

	rtlglue_drvMutexLock();
	retval = _rtl8651_addUrlFiltering(&urlFilter, NULL);
	rtlglue_drvMutexUnlock();

	/* free memory */
	rtlglue_free(urlFilter.urlFilterString);

out:
	return retval;
}

/*
@func int32 | rtl8651a_addURLExactlyfilterString | Add filter string to filter database.
@parm uint32 | sessionId | Session Id to add filter string.
@parm int8 * | string | String to be filtered.
@parm int32 | strlen | String length of string-to-be-filtered.
@rvalue SUCCESS | Success.
@rvalue FAILED | Not success.
@comm
The extended API is from <p rtl8651_addURLfilterString> and matches the string exactly.
Ex.
	Exactly match: abc.com

	Host name == [abc.com] ... match
	Host name == [www.abc.com] ... not match
	Host name == [abc.com.tw] ... not match
*/
int32 rtl8651a_addURLExactlyfilterString(uint32 sessionId, int8 *string, int32 strlen) 
{
	int32 retval;
	rtl8651_tblDrv_urlFilter_t urlFilter;

	bzero(&urlFilter, sizeof(rtl8651_tblDrv_urlFilter_t));
	urlFilter.urlFilterString = (int8 *)rtlglue_malloc(rtl8651_tblDrvPara.urlfilterStringLen);
	if (urlFilter.urlFilterString == NULL)
	{
		retval = TBLDRV_EMEMALLOCATEFAILEDFORURLFILTER;
		goto out;
	}
	_strncpy(urlFilter.urlFilterString, string, strlen);
	urlFilter.urlFilterStrLen = strlen;
	urlFilter.urlExactlyMatch = TRUE;
	urlFilter.sessionId = sessionId;
	
	rtlglue_drvMutexLock();
	retval = _rtl8651_addUrlFiltering(&urlFilter, NULL);
	rtlglue_drvMutexUnlock();

	/* free memory */
	rtlglue_free(urlFilter.urlFilterString);
out:
	return retval;
}


/*
@func int32 | rtl8651a_delURLfilterString | Delete filter string from filter database.
@parm uint32 | sessionId | Session Id to delete filter string.
@parm int8 * | string | String to be filtered.
@parm int32  | strlen | String length of string-to-be-filtered.
@rvalue SUCCESS | Success.
@rvalue FAILED | Not success.
@comm
strlen: NULL char NOT counted.
*/

int32 rtl8651a_delURLfilterString(uint32 sessionId, int8 *string, int32 strlen) 
{
	int32 retval;
	rtl8651_tblDrv_urlFilter_t urlFilter;

	bzero(&urlFilter, sizeof(rtl8651_tblDrv_urlFilter_t));
	urlFilter.urlFilterString = (int8 *)rtlglue_malloc(rtl8651_tblDrvPara.urlfilterStringLen);
	if (urlFilter.urlFilterString == NULL)
	{
		retval = TBLDRV_EMEMALLOCATEFAILEDFORURLFILTER;
		goto out;
	}
	_strncpy(urlFilter.urlFilterString, string, strlen);
	urlFilter.urlFilterStrLen = strlen;
	urlFilter.urlExactlyMatch = FALSE;
	urlFilter.sessionId = sessionId;
	
	rtlglue_drvMutexLock();
	retval = _rtl8651_delUrlFiltering(&urlFilter, NULL);
	rtlglue_drvMutexUnlock();

	/* free memory */
	rtlglue_free(urlFilter.urlFilterString);

out:
	return retval;
}

/*
@func int32 | rtl8651a_delURLExactlyfilterString | Delete filter string from exactly filter database.
@parm uint32 | sessionId | Session Id to delete filter string.
@parm int8 * | string | String to be filtered.
@parm int32  | strlen | String length of string-to-be-filtered.
@rvalue SUCCESS | Success.
@rvalue FAILED | Not success.
@comm
remove the url filter string added by <p rtl8651_addURLExactlyfilterString>
strlen: NULL char NOT counted.
*/

int32 rtl8651a_delURLExactlyfilterString(uint32 sessionId, int8 *string, int32 strlen) 
{
	int32 retval;
	rtl8651_tblDrv_urlFilter_t urlFilter;

	bzero(&urlFilter, sizeof(rtl8651_tblDrv_urlFilter_t));
	urlFilter.urlFilterString = (int8 *)rtlglue_malloc(rtl8651_tblDrvPara.urlfilterStringLen);
	if (urlFilter.urlFilterString == NULL)
	{
		retval = TBLDRV_EMEMALLOCATEFAILEDFORURLFILTER;
		goto out;
	}
	_strncpy(urlFilter.urlFilterString, string, strlen);
	urlFilter.urlFilterStrLen = strlen;
	urlFilter.urlExactlyMatch = TRUE;
	urlFilter.sessionId = sessionId;
	
	rtlglue_drvMutexLock();
	retval = _rtl8651_delUrlFiltering(&urlFilter, NULL);
	rtlglue_drvMutexUnlock();

	/* free memory */
	rtlglue_free(urlFilter.urlFilterString);

out:
	return retval;
}


/*
@func int32 | rtl8651_addURLFilterRule | add URL Filter Rule
@parm uint32 | sessionId | Session ID.
@parm int8 * | string | URL Filtering string.
@parm int32 | strlen | The length of URL Filtering string.
@parm ipaddr_t | ip_start | Specify IP Range. The starting IP address.
@parm ip_addr_t |ip_end | Specify IP Range. The ending IP address.
@rvalue TBLDRV_EINVALIDINPUT | The starting IP adress can not exceed ending IP address.
@rvalue TBLDRV_ENOFREEBUFFER | Out of memory.
@rvalue SUCCESS | Success API call.
@comm
Beside matching the specified URL string, this API also specify that packet's source IP address should in the user-specified
IP Range.
*/
int32 rtl8651_addURLFilterRule(uint32 sessionId, int8 *string, int32 strlen, ipaddr_t ip_start, ipaddr_t ip_end)
{
	int32 retval;
	rtl8651_tblDrv_urlFilter_t urlFilter;
	rtl8651_tblDrv_urlPktFilter_t pktFilter;

	/* Handling urlFilter */
	bzero(&urlFilter, sizeof(rtl8651_tblDrv_urlFilter_t));
	urlFilter.urlFilterString = (int8 *)rtlglue_malloc(rtl8651_tblDrvPara.urlfilterStringLen);
	if (urlFilter.urlFilterString == NULL)
	{
		retval = TBLDRV_EMEMALLOCATEFAILEDFORURLFILTER;
		goto out;
	}
	_strncpy(urlFilter.urlFilterString, string, strlen);
	urlFilter.urlFilterStrLen = strlen;
	urlFilter.urlExactlyMatch = FALSE;
	urlFilter.sessionId = sessionId;

	/* Handling packet filter */
	if (ip_start > ip_end) {
		retval = TBLDRV_EINVALIDINPUT;
		goto out1;
	}

	pktFilter.rule_type = RTL8651_URLPKTFILTER_TYPE_SIP;
	if (ip_start == 0 && ip_end == 0)
	{	/* not set */
		pktFilter.sip_start = 0;
		pktFilter.sip_end = 0xffffffff;
	}else
	{
		pktFilter.sip_start = ip_start;
		pktFilter.sip_end = ip_end;
	}

	rtlglue_drvMutexLock();
	retval = _rtl8651_addUrlFiltering(&urlFilter, &pktFilter);
	rtlglue_drvMutexUnlock();

out1:
	/* free memory */
	rtlglue_free(urlFilter.urlFilterString);
out:
	return retval;

}

/*
@func int32 | rtl8651_addURLExactlyFilterRule | add URL Filter Rule
@parm uint32 | sessionId | Session ID.
@parm int8 * | string | URL Filtering string.
@parm int32 | strlen | The length of URL Filtering string.
@parm ipaddr_t | ip_start | Specify IP Range. The starting IP address.
@parm ip_addr_t |ip_end | Specify IP Range. The ending IP address.
@rvalue TBLDRV_EINVALIDINPUT | The starting IP adress can not exceed ending IP address.
@rvalue TBLDRV_ENOFREEBUFFER | Out of memory.
@rvalue SUCCESS | Success API call.
@comm
The extended API is from <p rtl8651_addURLFilterRule> and matches the string exactly.
Ex.
	Exactly match: abc.com

	Host name == [abc.com] ... match
	Host name == [www.abc.com] ... not match
	Host name == [abc.com.tw] ... not match
*/
int32 rtl8651_addURLExactlyFilterRule(uint32 sessionId, int8 *string, int32 strlen, ipaddr_t ip_start, ipaddr_t ip_end)
{
	int32 retval;
	rtl8651_tblDrv_urlFilter_t urlFilter;
	rtl8651_tblDrv_urlPktFilter_t pktFilter;

	/* Handling urlFilter */
	bzero(&urlFilter, sizeof(rtl8651_tblDrv_urlFilter_t));
	urlFilter.urlFilterString = (int8 *)rtlglue_malloc(rtl8651_tblDrvPara.urlfilterStringLen);
	if (urlFilter.urlFilterString == NULL)
	{
		retval = TBLDRV_EMEMALLOCATEFAILEDFORURLFILTER;
		goto out;
	}
	_strncpy(urlFilter.urlFilterString, string, strlen);
	urlFilter.urlFilterStrLen = strlen;
	urlFilter.urlExactlyMatch = TRUE;
	urlFilter.sessionId = sessionId;

	/* Handling packet filter */
	if (ip_start > ip_end) {
		retval = TBLDRV_EINVALIDINPUT;
		goto out1;
	}
	
	pktFilter.rule_type = RTL8651_URLPKTFILTER_TYPE_SIP;
	if (ip_start == 0 && ip_end == 0)
	{	/* not set */
		pktFilter.sip_start = 0;
		pktFilter.sip_end = 0xffffffff;
	}else
	{
		pktFilter.sip_start = ip_start;
		pktFilter.sip_end = ip_end;
	}

	rtlglue_drvMutexLock();
	retval = _rtl8651_addUrlFiltering(&urlFilter, &pktFilter);
	rtlglue_drvMutexUnlock();

out1:
	/* free memory */
	rtlglue_free(urlFilter.urlFilterString);
out:
	return retval;
}

/*
@func int32 | rtl8651_delURLFilterRule | Remove a URL Filtering rule.
@parm uint32 | sessionId | Session ID.
@parm int8 * | string | URL Filtering string.
@parm int32 | strlen | The length of URL Filtering string.
@parm ipaddr_t | ip_start | Specify IP Range. The starting IP address.
@parm ip_addr_t |ip_end | Specify IP Range. The ending IP address.
@rvalue TBLDRV_EINVALIDINPUT | The starting IP adress can not exceed ending IP address.
@rvalue TBLDRV_EENTRYNOTFOUND | The specified entry was not found.
@comm
This API removes URL Filtering rules which were added by rtl8651_addURLFilterRule() API.
*/
int32 rtl8651_delURLFilterRule(uint32 sessionId, int8 *string, int32 strlen, ipaddr_t ip_start, ipaddr_t ip_end)
{
	int32 retval;
	rtl8651_tblDrv_urlFilter_t urlFilter;
	rtl8651_tblDrv_urlPktFilter_t pktFilter;

	/* Handling urlFilter */
	bzero(&urlFilter, sizeof(rtl8651_tblDrv_urlFilter_t));
	urlFilter.urlFilterString = (int8 *)rtlglue_malloc(rtl8651_tblDrvPara.urlfilterStringLen);
	if (urlFilter.urlFilterString == NULL)
	{
		retval = TBLDRV_EMEMALLOCATEFAILEDFORURLFILTER;
		goto out;
	}
	_strncpy(urlFilter.urlFilterString, string, strlen);
	urlFilter.urlFilterStrLen = strlen;
	urlFilter.urlExactlyMatch = FALSE;
	urlFilter.sessionId = sessionId;

	/* Handling packet filter */
	if (ip_start > ip_end) {
		retval = TBLDRV_EINVALIDINPUT;
		goto out1;
	}

	pktFilter.rule_type = RTL8651_URLPKTFILTER_TYPE_SIP;
	if (ip_start == 0 && ip_end == 0)
	{	/* not set */
		pktFilter.sip_start = 0;
		pktFilter.sip_end = 0xffffffff;
	}else
	{
		pktFilter.sip_start = ip_start;
		pktFilter.sip_end = ip_end;
	}

	rtlglue_drvMutexLock();
	retval = _rtl8651_delUrlFiltering(&urlFilter, &pktFilter);
	rtlglue_drvMutexUnlock();

out1:
	/* free memory */
	rtlglue_free(urlFilter.urlFilterString);
out:
	return retval;
}

/*
@func int32 | rtl8651_delURLExactlyFilterRule | Remove a URL exaclty Filtering rule.
@parm uint32 | sessionId | Session ID.
@parm int8 * | string | URL Filtering string.
@parm int32 | strlen | The length of URL Filtering string.
@parm ipaddr_t | ip_start | Specify IP Range. The starting IP address.
@parm ip_addr_t |ip_end | Specify IP Range. The ending IP address.
@rvalue TBLDRV_EINVALIDINPUT | The starting IP adress can not exceed ending IP address.
@rvalue TBLDRV_EENTRYNOTFOUND | The specified entry was not found.
@comm
This API removes URL Filtering rules which were added by rtl8651_addURLExactlyFilterRule() API.
*/
int32 rtl8651_delURLExactlyFilterRule(uint32 sessionId, int8 *string, int32 strlen, ipaddr_t ip_start, ipaddr_t ip_end)
{
	int32 retval;
	rtl8651_tblDrv_urlFilter_t urlFilter;
	rtl8651_tblDrv_urlPktFilter_t pktFilter;

	/* Handling urlFilter */
	bzero(&urlFilter, sizeof(rtl8651_tblDrv_urlFilter_t));
	urlFilter.urlFilterString = (int8 *)rtlglue_malloc(rtl8651_tblDrvPara.urlfilterStringLen);
	if (urlFilter.urlFilterString == NULL)
	{
		retval = TBLDRV_EMEMALLOCATEFAILEDFORURLFILTER;
		goto out;
	}
	_strncpy(urlFilter.urlFilterString, string, strlen);
	urlFilter.urlFilterStrLen = strlen;
	urlFilter.urlExactlyMatch = TRUE;
	urlFilter.sessionId = sessionId;

	/* Handling packet filter */
	if (ip_start > ip_end) {
		retval = TBLDRV_EINVALIDINPUT;
		goto out1;
	}

	pktFilter.rule_type = RTL8651_URLPKTFILTER_TYPE_SIP;
	if (ip_start == 0 && ip_end == 0)
	{	/* not set */
		pktFilter.sip_start = 0;
		pktFilter.sip_end = 0xffffffff;
	}else
	{
		pktFilter.sip_start = ip_start;
		pktFilter.sip_end = ip_end;
	}

	rtlglue_drvMutexLock();
	retval = _rtl8651_delUrlFiltering(&urlFilter, &pktFilter);
	rtlglue_drvMutexUnlock();

out1:
	/* free memory */
	rtlglue_free(urlFilter.urlFilterString);
out:
	return retval;
}

/*
@func int32 | rtl8651_registerURLFilterCallBackFunction | Register Callback function to process blocked URL HTTP-request
@parm rtl8651_tblDrv_urlFiltering_CallBackFunc_t | callBackFunc | callback function of url filter, NULL to disable URL filter .
@rvalue SUCCESS | This API always return SUCCESS
@comm
This API sets the CallBack Function into URL-Filter module to do further things for Blocked-URL-HTTP-request packets.
If URL-Filter module detect the URL is bloced, it would call this callback function.
If user want to dynamically disable the function, just call this API with <p callBackFunc> being NULL.
*/
int32 rtl8651_registerURLFilterCallBackFunction(rtl8651_tblDrv_urlFiltering_CallBackFunc_t callBackFunc)
{
	rtlglue_drvMutexLock();
	_urlFilter.callBackFunc = (urlFiltering_CallBackFunc_t)callBackFunc;
	rtlglue_drvMutexUnlock();
	return SUCCESS;
}

static int32 _rtl8651_checkGenericPktFilterRule(rtl8651_tblDrv_urlPktFilter_t *pktFilter)
{
	int32 retval = SUCCESS;
	if (pktFilter)
	{
		switch (pktFilter->rule_type)
		{
			case RTL8651_URLPKTFILTER_TYPE_SIP:
				if (pktFilter->sip_start > pktFilter->sip_end)
					retval = TBLDRV_EINVALIDINPUT;
				break;
			case RTL8651_URLPKTFILTER_TYPE_SMAC:
				break;
			default:
				retval = TBLDRV_EINVALIDINPUT;
		}
	}
	return retval;
}

/*
@func int32 | rtl8651_addGenericURLFilterRule | add URL Filter Rule using the generic form
@parm uint32 | sessionId | Session ID.
@parm int8 * | string | URL Filtering string.
@parm int32 | strlen | The length of URL Filtering string.
@parm uint8 | isExactlyMatch | Specify if this entry need exactly match or not.
@parm rtl8651_tblDrv_urlPktFilter_t* |pktFilter | Specify Packet information to filter.
@rvalue TBLDRV_EINVALIDINPUT | The filter information is invalid.
@rvalue TBLDRV_ENOFREEBUFFER | Out of memory.
@rvalue SUCCESS | Success API call.
@comm
This API can add URL filter rule with many different constraints,
including source IP range or SMAC.
If <p isExactlyMatch> != 0, then we add this entry to exactly match.
Ex.
	Exactly match: abc.com

	Host name == [abc.com] ... match
	Host name == [www.abc.com] ... not match
	Host name == [abc.com.tw] ... not match

*/
int32 rtl8651_addGenericURLFilterRule(rtl8651_tblDrv_urlFilter_t *urlFilter, rtl8651_tblDrv_urlPktFilter_t *pktFilter)
{
	int32 retval;

	if (!urlFilter)
		return TBLDRV_EINVALIDINPUT;
	if ((retval = _rtl8651_checkGenericPktFilterRule(pktFilter)) != SUCCESS)
		return retval;

	rtlglue_drvMutexLock();
	retval = _rtl8651_addUrlFiltering(urlFilter, pktFilter);
	rtlglue_drvMutexUnlock();
	return retval;
}

/*
@func int32 | rtl8651_delGenericURLFilterRule | Remove a URL Filtering rule using the generic form.
@parm uint32 | sessionId | Session ID.
@parm int8 * | string | URL Filtering string.
@parm int32 | strlen | The length of URL Filtering string.
@parm uint8 | isExactlyMatch | Specify if the entry which need to remove is exactly match or not.
@parm rtl8651_tblDrv_urlPktFilter_t* |pktFilter | Specify Packet information to remove.
@rvalue TBLDRV_EINVALIDINPUT | The filter information is invalid.
@rvalue TBLDRV_EURLFILTER_URLSTRINGLEN_EXCEEND | The string lengh exceeds the system constraint.
@rvalue TBLDRV_EENTRYNOTFOUND | The specified entry was not found.
@comm
This API removes URL Filtering rules.
*/
int32 rtl8651_delGenericURLFilterRule(rtl8651_tblDrv_urlFilter_t *urlFilter, rtl8651_tblDrv_urlPktFilter_t *pktFilter)
{
	int32 retval;

	if ((retval = _rtl8651_checkGenericPktFilterRule(pktFilter)) != SUCCESS)
		return retval;

	rtlglue_drvMutexLock();
	retval = _rtl8651_delUrlFiltering(urlFilter, pktFilter);
	rtlglue_drvMutexUnlock();
	return retval;
}


static int _rtl8651_initUrlFilterTrustedUserDb(void){


        if (rtl8651_tblDrvPara.urlfilterTrustedUserEntryCnt == 0)
        {
                return 0;
        }

        do { urlFilterTrustedUsers = (void*)rtlglue_malloc(rtl8651_tblDrvPara.urlfilterTrustedUserEntryCnt * sizeof(uint32)); if ( urlFilterTrustedUsers ) do { if (rtl8651_tblDrvPara.urlfilterTrustedUserEntryCnt * sizeof(uint32)) { memset(urlFilterTrustedUsers, 0, rtl8651_tblDrvPara.urlfilterTrustedUserEntryCnt * sizeof(uint32)); } } while (0);; } while (0);;

        if ( urlFilterTrustedUsers == ((void *)0) )
        {
                return -1;
        }

        _rtl8651_ReinitUrlFilterTrustedUserDb();

        return 0;

}

static void _rtl8651_ReinitUrlFilterTrustedUserDb(void){
        memset(urlFilterTrustedUsers,0,sizeof(uint32)*rtl8651_tblDrvPara.urlfilterTrustedUserEntryCnt);
}
int rtl8651_addUrlFilterTrustedUser(uint32 ipaddr){
        int i;
        rtlglue_drvMutexLock();
        for(i=0;i<rtl8651_tblDrvPara.urlfilterTrustedUserEntryCnt;i++)
        {
                if(urlFilterTrustedUsers[i]==0)
                {
                        urlFilterTrustedUsers[i]=ipaddr;
                        rtlglue_drvMutexUnlock();
                        return 0;
                }
        }
        rtlglue_drvMutexUnlock();
        return -1;
}







void rtl8651_flushUrlFilterTrustedUser(void){
        rtlglue_drvMutexLock();
        _rtl8651_ReinitUrlFilterTrustedUserDb();
        rtlglue_drvMutexUnlock();

}

static int _rtl8651_isUrlFilterTrustedUser(uint32 ipaddr){
        int i;
        for(i=0;i<rtl8651_tblDrvPara.urlfilterTrustedUserEntryCnt;i++)
        {
                if(urlFilterTrustedUsers[i]==0) break;
                if(urlFilterTrustedUsers[i]==ipaddr)
                        return 0;
        }
        return -1;
}




int32 _rtl8651_urlFilter(uint32 dsid, struct rtl_pktHdr *phdr, struct ip *pip, uint8 dir, int8 *string, int32 *strlen)
{
	uint8 httphdrPattern[5]="\r\n\r\n";
	uint8 urlHostPattern[9] = "\r\nHost: ";
	uint8 urlGetPattern[6] = "GET \\";
	uint8 urlGet1Pattern[5] = "GET ";
	uint8 urlhttpPattern[8] = "http://";
	uint8 pathGetPattern[5] = "GET /";
	int32 pos, entry=0, dn, match=0, count=0, c=0, m_len, shift;
	int32 path_dn, path_match=0, path_c=0, path_m_len, path_shift;
	uint8 *char_t, pchar, *match_t, action;
	uint8 *path_char_t, path_pchar, *path_match_t;
	
	assert(string && strlen);	

	pos = mBuf_BMpatternMatch(phdr->ph_mbuf, phdr->ph_len, (uint8*)httphdrPattern, 4, _urlFilter.urlHdrJmpTbl, TRUE);
	if (pos < 1) //no HTTP found
		return FAILED;

	/* ===============================
		Retrieve the host part of the url of this packet
	     =============================== */
	
	/* find keyword: "\r\nHost: " */
	if ((dn=mBuf_BMpatternMatch(phdr->ph_mbuf, pos, urlHostPattern, 8, _urlFilter.urlHostJmpTbl, TRUE)) >= 1) {
		shift = dn + 8;
		match_t = (uint8 *)&phdr->ph_mbuf->m_data[shift];		
	}
	/* Find keyword: "GET \" */
	else if ((dn=mBuf_BMpatternMatch(phdr->ph_mbuf, pos, urlGetPattern, 5, _urlFilter.urlGetJmpTbl, TRUE)) >= 1) {
		shift = dn + 5;
		match_t = (uint8 *)&phdr->ph_mbuf->m_data[shift];
	}
	/* Find keyword: "GET " */
	else if ((dn=mBuf_BMpatternMatch(phdr->ph_mbuf, pos, urlGet1Pattern, 4, _urlFilter.urlGet1JmpTbl, TRUE)) >= 1) {
		int32 dn1;
		shift = dn + 4;
		/* Find keyword: "http://" case-insensitively */
		if ((dn1 = mBuf_BMpatternMatch(phdr->ph_mbuf, pos, urlhttpPattern, 7, _urlFilter.urlhttpJmpTbl, FALSE)) == shift) {
			shift = dn1 + 7;
		match_t = (uint8 *)&phdr->ph_mbuf->m_data[shift];
	}
	else
		return FAILED;
	}
	else
		return FAILED;

	/* get the string length for url-filtering */
	assert( dn >= 1 && match_t);
	c = 0;
	m_len = phdr->ph_mbuf->m_len - dn + 1;
	char_t = match_t;
	pchar = *char_t;
	while(c < m_len &&  pchar != ' ' && pchar != '/' && pchar != '?' && pchar != '\\' && pchar != '\r' && pchar != '\n' && c < (_RTL8651_URLSTRBUFLEN-1)) {
		c++;
		pchar = *(++char_t);
	}

	if (c == 0)
		return FAILED;

	/* ===============================
		Retrieve the path part of the url of this packet
	     =============================== */

	/* CASE I: find the start position of abs_path (using keyword "GET /") */
	if ((path_dn = mBuf_BMpatternMatch(phdr->ph_mbuf, pos, pathGetPattern, 5, _urlFilter.pathGetJmpTbl, TRUE)) >= 1) {
		path_shift = path_dn + 5;
		path_match_t = (uint8 *)&phdr->ph_mbuf->m_data[path_shift];	// variable "path_match_t" is used to store the source string.

		/* Get path string length of this url. */
		path_c = 0;												// variable "path_c" is used to store source string length.
		path_m_len = phdr->ph_mbuf->m_len - path_dn + 1;			// variable "path_m_len" is used to store possible source string length.
		path_char_t = path_match_t;								// variable "path_char_t" is used to store the source string temporarily.
		path_pchar = *path_char_t;									// variable "path_pchar" is used to store the character to be checked.
		while(path_c < path_m_len && path_pchar != ' ') {
			path_c++;
			path_pchar = *(++path_char_t);
		}
	}
	/* CASE II: find the start position of path part of an absoluteURI (using keyword "GET ") */
	else if ((path_dn=mBuf_BMpatternMatch(phdr->ph_mbuf, pos, urlGet1Pattern, 4, _urlFilter.urlGet1JmpTbl, TRUE)) >= 1) {
		int32 path_dn1;
		path_shift = path_dn + 4;
		/* Find keyword: "http://" case-insensitively */
		if ((path_dn1 = mBuf_BMpatternMatch(phdr->ph_mbuf, pos, urlhttpPattern, 7, _urlFilter.urlhttpJmpTbl, FALSE)) == path_shift) {
			path_shift = path_dn1 + 7;
		path_char_t = (uint8 *)&phdr->ph_mbuf->m_data[path_shift];
		path_pchar = *path_char_t;
		while(path_pchar != '/') {
			path_shift++;
			path_pchar = *(++path_char_t);
		}
		path_match_t = (uint8 *)&phdr->ph_mbuf->m_data[path_shift];	// variable "path_match_t" is used to store the source string.

		/* Get path string length of this url. */
		path_c = 0;												// variable "path_c" is used to store source string length.
		path_m_len = phdr->ph_mbuf->m_len - path_shift + 1;			// variable "path_m_len" is used to store possible source string length.
		path_char_t = path_match_t;								// variable "path_char_t" is used to store the source string temporarily.
		path_pchar = *path_char_t;									// variable "pchar" is used to store the character to be checked.
		while(path_c < path_m_len && path_pchar != ' ') {
			path_c++;
			path_pchar = *(++path_char_t);
		}
	}
	else
		return FAILED;
	}
	else
		return FAILED;


	/* Traverse the url filter database to find out this packet should be filtered or not. */
	action = RTL8651_LOGACTION_FORWARD;


        if(dir==0)
        {
                if(_rtl8651_isUrlFilterTrustedUser(phdr->ph_sip)==0)
                {
                        goto _logging_it;
                }
        }
        else
        {
                if(_rtl8651_isUrlFilterTrustedUser(phdr->ph_dip)==0)
                {
                        goto _logging_it;
                }
        }
		
	
	for(entry = 0; entry < rtl8651_tblDrvPara.urlfilterDBSize * rtl8651_tblDrvPara.dialSessionNumber &&
	      count < _urlFilter.urlFilterCount; entry++) {

		int32 stringLen, path_stringLen, compareLen, path_compareLen;

		if (_urlFilter.urlFilterStrLen[entry] == 0 && _urlFilter.pathFilterStrLen[entry] == 0)
			continue;
		count++;
		if (_urlFilter.sessionId[entry] != dsid)
			continue;
		if (_urlFilter.urlExactlyMatch[entry] && c != _urlFilter.urlFilterStrLen[entry])
			continue;
		if (_urlFilter.pathExactlyMatch[entry] && path_c != _urlFilter.pathFilterStrLen[entry])
			continue;

			stringLen = c;
		path_stringLen = path_c;
			compareLen = _urlFilter.urlFilterStrLen[entry];
		path_compareLen = _urlFilter.pathFilterStrLen[entry];
#ifdef RTL865XB_URLFILTER_UNKNOWNURLTYPE_SUPPORT
		/* This URL rule type is UNKNOWN, we can't compare the keyword string with host part or path part of the URI specifically. */
		if ( _urlFilter.ruleType[entry] == RTL8651_URLSTRING_UNKNOWNTYPE )
		{
			/* match the string with the "path part" of the packet */
			path_match = -1;

			if ( compareLen <= path_stringLen )	/* The keyword string must be "shorter than / equal to" path part of the URI, otherwise no need to compare. */
			{
				phdr->ph_mbuf->m_data += path_shift;	/* change m_data pointer to start position of path part of the URI */
				phdr->ph_len -= path_shift;
				phdr->ph_mbuf->m_len -= path_shift;
				path_match = mBuf_BMpatternMatch(	phdr->ph_mbuf,
													path_stringLen,				/* path string length of the URI */
													_urlFilter.urlFilterDB[entry]		/* keyword string */,
													compareLen 					/* keyword string length */,
													_urlFilter.urlFilterJmpTbl[entry] 	/* jump table of keyword string */, 
													rtl8651_urlStringUnknownType_caseSensitive? TRUE: FALSE	);
				phdr->ph_mbuf->m_data -= path_shift;	/* restore m_data pointer to start position of the packet */
				phdr->ph_len += path_shift;
				phdr->ph_mbuf->m_len += path_shift;
		}

			/* the keyword string is matched with "path part" of the packet */
			if (path_match >= 0)
			{
				/* If packet filter is defined, we would try to match it */
				urlPktFiltering_t *url_pf_t;
				ipaddr_t ip_src;
				ether_addr_t mac_src;

				ip_src = phdr->ph_sip;
				memcpy(&mac_src, &(phdr->ph_mbuf->m_data[6]), sizeof(ether_addr_t));

				SLIST_FOREACH(url_pf_t, &(_urlFilter.pktFilter[entry]), nextUse) {
					switch (url_pf_t->rule_type) {
						case RTL8651_URLPKTFILTER_TYPE_SIP:
							if (ip_src >= url_pf_t->sip_start && ip_src <= url_pf_t->sip_end)
								goto _illegal_url_;
							break;
						case RTL8651_URLPKTFILTER_TYPE_SMAC:
							if (memcmp(&mac_src, &(url_pf_t->smac), sizeof(ether_addr_t)) == 0)
								goto _illegal_url_;
							break;
					}
				}
			}
			else		/* the keyword string is NOT matched with "path part" of the packet */
			{
				int32 prefixLen = 0;
				char_t = _urlFilter.urlFilterDB[entry];
				pchar = *char_t;

				while ( pchar != '/' )
				{
					prefixLen++;	/* count the number of characters which precede '/' */
					if ( prefixLen >= compareLen )	/* cannot find '/' character in the keyword string */
						break;
					pchar = *(++char_t);
				}

				/* if the string has a special character '/' */
				if ( prefixLen < compareLen )
				{
					if ( stringLen >= prefixLen )	/* if the "host part" of the URI is "longer than" / "equal to" the prefix keyword string, otherwise, no need to compare */
					{
						int32 postfixLen = 0;
						postfixLen = _urlFilter.urlFilterStrLen[entry] - prefixLen - 1;

						match = 0;
						
						if ( prefixLen != 0 )	/* if the prefix keyword string is not null, we are going to compare it with "host part" of the URI */
						{
							int32 host_shift = shift + stringLen - prefixLen;

							phdr->ph_mbuf->m_data += host_shift;
							/* These two strings must be the same. Otherwise, mis-match!! */
							match = _strncasecmp(phdr->ph_mbuf->m_data, _urlFilter.urlFilterDB[entry], prefixLen);
							phdr->ph_mbuf->m_data -= host_shift;
						}

						if ( match == 0 )	/* the prefix keyword string match! */
						{
							path_match = 0;
							
							phdr->ph_mbuf->m_data += path_shift;

							/* These two strings must be the same. Otherwise, mis-match!! */
							if (rtl8651_urlStringUnknownType_caseSensitive)
								path_match = _strncmp(phdr->ph_mbuf->m_data, ++char_t, postfixLen);
							else
								path_match = _strncasecmp(phdr->ph_mbuf->m_data, ++char_t, postfixLen);
							
							phdr->ph_mbuf->m_data -= path_shift;
							if ( path_match == 0 )
							{
								/* If packet filter is defined, we would try to match it */
								urlPktFiltering_t *url_pf_t;
								ipaddr_t ip_src;
								ether_addr_t mac_src;

								ip_src = phdr->ph_sip;
								memcpy(&mac_src, &(phdr->ph_mbuf->m_data[6]), sizeof(ether_addr_t));

								SLIST_FOREACH(url_pf_t, &(_urlFilter.pktFilter[entry]), nextUse) {
									switch (url_pf_t->rule_type) {
										case RTL8651_URLPKTFILTER_TYPE_SIP:
											if (ip_src >= url_pf_t->sip_start && ip_src <= url_pf_t->sip_end)
												goto _illegal_url_;
											break;
										case RTL8651_URLPKTFILTER_TYPE_SMAC:
											if (memcmp(&mac_src, &(url_pf_t->smac), sizeof(ether_addr_t)) == 0)
												goto _illegal_url_;
											break;
									}
								}

							}
						}
					}
				}
				else
				/* if the string does not have a special character '/', match the keyword string with the "host part" of the packet */
				{
					match = -1;
					if ( compareLen <= stringLen ) /* The keyword string must be "shorter than / equal to" host part of the URI, otherwise no need to compare. */
					{
						phdr->ph_mbuf->m_data += shift;
						phdr->ph_len -= shift;
						phdr->ph_mbuf->m_len -= shift;
						match = mBuf_BMpatternMatch(	phdr->ph_mbuf,
														stringLen,
														_urlFilter.urlFilterDB[entry],
														compareLen,
														_urlFilter.urlFilterJmpTbl[entry], FALSE);
						phdr->ph_mbuf->m_data -= shift;
						phdr->ph_len += shift;
						phdr->ph_mbuf->m_len += shift;
					}
					if (match >= 0)
					{
						/* If packet filter is defined, we would try to match it */
						urlPktFiltering_t *url_pf_t;
						ipaddr_t ip_src;
						ether_addr_t mac_src;

						ip_src = phdr->ph_sip;
						memcpy(&mac_src, &(phdr->ph_mbuf->m_data[6]), sizeof(ether_addr_t));

						SLIST_FOREACH(url_pf_t, &(_urlFilter.pktFilter[entry]), nextUse) {
							switch (url_pf_t->rule_type) {
								case RTL8651_URLPKTFILTER_TYPE_SIP:
									if (ip_src >= url_pf_t->sip_start && ip_src <= url_pf_t->sip_end)
										goto _illegal_url_;
									break;
								case RTL8651_URLPKTFILTER_TYPE_SMAC:
									if (memcmp(&mac_src, &(url_pf_t->smac), sizeof(ether_addr_t)) == 0)
										goto _illegal_url_;
									break;
							}
						}
					}
				}
			}

			
		}
		else		/* This URL rule type is KNOWN, we can compare the string with corresponding part of the packet. */
#endif /* RTL865XB_URLFILTER_UNKNOWNURLTYPE_SUPPORT */
		{
			if (c < _urlFilter.urlFilterStrLen[entry] && path_c < _urlFilter.pathFilterStrLen[entry])		/* we don't need to compare it and we know it would not match */
				continue;

		/* Start comparison of host part of the url. */
		match = 0;
		
		if (compareLen != 0) {
			phdr->ph_mbuf->m_data += shift;
			phdr->ph_len -= shift;
			phdr->ph_mbuf->m_len -= shift;
			match = mBuf_BMpatternMatch(	phdr->ph_mbuf,
											stringLen,
											_urlFilter.urlFilterDB[entry],
											compareLen,
												_urlFilter.urlFilterJmpTbl[entry], FALSE);
			phdr->ph_mbuf->m_data -= shift;
			phdr->ph_len += shift;
			phdr->ph_mbuf->m_len += shift;
		}

		if (match >= 0) {
			/* Start comparison of path part of the url. */
			path_match = 0;			/* variable "path_match" is used to indicate if this pkt matches any one of url path filter. */
			
			if (path_compareLen != 0) {
				phdr->ph_mbuf->m_data += path_shift;
				phdr->ph_len -= path_shift;
				phdr->ph_mbuf->m_len -= path_shift;
				path_match = mBuf_BMpatternMatch(	phdr->ph_mbuf,
													path_stringLen,
													_urlFilter.pathFilterDB[entry],
													path_compareLen,
														_urlFilter.pathFilterJmpTbl[entry], TRUE);
				phdr->ph_mbuf->m_data -= path_shift;
				phdr->ph_len += path_shift;
				phdr->ph_mbuf->m_len += path_shift;
			}
			if (path_match >= 0) {
				/* If packet filter is defined, we would try to match it */
				urlPktFiltering_t *url_pf_t;
				ipaddr_t ip_src;
				ether_addr_t mac_src;

				ip_src = phdr->ph_sip;
				memcpy(&mac_src, &(phdr->ph_mbuf->m_data[6]), sizeof(ether_addr_t));

				SLIST_FOREACH(url_pf_t, &(_urlFilter.pktFilter[entry]), nextUse) {
					switch (url_pf_t->rule_type) {
						case RTL8651_URLPKTFILTER_TYPE_SIP:
							if (ip_src >= url_pf_t->sip_start && ip_src <= url_pf_t->sip_end)
								goto _illegal_url_;
							break;
						case RTL8651_URLPKTFILTER_TYPE_SMAC:
							if (memcmp(&mac_src, &(url_pf_t->smac), sizeof(ether_addr_t)) == 0)
								goto _illegal_url_;
							break;
					}
				}
			}
		}
	}
	}
	goto _logging_it;

_illegal_url_:
	/* rule match: set action to DROP and log it */
	{
		int32 max = _urlFilter.urlFilterStrLen[entry] > (*strlen)-1 ? (*strlen-1) : _urlFilter.urlFilterStrLen[entry];
		assert(*strlen > 1);
		_strncpy(string, _urlFilter.urlFilterDB[entry], max);
		
		string[max] = '\0';
		*strlen = max;
		action = RTL8651_LOGACTION_DROP;

		/* For URLs being blocked and user-defined callback function exists, RomeDrv would call it for further process */
		if (_urlFilter.callBackFunc)
		{
			char url[_RTL8651_URLSTRBUFLEN] = {0};
			char path[_RTL8651_URLSTRBUFLEN] = {0};
			int32 urlMax = (c > (_RTL8651_URLSTRBUFLEN - 1)) ? (_RTL8651_URLSTRBUFLEN - 1) : c;

			_strncpy(url, match_t, urlMax);

			if (_urlFilter.pathFilterStrLen[entry] > 0)
			{
				int32 pathMax = (_urlFilter.pathFilterStrLen[entry] > (_RTL8651_URLSTRBUFLEN-1))?
								(_RTL8651_URLSTRBUFLEN-1):
								(_urlFilter.pathFilterStrLen[entry]);

				_strncpy(	path,
							_urlFilter.pathFilterDB[entry],
							pathMax);
			}
			_urlFilter.callBackFunc(dsid, phdr, pip, url, path);
		}
	}

_logging_it:

	assert(c > 0);
#if 0
	if (c == 0) {
		/* Try to match url pattern */
		m_len = phdr->ph_mbuf->m_len - dn + 1;
		char_t = match_t;
		pchar = *char_t;
		while(c<m_len &&  pchar!=' ' && pchar!='/' && pchar!='?' && pchar!='\\' && pchar!='\r' && pchar!='\n' && pchar<127) {
			c++;		
			pchar = *(++char_t);
		}
	}
#endif

#ifdef _RTL_LOGGING				
	{
		ipaddr_t sip  = phdr->ph_sip;
		ipaddr_t dip  = phdr->ph_dip;
		struct tcphdr *tc = NULL;
		uint16  sport = 0xffff;
		uint16  dport = 0xffff;

		if ((ntohs(pip->ip_off) & IP_OFFMASK) == 0)
		{
			tc=(struct tcphdr *) ((int8 *) pip + ((*(uint8*)pip&0xf) << 2));
			sport = ntohs(tc->th_sport);
			dport = ntohs(tc->th_dport);
		}

		memset(_urlFilter.urlString, 0, _RTL8651_URLSTRBUFLEN);
		rtl8651_memcpy((void*)_urlFilter.urlString, match_t, c>(_RTL8651_URLSTRBUFLEN-1)?(_RTL8651_URLSTRBUFLEN-1):c);	

                if ( path_c > 0 )
                {
                        memset(_urlFilter.log_pathString, 0, _RTL8651_URLSTRBUFLEN);
                        rtl8651_memcpy((void *)_urlFilter.log_pathString, path_match_t, path_c > (_RTL8651_URLSTRBUFLEN - 1)? (_RTL8651_URLSTRBUFLEN - 1): path_c);
                }

                if ( _urlFilter.urlFilterStrLen[entry] > 0 )
                {
                        memset(_urlFilter.log_urlFilterString, 0, _RTL8651_URLSTRBUFLEN);
                        rtl8651_memcpy((void *)_urlFilter.log_urlFilterString, _urlFilter.urlFilterDB[entry], _urlFilter.urlFilterStrLen[entry] > (_RTL8651_URLSTRBUFLEN - 1)? (_RTL8651_URLSTRBUFLEN - 1): _urlFilter.urlFilterStrLen[entry]);
                }

                if ( _urlFilter.pathFilterStrLen[entry] > 0 )
                {
                        memset(_urlFilter.log_pathFilterString, 0, _RTL8651_URLSTRBUFLEN);
                        rtl8651_memcpy((void *)_urlFilter.log_pathFilterString, _urlFilter.pathFilterDB[entry], _urlFilter.pathFilterStrLen[entry] > (_RTL8651_URLSTRBUFLEN - 1)? (_RTL8651_URLSTRBUFLEN - 1): _urlFilter.pathFilterStrLen[entry]);
                }


		
		if ((_loggingModule[dsid]&RTL8651_LOGMODULE_URL)&&_pUserLoggingFunc)
		{
#ifdef _RTL_NEW_LOGGING_MODEL
		rtl8651_logInfo_t info;
		info.infoType		= RTL8651_LOG_INFO_URL;
		info.url_dsid		= dsid;
		info.url_sip		= sip;
		info.url_dip		= dip;
		info.url_proto		= pip->ip_p;
		info.url_direction	= dir;
		info.url_sport		= sport;
		info.url_dport		= dport;
		info.url_string		= _urlFilter.urlString;

                if ( path_c > 0 )
                        info.un.url.pathString = _urlFilter.log_pathString;
                else
                        info.un.url.pathString = '\0';

                if ( _urlFilter.urlFilterStrLen[entry] > 0 )
                        info.un.url.urlFilterString = _urlFilter.log_urlFilterString;
                else
                        info.un.url.urlFilterString = '\0';

                if ( _urlFilter.pathFilterStrLen[entry] > 0 )
                        info.un.url.pathFilterString = _urlFilter.log_pathFilterString;
                else
                        info.un.url.pathFilterString = '\0';
				
		info.action		= action;
		(* _pUserLoggingFunc)(RTL8651_LOGMODULE_URL, RTL8651_LOG_URL_MatchUrlFilter, &info);
#else
			(*_pUserLoggingFunc)(
				dsid,
				RTL8651_LOGMODULE_URL,
				(unsigned char)pip->ip_p, /* protocol */
				(unsigned char)dir, /* direction */
				(unsigned long)sip,
				(unsigned long)dip,
				(unsigned short)sport,
				(unsigned short)dport,
				0, /* type: only meaningful for ICMP */
				action, /* action */
				_urlFilter.urlString/* msg */);
#endif
		}
	}
#endif /* _RTL_LOGGING */

	if (action == RTL8651_LOGACTION_DROP)
		return (shift+match);
	else 
		return FAILED;
}



#ifdef _RTL_LOGGING

/*
@func int32 | rtl8651_installLoggingFunction | install the logging callback function.
@parm void * | pMyLoggingFunc | Logging function pointer.
@rvalue SUCCESS | Success.
@rvalue TBLDRV_EINVALIDINPUT | Invalid input parameter.
*/
int32 rtl8651_installLoggingFunction(void * pMyLoggingFunc)
{
	if (pMyLoggingFunc==NULL)
		return TBLDRV_EINVALIDINPUT;
	_pUserLoggingFunc = (rtl8651_loggingFuncPtr_t)pMyLoggingFunc;
	return SUCCESS;
} /* end rtl8651_installLoggingFunction */

/*
@func int32 | rtl8651_enableLogging | Enable/Disable logging of specified module.
@parm uint32| moduleId | module identifier (see rtl861_tblDrv.h).
@parm int8  | enable | TRUE to enable logging and FALSE to disable it.
@rvalue SUCCESS | Success.
@rvalue TBLDRV_EINVALIDINPUT | Invalid input parameter.
*/
int32 rtl8651a_enableLogging(uint32 SessionID,uint32 moduleId,int8 enable){
	//if(dsid!=0) return FAILED;
	uint32 dsid=_rtl8651_SessionIDToDSID(SessionID);
	if (moduleId==0||(enable!=1&&enable!=0))
		return TBLDRV_EINVALIDINPUT;
	#if 1
	if (enable)
		_loggingModule[dsid] |= moduleId;
	else
		_loggingModule[dsid] &= ~moduleId;
	#endif
	return SUCCESS;
} /* rtl8651_enableLogging */

#endif /* _RTL_LOGGING */





/*==========================================
  * GROUP: IP Interface APIs
  *==========================================*/
#define RTL8651_IP_INTERFACE_APIs

static void _rtl8651_updateGidxRegister(void) {
#if 0
	rtl8651_tblDrv_ipEntry_t *ipEntry_t = DrvTbl.iptbl;
	uint32 i, j, idxCount, regValue;
	for(i=0, idxCount=0; i<RTL8651_IPTABLE_SIZE; i++, ipEntry_t++)
		if(ipEntry_t->valid == 1 && ipEntry_t->type==IP_ATTR_NAPT) {//NAPT external IP
			_gidxRegister[idxCount] = i;
			idxCount++;
		}
	for(i=idxCount; i<RTL8651_IPTABLE_SIZE; i++)
		_gidxRegister[i] = -1;
	assert(idxCount <= RTL8651_IPTABLE_SIZE);
	assert(idxCount == naptIpCount);
	regValue = 0;
	if(idxCount>0) {
		for(i=0, j=0; i<8; i++) {
			regValue |= (_gidxRegister[j]<<(3*i));
			j = (j+1)%idxCount;
		}
	}
#else
	uint32 i, regValue, def_napt_ipidx = 0;
	if (def_napt_t)
		def_napt_ipidx = def_napt_t - DrvTbl.iptbl;
	_gidxRegister[0] = def_napt_ipidx;	/* In RTL865x mode, we can only use one napt ip when using GIDX; use default one! */
	for (i = 1; i < RTL8651_IPTABLE_SIZE; i++)
		_gidxRegister[i] = -1;
	regValue = 0;
	for (i = 0; i < 8; i++)
		regValue |= (_gidxRegister[0] << (3 * i));
#endif

#if 0 //chhuang: Because the ASIC bug, we still need to use GIDX in version B
	{
		char chipVersion[16];
		rtl8651_getChipVersion(chipVersion, sizeof(chipVersion), NULL);
		if(chipVersion[strlen(chipVersion)-1]!='B'){
			rtl8651_setAsicGidxRegister(regValue);
		}
	}
#else
	rtl8651_setAsicGidxRegister(regValue);
#endif
	/* if Multiple nexthop route is present, rearrange ACL */
	if (multiNxtHop != NULL)
		_rtl8651_arrangeAllACL();
}


rtl8651_tblDrv_ipIntfEntry_t * _rtl8651_getIpIntfEntryByAddr(ipaddr_t ipAddr) {
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	rtl8651_tblDrv_ipIntfEntry_t * ipIfPtr;
	uint32 ifIndex;
	
	//Request local network host, try to generate arp query
	netIfPtr = &DrvTbl.netIface[0];
	for(ifIndex=0; ifIndex<rtl8651_tblDrvPara.networkIntfTableSize; ifIndex++,netIfPtr++) {
		if (netIfPtr->valid == 0)
			continue;
		SLIST_FOREACH(ipIfPtr, &(netIfPtr->IpHead), nextIp)
			if((ipIfPtr->ipAddr.slh_first->ipAddr & ipIfPtr->ipMask) == (ipAddr & ipIfPtr->ipMask)) 
				return ipIfPtr;//ipAddr belonged IP interface is found
	}
	return NULL;
}


rtl8651_tblDrv_networkIntfTable_t *
_rtl8651_getIpInfo(ipaddr_t ipaddr, rtl8651_tblDrv_ipIntfEntry_t **ipintf_t, int8 *is_gip, int8 *is_gwip)
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_ipIntfEntry_t *tmp_ipintf_t;
	rtl8651_tblDrv_ipIntfIpAddrEntry_t *ip_t;
	uint32 entry;

	/*
	  * Assumption:
	  * This function is used to the following information:
	  * 	(1) to which network the specified IP address belongs 
	  * 	(2) to which IP interface (subnet) the specified IP address belongs
	  * If the parameter, ipintf_t, can not be specified.
	  */

	netif_t = DrvTbl.netIface;
	for(entry=0; entry<rtl8651_tblDrvPara.networkIntfTableSize; entry++, netif_t++) {
		if (netif_t->valid == 0)
			continue;

// FIXME: IP unnumbered is changed to new model
#if 0
		if (netif_t->IpUn == RTL8651_UNNUMBER_WAN && (tmp_ipintf_t=SLIST_FIRST(&netif_t->IpHead))) {
			if ((ipaddr&tmp_ipintf_t->ipMask) == (tmp_ipintf_t->ipAddr.slh_first->ipAddr&tmp_ipintf_t->ipMask)) {
				if (ipintf_t) *ipintf_t = tmp_ipintf_t;
				goto found;
			}
			continue;
		}
#endif//0

		SLIST_FOREACH(tmp_ipintf_t, &netif_t->IpHead, nextIp) {
			/* find network interface and IP interface */
#if 0
			if (tmp_ipintf_t->lanSideExternal == 1)
				continue;
#endif//0
			if ((ipaddr&tmp_ipintf_t->ipMask) == (tmp_ipintf_t->ipAddr.slh_first->ipAddr&tmp_ipintf_t->ipMask)) {
				if (ipintf_t) *ipintf_t = tmp_ipintf_t;
				goto found;
			}
		}
	}
	return (rtl8651_tblDrv_networkIntfTable_t *)NULL;

found:

	if ( ipintf_t )
	{
		*ipintf_t = _rtl8651_getIpIntfEntryByAddr( ipaddr );
	}
	
	/* 
	 * Decide if external IP:
	 * This paramter can be NULL. If it is NULL, we ignore making decision; otherwise, if 
	 * the VLAN which the specified IP address attachs is external, then "TRUE" will be 
	 * returned.
	 */
	if ( is_gip )
	{
		if (tmp_ipintf_t->lanSideExternal == 1)
		{
			/* IP-unnumbered */
			*is_gip = TRUE;
		}
		else
		{
			if ( _rtl8651_isExtIpAddr( ipaddr ) )
				*is_gip = TRUE;
			else
				*is_gip = FALSE;
		}
	}
	
	/*
	 * Decide if gateway IP:
	 * This parameter can be NULL. If it is NULL, we ignore making decison; otherwise, if
	 * the IP is gateway IP address, then "TRUE" will be returned.
	 */
	if (is_gwip) {
		*is_gwip = FALSE;
		SLIST_FOREACH(ip_t, &tmp_ipintf_t->ipAddr, nextIpAddr) {
			if (ip_t->ipAddr == ipaddr) {
				*is_gwip = TRUE;
				break;
			}
		}
	}
	
	return (rtl8651_tblDrv_networkIntfTable_t *)netif_t;
}


/*
@func int32 | rtl8651_addIpIntf | Add an IP interface to a network interface.
@parm int8 * | ifName | Network interface name.
@parm ipaddr_t | ipAddr | Specify Gateway IP address.
@parm ipaddr_t | ipMask | Specify network mask.
@rvalue SUCCESS | An IP interface was successfully added.
@rvalue TBLDRV_ENETMASKCANNOTBEZERO | The netmask of IP interface cannot be zero.
@rvalue TBLDRV_EINVNETIFNAME | Invalid network interface name.
@rvalue TBLDRV_EUNKNOWLINKLAYERTYPE | The specified network interface has no link-layer type specified.
@rvalue TBLDRV_ENOFREEBUFFER | Out of memory buffer.
@rvalue TBLDRV_EIPUNEXTIFCANNOTHASHIPINTF | Cannot attach IP interface to External Network itnerface in an IP Unnumbered network.
@rvalue TBLDRV_EIPIFALREADYEXIST | The specified IP interface already exists.
@rvalue TBLDRV_EINVNETMASK | Invalid network mask. The specified netmask may be 0xffffffff in a VLAN type network.
@comm
Besides adding an IP interface to a specified netwrok interface, the <p rtl8651_addIpIntf()> also specifies the Gateway IP address
for the network interface. The Gateway IP address is specified by <p ipAddr> parameter and the network mask is derived from the logical
AND of <p ipAddr> and <p ipMask>.
*/

#if 1


static int32 _rtl8651_allocateArpEntry(rtl8651_tblDrv_networkIntfTable_t *netif_t, rtl8651_tblDrv_ipIntfEntry_t *ipintf_t)
{
	uint32 bestStartPos = 0xffffffff, bestSize=0, curStartPos, curSize, netSize = 0xffffffff;
	rtl8651_tblDrv_arpAsicArrangementEntry_t *asicArp;
	ether_addr_t mac = { {0xff, 0xff, 0xff, 0xff, 0xff, 0xff} };
	int32 entry;

	/* Calculate network size */
	for(entry=0; entry<32; entry++) {
		if(ipintf_t->ipMask & (1<<entry))	
			break;
	}
	/* Forbid 32-bits network mask secified if link layer type is RTL8651_LL_VLAN */
	if ((netSize = (1<<entry)) > 1) {
		/* Search for maximum free arp slots */
		bestSize = curSize = 0;
		curStartPos = 0;
		for(entry = 0; entry <= RTL8651_ARPARRANGE_NUM; entry++) {
			if(entry == RTL8651_ARPARRANGE_NUM || AsicTableUsage.arp[entry].valid == 1) {//End of empty slot
				if(curSize > bestSize) {
					bestStartPos = curStartPos;
					bestSize = curSize;
				}
				curStartPos = entry+1;
				curSize = 0;
			} else curSize++;
		}
	} else assert(0);


	/* Fill the occupied arp entry with current network information */
	if (netSize>1 && (bestSize<<3) >= netSize) {
		ipintf_t->arpAllocated = TRUE;
		ipintf_t->arpStartPos = bestStartPos;
		ipintf_t->arpEndPos = bestStartPos + (netSize>>3) - ((netSize&0x7)==0? 1: 0);
		asicArp = &AsicTableUsage.arp[bestStartPos];
		for(entry=ipintf_t->arpStartPos ; entry<=ipintf_t->arpEndPos; entry++, asicArp++) 
			asicArp->valid = 1;
		_rtl8651_addInterfaceRoute(netif_t, ipintf_t, ipintf_t->arpStartPos, ipintf_t->arpEndPos );
		/* add Broadcast arp for this subnet */
	 	_rtl8651_addLocalArp(TRUE, ((ipintf_t->ipAddr.slh_first->ipAddr&ipintf_t->ipMask)|~ipintf_t->ipMask), &mac, netif_t, 0xffffffff);
		return SUCCESS;
	} 
	else {
		ipintf_t->arpAllocated = FALSE;
		_rtl8651_addInterfaceRoute(netif_t, ipintf_t, 0xffffffff, 0xffffffff);
		return SUCCESS;
	}
}



static int32 _rtl8651_addIpInterface(rtl8651_tblDrv_networkIntfTable_t *netif_t, ipaddr_t ipAddr, ipaddr_t ipMask)
{
	rtl8651_tblDrv_ipIntfIpAddrEntry_t *ipPtr;
	rtl8651_tblDrv_ipIntfEntry_t *ipintf_t;
	rtl8651_tblDrv_vlanTable_t *vlan_t;
	rtl8651_tblDrv_routeTable_t *rt;
	uint32 entry, freeIpCount, ipIntfCount;
	int8 extIf=0;


	assert(netif_t);

#if 0
	if (ipMask == 0) 
		return TBLDRV_ENETMASKCANNOTBEZERO;
#endif

	/* Currently, only network interface with VLAN-type can be attached an ip interface with zero mask (infinitely large mask). */
	if (ipMask == 0 && netif_t->linkLayerType != RTL8651_LL_VLAN)
		return TBLDRV_ENETMASKCANNOTBEZERO;
	
	/* Forbid unknown-link layer Network IP interface assignment */
	if (netif_t->linkLayerType == 0)
		return TBLDRV_EUNKNOWLINKLAYERTYPE;

	/* Check to see if still have enough buffer */
	if (SLIST_FIRST(&DrvTbl.freeList.ipAddr) == NULL)
		return TBLDRV_ENOFREEBUFFER;

	/* Check whether the interface is an external IP interface */
	vlan_t = &DrvTbl.vlan[rtl8651_vlanTableIndex(netif_t->vid)];
	if (vlan_t->internal == 0) 
		extIf = TRUE;

	/* If the network interface is LandSide IP-Unnumbered type and is an external 
	  * interface, reject attach IP interface.
	  */
	if ( (vlan_t->internal == 0) && (netif_t->linkLayerType == RTL8651_LL_LIPUNNUMBERED)) 
		return TBLDRV_EIPUNEXTIFCANNOTHASHIPINTF;

	/* Check to see if this is an existed IP interface(subnet) */
	SLIST_FOREACH(ipintf_t, &(netif_t->IpHead), nextIp) {
		if(((ipintf_t->ipAddr.slh_first->ipAddr & ipintf_t->ipMask)==(ipAddr & ipMask)) && (ipintf_t->ipMask==ipMask)) {
			/* Check to see if duplicated IP address assignment */
	 		SLIST_FOREACH(ipPtr, &(ipintf_t->ipAddr), nextIpAddr) 
	 			UNLOCK_ERROR_CHECK((ipPtr->ipAddr == ipAddr), TBLDRV_EIPIFALREADYEXIST);
	 		/* Allocate new one */
			if ((ipPtr = SLIST_FIRST(&DrvTbl.freeList.ipAddr)) == NULL)
				return TBLDRV_ENOFREEBUFFER;
			SLIST_REMOVE_HEAD(&DrvTbl.freeList.ipAddr, nextIpAddr);
			SLIST_INSERT_HEAD(&ipintf_t->ipAddr, ipPtr, nextIpAddr);
			ipPtr->ipAddr = ipAddr;
			return SUCCESS;			
		}
	}

	/* Otherwise, no existed IP interface exists, Create one! */
	if (SLIST_FIRST(&DrvTbl.freeList.ipIface) == NULL)
		return TBLDRV_ENOFREEBUFFER;

	/* Search for available routing entry */
	for(entry=0,rt=&DrvTbl.route[0]; entry<RTL8651_ROUTINGTBL_SIZE-1; entry++,rt++) {
		if (rt->valid == 0)
			break;
	}
	/* If there's only one more ASIC routing entry... */
	if (entry  == RTL8651_ROUTINGTBL_SIZE-1 && ipMask != 0xffffffff && ipMask != 0)
		return TBLDRV_ENOFREEBUFFER;

	/* Allocate new IP interface And IP Adress*/
	if ((ipintf_t=SLIST_FIRST(&DrvTbl.freeList.ipIface))==NULL ||(ipPtr=SLIST_FIRST(&DrvTbl.freeList.ipAddr)) == NULL)
		return TBLDRV_ENOFREEBUFFER;

	if (ipMask != 0xffffffff || (netif_t->linkLayerType!=RTL8651_LL_VLAN && netif_t->linkLayerType!=RTL8651_LL_LIPUNNUMBERED)) {
		/* Attach IP interface and IP address to network interface */
		SLIST_REMOVE_HEAD(&DrvTbl.freeList.ipIface, nextIp);
		SLIST_INSERT_HEAD(&(netif_t->IpHead), ipintf_t, nextIp);
		SLIST_REMOVE_HEAD(&DrvTbl.freeList.ipAddr, nextIpAddr);
		SLIST_INSERT_HEAD(&(ipintf_t->ipAddr), ipPtr, nextIpAddr);

		/* Initial network content  */
		ipPtr->ipAddr = ipAddr;
		ipintf_t->ipMask = ipMask;
		ipintf_t->lanSideExternal = 0;
		ipintf_t->ipUnnumber = 0;
		SLIST_INIT(&ipintf_t->nextArp);
		SLIST_INIT(&ipintf_t->localServer);
	} else return TBLDRV_EINVNETMASK;
	
	switch (netif_t->linkLayerType)
	{
	case RTL8651_LL_VLAN:
	case RTL8651_LL_LIPUNNUMBERED:
		/* When adding an ip interface with zero mask (infinitely large mask), 
		     we don't have to add any interface route. */
		if (ipMask != 0)
			_rtl8651_allocateArpEntry(netif_t, ipintf_t);
	 	break;
	
	case RTL8651_LL_PPPOE:
		if (ipMask != 0xffffffff)
			return TBLDRV_EINVNETMASK;
		ipintf_t->arpAllocated = FALSE;
		break;

	case RTL8651_LL_L2TP:
	case RTL8651_LL_PPTP:
		/*
		  * This link-layer type allows the coexistence of Broadcast and tunnel network. If a
		  * non-0xffffffff Mask is specified, it is treated as broadcast network; otherwise is treated
		  * as tunneling network.
		  */
		if (ipMask != 0xffffffff)
			_rtl8651_allocateArpEntry(netif_t, ipintf_t);
		else 
			ipintf_t->arpAllocated = FALSE;
		break;
		
	default: assert(0);
	}
	
	
	/* Update operation layer */
	freeIpCount = RTL8651_IPTABLE_SIZE - naptIpCount - natIpCount - localServerCount;
	if (freeIpCount == RTL8651_IPTABLE_SIZE) {
		if (extIf == TRUE) {
			rtl8651_setAsicOperationLayer(4);
		}	
		else {
			netif_t = DrvTbl.netIface;
			ipIntfCount = 0;
			for(entry=0, ipIntfCount=0; entry<rtl8651_tblDrvPara.networkIntfTableSize; entry++, netif_t++) {
				if(netif_t->valid == 1) {
					SLIST_FOREACH(ipintf_t, &(netif_t->IpHead), nextIp)
						ipIntfCount++;
				}
			}
			if(ipIntfCount == 1) {//Add ip interface success and have one ip interface only means change from not IP interface to IP interface exist
				rtl8651_setAsicOperationLayer(3);
			}
		}
	} 
	/* Update rouitng enable field in Vlan Table */
	if (vlan_t->ipAttached == 0) {
		vlan_t->ipAttached = 1;
		RTL8651_SETASICVLAN(vlan_t);
	}			
	return SUCCESS;	
}


int32 _rtl8651_delIpInterface(rtl8651_tblDrv_networkIntfTable_t * netif_t, ipaddr_t ipAddr, ipaddr_t ipMask) 
{
	rtl8651_tblDrv_arpAsicArrangementEntry_t *asicArp;
	rtl8651_tblDrv_ipIntfIpAddrEntry_t *delIpPtr;
	rtl8651_tblDrv_ipIntfEntry_t * ipintf_t;
	rtl8651_tblDrv_vlanTable_t *vlan_t;
	rtl8651_tblDrv_arpEntry_t *arp_t;
	rtl8651_tblDrv_routeTable_t *rt;
	int32 entry, ipIntfCount;

	assert(netif_t);
	
	/* Find the specified IP interface */
	SLIST_FOREACH(ipintf_t, &(netif_t->IpHead), nextIp)		
		if((ipintf_t->ipAddr.slh_first->ipAddr & ipintf_t->ipMask) == (ipAddr & ipMask) && ipintf_t->ipMask == ipMask)
			break;
		
	/* Specified IP interface not exist */
	if (ipintf_t == NULL)
		return TBLDRV_ENOIPINTFFOUND;
	if (ipintf_t->lanSideExternal == 1) 
		return TBLDRV_ECANNOTREMOVEIPUNEXTIPINTF;
	vlan_t = &DrvTbl.vlan[rtl8651_vlanTableIndex(netif_t->vid)];

	/*
	  *	First, we remove all related multicast entry
	  */
	_rtl8651_delMulticastByVid(netif_t->vid);

	/*
	  * (1) If deletion will cause ip interface to be deleted, make sure
	  *      no arp entries or localSrever entries attach to it.
	  * (2) If deletion will cause ip interface to be deleted and a route 's nexthop
	  *      is referencing the IP interface, forbid the request!!
	  */
	if (ipintf_t->ipAddr.slh_first->nextIpAddr.sle_next == NULL) {
		if (SLIST_FIRST(&ipintf_t->localServer)) 
			return TBLDRV_EIPINTFISREFERENCEDBYLS; /* Case 1: */
		for(entry=0,rt=&DrvTbl.route[0]; entry<rtl8651_tblDrvPara.routingTableSize; entry++,rt++) {
			if (rt->valid && (rt->nextHop&ipMask)==(ipAddr&ipMask)) 
				return TBLDRV_EIPINTFISREFERENCEDBYRT; /* Case 2: */
		}
	}
	
	/* Find the specified IP address */
	SLIST_FOREACH(delIpPtr, &(ipintf_t->ipAddr), nextIpAddr)
		if (delIpPtr->ipAddr == ipAddr)
			break;
	/* Specified IP address not found */
	if (delIpPtr == NULL)
		return TBLDRV_ENOIPINTFFOUND;
	
	/* Check whetehr the interface is an external NAT/NAPT IP interface */
	if (vlan_t->internal == 0) {
		if (_rtl8651_isNaptMapping(ipAddr)==TRUE || _rtl8651_isNatMapping(&ipAddr, NULL)==TRUE)
			return TBLDRV_EIPINTFISREFERENCEDNYNATNAPT;
	}

	/* If the network is session-based network type , try to unbind IP and session ID */
	if (netif_t->linkLayerType != RTL8651_LL_VLAN)
		_rtl8651_unbindSession(delIpPtr, NULL);		
	 /* If only one IP address, remove all arp entries */
	if (ipintf_t->ipAddr.slh_first->nextIpAddr.sle_next == NULL) {
		  /* Remove all arps belong to the IP interface */
		  while ((arp_t=SLIST_FIRST(&ipintf_t->nextArp)))
		   	_rtl8651_delLocalArp(ARP_FORCE_DEL, arp_t->ipAddr, netif_t);
	 }
  
	/* Remove the specified IP address */
	SLIST_REMOVE(&(ipintf_t->ipAddr), delIpPtr, rtl8651_tblDrv_ipIntfIpAddrEntry_s, nextIpAddr);
	SLIST_INSERT_HEAD(&DrvTbl.freeList.ipAddr, delIpPtr, nextIpAddr);

	/* Remove the ip unnumber entry */
	if (ipintf_t->ipUnnumber) {
		SLIST_REMOVE(&DrvTbl.inuseList.ipUnnumber, ipintf_t->ipUnnumber, rtl8651_tblDrv_ipUnnumbered_s, nextIpUnnumberEntry);
		SLIST_INSERT_HEAD(&DrvTbl.freeList.freeIpUnnumber, ipintf_t->ipUnnumber, nextIpUnnumberEntry);
		ipintf_t->ipUnnumber = NULL;
	}
	
	/* If already no IP address attached on this IP interface, remove the IP interface */
	if(SLIST_FIRST(&ipintf_t->ipAddr) == NULL) {
		assert(ipintf_t->nextArp.slh_first == NULL);
		/* Remove arp and route arrangement */
		if(ipintf_t->arpAllocated) {//Remove ARP allocation
			uint32 arpPos;
			for(arpPos=ipintf_t->arpStartPos,asicArp=&AsicTableUsage.arp[arpPos]; arpPos<=ipintf_t->arpEndPos; arpPos++,asicArp++) 
				asicArp->valid = 0;
		}
		/* Remove the interface route */
		//if (netif_t->linkLayerType == RTL8651_LL_PPPOE) {
		if (ipintf_t->ipMask != 0xffffffff) {

		/* when RTL865X_DEBUG is not defined, 'retval' will be no used. */
#ifdef RTL865X_DEBUG 
			int32 retval = _rtl8651_delInterfaceRoute(ipAddr, ipMask);
			assert(retval == SUCCESS);
#else
			_rtl8651_delInterfaceRoute(ipAddr, ipMask);
#endif
		}
		
		//Remove ipIntfPtr from list
		SLIST_REMOVE(&(netif_t->IpHead), ipintf_t, rtl8651_tblDrv_ipIntfEntry_s, nextIp);
		SLIST_INSERT_HEAD(&DrvTbl.freeList.ipIface, ipintf_t, nextIp);	
		/* Update rouitng enable field in Vlan Table */		
		if (SLIST_FIRST(&netif_t->IpHead) == NULL) {
			vlan_t->ipAttached = 0;
			RTL8651_SETASICVLAN(vlan_t);	
		}		
	}


	/* Update operation layer */
	ipIntfCount = 0;
	netif_t = DrvTbl.netIface;
	for(entry=0 ; entry<rtl8651_tblDrvPara.networkIntfTableSize; entry++, netif_t++)
		if(netif_t->valid == 1) {
			SLIST_FOREACH(ipintf_t, &(netif_t->IpHead), nextIp) 
				ipIntfCount++;
		}
	if(ipIntfCount == 0) { //Delete ip interface success and have zero ip interface means change from last IP interface to no IP interface exist
		rtl8651_setAsicOperationLayer(2);
	}
	return SUCCESS;

}


/*
@func int32 | rtl8651_addIpIntf | Add an IP interface to a network interface.
@parm int8 * | ifName | Network interface name.
@parm ipaddr_t | ipAddr | Specify Gateway IP address.
@parm ipaddr_t | ipMask | Specify network mask.
@rvalue SUCCESS | An IP interface was successfully added.
@rvalue TBLDRV_ENETMASKCANNOTBEZERO | The netmask of IP interface cannot be zero.
@rvalue TBLDRV_EINVNETIFNAME | Invalid network interface name.
@rvalue TBLDRV_EUNKNOWLINKLAYERTYPE | The specified network interface has no link-layer type specified.
@rvalue TBLDRV_ENOFREEBUFFER | Out of memory buffer.
@rvalue TBLDRV_EIPUNEXTIFCANNOTHASHIPINTF | Cannot attach IP interface to External Network itnerface in an IP Unnumbered network.
@rvalue TBLDRV_EIPIFALREADYEXIST | The specified IP interface already exists.
@rvalue TBLDRV_EINVNETMASK | Invalid network mask. The specified netmask may be 0xffffffff in a VLAN type network.
@comm
Besides adding an IP interface to a specified netwrok interface, the <p rtl8651_addIpIntf()> also specifies the Gateway IP address
for the network interface. The Gateway IP address is specified by <p ipAddr> parameter and the network mask is derived from the logical
AND of <p ipAddr> and <p ipMask>.
*/
int32 rtl8651_addIpIntf(int8 * ifName, ipaddr_t ipAddr, ipaddr_t ipMask) 
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	int32 retval;
	rtlglue_drvMutexLock();
	if ((netif_t=GET_NETWORK_INTERFACE(ifName, NULL)) == NULL) 
		retval = TBLDRV_EINVNETIFNAME;
	else retval = _rtl8651_addIpInterface(netif_t, ipAddr, ipMask);
	rtlglue_drvMutexUnlock();
	return retval;
}


/*
@func int32 | rtl8651_delIpIntf | Remove an IP interface from the specified network interface.
@parm int8 * | ifName | Network interface name.
@parm ipaddr_t | ipAddr | Specify Gateway IP address.
@parm ipaddr_t | ipMask | Specify network mask.
@rvalue SUCCESS | The specified IP interface was successfully removed form the network interface.
@rvalue TBLDRV_EINVNETIFNAME | Invalid network interface name.
@rvalue TBLDRV_ECANNOTREMOVEIPUNEXTIPINTF | Global IP interface of IP Unnumbered cannot be removed. Use rtl8651_setLanSideExternalIpInterface() first.
@rvalue TBLDRV_EIPINTFISREFERENCEDBYARP | IP interface is referenced by ARP entry. It cannot be removed.
@rvalue TBLDRV_EIPINTFISREFERENCEDBYLS | IP interface is referenced by Local Server. It cannot be removed.
@rvalue TBLDRV_EIPINTFISREFERENCEDBYRT | IP interface is referenced by routing entry. It cannot be removed.
@rvalue TBLDRV_EIPINTFISREFERENCEDNYNATNAPT | IP interface is referenced by NAT/NAPT. It cannot be removed.
@rvalue TBLDRV_ENOIPINTFFOUND | The specified IP interface was not found.
@comm
The <p rtl8651_delIpIntf()> API removes an IP interface from the specified network interface. The <p ipAddr> specifies Gateway IP address.
*/
int32 rtl8651_delIpIntf(int8 * ifName, ipaddr_t ipAddr, ipaddr_t ipMask) 
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	int32 retval;
	rtlglue_drvMutexLock();
	if ((netif_t=GET_NETWORK_INTERFACE(ifName, NULL)) == NULL) 
		retval = TBLDRV_EINVNETIFNAME;
	else retval = _rtl8651_delIpInterface(netif_t, ipAddr, ipMask);
	rtlglue_drvMutexUnlock();
	return retval;

}


#else

int32 rtl8651_addIpIntf(int8 * ifName, ipaddr_t ipAddr, ipaddr_t ipMask) {
	uint32 netIdx, j, bestStartPos = 0xffffffff, bestSize, curStartPos, curSize, netSize = 0xffffffff;
	rtl8651_tblDrv_ipIntfEntry_t * newNetIntf, * ipIntfPtr;
	uint32 routePos, rtIdx, ipIntfCount, freeIpCount;
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr, *tmpNetPtr;
	rtl8651_tblDrv_routeAsicArrangementEntry_t *asicRT;
	rtl8651_tblDrv_arpAsicArrangementEntry_t *asicArp;
	rtl8651_tblDrv_vlanTable_t *vlanPtr;
	rtl8651_tblDrv_ipIntfIpAddrEntry_t *ipPtr;
	rtl8651_tblDrv_routeTable_t *rt;
	int8 extIf = FALSE, allocArp;
	uint16 vid;

	UNLOCK_ERROR_CHECK((ipMask == 0), TBLDRV_ENETMASKCANNOTBEZERO);
	rtlglue_drvMutexLock();
	netIfPtr = GET_NETWORK_INTERFACE(ifName, &netIdx);
	LOCK_ERROR_CHECK((netIfPtr==NULL), TBLDRV_EINVNETIFNAME);
	/* Forbid unknown-link layer Network IP interface assignment */
	LOCK_ERROR_CHECK((netIfPtr->linkLayerType == 0), TBLDRV_EUNKNOWLINKLAYERTYPE);
	/* Check to see if still have enough buffer */
	LOCK_ERROR_CHECK((SLIST_FIRST(&DrvTbl.freeList.ipAddr) == NULL), TBLDRV_ENOFREEBUFFER);
	/* Calculate usable ASIC IP entry */
	freeIpCount = RTL8651_IPTABLE_SIZE - naptIpCount - natIpCount - localServerCount;
	/* Check whether the interface is NAT external IP interface */
	vid = netIfPtr->vid;
	vlanPtr = &DrvTbl.vlan[rtl8651_vlanTableIndex(vid)];
	if (vlanPtr->internal == 0) {
		/* Added new network interface is NAT external IP interface. 
			 * However, no more free IP table entry is available.
			 */
		LOCK_ERROR_CHECK((freeIpCount == 0), TBLDRV_ENOFREEBUFFER);
		extIf = TRUE;
	}
	/* If the network interface is LandSide IP-Unnumbered type and is an external 
	 * interface, reject attach IP interface.
	 */
	LOCK_ERROR_CHECK(
		( (vlanPtr->internal == 0) &&
		  (netIfPtr->linkLayerType == RTL8651_LL_LIPUNNUMBERED)
		), TBLDRV_EIPUNEXTIFCANNOTHASHIPINTF
	);

	/* Check to see if this is an existed IP interface(subnet) */
	SLIST_FOREACH(ipIntfPtr, &(netIfPtr->IpHead), nextIp) {
		if((ipIntfPtr->ipAddr.slh_first->ipAddr & ipIntfPtr->ipMask) == (ipAddr & ipMask) && ipIntfPtr->ipMask == ipMask) {
			/* Check to see if duplicated IP address assignment */
	 		SLIST_FOREACH(ipPtr, &(ipIntfPtr->ipAddr), nextIpAddr) 
	 			LOCK_ERROR_CHECK((ipPtr->ipAddr == ipAddr), TBLDRV_EIPIFALREADYEXIST);
	 		/* Allocate new one */
			ipPtr = SLIST_FIRST(&DrvTbl.freeList.ipAddr);
			LOCK_ERROR_CHECK((ipPtr==NULL), TBLDRV_ENOFREEBUFFER);
			SLIST_REMOVE_HEAD(&DrvTbl.freeList.ipAddr, nextIpAddr);
			SLIST_INSERT_HEAD(&ipIntfPtr->ipAddr, ipPtr, nextIpAddr);
			ipPtr->ipAddr = ipAddr;
			rtlglue_drvMutexUnlock();//Unlock resource
			return SUCCESS;			
		}
	}
	
	/* Otherwise, no existed IP interface exists, Create one! */
	LOCK_ERROR_CHECK((SLIST_FIRST(&DrvTbl.freeList.ipIface) == NULL), TBLDRV_ENOFREEBUFFER);

	/* Search for available routing entry */
	for(rtIdx=0,rt=&DrvTbl.route[0]; rtIdx<RTL8651_ROUTINGTBL_SIZE-1; rtIdx++,rt++) {
		if (rt->valid == 0)
			break;
	}
	if (rtIdx==RTL8651_ROUTINGTBL_SIZE-1)
		return TBLDRV_ENOFREEBUFFER;


	/*
	 * 1. Because the interface route is created by IP interface creation, 
	 *    no pending route is allowed. 
	 * 2. The PPPoE type network interface creation won't casue a interface
	 *	  route to be added into ASIC/driver routing table.
	 */
	LOCK_ERROR_CHECK(
		( ((routePos==RTL8651_ROUTINGTBL_SIZE-1) ||
		  (rtIdx==rtl8651_tblDrvPara.routingTableSize-1)) &&
		  (netIfPtr->linkLayerType != RTL8651_LL_PPPOE)
		), TBLDRV_ENOFREEBUFFER
	);
	
	

	  
	if (netIfPtr->linkLayerType != RTL8651_LL_PPPOE) {
		/* Calculate network size */
		for(j=0; j<32; j++)
			if(ipMask & 1<<j)	break;
		netSize = 1<<j;
		if(netSize >1) {
			/* Search for maximum free arp slots */
			bestSize = curSize = 0;
			curStartPos = 0;
			for(j = 0; j <= RTL8651_ARPARRANGE_NUM; j++) {
				if(j == RTL8651_ARPARRANGE_NUM || AsicTableUsage.arp[j].valid == 1) {//End of empty slot
					if(curSize > bestSize) {
						bestStartPos = curStartPos;
						bestSize = curSize;
					}
					curStartPos = j+1;
					curSize = 0;
				} else curSize++;
			}
		}
		if (netSize>1 && (bestSize<<3) < netSize) //ARP table not enough to store new network
			allocArp = 0;
		else allocArp = 1;			
		/* Forbid 32-bits network mask secified if link layer type is RTL8651_LL_VLAN */
		LOCK_ERROR_CHECK((netSize == 1), TBLDRV_EINVNETMASK);
	}

	/* Allocate new IP interface And IP Adress*/
	newNetIntf = SLIST_FIRST(&DrvTbl.freeList.ipIface);
	ipPtr = SLIST_FIRST(&DrvTbl.freeList.ipAddr);
	LOCK_ERROR_CHECK((newNetIntf==NULL||ipPtr==NULL), TBLDRV_ENOFREEBUFFER);
	SLIST_REMOVE_HEAD(&DrvTbl.freeList.ipIface, nextIp);
	SLIST_INSERT_HEAD(&(netIfPtr->IpHead), newNetIntf, nextIp);
	SLIST_REMOVE_HEAD(&DrvTbl.freeList.ipAddr, nextIpAddr);
	SLIST_INSERT_HEAD(&(newNetIntf->ipAddr), ipPtr, nextIpAddr);
	
	/* Initial network content and corresponding arp and routing entry */
	newNetIntf->ipAddr.slh_first->ipAddr = ipAddr;
	newNetIntf->ipAddr.slh_first->nextIpAddr.sle_next = NULL;
	newNetIntf->ipMask = ipMask;
	newNetIntf->lanSideExternal = 0;
	SLIST_INIT(&newNetIntf->nextArp);
	SLIST_INIT(&newNetIntf->localServer);
	
	/* If the PPPoE type, no route to be added */
	if (netIfPtr->linkLayerType == RTL8651_LL_PPPOE) {
		newNetIntf->arpAllocated = 0;
		newNetIntf->networkType = RTL8651_LL_PPPOE;
		goto __out;
	}
		
	/* Allocate ARP entry */
	newNetIntf->arpAllocated = allocArp;
	newNetIntf->arpStartPos = bestStartPos;
	newNetIntf->arpEndPos = bestStartPos + (netSize>>3) - ((netSize&0x7)==0? 1: 0);//ASIC using 8 entry per slot, since the endPosition is included, must -1 to avoid starting poisition collision

	/* Fill the occupied arp entry with current network information */
	if (allocArp == 1) {
		for(j=newNetIntf->arpStartPos, asicArp=&AsicTableUsage.arp[j]; j<=newNetIntf->arpEndPos; j++, asicArp++)
			asicArp->valid = 1;
		_rtl8651_addInterfaceRoute(netIfPtr, newNetIntf, newNetIntf->arpStartPos, newNetIntf->arpEndPos );
	} else _rtl8651_addInterfaceRoute(netIfPtr, newNetIntf, 0xffffffff, 0xffffffff);
	
	newNetIntf->networkType = netIfPtr->linkLayerType;

	rt->rt_nhpppoe_t	= NULL;
	rt->rt_nhsession_t	= NULL;
	rt->rt_nharp_t		= NULL;
	if(netIfPtr->linkLayerType!=RTL8651_LL_PPPOE)
		rt->nextHopType = RTL8651_LL_VLAN;
	else
		rt->nextHopType = netIfPtr->linkLayerType;

	{	/* add Broadcast arp for this subnet */
		ether_addr_t mac = { {0xff, 0xff, 0xff, 0xff, 0xff, 0xff} };
	 	_rtl8651_addLocalArp(TRUE, ((ipAddr&ipMask)|~ipMask), &mac, netIfPtr, 0xffffffff);
	 }

__out:

	/* Update operation layer */
	if (freeIpCount == RTL8651_IPTABLE_SIZE) {
		if (extIf == TRUE) 
			rtl8651_setAsicOperationLayer(4);
		else {
			for(j=0, ipIntfCount=0,tmpNetPtr=&DrvTbl.netIface[0]; j<rtl8651_tblDrvPara.networkIntfTableSize; j++, tmpNetPtr++)
				if(tmpNetPtr->valid == 1) {
					rtl8651_tblDrv_ipIntfEntry_t * ipIntfPtr;
					SLIST_FOREACH(ipIntfPtr, &(tmpNetPtr->IpHead), nextIp)
						ipIntfCount++;
				}
			if(ipIntfCount == 1) //Add ip interface success and have one ip interface only means change from not IP interface to IP interface exist
				rtl8651_setAsicOperationLayer(3);
		}
	} 
	/* Update rouitng enable field in Vlan Table */
	vlanPtr = &DrvTbl.vlan[rtl8651_vlanTableIndex(vid)];
	if (vlanPtr->ipAttached == 0) {
		vlanPtr->ipAttached = 1;
		RTL8651_SETASICVLAN(vlanPtr);
	}			
	rtlglue_drvMutexUnlock();//Unlock resource
	return SUCCESS;
}



/*
@func int32 | rtl8651_delIpIntf | Remove an IP interface from the specified network interface.
@parm int8 * | ifName | Network interface name.
@parm ipaddr_t | ipAddr | Specify Gateway IP address.
@parm ipaddr_t | ipMask | Specify network mask.
@rvalue SUCCESS | The specified IP interface was successfully removed form the network interface.
@rvalue TBLDRV_EINVNETIFNAME | Invalid network interface name.
@rvalue TBLDRV_ECANNOTREMOVEIPUNEXTIPINTF | Global IP interface of IP Unnumbered cannot be removed. Use rtl8651_setLanSideExternalIpInterface() first.
@rvalue TBLDRV_EIPINTFISREFERENCEDBYARP | IP interface is referenced by ARP entry. It cannot be removed.
@rvalue TBLDRV_EIPINTFISREFERENCEDBYLS | IP interface is referenced by Local Server. It cannot be removed.
@rvalue TBLDRV_EIPINTFISREFERENCEDBYRT | IP interface is referenced by routing entry. It cannot be removed.
@rvalue TBLDRV_EIPINTFISREFERENCEDNYNATNAPT | IP interface is referenced by NAT/NAPT. It cannot be removed.
@rvalue TBLDRV_ENOIPINTFFOUND | The specified IP interface was not found.
@comm
The <p rtl8651_delIpIntf()> API removes an IP interface from the specified network interface. The <p ipAddr> specifies Gateway IP address.
*/
int32 rtl8651_delIpIntf(int8 * ifName, ipaddr_t ipAddr, ipaddr_t ipMask) {
	uint32 netIdx, j, k, ipIntfCount;//, freeIpCount;
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr, *tmpNetPtr;
	rtl8651_tblDrv_ipIntfEntry_t * ipIntfPtr;
	rtl8651_tblDrv_ipIntfIpAddrEntry_t * delIpPtr;
 rtl8651_tblDrv_arpEntry_t *arp_t;
	rtl8651_tblDrv_routeTable_t *rt;
	rtl8651_tblDrv_arpAsicArrangementEntry_t *asicArp;
	uint16 vid;

	rtlglue_drvMutexLock();//Lock resource
	netIfPtr = GET_NETWORK_INTERFACE(ifName, &netIdx);
	LOCK_ERROR_CHECK((netIfPtr==NULL), TBLDRV_EINVNETIFNAME);
	
	/* Find the specified IP interface */
	SLIST_FOREACH(ipIntfPtr, &(netIfPtr->IpHead), nextIp)
		if((ipIntfPtr->ipAddr.slh_first->ipAddr & ipIntfPtr->ipMask) == (ipAddr & ipMask) && ipIntfPtr->ipMask == ipMask)
			break;
	/* Specified network not exist */
	LOCK_ERROR_CHECK((ipIntfPtr==NULL), TBLDRV_ENOIPINTFFOUND);
	if (ipIntfPtr->lanSideExternal == 1) {
		rtlglue_drvMutexUnlock();
		return TBLDRV_ECANNOTREMOVEIPUNEXTIPINTF;
	}
	vid = netIfPtr->vid;
	//cfliu 2003.06.20: Confirmed with chhuang, if vlan is external, Ip interface is allowed to be added. However, 
	//the whole NAPT table would be flushed. 
	//We don't apply special check if vlan is internal.
	
        /* Otherwise, the removed IP interface is found */
	assert(SLIST_FIRST(&ipIntfPtr->ipAddr));
	/*
	 * (1)If deletion will cause ip interface to be deleted, make sure
	 *    no arp entries or localSrever entries attach to it.
	 * (2)If deletion will cause ip interface to be deleted and a route 
	 *    is referencing the IP interface, forbid the request!!
	 */
	if (ipIntfPtr->ipAddr.slh_first->nextIpAddr.sle_next == NULL) {
		if (SLIST_FIRST(&ipIntfPtr->localServer)) {
			rtlglue_drvMutexUnlock(); /* Case 1: */
			return(TBLDRV_EIPINTFISREFERENCEDBYLS);
		}
		for(k=0,rt=&DrvTbl.route[0]; k<rtl8651_tblDrvPara.routingTableSize; k++,rt++) {
			if (rt->valid == 0) continue;
			if (rt->nextHopType == RTL8651_LL_VLAN && (rt->nextHop&ipMask) == (ipAddr&ipMask)) { /* Case 2: */
					rtlglue_drvMutexUnlock();
				return(TBLDRV_EIPINTFISREFERENCEDBYRT);
			}
		}
	}
	
	/* Find the specified IP address */
	SLIST_FOREACH(delIpPtr, &(ipIntfPtr->ipAddr), nextIpAddr)
		if (delIpPtr->ipAddr == ipAddr)
			break;
	/* Specified IP address not found */
	LOCK_ERROR_CHECK((delIpPtr == NULL), TBLDRV_ENOIPINTFFOUND);
	/* Check whetehr the interface is external NAT/NAPT IP interface */
	if (DrvTbl.vlan[rtl8651_vlanTableIndex(vid)].internal == 0) {
		LOCK_ERROR_CHECK(
			( (_rtl8651_isNaptMapping(ipAddr)==TRUE) ||
			  (_rtl8651_isNatMapping(&ipAddr, NULL)==TRUE)
			), TBLDRV_EIPINTFISREFERENCEDNYNATNAPT
		);
			}
	/* If the network type is PPPoE, try to unbind IP and session ID */
	if (netIfPtr->linkLayerType != RTL8651_LL_VLAN)
		_rtl8651_unbindPppoeSession(delIpPtr, NULL);		
 /* If only one IP address, remove all arp entries */
 if (ipIntfPtr->ipAddr.slh_first->nextIpAddr.sle_next == NULL) {
  /* Remove all arps belong to the IP interface */
  while ((arp_t=SLIST_FIRST(&ipIntfPtr->nextArp))) {
   _rtl8651_delLocalArp(ARP_FORCE_DEL, arp_t->ipAddr, netIfPtr);
  }
 }
  
	/* Remove the specified IP address */
	SLIST_REMOVE(&(ipIntfPtr->ipAddr), delIpPtr, rtl8651_tblDrv_ipIntfIpAddrEntry_s, nextIpAddr);
	SLIST_INSERT_HEAD(&DrvTbl.freeList.ipAddr, delIpPtr, nextIpAddr);
		
	/* If already no IP address attached on this IP interface, remove the IP interface */
	if(SLIST_FIRST(&ipIntfPtr->ipAddr) == NULL) {
		assert(ipIntfPtr->nextArp.slh_first == NULL);
		/* Remove arp and route arrangement */
		if(ipIntfPtr->arpAllocated) {//Remove ARP allocation
			uint32 arpPos;
			for(arpPos=ipIntfPtr->arpStartPos,asicArp=&AsicTableUsage.arp[arpPos]; arpPos<=ipIntfPtr->arpEndPos; arpPos++,asicArp++) 
				asicArp->valid = 0;
		}
#if 0
		/* Remove the interface route */
		if (netIfPtr->linkLayerType != RTL8651_LL_PPPOE) {
			for(k=0, vid=0,rt=&DrvTbl.route[0]; k<rtl8651_tblDrvPara.routingTableSize-1; k++,rt++)
				if ((rt->ipAddr&rt->ipMask)==(ipAddr&ipMask) &&	ipMask == rt->ipMask) {
					rt->valid = 0;
					rtEntry--;
					break;
				}
			assert(k < rtl8651_tblDrvPara.routingTableSize-1);
			if(ipIntfPtr->routeAllocated) {
				AsicTableUsage.route[ipIntfPtr->routingTablePos].rt_entry = NULL;
				_rtl8651_refillAsicRoute(ipIntfPtr->routingTablePos);
			}						
			_rtl8651_updateDefaultRoute();
		}
#else
		/* Remove the interface route */
		if (netIfPtr->linkLayerType != RTL8651_LL_PPPOE) {
			int32 retval = _rtl8651_delInterfaceRoute(ipAddr, ipMask);
			assert(retval == SUCCESS);
		}
		

#endif
		
		//Remove ipIntfPtr from list
		SLIST_REMOVE(&(netIfPtr->IpHead), ipIntfPtr, rtl8651_tblDrv_ipIntfEntry_s, nextIp);
		SLIST_INSERT_HEAD(&DrvTbl.freeList.ipIface, ipIntfPtr, nextIp);			
		/* Update rouitng enable field in Vlan Table */		
		if (SLIST_FIRST(&netIfPtr->IpHead) == NULL) {
			rtl8651_tblDrv_vlanTable_t *vlanPtr;
			vlanPtr = &DrvTbl.vlan[rtl8651_vlanTableIndex(netIfPtr->vid)];
			vlanPtr->ipAttached = 0;
			RTL8651_SETASICVLAN(vlanPtr);
			
			/* If this is the last external IP interface, after remove it, try to clear
			 * Server Port Table.
			 * PS: Only one external network interface is supported.
			 */
			//_rtl8651_flushNaptServerPortTable(netIfPtr);			//tony:support for multiple session can't use this function
			rtl8651_flushNaptServerPortbyExtIp(ipAddr);		// only clear server port entries by this external ip

		}		
	}

	/* Update operation layer */
	for(j=0, ipIntfCount=0,tmpNetPtr=&DrvTbl.netIface[0]; j<rtl8651_tblDrvPara.networkIntfTableSize; j++, tmpNetPtr++)
		if(tmpNetPtr->valid == 1) {
			rtl8651_tblDrv_ipIntfEntry_t * ipIntfPtr;
			SLIST_FOREACH(ipIntfPtr, &(tmpNetPtr->IpHead), nextIp) 
				ipIntfCount++;
		}
	if(ipIntfCount == 0) //Delete ip interface success and have zero ip interface means change from last IP interface to no IP interface exist
		rtl8651_setAsicOperationLayer(2);
	rtlglue_drvMutexUnlock();
	return SUCCESS;
}

#endif

#undef IPUNNUMBER_DEBUG


/* bit 1: want to remove LS patch, bit 2 : want to add LS patch */
#define REMOVE_LS				0x01
#define ADD_LS					0x02
#define RESET_LS					0x03

#define REMOVE_PURPOSE			0x01
#define ADD_PURPOSE			0x02

static void _rtl8651_resetLanSideExternalIpInterface( rtl8651_tblDrv_networkIntfTable_t *netif_t, rtl8651_tblDrv_ipIntfEntry_t *ipintf_t, uint8 operation)
{
	rtl8651_tblDrv_ipUnnumbered_t *ipUnnumber_t = ipintf_t->ipUnnumber;
	#ifdef IPUNNUMBER_DEBUG
	int32 retval;
	#endif

	assert(ipintf_t->lanSideExternal==1 && ipintf_t->ipUnnumber);

	#ifdef IPUNNUMBER_DEBUG
	printk("%s(%d):\n\t Reset network interface : %s , operation : 0x%x\n", __FUNCTION__, __LINE__, netif_t->ifName, operation);
	#endif

	/*
	  *	Check for Local Public entry remove first.
	  *
	  */
	if ( operation & REMOVE_PURPOSE )
	{
		int32 ipIdx;
		rtl8651_tblDrv_ipEntry_t *ipEntry_t;

		#ifdef IPUNNUMBER_DEBUG
		printk("%s(%d):\n\t Remove Local Public entry, current count [%d]\n", __FUNCTION__, __LINE__, ipUnnumber_t->ip_as_LS);
		#endif

		if (ipUnnumber_t->ip_as_LS == 0)
			goto remove_done;

		/*
			We trace whole IP table to find the Local public entry and remove the corresponding entry
			for this unnumber interface.
																						*/
		ipEntry_t = DrvTbl.iptbl;
		for(ipIdx=0 ; ipIdx<RTL8651_IPTABLE_SIZE ; ipIdx++, ipEntry_t++)
		{
			if (ipEntry_t->type != IP_ATTR_LS)
				continue;
			if ( (ipEntry_t->extip&ipUnnumber_t->ipMask) ==
			      (ipUnnumber_t->ipAddr&ipUnnumber_t->ipMask))
			{
				#ifdef IPUNNUMBER_DEBUG
				printk("%s(%d):\n\t Remove Local Public entry [%u.%u.%u.%u]\n", __FUNCTION__, __LINE__, NIPQUAD(ipEntry_t->extip));
				#endif

 				/*
					remove IP entry
				*/
				if ((
					#ifdef IPUNNUMBER_DEBUG			
					retval =
					#endif
					_rtl8651_delIpEntry(IP_ATTR_LS, ipEntry_t->extip, 0)) == SUCCESS)
				{
					/* update counter */
					rtl8651_ipUnnumberGlobalCtrlBlock.ip_unnumber_LS_budget ++;
					ipUnnumber_t->ip_as_LS --;
				}
				#ifdef IPUNNUMBER_DEBUG
				else
				{
					printk("%s(%d):\n\t\t ===> Remove not SUCCESS [%d]\n", __FUNCTION__, __LINE__, retval);
				}
				#endif
			}
		}

		assert(ipUnnumber_t->ip_as_LS == 0);
	}
remove_done:

	/*
	  *	Check for Local Public entry add after processing removing.
	  *
	  */
	if ( operation & ADD_PURPOSE )
	{
		assert(ipUnnumber_t->ip_as_LS == 0);

		#ifdef IPUNNUMBER_DEBUG
		printk("%s(%d):\t Add Local Public entry, current budget [%d]\n", __FUNCTION__, __LINE__, rtl8651_ipUnnumberGlobalCtrlBlock.ip_unnumber_LS_budget);
		#endif

		/*
			We have no any Local Public entry budget to patch, exist.
		*/
		if (rtl8651_ipUnnumberGlobalCtrlBlock.ip_unnumber_LS_budget == 0)
		{
			goto add_done;
		}

		{
			uint32 ipToBeAdded;
			uint32 i, addedCount=0, subnetCount=0xffffffff & (uint32)(~((uint32)ipUnnumber_t->ipMask));

			for ( i=1;
				 i<subnetCount && addedCount<rtl8651_ipUnnumberGlobalCtrlBlock.ip_unnumber_LS_budget;
				 i++ )
			{
				ipToBeAdded = (uint32)(ipUnnumber_t->ipAddr & ipUnnumber_t->ipMask) + i;

				if (ipToBeAdded == ipUnnumber_t->ipAddr)
					continue;

				#ifdef IPUNNUMBER_DEBUG
				printk("%s(%d):\t Add Local Public entry: %u.%u.%u.%u\n", __FUNCTION__, __LINE__, NIPQUAD(ipToBeAdded));
				#endif

				/* add to IP Table */
				if (_rtl8651_addIpEntry((ipaddr_t)ipToBeAdded, (ipaddr_t)ipToBeAdded, IP_ATTR_LS, NULL) == NULL)
				{
					#ifdef IPUNNUMBER_DEBUG
					printk("%s(%d):\t Add Local Public FAIL!! Skip Local Public entry add\n", __FUNCTION__, __LINE__);
					#endif

					break;
				}
				addedCount++;
			}
			/* update counter */
			ipUnnumber_t->ip_as_LS += addedCount;
			rtl8651_ipUnnumberGlobalCtrlBlock.ip_unnumber_LS_budget -= addedCount;
		}
	}
add_done:
	_rtl8651_arrangeAllACL();

}

/*
	Re-arrange LAN side External Ip Interface's patch: including Local Public entry patch and
	ACL patch/Routing Entry patch
*/
void _rtl8651_reArrangeLanSideExternalIpInterfacePatch(void)
{
	rtl8651_tblDrv_ipIntfEntry_t *ipintf_t;

	if (!lanSideIpUnnumberedLan)
	{
		return;	/* don't reset it */
	}

	SLIST_FOREACH(ipintf_t, &lanSideIpUnnumberedLan->IpHead, nextIp)
	{
		if (ipintf_t->lanSideExternal)
		{
			_rtl8651_resetLanSideExternalIpInterface(lanSideIpUnnumberedLan, ipintf_t, RESET_LS);
		}
	}
}


/*
@func int32 | rtl8651_setLanSideExternalIpInterface | Set IP Unnumebr IP interface to global IP interface.
@parm int8 * | ifName | Netwrok interface name. It should be internal network interface in IP Unnumbered network.
@parm ipaddr_t | ipAddr | Specify Gateway IP address.
@parm ipaddr_t | ipMask | Network mask.
@parm int8 | isExternal | TRUE to set IP interface to be external interface. FALSE to set IP interface to be internal interface.
@rvalue SUCCESS | The specified IP interface was set to global IP interface successfully.
@rvalue TBLDRV_EINVNETIFNAME | Invalid network interface name.
@rvalue TBLDRV_ENOTIPUNNUMBEREDNETIF | The specified network interface is not Lan Side Ip Unnumbered Network interface.
@rvalue TBLDRV_EIPUNINTIPINTFNEED | The specified network interface should be IP Unnumbered internal network interface.
@rvalue TBLDRV_ENOIPINTFFOUND | The specified IP interface was not found.
@rvalue TBLDRV_EIPUNISREFERENCEDBYPPPOE | The IP Unnumbered IP interface is referenced by PPPoE Session.
@comm
The <p rtl8651_setLanSideExternalIpInterface()> sets an IP interface to be a global IP interface. The <p ifName> paramater should be an
IP Unnumbered Internal Netwrok interface.
*/
int32 rtl8651_setLanSideExternalIpInterface(int8 * ifName, ipaddr_t ipAddr, ipaddr_t ipMask, int8 isExternal) {
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	int32 retval = FAILED;

	netif_t = GET_NETWORK_INTERFACE(ifName, NULL);
	UNLOCK_ERROR_CHECK((netif_t==NULL), TBLDRV_EINVNETIFNAME);
	rtlglue_drvMutexLock();
	retval = _rtl8651_setLanSideExternalIpInterface(netif_t, ipAddr, ipMask, isExternal);
	rtlglue_drvMutexUnlock();
	return retval;
}

/*
	Internal Function to set Lan Side External IP interface
*/
int32 _rtl8651_setLanSideExternalIpInterface(rtl8651_tblDrv_networkIntfTable_t *netif_t, ipaddr_t ipAddr, ipaddr_t ipMask, int8 isExternal)
{
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	rtl8651_tblDrv_ipIntfEntry_t *ipintf_t;
	rtl8651_tblDrv_ipIntfIpAddrEntry_t *ip_t;
	rtl8651_tblDrvSession_t *psession_t;
	rtl8651_tblDrv_arpEntry_t *arp_t;
	rtl8651_tblDrv_ipUnnumbered_t *ipUnnumber_t;
	uint32 netIdx;

	UNLOCK_ERROR_CHECK(
		(netif_t->linkLayerType != RTL8651_LL_LIPUNNUMBERED), 
		TBLDRV_ENOTIPUNNUMBEREDNETIF
	)
	UNLOCK_ERROR_CHECK(
		(DrvTbl.vlan[rtl8651_vlanTableIndex(netif_t->vid)].internal == 0),
		TBLDRV_EIPUNINTIPINTFNEED
	);
	SLIST_FOREACH(ipintf_t, &netif_t->IpHead, nextIp) 
		if (((ipintf_t->ipMask&ipintf_t->ipAddr.slh_first->ipAddr)==(ipAddr&ipMask)) && 
			(ipintf_t->ipMask == ipMask)) 
			break;
	UNLOCK_ERROR_CHECK((ipintf_t==NULL), TBLDRV_ENOIPINTFFOUND);

	if (isExternal == TRUE)
	{
		if ( ipintf_t->ipUnnumber )
		{
			/*
				IP unnumber data structure exists, it means that this ip interface has been assigned to
				LanSideExternal before.
				If nothing is changed, we seems this as an duplicate setting; if something are changed,
				we would note user to remove the previous setting first.
			*/
			assert (ipintf_t->lanSideExternal == 1);

			if (ipintf_t->ipUnnumber->ipAddr != ipAddr)
			{
				rtlglue_printf("%s(%d) Fail: ipAddr(org:%u.%u.%u.%u new:%u.%u.%u.%u) mismatch.\n", __FUNCTION__, __LINE__, NIPQUAD(ipintf_t->ipUnnumber->ipAddr), NIPQUAD(ipAddr));
				return TBLDRV_EINVALIDINPUT;
			}
			if (ipintf_t->ipUnnumber->ipMask != ipMask)
			{
				rtlglue_printf("%s(%d) Fail: ipMask(org:%u.%u.%u.%u new:%u.%u.%u.%u) mismatch.\n", __FUNCTION__, __LINE__, NIPQUAD(ipintf_t->ipUnnumber->ipMask), NIPQUAD(ipMask));
				return TBLDRV_EINVALIDINPUT;
			}
			return SUCCESS;	/* duplicate setting, we don't do any thing */

		}else
		{
			/*
				We need to allocate the new IP unnumber structure for this lanSideExternal interface.
			*/
			/* Allocate Entry*/
			if (SLIST_FIRST(&DrvTbl.freeList.freeIpUnnumber) == NULL)
				return TBLDRV_ENOFREEBUFFER;
			ipUnnumber_t = ipintf_t->ipUnnumber = SLIST_FIRST(&DrvTbl.freeList.freeIpUnnumber);
			SLIST_REMOVE_HEAD(&DrvTbl.freeList.freeIpUnnumber, nextIpUnnumberEntry);
			SLIST_INSERT_HEAD(&DrvTbl.inuseList.ipUnnumber, ipintf_t->ipUnnumber, nextIpUnnumberEntry);
			assert(ipUnnumber_t);

			/* Init Entry */
			ipUnnumber_t->ipIntf_t = ipintf_t;
			ipUnnumber_t->ipAddr = ipAddr;
			ipUnnumber_t->ipMask = ipMask;
			ipUnnumber_t->ip_as_LS = 0;
			ipUnnumber_t->ip_as_ACL_budget = 0;
		}

		ipintf_t->lanSideExternal = 1;

		/* add Local Public patch */
		_rtl8651_resetLanSideExternalIpInterface(netif_t, ipintf_t, ADD_LS);

	}else
	{
		/* If PPPoE session is binding to IP interface, forbid the request */
		netIfPtr = &DrvTbl.netIface[0];
		for(netIdx=0; netIdx<rtl8651_tblDrvPara.networkIntfTableSize; netIdx++, netIfPtr++)
			if (DrvTbl.vlan[rtl8651_vlanTableIndex(netIfPtr->vid)].internal == 0 &&
				netIfPtr->linkLayerType == RTL8651_LL_LIPUNNUMBERED)
				break;

		UNLOCK_ERROR_CHECK((netIdx == rtl8651_tblDrvPara.networkIntfTableSize), SUCCESS);
		SLIST_FOREACH(ip_t, &ipintf_t->ipAddr, nextIpAddr)
		{
			CTAILQ_FOREACH(psession_t, &netIfPtr->sessionHead, nextSession)
				UNLOCK_ERROR_CHECK((ip_t == psession_t->ip_t), TBLDRV_EIPUNISREFERENCEDBYPPPOE);
		}

		/* Remove ARP and NAT mapping */
		while((arp_t=SLIST_FIRST(&ipintf_t->nextArp))) 
			_rtl8651_delLocalArp(ARP_FORCE_DEL, arp_t->ipAddr, netif_t);

		/* remove Local Public patch */
		_rtl8651_resetLanSideExternalIpInterface(netif_t, ipintf_t, REMOVE_LS);

		ipintf_t->lanSideExternal = 0;

		/* Remove Ip Unnumber entry */
		if(ipintf_t->ipUnnumber)
		{
			SLIST_REMOVE(&DrvTbl.inuseList.ipUnnumber, ipintf_t->ipUnnumber, rtl8651_tblDrv_ipUnnumbered_s, nextIpUnnumberEntry);
			SLIST_INSERT_HEAD(&DrvTbl.freeList.freeIpUnnumber, ipintf_t->ipUnnumber, nextIpUnnumberEntry);
			ipintf_t->ipUnnumber = NULL;
		}

	}
	_rtl8651_arrangeAllACL();
	return SUCCESS;
}

/*==============================================
  * GROUP: ARP APIs
  *==============================================*/
#define RTL8651_ARP_APIs

static void _rtl8651_arrangeAsicArpEntry(rtl8651_tblDrv_ipIntfEntry_t *ipIntf, rtl8651_tblDrv_arpEntry_t *arp_t) 
{
	rtl8651_tblDrv_routeTable_t *rt_t = DrvTbl.route;
	rtl8651_tblDrv_filterDbTableEntry_t *macInfo;
	rtl8651_tblDrv_networkIntfTable_t *netIntf;
	rtl8651_tblDrv_arpEntry_t *arp;
	uint32  arpPos, i;
	int8 isDelete;

       /* 
	 * The _rtl8651_arrangAsicArpEntry() re-arranges specified ARP entry according
	 * to the following rules:
	 *		1. If no specified ARP entry exists, remove ASIC ARP entry.
	 *		2. If specified ARP lacks its corresponding L2 entry in the ASIC,
	 *		   pending the ARP (Do not write the ARP into ASIC ARP table)
	 *		3. If the specified ARP entry has corresponding L2 entry in the ASIC 
	 *		   and the entry is not DMZ pending, write it into ASIC ARP table; 
	 *		   otherwise, pending the ARP entry.
	 *		4. If the IP interface has no arp space allocated, do nothing!
        */

	if (ipIntf == (rtl8651_tblDrv_ipIntfEntry_t *)NULL) {
		netIntf = &DrvTbl.netIface[0];
		for(i=0; i<rtl8651_tblDrvPara.networkIntfTableSize; i++, netIntf++)
			if (netIntf->valid == 1) 
				SLIST_FOREACH(ipIntf, &(netIntf->IpHead), nextIp)
					SLIST_FOREACH(arp, &(ipIntf->nextArp), next) {
						if (arp->ipAddr == arp_t->ipAddr)
							goto __ipIntf_found;
		}
	}

__ipIntf_found:

	if (!arp_t  || !ipIntf)
		return;

	if (ipIntf->arpAllocated == 1) {		
		arpPos = (ipIntf->arpStartPos<<3)+(arp_t->ipAddr & ~ipIntf->ipMask);	
		if (arp_t->fromApp == 0 && arp_t->fromDrv == 0 && arp_t->age == 0) {
			isDelete = TRUE;
			rtl8651_delAsicArp(arpPos);	
		}
		else {
			isDelete = FALSE;
			macInfo = arp_t->macInfo;
			assert(macInfo);
			if (macInfo->configToAsic == 1 && arp_t->dmzIpPending == 0) {
				rtl865x_tblAsicDrv_arpParam_t asic_arp;
				asic_arp.nextHopColumn = macInfo->asicPos;
				asic_arp.nextHopRow =  rtl8651_filterDbIndex(&macInfo->macAddr);
				rtl8651_setAsicArp(arpPos, &asic_arp);
			} else rtl8651_delAsicArp(arpPos);
		}
		_rtl8651_arrangeNATbyMacAccessControl();
		_rtl8651_arrangeSPbyMacAccessControl();
	}
	else {
		if (arp_t->fromApp == 0 && arp_t->fromDrv == 0 && arp_t->age == 0) 
			isDelete = TRUE;
		else isDelete = FALSE;
	}

	/* if routing entry references this entry, update it!! */
	for(i=0; i<rtl8651_tblDrvPara.routingTableSize; i++, rt_t++) {
		if (rt_t->valid == 0 || (rt_t->process!=RT_L2&&rt_t->process!=RT_NEXTHOP))
			continue;			
		if (arp_t->ipAddr == rt_t->nextHop) {
			if (isDelete == FALSE)
				rt_t->rt_nharp_t = arp_t;
			else rt_t->rt_nharp_t = NULL;
			_rtl8651_updateL3Entry(rt_t); 
		}
	}
	_rtl8651_updateNextHop_NHIDX(arp_t, NULL);

}
	



#if 0
/*
 *  Louis note: In current version, ONLY ONE nexthop is needed.
 *              Thus, we don't need complex algorithm.
 *              Finally, holly Chhuang will re-write this code.
 *
 *  Called by rtl8651_addNaptMapping(), _rtl8651_addArp() and _rtl8651_addRoute().
 *
 *  rtl8651_addNaptMapping(): extIpAddr and ipEntry_t MUST be given.
 *  _rtl8651_addRoute(): extIpAddr and ipEntry_t are assigned to NULL (update default route only).
 *  _rtl8651_addArp(): extIpAddr and ipEntry_t are assigned to NULL (update default route only).
 */
int32 _try_to_setAsicNexthopTable( ipaddr_t extIpAddr, rtl8651_tblDrv_ipEntry_t *ipEntry_t )
{
	rtl865x_tblAsicDrv_extIntIpParam_t asic_ip;
	rtl865x_tblAsicDrv_nextHopParam_t asic_nexthop;
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	void* nexthop_t = NULL;
	int retval;
	uint32 nexthopIndex = RTL8651_NEXTHOPTBL_SIZE-1;

	if ( extIpAddr == 0 && ipEntry_t == NULL )
	{
		/* for _rtl8651_addRoute() and _rtl8651_addLocalArp() */
		rtl8651_tblDrv_routeTable_t* defRoute;
		
		retval = rtl8651_getAsicNextHopTable( nexthopIndex, &asic_nexthop );
		assert(retval == SUCCESS);

		defRoute = &DrvTbl.route[rtl8651_tblDrvPara.routingTableSize-1];

		if ( defRoute->valid )
		{
			if ( defRoute->nextHopType == RTL8651_LL_VLAN )
			{
				/* update nexthop field, if posible */
				nexthop_t = defRoute->rt_nharp_t;

				if ( nexthop_t )
				{
					asic_nexthop.nextHopColumn = ((rtl8651_tblDrv_arpEntry_t*)nexthop_t)->macInfo->asicPos;
					asic_nexthop.nextHopRow = rtl8651_filterDbIndex(&(((rtl8651_tblDrv_arpEntry_t*)nexthop_t)->macInfo->macAddr));

					retval = rtl8651_setAsicNextHopTable( nexthopIndex, &asic_nexthop );
					assert(retval == SUCCESS);
				}
			}
			else
			{
				/* PPPoE/L2TP/PPTP ... */
				rtl8651_tblDrv_pppTable_t *pppSession;
				pppSession = defRoute->rt_nhSession_t;
				assert( pppSession );
				if ( pppSession->macInfo )
				{
					asic_nexthop.nextHopColumn = pppSession->macInfo->asicPos;
					asic_nexthop.nextHopRow = rtl8651_filterDbIndex(&(pppSession->macInfo->macAddr));

					retval = rtl8651_setAsicNextHopTable( nexthopIndex, &asic_nexthop );
					assert(retval == SUCCESS);
				}
				
			}
		}
	}
	else
	{
		/* for rtl8651_addNaptMapping() */
		
		/* first, fill software entry */
		netif_t = _rtl8651_getIpInfo( extIpAddr, NULL, NULL, NULL );
		if ( netif_t == NULL )
		{
			rtlglue_drvMutexUnlock();
			return FAILED;
		}

		/* second, write to ASIC (Nexthop table)
		 *
		 * We note that:
		 *   For isPppoe bit, only PPPoE and IP-unnumbered are set TRUE.
		 *   For dest VLAN index, PPTP/L2TP/IP-unnumbered are set to tunnel.loopBackVid.
		 *   For nexthop ARP, PPPoE/IP-unnumbered/PPTP/L2TP are sharing pppTable's MacInfo.
		 *
		 */	
		bzero( &asic_nexthop, sizeof(asic_nexthop) );

		switch( netif_t->linkLayerType )
		{
			case RTL8651_LL_PPTP:          /* PPTP */
			case RTL8651_LL_L2TP:          /* L2TP */
				asic_nexthop.dvid = tunnel.loopBackVid;
				break;
			default:
				asic_nexthop.dvid = netif_t->vid;
				break;
		}
		asic_nexthop.extIntIpIdx = IP_TABLE_INDEX(ipEntry_t);
		asic_nexthop.isPppoe = ( netif_t->linkLayerType==RTL8651_LL_PPPOE ||
		                         netif_t->linkLayerType==RTL8651_LL_LIPUNNUMBERED );
		switch( netif_t->linkLayerType )
		{
			case RTL8651_LL_PPPOE:         /* PPPOE */
			case RTL8651_LL_LIPUNNUMBERED: /* PPPOE IP un-numbered */
			case RTL8651_LL_WIPUNNUMBERED: /* PPPOE IP un-numbered */
			case RTL8651_LL_PPTP:          /* PPTP */
			case RTL8651_LL_L2TP:          /* L2TP */
				/* point to PPP table */
				nexthop_t = _rtl8651_getPppoeSessionByIpAddr( extIpAddr );
				if ( nexthop_t )
				{
					asic_nexthop.nextHopColumn = ((rtl8651_tblDrv_pppTable_t*)nexthop_t)->macInfo->asicPos;
					asic_nexthop.nextHopRow = rtl8651_filterDbIndex(&(((rtl8651_tblDrv_pppTable_t*)nexthop_t)->macInfo->macAddr));
					asic_nexthop.pppoeIdx = PPPOE_TABLE_INDEX((rtl8651_tblDrv_pppTable_t *)nexthop_t);
				}
				break;
			default:
				/* VLAN, point to ARP table */
				nexthop_t = DrvTbl.route[rtl8651_tblDrvPara.routingTableSize-1].rt_nharp_t;
				if ( nexthop_t )
				{
					asic_nexthop.nextHopColumn = ((rtl8651_tblDrv_arpEntry_t*)nexthop_t)->macInfo->asicPos;
					asic_nexthop.nextHopRow = rtl8651_filterDbIndex(&(((rtl8651_tblDrv_arpEntry_t*)nexthop_t)->macInfo->macAddr));
				}
				break;
		}

		retval = rtl8651_setAsicNextHopTable( nexthopIndex, &asic_nexthop );
		assert(retval == SUCCESS);
	
	}
	
	/* third, update nhIndex field of IP table */
	if ( ipEntry_t )
	{
		rtl8651_getAsicExtIntIpTable( IP_TABLE_INDEX(ipEntry_t), &asic_ip );
		asic_ip.nhIndex	= nexthopIndex;
		retval = rtl8651_setAsicExtIntIpTable( IP_TABLE_INDEX(ipEntry_t), &asic_ip );
		assert(retval == SUCCESS);
	}

	return SUCCESS;
}
#endif



void _rtl8651_removeArpByL2Entry(rtl8651_tblDrv_filterDbTableEntry_t *l2entry_t)
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_ipIntfEntry_t *ipEntry_t;
	rtl8651_tblDrv_arpEntry_t *arp_t;
	int32 i;
	
	assert(l2entry_t);
	netif_t = &DrvTbl.netIface[0];
	for(i=0; i<rtl8651_tblDrvPara.networkIntfTableSize; i++, netif_t++) {
		SLIST_FOREACH(ipEntry_t, &netif_t->IpHead, nextIp) {
			SLIST_FOREACH(arp_t, &ipEntry_t->nextArp, next) {
				if (arp_t->macInfo == l2entry_t){
					/* arp found, remove it */
					arp_t->fromApp 	= 0;
					arp_t->fromDrv	= 0;
					arp_t->age		= 0;
					_rtl8651_arrangeAsicArpEntry(ipEntry_t, arp_t);
					SLIST_REMOVE(&ipEntry_t->nextArp, arp_t, rtl8651_tblDrv_arpEntry_s, next);
					CSLIST_INSERT_HEAD(&DrvTbl.freeList.arp, arp_t, next);
					break;
				}
			}
		}
	}
}



/* 
if port is configured to be 0xffffffff, this means local broadcast or port unknown.
*/
int32 _rtl8651_addLocalArp(int8 fromDrv, ipaddr_t ipAddr, ether_addr_t * macAddr, rtl8651_tblDrv_networkIntfTable_t *netif_t, uint32 port) 
{
	rtl8651_tblDrv_arpEntry_t * allocArpEntry, *trackArpEntry;
	rtl8651_tblDrv_ipIntfEntry_t *ipintf_t;
	rtl8651_tblDrv_ipIntfIpAddrEntry_t *ip_t;
	uint32 i;

	assert(macAddr && netif_t);
	assert(netif_t->vid > 1 || netif_t->vid<4095);
	

	/* Check if the arp IP address is the gateway IP address */
	SLIST_FOREACH(ipintf_t, &netif_t->IpHead, nextIp) {
		SLIST_FOREACH(ip_t, &ipintf_t->ipAddr, nextIpAddr)
			if (ip_t->ipAddr == ipAddr)
				return TBLDRV_EINVALIDINPUT;
	}
	
	if (CSLIST_TOTAL(&DrvTbl.freeList.arp) == 0)
		return TBLDRV_ENOFREEBUFFER;//No free ARP entry
	if(netif_t->linkLayerType == RTL8651_LL_PPPOE)
		return TBLDRV_ENONBROADCASTNET;//Forbid non-broadcast network to add ARP entry
	
	
	/* Find the IP network interface to which the added arp belongs */
	SLIST_FOREACH(ipintf_t, &(netif_t->IpHead), nextIp){
		if((ipAddr & ipintf_t->ipMask) == (ipintf_t->ipAddr.slh_first->ipAddr & ipintf_t->ipMask))
			break;
	}
	if(ipintf_t == NULL)
		return TBLDRV_ENOIPINTFFOUND;//Specified netPtr does not belong to specified interface

	/*==========================================================
	  * 						Patch for PPTP/L2TP
	  * If the specified IP addess is PPTP/L2TP server's IP address, its member port 
	  * should be modified to MII port.
	  *==========================================================*/
#if defined(CONFIG_RTL865X_PPTPL2TP)||defined(CONFIG_RTL865XB_PPTPL2TP)
	{
		rtl8651_tblDrv_pppTable_t *pppEntry_t = &DrvTbl.pppoe[RTL8651_PPPOE_NUMBER];
		for(i=0; i<rtl8651_tblDrvPara.pptpSize+rtl8651_tblDrvPara.l2tpSize; i++, pppEntry_t++) {
			if (pppEntry_t->valid && pppEntry_t->initiated == 0)
				continue;
			if ((pppEntry_t->type == RTL8651_PPTP && (pppEntry_t->pptpServerIp_ == ipAddr  || pppEntry_t->pptpDefaultGwIp_ == ipAddr)) ||
			    (pppEntry_t->type == RTL8651_L2TP && (pppEntry_t->l2tpServerIp_ == ipAddr || pppEntry_t->l2tpDefaultGwIp_ == ipAddr)) ) {
				tunnel.wanPort = port;
				port = tunnel.loopbackPort;
				break;
			}
		}
	}
#endif /* CONFIG_RTL865X(B)_PPTPL2TP */


	SLIST_FOREACH(trackArpEntry, &(ipintf_t->nextArp), next) {
		if (trackArpEntry->ipAddr != ipAddr)
			continue;
		if(fromDrv == FALSE) {
			/* The request is from protocol stack */
			if (trackArpEntry->fromApp == 0) {
				trackArpEntry->fromApp = 1;
				return SUCCESS;
			} else return TBLDRV_EARPALREADYEXIST;
		}
		else {
			/* The request is from forwarding engine */
			if (trackArpEntry->fromDrv == 0) {
				trackArpEntry->fromDrv = 1;
				return SUCCESS;
			} 
			else {
					/* 
					 * Duplicate entry caused by forwarding engine.
					 * Remove old and add new one.
					 */
					i = rtl8651_vlanTableIndex(trackArpEntry->vid);
					if(_rtl8651_delVlanRefFilterDatabaseEntry(RTL8651_L2_TYPEI, DrvTbl.vlan[i].fid, &trackArpEntry->macInfo->macAddr)!=SUCCESS)
						return TBLDRV_EARPCANNOTDELL2ENTRY;		
					if(_rtl8651_addVlanRefFilterDatabaseEntry(RTL8651_L2_TYPEI, DrvTbl.vlan[i].fid, trackArpEntry->vid, macAddr, RTL8651_FORWARD_MAC, port==0xffffffff? port: 1<<port) != SUCCESS) 
						return TBLDRV_EARPCANNOTADDL2ENTRY;
					trackArpEntry->macInfo  = _rtl8651_getVlanFilterDatabaseEntry(netif_t->vid, macAddr);
					trackArpEntry->age = arpAgingTime;
					assert(trackArpEntry->macInfo != NULL);
					_rtl8651_arrangeAsicArpEntry(ipintf_t, trackArpEntry);
					_rtl8651_arrangeAllACL();	// Arp addition will affect "L2 srcMac & L4 dstPort" ACL rule.
					return SUCCESS;
			}
		}
	}

	//Start to allocate ARP entry
	i = rtl8651_vlanTableIndex(netif_t->vid);
	if(_rtl8651_addVlanRefFilterDatabaseEntry(RTL8651_L2_TYPEI, DrvTbl.vlan[i].fid, netif_t->vid, macAddr, RTL8651_FORWARD_MAC, port==0xffffffff? port: 1<<port) != SUCCESS) 
		return TBLDRV_EARPCANNOTADDL2ENTRY;//Unable to allocate layer2 entry

	assert(trackArpEntry == NULL);
	allocArpEntry = CSLIST_FIRST(&DrvTbl.freeList.arp);
	CSLIST_REMOVE_HEAD(&DrvTbl.freeList.arp, next);
	allocArpEntry->ipAddr = ipAddr;
	allocArpEntry->vid 			= netif_t->vid;
	allocArpEntry->fromDrv = (fromDrv == TRUE)? 1: 0;
	allocArpEntry->fromApp = (fromDrv == TRUE)? 0: 1;
	allocArpEntry->dmzIpPending = 0;
	allocArpEntry->age = arpAgingTime;
	allocArpEntry->macInfo  		= _rtl8651_getVlanFilterDatabaseEntry(netif_t->vid, macAddr);
	assert(allocArpEntry->macInfo);
	
	if (ipAddr == ((ipintf_t->ipMask&ipAddr)|~ipintf_t->ipMask))
		allocArpEntry->isBcArp = 1;
	else allocArpEntry->isBcArp = 0;
	
	//Hook allocated arp to network interface
	SLIST_INSERT_HEAD(&(ipintf_t->nextArp), allocArpEntry, next);		
	_rtl8651_arrangeAsicArpEntry(ipintf_t, allocArpEntry);

	_rtl8651_arrangeAllACL();	// Arp addition will affect "L2 srcMac & L4 dstPort" ACL rule.
	return SUCCESS;
}

	
int32 _rtl8651_delLocalArp(int8 fromDrv, ipaddr_t ipAddr, rtl8651_tblDrv_networkIntfTable_t *netif_t) 
{
	rtl8651_tblDrv_ipIntfEntry_t * ipintf_t;
	rtl8651_tblDrv_arpEntry_t * delArpEntry, * trackArpEntry;
//	rtl8651_tblDrv_routeTable_t *rt;
	uint32 i;

	assert(netif_t);
	assert(netif_t->vid > 1 || netif_t->vid<4095);

	
	if(netif_t->linkLayerType == RTL8651_LL_PPPOE)
		return TBLDRV_ENONBROADCASTNET;//Forbid non-broadcast network to del ARP entry

	//From the first IP interface of this network interface, find the subnet which 'ipAddr' belongs to.
	SLIST_FOREACH(ipintf_t, &(netif_t->IpHead), nextIp) 
		if((ipAddr & ipintf_t->ipMask) == (ipintf_t->ipAddr.slh_first->ipAddr & ipintf_t->ipMask))
			break;
	if(ipintf_t == NULL)
		return TBLDRV_EENTRYNOTFOUND;//Specified netPtr does not belong to specified interface

	//IP interface found. locate the ARP entry in ARP linked list
	SLIST_FOREACH(trackArpEntry, &(ipintf_t->nextArp), next) {
		if(trackArpEntry->ipAddr == ipAddr) {//Entry match
			if (fromDrv == ARP_FORCE_DEL)
				break;
			if(fromDrv == TRUE && trackArpEntry->fromDrv == 1)
				trackArpEntry->fromDrv = 0;
			else if(fromDrv == FALSE && trackArpEntry->fromApp == 1) {
				trackArpEntry->fromApp = 0;
				break;
				//trackArpEntry->age = arpAgingTime;
				//return SUCCESS;
				//cfliu: In current implementation, arp entries deletion from application  is 
				//deferred until aged out. Shall we remove the entry immediately the entry that application 
				//wants to delete is indeed unused, rather than defer ALL such operations?
			}
			if (trackArpEntry->fromDrv == 1 || trackArpEntry->fromApp == 1)
				return SUCCESS;
			break;//Entry found
		}
	}

	if(trackArpEntry == NULL)
		return TBLDRV_EENTRYNOTFOUND;//Entry not exist
	if (fromDrv == ARP_FORCE_DEL) {
		trackArpEntry->fromApp = 0;
		trackArpEntry->fromDrv = 0;
		trackArpEntry->age = 0;
	} 
	assert(trackArpEntry->fromDrv == 0 && trackArpEntry->fromApp == 0);
	delArpEntry = trackArpEntry;
	assert(delArpEntry);//delArpEntry should store the arp entry to be removed

	SLIST_REMOVE(&(ipintf_t->nextArp), delArpEntry, rtl8651_tblDrv_arpEntry_s, next);
	CSLIST_INSERT_HEAD(&DrvTbl.freeList.arp, delArpEntry, next);
	i = rtl8651_vlanTableIndex(delArpEntry->vid);
	_rtl8651_delVlanRefFilterDatabaseEntry(RTL8651_L2_TYPEI, DrvTbl.vlan[i].fid, &delArpEntry->macInfo->macAddr);
	delArpEntry->age = 0;
	_rtl8651_arrangeAsicArpEntry(ipintf_t, delArpEntry);
	_rtl8651_arrangeAllACL();	// Arp addition will affect "L2 srcMac & L4 dstPort" ACL rule.
	return SUCCESS;	
}


//Given an IP address, check if it is any of the local subnet's broadcast address.
int32 _rtl8651_isLocalBcastAddr(uint32 vid, ipaddr_t ipAddr) {
	rtl8651_tblDrv_ipIntfEntry_t * ipIntfPtr;
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	rtl8651_tblDrv_vlanTable_t *thisVlan;

	thisVlan=&DrvTbl.vlan[ rtl8651_vlanTableIndex(vid)];
	assert(thisVlan->valid);
	netIfPtr=thisVlan->netif_t;
	if(netIfPtr&&netIfPtr->valid == 1) {//Interface valid, network interface is meaningful
		SLIST_FOREACH(ipIntfPtr, &(netIfPtr->IpHead), nextIp) {
			assert(SLIST_FIRST(&ipIntfPtr->ipAddr));//Network interface must have at least one address
			if(ipAddr== ((ipIntfPtr->ipAddr.slh_first->ipAddr & ipIntfPtr->ipMask)|(~ipIntfPtr->ipMask))){
				if (ipIntfPtr->ipMask == 0xffffffff){// mask = 32bits will never "broadcast"
					return FALSE;
				}else{
					return TRUE;
				}
		}
	}
	}
	return FALSE;
}

rtl8651_tblDrv_arpEntry_t * _rtl8651_getArpEntryByMac(rtl8651_tblDrv_networkIntfTable_t *netif_t, ether_addr_t *macAddr, int8 isRefresh)
{
	rtl8651_tblDrv_arpEntry_t * trackArpEntry;
	rtl8651_tblDrv_ipIntfEntry_t * ipIntfPtr;
	uint32 index, MAX;

	if (netif_t) 
		MAX = 1;
	else {
		MAX =  rtl8651_tblDrvPara.networkIntfTableSize;
		netif_t = DrvTbl.netIface;
	}

	for(index = 0; index < MAX; index++, netif_t++) {
		if (netif_t->valid == 0)
			continue;
		SLIST_FOREACH(ipIntfPtr, &(netif_t->IpHead), nextIp) {
			SLIST_FOREACH(trackArpEntry, &(ipIntfPtr->nextArp), next) {
				if (!memcmp(&trackArpEntry->macInfo->macAddr, macAddr, sizeof(ether_addr_t))) {
					if (isRefresh == TRUE)
						trackArpEntry->age = arpAgingTime;
					return trackArpEntry;
				}
			}
		}
	}
	return (rtl8651_tblDrv_arpEntry_t *) NULL;
}

rtl8651_tblDrv_arpEntry_t * _rtl8651_getArpEntry(rtl8651_tblDrv_networkIntfTable_t *netif_t, ipaddr_t ipAddr, int8 isRefresh) 
{
	rtl8651_tblDrv_arpEntry_t * trackArpEntry;
	rtl8651_tblDrv_ipIntfEntry_t * ipIntfPtr;
	uint32 index, MAX;


	/*
	  * Assumption:
	  * 	(1) netif_t == NULL:
	  *		Traverse all network interface to find the specified ARP. This will take a lot of time 
	  *	   	to find the specified arp entry.
	  *	(2) netif_t != NULL:
	  *		Use the specified network interface to find the specified ARP. This will have good
	  *		performance of arp lookup.
	  */

	if (netif_t) 
		MAX = 1;
	else {
		MAX =  rtl8651_tblDrvPara.networkIntfTableSize;
		netif_t = DrvTbl.netIface;
	}
	for(index=0; index< MAX; index++, netif_t++) {
		if (netif_t->valid == 0)
			continue;
		SLIST_FOREACH(ipIntfPtr, &(netif_t->IpHead), nextIp) {
				assert(SLIST_FIRST(&ipIntfPtr->ipAddr));//Network interface must have at least one address
				if((ipAddr & ipIntfPtr->ipMask) == (ipIntfPtr->ipAddr.slh_first->ipAddr & ipIntfPtr->ipMask)) {//Local host
					SLIST_FOREACH(trackArpEntry, &(ipIntfPtr->nextArp), next) {
						if (trackArpEntry->ipAddr == ipAddr) {
							/* If the arp is referenced, update its aging time */
							if (isRefresh == TRUE)
								trackArpEntry->age = arpAgingTime;
								return trackArpEntry;
						}
					}
				}
			}
		}

	return (rtl8651_tblDrv_arpEntry_t *)NULL;
}


static void _rtl8651_timeUpdateArp(uint32 secPassed) {
	rtl865x_tblAsicDrv_l2Param_t asic_l2;
	rtl8651_tblDrv_ipIntfEntry_t * ipIntfPtr;
	rtl8651_tblDrv_arpEntry_t *arpEntryPtr, *delArpEntry;
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	uint32 ifIndex;

	for(ifIndex=0,netIfPtr=&DrvTbl.netIface[0]; ifIndex<rtl8651_tblDrvPara.networkIntfTableSize; ifIndex++,netIfPtr++) {
		if (netIfPtr->valid == 0)
			continue;
		SLIST_FOREACH(ipIntfPtr, &(netIfPtr->IpHead), nextIp) {
			arpEntryPtr=ipIntfPtr->nextArp.slh_first;
			while (arpEntryPtr) {
				if (arpEntryPtr->isBcArp == 1) {
					arpEntryPtr = arpEntryPtr->next.sle_next;
					continue;
				}
				
				/* 
					If software arp entry should not be aged out... 
					(1) we decrease the passed time
					(2) check hardware arp entry's age
				*/
				if (arpEntryPtr->age > secPassed) {
					arpEntryPtr->age -= secPassed;
					if (arpEntryPtr->macInfo->configToAsic == 1) {
						if (rtl8651_getAsicL2Table(rtl8651_filterDbIndex(&(arpEntryPtr->macInfo->macAddr)), arpEntryPtr->macInfo->asicPos, &asic_l2) == SUCCESS) {
							/* If hardware arp entry has not yet aged out, do nothing. */
							if (asic_l2.ageSec > 0) {
								arpEntryPtr = arpEntryPtr->next.sle_next;
								continue;
							}
						}
						/* If hardware arp entry has been aged out, but it is configured by PS, we refill the timer. */
						if (arpEntryPtr->fromApp == 1) {
							arpEntryPtr->age = arpAgingTime;
							asic_l2.ageSec = arpAgingTime;
							rtl8651_setAsicL2Table(rtl8651_filterDbIndex(&(arpEntryPtr->macInfo->macAddr)), arpEntryPtr->macInfo->asicPos, &asic_l2);
							arpEntryPtr = arpEntryPtr->next.sle_next;
							continue;
						}						
						/* If hardware arp entry has been aged out, set software arp age to 0, and preparing to delete the arp entry. */
						else
							arpEntryPtr->age = 0;

					}
					else {
						arpEntryPtr = arpEntryPtr->next.sle_next;
						continue;
					}
				}
				/* 
					If software arp entry should be aged out...
					(1) we set the software arp age to 0
					(2) check hardware arp entry's age
				*/
				else	 {
					arpEntryPtr->age = 0;
					if (arpEntryPtr->macInfo->configToAsic == 1) {
						if (rtl8651_getAsicL2Table(rtl8651_filterDbIndex(&(arpEntryPtr->macInfo->macAddr)), arpEntryPtr->macInfo->asicPos, &asic_l2) == SUCCESS) {
							/* If hardware arp entry has not yet aged out, do nothing. */
							if (asic_l2.ageSec > 0) {
								arpEntryPtr->age = asic_l2.ageSec >> 2;
								arpEntryPtr = arpEntryPtr->next.sle_next;
								continue;
							}
						}
						/* If hardware arp entry has been aged out, but it is configured by PS, we refill the timer. */
						if (arpEntryPtr->fromApp == 1) {
							arpEntryPtr->age = arpAgingTime;
							asic_l2.ageSec = arpAgingTime;
							rtl8651_setAsicL2Table(rtl8651_filterDbIndex(&(arpEntryPtr->macInfo->macAddr)), arpEntryPtr->macInfo->asicPos, &asic_l2);
							arpEntryPtr = arpEntryPtr->next.sle_next;
							continue;
						}
						/* If hardware arp entry has been aged out, prepare to delete the arp entry. */
					}
				}
				/*
				 * Note: Function is not complete.
				 *		If the mac address is pending, currently, we remove it immediately.
				 *	In the future, each mac entry should maintain an aging mechanism by 
				 *  driver.
				 */
				delArpEntry = arpEntryPtr;
				arpEntryPtr = arpEntryPtr->next.sle_next;
				_rtl8651_delLocalArp(ARP_FORCE_DEL, delArpEntry->ipAddr, netIfPtr);

			}
		}
	}
}


void _rtl8651_removeArpAndNaptFlowWhenLinkDown(int32 port, uint32 *linkId) 
{
		rtl8651_tblDrv_networkIntfTable_t *netif_t = DrvTbl.netIface;
		rtl8651_tblDrv_ipIntfEntry_t *ipintf_t;
		rtl8651_tblDrv_arpEntry_t *arp_t;
		uint32 entry;

		//cfliu: What is link Id? link Id is a virtual port number given by WLAN driver(so we don't interpret it)
		//which represents a WDS link for wireless application.
		//since 8650 uses MII loopback port and 8650B has only 3 ext ports, it is always insufficient to mapt 8650B ext ports
		//to typical 8 WDS links.
		// This function would be called by fwd engine when WLAN driver tells it to shutdown a WDS link.
		for(entry=0; entry<rtl8651_tblDrvPara.networkIntfTableSize; entry++, netif_t++) {
			if (netif_t->valid == 0 || !(netif_t->linkLayerType==RTL8651_LL_VLAN||netif_t->linkLayerType==RTL8651_LL_LIPUNNUMBERED))
				continue;
			SLIST_FOREACH(ipintf_t, &netif_t->IpHead, nextIp) {
				arp_t = SLIST_FIRST(&ipintf_t->nextArp);
				while (arp_t) {
					if (arp_t->fromDrv==1 && ((port>=0&&(arp_t->macInfo->memberPortMask & (1<<port)))
						||(linkId&&arp_t->macInfo->linkId&&*linkId))) {
					
						/* Remove Napt TCP/UDP flow */
						_rtl8651_removeNaptConnectionBySrcIP(arp_t->ipAddr);
						/* Remove Napt ICMP flow */
						_rtl8651_removeIcmpFlowBySrcIP(arp_t->ipAddr);
						if (arp_t->isBcArp == 0) {
							_rtl8651_delLocalArp(TRUE, arp_t->ipAddr, netif_t);
							arp_t = SLIST_FIRST(&ipintf_t->nextArp);
						} else arp_t = SLIST_NEXT(arp_t, next);
					} else arp_t = SLIST_NEXT(arp_t, next);
				}
			}
		}
}

/*
@func int32 | rtl8651_setArpAgingTime | Set arp aging time.
@parm uint16 | agTime | Aging time.
@rvalue SUCCESS | The aging time was set successfully.
*/
int32 rtl8651_setArpAgingTime(uint16 agTime) {
	rtlglue_drvMutexLock();//Lock resource
	arpAgingTime = agTime;
	rtlglue_drvMutexUnlock();//Unlock resource
	return SUCCESS;
}


/*
@func int32 | rtl8651_addArp | Add an ARP entry.
@parm ipaddr_t | ipAddr | IP address.
@parm ether_addr_t * | macAddr | Pointer to MAC address.
@parm int8 * | ifName | Network interface name.
@parm uint32 | port | Physical port number.
@rvalue SUCCESS | An Arp entry was successfully added.
@rvalue TBLDRV_EINVNETIFNAME | The specified network interface was not found.
@rvalue TBLDRV_ENOFREEBUFFER | Out of memory buffer.
@rvalue TBLDRV_ENONBROADCASTNET | Non Broadcast network is forbidden to add ARP entry.
@rvalue TBLDRV_EARPALREADYEXIST | ARP entry already exists.
@rvalue TBLDRV_EARPCANNOTADDL2ENTRY | The L2 Entry cannot be added.
@rvalue TBLDRV_ENOIPINTFFOUND | The specified IP interface was not found.
@rvalue TBLDRV_EINVALIDINPUT | Invalid Input parameter.
*/
int32 rtl8651_addArp(ipaddr_t ipAddr, ether_addr_t * macAddr, int8 * ifName, uint32 port) 
{
	rtl8651_tblDrv_ipIntfIpAddrEntry_t *ip_t;
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_ipIntfEntry_t *ipintf_t;
	int32 retval = TBLDRV_EINVNETIFNAME;
	uint32 index;

	rtlglue_drvMutexLock();//Lock resource	

	netif_t = DrvTbl.netIface;
	for(index=0; index<rtl8651_tblDrvPara.networkIntfTableSize; index++, netif_t++) {
		if(netif_t->valid == 1 && _strncmp(ifName, netif_t->ifName, RTL8651_NETINTF_NAME_LEN) == 0) {//Interface is specified
			/*
			 * Currently, We forbid adding ARP entry for interface gateway.
			 * Because, the interface gateway already has it own mac address, if 
			 * permit, it will cause confusion.
			 */
			SLIST_FOREACH(ipintf_t, &(netif_t->IpHead), nextIp) {
				SLIST_FOREACH(ip_t, &ipintf_t->ipAddr, nextIpAddr)
					if (ip_t->ipAddr == ipAddr) {
						retval = TBLDRV_EINVALIDINPUT;
						goto out;
					}
			}
			retval = _rtl8651_addLocalArp(FALSE, ipAddr, macAddr, netif_t, port);
			goto out;
		}
	}
		

out:
	rtlglue_drvMutexUnlock();
	return retval;
}

/*
@func int32 | rtl8651_delArp | Remove an arp entry.
@parm ipaddr_t | ipAddr | IP Address.
@rvalue SUCCESS | The specified arp entry was successfully removed.
@rvalue TBLDRV_EENTRYNOTFOUND | The specified ARP entry was not found.
@rvalue TBLDRV_ENONBROADCASTNET | Non Boradcast network is forbidden to add ARP entry.
*/
int32 rtl8651_delArp(ipaddr_t ipAddr) 
{
	rtl8651_tblDrv_ipIntfEntry_t *ipIntf_t;
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	int32 retval = TBLDRV_EENTRYNOTFOUND;
	uint32 index;

	rtlglue_drvMutexLock();	/* Lock resource */
	
	netif_t = DrvTbl.netIface;
	for(index=0; index< rtl8651_tblDrvPara.networkIntfTableSize; index++, netif_t++) {
		if(netif_t->valid == 1) {	/* Interface valid, network interface is meaningful */
			SLIST_FOREACH(ipIntf_t, &(netif_t->IpHead), nextIp) 
				if ((ipIntf_t->ipMask&ipIntf_t->ipAddr.slh_first->ipAddr) == (ipIntf_t->ipMask&ipAddr)) {
					retval = _rtl8651_delLocalArp(FALSE, ipAddr, netif_t);
					goto out;
				}
			}
		}
	
out:
	rtlglue_drvMutexUnlock();
	return retval;
}




#define RTL8651_LOCAL_SERVER_APIs
/*=======================================================================================
 * RTL8651 Local Server APIs:
 *	1. Internal APIs
 *		. _rtl8651_localServerIpAddr()
 *	2. External APIs
 *		. rtl8651_addLocalServer()
 *		. rtl8651_delLocalServer()
 *=======================================================================================*/
int8 _rtl8651_localServerIpAddr(ipaddr_t ipAddr) {
	rtl8651_tblDrv_ipIntfEntry_t * ipIntfPtr;
	rtl8651_tblDrv_ipIntfIpAddrEntry_t * ipAddrPtr;
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	uint32 netIdx;

	for(netIdx=0, netIfPtr=&DrvTbl.netIface[0]; netIdx<rtl8651_tblDrvPara.networkIntfTableSize; netIdx++, netIfPtr++)
		if(netIfPtr->valid == 1) {
			/* 
			  *	chenyl (2005 / 10 / 04 ):
			  *
			  *		Local Server defined in ASIC would allow the NPI being LOCAL_SERVER in IP table.
			  *		Therefore, we don't prevent this kind of setting.
			  *
			  */
			if ( 1 /* DrvTbl.vlan[rtl8651_vlanTableIndex(netIfPtr->vid)].internal == 0 */) {
				SLIST_FOREACH(ipIntfPtr, &(netIfPtr->IpHead), nextIp) {
					if (SLIST_FIRST(&ipIntfPtr->localServer) == NULL)
						continue;
					SLIST_FOREACH(ipAddrPtr, &ipIntfPtr->localServer, nextIpAddr)
					{
						assert(ipAddrPtr);
						if(ipAddrPtr->ipAddr == ipAddr)
						{
							return TRUE;
						}
					}
				}
			}
		}

	return FALSE;	/* No such ip address appear in network address list */
}

/*
@func int32 | rtl8651_addLocalServer | Add a local server.
@parm ipaddr_t | ipAddr | Local Server IP address. It should be an external 
@rvalue TBLDRV_EINVALIDLSIPADDR | Invlaid Local Server IP address.
@rvalue TBLDRV_ELSCANNOTBEGWIP | Local Server IP address cannot be gateway's IP address.
@rvalue TBLDRV_ELSALREADYEXISTS | The specified local server already exists.
@rvalue TBLDRV_ENOFREEBUFFER | Out of memory buffer.
*/
int32 rtl8651_addLocalServer(ipaddr_t ipAddr) {
	rtl8651_tblDrv_ipIntfEntry_t * ipintf_t;
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	rtl8651_tblDrv_ipIntfIpAddrEntry_t *tempIpAddr;
	int8 lanSideExtIp, arrangeAcl = FALSE;
	uint32 netIdx;

	rtlglue_drvMutexLock();
	netIfPtr = &DrvTbl.netIface[0];
	for(netIdx=0; netIdx< rtl8651_tblDrvPara.networkIntfTableSize; netIdx++, netIfPtr++) {
		if (netIfPtr->valid == 0)
			continue;	
			SLIST_FOREACH(ipintf_t, &(netIfPtr->IpHead), nextIp) {
				assert(SLIST_FIRST(&ipintf_t->ipAddr));	/* Network interface must have at least one address */
			if((ipAddr & ipintf_t->ipMask) == (ipintf_t->ipAddr.slh_first->ipAddr & ipintf_t->ipMask)) {
				lanSideExtIp = (ipintf_t->lanSideExternal==1)? TRUE: FALSE;
					goto subnet_found; /* Local Server at this subnet */
			}
		} 
	}
	/*
	 * The IP interface which the added Local Server belongs
	 * to is not found.
	 */
	rtlglue_drvMutexUnlock();
	return TBLDRV_EINVALIDLSIPADDR;
	
subnet_found:
	/*
	 * Check Whether the added Local Server is the gateway interfaece
	 * IP Address. If yes, reject the request!
	 */
	SLIST_FOREACH(tempIpAddr, &ipintf_t->ipAddr, nextIpAddr)
		LOCK_ERROR_CHECK((tempIpAddr->ipAddr == ipAddr), TBLDRV_ELSCANNOTBEGWIP);
	/* Check Whether the added Local Server already exists */
	SLIST_FOREACH(tempIpAddr, &(ipintf_t->localServer), nextIpAddr) 
		LOCK_ERROR_CHECK((tempIpAddr->ipAddr == ipAddr), TBLDRV_ELSALREADYEXISTS);
	/* Check if there still has enough free slot for new added one */
	LOCK_ERROR_CHECK((SLIST_FIRST(&DrvTbl.freeList.ipAddr) == NULL), TBLDRV_ENOFREEBUFFER);
	if ((naptIpCount+natIpCount+localServerCount) == RTL8651_IPTABLE_SIZE)
		arrangeAcl = TRUE;
	
	/*
	 * If the Interface is an external interface, Try to write the Local Server
	 * to the ASIC IP Table.
	 */
	if (DrvTbl.vlan[rtl8651_vlanTableIndex(netIfPtr->vid)].internal == 0 || lanSideExtIp == TRUE) { 
		LOCK_ERROR_CHECK(
			(_rtl8651_addIpEntry(ipAddr, ipAddr, IP_ATTR_LS, NULL) == NULL), 
			TBLDRV_ENOFREEBUFFER
		);
	}
	/*
	 * Allocate free slot for new added Local Server and insert it into
	 * internal data structure.
	 */
	tempIpAddr = SLIST_FIRST(&DrvTbl.freeList.ipAddr);
	SLIST_REMOVE_HEAD(&DrvTbl.freeList.ipAddr, nextIpAddr);
	tempIpAddr->ipAddr = ipAddr;
	SLIST_INSERT_HEAD(&(ipintf_t->localServer), tempIpAddr, nextIpAddr);
	if (arrangeAcl == TRUE)
		_rtl8651_arrangeAllACL();
	rtlglue_drvMutexUnlock();
	return SUCCESS;
}

/*
@func int32 | rtl8651_delLocalServer | Remove a local server.
@parm ipaddr_t | ipAddr | Local Server IP address.
@rvalue SUCCESS | The Local Server was successfully removed.
@rvalue TBLDRV_EINVALIDLSIPADDR | Invalid Local Server specified.
*/
int32 rtl8651_delLocalServer(ipaddr_t ipAddr) {
	rtl8651_tblDrv_ipIntfIpAddrEntry_t *trackIpAddr;
	rtl8651_tblDrv_ipIntfEntry_t * netPtr=NULL;
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	uint32 netIdx;

	rtlglue_drvMutexLock();
	
	for(netIdx=0,netIfPtr=&DrvTbl.netIface[0]; netIdx<rtl8651_tblDrvPara.networkIntfTableSize; netIdx++,netIfPtr++) 
		if(netIfPtr->valid == 1) {
			SLIST_FOREACH(netPtr, &(netIfPtr->IpHead), nextIp) {
				assert(SLIST_FIRST(&netPtr->ipAddr));//Network interface must have at least one address
				if((ipAddr & netPtr->ipMask) == (netPtr->ipAddr.slh_first->ipAddr & netPtr->ipMask))
					goto subnet_found; /* Local Server at this subnet */
			}
		}
		
subnet_found:

	if (netIdx < rtl8651_tblDrvPara.networkIntfTableSize) {
		assert(netPtr);
		SLIST_FOREACH(trackIpAddr, &(netPtr->localServer), nextIpAddr) {	
			if (trackIpAddr->ipAddr == ipAddr) { /* entry found */				
				SLIST_REMOVE(&(netPtr->localServer), trackIpAddr, rtl8651_tblDrv_ipIntfIpAddrEntry_s, nextIpAddr);
				SLIST_INSERT_HEAD(&DrvTbl.freeList.ipAddr, trackIpAddr, nextIpAddr);
				_rtl8651_delIpEntry(IP_ATTR_LS, ipAddr, ipAddr);			
				rtlglue_drvMutexUnlock();
				return SUCCESS;
			}
		}
	}
	rtlglue_drvMutexUnlock();
	return(TBLDRV_EINVALIDLSIPADDR); /* Local Server not found */
}



/*=======================================
  * Routing APIs
  *=======================================*/
#define RTL8651_ROUTING_TABLE_APIs

void _rtl8651_rmPPTPL2TPInterfaceRoute(ipaddr_t s_ipaddr)
{
	rtl8651_tblDrv_routeTable_t *rt;
	ipaddr_t mask;
	int32 retval;

	rt = _rtl8651_getRoutingEntry(s_ipaddr);
	mask = rt->ipMask;
#if 0
	printk("_rtl8651_getRoutingEntry -- ipaddr: %u.%u.%u.%u, ipmask: %u.%u.%u.%u\n", 
		NIPQUAD(rt->ipAddr), NIPQUAD(rt->ipMask));
#endif
	
	retval = _rtl8651_delInterfaceRoute(s_ipaddr, mask);
	assert(retval == SUCCESS);
}

void _rtl8651_addPPTPL2TPInterfaceRoute(rtl8651_tblDrv_pppTable_t *pppEntry_t, ipaddr_t s_ipaddr)
{
	int32 retval;
	rtl8651_tblDrv_networkIntfTable_t *netif_t = pppEntry_t->netif_t;
	rtl8651_tblDrv_ipIntfEntry_t *ipintf_t;
	SLIST_FOREACH(ipintf_t, &(netif_t->IpHead), nextIp)
		if((ipintf_t->ipAddr.slh_first->ipAddr & ipintf_t->ipMask) == ( s_ipaddr & ipintf_t->ipMask)) 
			break;

	if (ipintf_t) {
		retval = _rtl8651_addInterfaceRoute(netif_t, ipintf_t, ipintf_t ->arpStartPos, ipintf_t->arpEndPos);
		assert(retval == SUCCESS);
		return;
	}
	assert(0);
}

void _rtl8651_updateDefaultRoute(void)  
{
	rtl8651_tblDrv_routeTable_t *default_rt;
	rtl865x_tblAsicDrv_routingParam_t asic_rt;
	int32 retval;
	

	/*
	  * (1) set default route:  		default route present && rtEntry <= RTL8651_ROUTINGTBL_SIZE-1
	  * (2) set default route to CPU: 	rtEntry > RTL8651_ROUTINGTBL_SIZE-1
	  * (3) set default route to DROP:	default route not present and rtEntry <= RTL8651_ROUTINGTBL_SIZE-1
	  */

	bzero(&asic_rt, sizeof(rtl865x_tblAsicDrv_routingParam_t));
	
	if (rtEntry >  RTL8651_ROUTINGTBL_SIZE-1) {
		/*   
		  *   Although the process of this routing entry is CPU, 
		  *   we still have to assign "vid" field for packet decision process use. 
		  *   Always assign one "external" VLAN index to 'vid' field.
		  *   Therefore, the action will be either "Routing" or "NAPT".
		  *   If the action is Routing, the packet will be trapped to CPU due to L3 table lookup.
		  *   If the action is NAPT, the packet will be trapped to CPU due to ASIC NAPT table lookup miss.
		  *   (Assumption: ASIC SPI function is turned off.)
		  */
		asic_rt.vidx = _rtl8651_getOneVidx(FALSE);
		asic_rt.process = RT_CPU;
	}
	else {
		default_rt = &DrvTbl.route[rtl8651_tblDrvPara.routingTableSize-1];
		if (default_rt->valid == 1) {
			_rtl8651_updateL3Entry(default_rt);
			return;
		}
		/*   Although the process of this routing entry is DROP, 
		  *   we still have to assign "vid" field for packet decision process use. 
		  *   Always assign one "internal" VLAN index to 'vid' field.
		  *   Therefore, the action will be either "Routing" or "DROP".
		  *   If the action is Routing, the packet will be dropped due to L3 table lookup.
		  *   (Assumption: ASIC SPI function is turned off.)
		  */
		asic_rt.vidx = _rtl8651_getOneVidx(TRUE);

		/* From now on (2005.7.13), when default route is un-defined, driver will config its PROCESS to CPU. */
		//asic_rt.process = RT_DROP;
		asic_rt.process = RT_CPU;
	}
	retval = rtl8651_setAsicRouting(RTL8651_ROUTINGTBL_SIZE-1,  &asic_rt);
	assert(retval == SUCCESS);	
}


#if 0
void _rtl8651_updateDefaultRoute(void) 
{
	rtl8651_tblDrv_routeTable_t *default_rt, entry_t;
	rtl865x_tblAsicDrv_routingParam_t asic_rt;
	rtl8651_tblDrv_pppTable_t *pppoe_t;
	int32 entry, retval, sessionEntry;

	default_rt = &DrvTbl.route[rtl8651_tblDrvPara.routingTableSize-1];
		
	/*
	  * 1. The last one routing entry is invalid, which means no default route is present.
	  *     In this situation, we shall add a default route to DROP automatically. 
	  * 2. Note that if there exists PPPoE session (no matter dial up or not), we shall add
	  *     to CPU instead of DROP when in no default route environment.
	  * 3. If routing entries > RTL8651_ROUTINGTBL_SIZE-1, the ASIC default route should
	  *	 be set to toCPU
	  *
	  * case 1: DROP 
	  *		no default route  && rtEntry<RTL8651_ROUTINGTBL_SIZE-1
	  * case 2: CPU
	  *		a. rtEntry > RTL8651_ROUTINGTBL_SIZE-1  ||
	  *		b. present pppoe/l2tp/pptp && present default route
	  * case 3: set default route
	  *		present default route &&  rtEntry<=RTL8651_ROUTINGTBL_SIZE-1
	  *
	  * Note that: rtEntry not include default route
	  */

	/* case 3: */
	if (default_rt->valid && rtEntry<=RTL8651_ROUTINGTBL_SIZE-1) {
		_rtl8651_updateL3Entry(default_rt);
		return;
	}

	sessionEntry = RTL8651_PPPOE_NUMBER+rtl8651_tblDrvPara.pptpSize+rtl8651_tblDrvPara.l2tpSize;
	bzero(&asic_rt, sizeof(rtl865x_tblAsicDrv_routingParam_t));
	pppoe_t = DrvTbl.pppoe;
	for(entry=0; entry<sessionEntry; entry++, pppoe_t++) {
		if (pppoe_t->valid) 
			break;
	}

	/* case 2: */
	if (rtEntry > RTL8651_ROUTINGTBL_SIZE-1) {// || (entry<sessionEntry && default_rt->valid==0)) {
		asic_rt.process = RT_CPU;
		goto update_;
	}

	/* case 1: */
	if (default_rt->valid==0 &&  rtEntry<=RTL8651_ROUTINGTBL_SIZE-1) { //entry==sessionEntry
		asic_rt.process = RT_DROP;
		goto update_;
	}
	assert(0);

update_:
	retval = rtl8651_setAsicRouting(RTL8651_ROUTINGTBL_SIZE-1,  &asic_rt);
	assert(retval == SUCCESS);
}
#endif

		 	

static void _rtl8651_refillL3Entry(rtl8651_tblDrv_routeTable_t *rt_t)
{	
	rtl8651_tblDrv_routeTable_t *entry_t;
	int32 entry, retval;
	uint8 process;

	/* The input parameter should be either default route or ranged from 0 to 6 */
	assert((rt_t - DrvTbl.route) < RTL8651_ROUTINGTBL_SIZE -1 || (rt_t - DrvTbl.route)==rtl8651_tblDrvPara.routingTableSize-1);
	entry_t = &DrvTbl.route[RTL8651_ROUTINGTBL_SIZE-1];
	/* Find a pending entry to write to ASIC */
	for(entry=RTL8651_ROUTINGTBL_SIZE-1; entry<rtl8651_tblDrvPara.routingTableSize-1; entry++, entry_t++) {
		if (entry_t->valid == 1) {
			rtl8651_memcpy(rt_t, entry_t, sizeof(rtl8651_tblDrv_routeTable_t));
			entry_t->valid = 0;
			/* 
			 * Try to write pending entry into ASIC:
			 * It may be fail due to out of nexthop entry. In such case, we still need to
			 * pend the l3 entry in the software. Such situation means that even still have 
			 * free ASIC L3 entry but no nexthop entry allocated, the ASIC still cannot take 
			 * care of forwarding packet.
			 */
			process					= rt_t->process;
			rt_t->process			= RT_NEXTHOP;
			rt_t->rt_nhalog			= RT_ALOG_SIP; /* use per-source IP */
			rt_t->rt_ipDomain		= RT_DOMAIN_16_1;
			rt_t->rt_nhnext_t		= NULL;

			if (rt_t->nextHopType== RTL8651_LL_VLAN)
				retval = _rtl8651_addNextHopEntry(NEXTHOP_L3, rt_t, rt_t->netif_t, rt_t->nextHop, 0, NULL);
			else retval = _rtl8651_addNextHopEntry(NEXTHOP_L3, rt_t, rt_t->netif_t, 0, 0, rt_t->rt_nhSession_t);
			if (retval == FAILED) {
				/* 
				 * It seems the l3 entry cannot get a nexthop entry. Hence
				 * pending it in the software!!
				 */
				rt_t->process = process;
			} 
			else rt_t->rt_nexthop_t = rt_t->rt_nhstart_t;
			_rtl8651_updateL3Entry(rt_t);
			break;
		}
	}
}

		 	
static rtl8651_tblDrv_routeTable_t *
_rtl8651_getRTentry(ipaddr_t ipAddr, ipaddr_t ipMask) 
{
	rtl8651_tblDrv_routeTable_t *rt = DrvTbl.route;
	uint32 rtIdx;
	
	for(rtIdx=0; rtIdx<rtl8651_tblDrvPara.routingTableSize; rtIdx++, rt++) 
		if ((rt->valid==1)&&((rt->ipAddr&rt->ipMask)==(ipAddr&ipMask))&&(rt->ipMask==ipMask))
			return (rtl8651_tblDrv_routeTable_t *)rt;
	return (rtl8651_tblDrv_routeTable_t *)NULL;
}


rtl8651_tblDrv_routeTable_t * _rtl8651_getRoutingEntry(ipaddr_t dstIpAddr) 
{
	rtl8651_tblDrv_routeTable_t *tmpRtEntry = NULL;
	rtl8651_tblDrv_routeTable_t *rt=DrvTbl.route;
	uint32 rtIdx, mask;

	for(rtIdx=0, mask=0; rtIdx<rtl8651_tblDrvPara.routingTableSize; rtIdx++,rt++) {
		if (rt->valid == 1 && rt->ipAddr == (rt->ipMask & dstIpAddr) &&	mask <= rt->ipMask) {
			mask = rt->ipMask;
			tmpRtEntry = rt;
		}
	}
	return (rtl8651_tblDrv_routeTable_t *)tmpRtEntry;
}

static void _rtl8651_updateL3Entry(rtl8651_tblDrv_routeTable_t *rt_t)
{
	rtl865x_tblAsicDrv_routingParam_t asic_t;
	uint32 entry, mbrMask, entryNum;
	int32 ret;
	
	assert(rt_t);

	/* If the route references PPTP/L2TP, try to update L2 entry member port */
	if (rt_t->nextHopType == RTL8651_LL_PPTP || rt_t->nextHopType == RTL8651_LL_L2TP) {
		rtl8651_tblDrv_filterDbTableEntry_t *macInfo;
		rtl865x_tblAsicDrv_l2Param_t asic_l2;
		uint32 nextHopRow, nextHopColumn;

		/*
		 * If a rotue references to a dialed-up PPTP/L2TP session, always modify the member port of 
		 * layer 2 entry to extension port.
		 */

		if (rt_t->rt_nhSession_t->initiated == 1) {
			/* (1) modify routing information for PPTP/L2TP */
			rt_t->dvid	= tunnel.loopBackVid;
			if (rt_t->process == RT_NEXTHOP)
				rt_t->rt_nhstart_t->dvid = rtl8651_vlanTableIndex(rt_t->dvid);	
			
			/* (2) modify L2 entry information for PPTP/L2TP */
			macInfo=rt_t->rt_nhSession_t->macInfo;
			assert(macInfo);
			nextHopRow				= rtl8651_filterDbIndex(&macInfo->macAddr);
			nextHopColumn			= macInfo->asicPos;
			mbrMask 				= (1 << tunnel.loopbackPort);
			macInfo->memberPortMask = mbrMask;
			if (macInfo->configToAsic == 1) {
				rtl8651_getAsicL2Table(nextHopRow, nextHopColumn,  &asic_l2);
				asic_l2.nhFlag			= 0 ;
				asic_l2.memberPortMask	= mbrMask;
				asic_l2.isStatic		= 1;
				rtl8651_setAsicL2Table(nextHopRow, nextHopColumn, &asic_l2);
			}
		}
	}
	
	/* Only first seven entries and last one entry can be written to ASIC */
	entry = L3_TABLE_INDEX(rt_t);
	if (entry>=RTL8651_ROUTINGTBL_SIZE-1 && entry!=rtl8651_tblDrvPara.routingTableSize-1)	
		return;
	
	if (entry == rtl8651_tblDrvPara.routingTableSize-1)
		entry = RTL8651_ROUTINGTBL_SIZE-1;
	
	bzero(&asic_t, sizeof(rtl865x_tblAsicDrv_routingParam_t));
	asic_t.ipAddr	= rt_t->ipAddr;
	asic_t.ipMask	= rt_t->ipMask;
	switch (rt_t->process)
	{
	case RT_ARP:
		asic_t.process	= RT_ARP;
		asic_t.vidx	= rtl8651_vlanTableIndex(rt_t->dvid);
		asic_t.arpStart	= rt_t->rt_nharpsta;
		asic_t.arpEnd	= rt_t->rt_nharpend;	
		break;

	case RT_L2:
	case RT_SESSION:
		/* 
		 * We already enable 8650 new feature, hence no longer to use RT_L2 and 
		 * RT_SESSION type in ASIC. But these types still availabe in software. Software
		 * uses RT_L2 and RT_SESSION instead of RT_NEXTHOP. If process is either RT_L2 or  
		 * RT_SESSION, it implies the routing entry cannot allocate a free nexthop entry,
		 * hence use trap packets to cpu.
		 */		  
		asic_t.process = RT_CPU;	
		/*   
		  *   Although the process of this routing entry is CPU, 
		  *   we still have to assign "vid" field for ASIC packet decision process use. 
		  *   Always assign one "external" VLAN index to 'vid' field.
		  *   Therefore, the action will be either "Routing" or "NAPT".
		  *   If the action is Routing, the packet will be trapped to CPU due to L3 table lookup.
		  *   If the action is NAPT, the packet will be trapped to CPU due to ASIC NAPT table lookup miss.
		  *   (Assumption: ASIC SPI function is turned off.)
		  */
		// Driver should use route's dvid.
		//asic_t.vidx = _rtl8651_getOneVidx(FALSE);
		asic_t.vidx = rtl8651_vlanTableIndex(rt_t->dvid);
		break;

	case RT_CPU:
		// Driver should use route's dvid.
		//asic_t.vidx = _rtl8651_getOneVidx(FALSE);
		asic_t.vidx = rtl8651_vlanTableIndex(rt_t->dvid);
		asic_t.process	= rt_t->process;
		break;

	case RT_DROP:
		/*   Although the process of this routing entry is DROP, 
		  *   we still have to assign "vid" field for ASIC packet decision process use. 
		  *   Always assign one "internal" VLAN index to 'vid' field.
		  *   Therefore, the action will be either "Routing" or "DROP".
		  *   If the action is Routing, the packet will be dropped due to L3 table lookup.
		  *   (Assumption: ASIC SPI function is turned off.)
		  */
  		// Driver should use route's dvid.
		//asic_t.vidx = _rtl8651_getOneVidx(TRUE);
		asic_t.vidx = rtl8651_vlanTableIndex(rt_t->dvid);		
		asic_t.process	= rt_t->process;
		break;

	case RT_NEXTHOP:
		entryNum = rt_t->rt_nhend_t->entryIndex - rt_t->rt_nhstart_t->entryIndex + 1;	
		asic_t.process		 = rt_t->process;
 		asic_t.nhStart		 = rt_t->rt_nhstart_t->entryIndex;
		asic_t.nhNxt	 	 = asic_t.nhStart;
		asic_t.ipDomain		 = rt_t->rt_ipDomain;
		asic_t.nhAlgo	 	 = rt_t->rt_nhalog;
		asic_t.nhNum	 	 = entryNum;
		break;

	default: assert(0);

	}
	ret = rtl8651_setAsicRouting(entry, &asic_t);
	assert( ret == SUCCESS );
}

static
int32 _rtl8651_addInterfaceRoute(rtl8651_tblDrv_networkIntfTable_t *netif_t, rtl8651_tblDrv_ipIntfEntry_t *ipintf_t, uint32 arpsta, uint32 arpend)
{
	rtl8651_tblDrv_routeTable_t *rt_t = &DrvTbl.route[0];
	rtl865x_tblAsicDrv_routingParam_t asic_t;
	rtl8651_tblDrv_ipIntfIpAddrEntry_t *ip_t;
	uint32 entry;

	/* Used for inserting a new interface route. */	
	rtl8651_tblDrv_networkIntfTable_t *tmp_netif_t;	
	rtl8651_tblDrv_ipIntfEntry_t *tmp_ipintf_t;	
	ipaddr_t tmp_ip;	
	uint32 i, insIdx = 0;	
	uint32 intfRouteFound = 1;

	/* Marked by shliu. */
	/*
	for(entry=0; entry<RTL8651_ROUTINGTBL_SIZE-1; entry++, rt_t++)
		if (rt_t->valid == 0)
			break;
	*/


	for(entry = 0; entry < rtl8651_tblDrvPara.routingTableSize-1; entry++, rt_t++) {
		if(rt_t->valid == 0)
			break;
		if(intfRouteFound == 1)
			for(intfRouteFound = 0, tmp_netif_t = &DrvTbl.netIface[0]; tmp_netif_t->valid == 1; tmp_netif_t++) {		// Check whether this entry is an interface route. Once we found non-interface route, we don't check anymore. 
				SLIST_FOREACH(tmp_ipintf_t, &(tmp_netif_t->IpHead), nextIp) {
					tmp_ip = (tmp_ipintf_t->ipAddr.slh_first->ipAddr & tmp_ipintf_t->ipMask);
					if(tmp_ip == rt_t->ipAddr && tmp_ipintf_t->ipMask == rt_t->ipMask) {
						intfRouteFound = 1;
						insIdx++;
						break;
					}
				}
				if(intfRouteFound == 1 )
					break;
			} // End of checking a entry is an interface route.
	} // End of visiting the software routing table.

	if(insIdx >= RTL8651_ROUTINGTBL_SIZE-1 || entry == rtl8651_tblDrvPara.routingTableSize-1)	// If the insertion index is greater than or equal to (ASIC routing table size - 1), give up adding the new interface route.
		return FAILED;																	// If the software routing table is full, give up adding the new interface route.

	// Entries after 'insIdx' should be shifted to (original index + 1).
	for(i = entry; i > insIdx; i--)
		DrvTbl.route[i] = DrvTbl.route[i-1];

	rt_t = &DrvTbl.route[insIdx];

	/* Marked by shliu */
	/*
	if (entry == RTL8651_ROUTINGTBL_SIZE-1)
		return FAILED;
	*/

	/* Add interface route */
	ip_t = SLIST_FIRST(&ipintf_t->ipAddr);
	rt_t->ipAddr		= ip_t->ipAddr & ipintf_t->ipMask;
	rt_t->ipMask		= ipintf_t->ipMask;
	rt_t->nextHop		= 0;
	if (netif_t->linkLayerType==RTL8651_LL_LIPUNNUMBERED) {
		rt_t->nextHopType = RTL8651_LL_VLAN;
		rt_t->process		= (arpsta==0xffffffff)? RT_CPU: RT_ARP;
	}
	else {
		rt_t->nextHopType	= netif_t->linkLayerType;
		/*
		  *   When the linkLayerType is PPTP,
		  *   the PROCESS of the interface routing entry is always to CPU.
		  *   This is for "pptp dial demand (when pinging  pptp server IP)" spec.
		  */
		//if (netif_t->linkLayerType == RTL8651_LL_PPTP && DrvTbl.vlan[rtl8651_vlanTableIndex(netif_t->vid)].internal == 0) {

		if (netif_t->linkLayerType == RTL8651_LL_PPTP || netif_t->linkLayerType == RTL8651_LL_L2TP)
			rt_t->process		= RT_CPU;
		else
			rt_t->process		= (arpsta==0xffffffff)? RT_CPU: RT_ARP;
		
	}
	rt_t->ipClass		= (arpsta==0xffffffff)? 1: 0;
	rt_t->ifrt_flag		= TRUE;
	rt_t->valid		= 1;
	rt_t->netif_t		= netif_t;
	rt_t->ipintf_t		= ipintf_t;
	rt_t->dvid		= netif_t->vid;
	rt_t->rt_nharpsta 	= arpsta;
	rt_t->rt_nharpend	= arpend;

	asic_t.ipAddr 		= rt_t->ipAddr;
	asic_t.ipMask 		= rt_t->ipMask;
	asic_t.process 	= rt_t->process;

	/* Vidx decision */
	if (rt_t->process == RT_CPU)
		asic_t.vidx = _rtl8651_getOneVidx(FALSE);
	else if (rt_t->process == RT_DROP)
		asic_t.vidx = _rtl8651_getOneVidx(TRUE);
	else
		asic_t.vidx	= rtl8651_vlanTableIndex(rt_t->dvid);
	
	asic_t.arpStart	= rt_t->rt_nharpsta;
	asic_t.arpEnd		= rt_t->rt_nharpend;

#if 0	// RTL865xB d-cut testing hash-1.
	// ----- Testing ---------
	asic_t.arpIpIdx	= 3;
	// ----- End of testing ----
#endif

	/* Marked by shliu */
	/*
	rtl8651_setAsicRouting(entry, &asic_t);
	*/
	rtl8651_setAsicRouting(insIdx, &asic_t);
	rtEntry ++;

	// If there is any non-interface route, add it to the right place of ASIC L3 table.
	for(i = insIdx + 1; i < RTL8651_ROUTINGTBL_SIZE-1 && i < rtEntry; i++)
		_rtl8651_updateL3Entry(&DrvTbl.route[i]);
	
	return SUCCESS;
}


static int32 _rtl8651_delInterfaceRoute(ipaddr_t ipAddr, ipaddr_t ipMask)
{
	rtl8651_tblDrv_arpAsicArrangementEntry_t *asic_arp_t;
	rtl8651_tblDrv_routeTable_t *rt_t=DrvTbl.route;
	uint32 entry;
	
	for(entry=0; entry<RTL8651_ROUTINGTBL_SIZE-1; entry++, rt_t++) {
		if (rt_t->valid && ((rt_t->ipAddr&rt_t->ipMask)==(ipAddr&ipMask)) && (ipMask==rt_t->ipMask)) {
			rt_t->valid 	= 0; /* Invalid the entry */
			rt_t->ipAddr	= 0;
			rtl8651_delAsicRouting(entry);
			if (rt_t->rt_nharpsta != 0xffffffff) {
				asic_arp_t = &AsicTableUsage.arp[rt_t->rt_nharpsta];
				for(entry=rt_t->rt_nharpsta; entry<=rt_t->rt_nharpend; entry++)
					asic_arp_t->valid = 0;
			}
			rtEntry --;
			return SUCCESS;
		}
	}
	return FAILED;
}



static int32 _rtl8651_addRoute(ipaddr_t ipAddr, ipaddr_t ipMask, int8 * ifName, ipaddr_t nextHop)
{
//	rtl8651_tblDrvSession_t *session1_t = NULL;
	rtl8651_tblDrvSession_t	*session_t = NULL;
	rtl8651_tblDrv_arpEntry_t *arpEntry_t = NULL;
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_ipIntfIpAddrEntry_t *ip_t;
	rtl8651_tblDrv_ipIntfEntry_t *ipintf_t;
	rtl8651_tblDrv_routeTable_t *rt;
	uint32 rtIdx, llType;
	int32 retval;

	/* Used for inserting new routing entry. */	
	rtl8651_tblDrv_networkIntfTable_t *tmp_netif_t;	
	rtl8651_tblDrv_ipIntfEntry_t *tmp_ipintf_t;	
	ipaddr_t tmp_ip;	
	uint32 i, insIdx = 0, zeroMask_ipintf = 0;
	uint32 intfRouteFound = 1, insIdxFixed = 0;
	
/* Debugging */
/*
	printk("Add Route To Rome Driver : ip %u.%u.%u.%u \nMask %u.%u.%u.%u IfName %s NextHop %u.%u.%u.%u\n",
		NIPQUAD(ipAddr), NIPQUAD(ipMask), ifName, NIPQUAD(nextHop));
*/
	/* Find the specified network interface (next hop belongs to)*/
	if ((netif_t=GET_NETWORK_INTERFACE(ifName, NULL)) == NULL)
		return TBLDRV_EINVNETIFNAME;

	/* check if the network interface of next hop is a legal network interface */
	if (netif_t->linkLayerType == 0)
		return TBLDRV_ENOLLTYPESPECIFY;
	
	/* Forbid Duplicate routing entry addition */
	if (_rtl8651_getRTentry(ipAddr, ipMask) != NULL)
		return TBLDRV_ERTALREADYEXIST;

	/* If the specified nexthop is the Gateway's IP address, forbid it!!! */
	SLIST_FOREACH(ipintf_t, &(netif_t->IpHead), nextIp) {
		SLIST_FOREACH(ip_t, &ipintf_t->ipAddr, nextIpAddr) {
			if (!(netif_t->linkLayerType == RTL8651_LL_VLAN && ipintf_t->ipMask == 0) && nextHop == ip_t->ipAddr)
				return TBLDRV_ERTINVALNEXTHOP;
		}
	}

	if (	((llType= netif_t->linkLayerType) == RTL8651_LL_LIPUNNUMBERED) ||
		(llType == RTL8651_LL_WIPUNNUMBERED))
		llType = RTL8651_LL_PPPOE;
	

	switch(llType)	{
	case RTL8651_LL_VLAN:
		/* 
		  * The Next Hop's network interface Type is ethernet, Try to make sure if the
		 * next hop belongs to a IP interface. And try to find next hop's mac
		 */
		SLIST_FOREACH(ipintf_t, &(netif_t->IpHead), nextIp)
			/* Two Cases:
			 *	a. We can make sure the next hop belongs to an existing
			 *	   network interface and we also found its MAC address.
			 *	b. We only can make sure the next hop belongs to an existing 
			 *	   network interface, but we can not found its mac address.
			 */
			if ((ipintf_t->ipAddr.slh_first->ipAddr & ipintf_t->ipMask) == 
				(nextHop & ipintf_t->ipMask)) {
				SLIST_FOREACH(arpEntry_t, &(ipintf_t->nextArp), next)
					if (arpEntry_t->ipAddr == nextHop)
						break; /* case a */
				break; /* case b */
			}
		/* Error: if no network interface was found! */
		if (!ipintf_t)
			return TBLDRV_ERTINVALNEXTHOP;
		else if (ipintf_t->ipMask == 0)
			zeroMask_ipintf = 1;
		
		break;

	case RTL8651_LL_PPTP:
	case RTL8651_LL_L2TP:
	case RTL8651_LL_PPPOE:
		/*
		  * The nexthop is a PPPoE session, the parameter "nextHop" could be a zero value.
		  * If it is a zero value, we always select the first PPPoE session as the route's nexthop.
		  * If not, the nextHop will be used as session ID to find correct PPPoE session.
		 */
		assert(netif_t->valid == 1);
		if(nextHop == 0) {
			if ((session_t=CTAILQ_FIRST(&netif_t->sessionHead)) == NULL)
			{
				printk("Get First session fail!\n");
				return TBLDRV_ENOSESSIONALLOCATE;
			}
		}
		else {
			CTAILQ_FOREACH(session_t, &netif_t->sessionHead, nextSession) {
				assert(session_t->session_t);
#if 0
				printk("Trace Session ID %d ----- nextHop %d (%u.%u.%u.%u)\n", session_t->session_t->sessionId, nextHop, NIPQUAD(nextHop));
#endif /* 0 */
				if (session_t->session_t->sessionId == nextHop)
					break;
			}
			if (!session_t)
			{
				printk("NextHop Specify fail!\n");
				return TBLDRV_ENOSESSIONALLOCATE;
			}
		}
		break;
		
	default: assert(0);
	}

/*
	{
		rtl8651_tblDrv_networkIntfTable_t *netif_t_shliu = &DrvTbl.netIface[0];
		rtl8651_tblDrv_ipIntfEntry_t *ipintf_t_shliu;
		ipaddr_t myip;

		for(; netif_t_shliu->valid == 1; netif_t_shliu++)
			SLIST_FOREACH(ipintf_t_shliu, &(netif_t_shliu->IpHead), nextIp)
			{
				myip = (ipintf_t_shliu->ipAddr.slh_first->ipAddr & ipintf_t_shliu->ipMask);
				printk("Interface route -- IP: %u.%u.%u.%u, Mask: %u.%u.%u.%u\n", NIPQUAD(myip), NIPQUAD(ipintf_t_shliu->ipMask));
			}
	}
*/		
	if (ipMask != 0) {
		/* The specified routing entry is a non-default route. */
		rt = DrvTbl.route;

		for(rtIdx=0; rtIdx<rtl8651_tblDrvPara.routingTableSize-1; rtIdx++, rt++) {
			if(rt->valid == 0)				
				break;			
			if(intfRouteFound != 0) {
				for(intfRouteFound = 0, tmp_netif_t = &DrvTbl.netIface[0]; tmp_netif_t->valid == 1; tmp_netif_t++) {		// Check whether this entry is an interface route. Once we found non-interface route, we don't check anymore. 					
					SLIST_FOREACH(tmp_ipintf_t, &(tmp_netif_t->IpHead), nextIp) {						
						tmp_ip = (tmp_ipintf_t->ipAddr.slh_first->ipAddr & tmp_ipintf_t->ipMask);
						/*
						printk("-------------------------\n");
						printk("Interface route -- IP: %u.%u.%u.%u, Mask: %u.%u.%u.%u\n", NIPQUAD(tmp_ip), NIPQUAD(tmp_ipintf_t->ipMask));
						printk("rt->ipAddr: %u.%u.%u.%u, rt->ipMask: %u.%u.%u.%u\n", NIPQUAD(rt->ipAddr), NIPQUAD(rt->ipMask));
						*/
						if(tmp_ip == rt->ipAddr && tmp_ipintf_t->ipMask == rt->ipMask) {
							//printk("Interface route -- IP: %u.%u.%u.%u, Mask: %u.%u.%u.%u\n", NIPQUAD(tmp_ip), NIPQUAD(tmp_ipintf_t->ipMask));
							intfRouteFound = 1;							
							insIdx++;							
							break;						
						}					
					}					
					if(intfRouteFound == 1)						
						break;				
				}			
			}	// End of checking a entry is an interface route.			
			if(intfRouteFound == 0 && insIdxFixed == 0) {				
				if(rt->ipMask < ipMask) {	// When this condition is satisfied, it means that this entry is the first one whose subnet mask is smaller than that of the new route.					
					insIdx = rtIdx;					
					insIdxFixed = 1;				
				}				
				else					
					insIdx++;			
			}		
		} // End of visiting the software routing table.

		/* Marked by shliu */
		/*
		for(rtIdx=0; rtIdx<rtl8651_tblDrvPara.routingTableSize-1; rtIdx++, rt++)
			if (rt->valid == 0)
				break;
		*/

		if (rtIdx == rtl8651_tblDrvPara.routingTableSize-1)
			return TBLDRV_ENOFREEBUFFER;

		for(i = rtIdx; i > insIdx; i--)			
			DrvTbl.route[i] = DrvTbl.route[i-1];

		rtEntry ++;
		
		rt = &DrvTbl.route[insIdx];
	}
	else {
		/* 
		 * The specified routing entry is a default-route:
		 * NOTE:
		 * Default route uses the last rouitng table entry and the last ASIC
		 * routing table entry.
		 */		 
		rt = &DrvTbl.route[rtl8651_tblDrvPara.routingTableSize-1];
		assert(rt->valid == 0);		
	}

	/* Add a new routing entry */
	rt->valid				= 1;
	rt->ipAddr 				= ipAddr & ipMask;
	rt->ipMask 				= ipMask;
	rt->nextHop				= nextHop;
	rt->nextHopType			= llType;
	/* (1) When adding default route...
	
	    If the ip interface of nexhop is zero mask, driver has to treat it as interface route. 
	    The reason is that any (nexthop ip & zero mask) == (865xB ip & zero mask).
	    They are like in the same interface. 
	    But we only have limited asic arp entries, method to handle ip class A/B setting is also used in this situation. 

	     (2) When adding non-default route...
	          Although the ip interface of nexthop is zero mask, driver will not treat it as interface route.
	    */
	if (zeroMask_ipintf == 1 && ipMask == 0) {
		rt->ifrt_flag 			= TRUE;
		rt->ipClass 			= 1;
	}
	else {
		rt->ifrt_flag			= FALSE;
		rt->ipClass 			= 0;
	}
	
	rt->ipintf_t			= ipintf_t;
	rt->netif_t				= netif_t;
	rt->dvid 				= netif_t->vid;
	/* According to destination vlan type to use different structure */
	if (llType == RTL8651_LL_VLAN) {
		rt->process			= RT_L2;
		rt->rt_nharp_t		= arpEntry_t;
	}
	else {
		rt->process			= RT_SESSION;
		rt->rt_nhSession_t 	= session_t->session_t;
		if (llType == RTL8651_LL_PPTP || llType == RTL8651_LL_L2TP)
			rt->dvid = tunnel.loopBackVid;
	}


	/* 
	 * If the entry can be written into ASIC, try to allocate nexthop entry. If no 
	 * nexthop entry can be allocated for it. Pending this entry in the software instead
	 * of writting it to L3 ASIC table.
	 * 
	 * If the ip interface of nexthop is zero mask and when adding default route, driver will not add a nexthop-PROCESS route.
	 */
	if ((!(zeroMask_ipintf == 1 && ipMask == 0)) && ((L3_TABLE_INDEX(rt)<(RTL8651_ROUTINGTBL_SIZE-1))||(L3_TABLE_INDEX(rt)==(rtl8651_tblDrvPara.routingTableSize-1)))) {
		uint8 process = rt->process;
		rt->process				= RT_NEXTHOP;
		rt->rt_nhalog				= RT_ALOG_SIP; /* use per-source IP */
		rt->rt_ipDomain			= RT_DOMAIN_16_1;
		rt->rt_nhnext_t			= NULL;

		if (llType == RTL8651_LL_VLAN)
			retval = _rtl8651_addNextHopEntry(NEXTHOP_L3, rt, netif_t, nextHop, 0, NULL);
		else retval = _rtl8651_addNextHopEntry(NEXTHOP_L3, rt, netif_t, 0, 0, session_t->session_t);
		if (retval == FAILED) {
			/* 
			 * It seems the l3 entry cannot get a nexthop entry. Hence
			 * pending it in the software!!
			 */
			rt->process = process;
		} 
		else rt->rt_nexthop_t = rt->rt_nhstart_t;
	}
	
	_rtl8651_updateL3Entry(rt);
	_rtl8651_updateDefaultRoute();

	if (ipMask == 0)
		_rtl8651_ipTbl_setDefNexthop(rt);

	/*
	printk("insIdx: %d, rtEntry: %d\n", insIdx, rtEntry);
	*/
	if(ipMask!=0)
		/* Write the entry into ASIC if possible. */
		for(i = insIdx + 1; i < RTL8651_ROUTINGTBL_SIZE-1 && i < rtEntry; i++)
			_rtl8651_updateL3Entry(&DrvTbl.route[i]);

	return SUCCESS;
}

#if 0
static int32 _rtl8651_delRoute(ipaddr_t ipAddr, ipaddr_t ipMask)
{
	rtl8651_tblDrv_routeTable_t *rt_t;
	ether_addr_t *mac_t=NULL;
	uint32  entry, mbrMask;
	int32 retval;

/* Debugging*/
/*
	printk("Del Route To Rome Driver : ip %u.%u.%u.%u \nMask %u.%u.%u.%u\n",
		NIPQUAD(ipAddr), NIPQUAD(ipMask));
*/
	if ((rt_t=_rtl8651_getRTentry(ipAddr, ipMask)) == NULL)
		return TBLDRV_EENTRYNOTFOUND;

	/*
	 * If the deleted routing entry is created by interface creation, forbid 
	 * the entry to be removed. This is because when an IP netwrok interface
	 * is created, its route will be added automatically.
	 */
	if (rt_t->ifrt_flag == 1)
		return TBLDRV_ERTCREATEBYIF;
	rt_t->valid 	= 0; /* Invalidate route */
	rt_t->ipAddr 	= 0;
	if (tunnel.valid == 1) 
		mac_t = &rt_t->rt_nhSession_t->macInfo->macAddr;
	if (rt_t->process == RT_NEXTHOP)
		_rtl8651_delNextHopEntry(NEXTHOP_L3, rt_t);
	if ((entry = L3_TABLE_INDEX(rt_t)) !=  rtl8651_tblDrvPara.routingTableSize-1)
		rtEntry --;
	if (entry<RTL8651_ROUTINGTBL_SIZE-1 || entry == rtl8651_tblDrvPara.routingTableSize-1) {
		if (entry == rtl8651_tblDrvPara.routingTableSize-1)
			entry = RTL8651_ROUTINGTBL_SIZE - 1;
		retval = rtl8651_delAsicRouting(entry);
		assert(retval == SUCCESS);
		_rtl8651_refillL3Entry(rt_t);
	}

	/* Check if route references alive PPTP/L2TP */
	if (tunnel.valid == 1) {
		rt_t = DrvTbl.route;
		for(entry=0; entry<rtl8651_tblDrvPara.routingTableSize; entry++, rt_t++) {
			if (rt_t->valid == 1 && rt_t->ifrt_flag == 0 && 
			   (rt_t->nextHopType==RTL8651_LL_PPTP || rt_t->nextHopType==RTL8651_LL_L2TP))
			   break;
		}
		if (entry == rtl8651_tblDrvPara.routingTableSize) {
			mbrMask = (1 << tunnel.wanPort);
			_rtl8651_modifyVlanRefFilterDatabaseEntry(0, mac_t , &mbrMask, RTL8651_FORWARD_MAC, NULL, L2_FORCE_STATIC);
		}
	}
	
	if (ipMask == 0)
		_rtl8651_ipTbl_setDefNexthop(NULL);
	_rtl8651_updateDefaultRoute();

	return SUCCESS;
}
#else
static int32 _rtl8651_delRoute(ipaddr_t ipAddr, ipaddr_t ipMask)
{
	rtl8651_tblDrv_routeTable_t *rt_t;
	ether_addr_t *mac_t=NULL;
	uint32  entry, mbrMask;
	int32 retval, i;

	if ((rt_t=_rtl8651_getRTentry(ipAddr, ipMask)) == NULL)
		return TBLDRV_EENTRYNOTFOUND;

	/*
	 * If the deleted routing entry is created by interface creation, forbid 
	 * the entry to be removed. This is because when an IP netwrok interface
	 * is created, its route will be added automatically.
	 */
	if (rt_t->ifrt_flag == 1)
		return TBLDRV_ERTCREATEBYIF;
	rt_t->valid 	= 0; /* Invalidate route */
	rt_t->ipAddr 	= 0;

 	if (tunnel.valid == 1)
 	{
 		/*
 			When tunnel is valid (ie. tunneling link-layer type network interface is configured, ex: PPTP/L2TP), 
 			we need to check that the route to be deleted has session information.

 			If the nexthop of the route belongs to session type,
 			we need to retrieve L2 entry of the nexthop. For what ?
 			Because when no route's nexthop belongs to tunneling link-layer type network interface,
 			we need to re-cover the member port of that nexthop.
 		*/
 		if (rt_t->rt_nhSession_t)
			mac_t = &rt_t->rt_nhSession_t->macInfo->macAddr;
 	}

	if (rt_t->process == RT_NEXTHOP)
		_rtl8651_delNextHopEntry(NEXTHOP_L3, rt_t);
	if ((entry = L3_TABLE_INDEX(rt_t)) !=  rtl8651_tblDrvPara.routingTableSize-1)
		rtEntry --;

	/* If the routing entry to be deleted is also in ASIC L3 table, we need to delete the corresponding ASIC entry. */
	if ( entry < RTL8651_ROUTINGTBL_SIZE - 1 )
	{
		retval = rtl8651_delAsicRouting(entry);
		assert(retval == SUCCESS);
	}

	/* Entries after deleted route should be shifted to (original index - 1). */
	for ( i = entry + 1; i <= rtEntry; i++) 
	{
		DrvTbl.route[i-1] = DrvTbl.route[i];
		bzero(&DrvTbl.route[i], sizeof(rtl8651_tblDrv_routeTable_t));

		/* some entries need to be written into ASIC L3 table */
		if ( ( i - 1 ) < RTL8651_ROUTINGTBL_SIZE - 1 )
		{
			_rtl8651_updateL3Entry( &DrvTbl.route[ i - 1 ] );
		}

		if ( i < RTL8651_ROUTINGTBL_SIZE - 1 )
		{
			retval = rtl8651_delAsicRouting(i);
			assert(retval == SUCCESS);
		}
	}

	/*
		Refill mechanism:

		If (1) number of routing entries exceeds ( size of L3 table - 1 ) (2) the deleted route is also in ASIC L3 table (except default route),
		we need to choose a software routing entry to be written into ASIC.

		=> Note: the ASIC entry number to be refilled is always "RTL8651_ROUTINGTBL_SIZE - 2".
	*/
	if ( rtEntry > RTL8651_ROUTINGTBL_SIZE - 1 && entry < RTL8651_ROUTINGTBL_SIZE - 1 )
		_rtl8651_refillL3Entry( &DrvTbl.route[RTL8651_ROUTINGTBL_SIZE - 2] );

	/* case of default route deletion */
	if ( entry == rtl8651_tblDrvPara.routingTableSize - 1 )
	{
		if (entry == rtl8651_tblDrvPara.routingTableSize-1)
			entry = RTL8651_ROUTINGTBL_SIZE - 1;
		retval = rtl8651_delAsicRouting(entry);
		assert(retval == SUCCESS);
	}

	/* Check if route references alive PPTP/L2TP */
	if (tunnel.valid == 1) 
	{
		rt_t = DrvTbl.route;

		/* 
			Traverse the routing table. 
			If no routes refer to alive PPTP/L2TP, we need to recover the member port of the nexthop of PPTP/L2TP session. 
		*/
		for(entry = 0; entry < rtl8651_tblDrvPara.routingTableSize; entry++, rt_t++) 
		{
			if (rt_t->valid == 1 && rt_t->ifrt_flag == 0 && 
			   (rt_t->nextHopType == RTL8651_LL_PPTP || rt_t->nextHopType == RTL8651_LL_L2TP) )
			   break;
		}
		if (entry == rtl8651_tblDrvPara.routingTableSize) 
		{
			mbrMask = (1 << tunnel.wanPort);

			/* Check if we have information about the nexthop of PPTP/L2TP session. */
			if (mac_t)
				_rtl8651_modifyVlanRefFilterDatabaseEntry(0, mac_t , &mbrMask, RTL8651_FORWARD_MAC, NULL, L2_FORCE_STATIC);
		}
	}
	
	if (ipMask == 0)
		_rtl8651_ipTbl_setDefNexthop(NULL);
	_rtl8651_updateDefaultRoute();

	return SUCCESS;
}

#endif

/*
Note:
1. Reserve the last entry for default route purpose due to ASIC treate default route differently.
*/
/*
@func int32 | rtl8651_addRoute | Add a routing entry.
@parm ipaddr_t | ipAddr | Destination IP address or network address.
@parm ipaddr_t | ipMask | Network mask.
@parm int8 * | ifName | Network interface name.
@parm ipaddr_t | nextHop | Next hop IP address. Meaningful in Broadcast network.
@rvalue SUCCESS | A routing entry was successfully added.
@rvalue TBLDRV_EINVNETIFNAME | Invalid Network interface name.
@rvalue TBLDRV_ENOLLTYPESPECIFY | The network interface has no link layer type specified.
@rvalue TBLDRV_ERTALREADYEXIST | The routing entry already exists.
@rvalue TBLDRV_ERTINVALNEXTHOP | Invalid next hop.
@rvalue TBLDRV_ENOSESSIONALLOCATE | Need PPPoE session allocated.
@rvalue TBLDRV_ENOFREEBUFFER | Out of memory buffer.
@comm
The <p rtl8651_addRoute()> adds a routing entry to the routing table. If a route is to a PPPoE, the <p nextHop> should be zero.
*/
int32 rtl8651_addRoute(ipaddr_t ipAddr, ipaddr_t ipMask, int8 * ifName, ipaddr_t nextHop) {
	int32 retval ;
	rtlglue_drvMutexLock();
	retval = _rtl8651_addRoute(ipAddr, ipMask, ifName, nextHop) ;
	rtlglue_drvMutexUnlock();
	return retval;	
}

/*
@func int32 | rtl8651_delRoute | Remove a routing entry.
@parm ipaddr_t | ipAddr | The Destination IP address or network address.
@parm ipaddr_t | ipMask | Network mask.
@rvalue SUCCESS | A routing entry was successfully removed from routing table.
@rvalue TBLDRV_EENTRYNOTFOUND | The specified routing entry was not found.
@rvalue TBLDRV_ERTCREATEBYIF | The specified routing entry was added due to network interface created, hence it can't be removed.
@comm
Whenever a netwrok interface is created, an interface route will be automatically added. This kind of route
can not be removed.
*/
int32 rtl8651_delRoute(ipaddr_t ipAddr, ipaddr_t ipMask) {
	int32 retval;
	rtlglue_drvMutexLock();
	retval = _rtl8651_delRoute(ipAddr, ipMask);
	rtlglue_drvMutexUnlock();
	return retval;
}





/*=================================================
  * Policy Route
  *=================================================*/
#define RTL8651_ROUTING_TABLE_APIs

int hashcode(int value)
{
	value ^= (value>>16);
	value ^= (value>>8);
	value^= (value>>3);
	value = value&(DYNAMICPOLICYSIZE-1);
	return value;
}
uint32 _rtl8651_SessionIDToDSID(uint32 SessionID)
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrvSession_t *pppoes_t = NULL;
	uint32 i;
	netif_t = &DrvTbl.netIface[0];
	for(i=0; i<rtl8651_tblDrvPara.networkIntfTableSize; i++, netif_t++)
		if (netif_t->valid == 1) {
			CTAILQ_FOREACH(pppoes_t, &netif_t->sessionHead, nextSession) {
				assert(pppoes_t->session_t);
				if (pppoes_t->session_t->sessionId == SessionID)
					{
					return pppoes_t->session_t->dsid;
					}
			}
		}
	return 0;
}

rtl8651_tblDrv_pppTable_t *
_rtl8651_getPppoeSessionByDsid(uint32 dsid) 
{
	rtl8651_tblDrv_pppTable_t *entry_t = DrvTbl.pppoe;
	uint32 entry;

	for(entry=0; entry<RTL8651_PPPOETBL_SIZE; entry++, entry_t++) {
		if (entry_t->valid && entry_t->dsid == dsid)
			return entry_t;
	}
	return (rtl8651_tblDrv_pppTable_t *)NULL;
}

int32 rtl8651_setPppoeDefaultSessionId(uint32 SessionID)
{
	uint32 dsid=_rtl8651_SessionIDToDSID(SessionID);
	_rtl8651_DefaultDialSessionId=dsid;
	return SUCCESS;
}

int _rtl8651_getpppDSID(uint32 ip_alias)
{

		rtl8651_tblDrv_pppTable_t *poe_tb=NULL;
		int id=0;
		poe_tb = _rtl8651_getPppoeSessionByIpAddr(ip_alias);
		if (poe_tb)
			id= poe_tb->dsid;
		return id;
	
}
int  _rtl8651_getPolicyRoutingDSID(struct rtl_pktHdr *pktHdr,uint32 ip_d,uint32 ip_s,uint8 proto,int ip_dp)
{
	int type,hash;
	_rtl8651_tblDrvPolicyRoute_t *rtEntry=&rtStaticPolicy[0];	
	switch(proto)
	{
		case IPPROTO_TCP:
			type=TRIGGER_DSTTCPPORT;
			break;
		case IPPROTO_UDP:
			type=TRIGGER_DSTUDPPORT;
			break;
		default:
			type=0;
		
	}
	for (;rtEntry->type!=0;rtEntry++)
	{
		switch (rtEntry->type)
		{
			case TRIGGER_DSTIP:
				if ((rtEntry->ip_start<=ip_d)&&
					(rtEntry->ip_end>=ip_d))
					goto match;
				break;
			case TRIGGER_SRCIP:
				if ((rtEntry->ip_start<=ip_s)&&
					(rtEntry->ip_end>=ip_s))
					goto match;
				break;
			case TRIGGER_DSTTCPPORT:
			case TRIGGER_DSTUDPPORT:
				if ((rtEntry->trigger.dport.start<=ip_dp)&&
					(rtEntry->trigger.dport.end>=ip_dp)&&
					(rtEntry->type==type))
					goto match;
				break;
			case TRIGGER_SRCGLOBALIP:
				if ((rtEntry->ip_start<=ip_s)&&
					(rtEntry->ip_end>=ip_s))

					{
						pktHdr->ph_unnumber=1;
						goto match;
					}
				break;

				
		}
	}	

	hash = hashcode(ip_d);
	rtEntry = rtHashTable[hash];

	while (rtEntry)
	{
		if ( rtEntry->age > 0 )
		{
			switch ( rtEntry->type )
			{
				/* dynamic entry only support DIP and SIPDIPPAIR yet */
				case TRIGGER_DSTIP:
					if ( rtEntry->trigger.ip_d == ip_d )
					{
						goto match;
					}
					break;
				case TRIGGER_SIPDIPPAIR:
					if (	(rtEntry->trigger.sipDipPair.sip == ip_s) &&
						(rtEntry->trigger.sipDipPair.dip == ip_d) )
					{
						goto match;
					}
					break;
			}
		}
		rtEntry=rtEntry->linknext;
	}
	DLOG(1,"fail  softrouting\n");
	
	return _rtl8651_DefaultDialSessionId;
match:	
	
	return _rtl8651_getpppDSID(rtEntry->ip_alias);

}

static void _rtl8651_dynamicPolicyRouteTimeupdate(uint32 secpassed)
{
	_rtl8651_tblDrvPolicyRoute_t *ptr;

	ptr = rtDynamicPolicyHeadPtr;

	while ( ptr )
	{
		if ( ptr->age > secpassed )
		{
			ptr->age -= secpassed;
		} else
		{
			if ( ptr->age > 0 )
			{
				ipaddr_t dip = 0;

				dip = ((ptr->type & ~TYPE_BITS) == TRIGGER_DSTIP)?ptr->trigger.ip_d:((ptr->type & ~TYPE_BITS) == TRIGGER_SIPDIPPAIR)?ptr->trigger.sipDipPair.dip:0;

				RTL_DEBUG(	RTL_MSG_POLICYRT,
							"Dynamic routing entry:\n\tType %s\n\tSIP %u.%u.%u.%u\n\tDIP %u.%u.%u.%u\n\tAlias IP %u.%u.%u.%u\n==> Age %d arrive, Remove.",
							((ptr->type & ~TYPE_BITS) == TRIGGER_DSTIP)?"DIP trigger":((ptr->type & ~TYPE_BITS) == TRIGGER_SIPDIPPAIR)?"SIP/DIP Pair":"Others",
							NIPQUAD( ptr->trigger.sipDipPair.sip),
							NIPQUAD( ptr->trigger.sipDipPair.dip ),
							NIPQUAD( ptr->ip_alias ),
							ptr->fullAge);
			}

			ptr->age = 0;
		}

		/* find Next, if we have check back to rtDynamicPolicyHeadPtr, stop it */
		ptr = ptr->timenext;
		if ( ptr == rtDynamicPolicyHeadPtr )
		{
			ptr = NULL;
		}
	}
}

/* 
@ _rtl8651_getSoftRoutingEntry
@ no default route only some special route
@
*/
rtl8651_tblDrv_routeTable_t * _rtl8651_getSoftRoutingEntry(uint32 ip_s, uint32 ip_d,uint8 proto,int ip_dp)
{
	int type,hash;
	_rtl8651_tblDrvPolicyRoute_t *rtEntry=&rtStaticPolicy[0];	
	switch(proto)
	{
		case IPPROTO_TCP:
			type=TRIGGER_DSTTCPPORT;
			break;
		case IPPROTO_UDP:
			type=TRIGGER_DSTUDPPORT;
			break;
		default:
			type=0;
		
	}
	for (;rtEntry->type!=0;rtEntry++)
	{
		switch (rtEntry->type)
		{
			case TRIGGER_DSTIP:
				if ((rtEntry->ip_start<=ip_d)&&
					(rtEntry->ip_end>=ip_d))
					goto match;
				break;
			case TRIGGER_SRCIP:
				if ((rtEntry->ip_start<=ip_s)&&
					(rtEntry->ip_end>=ip_s))
					goto match;
				break;
			case TRIGGER_DSTTCPPORT:
			case TRIGGER_DSTUDPPORT:
				if ((rtEntry->trigger.dport.start<=ip_dp)&&
					(rtEntry->trigger.dport.end>=ip_dp)&&
					(rtEntry->type==type))
					goto match;
				break;
			case TRIGGER_SRCGLOBALIP:
				if ((rtEntry->ip_start<=ip_s)&&
					(rtEntry->ip_end>=ip_s))
					goto match;
				break;

		}
	}	

	hash = hashcode(ip_d);
	rtEntry = rtHashTable[hash];

	while (rtEntry)
	{
		if ( rtEntry->age > 0 )
		{
			switch ( rtEntry->type )
			{
				/* dynamic entry only support DIP and SIPDIPPAIR yet */
				case TRIGGER_DSTIP:
					if ( rtEntry->trigger.ip_d == ip_d )
					{
						goto match;
					}
					break;
				case TRIGGER_SIPDIPPAIR:
					if (	(rtEntry->trigger.sipDipPair.sip == ip_s) &&
						(rtEntry->trigger.sipDipPair.dip == ip_d) )
					{
						goto match;
					}
					break;
			}
		}

		rtEntry=rtEntry->linknext;
	}
	DLOG(1,"fail  softrouting\n");
	return 0;
match:
	DLOG(1,"match softrouting\n");
	return ( rtl8651_tblDrv_routeTable_t *)rtEntry->rt;

}




/*
@func int32 | rtl8651_flushPolicyRoute | remove policy route
@parm int	| ip	| IP Address
@rvalue SUCCESS | Always return SUCCESS
@comm
Delete all routing entry by external ip address
*/
int32 rtl8651_flushPolicyRoute(uint32 ip)
{
	int rtIdx,rtNewIdx,hash;
	_rtl8651_tblDrvPolicyRoute_t *rtEntry,*rt1,*rt2;
	DLOG(2," flush ip %x\n",ip);
	if (!ip) return 0;
	rtlglue_drvMutexLock();
	rtEntry=&rtStaticPolicy[0];	
	for(rtIdx=0; rtIdx<STATICPOLICYSIZE; rtIdx++,rtEntry++)
	{
		if (rtEntry->ip_alias==ip)
			rtEntry->type=0;
	}
	for(rtIdx=0; rtIdx<STATICPOLICYSIZE; rtIdx++)
	{
		if (rtStaticPolicy[rtIdx].type==0)
		{
			for(rtNewIdx=rtIdx+1; rtNewIdx<STATICPOLICYSIZE; rtNewIdx++)
				if (rtStaticPolicy[rtNewIdx].type!=0)
				{
					rtStaticPolicy[rtIdx]=rtStaticPolicy[rtNewIdx];
					rtStaticPolicy[rtNewIdx].type=0;
					break;
				}
			
		}
	}
	rtEntry=rtDynamicPolicyHeadPtr;
	for (rtIdx=0;rtIdx<DYNAMICPOLICYSIZE;rtIdx++,rtEntry=rtEntry->timenext)
	{
		if (rtEntry->ip_alias!=ip)
			continue;

		switch ( rtEntry->type & ~TYPE_BITS )
		{
			case TRIGGER_DSTIP:
				hash = hashcode (rtEntry->trigger.ip_d);
				break;
			case TRIGGER_SIPDIPPAIR:
				hash = hashcode (rtEntry->trigger.sipDipPair.dip);
				break;
			default:
				hash = 0;
				RTL_BUG ("Unsupport Dynamic entry.");
				break;
		}

		rt1 = rtHashTable[hash];

		if (rt1 == rtEntry)
		{
			rtHashTable[hash]=rt1->linknext;	
			rt1->linkprev=0;
		}
		else
		{
			rt1=rtEntry->linkprev;
			rt2=rtEntry->linknext;
			rt1->linknext=rtEntry->linknext;	
			if (rt2)
				rt2->linkprev=rt1;
			
		}
		rtEntry->linkprev=0;
		rtEntry->linknext=0;
		memset(&(rtEntry->trigger), 0, sizeof(rtEntry->trigger));
		rtEntry->ip_alias=0;
		rtEntry->fullAge = 0;
		rtEntry->age = 0;
	}		
	rtlglue_drvMutexUnlock();//Unlock resource
	return 0;
}
/*
@func int32 | rtl8651_addPolicyRoute | add policy route entry
@parm	rtl8651_tblDrvPolicyRoute_t * | rtAdd | policy route entry
@rvalue SUCCESS | The Entry is added successfully.
@rvalue RTENTRY_FOUND | The Entry is already existed.
@rvalue RTENTRY_FULL | The Table is full.
@comm
Add A Policy Route Entry
*/
int32 rtl8651_addPolicyRoute(rtl8651_tblDrvPolicyRoute_t *rtAdd)
{
	int32 retval = FAILED;

	UNLOCK_ERROR_CHECK(rtAdd == NULL, TBLDRV_EINVALIDINPUT);

	rtlglue_drvMutexLock();
	retval = _rtl8651_addPolicyRoute( rtAdd, RTL8651_DPOLICYRT_EXT_AGE );
	rtlglue_drvMutexUnlock();

	return retval;
}
int32 _rtl8651_addPolicyRoute(rtl8651_tblDrvPolicyRoute_t *rtAdd, uint32 dynamicPolicyRtAge)
{
	int rtIdx;
	int hash,hash_old;
	rtl8651_tblDrv_routeTable_t  *rt;
	_rtl8651_tblDrvPolicyRoute_t *rtEntry=&rtStaticPolicy[0];	
	_rtl8651_tblDrvPolicyRoute_t *rtTemp;	
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrvSession_t *session_t=NULL;

	if ((rtAdd->type&DYNAMIC_POLICY_ROUTE))
	{
		goto labelDynamicAdd;
	}

	/* ------------------------------
			Add static policy route
	     ------------------------------*/

	for(rtIdx=0; rtIdx<STATICPOLICYSIZE; rtIdx++, rtEntry++)
	{
		if (rtEntry->type != (rtAdd->type&(~TYPE_BITS)))
			continue;
		switch(rtEntry->type)
		{
			case TRIGGER_DSTIP:
			case TRIGGER_SRCIP:
			case TRIGGER_SRCGLOBALIP:
				if (rtEntry->ip_start==rtAdd->ip_start  && rtEntry->ip_end==rtAdd->ip_end)
					rtEntry->ip_alias=rtAdd->ip_alias;
				else
					break;
				return RTENTRY_FOUND;

			case TRIGGER_DSTTCPPORT:
			case TRIGGER_DSTUDPPORT:
				if (rtEntry->trigger.ip_d==rtAdd->trigger.ip_d)
					rtEntry->ip_alias=rtAdd->ip_alias;
				else
					break;
				return RTENTRY_FOUND;
		}

	}

	rtEntry=&rtStaticPolicy[0];	
	for(rtIdx=0; rtIdx<STATICPOLICYSIZE; rtIdx++, rtEntry++)
		if (rtEntry->type == 0)
			break;
	if (rtIdx==STATICPOLICYSIZE) 
	{
		return RTENTRY_FULL;
	}
	else
		goto labelStaticAdd;

	/* ------------------------------
			Add dynamic policy route
	     ------------------------------*/

labelDynamicAdd:

	/* age timer CAN NOT be 0 */
	UNLOCK_ERROR_CHECK(dynamicPolicyRtAge == 0, TBLDRV_EINVALIDINPUT);

	/* check type of added Dyanmic Entry */
	switch ( rtAdd->type & ~TYPE_BITS )
	{
		case TRIGGER_DSTIP:
			hash = hashcode (rtAdd->trigger.ip_d);
			break;
		case TRIGGER_SIPDIPPAIR:
			hash = hashcode (rtAdd->trigger.sipDipPair.dip);
			break;
		default:
			RTL_WARN(	RTL_MSG_POLICYRT,
						"The dynamic policy entry does not support the trigger type [%d]",
						(rtAdd->type & ~TYPE_BITS) );
			return TBLDRV_EINVALIDINPUT;
	}

	/* check if this entry exist or not */
	rtEntry = rtHashTable[hash];
	while ( rtEntry )
	{
		if ( rtEntry->age > 0 )
		{
			int32 isMatch = FALSE;

			switch ( rtEntry->type )
			{
				/* dynamic entry only support DIP and SIPDIPPAIR yet */
				case TRIGGER_DSTIP:
					if ( rtEntry->trigger.ip_d == rtAdd->trigger.ip_d )
					{
						isMatch = TRUE;
					}
					break;
				case TRIGGER_SIPDIPPAIR:
					if (	(rtEntry->trigger.sipDipPair.sip == rtAdd->trigger.sipDipPair.sip) &&
						(rtEntry->trigger.sipDipPair.dip == rtAdd->trigger.sipDipPair.dip) )
					{
						isMatch = TRUE;
					}
					break;
				default:
					RTL_BUG("Un-supported dynamic policy route type.");
			}

			if (isMatch == TRUE)
			{
				/* update / refresh exist entry */
				rtEntry->ip_alias = rtAdd->ip_alias;
				rtEntry->age = rtEntry->fullAge = dynamicPolicyRtAge;

				{
					ipaddr_t dip = 0;

					dip = ((rtAdd->type & ~TYPE_BITS) == TRIGGER_DSTIP)?rtAdd->trigger.ip_d:((rtAdd->type & ~TYPE_BITS) == TRIGGER_SIPDIPPAIR)?rtAdd->trigger.sipDipPair.dip:0;

					RTL_DEBUG(	RTL_MSG_POLICYRT,
								"Dynamic routing entry:\n\tType %s\n\tSIP %u.%u.%u.%u\n\tDIP %u.%u.%u.%u\n\tAlias IP %u.%u.%u.%u\n==> Refreah Age to %d.",
								((rtAdd->type & ~TYPE_BITS) == TRIGGER_DSTIP)?"DIP trigger":((rtAdd->type & ~TYPE_BITS) == TRIGGER_SIPDIPPAIR)?"SIP/DIP Pair":"Others",
								NIPQUAD( rtAdd->trigger.sipDipPair.sip ),
								NIPQUAD( rtAdd->trigger.sipDipPair.dip ),
								NIPQUAD( rtAdd->ip_alias ),
								dynamicPolicyRtAge
								);
				}

				return RTENTRY_FOUND;
			}
		}
		rtEntry = rtEntry->linknext;
	}

	/* no same entry found, try to add a new one */

	/*
		We get entry following the rule:

		1. we get entry with age being 0
		2. we use LRU to get the Least-Recently used entry
	*/
	{
		_rtl8651_tblDrvPolicyRoute_t *ptr, *lru = NULL;

		ptr = rtDynamicPolicyHeadPtr;

		while ( ptr )
		{
			if ( ptr->age == 0 )
			{
				break;
			} else
			{
				if (	( lru == NULL ) ||
					( (lru->fullAge-lru->age) < (ptr->fullAge-ptr->age) ) )
				{
					lru = ptr;
				}
			}
			/* find Next, if we have check back to rtDynamicPolicyHeadPtr, stop it */
			ptr = ptr->timenext;
			if ( ptr == rtDynamicPolicyHeadPtr )
			{
				ptr = NULL;
			}
		}

		if ( ptr )			/* age-out entry found */
		{
			rtEntry = ptr;
		} else if ( lru )	/* LRU entry found */
		{
			rtEntry = lru;
		} else
		{
			RTL_BUG("No any Dynamic Policy Route Entry!");
			return TBLDRV_ENOFREEBUFFER;
		}
	}

	rtTemp = rtHashTable[hash];

	if (	(rtEntry->linkprev == NULL) &&
		(rtEntry->fullAge /* means this entry is not at the initial state */))
	{
		switch ( rtEntry->type & ~TYPE_BITS )
		{
			case TRIGGER_DSTIP:
				hash_old = hashcode (rtEntry->trigger.ip_d);
				break;
			case TRIGGER_SIPDIPPAIR:
				hash_old = hashcode (rtEntry->trigger.sipDipPair.dip);
				break;
			default:
				hash_old = 0;
				RTL_BUG("un-supported dynamic routing type.");
		}

		if (rtHashTable[hash_old] == rtEntry)
		{
			rtHashTable[hash_old] = NULL;
		}
	
	}

	/* Add this newly added entry into policy route Hash Table */
	if (rtTemp != rtEntry)
	{
		if (rtEntry->linkprev)
			rtEntry->linkprev->linknext=rtEntry->linknext;
		if (rtEntry->linknext)
			rtEntry->linknext->linkprev=rtEntry->linkprev;
	
		rtEntry->linknext=rtTemp;
		if (rtTemp)
			rtTemp->linkprev = rtEntry;
		rtEntry->linkprev = 0;
	}
	rtEntry->age = rtEntry->fullAge = dynamicPolicyRtAge;
	rtHashTable[hash] = rtEntry;

	{
		ipaddr_t dip = 0;

		dip = ((rtAdd->type & ~TYPE_BITS) == TRIGGER_DSTIP)?rtAdd->trigger.ip_d:((rtAdd->type & ~TYPE_BITS) == TRIGGER_SIPDIPPAIR)?rtAdd->trigger.sipDipPair.dip:0;

		RTL_DEBUG(	RTL_MSG_POLICYRT,
					"Dynamic routing entry:\n\tType %s\n\tSIP %u.%u.%u.%u\n\tDIP %u.%u.%u.%u\n\tAlias IP %u.%u.%u.%u\n==> Add.",
					((rtAdd->type & ~TYPE_BITS) == TRIGGER_DSTIP)?"DIP trigger":((rtAdd->type & ~TYPE_BITS) == TRIGGER_SIPDIPPAIR)?"SIP/DIP Pair":"Others",
					NIPQUAD( rtAdd->trigger.sipDipPair.sip),
					NIPQUAD( rtAdd->trigger.sipDipPair.dip ),
					NIPQUAD( rtAdd->ip_alias )
					);
	}
	/*
		Do additional process and set related field into policy route entry.
	*/
labelStaticAdd:

	{
		int32 ifIndex = 0;
		netif_t =DrvTbl.netIface;
		for( ; ifIndex < rtl8651_tblDrvPara.networkIntfTableSize; ifIndex++, netif_t++){
			if (netif_t->valid == 0)
				continue;
			CTAILQ_FOREACH(session_t, &netif_t->sessionHead, nextSession) {
				if (session_t->ip_t && session_t->ip_t->ipAddr == rtAdd->ip_alias)
					goto found_session;
			}
		}
	}
found_session:
	if (session_t == NULL) {
		return RTENTRY_FOUND;
	} 
	
	if (rtEntry->type&TRIGGER_SRCGLOBALIP) {
		/* Try to add a route for the second session */
		_rtl8651_addRoute(rtAdd->ip_start, 0xffffffff, netif_t->ifName, session_t->session_t->sessionId);
		rt=_rtl8651_getRoutingEntry(rtAdd->ip_start);
	}
	else	{
		/* Try to add a route for the second session */
		_rtl8651_addRoute(rtAdd->ip_alias, 0xffffffff, netif_t->ifName, session_t->session_t->sessionId);
		rt=_rtl8651_getRoutingEntry(rtAdd->ip_alias);				
	}
	rtEntry->rt=(void*)rt;
	rtEntry->type=(rtAdd->type&(~TYPE_BITS));
	memcpy(&(rtEntry->trigger), &(rtAdd->trigger), sizeof(rtAdd->trigger));
	rtEntry->ip_alias=rtAdd->ip_alias;
	rtEntry->ip_start=rtAdd->ip_start;
	rtEntry->ip_end=rtAdd->ip_end;

	RTL_DEBUG(RTL_MSG_POLICYRT, "Policy Route Entry:");
	RTL_DEBUG(RTL_MSG_POLICYRT, "\tType 0x%x", rtEntry->type);
	RTL_DEBUG(RTL_MSG_POLICYRT, "\tIP start %u.%u.%u.%u", NIPQUAD( rtEntry->ip_start ));
	RTL_DEBUG(RTL_MSG_POLICYRT, "\tIP end %u.%u.%u.%u", NIPQUAD( rtEntry->ip_end ));
	RTL_DEBUG(RTL_MSG_POLICYRT, "\tIP Alias %u.%u.%u.%u\n", NIPQUAD( rtEntry->ip_alias ));
	RTL_DEBUG(RTL_MSG_POLICYRT, "\t\tTrigger SIP %u.%u.%u.%u", NIPQUAD( rtEntry->trigger.ip_s ));
	RTL_DEBUG(RTL_MSG_POLICYRT, "\t\tTrigger DIP %u.%u.%u.%u", NIPQUAD( rtEntry->trigger.ip_d ));
	RTL_DEBUG(RTL_MSG_POLICYRT, "\t\tTrigger DPort %d ~ %d", rtEntry->trigger.dport.start, rtEntry->trigger.dport.end );
	RTL_DEBUG(RTL_MSG_POLICYRT, "\t\tTrigger Pair SIP %u.%u.%u.%u", NIPQUAD( rtEntry->trigger.sipDipPair.sip ));
	RTL_DEBUG(RTL_MSG_POLICYRT, "\t\tTrigger Pair DIP %u.%u.%u.%u", NIPQUAD( rtEntry->trigger.sipDipPair.dip ));
	RTL_DEBUG(RTL_MSG_POLICYRT, "\tRouting Entry 0x%p", rtEntry->rt);
	RTL_DEBUG(RTL_MSG_POLICYRT, "\tDynamic Entry CurrentAge/FullAge time %d / %d", rtEntry->age, rtEntry->fullAge );

	return SUCCESS;
}


int32 _rtl8651_softNaptPolicyRoute(int type, ipaddr_t ip_s,ipaddr_t ip_d,uint16  ip_dp,
				int *ip_alias, uint16 *ip_aliasp)
{
	int hash;
	_rtl8651_tblDrvPolicyRoute_t *rtEntry=&rtStaticPolicy[0];	
	_rtl8651_tblDrvPolicyRoute_t *rtTemp,*rtPPrev;	
	for (;rtEntry->type!=0;rtEntry++)
	{
		switch (rtEntry->type)
		{
			case TRIGGER_SRCIP:
				if ((rtEntry->ip_start<=ip_s)&&
					(rtEntry->ip_end>=ip_s))
					goto match;
				break;
			case TRIGGER_DSTIP:
				if ((rtEntry->ip_start<=ip_d)&&
					(rtEntry->ip_end>=ip_d))
					goto match;
				break;
			case TRIGGER_DSTTCPPORT:
			case TRIGGER_DSTUDPPORT:
				if ((rtEntry->trigger.dport.start<=ip_dp)&&
					(rtEntry->trigger.dport.end>=ip_dp)&&
					(rtEntry->type==type))
					goto match;
				break;
		}

	}	

	hash = hashcode(ip_d);
	rtEntry = rtHashTable[hash];
	while (rtEntry)
	{
		if ( rtEntry->age > 0 )
		{
			int8 isMatch = FALSE;
			switch ( rtEntry->type & ~TYPE_BITS )
			{
				case TRIGGER_DSTIP:
					if ( rtEntry->trigger.ip_d == ip_d )
					{
						isMatch = TRUE;
					}
					break;
				case TRIGGER_SIPDIPPAIR:
					if (	( rtEntry->trigger.sipDipPair.sip == ip_s ) &&
						( rtEntry->trigger.sipDipPair.dip == ip_d ))
					{
						isMatch = TRUE;
					}
					break;
			}

			if ( isMatch == TRUE )
			{
				/* refresh Age-timer */
				rtEntry->age = rtEntry->fullAge;

				if (rtDynamicPolicyHeadPtr->timeprev!=rtEntry)
				{
					if (rtDynamicPolicyHeadPtr==rtEntry)
					{
						rtDynamicPolicyHeadPtr=rtEntry;
						goto match;
					}

					rtPPrev=rtDynamicPolicyHeadPtr->timeprev;
					rtTemp=rtEntry->timenext;
					rtTemp->timeprev=rtEntry->timeprev;
					
					rtTemp=rtEntry->timeprev;
					rtTemp->timenext=rtEntry->timenext;
					
					rtDynamicPolicyHeadPtr->timeprev=rtEntry;
					rtEntry->timenext=rtDynamicPolicyHeadPtr;
					rtEntry->timeprev=rtPPrev;
					rtPPrev->timenext=rtEntry;
				}

				goto match;
			}
		}
		rtEntry=rtEntry->linknext;
	}
	*(int*)ip_alias=0;
	return 0;
match:
	*ip_alias=rtEntry->ip_alias;
	*ip_aliasp=global_port++;
	return 1;
}
#if 0 // chenyl
int32 _rtl8651_getSoftIcmpNaptEntry(rtl8651_conntrack_info *info,int *ip,uint16 *id)
{

	_rtl8651_tblDrvSoftIcmpTable_t *softIcmp;
	int idx;
	softIcmp = &softIcmpNaptTable[0];
	for (idx=0;idx<SOFTICMPTABLE&&softIcmp->age>=1;softIcmp++)
	{
		idx++;
		if (softIcmp->globalId==info->icmpid &&
		     softIcmp->globalIp==info->ip_d)
		{
			*ip=softIcmp->internalIp;
			*id=softIcmp->internalId;
			return 1;
		}
	}
	return 0;

}
#endif
int32 _rtl8651_softIcmpPolicyRoute(ipaddr_t sip, uint16 sID, ipaddr_t dip, ipaddr_t *gip, uint16 *gID)
{
	int hash;
	_rtl8651_tblDrvPolicyRoute_t *rtEntry = &rtStaticPolicy[0];
	_rtl8651_tblDrvPolicyRoute_t *rtTemp,*rtPPrev;


	for (;rtEntry->type!=0;rtEntry ++)
	{
		switch (rtEntry->type)
		{
			case TRIGGER_SRCIP:
				if ((rtEntry->ip_start <= sip)&&(rtEntry->ip_end >= sip))
					goto match;
				break;
			case TRIGGER_DSTIP:
				if ((rtEntry->ip_start <= dip)&&(rtEntry->ip_end >= dip))
					goto match;
				break;
		}
	}
	hash = hashcode(dip);
	rtEntry = rtHashTable[hash];
	while (rtEntry)
	{
		if ( rtEntry->age > 0 )
		{
			int8 isMatch = FALSE;

			switch ( rtEntry->type & ~TYPE_BITS )
			{
				case TRIGGER_DSTIP:
					if ( rtEntry->trigger.ip_d == dip )
					{
						isMatch = TRUE;
					}
					break;
				case TRIGGER_SIPDIPPAIR:
					if (	( rtEntry->trigger.sipDipPair.sip == sip ) &&
						( rtEntry->trigger.sipDipPair.dip == dip ))
					{
						isMatch = TRUE;
					}
					break;
			}

			if ( isMatch == TRUE )
			{
				/* refresh Age-timer */
				rtEntry->age = rtEntry->fullAge;

				if (rtDynamicPolicyHeadPtr->timeprev!=rtEntry)
				{
					if (rtDynamicPolicyHeadPtr==rtEntry)
					{
						rtDynamicPolicyHeadPtr=rtEntry;
						goto match;
					}
					rtPPrev=rtDynamicPolicyHeadPtr->timeprev;
					rtTemp=rtEntry->timenext;
					rtTemp->timeprev=rtEntry->timeprev;
					
					rtTemp=rtEntry->timeprev;
					rtTemp->timenext=rtEntry->timenext;
					
					rtDynamicPolicyHeadPtr->timeprev=rtEntry;
					rtEntry->timenext=rtDynamicPolicyHeadPtr;
					rtEntry->timeprev=rtPPrev;
					rtPPrev->timenext=rtEntry;
				}			

				goto match;
			}
		}
		rtEntry=rtEntry->linknext;
	}
	*gip = 0;
	*gID = 0;
	return FAILED;
 
match:

	*gip = rtEntry->ip_alias;
	*gID = icmp_global_port++;

	if (icmp_global_port >= 65500) icmp_global_port = 3200;

	return SUCCESS;
}
#if 0 // chenyl
int32 _rtl8651_softIcmpPolicyRoute(rtl8651_conntrack_info *info)
{
	int hash,idx;
	int ip_d=info->ip_d;
	ipaddr_t ip_s = info->ip_s;
	int *ip_alias=info->ip_alias;
	uint16 *ip_aliasp=info->ip_aliasp;
	_rtl8651_tblDrvSoftIcmpTable_t *softIcmp;
	_rtl8651_tblDrvPolicyRoute_t *rtEntry=&rtStaticPolicy[0];	
	_rtl8651_tblDrvPolicyRoute_t *rtTemp,*rtPPrev;	

	for (;rtEntry->type!=0;rtEntry++)
	{
		switch (rtEntry->type)
		{
			case TRIGGER_SRCIP:
				if ((rtEntry->ip_start<=ip_s)&&
					(rtEntry->ip_end>=ip_s))
					goto match;
				break;
			case TRIGGER_DSTIP:
				if ((rtEntry->ip_start<=ip_d)&&
					(rtEntry->ip_end>=ip_d))
					goto match;
		}

	}	
	hash = hashcode(ip_d);
	rtEntry = rtHashTable[hash];
	while (rtEntry)
	{
 		if ( rtEntry->age > 0 )
		{
			int8 isMatch = FALSE;
			switch ( rtEntry->type & ~TYPE_BITS )
			{
				case TRIGGER_DSTIP:
					if ( rtEntry->trigger.ip_d == ip_d )
					{
						isMatch = TRUE;
					}
					break;
				case TRIGGER_SIPDIPPAIR:
					if (	( rtEntry->trigger.sipDipPair.sip == ip_s ) &&
						( rtEntry->trigger.sipDipPair.dip == ip_d ))
					{
						isMatch = TRUE;
					}
					break;
			}

			if ( isMatch == TRUE )
			{
				/* refresh Age-timer */
				rtEntry->age = rtEntry->fullAge;

				if (rtDynamicPolicyHeadPtr->timeprev!=rtEntry)
				{
					if (rtDynamicPolicyHeadPtr==rtEntry)
					{
						rtDynamicPolicyHeadPtr=rtEntry;
						goto match;
					}
					rtPPrev=rtDynamicPolicyHeadPtr->timeprev;
					rtTemp=rtEntry->timenext;
					rtTemp->timeprev=rtEntry->timeprev;
					
					rtTemp=rtEntry->timeprev;
					rtTemp->timenext=rtEntry->timenext;
					
					rtDynamicPolicyHeadPtr->timeprev=rtEntry;
					rtEntry->timenext=rtDynamicPolicyHeadPtr;
					rtEntry->timeprev=rtPPrev;
					rtPPrev->timenext=rtEntry;
				}			

				goto match;
			}
		}
		rtEntry=rtEntry->linknext;
	}
	*(int*)ip_alias=0;
	return 0;

match:
	softIcmp = &softIcmpNaptTable[0];
	for (idx=0;idx<SOFTICMPTABLE&&softIcmp->age>=1;softIcmp++)
	{
		idx++;
		if (softIcmp->internalId==info->icmpid &&
		     softIcmp->internalIp==info->ip_s)
		goto matchIcmp;
	}
	if (idx>=SOFTICMPTABLE)
		softIcmp = &softIcmpNaptTable[0];
	*ip_alias=rtEntry->ip_alias;
	*ip_aliasp=icmp_global_port++;
	softIcmp->globalId=*ip_aliasp;
	softIcmp->globalIp=*ip_alias;
	softIcmp->internalId=info->icmpid;
	softIcmp->internalIp=info->ip_s;
	softIcmp->age=64;
	return 1;
matchIcmp:
	*ip_alias=rtEntry->ip_alias;
	*ip_aliasp=softIcmp->globalId;
	softIcmp->globalIp=*ip_alias;
	softIcmp->age=64;
	return 1;
}
#endif
uint32 _rtl8651_getDemandRoutingEntry(uint32 ip_s,uint32 ip_d,uint8 proto,uint16 ip_dp)
{
	int rtIdx,identity;
	rtl8651_tblDrvDemandRoute_t *rtEntry=&rtDemandRouteTable[0],*rtTemp=NULL;	
	uint16 type=0;
	DLOG(1,"proto %d destion ip %x,port %d\n",proto,ip_d,ip_dp);
	
	switch(proto)
	{
		case IPPROTO_TCP:
			type=TRIGGER_DSTTCPPORT;
			break;
		case IPPROTO_UDP:
			type=TRIGGER_DSTUDPPORT;
			break;
	}
	for(rtIdx=0; rtIdx<STATICPOLICYSIZE && rtEntry->type!=0; rtIdx++, rtEntry++)
	{
		if (rtEntry->type == 0)
			break;
		switch (rtEntry->type)
		{
			case TRIGGER_DSTIP:
				if ((rtEntry->ip_start<=ip_d)&&
					(rtEntry->ip_end>=ip_d))
				{
					if ((rtEntry->ip_start==0)&&(rtEntry->ip_end==0xffffffff))
					{
							rtTemp=rtEntry;
							continue;
					}
					goto match;
				}
				break;
			case TRIGGER_SRCIP:
				if ((rtEntry->ip_start<=ip_s)&&
					(rtEntry->ip_end>=ip_s))
				{
					if ((rtEntry->ip_start==0)&&(rtEntry->ip_end==0xffffffff))
					{
							rtTemp=rtEntry;
							continue;
					}
					goto match;
				}
				break;

			case TRIGGER_DSTTCPPORT:
			case TRIGGER_DSTUDPPORT:
				if ((rtEntry->trigger.dport.start<=ip_dp)&&
					(rtEntry->trigger.dport.end>=ip_dp)&&
					(rtEntry->type==type))
					goto match;
				break;

		}
	}	
	if (rtTemp) {rtEntry=rtTemp;goto match;}
	return 0;
match:
	identity=rtEntry->identity;	
	if (rtEntry->p_handler){		
		rtEntry->p_handler(identity);	
		rtEntry=&rtDemandRouteTable[0];	
		for(rtIdx=0; rtIdx<STATICPOLICYSIZE; rtIdx++,rtEntry++)
			if (rtEntry->identity==identity)
				rtEntry->p_handler=0;
	}
	return 1;

}

/*@func uint32 | rtl8651_flushDemandRoute | delete demand route entry
@parm	int	|	identity |  pppoe Object Id
@rvalue SUCCESS | always return
@comm
Delete all route entry by external ip address
*/
int32 rtl8651_flushDemandRoute(int identity)
{
	int rtIdx,rtNewIdx;
	rtl8651_tblDrvDemandRoute_t  *rtEntry;
			
	rtlglue_drvMutexLock();//lock resource
	rtEntry=&rtDemandRouteTable[0];	
	for(rtIdx=0; rtIdx<STATICPOLICYSIZE; rtIdx++,rtEntry++)
	{
		if (rtEntry->identity==identity && rtEntry->type)
		{
			memset(rtEntry,0,sizeof(*rtEntry));
		}
	}

	for(rtIdx=0; rtIdx<STATICPOLICYSIZE; rtIdx++)
	{
		if (rtDemandRouteTable[rtIdx].type==0)
		{
			for(rtNewIdx=rtIdx+1; rtNewIdx<STATICPOLICYSIZE; rtNewIdx++)
				if (rtDemandRouteTable[rtNewIdx].type!=0)
				{
					rtDemandRouteTable[rtIdx]=rtDemandRouteTable[rtNewIdx];
					rtDemandRouteTable[rtNewIdx].type=0;
					break;
				}
		}
	}
	RTL_NOTICE(	RTL_MSG_POLICYRT,
					"Demand Route for session [%d] is flushed", identity);

	rtlglue_drvMutexUnlock();//lock resource
	return 0;
}
/*
@func uint32 | rtl8651_addDemandRoute | add demand route entry
@parm	rtl8651_tblDrvPolicyRoute_t * | rtAdd | demand route entry
@parm	int	|	identity |  pppoe Object Id
@parm int32 (*)(uint32) | p_callBack | Call back function.
@rvalue SUCCESS | The Entry is added successfully.
@rvalue RTENTRY_FOUND | The Entry is already existed.
@rvalue RTENTRY_FULL  | The Entry is already existed.
@comm
Add A Demand Route Entry
*/
int32 rtl8651_addDemandRoute(rtl8651_tblDrvDemandRoute_t * rtAdd, int identity,int32(*p_callBack)(uint32))
{
	int rtIdx,retVal=0;
	rtl8651_tblDrvDemandRoute_t  *rtEntry;
	DLOG(1,"add Demand Route %x idenity %d\n",rtAdd->trigger.ip_d,identity);

	rtlglue_drvMutexLock();//lock resource
	rtEntry=&rtDemandRouteTable[0];	
	for(rtIdx=0; rtIdx<STATICPOLICYSIZE; rtIdx++, rtEntry++)
	{
		if (!rtEntry->type)break;
		if (rtEntry->type != (rtAdd->type&(~TYPE_BITS)))
			continue;
		switch(rtEntry->type)
		{
			case TRIGGER_DSTIP:
			case TRIGGER_SRCIP:
			case TRIGGER_SRCGLOBALIP:
				if (rtEntry->ip_start==rtAdd->ip_start  && rtEntry->ip_end==rtAdd->ip_end)
				{
					retVal=RTENTRY_FOUND;
					goto fail;
				}
				break;
			case TRIGGER_DSTTCPPORT:
			case TRIGGER_DSTUDPPORT:
				if (rtEntry->trigger.ip_d==rtAdd->trigger.ip_d)
		{
					retVal=RTENTRY_FOUND;
					goto fail;
				}
				break;
		}

	}

	rtEntry=&rtDemandRouteTable[0];	
	for(rtIdx=0; rtIdx<STATICPOLICYSIZE; rtIdx++, rtEntry++)
		if (rtEntry->type == 0)
			break;
	if (rtIdx==STATICPOLICYSIZE) 
	{
		retVal=RTENTRY_FULL;
		goto fail;
	}
	rtEntry->type=(rtAdd->type&(~TYPE_BITS));
	rtEntry->trigger.ip_d=rtAdd->trigger.ip_d;
	rtEntry->ip_start=rtAdd->ip_start;
	rtEntry->ip_end=rtAdd->ip_end;
	rtEntry->identity=identity;
	rtEntry->p_handler = p_callBack;

	RTL_NOTICE(	RTL_MSG_POLICYRT,
					"Add Demand Route");

	RTL_NOTICE(	RTL_MSG_POLICYRT,
					"\tType: 0x%x", rtEntry->type);

	RTL_NOTICE(	RTL_MSG_POLICYRT,
					"\tTrigger dst Port: %d",
					rtEntry->trigger.ip_d);

	RTL_NOTICE(	RTL_MSG_POLICYRT,
					"\tIP start (%u.%u.%u.%u) -- end (%u.%u.%u.%u)",
					NIPQUAD(rtEntry->ip_start), NIPQUAD(rtEntry->ip_end));

	RTL_NOTICE(	RTL_MSG_POLICYRT,
					"\tIdentity: %d", rtEntry->identity);

	RTL_NOTICE(	RTL_MSG_POLICYRT,
					"\tCall Back Function: %p", rtEntry->p_handler);


fail:
	rtlglue_drvMutexUnlock();//unlock resource
	return retVal;


}


















static uint32 _rtl8651_hashValue(uint32 dip)
{
	uint32 result=0;

//	result = dip ^ (dip>>8) ^ (dip>>16) ^ (dip>>24);
//	return result&(rtl8651_tblDrvPara.dynamicPolicy-1);
	return result;
}



static int32 _rtl8651_delPolicy(rtl8651_tblDrvPolicy_t *spolicy_t, rtl8651_tblDrvPolicy_t *dpolicy_t)
{
	_rtl8651_tblDrvPolicy_t *delPolicy_t;

	assert(spolicy_t || dpolicy_t);

	TAILQ_FOREACH(delPolicy_t, &DrvTbl.inuseList.policyRoute, nextPolicy_t) {
		if ((spolicy_t && !delPolicy_t->p_handler && !memcmp(delPolicy_t, spolicy_t, sizeof( rtl8651_tblDrvPolicy_t))) ||
                    (dpolicy_t && delPolicy_t->p_handler && !memcmp(delPolicy_t, dpolicy_t, sizeof(rtl8651_tblDrvPolicy_t))) )
		{
			TAILQ_REMOVE(&DrvTbl.inuseList.policyRoute, delPolicy_t, nextPolicy_t);
			TAILQ_INSERT_HEAD(&DrvTbl.freeList.freePolicy, delPolicy_t, nextPolicy_t);
			return SUCCESS;
		}
	}
	return TBLDRV_EENTRYNOTFOUND;
}


static int32 _rtl8651_flushPolicy(ipaddr_t naptip, uint32 sessionId)
{
	_rtl8651_tblDrvPolicy_t *dpolicy_t, *temp_t;
	uint32 hash;

	assert(naptip || sessionId);

	/* Flush Demand Route or Static Policy Route */
	dpolicy_t = TAILQ_FIRST(&DrvTbl.inuseList.policyRoute);
	while (dpolicy_t) {	
		if ((!naptip && dpolicy_t->pppoe_t->sessionId == sessionId && dpolicy_t->p_handler) ||
		    (naptip && !dpolicy_t->p_handler && dpolicy_t->naptip == naptip)  ) 
		{
			TAILQ_REMOVE(&DrvTbl.inuseList.policyRoute, dpolicy_t, nextPolicy_t);
			TAILQ_INSERT_HEAD(&DrvTbl.freeList.freePolicy, dpolicy_t, nextPolicy_t);
			dpolicy_t = TAILQ_FIRST(&DrvTbl.inuseList.policyRoute);
			continue;
		}
		dpolicy_t = TAILQ_NEXT(dpolicy_t, nextPolicy_t);
	}

	if (naptip) {
		/* Flush Dynamic Route */
		dpolicy_t = CIRCLEQ_FIRST(&DrvTbl.dynamicPolicy.dyHead);
		while (dpolicy_t) {
			if (dpolicy_t == (void *)&DrvTbl.dynamicPolicy.dyHead)
				break;
			if (dpolicy_t->naptip == naptip) {
				temp_t = CIRCLEQ_NEXT(dpolicy_t, nextTime_t);
				hash = _rtl8651_hashValue(dpolicy_t->dip);
				TAILQ_REMOVE(&DrvTbl.dynamicPolicy.hashTable[hash], dpolicy_t, nextPolicy_t);
				CIRCLEQ_REMOVE(&DrvTbl.dynamicPolicy.dyHead, dpolicy_t, nextTime_t);
				CIRCLEQ_INSERT_HEAD(&DrvTbl.dynamicPolicy.dyHead, dpolicy_t, nextTime_t);
				bzero(dpolicy_t, (sizeof(_rtl8651_tblDrvPolicy_t)-sizeof(dpolicy_t->nextPolicy_t)-sizeof(dpolicy_t->nextTime_t)));
				dpolicy_t = temp_t;
				continue;
			}
			dpolicy_t = CIRCLEQ_NEXT(dpolicy_t, nextTime_t);
		}
	}	
	return SUCCESS;
}



_rtl8651_tblDrvPolicy_t *
_rtl8651_getPolicyRoute(int8 proto, ipaddr_t sip, uint16 sport, ipaddr_t dip, ipaddr_t dport)
{
	TAILQ_HEAD(protoType, _rtl8651_tblDrvPolicy_s) *policyHead_t[2];
	_rtl8651_tblDrvPolicy_t *policy_t; 
	uint32 hash;

	hash = _rtl8651_hashValue(dip);
	/* Find static policy and demand policy first */
	policyHead_t[0] = (struct protoType *)&DrvTbl.inuseList.policyRoute;
	policyHead_t[1] = (struct protoType *)&DrvTbl.dynamicPolicy.hashTable[hash];

	for(hash=0; hash<2; hash++) {
		TAILQ_FOREACH(policy_t, policyHead_t[hash], nextPolicy_t) {
			switch (policy_t->proto)
			{
			case RTL8651_POLICY_TCP:
				if (proto != IPPROTO_TCP)
					continue;
				goto comm_L4;
				
			case RTL8651_POLICY_UDP:
				if (proto != IPPROTO_UDP)
					continue;
comm_L4:
				if (!(dport <= policy_t->dport_end && dport >= policy_t->dport_start &&
				      sport <= policy_t->sport_end && sport >= policy_t->sport_start))
					continue;
				/* pass through */

			case RTL8651_POLICY_IP:
				if (((sip&policy_t->sipMask)==policy_t->sip) && ((dip&policy_t->dipMask)==policy_t->dip)) {
					if (hash != 0) {
						/* dynamic policy: upate time list */
						CIRCLEQ_REMOVE(&DrvTbl.dynamicPolicy.dyHead, policy_t, nextTime_t);
						CIRCLEQ_INSERT_TAIL(&DrvTbl.dynamicPolicy.dyHead, policy_t, nextTime_t);
					}
					return policy_t;
				}
				break;

			default: break;
			}
		}
	}
	return NULL;
}


/* NEW */
int32 rtl8651_flushPolicyRoute_1(ipaddr_t naptip)
{
	int32 retval;
	
	rtlglue_drvMutexLock();
	retval = _rtl8651_flushPolicy(naptip, 0);
	rtlglue_drvMutexUnlock();
	return SUCCESS;
}


/* NEW */
int32 rtl8651_delPolicyRoute(int8 type, rtl8651_tblDrvPolicy_t *policy_t)
{
	_rtl8651_tblDrvPolicy_t *delPolicy_t;
	uint32 hash;
	int32 retval=SUCCESS;
	
	if (policy_t == NULL)
		return TBLDRV_EINVALIDINPUT;
	rtlglue_drvMutexLock();
	switch (type)
	{
	case DYNAMIC_POLICY_ROUTE:
		CIRCLEQ_FOREACH(delPolicy_t, &DrvTbl.dynamicPolicy.dyHead, nextTime_t) {
			if (!memcmp(delPolicy_t, policy_t, sizeof(rtl8651_tblDrvPolicy_t))) {
				hash = _rtl8651_hashValue(delPolicy_t->dip);
				TAILQ_REMOVE(&DrvTbl.dynamicPolicy.hashTable[hash], delPolicy_t, nextPolicy_t);
				CIRCLEQ_REMOVE(&DrvTbl.dynamicPolicy.dyHead, delPolicy_t, nextTime_t);
				CIRCLEQ_INSERT_HEAD(&DrvTbl.dynamicPolicy.dyHead, delPolicy_t, nextTime_t);
				bzero(delPolicy_t, sizeof(_rtl8651_tblDrvPolicy_t)-sizeof(delPolicy_t->nextTime_t)-sizeof(delPolicy_t->nextPolicy_t));
				goto out;
			}
		}
		break;
		
	case STATIC_POLICY_ROUTE:
		retval = _rtl8651_delPolicy(policy_t, NULL);
		break;
		
	default: retval = TBLDRV_EINVALIDINPUT;
	}
out:
	rtlglue_drvMutexUnlock();
	return retval;	
}


/* NEW */
int32 rtl8651_addPolicyRoute_1(int8 type, rtl8651_tblDrvPolicy_t *policy_t)
{
	rtl8651_tblDrv_pppTable_t *pppoe_t;
	_rtl8651_tblDrvPolicy_t *spolicy_t;
	uint32 hash1, hash2;
	int32 retval = SUCCESS;

	if (policy_t == NULL || !(policy_t->proto == RTL8651_POLICY_IP || policy_t->proto==RTL8651_POLICY_TCP))
		return TBLDRV_EINVALIDINPUT;
	rtlglue_drvMutexLock();

	switch(type)
	{
	case DYNAMIC_POLICY_ROUTE:
		if ((pppoe_t=_rtl8651_getPPPInfo(0, policy_t->naptip)) == NULL) {
			retval = TBLDRV_EINVALIDINPUT;
			goto out;
		}
		hash1 = _rtl8651_hashValue(policy_t->dip);
		spolicy_t = CIRCLEQ_FIRST(&DrvTbl.dynamicPolicy.dyHead);
		if (spolicy_t->pppoe_t) {
			hash2 = _rtl8651_hashValue(spolicy_t->dip);
			TAILQ_REMOVE(&DrvTbl.dynamicPolicy.hashTable[hash2], spolicy_t, nextPolicy_t);
		}
		bzero(spolicy_t, sizeof(_rtl8651_tblDrvPolicy_t)-sizeof(spolicy_t->nextPolicy_t)-sizeof(spolicy_t->nextTime_t));
		rtl8651_memcpy(spolicy_t, policy_t, sizeof(rtl8651_tblDrvPolicy_t));
		TAILQ_INSERT_HEAD(&DrvTbl.dynamicPolicy.hashTable[hash1], spolicy_t, nextPolicy_t);
		CIRCLEQ_REMOVE(&DrvTbl.dynamicPolicy.dyHead, spolicy_t, nextTime_t);
		CIRCLEQ_INSERT_TAIL(&DrvTbl.dynamicPolicy.dyHead, spolicy_t, nextTime_t);
		spolicy_t->pppoe_t = pppoe_t;
		break;
		
	case STATIC_POLICY_ROUTE:
		if ((spolicy_t=TAILQ_FIRST(&DrvTbl.freeList.freePolicy)) == NULL) {
			retval = TBLDRV_ENOFREEBUFFER;
			goto out;
		}
		bzero(spolicy_t, sizeof(_rtl8651_tblDrvPolicy_t)-sizeof(spolicy_t->nextPolicy_t));
		rtl8651_memcpy(spolicy_t, policy_t, sizeof(rtl8651_tblDrvPolicy_t));
		if ((spolicy_t->pppoe_t = _rtl8651_getPPPInfo(0, policy_t->naptip)) == NULL) {
			retval = TBLDRV_EINVALIDINPUT;
			goto out;
		}
		if (_rtl8651_addNextHopEntry(NEXTHOP_POLICY, spolicy_t,  spolicy_t->pppoe_t->netif_t, 0, 0, spolicy_t->pppoe_t) == FAILED) {
			retval = TBLDRV_EINVALIDINPUT;
			goto out;
		}
		TAILQ_REMOVE(&DrvTbl.freeList.freePolicy, spolicy_t, nextPolicy_t);
		TAILQ_INSERT_HEAD(&DrvTbl.inuseList.policyRoute, spolicy_t, nextPolicy_t);
		break;
		
	default: retval = TBLDRV_EINVALIDINPUT; 

	}

	_rtl8651_arrangeAllACL();
out:
	rtlglue_drvMutexUnlock();
	return retval;
}

/* NEW */
int32 rtl8651_flushDemandRoute_1(uint32 sessionId)
{
	int32 retval;
	
	rtlglue_drvMutexLock();
	retval = _rtl8651_flushPolicy(0, sessionId);
	rtlglue_drvMutexUnlock();
	return retval;
}


/* NEW */
int32 rtl8651_delDemandRoute(rtl8651_tblDrvPolicy_t *policy_t)
{
	int32 retval;

	rtlglue_drvMutexLock();
	retval = _rtl8651_delPolicy(NULL, policy_t);
	rtlglue_drvMutexUnlock();
	return retval;
}

/* NEW */
int32 rtl8651_addDemandRoute_1(rtl8651_tblDrvPolicy_t *policy_t, uint32 sessionId, int32(*p_callBack)(uint32))
{
	rtl8651_tblDrv_pppTable_t *pppoe_t;
	_rtl8651_tblDrvPolicy_t *dpolicy_t;
	int32 retval = SUCCESS;
	
	
	if (sessionId == 0 || !p_callBack ||!policy_t)
		return TBLDRV_EINVALIDINPUT;
	rtlglue_drvMutexLock();

	/* Check duplicate entry */
	TAILQ_FOREACH(dpolicy_t, &DrvTbl.inuseList.policyRoute, nextPolicy_t) {
		if (!memcmp(dpolicy_t, policy_t, (sizeof(rtl8651_tblDrvPolicy_t)-sizeof(ipaddr_t)))) {
			retval = TBLDRV_EENTRYALREADYEXIST;
			goto out;
		}
	}
	if ((pppoe_t =_rtl8651_getPppoeBySessionId(sessionId)) == NULL) {
		retval = TBLDRV_EINVALIDSESSIONID;
		goto out;
	}
	if ((dpolicy_t=TAILQ_FIRST(&DrvTbl.freeList.freePolicy)) == NULL) {
		retval = TBLDRV_ENOFREEBUFFER;
		goto out;
	}
	TAILQ_REMOVE(&DrvTbl.freeList.freePolicy, dpolicy_t, nextPolicy_t);
	bzero(dpolicy_t, sizeof(_rtl8651_tblDrvPolicy_t)-sizeof(dpolicy_t->nextPolicy_t));
	rtl8651_memcpy(dpolicy_t, policy_t, sizeof(rtl8651_tblDrvPolicy_t)-sizeof(ipaddr_t));
	TAILQ_INSERT_HEAD(&DrvTbl.inuseList.policyRoute, dpolicy_t, nextPolicy_t);
	dpolicy_t->p_handler = p_callBack;
	dpolicy_t->pppoe_t = pppoe_t;
	
out:
	rtlglue_drvMutexUnlock();
	return retval;
}


/* NEW */
static uint32 _rtl8651_addPolicyRouteToAcl(uint32 aclStart, uint32 aclEnd, int8 *enoughFlag) 
{
	_rtl8651_tblDrvPolicy_t *policy_t;
	_rtl8651_tblDrvAclRule_t rule;
	uint32 count = 0;
	
	assert(aclEnd < RTL8651_ACLTBL_SIZE);
	if (aclStart > aclEnd)	return 0;

	TAILQ_FOREACH(policy_t, &DrvTbl.inuseList.policyRoute, nextPolicy_t) {
		assert(policy_t->pppoe_t);
		if (aclStart > aclEnd) {
			*enoughFlag = 0;	// There's not enough space for adding ACL rules.
			return count;
		}

		bzero(&rule, sizeof(_rtl8651_tblDrvAclRule_t));
		rule.srcIpAddr_			= policy_t->sip;
		rule.srcIpAddrMask_		= policy_t->sipMask;
		rule.dstIpAddr_			= policy_t->dip;
		rule.dstIpAddrMask_		= policy_t->dipMask;
		switch( policy_t->proto )
		{
		case RTL8651_POLICY_IP:
			 rule.ruleType_ 	= RTL8651_ACL_IP;
			 break;
		case RTL8651_POLICY_TCP:
			 rule.ruleType_ 	= RTL8651_ACL_TCP;
			 rule.tcpSrcPortUB_	= policy_t->sport_end;
			 rule.tcpSrcPortLB_	= policy_t->sport_start;
			 rule.tcpDstPortUB_	= policy_t->dport_end;
			 rule.tcpDstPortLB_	= policy_t->dport_start;
			 break;
		case RTL8651_POLICY_UDP:
			 rule.ruleType_ 	= RTL8651_ACL_UDP;
			 rule.udpSrcPortUB_	= policy_t->sport_end;
			 rule.udpSrcPortLB_	= policy_t->sport_start;
			 rule.udpDstPortUB_	= policy_t->dport_end;
			 rule.udpDstPortLB_	= policy_t->dport_start;
			 break;
		default: assert(0);
		}
		
		rule.actionType_ = RTL8651_ACL_POLICY;
		rule.nhIndex = policy_t->nexthop_t->entryIndex;
		rtl8651_setAsicAclRule(aclStart, &rule);
		aclStart ++; count ++;
	}
	return count;
}


#define RTL8651_LAN_SIDE_IP_UNNUMBERED_APIs
/*======================================================================================
 *
 *
 *======================================================================================*/
static uint32 
_rtl8651_addAclForLanSideIpUnnumbered(int8 isAdd, rtl8651_tblDrv_networkIntfTable_t *netif_t, uint32 aclStart, uint32 aclEnd) {
	_rtl8651_tblDrvAclRule_t aclRule;
	//rtl8651_tblDrv_arpEntry_t *arp_t;
	rtl8651_tblDrv_ipIntfEntry_t *ipintf_t;
	ipaddr_t sip, sipm, dip ,dipm, globalIp;
	uint32 count = 0;
	int32 ret;

	if (!lanSideIpUnnumberedLan || !lanSideIpUnnumberedWan)
		return 0;
	assert(netif_t->linkLayerType == RTL8651_LL_LIPUNNUMBERED);

	/* add rule to trap all tcp/syn packets */
	{
 		_rtl8651_tblDrvAclRule_t rule;

	 	/* Layer 4 => TCP */
 		/* Trap all TCP packets with SYN bit set to CPU */
		if (isAdd == TRUE) {
			if (aclStart > aclEnd)
				return count;
			memset(&rule, 0, sizeof(_rtl8651_tblDrvAclRule_t));
			rule.ruleType_     = RTL8651_ACL_TCP;
		 	rule.actionType_   = RTL8651_ACL_CPU;
	 		rule.tcpSYN_       = 0x01;
		 	rule.tcpFlagMask_  = rule.tcpFlag_; /* SYN bit set */
		 	rule.tcpSrcPortUB_ = 0xffff;
		 	rule.tcpSrcPortLB_ = 0x00;
		 	rule.tcpDstPortUB_ = 0xffff;
	 		rule.tcpDstPortLB_ = 0x00;
		 	rtl8651_setAsicAclRule(aclStart, &rule);
			aclStart++;
 		}
		count=1;
	}

#if 0
// obsolleted 2003/11/17
	SLIST_FOREACH(ipintf_t, &lanSideIpUnnumberedLan->IpHead, nextIp) {
		if (ipintf_t->lanSideExternal == 0)
			continue;
		SLIST_FOREACH(arp_t, &ipintf_t->nextArp, next) {
			if (arp_t->dmzIpPending == 0)
				continue;
			/* add ACL rule to trap to cpu */
			globalIp = arp_t->ipAddr;
			if (netif_t == lanSideIpUnnumberedLan) {
				sip = globalIp; sipm = 0xffffffff;
				dip = 0x00; dipm = 0x00;
			} else {
				sip = 0x00; sipm = 0x00;
				dip = globalIp; dipm = 0xffffffff;
			}			
			RTL8651_IP_ACL_RULE(&aclRule, RTL8651_ACL_CPU, sip, sipm, dip, dipm, 0, 0, 0, 0, 0, 0);
			if (isAdd == TRUE) rtl8651_setAsicAclRule(aclStart, &aclRule);
			count ++; aclStart++;
			if (aclStart > aclEnd)
				return count;	
			}
	}
#else

 	SLIST_FOREACH(ipintf_t, &lanSideIpUnnumberedLan->IpHead, nextIp)
 	{
		uint32 subnetIpToBeAddedInAclTotalCount;
		uint32 subnetIpToBeAddedInAclToEachInterface;
		uint32 subnetGlobalIpCount;
		uint8 patchWithACL;
		rtl8651_tblDrv_ipUnnumbered_t *ipUnnumber_t;
		ipaddr_t ipToBeAdded,aclStartIp;
		ipaddr_t mySubnet;
		uint32   i;
		if (ipintf_t->lanSideExternal == 0)
		{ 
			continue;
		}

		assert(ipintf_t->ipUnnumber);
		ipUnnumber_t = ipintf_t->ipUnnumber;
		
		globalIp = ipintf_t->ipAddr.slh_first->ipAddr; /* assume always fetch the 1st only */
		mySubnet = globalIp&ipintf_t->ipMask;
		
		subnetGlobalIpCount=0xffffffff & (uint32)(~((uint32)ipintf_t->ipMask));

		/*
			If the ACL budget of the ip unnumber patch is not enough,
			we stop patch it and try to patch it using routing table			*/
		subnetIpToBeAddedInAclToEachInterface = (subnetGlobalIpCount - (ipUnnumber_t->ip_as_LS /* as Local Server */ + 1 /* globalIp */));
		subnetIpToBeAddedInAclTotalCount = (subnetIpToBeAddedInAclToEachInterface << 1 /* (*2) because same IP Address need to added to both LAN and WAN */);

		if (subnetIpToBeAddedInAclToEachInterface == 0)	/* we don't need to patch for unnumber IP */
			continue;


		/*
		  *
		  *	Check if the ACL rule budget can patch this unnumber interface.
		  *
		  *	For an unnumber interface without any ACL budget gotten by Global budget:
		  *
		  *	1.
		  *	If the amount of the unnumber hosts which need to patch is larger than the Global budget
		  *   or larger than the remaining ACL rule count, we would stop ACL rule patch and try to patch
		  *	unnumber host forwarding with Routing Table modification (action become "TO CPU").
		  *
		  *		@ Stop add ACL rule
		  *		@ Modify L3 table entry for LAN side Unnumber interface
		  *		    (modify the action from original to ToCPU)
		  *
		  *	2.
		  *	If the Global budget and the remaining ACL rule count can fulfil the need of this unnumber
		  *	interface, we could patch this unnumber interface with ACL rule.
		  *
		  *		@ Modify L3 table entry for LAN side Unnumber interface
		  *		   (modify the action from original to ToCPU)
		  *		@ Add ACL rule
		  *
		  */
		patchWithACL = TRUE;

		if (ipUnnumber_t->ip_as_ACL_budget > 0)	
		{
			/*
				The budget of this unnumber network interface is > 0, it means we have
				release the total Unnumber ACL budget to it.
			*/

			/* budget is not enough to patch ? we try to get more budget from shared unnumber ACL budget */
			if (ipUnnumber_t->ip_as_ACL_budget < subnetIpToBeAddedInAclToEachInterface)
			{
				if ((rtl8651_ipUnnumberGlobalCtrlBlock.ip_unnumber_ACL_budget + ipUnnumber_t->ip_as_ACL_budget) >= subnetIpToBeAddedInAclToEachInterface)
				{
					/*	Total budget still can fulfil the need of unnumbered IP patch, so we get extra budget
						from shared budget and use it to patch											*/
					uint32 addedBudget = subnetIpToBeAddedInAclToEachInterface - ipUnnumber_t->ip_as_ACL_budget;
					ipUnnumber_t->ip_as_ACL_budget += addedBudget;
					rtl8651_ipUnnumberGlobalCtrlBlock.ip_unnumber_ACL_budget -= addedBudget;
				}else
				{
					/*	Total budget still can NOT fulfil the need of whole
						unnumbered IP patch, and we stop patch it.				*/
					patchWithACL = FALSE; /* we can not patch whole unnumber host with ACL */
				}
			}
			if (ipUnnumber_t->ip_as_ACL_budget >= (aclEnd - aclStart))
			{
				patchWithACL = FALSE;	/* we can not patch whole unnumber host with ACL */
			}

			/*	If we stop ACL patch, we would give the remaining budget back to the shared budget. */
			if (patchWithACL == FALSE)
			{
				rtl8651_ipUnnumberGlobalCtrlBlock.ip_unnumber_ACL_budget += ipUnnumber_t->ip_as_ACL_budget;
				ipUnnumber_t->ip_as_ACL_budget = 0;
			}
		} else
		{
			/*
				The budget of this unnumber network interface is == 0, it means this unnumber interface (pair)
				need to share the remaining Total Unnumber ACL budget.

				So we need to check 2 things:

				1. If the remaining Unnumber ACL budget can fulfil the need of this unnumber interface?
				2. If the remaining ACL rule count can fulfil the need of this unnumber interface?
			*/
			if (subnetIpToBeAddedInAclTotalCount > rtl8651_ipUnnumberGlobalCtrlBlock.ip_unnumber_ACL_budget) /* exceed budget */
			{
				patchWithACL = FALSE;
			}

			if (subnetIpToBeAddedInAclTotalCount >= (aclEnd - aclStart) /* exceed ACL total count */)
			{
				patchWithACL = FALSE;
			}

			/*
				We get budget of this interface from shared ACL budget.
			*/
			if (patchWithACL == TRUE)
			{
				ipUnnumber_t->ip_as_ACL_budget += subnetIpToBeAddedInAclTotalCount;
				rtl8651_ipUnnumberGlobalCtrlBlock.ip_unnumber_ACL_budget -= subnetIpToBeAddedInAclTotalCount;
			}
		}

		if (patchWithACL == FALSE)
		{
			if (isAdd == TRUE)
			{
				int32 idx;
				rtl8651_tblDrv_routeTable_t *rt_t = &DrvTbl.route[0];

			       /* update Routing Entry, set the action of this subnet as {TOCPU} */
				for ( idx=0 ; idx<rtl8651_tblDrvPara.routingTableSize ; idx++ )
				{
					if (rt_t[idx].valid == 0)
						continue;
					if (rt_t[idx].netif_t == lanSideIpUnnumberedLan)
					{
						if (rt_t[idx].ipintf_t == ipintf_t)
						{
							assert((rt_t[idx].ipAddr&rt_t[idx].ipMask) == (ipintf_t->ipAddr.slh_first->ipAddr&ipintf_t->ipMask));
							// update this routing entry
							if (rt_t[idx].process != RT_CPU)
							{
								_rtl8651_delInterfaceRoute(globalIp, ipintf_t->ipMask);
								_rtl8651_addInterfaceRoute(lanSideIpUnnumberedLan, ipintf_t, 0xffffffff, 0xffffffff);
							}
							break;
						}
					}
				}
				assert((idx < (RTL8651_ROUTINGTBL_SIZE-1)) && (idx != (rtl8651_tblDrvPara.routingTableSize-1)));
			}

			/* don't add any ACL rule into ASIC */
			return count;

		} else
		{
			/* we can patch ip unnumber with LS and ACL */
			if (isAdd == TRUE)
			{
				int32 idx;
				rtl8651_tblDrv_routeTable_t *rt_t = &DrvTbl.route[0];

			       /* update Routing Entry, set the action of this subnet as original one */
				for ( idx=0 ; idx<rtl8651_tblDrvPara.routingTableSize ; idx++ )
				{
					if (rt_t[idx].valid == 0)
						continue;
					if (rt_t[idx].netif_t == lanSideIpUnnumberedLan)
					{
						if (rt_t[idx].ipintf_t == ipintf_t)
						{
							/* pdate this routing entry */
							if (rt_t[idx].process==RT_CPU && ipintf_t->arpStartPos!=0xffffffff)
							{
								_rtl8651_delInterfaceRoute(globalIp, ipintf_t->ipMask);
								_rtl8651_addInterfaceRoute(lanSideIpUnnumberedLan, ipintf_t, ipintf_t->arpStartPos, ipintf_t->arpEndPos);
							}
							break;
						}
					}
				}
				assert((idx < (RTL8651_ROUTINGTBL_SIZE-1)) && (idx != (rtl8651_tblDrvPara.routingTableSize-1)));
 			}
		}

		/*
		  *	ACL process -- fill/add ACL rules into ASIC
		  *
		  *		If program arrives here, it means this unnumber interface is allowed to use ACL to
		  *		patch its Unnumber Hosts forwarding.
		  *
		  *		Process:
		  *			1. Calculate the Start IP to patch (both WAN interface and LAN interface are the same).
		  *			2. If this round is to add LAN interface, we fill the IP rule with unnumber IP being SIP;
		  *			    If this round is to add WAN interface, we fill the IP rule with unnumber IP being DIP
		  *			3. We need to update remaing-ACL-budget of this unnumber interface when we add
		  *			    new ACL rule.
		  *
		  */


		assert(isAdd == FALSE || ipUnnumber_t->ip_as_ACL_budget > 0);

		/* the ACL to add is {Total count of subnet IP - Total count of IP in ASIC}*/ 
		if (globalIp<=(mySubnet+ipUnnumber_t->ip_as_LS+1))
			aclStartIp = (ipaddr_t)((uint32)mySubnet+ipUnnumber_t->ip_as_LS+2);
		else
			aclStartIp = (ipaddr_t)((uint32)mySubnet+ipUnnumber_t->ip_as_LS+1);
 
		for (i=1;i<subnetGlobalIpCount;i++){
			ipToBeAdded=(ipaddr_t)((uint32)mySubnet+i);
			if ((uint32)ipToBeAdded==(uint32)globalIp)
				continue; //skip this one
			if (ipToBeAdded<aclStartIp)
				continue; // this one is in h/w
 
			if (netif_t == lanSideIpUnnumberedLan){
				/* lan rule */
				sip = ipToBeAdded; sipm = 0xffffffff;
				dip = 0x00; dipm = 0x00;
				if (isAdd == TRUE){
					assert(ipUnnumber_t->ip_as_ACL_budget > 0);

					if (aclStart > aclEnd)
						return count;
					ipUnnumber_t->ip_as_ACL_budget--;	/* decrease ACL budget */
					memset(&aclRule, 0, sizeof(aclRule));
					RTL8651_IP_ACL_RULE(&aclRule, RTL8651_ACL_CPU, sip, sipm, dip, dipm, 0, 0, 0, 0, 0, 0);
					ret = rtl8651_setAsicAclRule(aclStart, &aclRule);
					assert( ret == SUCCESS );
					aclStart++;
				}
				count ++;
			}else{
				/* wan rule */
				sip = 0x00; sipm = 0x00;
				dip = ipToBeAdded; dipm = 0xffffffff;
				if (isAdd == TRUE){
					assert(ipUnnumber_t->ip_as_ACL_budget > 0);
					if (aclStart > aclEnd)
						return count;
					ipUnnumber_t->ip_as_ACL_budget --;	/* decrease ACL budget */
					memset(&aclRule, 0, sizeof(aclRule));
					RTL8651_IP_ACL_RULE(&aclRule, RTL8651_ACL_CPU, sip, sipm, dip, dipm, 0, 0, 0, 0, 0, 0);
					ret = rtl8651_setAsicAclRule(aclStart, &aclRule);
					assert( ret == SUCCESS );
					aclStart++;
				}
				count ++;
			}
		}
	}
#endif
	return count;
}

#if 0 
static uint32 
_rtl8651_addAclForLanSideIpUnnumbered(int8 isAdd, rtl8651_tblDrv_networkIntfTable_t *netif_t, uint32 aclStart, uint32 aclEnd) {
	rtl8651_tblDrvAclRule_t aclRule;
	rtl8651_tblDrv_arpEntry_t *arp_t;
	rtl8651_tblDrv_ipIntfEntry_t *ipintf_t;
	ipaddr_t sip, sipm, dip ,dipm, globalIp;
	uint32 count = 0;

	if (!lanSideIpUnnumberedLan || !lanSideIpUnnumberedWan)
		return 0;
	assert(netif_t->linkLayerType == RTL8651_LL_LIPUNNUMBERED);

	/* add rule to trap all tcp/syn packets */
#if 1
//debug: turn off this patched rule
	assert((aclStart<=aclEnd) && (aclEnd<RTL8651_ACLTBL_SIZE));	
	{
 		rtl8651_tblDrvAclRule_t rule;

	 	/* Layer 4 => TCP */
 		/* Trap all TCP packets with SYN bit set to CPU */
		if (isAdd == TRUE) {
			memset(&rule, 0, sizeof(rtl8651_tblDrvAclRule_t));
			rule.ruleType_     = RTL8651_ACL_TCP;
		 	rule.actionType_   = RTL8651_ACL_CPU;
	 		rule.tcpSYN_       = 0x01;
		 	rule.tcpFlagMask_  = rule.tcpFlag_; /* SYN bit set */
		 	rule.tcpSrcPortUB_ = 0xffff;
		 	rule.tcpSrcPortLB_ = 0x00;
		 	rule.tcpDstPortUB_ = 0xffff;
	 		rule.tcpDstPortLB_ = 0x00;
		 	rtl8651_setAsicAclRule(aclStart, &rule);
			aclStart++;
			if (aclStart > aclEnd)
				return count;		 	
 		}
		count=1;
	}
#endif

	SLIST_FOREACH(ipintf_t, &lanSideIpUnnumberedLan->IpHead, nextIp) {
		if (ipintf_t->lanSideExternal == 0)
			continue;
		SLIST_FOREACH(arp_t, &ipintf_t->nextArp, next) {
			if (arp_t->dmzIpPending == 0)
				continue;
			/* add ACL rule to trap to cpu */
			globalIp = arp_t->ipAddr;
			if (netif_t == lanSideIpUnnumberedLan) {
				sip = globalIp; sipm = 0xffffffff;
				dip = 0x00; dipm = 0x00;
			} else {
				sip = 0x00; sipm = 0x00;
				dip = globalIp; dipm = 0xffffffff;
			}			
			RTL8651_IP_ACL_RULE(&aclRule, RTL8651_ACL_CPU, sip, sipm, dip, dipm, 0, 0, 0, 0, 0, 0);
			if (isAdd == TRUE) rtl8651_setAsicAclRule(aclStart, &aclRule);
			count ++; aclStart++;
			if (aclStart > aclEnd)
				return count;	
			}
	}
	return count;	
}
#endif


 int8 _rtl8651_isLanSideIpUnnumberedIpAddr(ipaddr_t ipAddr) {
	rtl8651_tblDrv_ipIntfEntry_t *ipintf_t;

	if (!lanSideIpUnnumberedLan)
		return FALSE;
		
	SLIST_FOREACH(ipintf_t, &lanSideIpUnnumberedLan->IpHead, nextIp) {
		if (ipintf_t->lanSideExternal == 0)
			continue;
		if ((ipintf_t->ipMask&ipintf_t->ipAddr.slh_first->ipAddr)==(ipintf_t->ipMask&ipAddr))
			return TRUE;
	}
	return FALSE;
}


	
#define RTL8651_EXTERNAL_NETWORK_INTERFACE_APIs
/*=======================================================================================
 * RTL8651 External Network Interface APIs:
 *	1. Static Functions
 *
 *	2. Internal APIs
 *
 *	3. External APIs
 *
 *=======================================================================================*/



int8 _rtl8651_extIpAddr(ipaddr_t ipAddr, uint32 *netIdx) {
	rtl8651_tblDrv_ipIntfEntry_t * ipIntfPtr;
	rtl8651_tblDrv_ipIntfIpAddrEntry_t * ipAddrPtr;
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	uint32 i;
	
	netIfPtr = &DrvTbl.netIface[0];
	for(i=0; i<rtl8651_tblDrvPara.networkIntfTableSize; i++, netIfPtr++) {
		if (netIfPtr->valid == 0)	continue;
		if (DrvTbl.vlan[rtl8651_vlanTableIndex(netIfPtr->vid)].internal == 1 &&
			netIfPtr->linkLayerType != RTL8651_LL_LIPUNNUMBERED)
			continue;
		SLIST_FOREACH(ipIntfPtr, &(netIfPtr->IpHead), nextIp) {
			assert(SLIST_FIRST(&ipIntfPtr->ipAddr));
			if (netIfPtr->linkLayerType == RTL8651_LL_LIPUNNUMBERED &&
				ipIntfPtr->lanSideExternal == 0)
				continue;
			SLIST_FOREACH(ipAddrPtr, &ipIntfPtr->ipAddr, nextIpAddr)
				if (ipAddrPtr->ipAddr == ipAddr) {
					if (netIdx)
						*netIdx = i;
					return TRUE;
			}
		}
	}
	return FALSE;//No such ip address appear in network address list
}

int8 _rtl8651_extNetIpAddr(ipaddr_t ipAddr, uint32 *netIdx) {
	rtl8651_tblDrv_ipIntfEntry_t * ipIntfPtr;
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	uint32 i;

	netIfPtr = &DrvTbl.netIface[0];
	for(i=0; i<rtl8651_tblDrvPara.networkIntfTableSize; i++, netIfPtr++) {
		if (netIfPtr->valid == 0) 	continue;
		if (DrvTbl.vlan[rtl8651_vlanTableIndex(netIfPtr->vid)].internal == 1 &&
			netIfPtr->linkLayerType != RTL8651_LL_LIPUNNUMBERED) 
			continue;
		SLIST_FOREACH(ipIntfPtr, &(netIfPtr->IpHead), nextIp) {
			if (netIfPtr->linkLayerType == RTL8651_LL_LIPUNNUMBERED &&
				ipIntfPtr->lanSideExternal == 0)
				continue;					
			if ((ipAddr&ipIntfPtr->ipMask) == (ipIntfPtr->ipAddr.slh_first->ipAddr&ipIntfPtr->ipMask)) {
				if (netIdx) *netIdx = i;
						return TRUE;
			}
		}
	}
	return FALSE;//No such ip address appear in network address list
}





int8 _rtl8651_getVIDByGWIpAddr(ipaddr_t gwIpAddr, uint32 *vid) {
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	rtl8651_tblDrv_ipIntfEntry_t * ipIntfPtr;
	rtl8651_tblDrv_ipIntfIpAddrEntry_t *ipPtr;
	uint32 netIdx;

	netIfPtr = &DrvTbl.netIface[0];
	for(netIdx=0; netIdx<rtl8651_tblDrvPara.networkIntfTableSize; netIdx++,netIfPtr++) {
		if (netIfPtr->valid == 0)	continue;
		SLIST_FOREACH(ipIntfPtr, &(netIfPtr->IpHead), nextIp)
			SLIST_FOREACH(ipPtr, &ipIntfPtr->ipAddr, nextIpAddr) 
				if (ipPtr->ipAddr == gwIpAddr) {
					*vid = netIfPtr->vid;
					return SUCCESS;
				}
	}
	return FAILED;
}


int8 _rtl8651_intIpAddr(ipaddr_t ipAddr) {
	rtl8651_tblDrv_ipIntfEntry_t * ipIntfPtr;
	rtl8651_tblDrv_ipIntfIpAddrEntry_t * ipAddrPtr;
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	uint32 netIdx;

	netIfPtr = &DrvTbl.netIface[0];
	for(netIdx=0; netIdx<rtl8651_tblDrvPara.networkIntfTableSize; netIdx++, netIfPtr++) {
		if (netIfPtr->valid == 0)	continue;
		if (DrvTbl.vlan[rtl8651_vlanTableIndex(netIfPtr->vid)].internal == 1)
			SLIST_FOREACH(ipIntfPtr, &(netIfPtr->IpHead), nextIp) {
				assert(SLIST_FIRST(&ipIntfPtr->ipAddr));
				if (ipIntfPtr->lanSideExternal == 1)
					continue;
				SLIST_FOREACH(ipAddrPtr, &ipIntfPtr->ipAddr, nextIpAddr) 
					if (ipAddr == ipAddrPtr->ipAddr)
						return TRUE;
			}
	}
	return FALSE;
}

int8 _rtl8651_intNetIpAddr(ipaddr_t ipAddr) {
	
	rtl8651_tblDrv_routeTable_t *rt;
	rtl8651_tblDrv_ipIntfEntry_t * ipIntfPtr;
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	uint32 netIdx;

	/*
		We check routing entry first because there might be routing entries
		indicate some subnets being internal one.
	*/
	rt = _rtl8651_getRoutingEntry(ipAddr);
	if (	(rt) &&
		(rt->valid) &&
		(rt->process != RT_CPU) &&
		(rt->process != RT_DROP) &&
		(DrvTbl.vlan[rtl8651_vlanTableIndex(rt->dvid)].valid == TRUE) &&
		(DrvTbl.vlan[rtl8651_vlanTableIndex(rt->dvid)].internal == TRUE))
	{
		return TRUE;
	}

	

	netIfPtr = &DrvTbl.netIface[0];
	for(netIdx=0; netIdx<rtl8651_tblDrvPara.networkIntfTableSize; netIdx++, netIfPtr++) {
		if (netIfPtr->valid == 0)	continue;
		if (DrvTbl.vlan[rtl8651_vlanTableIndex(netIfPtr->vid)].internal == 1)
			SLIST_FOREACH(ipIntfPtr, &(netIfPtr->IpHead), nextIp) {
				assert(SLIST_FIRST(&ipIntfPtr->ipAddr));
				if (ipIntfPtr->lanSideExternal == 1) 
					continue;
				if ((ipAddr&ipIntfPtr->ipMask) == (ipIntfPtr->ipAddr.slh_first->ipAddr&ipIntfPtr->ipMask))
					return TRUE;
			}
	}
	return FALSE;
}


#define RTL8651_NAT_MAPPING_APIs
/*=======================================================================================
 * RTL8651 NAT Mapping APIs:
 *	1. Static Functions
 *		. _rtl8651_isNatMapping()
 *	2. External APIs
 *		. rtl8651_addNatMapping()
 *		. rtl8651_delNatMapping()
 *=======================================================================================*/
static int8 _rtl8651_isNatMapping(ipaddr_t *extip, ipaddr_t *intip) {
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_natEntry_t *nat_t;
	uint32 netIdx;

	assert(extip || intip);
	netif_t = &DrvTbl.netIface[0];
	for(netIdx=0; netIdx<rtl8651_tblDrvPara.networkIntfTableSize; netIdx++, netif_t++) {
		if (netif_t->valid == 0)	
			continue;
		SLIST_FOREACH(nat_t, &netif_t->nextNat, nextNatEntry) {
			if (extip && *extip != 0 && *extip != nat_t->globalIp)
			continue;
			if (intip && *intip != 0 && *intip != nat_t->localIp)
			continue;
			if (extip) *extip = nat_t->globalIp;
			if (intip) *intip = nat_t->localIp;
		        return TRUE;
	       }
	}
	return FALSE;
}

#if 0
rtl8651_tblDrv_natEntry_t *
_rtl8651_getNatInternalMappingEntry(ipaddr_t intIpAddr) {
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_ipIntfEntry_t *ipintf_t;
	rtl8651_tblDrv_natEntry_t * natPtr;
	uint32 netIdx;

	netif_t = &DrvTbl.netIface[0];
	for(netIdx=0; netIdx<rtl8651_tblDrvPara.networkIntfTableSize; netIdx++, netif_t++) {
		if (netif_t->valid == 0 )
			continue;
		SLIST_FOREACH(natPtr, &(netif_t->nextNat), nextNatEntry) {
			if (natPtr->localIp == intIpAddr)
				return natPtr;
		}
		if (netif_t->linkLayerType == RTL8651_LL_LIPUNNUMBERED) {
			SLIST_FOREACH(ipintf_t, &netif_t->IpHead, nextIp)
				if ((ipintf_t->lanSideExternal == 1) &&
					(ipintf_t->ipAddr.slh_first->ipAddr != intIpAddr) &&
					(ipintf_t->ipMask&ipintf_t->ipAddr.slh_first->ipAddr)==(ipintf_t->ipMask&intIpAddr)) {
					lanSideNatMapping.globalIp = intIpAddr;
					lanSideNatMapping.localIp = intIpAddr;
//rtlglue_printf("lanSide(outbound): %x, %x, find: %x\n", ipintf_t->ipMask, ipintf_t->ipAddr.slh_first->ipAddr, intIpAddr);
					return &lanSideNatMapping;
				}		
		}	
	}
	return NULL;
}
#else
rtl8651_tblDrv_natEntry_t *
_rtl8651_getNatInternalMappingEntry(ipaddr_t intIpAddr,struct rtl_pktHdr *phdr) {
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_ipIntfEntry_t *ipintf_t;
	rtl8651_tblDrv_natEntry_t * natPtr;
	uint32 netIdx;

	netif_t = &DrvTbl.netIface[0];
	for(netIdx=0; netIdx<rtl8651_tblDrvPara.networkIntfTableSize; netIdx++, netif_t++) {
		if (netif_t->valid == 0 )
			continue;
		SLIST_FOREACH(natPtr, &(netif_t->nextNat), nextNatEntry) {
			if (natPtr->localIp == intIpAddr)
				return natPtr;
		}
		if (netif_t->linkLayerType == RTL8651_LL_LIPUNNUMBERED) {
			SLIST_FOREACH(ipintf_t, &netif_t->IpHead, nextIp)
				if ((ipintf_t->lanSideExternal == 1) &&
					(ipintf_t->ipAddr.slh_first->ipAddr != intIpAddr) &&
					(ipintf_t->ipMask&ipintf_t->ipAddr.slh_first->ipAddr)==(ipintf_t->ipMask&intIpAddr)) {
					lanSideNatMapping.globalIp = intIpAddr;
					lanSideNatMapping.localIp = intIpAddr;
//rtlglue_printf("lanSide(outbound): %x, %x, find: %x\n", ipintf_t->ipMask, ipintf_t->ipAddr.slh_first->ipAddr, intIpAddr);
					return &lanSideNatMapping;
				}		
		}	
	}
	if (    (phdr->ph_unnumber & 0x1)
		 && (   ((phdr->ph_unnumber & 0x2    )/*wan->lan*/ && (phdr->ph_dip==(uint32)intIpAddr))
		      || (((phdr->ph_unnumber & 0x2)==0)/*lan->wan*/ && (phdr->ph_sip==(uint32)intIpAddr))
		    )
		){
		//rtlglue_printf("enter %s %d\n",__FUNCTION__,__LINE__);
		memset(&unnumNatEntry,0,sizeof(rtl8651_tblDrv_natEntry_t));
		unnumNatEntry.globalIp = intIpAddr;
		unnumNatEntry.localIp = intIpAddr;
		return (rtl8651_tblDrv_natEntry_t *)&unnumNatEntry;
	}else
		return NULL;
}

#endif

#if 0
/* patch _rtl8651_getNatExternalMappingEntry for
 * multiplePppoe/unnumber support 2004/2/6
 */
rtl8651_tblDrv_natEntry_t * 
_rtl8651_getNatExternalMappingEntry(ipaddr_t extIpAddr) {
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_ipIntfEntry_t *ipintf_t;
	rtl8651_tblDrv_natEntry_t * natPtr;
	uint32 netIdx;

	netif_t = &DrvTbl.netIface[0];
	for(netIdx=0; netIdx<rtl8651_tblDrvPara.networkIntfTableSize; netIdx++, netif_t++) {
		if (netif_t->valid == 0)
			continue;		
		SLIST_FOREACH(natPtr, &(netif_t->nextNat), nextNatEntry) {
			if (natPtr->globalIp == extIpAddr)
				return natPtr;
		}
		if (netif_t->linkLayerType == RTL8651_LL_LIPUNNUMBERED) {
			struct rtl8651_tblDrv_ipIntfIpAddrEntry_s  *gwIp;
			SLIST_FOREACH(ipintf_t, &netif_t->IpHead, nextIp){
				if (ipintf_t->lanSideExternal == 1){ 
					SLIST_FOREACH(gwIp, &ipintf_t->ipAddr, nextIpAddr){
						if(gwIp->ipAddr==extIpAddr) //equals to one of gateway's IP. Can't be NAT...
							return NULL;
					}
					//Not gateways' IP
					if((ipintf_t->ipMask&ipintf_t->ipAddr.slh_first->ipAddr)==(ipintf_t->ipMask&extIpAddr)) {
					lanSideNatMapping.globalIp = extIpAddr;
					lanSideNatMapping.localIp = extIpAddr;
//rtlglue_printf("lanSide(inbound): %x, %x, find: %x\n", ipintf_t->ipMask, ipintf_t->ipAddr.slh_first->ipAddr, extIpAddr);
					
					return &lanSideNatMapping;
				}
		         }	
	           }
	     }	
	}
	return NULL;
}
#else
rtl8651_tblDrv_natEntry_t * 
_rtl8651_getNatExternalMappingEntry(ipaddr_t extIpAddr,struct rtl_pktHdr *phdr) {
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_ipIntfEntry_t *ipintf_t;
	rtl8651_tblDrv_natEntry_t * natPtr;
	uint32 netIdx;

	netif_t = &DrvTbl.netIface[0];
	for(netIdx=0; netIdx<rtl8651_tblDrvPara.networkIntfTableSize; netIdx++, netif_t++) {
		if (netif_t->valid == 0)
			continue;		
		SLIST_FOREACH(natPtr, &(netif_t->nextNat), nextNatEntry) {
			if (natPtr->globalIp == extIpAddr)
				return natPtr;
		}
		if (netif_t->linkLayerType == RTL8651_LL_LIPUNNUMBERED) {
			struct rtl8651_tblDrv_ipIntfIpAddrEntry_s  *gwIp;
			SLIST_FOREACH(ipintf_t, &netif_t->IpHead, nextIp){
				if (ipintf_t->lanSideExternal == 1){ 
					SLIST_FOREACH(gwIp, &ipintf_t->ipAddr, nextIpAddr){
						if(gwIp->ipAddr==extIpAddr) //equals to one of gateway's IP. Can't be NAT...
							return NULL;
					}
					//Not gateways' IP
					if((ipintf_t->ipMask&ipintf_t->ipAddr.slh_first->ipAddr)==(ipintf_t->ipMask&extIpAddr)) {
					lanSideNatMapping.globalIp = extIpAddr;
					lanSideNatMapping.localIp = extIpAddr;
//rtlglue_printf("lanSide(inbound): %x, %x, find: %x\n", ipintf_t->ipMask, ipintf_t->ipAddr.slh_first->ipAddr, extIpAddr);
					
					return &lanSideNatMapping;
				}
		         }	
	           }
	     }	
	}

	if (    (phdr->ph_unnumber & 0x1)
		 && (   ((phdr->ph_unnumber & 0x2    )/*wan->lan*/ && (phdr->ph_dip==(uint32)extIpAddr))
		      ||(((phdr->ph_unnumber & 0x2)==0)/*lan->wan*/ && (phdr->ph_sip==(uint32)extIpAddr))
		    )
		){
		//rtlglue_printf("enter %s %d\n",__FUNCTION__,__LINE__);
		memset(&unnumNatEntry,0,sizeof(rtl8651_tblDrv_natEntry_t));
		unnumNatEntry.globalIp = extIpAddr;
		unnumNatEntry.localIp = extIpAddr;
		return (rtl8651_tblDrv_natEntry_t *)&unnumNatEntry;
	}else
		return NULL;
}
#endif

/*
@func int32 | rtl8651_addNatMapping | Add a NAPT Mapping.
@parm ipaddr_t | extIpAddr | External IP address.
@parm ipaddr_t | intIpAddr | Internal IP address.
@rvalue TBLDRV_ENATIPNOTEXTERNALIP | The specified NAT IP address is not an External IP address.
@rvalue TBLDRV_ENATIPNOTINTERNALIP | The specified NAT Internal IP is not an Internal IP address.
@rvalue TBLDRV_EDUPLOCATENATIP | The specified NAT IP address already maps to an Internal IP address.
@rvalue TBLDRV_ENATIPCANNOTBENAPTIP | The specified NAT IP address cannot be a NAPT IP address.
@rvalue TBLDRV_ENOFREEBUFFER | Out of memory buffer.
@rvalue TBLDRV_EENTRYALREADYEXIST | NAT pair already exists.
@comm
RTL8651 supports multiple NAT configuration. Note that one NAT IP address only can map to one Internal IP address.
*/
int32 rtl8651_addNatMapping(ipaddr_t extIpAddr, ipaddr_t intIpAddr) {
	rtl8651_tblDrv_natEntry_t * tempNatMappingPtr;
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_natEntry_t *nat1_t, *nat2_t=NULL;
	rtl8651_tblDrv_arpEntry_t *arp_t;
	uint32 netIdx;
	 
	rtlglue_drvMutexLock();
	/* Specified external IP address is not one of external IP interface addresses */
	LOCK_ERROR_CHECK((_rtl8651_extIpAddr(extIpAddr, &netIdx) == FALSE), TBLDRV_ENATIPNOTEXTERNALIP);
	/* Specified internal IP address is not host belong to internal network interface */
	LOCK_ERROR_CHECK((_rtl8651_intNetIpAddr(intIpAddr) == FALSE), TBLDRV_ENATIPNOTINTERNALIP);
	LOCK_ERROR_CHECK((_rtl8651_isNatMapping(&extIpAddr, NULL) == TRUE), TBLDRV_EDUPLOCATENATIP);
	LOCK_ERROR_CHECK((_rtl8651_isNaptMapping(extIpAddr) == TRUE), TBLDRV_ENATIPCANNOTBENAPTIP);
	netif_t = &DrvTbl.netIface[netIdx];
	LOCK_ERROR_CHECK((DrvTbl.freeList.natMapping.slh_first==NULL), TBLDRV_ENOFREEBUFFER);
	assert(netif_t->valid == 1);
	SLIST_FOREACH(tempNatMappingPtr, &(netif_t->nextNat), nextNatEntry) {
		/* Defined NAT mapping with duplicated address */
		LOCK_ERROR_CHECK(
			( (tempNatMappingPtr->globalIp == extIpAddr) ||
			  ( tempNatMappingPtr->localIp == intIpAddr)
			), TBLDRV_EENTRYALREADYEXIST
		);
	}
#ifdef RTL865X_EXP_MAC_ACCESS_CONTROL
	if ((arp_t=_rtl8651_getArpEntry(NULL, intIpAddr, FALSE)) && _rtl8651_isMacAccessControlPermit(&arp_t->macInfo->macAddr)==FALSE) {
		/* DO NOT write it to ASIC */
	}
	else {
#endif /* RTL865X_EXP_MAC_ACCESS_CONTROL */
		LOCK_ERROR_CHECK(
			(_rtl8651_addIpEntry( extIpAddr, intIpAddr, IP_ATTR_NAT, NULL) == NULL),
			TBLDRV_ENOFREEBUFFER
		);
#ifdef RTL865X_EXP_MAC_ACCESS_CONTROL
	}
#endif /* RTL865X_EXP_MAC_ACCESS_CONTROL */
	
	tempNatMappingPtr = SLIST_FIRST(&DrvTbl.freeList.natMapping);
	SLIST_REMOVE_HEAD(&DrvTbl.freeList.natMapping, nextNatEntry);
	tempNatMappingPtr->globalIp = extIpAddr;
	tempNatMappingPtr->localIp = intIpAddr;
	/* Ascending order */
	SLIST_FOREACH(nat1_t, &netif_t->nextNat, nextNatEntry) 
		if (intIpAddr < nat1_t->localIp)
			break;
		else nat2_t = nat1_t;
	if (nat2_t == NULL)
		SLIST_INSERT_HEAD(&(netif_t->nextNat), tempNatMappingPtr, nextNatEntry);
	else SLIST_INSERT_AFTER(nat2_t, tempNatMappingPtr, nextNatEntry);	
	rtlglue_drvMutexUnlock();//Unlock resource
	return SUCCESS;
}

/*
@func int32 | rtl8651_delNatMapping | Remove a NAPT mapping.
@parm ipaddr_t | extip | External IP address.
@parm ipaddr_t | intip | Internal IP address.
@rvalue SUCCESS | The specified NAT pair was successfully removed.
@rvalue TBLDRV_ENATIPNOTEXTERNALIP | The specified NAT IP address is not an External IP address.
@rvalue TBLDRV_ENATIPNOTINTERNALIP | The specified NAT Internal IP is not an Internal IP address.
@rvalue TBLDRV_EENTRYNOTFOUND | The specified NAT pair was not found.
*/
int32 rtl8651_delNatMapping(ipaddr_t extip, ipaddr_t intip) {
	rtl8651_tblDrv_natEntry_t *deletedNatMappingPtr;
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	uint32 netIdx;
	int32 retval;

	rtlglue_drvMutexLock();//Lock resource
	LOCK_ERROR_CHECK((_rtl8651_extIpAddr(extip, &netIdx) == FALSE), TBLDRV_ENATIPNOTEXTERNALIP);
	LOCK_ERROR_CHECK((_rtl8651_intNetIpAddr(intip) == FALSE), TBLDRV_ENATIPNOTINTERNALIP);
	netif_t = &DrvTbl.netIface[netIdx];
	LOCK_ERROR_CHECK((netif_t->nextNat.slh_first == NULL), TBLDRV_EENTRYNOTFOUND);
	assert(netif_t->valid == 1);
	SLIST_FOREACH(deletedNatMappingPtr, &(netif_t->nextNat), nextNatEntry)
		if (deletedNatMappingPtr->globalIp == extip && deletedNatMappingPtr->localIp == intip)
				break;
	LOCK_ERROR_CHECK((deletedNatMappingPtr==NULL), TBLDRV_EENTRYNOTFOUND);
	SLIST_REMOVE(&(netif_t->nextNat), deletedNatMappingPtr, rtl8651_tblDrv_natEntry_s, nextNatEntry);
	retval = _rtl8651_delIpEntry(IP_ATTR_NAT, extip, intip);
	SLIST_INSERT_HEAD(&DrvTbl.freeList.natMapping, deletedNatMappingPtr, nextNatEntry);
	rtlglue_drvMutexUnlock();//Unlock resource
	return SUCCESS;
}



#define RTL8651_NAT_MAPPING_APIs
/*=======================================================================================
 * RTL8651 NAPT APIs:
 *	1. Static Functions
 *		. _rtl8651_isNaptMapping()
 *	2. Internal APIs
 *		. _rtl8651_getNatInternalMappingEntry()
 *		. _rtl8651_getNatExternalMappingEntry()
 *	3. External APIs
 *		. rtl8651_addNaptMapping()
 *		. rtl8651_delNaptMapping()
 *=======================================================================================*/
int8 _rtl8651_isNaptMapping(ipaddr_t extip) {
	rtl8651_tblDrv_ipEntry_t *ipEntry_t=DrvTbl.iptbl;
	uint32 entry;

	for(entry=0; entry<RTL8651_IPTABLE_SIZE; entry++, ipEntry_t++) {
		if (ipEntry_t->valid == 0)
			continue;
		if (ipEntry_t->type == IP_ATTR_NAPT && ipEntry_t->extip == extip)
			return TRUE;
	}
	return FALSE;
}

static int32 _rtl8651_addSoftNaptToAcl(uint32 aclStart, uint32 aclEnd)
{
	_rtl8651_tblDrvAclRule_t rule;
	
	assert(aclEnd < RTL8651_ACLTBL_SIZE);
	if (aclStart > aclEnd)	return 0;
	bzero(&rule, sizeof(_rtl8651_tblDrvAclRule_t));
	rule.ruleType_			= RTL8651_ACL_IP;
	rule.actionType_		= RTL8651_ACL_CPU;
	rule.dstIpAddr_		= softNaptIp;
	rule.dstIpAddrMask_	= 0xffffffff;
	rtl8651_setAsicAclRule(aclStart, &rule);
	rtl8651_setAsicAclRule(RTL8651_ACLTBL_PPPOEPASSTHRU, &rule);
	return 1;
}

static int32 _rtl8651_delNaptMapping(ipaddr_t naptip) {
	if(_rtl8651_extIpAddr(naptip, NULL) == FALSE)
		return TBLDRV_ENAPTIPNOTEXTIP;
	if (_rtl8651_delIpEntry(IP_ATTR_NAPT, naptip, 0)== SUCCESS) {
		_rtl8651_delDmzHost(_rtl8651_DefaultDialSessionId, naptip);
		return SUCCESS;
	}
	return TBLDRV_EENTRYNOTFOUND;
}


static int32 _rtl8651_addNaptMapping(ipaddr_t naptip, rtl8651_tblDrv_ipEntry_t **ie_t)
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_ipEntry_t *ipEntry_t;
	uint32 netIdx;
	
	if (_rtl8651_extIpAddr(naptip, &netIdx) == FALSE) 
		return TBLDRV_ENAPTIPNOTEXTIP;
	if (_rtl8651_isNatMapping(&naptip, NULL) == TRUE)
		return TBLDRV_ENAPTIPCANNOTBENAT;
		
	/* ok! try to add naptip to IP Table, and its nexthop is set to default gateway's nexthop */
	netif_t = &DrvTbl.netIface[netIdx];
	if ((ipEntry_t=_rtl8651_addIpEntry(naptip, 0, IP_ATTR_NAPT, netif_t)) == NULL)
		return TBLDRV_ENOFREEBUFFER;
	if (ie_t)  *ie_t = ipEntry_t;
	return SUCCESS;
}


/*
@func int32 | rtl8651_addNaptMapping | Add a NAPT IP address.
@parm ipaddr_t | extIpAddr | External IP address.
@rvalue SUCCESS | Success.
@rvalue TBLDRV_ENAPTIPNOTEXTIP | The specified IP address is not an external IP address.
@rvalue TBLDRV_ENAPTIPCANNOTBENAT | The specified IP address is already a NAT IP address.
@rvalue TBLDRV_ENOFREEBUFFER | Out of memory buffer.
@comm
Currently, the RTL8651 only supports one NAPT IP address. 
*/
int32 rtl8651_addNaptMapping(ipaddr_t naptip) 
{
	int32 retval = SUCCESS;

	rtlglue_drvMutexLock();
	pppoeDefaultGlobalIpAddr = naptip;
	if ( def_napt_t != NULL )  {
		retval = TBLDRV_EDEFAULTNAPTIPEXIST;
		goto out;
	}
	retval = _rtl8651_addNaptMapping(naptip, &def_napt_t);
	/*
	 *  Louis note: In current version, ONLY ONE nexthop is needed.
	 *              Thus, we don't need complex algorithm.
	 *              Finally, holly Chhuang will re-write this code.
	 */
//	_try_to_setAsicNexthopTable( extIpAddr, ipEntry_t );
//	_rtl8651_setServerPortExternalIp(0, extIpAddr);

out:
	rtlglue_drvMutexUnlock();
	return retval;
}

/* 
    2005.6.7
    The following is the reason why we create this "redundant" API.
    The difference between this API and rtl8651_addNaptMappingExt() is that...this API won't add napt ip to ASIC.

    (1) When multiple PPPoE, we don't add second napt ip to ASIC ip table 
         => call rtl8651_addDriverNaptMapping().
    (2) When multiple DMZ-host, we add second, third, .. napt ip to ASIC ip table
         => call rtl8651_addNaptMappingExt().
    (3) Because now no customer uses multiple pppoe, so we choose this newly-added API to be used
         in multiple PPPoE.
*/
int32 rtl8651_addDriverNaptMapping(ipaddr_t naptip) {
	int32 retval = SUCCESS;
	rtlglue_drvMutexLock();
	softNaptIp = naptip;
	_rtl8651_arrangeAllACL();
	rtlglue_drvMutexUnlock();
	return retval;
}


int32 rtl8651_addNaptMappingExt(ipaddr_t naptip) {
	int32 retval = SUCCESS;
	rtlglue_drvMutexLock();
	if ((retval=_rtl8651_addNaptMapping(naptip, NULL)) == SUCCESS) {
		softNaptIp = naptip;
		_rtl8651_arrangeAllACL();
	}

	rtlglue_drvMutexUnlock();
	return retval;
}

/*
@func int32 | rtl8651_delNaptMapping | Remove a NAPT IP address.
@parm ipaddr_t | extIpAddr | NAPT IP address.
@rvalue TBLDRV_ENAPTIPNOTEXTIP | The specified IP address is not a NAPT IP address.
@rvalue TBLDRV_EENTRYNOTFOUND | The specified NAPT IP address was not found.
@comm
Currently, RTL8651 only supports on NAPT IP address. If the NAPT IP address is removed, the system
will have no NAPT capability.
*/
int32 rtl8651_delNaptMapping(ipaddr_t extIpAddr) {
	uint32 retval=SUCCESS;

	if (extIpAddr == 0)
		return TBLDRV_EINVALIDINPUT;
	rtlglue_drvMutexLock();

#if 0
	_rtl8651_delNaptMapping(extIpAddr);
	
	if (extIpAddr == softNaptIp) {
		softNaptIp = 0;
		_rtl8651_arrangeAllACL();
		_rtl8651_removeNaptConnectionByExtIP(extIpAddr);
	}
	else
	{
		//_rtl8651_setServerPortExternalIp(extIpAddr, 0);
		 /*_rtl8651_flushNaptConnection(); rupert*/
		_rtl8651_removeNaptConnectionByExtIP(extIpAddr);
		rtl8651_flushNaptServerPortbyExtIp(extIpAddr); //tony	
		if (def_napt_t && def_napt_t->extip == extIpAddr)
			def_napt_t = NULL;

	}
#else

	if (extIpAddr == softNaptIp) {
		softNaptIp = 0;
		_rtl8651_arrangeAllACL();
		_rtl8651_removeNaptConnectionByExtIP(extIpAddr);
	} else if ((retval = _rtl8651_delNaptMapping(extIpAddr)) == SUCCESS) {
		_rtl8651_removeNaptConnectionByExtIP(extIpAddr);
		rtl8651_flushNaptServerPortbyExtIp(extIpAddr);
		if (def_napt_t && def_napt_t->extip == extIpAddr)
			def_napt_t = NULL;
	}
#endif

	rtlglue_drvMutexUnlock();
	return retval;
}




#define RTL8651_DMZ_HOST_APIs

static int32 _rtl8651_delDmzHost(uint32 dsid,ipaddr_t naptIp) {
	uint32 i;

	#if 0
	UNLOCK_ERROR_CHECK(
		( (rtl8651DmzHost[dsid].naptIp==0) || (rtl8651DmzHost[dsid].naptIp!=naptIp) ), 
		TBLDRV_EDMZHOSTNOTFOUND
	);
	#endif

	/* For multiple DMZ-host spec. */
	for (i = 0; i < dmzHostNumInOneSession; i++) {
		if (rtl8651DmzHost[dsid].naptIp[i] == naptIp) {
			rtl8651DmzHost[dsid].naptIp[i] = 0;
			return SUCCESS;
		}
	}
	return TBLDRV_EDMZHOSTNOTFOUND;	
}


int32 _rtl8651_getDmzHost(uint32 dsid,ipaddr_t naptIp, ipaddr_t *dmzHostIp) {

	uint32 i;
	
	// rtlglue_drvMutexLock();
	#if 0
	LOCK_ERROR_CHECK(
		( (rtl8651DmzHost[dsid].naptIp==0) || (rtl8651DmzHost[dsid].naptIp!=naptIp) ),
  		TBLDRV_EDMZHOSTNOTFOUND
	);
	#endif

	/* For multiple DMZ-host spec. */
	for (i = 0; i < dmzHostNumInOneSession; i++) {
		if (rtl8651DmzHost[dsid].naptIp[i] == naptIp) {			
			if (dmzHostIp)
				*dmzHostIp = rtl8651DmzHost[dsid].dmzHost[i];
			return SUCCESS;
		}
	}	
	// rtlglue_drvMutexUnlock();
	return TBLDRV_EDMZHOSTNOTFOUND;
}

int32 _rtl8651_fromDmzHost(uint32 dsid, ipaddr_t dmzHostIp, ipaddr_t *naptIp) {

	uint32 i;
	// rtlglue_drvMutexLock();
	#if 0
	LOCK_ERROR_CHECK(
		( (rtl8651DmzHost[dsid].naptIp==0) || (rtl8651DmzHost[dsid].dmzHost!=dmzHostIp) ),
  		TBLDRV_EDMZHOSTNOTFOUND
	);
	#endif
	/* For multiple DMZ-host spec. */
	for (i = 0; i < dmzHostNumInOneSession; i++) {
		if (rtl8651DmzHost[dsid].dmzHost[i] ==dmzHostIp) {
			if (naptIp)
				*naptIp = rtl8651DmzHost[dsid].naptIp[i];
			return SUCCESS;
		}
	}
	// rtlglue_drvMutexUnlock();
	return TBLDRV_EDMZHOSTNOTFOUND;
}


/*
@func int32 | rtl8651_addDmzHost | Add a DMZ host.
@parm ipaddr_t | naptIp | External NAPT IP address.
@parm ipaddr_t | dmzHostIp | Private host IP address.
@rvalue SUCCESS | The DMZ host was added successfully.
@rvalue TBLDRV_EDMZHOSTNEEDNAPTIP | The specified external IP addresss should be a NAPT IP address.
@rvalue TBLDRV_EDMZHOSTSHOULDBEINTIP | The specified DMZ host IP address should be an internal IP address.
@rvalue TBLDRV_EDMZHOSTCANNOTBEGWIP | The specified DMZ host IP address should not be gateway's IP address.
@rvalue TBLDRV_EENTRYALREADYEXIST | The DMZ host already exists.
@comm
Only one DMZ host is allowed in whole system and the DMZ host should be bound with NAPT IP address.
*/
int32 rtl8651a_addDmzHost(uint32 SessionID,ipaddr_t naptIp, ipaddr_t dmzHostIp) {
	uint32 dsid, i;
	rtlglue_drvMutexLock();
	dsid=_rtl8651_SessionIDToDSID(SessionID);
	/* Check if naptIp is an external NAPT IP */
	if (dsid==_rtl8651_DefaultDialSessionId)
	{
		LOCK_ERROR_CHECK((_rtl8651_isNaptMapping(naptIp)==FALSE), TBLDRV_EDMZHOSTNEEDNAPTIP);
		LOCK_ERROR_CHECK((_rtl8651_intNetIpAddr(dmzHostIp)==FALSE), TBLDRV_EDMZHOSTSHOULDBEINTIP);
		LOCK_ERROR_CHECK((_rtl8651_intIpAddr(dmzHostIp)==TRUE), TBLDRV_EDMZHOSTCANNOTBEGWIP);
	}
	for (i = 0; i < dmzHostNumInOneSession; i++)
		LOCK_ERROR_CHECK((rtl8651DmzHost[dsid].naptIp[i]==naptIp), TBLDRV_EENTRYALREADYEXIST);

	for (i = 0; i < dmzHostNumInOneSession; i++) {
		if (rtl8651DmzHost[dsid].naptIp[i] == 0) {
			rtl8651DmzHost[dsid].naptIp[i] = naptIp;
			rtl8651DmzHost[dsid].dmzHost[i] = dmzHostIp;
			rtlglue_drvMutexUnlock();
			return SUCCESS;
		}
	}
	rtlglue_drvMutexUnlock();
	return TBLDRV_EDMZHOSTFULL;
}

/*
@func int32 | rtl8651_delDmzHost | Remove DMZ host.
@parm ipaddr_t | naptIp | External NAPT IP address.
@rvalue SUCCESS | The DMZ host was added successfully.
@rvalue TBLDRV_EDMZHOSTNOTFOUND | The specified NAPT IP address was not found.
*/
int32 rtl8651a_delDmzHost(uint32 SessionID,ipaddr_t naptIp) {
	int32 retval;
	uint32 dsid;
	rtlglue_drvMutexLock();
	dsid=_rtl8651_SessionIDToDSID(SessionID);
	retval = _rtl8651_delDmzHost(dsid,naptIp);
	rtlglue_drvMutexUnlock();
	return retval;
}



#define RTL8651_SERVERP_PORT_APIs
/*=======================================================================================
 * RTL8651 Server Port APIs:
 *	1. Static Functions
 *		. _rtl8651_servPortToAclCount()
 *	2. Internal APIs
 *		. _rtl8651_getOutsideNaptServerPortMapping()
 *		. _rtl8651_getInsideNaptServerPortMapping()
 *	3. External APIs
 *		. rtl8651_addNaptServerPortMapping()
 *		. rtl8651_delNaptServerPortMapping()
 *=======================================================================================*/
#if 0
static uint32 _rtl8651_servPortToAclCount() {
	rtl8651_tblDrv_naptServerPortEntry_t *servPort;
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	uint32 i, count = 0;

	netIfPtr = &DrvTbl.netIface[0];
	for(i=0;i< rtl8651_tblDrvPara.networkIntfTableSize; i++,netIfPtr++) {
		if (netIfPtr->valid == 0)
			continue;
		SLIST_FOREACH(servPort, &netIfPtr->nextNaptServerPort, nextServerPort) {
			if (servPort->toAsic == 1 || servPort->globalIp == 0)	continue;
			if (servPort->tcp == 1)		count++;
			if (servPort->udp == 1)		count++;
		}		
	}
	return count;
}
#endif

rtl8651_tblDrv_naptServerPortEntry_t *  
_rtl8651_getOutsideNaptServerPortMapping(int8 isTcp, ipaddr_t extIpAddr, uint16 extPort) {
	rtl8651_tblDrv_naptServerPortEntry_t * naptServerPortPtr;
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	uint32 netIdx;

	for(netIdx=0,netIfPtr=&DrvTbl.netIface[0]; netIdx<rtl8651_tblDrvPara.networkIntfTableSize; netIdx++,netIfPtr++)
		if (netIfPtr->valid == 1) {
			SLIST_FOREACH(naptServerPortPtr, &(netIfPtr->nextNaptServerPort), nextServerPort) {
				if(naptServerPortPtr->globalIp == extIpAddr && (naptServerPortPtr->globalPort <= extPort && naptServerPortPtr->endGlobalPort >= extPort))
					if ((isTcp && naptServerPortPtr->tcp) ||
						(!isTcp && naptServerPortPtr->udp))
					return naptServerPortPtr;
			}
		}
	return NULL;
}

rtl8651_tblDrv_naptServerPortEntry_t *  
_rtl8651_getInsideNaptServerPortMapping(int8 isTcp, ipaddr_t intIpAddr, uint16 intPort) {
	rtl8651_tblDrv_naptServerPortEntry_t * naptServerPortPtr;
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	uint32 netIdx;

	for(netIdx=0,netIfPtr=&DrvTbl.netIface[0]; netIdx<rtl8651_tblDrvPara.networkIntfTableSize; netIdx++, netIfPtr++)
		if (netIfPtr->valid == 1) {
			SLIST_FOREACH(naptServerPortPtr, &(netIfPtr->nextNaptServerPort), nextServerPort) {
				if(naptServerPortPtr->localIp== intIpAddr && (naptServerPortPtr->localPort <= intPort && naptServerPortPtr->endLocalPort >= intPort))
					if ((isTcp && naptServerPortPtr->tcp) ||
						(!isTcp && naptServerPortPtr->udp))
					return naptServerPortPtr;
			}
		}
	return NULL;
}


#if 0
static void _rtl8651_setServerPortExternalIp(ipaddr_t orgNaptIp, ipaddr_t newNaptIp) {
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_naptServerPortEntry_t *servport_t;
	uint32 netIdx;
	int8 arrangeAcl = FALSE;

	netif_t = &DrvTbl.netIface[0];
	for(netIdx=0; netIdx<rtl8651_tblDrvPara.networkIntfTableSize; netIdx++,netif_t++) {
		if (netif_t->valid == 0)	continue;
		SLIST_FOREACH(servport_t, &(netif_t->nextNaptServerPort), nextServerPort)
			if (servport_t->globalIp == orgNaptIp) {
				servport_t->globalIp = newNaptIp;
				if (servport_t->toAsic == 1)
					convert_setAsicServerPortTable(servport_t->asicTablePos, servport_t->globalIp, servport_t->globalPort, servport_t->localIp, servport_t->localPort);
				else arrangeAcl = TRUE;				
			}
	}
	if (arrangeAcl == TRUE)
		_rtl8651_arrangeAllACL();
}
#endif

#if 0
static void _rtl8651_flushNaptServerPortTable(rtl8651_tblDrv_networkIntfTable_t *netif_t) {
	rtl8651_tblDrv_naptServerPortEntry_t *servPort_t;
	int8 arrangeAcl = FALSE;

	while((servPort_t=SLIST_FIRST(&netif_t->nextNaptServerPort))) {
		SLIST_REMOVE_HEAD(&netif_t->nextNaptServerPort, nextServerPort);
		SLIST_INSERT_HEAD(&DrvTbl.freeList.serverPort, servPort_t, nextServerPort);
		if (servPort_t->toAsic == 1) {
			rtl8651_delAsicServerPortTable(servPort_t->asicTablePos);
		} else arrangeAcl = TRUE;
	}
	if (arrangeAcl == TRUE)
		_rtl8651_arrangeAllACL();	
}
#endif

int32 rtl8651_flushNaptServerPortbyExtIp(ipaddr_t extIp) {

	rtl8651_tblDrv_serverPortAsicArrangementEntry_t *asic_t;
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_naptServerPortEntry_t *servPort_t;
	rtl8651_tblDrv_naptServerPortEntry_t *servPort_next;
	uint32 netIdx, i, asicPos;
	//rtlglue_printf("rtl8651_flushNaptServerPortbyExtIp %u.%u.%u.%u\n",NIPQUAD(extIp));

	if (_rtl8651_extIpAddr(extIp, &netIdx) == FALSE) 
		return TBLDRV_ENOTEXTIP;
//	if (_rtl8651_isLanSideIpUnnumberedIpAddr(gip) == TRUE) 
//		return TBLDRV_EUNNUMBEREDIPNOTALLOW;

	netif_t = &DrvTbl.netIface[netIdx];
	assert(netif_t->valid == 1);
	
	// Using SLIST_REMOVE() in SLIST_FOREACH() macro causes bug!
	// Therefore, we replace the following line into while loop.
	//  SLIST_FOREACH(servPort_t, &netif_t->nextNaptServerPort, nextServerPort)	{
	servPort_t = SLIST_FIRST(&netif_t->nextNaptServerPort);
	while ( 1 )
	{
		if ( servPort_t == NULL ) break;
		servPort_next = SLIST_NEXT( servPort_t, nextServerPort );
		
		//rtlglue_printf("search ip=%u.%u.%u.%u port=%d\n",NIPQUAD(servPort_t->globalIp),servPort_t->globalPort);
		if (servPort_t->globalIp == extIp)
		{
			int nCount;

			nCount=servPort_t->endGlobalPort-servPort_t->globalPort;
			
			//rtlglue_printf("del gip=%u.%u.%u.%u gport=%d\n",NIPQUAD(extIp),servPort_t->globalPort);
			SLIST_REMOVE(&(netif_t->nextNaptServerPort), servPort_t, 
			      rtl8651_tblDrv_naptServerPortEntry_s, nextServerPort);
	
			SLIST_INSERT_HEAD(&DrvTbl.freeList.serverPort, servPort_t, nextServerPort);
			if (servPort_t->toAsic == 1) {	
				asicPos = servPort_t->asicTablePos;
				asic_t = &AsicTableUsage.serverPort[asicPos];

				for(i=0; i<=nCount; i++) {
					while(1) {
						if (asic_t->valid == 1 && asic_t->servPort_t == servPort_t)
							break;
						asicPos++; asic_t++;
					}
					assert(asicPos < RTL8651_SERVERPORTTBL_SIZE);
					rtl8651_delAsicServerPortTable(asicPos);
					asic_t->valid = 0;
				}	
				
			}

			//if (servPort_t->toAsic == 0)
			//	_rtl8651_arrangeAllACL();			

		}

		// next one
		servPort_t = servPort_next;
	}
	return SUCCESS;


}

#if 0 //debug, show all the server port entry
void _rtl8651_showNaptSeverPort(void)
{
	uint32 netIdx;
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_naptServerPortEntry_t *servPort_t;
	GET_NETWORK_INTERFACE(extIfName,&netIdx);
	netif_t = &DrvTbl.netIface[netIdx];
	rtlglue_printf("\n\n===============================\n");
	SLIST_FOREACH(servPort_t, &netif_t->nextNaptServerPort, nextServerPort) {
	rtlglue_printf("global=%u.%u.%u.%u:%d-%d local=%u.%u.%u.%u:%d-%d \n"
		,NIPQUAD(servPort_t->globalIp),servPort_t->globalPort,servPort_t->endGlobalPort
		,NIPQUAD(servPort_t->localIp),servPort_t->localPort ,servPort_t->endLocalPort);		
	}	
	rtlglue_printf("=================================\n");	
}
#endif

int32 _rtl8651_addNaptServerPort(int8 isTcp, ipaddr_t gip, uint16 gport, ipaddr_t lip, uint16 lport, uint16 nCount) {
	rtl8651_tblDrv_serverPortAsicArrangementEntry_t *asic_t, *asic_t1;
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_naptServerPortEntry_t *servPort_t;
	uint16 lastPort, startPort, endPort;
	uint32 netIdx, asicPos=0, i, freeCount;

	//rtlglue_printf("addnaptserverport ipTcp=%d,lip=%u.%u.%u.%u\n",isTcp,NIPQUAD(lip));

	
	
	if (_rtl8651_extIpAddr(gip, &netIdx) == FALSE || _rtl8651_isNaptMapping(gip) == FALSE) 
	{
		GET_NETWORK_INTERFACE(extIfName,&netIdx);
		//return TBLDRV_ENOTEXTIP;
	}

	
//	if (_rtl8651_isLanSideIpUnnumberedIpAddr(gip) == TRUE) 
//		return TBLDRV_EUNNUMBEREDIPNOTALLOW;
	if (_rtl8651_intNetIpAddr(lip) == FALSE) 
		return TBLDRV_ENOTINTIP;
	netif_t = &DrvTbl.netIface[netIdx];
	assert(netif_t->valid == 1);
	
	/* NAPT Server Port forbids the following mapping:
	 * (1) Same external pairs(external IP, external Port) map to an internal pairs
	 * (2) Same internal pairs(internal IP, internal Port) map to an external pairs
	 */
	nCount--;
	SLIST_FOREACH(servPort_t, &netif_t->nextNaptServerPort, nextServerPort) {
		/* If range match, check to see if this is a duplicate one */
		if ((servPort_t->globalIp == gip)&&(servPort_t->localIp == lip)&&
			(servPort_t->globalPort == gport)&&(servPort_t->endGlobalPort == (gport+nCount))&&
			(servPort_t->localPort == lport)&&(servPort_t->endLocalPort == (lport+nCount)))
			goto __out;

		if (((isTcp&1) == TRUE && servPort_t->tcp == 0) ||
		     ((isTcp&1) == FALSE && servPort_t->udp == 0))
			continue;

		/* check global overlapping :
			when we processing inbound flow, we get (isTcp, port, gip), we need to check if there is overlap or not  */
		if (servPort_t->globalIp == gip) {
			lastPort = gport + nCount;
			startPort = servPort_t->globalPort;
			endPort = servPort_t->endGlobalPort;
			if ((lastPort>=startPort && lastPort<=endPort) || 
				(gport>=startPort && gport<=endPort) ||
				(gport<=startPort && lastPort>=endPort)) {
				return TBLDRV_ERANGEOVERLAP;//Range overlap but not exactly same
			}
		} 
		/* check internal overlapping :
			when we processing 'FIRST' outbound flow(no flow entry in NAPT table), we get (isTcp, port, lip), we need to check if there is overlap or not  */
		if (servPort_t->localIp == lip /*&& servPort_t->globalIp == gip*/) {
			lastPort = lport + nCount;
			startPort = servPort_t->localPort;
			endPort = servPort_t->endLocalPort;
			if ((lastPort>=startPort && lastPort<=endPort) ||
				(lport>=startPort && lport<=endPort) ||
				(lport<=startPort && lastPort>=endPort)) {
				return TBLDRV_ERANGEOVERLAP;//Range overlap but not exactly same
			}
		}
	}
	
__out:

	if (servPort_t != (rtl8651_tblDrv_naptServerPortEntry_t *)NULL) {
		if ((isTcp&1) == TRUE && servPort_t->tcp == 0)
			servPort_t->tcp = 1;
		else if ((isTcp&1) == FALSE && servPort_t->udp == 0)
			servPort_t->udp = 1;
		else
			return TBLDRV_EENTRYALREADYEXIST;
		
#ifdef SERVERPORT_PARK_TCPUDP
		/* In this mode, server port with both TCP and UDP can be ASIC accelerated. */
		goto find_empty_server_port_entry;
#else/*SERVERPORT_PARK_TCPUDP*/
		if (servPort_t->toAsic == 0)
			_rtl8651_arrangeAllACL();
		return SUCCESS;
#endif/*SERVERPORT_PARK_TCPUDP*/
	}
	//it's ok to add a new one...
	if(DrvTbl.freeList.serverPort.slh_first == (rtl8651_tblDrv_naptServerPortEntry_t *)NULL)
		return TBLDRV_ENOFREEBUFFER;//Unable to insert new NAPT server port mapping entry
	servPort_t = SLIST_FIRST(&DrvTbl.freeList.serverPort);
	SLIST_REMOVE_HEAD(&DrvTbl.freeList.serverPort, nextServerPort);
	SLIST_INSERT_HEAD(&(netif_t->nextNaptServerPort), servPort_t, nextServerPort);
	servPort_t->globalIp = gip;
	servPort_t->globalPort = gport;
	servPort_t->endGlobalPort = gport+nCount;
	servPort_t->localIp = lip;
	servPort_t->localPort = lport;
	servPort_t->endLocalPort = lport+nCount;
	if((isTcp&1) == TRUE) {
		servPort_t->tcp = 1; servPort_t->udp = 0;
	}else {	servPort_t->tcp = 0; servPort_t->udp = 1; }

find_empty_server_port_entry:
	/* The the new napt server port to ASIC server port Table. If out of ASIC
	 * table or no enough free entry for new entries, write it to ACL.
	 */
	freeCount = 0; asic_t1 = NULL; 
	asic_t=&AsicTableUsage.serverPort[0];
	for(i=0; i<RTL8651_SERVERPORTTBL_SIZE; i++,asic_t++) {
		if(asic_t->valid == 0) {
			freeCount++;
			if (!asic_t1) {
				asic_t1 = asic_t; asicPos = i;
			}
		}
	}
	if( (asic_t1 && freeCount>=(nCount+1))&&
	    ((isTcp&2)==0) &&
#ifdef SERVERPORT_PARK_TCPUDP
		/* In this mode, flow can be ASIC-accelerated only when both TCP and UDP are added. */
	    servPort_t->tcp &&
	    servPort_t->udp
#else/*SERVERPORT_PARK_TCPUDP*/
		1
#endif/*SERVERPORT_PARK_TCPUDP*/
	    ) 
	{
#ifdef RTL865X_EXP_MAC_ACCESS_CONTROL
		rtl8651_tblDrv_arpEntry_t *arp_t;
		if ((arp_t=_rtl8651_getArpEntry(NULL, lip, FALSE)) && _rtl8651_isMacAccessControlPermit(&arp_t->macInfo->macAddr)==FALSE) {
			/* DO NOT write it to ASIC due to MAC Access Control match */
			servPort_t->toAsic = 0;
			return SUCCESS;
		}
#endif /* RTL865X_EXP_MAC_ACCESS_CONTROL */

		servPort_t->asicTablePos = asicPos;
		servPort_t->toAsic = 1;
		for(i=0; i<=nCount; i++) {
			while(asic_t1->valid == 1 && asicPos < RTL8651_SERVERPORTTBL_SIZE ) {
				asic_t1++; asicPos++;
			}
			asic_t1->valid = 1;
			asic_t1->servPort_t = servPort_t;
			assert(asicPos < RTL8651_SERVERPORTTBL_SIZE);
			convert_setAsicServerPortTable(asicPos, servPort_t->globalIp, servPort_t->globalPort+i, servPort_t->localIp, servPort_t->localPort+i);
		}
	}
	else {
		servPort_t->toAsic = 0;//Using ACL to replace this
		//_rtl8651_arrangeAllACL(); // tony: do not run this code when turn auto-learn off.
	}

	//_rtl8651_showNaptSeverPort();
	return SUCCESS;	
}


int32 _rtl8651_delNaptServerPort(int8 isTcp, ipaddr_t gip, uint16 gport, ipaddr_t lip, uint16 lport, uint16 nCount) {
	rtl8651_tblDrv_serverPortAsicArrangementEntry_t *asic_t;
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_naptServerPortEntry_t *servPort_t;
	uint32 netIdx, i, asicPos;

	if (_rtl8651_extIpAddr(gip, &netIdx) == FALSE || _rtl8651_isNaptMapping(gip) == FALSE)
	{
		GET_NETWORK_INTERFACE(extIfName,&netIdx);
		//return TBLDRV_ENOTEXTIP;
	}
//	if (_rtl8651_isLanSideIpUnnumberedIpAddr(gip) == TRUE) 
//		return TBLDRV_EUNNUMBEREDIPNOTALLOW;
	if (_rtl8651_intNetIpAddr(lip) == FALSE) 
		return TBLDRV_ENOTINTIP;
	netif_t = &DrvTbl.netIface[netIdx];
	assert(netif_t->valid == 1);

	nCount--;
	SLIST_FOREACH(servPort_t, &netif_t->nextNaptServerPort, nextServerPort)
		if (servPort_t->globalIp == gip && servPort_t->localIp == lip &&
			servPort_t->globalPort == gport && servPort_t->endGlobalPort == (gport+nCount) &&
			servPort_t->localPort == lport && servPort_t->endLocalPort == (lport+nCount))
			break;
	if (servPort_t == (rtl8651_tblDrv_naptServerPortEntry_t *)NULL)
		return TBLDRV_EENTRYNOTFOUND;
		
	if ((isTcp&1) == TRUE && servPort_t->tcp == 1)
		servPort_t->tcp = 0;
	else if ((isTcp&1) == FALSE && servPort_t->udp == 1)
		servPort_t->udp = 0;
	else return TBLDRV_EENTRYNOTFOUND;

#ifdef SERVERPORT_PARK_TCPUDP
	/* In this mode, clear ASIC if any type is deleted. */
	if ( servPort_t->tcp == 0 || servPort_t->udp == 0 )
#else/*SERVERPORT_PARK_TCPUDP*/
	/* In old mode, only when both type are deleted, the ASIC entry is cleared. */
	if (servPort_t->tcp == 0 && servPort_t->udp == 0) 
#endif/*SERVERPORT_PARK_TCPUDP*/
	{
		if (servPort_t->toAsic == 1) {	
			asicPos = servPort_t->asicTablePos;
			asic_t = &AsicTableUsage.serverPort[asicPos];
			for(i=0; i<=nCount; i++) {
				while( asicPos < RTL8651_SERVERPORTTBL_SIZE ) {
					if (asic_t->valid == 1 && asic_t->servPort_t == servPort_t)
						break;
					asicPos++; asic_t++;
				}
				assert(asicPos < RTL8651_SERVERPORTTBL_SIZE);
				rtl8651_delAsicServerPortTable(asicPos);
				asic_t->valid = 0;
			}
			servPort_t->toAsic = 0;
		}
	}

	if (servPort_t->tcp == 0 && servPort_t->udp == 0) 
	{
		SLIST_REMOVE(&(netif_t->nextNaptServerPort), servPort_t, 
			      rtl8651_tblDrv_naptServerPortEntry_s, nextServerPort);
		SLIST_INSERT_HEAD(&DrvTbl.freeList.serverPort, servPort_t, nextServerPort);
	}

//	if (servPort_t->toAsic == 0)
//		_rtl8651_arrangeAllACL();
	//_rtl8651_showNaptSeverPort();
	return SUCCESS;
}



/*
@func int32 | rtl8651_addNaptServerPortMapping | Add a server port mapping.
@parm uint8 | isTcp | TRUE for TCP protocol, FALSE for UDP protocol.
@parm ipaddr | extIp | External NAPT IP address.
@parm int16 | extPort | The start external port number.
@parm ipaddr | intIp | Internal IP address.
@parm int16 | intPort | The start internal port number.
@rvalue SUCCESS | The specified napt server port mapping was removed successfully.
@rvalue TBLDRV_ENOTEXTIP | The specified external IP address is not external NAPT IP address.
@rvalue TBLDRV_ENOTINTIP | The specified internal IP address is not internal private IP address.
@rvalue TBLDRV_ERANGEOVERLAP | The specified port rang overlaps the existing one.
@rvalue TBLDRV_ENOFREEBUFFER | Out of free space.
*/
int32 rtl8651a_addNaptServerPortMapping(uint32 SessionID,int8 isTcp, ipaddr_t extIpAddr, uint16 extPort, ipaddr_t intIpAddr, uint16 intPort) {
	int32 retval;
	uint32 dsid;
	dsid=_rtl8651_SessionIDToDSID(SessionID);
		
	if( _rtl8651_getExtIpAddrByDsid(dsid)!=extIpAddr)
		return TBLDRV_ENOTEXTIP;

	rtlglue_drvMutexLock();//Lock resource
	if(dsid==_rtl8651_DefaultDialSessionId) //add hardware entry
		retval = _rtl8651_addNaptServerPort(isTcp, extIpAddr, extPort, intIpAddr, intPort, 1);
	else //add software entry
		retval = _rtl8651_addNaptServerPort((isTcp|2), extIpAddr, extPort, intIpAddr, intPort, 1);
	rtlglue_drvMutexUnlock();//Unlock resource
	return retval;
}

/*
@func int32 | rtl8651_delNaptServerPortMapping | Remove a server port mapping.
@parm uint32 | SessionID | Session ID.
@parm uint8 | isTcp | TRUE for TCP protocol, FALSE for UDP protocol.
@parm ipaddr | extIp | External NAPT IP address.
@parm int16 | extPort | The start external port number.
@parm ipaddr | intIp | Internal IP address.
@parm int16 | intPort | The start internal port number.
@rvalue SUCCESS | The specified napt server port mapping was removed successfully.
@rvalue TBLDRV_ENOTEXTIP | The specified external IP address is not external NAPT IP address.
@rvalue TBLDRV_ENOTINTIP | The specified internal IP address is not internal private IP address.
@rvalue TBLDRV_EENTRYNOTFOUND | The specified napt server port mapping was not found.
*/
int32 rtl8651a_delNaptServerPortMapping(uint32 SessionID,int8 isTcp, ipaddr_t extIpAddr, uint16 extPort, ipaddr_t intIpAddr, uint16 intPort) {
	int32 retval;
	uint32 dsid;
	dsid=_rtl8651_SessionIDToDSID(SessionID);

	if( _rtl8651_getExtIpAddrByDsid(dsid)!=extIpAddr)
		return TBLDRV_ENOTEXTIP;
	
	rtlglue_drvMutexLock();//Lock resource
	retval = _rtl8651_delNaptServerPort(isTcp, extIpAddr, extPort, intIpAddr, intPort, 1);
	rtlglue_drvMutexUnlock();//Unlock resource
	return retval;
}

/*
@func int32 | rtl8651_addNaptServerPortRange | Add a ranged server port mapping.
@parm uint32 | SessionID | Session ID.
@parm uint8 | isTcp | TRUE for TCP protocol, FALSE for UDP protocol.
@parm ipaddr | extIp | External NAPT IP address.
@parm int16 | extPort | The start external port number.
@parm ipaddr | intIp | Internal IP address.
@parm int16 | intPort | The start internal port number.
@parm int16 | nCount | The number of mapping ports.
@rvalue SUCCESS | The specified napt server port mapping was removed successfully.
@rvalue TBLDRV_ENOTEXTIP | The specified external IP address is not external NAPT IP address.
@rvalue TBLDRV_ENOTINTIP | The specified internal IP address is not internal private IP address.
@rvalue TBLDRV_ERANGEOVERLAP | The specified port rang overlaps the existing one.
@rvalue TBLDRV_ENOFREEBUFFER | Out of free space.
*/
int32 rtl8651a_addNaptServerPortRange(uint32 SessionID,int8 isTcp, ipaddr_t extIp, uint16 extPort, ipaddr_t intIp, uint16 intPort, uint16 nCount) {
	int32 retval;
	uint32 dsid;
	dsid=_rtl8651_SessionIDToDSID(SessionID);

	//ipaddr_t wanip=_rtl8651_getExtIpAddrByDsid(dsid);
	//rtlglue_printf("get server ip in romedriver=%u.%u.%u.%u dsid=%d\n",NIPQUAD(wanip),dsid);
	if( _rtl8651_getExtIpAddrByDsid(dsid)!=extIp)
		return TBLDRV_ENOTEXTIP;
	
	rtlglue_drvMutexLock();//Lock resource
	retval = _rtl8651_addNaptServerPort((isTcp|2), extIp, extPort, intIp, intPort, nCount);
	rtlglue_drvMutexUnlock();//Unlock resource
	return retval;
}

/*
@func int32 | rtl8651_delNaptServerPortRange | Remove a ranged server port mapping.
@parm uint32 | SessionID | Session ID.
@parm uint8 | isTcp | TRUE for TCP protocol, FALSE for UDP protocol.
@parm ipaddr | extIp | External NAPT IP address.
@parm int16 | extPort | The start external port number.
@parm ipaddr | intIp | Internal IP address.
@parm int16 | intPort | The start internal port number.
@parm int16 | nCount | The number of mapping ports.
@rvalue SUCCESS | The specified napt server port mapping was removed successfully.
@rvalue TBLDRV_ENOTEXTIP | The specified external IP address is not external NAPT IP address.
@rvalue TBLDRV_ENOTINTIP | The specified internal IP address is not internal private IP address.
@rvalue TBLDRV_EENTRYNOTFOUND | The specified napt server port mapping was not found.
*/
int32 rtl8651a_delNaptServerPortRange(uint32 SessionID,int8 isTcp, ipaddr_t extIp, uint16 extPort, ipaddr_t intIp, uint16 intPort, uint16 nCount) {
	int32 retval;
	uint32 dsid;
	dsid=_rtl8651_SessionIDToDSID(SessionID);

	if( _rtl8651_getExtIpAddrByDsid(dsid)!=extIp)
		return TBLDRV_ENOTEXTIP;
	
	rtlglue_drvMutexLock();//Lock resource
	retval = _rtl8651_delNaptServerPort(isTcp, extIp, extPort, intIp, intPort, nCount);
	rtlglue_drvMutexUnlock();//Unlock resource
	return retval;
}



#define RTL8651_NAPT_TCPUDP_APIS
#undef ADD_NAPT_DEBUG /* debug flag for add napt related function */
#undef CACHE_FIND_NAPT_DEBUG_RESULT /* debug flag for cache find result */

int32 rtl8651_enable4WayHash(uint8 isEnable)
{
	rtlglue_drvMutexLock();
	_set4WayHash((int32)isEnable);
	enable4WayHash = (isEnable==TRUE)? TRUE: FALSE;
	rtlglue_drvMutexUnlock();
	return SUCCESS;

}

int32 rtl8651_is4WayHashEnable(uint8 *isEnable)
{
	rtlglue_drvMutexLock();
	if (isEnable != NULL) {
		*isEnable = (_Is4WayHashEnabled()!=0)? TRUE: FALSE;
	}
	rtlglue_drvMutexUnlock();
	return SUCCESS;
}


/*
 *  This function will actually set ASIC age ( TCP/UDP/ICMP NAPT table) to HW.
 *  It can compare rtl8651AsicAge's timeout values and idle timeout values.
 *  ==> Now, The NAPT age timer is set up to RTL8651_MAX_NAPT_ASIC_AGE sec for granularity.
 */
static void _rtl8651_setNaptAsicAge( void )
{
	/* Restore default age to original value. */
	rtl8651AsicAge = rtl8651GlobalControl;

	/************* We need to suppress ASIC age due to AGE timer granularity issue *************/
#ifdef RTL865X_DEBUG
	if (0)
#endif
	{
		uint32 asicTimeout;

		asicTimeout = _rtl8651_NaptAgingToSec( _rtl8651_NaptAgingToUnit(RTL8651_MAX_NAPT_ASIC_AGE) );

		assert(asicTimeout <= RTL8651_MAX_NAPT_ASIC_AGE);

		/* Suppress ASIC entries' timeout */
		if ( rtl8651AsicAge.tcpFastTimeout > asicTimeout )
		{
			rtl8651AsicAge.tcpFastTimeout = asicTimeout;
		}
		if ( rtl8651AsicAge.tcpMediumTimeout > asicTimeout )
		{
			rtl8651AsicAge.tcpMediumTimeout = asicTimeout;
		}
		if ( rtl8651AsicAge.tcpLongTimeout > asicTimeout )
		{
			rtl8651AsicAge.tcpLongTimeout = asicTimeout;
		}
		if ( rtl8651AsicAge.udpTimeout > asicTimeout )
		{
			rtl8651AsicAge.udpTimeout = asicTimeout;
		}
		if ( rtl8651AsicAge.icmpTimeout > asicTimeout )
		{
			rtl8651AsicAge.icmpTimeout = asicTimeout;
		}
		if ( rtl8651AsicAge.tcpFinTimeout > asicTimeout )
		{
			rtl8651AsicAge.tcpFinTimeout = asicTimeout;
		}
	}

	/************* Set ASIC entries' timeout to HW *************/
	rtl8651_setAsicNaptTcpFastTimeout( rtl8651AsicAge.tcpFastTimeout );
	rtl8651_setAsicNaptTcpMediumTimeout( rtl8651AsicAge.tcpMediumTimeout );
	rtl8651_setAsicNaptTcpLongTimeout( rtl8651AsicAge.tcpLongTimeout );
	rtl8651_setAsicNaptUdpTimeout( rtl8651AsicAge.udpTimeout );
	rtl8651_setAsicNaptIcmpTimeout( rtl8651AsicAge.icmpTimeout );
}


/*
@func int32 | rtl8651_setNaptIcmpTimeout | Set NAPT ICMP entry aging time.
@parm uint32 | timeout | Aging time in second.
@rvalue SUCCESS | Success.
@comm
The short aging time will get better performance, but too short aging time will casue ICMP connection fail.
*/
int32 rtl8651_setNaptIcmpTimeout(uint32 timeout) {

	rtlglue_drvMutexLock();		/* Lock resource */
	rtl8651GlobalControl.icmpTimeout = timeout;
	_rtl8651_setNaptAsicAge();
	rtlglue_drvMutexUnlock();	/* Unlock resource */
	
	return SUCCESS;
}

/*
@func int32 | rtl8651_setNaptIcmpFastTimeout | Set NAPT ICMP entry Fast aging time.
@parm uint32 | timeout | Aging time in second.
@rvalue SUCCESS | Success.
@comm
This API is to set the age-timer for ACKed NAPT ICMP entries.
The short aging time will get better performance, but too short aging time will casue ICMP connection fail.
*/
int32 rtl8651_setNaptIcmpFastTimeout(uint32 timeout) {

	rtlglue_drvMutexLock();		/* Lock resource */
	rtl8651GlobalControl.icmpFastTimeout = timeout;
	_rtl8651_setNaptAsicAge();
	rtlglue_drvMutexUnlock();	/* Unlock resource */
	
	return SUCCESS;
}


/*
@func int32 | rtl8651_setNaptUdpTimeout | Set NAPT UDP entry aging time.
@parm uint32 | timeout | Aging time in second.
@rvalue SUCCESS | Success.
@comm
The short aging time will get better performance, but too short aging time will casue UDP connection fail.
*/
int32 rtl8651_setNaptUdpTimeout(uint32 timeout) {

	rtlglue_drvMutexLock();		/* Lock resource */
	rtl8651GlobalControl.udpTimeout = timeout;
	_rtl8651_setNaptAsicAge();
	rtlglue_drvMutexUnlock();	/* Unlock resource */

	return SUCCESS;
}

/*
@func int32 | rtl8651_setNaptTcpLongTimeout | Set NAPT TCP entry long aging time.
@parm uint32 | timeout | Aging time in second.
@rvalue SUCCESS | Success.
@comm
There are three aging time for any NAPT TCP entry - Fast, Long and Medium aging time. 
Fast timeout: Before the three-way handshaking complete. Shorter Fast timeout may get more good performance, but too short timeout value will casue TCP connection fail.
Long timeout: After the three-way handshaking and before the first FIN is received. 
Medium timeout: Once a FIn is received, the TCP entry changes to Medium timeout.
*/
int32 rtl8651_setNaptTcpLongTimeout(uint32 timeout) {

	rtlglue_drvMutexLock();		/* Lock resource */
	rtl8651GlobalControl.tcpLongTimeout = timeout;
	_rtl8651_setNaptAsicAge();
	rtlglue_drvMutexUnlock();	/* Unlock resource */

	return SUCCESS;
}

/*
@func int32 | rtl8651_setNaptTcpMediumTimeout | Set NAPT TCP entry Medium aging time.
@parm uint32 | timeout | Aging time in second.
@rvalue SUCCESS | Success.
@comm
There are three aging time for any NAPT TCP entry - Fast, Long and Medium aging time. 
Fast timeout: Before the three-way handshaking complete. Shorter Fast timeout may get more good performance, but too short timeout value will casue TCP connection fail.
Long timeout: After the three-way handshaking and before the first FIN is received. 
Medium timeout: Once a FIn is received, the TCP entry changes to Medium timeout.
*/
int32 rtl8651_setNaptTcpMediumTimeout(uint32 timeout) {

	rtlglue_drvMutexLock();		/* Lock resource */
	rtl8651GlobalControl.tcpMediumTimeout = timeout;
	_rtl8651_setNaptAsicAge();
	rtlglue_drvMutexUnlock();	/* Unlock resource */

	return SUCCESS;
}

/*
@func int32 | rtl8651_setNaptTcpFastTimeout | Set NAPT TCP entry Fast aging time.
@parm uint32 | timeout | Aging time in second.
@rvalue SUCCESS | Success.
@comm
There are three aging time for any NAPT TCP entry - Fast, Long and Medium aging time. 
Fast timeout: Before the three-way handshaking complete. Shorter Fast timeout may get more good performance, but too short timeout value will casue TCP connection fail.
Long timeout: After the three-way handshaking and before the first FIN is received. 
Medium timeout: Once a FIn is received, the TCP entry changes to Medium timeout.
*/
int32 rtl8651_setNaptTcpFastTimeout(uint32 timeout) {

	rtlglue_drvMutexLock();		/* Lock resource */
	_userConfigFastTimeoutValue	 = timeout;	/* For fast-timeout adjustment, used in SW only */
	rtl8651GlobalControl.tcpFastTimeout = timeout;
	_rtl8651_setNaptAsicAge();
	rtlglue_drvMutexUnlock();	/* Unlock resource */

	return SUCCESS;
}


/*
@func int32 | rtl8651_setNaptTcpFinTimeout | Set NAPT TCP entry aging time when recv double-FIN.
@parm uint32 | timeout | Aging time in second.
@rvalue SUCCESS | Success.
@comm
This timeout is only applied on HASH2 double-FINed connection.
*/
int32 rtl8651_setNaptTcpFinTimeout(uint32 timeout) {

	rtlglue_drvMutexLock();		/* Lock resource */
	rtl8651GlobalControl.tcpFinTimeout = timeout;
	_rtl8651_setNaptAsicAge();
	rtlglue_drvMutexUnlock();	/* Unlock resource */

	return SUCCESS;
}




/*
@func int32		| _rtl8651_setNaptEntry	| Table driver internal use. Given 5 tuples in outbound packet, find and return the NAPT flow pointer from software table or ASIC.
@parm uint16 	| tblIndex	| NAPT entry index.
@parm rtl8651DrvNaptLine_t *	| naptLine	| Pointer to #<p tblIndex>-th NAPT line.
@parm rtl8651_tblDrv_naptTcpUdpFlowEntry_t * 	| entry	| Pointer to the entry to be written to ASIC. If <p NULL>, the entry is to be deleted.
@parm int8  		| statusUpdate | When <p entry> is NULL, TRUE to sync information between ASIC and software table. False to delete ASIC entry.
@rvalue SUCCESS	| 	NAPT entry set successfully.
@rvalue FAILED	| 	failed to set NAPT entry. 
@comm 
This function is provided for three purposes:
1. If < p entry> is not NULL,<p statusUpdate> is TRUE, then "force" write content in <p entry> to ASIC cache line <p naptLine> whose table index is <p tblIndex>. 
2. If < p entry> is not NULL,<p statusUpdate> is FALSE, then "soft" write content in <p entry> to ASIC cache line <p naptLine> whose table index is <p tblIndex>. connection written to ASIC is static entry and also cached in software.
2. If <p entry> is NULL and <p statusUpdate> is TRUE, then status information between ASIC and software table is synchronized. (collision,autolearn check bits got updated.)
3. If <p entry> is NULL and <p statusUpdate> is FALSE, this ASIC entry content is cleared.
@devnote
 */
int32 _rtl8651_setNaptEntry( uint16 tblIndex, rtl8651DrvNaptLine_t *naptLine, rtl8651_tblDrv_naptTcpUdpFlowEntry_t *entry, int8 statusUpdate)
{
	int32 collision = 0, collision2 = 0, index;
	uint32 entryType;
	rtl8651_tblAsic_naptTcpUdpTable_t  asicEntry;
	rtl865x_tblAsicDrv_naptTcpUdpParam_t asic_napt;
 	uint32 ageAsic;
 	int32 inAsic; /* (HASH2 only) Inbound is ASIC-accelerated. */
 	int32 outAsic; /* (HASH2 only) Outbound is ASIC-accelerated. */

	assert(naptLine);
	//update dynamic status infor between software and ASIC table.
	// update collision bit
	/* Louis commented it to accelerate !
	rtl8651_getAsicRawNaptTable(tblIndex, &asicEntry);
	*/

	if(!entry){

		if(statusUpdate){
			//Write is Force write.
			assert(0);/* Louis: seems strang */
			return rtl8651_setAsicRawNaptTable( tblIndex, &asicEntry, TRUE);
		}

		entryType=RTL8651_DYNAMIC_NAPT_ENTRY;
	
		//clear the entry, update collision  bits, reset to initial port offset value.
		asic_napt.ageSec = 0;
		asic_napt.insideLocalIpAddr = 0;
		asic_napt.insideLocalPort = 0;
		asic_napt.isCollision = (collision==TRUE? 1: 0);
		asic_napt.isCollision2 = (collision2==TRUE? 1: 0);
		asic_napt.isDedicated = 0;
		asic_napt.isStatic = 0;
		asic_napt.isTcp = 0;
		asic_napt.isValid = 0;
		asic_napt.offset = 0;
		asic_napt.selEIdx = 0;
		asic_napt.selExtIPIdx = 0;
		asic_napt.tcpFlag = 0;
		index = tblIndex & (RTL8651_TCPUDPTBL_SIZE-1);
		return rtl8651_setAsicNaptTcpUdpTable(TRUE, index, &asic_napt);
	}

	assert(entry);

	/* age cannot be larger than ASIC timer. */
	if ( entry->tcp )
		ageAsic = min( entry->age, rtl8651AsicAge.tcpLongTimeout );
	else
		ageAsic = min( entry->age, rtl8651AsicAge.udpTimeout );

	if( entry->hashing==NAPT_HASH1 ){
		if(statusUpdate)
			entryType=entry->isStatic?
				RTL8651_STATIC_NAPT_ENTRY:RTL8651_DYNAMIC_NAPT_ENTRY;
		else
			entryType=RTL8651_STATIC_NAPT_ENTRY;

		asic_napt.ageSec = ageAsic; /*entry->age;*/
		asic_napt.insideLocalIpAddr = entry->insideLocalIpAddr;
		asic_napt.insideLocalPort = entry->insideLocalPort;
		asic_napt.isCollision = (collision==TRUE? 1: 0);
		asic_napt.isCollision2 = (collision2==TRUE? 1: 0);
		asic_napt.isDedicated = 0;
		asic_napt.isStatic = entryType&RTL8651_STATIC_NAPT_ENTRY? 1: 0;
		asic_napt.isTcp = (entry->tcp==TRUE?1: 0);
		asic_napt.isValid = 1;
		asic_napt.offset = entry->insideGlobalPort>>RTL8651_TCPUDPTBL_BITS;
		asic_napt.selEIdx = 0;
		asic_napt.selExtIPIdx = 0;
		asic_napt.tcpFlag = (entry->tcp==TRUE)?entry->tcpFlag: 0;
		index = entry->insideGlobalPort&(RTL8651_TCPUDPTBL_SIZE-1);
		return rtl8651_setAsicNaptTcpUdpTable(statusUpdate, index, &asic_napt);
	}

	/* HASH2 */
	assert( entry->hashing==NAPT_HASH2 );
	entryType=RTL8651_STATIC_NAPT_ENTRY|RTL8651_LIBERAL_NAPT_ENTRY;
	if( entry->connectionType == NAPT_TYPE_SERVERPORT )
		SETBITS(entryType,RTL8651_NAPT_SYNFIN_QUIET);

	CLEARBITS(entryType, RTL8651_NAPT_UNIDIRECTIONAL_FLOW);
	CLEARBITS(entryType, RTL8651_NAPT_OUTBOUND_FLOW);
	inAsic = outAsic = FALSE;
	if(entry->inboundTblIdx==entry->outboundTblIdx)
	{
		if ( entry->inListType == NAPT_ASIC_CACHED_IN &&
		     entry->outListType == NAPT_ASIC_CACHED_OUT )
		{
			/* inbound and outbound are ASIC-accelerated */
			CLEARBITS(entryType, RTL8651_NAPT_UNIDIRECTIONAL_FLOW);
			CLEARBITS(entryType, RTL8651_NAPT_OUTBOUND_FLOW);
			inAsic = outAsic = TRUE;
		}
		else if ( entry->inListType == NAPT_ASIC_CACHED_IN &&
		          entry->outListType != NAPT_ASIC_CACHED_OUT )
		{
			/* inbound is ASIC-accelerated */
			SETBITS(entryType, RTL8651_NAPT_UNIDIRECTIONAL_FLOW);
			SETBITS(entryType,RTL8651_NAPT_INBOUND_FLOW);
			inAsic = TRUE;
		}
		else if ( entry->inListType != NAPT_ASIC_CACHED_IN &&
		          entry->outListType == NAPT_ASIC_CACHED_OUT )
		{
			/* outbound is ASIC-accelerated */
			SETBITS(entryType, RTL8651_NAPT_UNIDIRECTIONAL_FLOW);
			SETBITS(entryType,RTL8651_NAPT_OUTBOUND_FLOW);
			outAsic = TRUE;
		}
		else
		{
			/* inbound and outbound are NOT ASIC-accelerated, this function shoulde NOT be called! */
			assert( 0 );
			return FAILED;
		}
	}
	else if(tblIndex==entry->outboundTblIdx)
	{
		/* outbound is ASIC-accelerated */
		if ( entry->outListType == NAPT_ASIC_CACHED_OUT )
		{
			SETBITS(entryType, RTL8651_NAPT_UNIDIRECTIONAL_FLOW);
			SETBITS(entryType,RTL8651_NAPT_OUTBOUND_FLOW);
			outAsic = TRUE;
		}
	}
	else
	{
		/* inbound is ASIC-accelerated */
		assert( tblIndex == entry->inboundTblIdx );
		if ( entry->inListType == NAPT_ASIC_CACHED_IN )
		{
			SETBITS(entryType, RTL8651_NAPT_UNIDIRECTIONAL_FLOW);
			SETBITS(entryType,RTL8651_NAPT_INBOUND_FLOW);
			inAsic = TRUE;
		}
	}

	/* We assume either inbound or outbound entry is equal to tblIndex. Otherwise, it is error. */
	assert( entry->inboundTblIdx == tblIndex ||
	        entry->outboundTblIdx == tblIndex );

	if ( inAsic && entry->inboundTblIdx == tblIndex &&
	     entry->inExtIpIdx < 0 )
	{
		entry->inExtIpIdx = _rtl8651_addHash2ToNexthop( entry->interfaceRoute? entry->dstIpAddr: 0,
		                                                entry->insideGlobalIpAddr );
		if ( entry->inExtIpIdx==FAILED ) return FAILED;
	}
	if ( outAsic && entry->outboundTblIdx == tblIndex &&
	     entry->outExtIpIdx < 0 )
	{
		entry->outExtIpIdx = _rtl8651_addHash2ToNexthop( entry->interfaceRoute? entry->dstIpAddr: 0,
		                                                 entry->insideGlobalIpAddr );
		if ( entry->outExtIpIdx==FAILED )
		{
			/* If failed, de-refer the inbound count. */
			_rtl8651_delHash2FromNexthop( entry->interfaceRoute? entry->dstIpAddr: 0,
			                              entry->insideGlobalIpAddr );
			entry->inExtIpIdx = -1;
			return FAILED;
		}
	}
	if ( entry->inExtIpIdx >= 0 &&
	     entry->outExtIpIdx >= 0 )
		assert( entry->inExtIpIdx == entry->outExtIpIdx );

#if 0/* Since holly chhuang has supported multiple nexthop, we do not concern the interface route issue anymore if extIpIdx is availiable. */
	/* If it is an interface route connection, we only allow inbound flow !! */
	if (entry->interfaceRoute)
	{
		assert( ISSET(entryType, RTL8651_NAPT_UNIDIRECTIONAL_FLOW)==0 ||  /* bi-dir */
		        ( ISSET(entryType, RTL8651_NAPT_UNIDIRECTIONAL_FLOW) &&   /* or uni-dir */
		          ISSET(entryType,RTL8651_NAPT_OUTBOUND_FLOW)==0 ) );     /*    inbound */
		SETBITS(entryType, RTL8651_NAPT_UNIDIRECTIONAL_FLOW);
		CLEARBITS(entryType,RTL8651_NAPT_OUTBOUND_FLOW);
	}
#endif/*0*/

	return rtl8651_setAsicLiberalNaptTcpUdpTable(statusUpdate, tblIndex, entry->insideLocalIpAddr, 
		entry->insideLocalPort, entry->inExtIpIdx>=0?entry->inExtIpIdx:entry->outExtIpIdx, entry->insideGlobalPort, 
		ageAsic/*entry->age*/,entryType, entry->tcp, collision, collision2, TRUE);
		 
}

/*
@func int32					| _rtl8651_getNaptEntry	| Table driver internal use. Read an entry from ASIC or from ASIC cache. If <p dontcache> is TRUE, read directly from ASIC. 
@parm uint16 				| tblIndex		| NAPT entry index.
@parm rtl8651DrvNaptLine_t *	| naptLine		| Pointer to #<p tblIndex>-th NAPT line.
@parm rtl8651_tblDrv_naptTcpUdpFlowEntry_t ** 	| entry	| <p *entry> pointers to the temp sotrage provided by caller. If entry already cached in softare, <p *entry>, on return, points to that cached connection. Otherwise, caller's temp storage is used to save entry read from ASIC.
@parm int8					| direct			| Don't read from cache. Read from ASIC directly
@rvalue SUCCESS	| 	Always returns success
@comm 
Read entry from ASIC cache. If cache misses or <p direct> is TRUE, read directly from ASIC.
@devnote
 */
int32  _rtl8651_getNaptEntry(uint16 tblIndex, rtl8651DrvNaptLine_t *naptLine, rtl8651_tblDrv_naptTcpUdpFlowEntry_t **entry,  int8 direct){
	struct naptEntryListHead *head;
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *asicCached;
	rtl8651_tblDrv_naptTcpUdpFlowEntry_t *storage;
	rtl8651_tblAsic_naptTcpUdpTable_t  asicEntry;

	assert(entry);
	head=&naptLine->list[NAPT_ASIC_CACHED_OUT];
	asicCached=SLIST_FIRST(head);
	if(!asicCached){
		head=&naptLine->list[NAPT_ASIC_CACHED_IN];
		asicCached=SLIST_FIRST(head);
	}
	assert(asicCached); /* Louis: Since we disabled ASIC auto-learn, only software will add connection into ASIC entry. */

	if(direct|| !asicCached){
		//Read from ASIC directly if requested
#if 1 /* New-fashion: only read few information from ASIC. */
#else /* Old-fashion: read connection infomation from ASIC entry as many as possible. */
		uint32 index;
#endif
		storage= *entry;
		assert(storage);
#if 1 /* New-fashion: only read few information from ASIC. */
		*storage = *asicCached;
		storage->asicSnapshot = 1;
		rtl8651_getAsicRawNaptTable(tblIndex, (void *)&asicEntry);
		storage->age = _rtl8651_NaptAgingToSec(asicEntry.agingTime);
		storage->tcpFlag = asicEntry.TCPFlag;
		storage->fromDrv = 0;
#else /* Old-fashion: read connection infomation from ASIC entry as many as possible. */		
		memset(storage,0,sizeof(rtl8651_tblDrv_naptTcpUdpFlowEntry_t));
		rtl8651_getAsicRawNaptTable(tblIndex, (void *)&asicEntry);
		storage->asicSnapshot=1;
		storage->dstIpAddr=0;
		storage->insideLocalPort= asicEntry.intPort;
		storage->insideLocalIpAddr=asicEntry.intIPAddr;
		storage->dstPort=0;
		storage->age = _rtl8651_NaptAgingToSec(asicEntry.agingTime);
		storage->tcp = asicEntry.isTCP;
		storage->tcpFlag = asicEntry.TCPFlag;
		storage->alive=asicEntry.valid;
		storage->fromDrv=0;
/* Louis commented it.
		//bit got set if entry was auto learnt by ASIC (ie. a dynamic entry)
		storage->asicAutoLearnt=!asicEntry.isStatic;
*/
		storage->pureSoftware = FALSE;
		storage->col1 = asicEntry.collision;
		storage->col2=asicEntry.collision2;
		storage->isStatic=asicEntry.isStatic;
		storage->hashing = asicEntry.dedicate?NAPT_HASH2:NAPT_HASH1;
		if(asicEntry.dedicate){
			int32 retval;
			rtl865x_tblAsicDrv_extIntIpParam_t ipt_entry;

			retval = rtl8651_getAsicExtIntIpTable(asicEntry.selIPIdx , &ipt_entry);
			assert(retval==SUCCESS);
			storage->insideGlobalIpAddr = ipt_entry.extIpAddr;
			storage->insideGlobalPort= asicEntry.offset << RTL8651_TCPUDPTBL_BITS | asicEntry.selEIdx;
			storage->quiet=(asicEntry.TCPFlag&0x4)>>2;
			storage->outboundTblIdx = tblIndex;
			storage->inboundTblIdx = asicEntry.selEIdx;
		}else if (asicEntry.valid){
			index = (asicEntry.intIPAddr & (RTL8651_IPTABLE_SIZE-1))%naptIpCount;
			assert(_gidxRegister[index]>=0);
			storage->insideGlobalIpAddr=DrvTbl.iptbl[index].extip; // AsicTableUsage.ip[index].externalIp;
			storage->insideGlobalPort= asicEntry.offset << RTL8651_TCPUDPTBL_BITS | tblIndex;
			storage->quiet=0;
			storage->outboundTblIdx =storage->inboundTblIdx= tblIndex;
		}
#endif		
		return SUCCESS;
	}

	//cache hit
	*entry=asicCached;
	return SUCCESS;
}


/*
@func int32	| rtl8651_setNaptTcpUdpPortRange	| Table driver internal use. Read an entry from ASIC or from ASIC cache. If <p dontcache> is TRUE, read directly from ASIC. 
@parm uint16 | start	| first port number for NAPT translation.
@parm uint16	| end	| last port number for NAPT translation.
@rvalue SUCCESS			| Port range configured to ASIC successfully.
@rvalue TBLDRV_EINVALIDINPUT	| Both port numbers given must be multiples of 1024 and <p end> must greater than <p start>.
@comm 
Set TCP/UDP port range for ASIC NAPT engine and software NAPT driver.
@devnote
 */

int32 rtl8651_setNaptTcpUdpPortRange(uint16 start, uint16 end) {
	if ((start&0x3ff) != 0 || (end&0x3ff) != 0x3ff || start > end) {
		return(TBLDRV_EINVALIDINPUT); 
	}
	start = start >> 10; end = end >> 10;
	if (end > 0x3f || start > 0x3f) {
		return(TBLDRV_EINVALIDINPUT); 
	}
	rtlglue_drvMutexLock();//Lock resource
	 
	rtl8651GlobalControl.l4PortRangeEnd = end;
	rtl8651GlobalControl.l4PortRangeStart = start;
	rtl8651_setAsicL4Offset(rtl8651GlobalControl.l4PortRangeStart, rtl8651GlobalControl.l4PortRangeEnd);
	rtlglue_drvMutexUnlock();//Unlock resource
	return SUCCESS;
}





/*
@func int32	| _rtl8651_getDefaultNextHop	| Subroutine to get default nexthop from default route entry.
@rvalue NULL | The default route is not set.
@comm 
Subroutine to get default nexthop from default route entry.
@devnote
 */
rtl8651_tblDrv_nextHopEntry_t* 
_rtl8651_getDefaultNextHop( void )
{
	rtl8651_tblDrv_routeTable_t *rt_t = NULL;

	rt_t = &DrvTbl.route[RTL8651_ROUTINGTBL_SIZE-1];
	if ( rt_t->valid==1 )
	{
		return rt_t->un.nxthop.start_t;
	}
	
	return NULL;
}


/*
@func int32	| _rtl8651_ExtPortIsShared	|Internal use only. Check if the two connection is using the same port.
@parm uint32    | isTcp		| TCP or UDP
@parm ipaddr_t	| ExtIp		| Ip address
@parm uint16    | ExtPort	| Port number
@parm uint16    | assumedPort	| Port number assumed to add (0 means no assumption).
@rvalue SUCCESS	| A vacant offset value was found and returned via <p *offset>.
@rvalue FAILED 	| All offset values inuse. 
@comm 
This function will return if the specified port is re-used. 
For several purposes such as Trigger Port and Server Port, we must implement the functionality of re-using port number.
Therefore, we must have a function to tell us if a port is shared by any two connections.
In addition, the 'assumed port' is designed for addNaptConnection() to check if it will share with existing connections.
@devnote
*/
int8 _rtl8651_ExtPortIsShared( uint32 isTcp, ipaddr_t ExtIp, uint16 ExtPort, uint16 assumedPort )
{
	uint32 mask[2], thisBit[2]; //an empty bitmap. flatten all possible offset values.
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *thisConn;
	rtl8651DrvNaptLine_t *tcpudpPtr;
	uint32 thisOffset;
	int32 i;
	isTcp = (isTcp?TRUE:FALSE);
	tcpudpPtr = &DrvTbl.tcpudp[ ExtPort & (RTL8651_TCPUDPTBL_SIZE-1) ];

	mask[0] = mask[1] = thisBit[0] = thisBit[1] = 0;
	assert( ExtIp != 0 /* Louis note: ExtPort is allowed now! For CDRouter. && ExtPort != 0 */ );

	if ( assumedPort > 0 )
	{
		thisOffset = ( assumedPort >> RTL8651_TCPUDPTBL_BITS );
		thisBit[0] = thisBit[1] = 0;
		thisBit[thisOffset>>5] = 1 << (thisOffset&0x1f);
		mask[thisOffset>>5] |= thisBit[thisOffset>>5];
	}

	/* check server port usage */
	{
		rtl8651_tblDrv_networkIntfTable_t *netif_t;
		rtl8651_tblDrv_naptServerPortEntry_t *servPort_t;
		uint32 netIdx;

		if (_rtl8651_extIpAddr(ExtIp, &netIdx) == FALSE ||
		    _rtl8651_isNaptMapping(ExtIp) == FALSE) 
		{
			GET_NETWORK_INTERFACE(extIfName,&netIdx);
			//return TBLDRV_ENOTEXTIP;
		}
		
		netif_t = &DrvTbl.netIface[netIdx];
		assert(netif_t->valid == 1);
		
		SLIST_FOREACH(servPort_t, &netif_t->nextNaptServerPort, nextServerPort) 
		{
			if ( servPort_t->tcp == isTcp &&
			     servPort_t->globalIp == ExtIp )
			{
				if ( servPort_t->globalPort<= ExtPort &&
				     ExtPort <= servPort_t->endGlobalPort )
					return TRUE;
			}
		}
	}

	// Consider NAPT connections (including ALG connections).
	for( i = NAPT_ASIC_CACHED_IN; i < NAPT_TOTAL_LISTS; i+=2 )
	{
		SLIST_FOREACH( thisConn, &tcpudpPtr->list[i], nextIn )
		{
			assert( SLIST_NEXT( thisConn, nextIn ) != thisConn );
			if ( thisConn->tcp != isTcp ) continue;
			if ( thisConn->insideGlobalIpAddr != ExtIp ) continue;
			
			thisOffset = ( thisConn->insideGlobalPort >> RTL8651_TCPUDPTBL_BITS );
			thisBit[0] = thisBit[1] = 0;
			thisBit[thisOffset>>5] = 1 << (thisOffset&0x1f);
			if ( mask[thisOffset>>5] & thisBit[thisOffset>>5] )
			{
				// The port has been used! Yes, this port is shared.
				return TRUE;
			}
			else
			{
				mask[thisOffset>>5] |= thisBit[thisOffset>>5];
			}
		}
	}
	return FALSE;
}


/*
@func int32		| _rtl8651_getOutboundAlgConnection	| For forwarding engine. Given 5 tuples in outbound packet, find and return the ALG control flow pointer from software table.
@parm int8 		| isTcp	| TRUE for TCP, FALSE for UDP.
@parm ipaddr_t 	| sip		| Internal IP
@parm uint16 	| sport	| Internal port
@parm ipaddr_t	| dip	| Destination IP
@parm uint16		| dport	| Destination port
@parm struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s **		| ppEntry	| pointer to a temp storage from caller
@rvalue SUCCESS					| 	ALG control flow found. 
@rvalue TBLDRV_EENTRYNOTFOUND	| 	Flow not found.
@comm 
Given 5 tuples from ALG outbound packet, find and get ALG control flow from software table if connection exists.
*ppEntry should points to an rtl8651_tblDrv_naptTcpUdpFlowEntry_s instacnce on caller's stack. 
If we found the ALG flow in question is in table driver,  *ppEntry, on return,points to that flow inside table driver. 
If the ALG flow doesn't not exists, *ppEntry=NULL is returned.
 */
int32 
_rtl8651_getOutboundAlgConnection( int8 isTcp, ipaddr_t sip, uint16 sport, 
                                 ipaddr_t dip, uint16 dport,
                                 struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s **ppEntry)
{
	rtl8651_tblDrv_naptTcpUdpFlowEntry_t *pEntry, *thisConn;
	int32 flowTblIdx;
	rtl8651DrvNaptLine_t *thisLine;

	assert( ppEntry /* Louis: we no longer assist this: && *ppEntry */ );
	assert( sip  /* Louis: we no longer assist this: && sport */ );
	pEntry = *ppEntry;
	
	flowTblIdx = rtl8651_naptTcpUdpTableIndex( isTcp, sip, sport, 0, 0 );//use 3-tuple liberal hashing
	thisLine = &DrvTbl.tcpudp[flowTblIdx];

	SLIST_FOREACH(thisConn, &thisLine->list[NAPT_ALG_OUT], nextOut)
	{
		assert( thisConn->connectionType == NAPT_TYPE_ALG_CONNECTION );
		
		if(thisConn->age==0)
			continue;
		if((thisConn->tcp ==1? TRUE: FALSE) != isTcp)
			continue;
		if(thisConn->insideLocalIpAddr != sip)
			continue;
		if(thisConn->insideLocalPort != sport)
			continue;
		if(dip&&thisConn->dstIpAddr!= dip)
			continue;
		if(dport &&thisConn->dstPort!= dport)
			continue;
		pEntry= thisConn;
		goto found;
	}
	*ppEntry=NULL;
	return TBLDRV_EENTRYNOTFOUND; 		

found:

	// Refresh age value of UDP connection.
	//  (The age value of TCP is updated by _rtl8651_l4TcpMonitorIn/Out().
	if ( pEntry->tcp == FALSE )
		pEntry->age = rtl8651GlobalControl.udpTimeout;

	assert(pEntry);
	*ppEntry= pEntry;
	return SUCCESS;
}

/*
@func int32		| _rtl8651_getInboundAlgConnection	| For forwarding engine. Given 5 tuples in inbound packet, find and return the inbound NAPT flow pointer from software table.
@parm int8 		| isTcp	| TRUE for TCP, FALSE for UDP.
@parm ipaddr_t 	| gip	| Destination IP in inbound packet. It's one of the gateway's IP.
@parm uint16 	| gport	| Destination port in inbound packet. It's the gateway selected external port.
@parm ipaddr_t	| dip	| Source IP in inbound packet.
@parm uint16		| dport	| Source port in inbound packet.
@parm struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s **		| ppEntry	| pointer to a temp storage from caller
@rvalue SUCCESS					| 	NAPT flow found. 
@rvalue TBLDRV_EENTRYNOTFOUND	| 	Flow not found.
@comm 
Given 5 tuples of an ALG inbound packet, check and get if connection exists.
<p *ppEntry> should already points to an struct <p rtl8651_tblDrv_naptTcpUdpFlowEntry_s> instacnce prepared by caller.
If we found the connection in question in table driver,  <p *ppntry> returns and points to that  connection address
If we found the entry in ASIC, then caller's entry instance storage is used to save this temporary snapshot information. Note that entry read from ASIC WON'T be cached in software table.
 */
int32 
_rtl8651_getInboundAlgConnection( int8 isTcp, ipaddr_t gip, uint16 gport, 
                                ipaddr_t dip, uint16 dport, 
                                struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s **ppEntry )
{
	rtl8651_tblDrv_naptTcpUdpFlowEntry_t *pEntry, *thisConn;
	uint32 flowTblIdx = gport & (RTL8651_TCPUDPTBL_SIZE-1);
	rtl8651DrvNaptLine_t *thisLine;
	assert( ppEntry /* Louis: we no longer assist this: && *ppEntry */ );
	pEntry = *ppEntry;
	thisLine = &DrvTbl.tcpudp[flowTblIdx];
	assert( gip  /* Louis: we no longer assist this: && gport */ );
	SLIST_FOREACH( thisConn, &thisLine->list[NAPT_ALG_IN], nextIn )
	{
		assert( thisConn->connectionType == NAPT_TYPE_ALG_CONNECTION );
		
		if(thisConn->age==0)
			continue;
		if((thisConn->tcp ==1? TRUE: FALSE) != isTcp)
			continue;
		if(thisConn->insideGlobalIpAddr != gip)
			continue;
		if(thisConn->insideGlobalPort != gport)
			continue;
		if(dip&&thisConn->dstIpAddr!= dip)
			continue;
		if(dport &&thisConn->dstPort!= dport)
			continue;
		pEntry= thisConn;
		goto found;
	}
	*ppEntry=NULL;
	return TBLDRV_EENTRYNOTFOUND; 		

found:

	// Refresh age value of UDP connection.
	//  (The age value of TCP is updated by _rtl8651_l4TcpMonitorIn/Out().
	if ( pEntry->tcp == FALSE )
		pEntry->age = rtl8651GlobalControl.udpTimeout;

	assert(pEntry);
	*ppEntry= pEntry;
	return SUCCESS;
}


/*
@func rtl8651_tblDrv_naptTcpUdpFlowEntry_t *	| _rtl8651_findNaptConnection	| Table driver internal use. Given local IP, local port and protocol, find whether there already exists a outbound flow with these 3 tuples in software table.
@parm uint32 | tblIndex		| NAPT entry index calculated from hash algorithm selected.
@parm int8	| isTcp		| TRUE for TCP, FALSE for UDP
@parm ipaddr_t | intIp | <p *entry> pointers to the temp sotrage provided by caller. If entry already cached in softare, <p *entry>, on return, points to that cached connection. Otherwise, caller's temp storage is used to save entry read from ASIC.
@parm uint16	 | intPort	| Don't read from cache. Read from ASIC directly
@parm ipaddr_t | remoteIp | 
@parm uint16	 | remotePort	|
@rvalue Non-NULL	| 
@comm 
Table driver internal use. Given local IP, local port and protocol, find whether there already exists a connection with these 3 tuples in software table.
@devnote
Calculated hash result <p tblIndex> is always used for outbound flow. 
Search only in the outbound list of <p tblIndex>-th NAPT line. 
This function is exactly-matched.
See partially-matched: _rtl8651_getOutboundNaptFlow()
 */
rtl8651_tblDrv_naptTcpUdpFlowEntry_t *  
_rtl8651_findNaptConnection( uint32 tblIndex, int8 isTcp, ipaddr_t intIp, uint16 intPort,
                                                          ipaddr_t remoteIp, uint16 remotePort ) 
{
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *thisConn, *candConn;
	rtl8651DrvNaptLine_t *thisLine = &DrvTbl.tcpudp[tblIndex];
	uint16 i;
	uint16 allowSp;

	allowSp = isTcp & ALLOW_SP;
	isTcp &= 1;
	candConn = NULL;
	
	// the search order is ASIC -> CANDIDATE
	for( i = NAPT_ASIC_CACHED_OUT; i < NAPT_TOTAL_LISTS; i+=2 )
	{
		/* We DO search ALG in this function.
		 * Therefore, fast path must check ALG call-back function to ensure the found entry is in fast path.
		if ( i == NAPT_ALG_OUT ) continue;
		 */
	
		SLIST_FOREACH( thisConn, &thisLine->list[i], nextOut )
		{
			if( thisConn->age > 0 && 
			    (thisConn->tcp==1 ?TRUE :FALSE) == isTcp && 
			    thisConn->insideLocalIpAddr == intIp &&
			    thisConn->insideLocalPort == intPort )
			{
				// The extIp and extPort is matched.
				// In addition, check remoteIp and remotePort if necessary.

				// if we want to find a connection with specified IP and port.
				if ( remoteIp != 0 && remotePort != 0 &&
				     thisConn->dstIpAddr == remoteIp &&
				     thisConn->dstPort == remotePort )
				{
					return thisConn; // bingo! the BEST match.
				}
				// else if we want to find a connection without specified IP and port.
				else if ( remoteIp == 0 && remotePort == 0 &&
				          thisConn->dstIpAddr == 0 &&
				          thisConn->dstPort == 0 )
				{
					return thisConn; // bingo! we've got one.
				}
				else if ( allowSp && thisConn->dstIpAddr==0 && thisConn->dstPort == 0 )
				{
					candConn = thisConn; // bingo! we've got one.
				}
			}
		}
	}
	
	return candConn;
}


/*
@func rtl8651_tblDrv_naptTcpUdpFlowEntry_t *	| _rtl8651_findNaptConnectionByExtIpPort	|  Internal use only. Table driver internal use. Given external port and protocol, find whether this port number already inuse in software.
@parm int8	| isTcp		| TRUE for TCP, FALSE for UDP.
@parm uint16 | extPort 	| The external port searching for.
@rvalue Non-NULL	| Port alrady in use. Returns the connection pointer.
@rvalue NULL		| Port NOT inuse. No connection found.
@comm 
Table driver internal use. Given external port and protocol, find whether this port number already inuse in software.
This function is exactly-matched.
See partially-matched: _rtl8651_getInboundNaptFlow()
 */
rtl8651_tblDrv_naptTcpUdpFlowEntry_t *  
_rtl8651_findNaptConnectionByExtIpPort( int8 isTcp, ipaddr_t ExtIp, uint16 ExtPort,
                                        ipaddr_t remoteIp, uint16 remotePort ) 
{
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *thisConn, *candConn;
	rtl8651DrvNaptLine_t *thisLine;
	uint16 i, j;
	uint16 allowSP;
#ifdef SUPPORT_HASH3
	uint32 idx[2];
#else
	uint32 idx[1];
#endif

	// check non-sense IP and port
	//assert( ExtIp != 0 && ExtPort != 0 );
	//assert( ( remoteIp == 0 && remotePort == 0 ) || ( remoteIp != 0 && remotePort != 0 ) );
	if (!(ExtIp != 0 /* Louis note: ExtPort is allowed now! For CDRouter. && ExtPort != 0*/))
		return NULL;
	if (!((remoteIp == 0 && remotePort == 0 ) || ( remoteIp != 0 && remotePort != 0 )))
		return NULL;

	/*
	rtlglue_printf( "findNaptConnectionByExtIpPort(): isTcp=%d  Ext(%08x:%-5d)  Rem(%08x:%-5d)\n",
	                isTcp, ExtIp, ExtPort, remoteIp, remotePort );
	*/

	allowSP = isTcp & ALLOW_SP;
	isTcp &= 1;
	candConn = NULL;

#ifdef SUPPORT_HASH3
	/* search Hash3, then Hash2 */
	idx[0] = rtl8651_naptTcpUdpTableIndex( isTcp, ExtIp, ExtPort, remoteIp, remotePort );
	idx[1] = ExtPort&(RTL8651_TCPUDPTBL_SIZE-1);
#else
	idx[0] = ExtPort&(RTL8651_TCPUDPTBL_SIZE-1);
#endif

	for( j = 0; j < sizeof(idx)/sizeof(idx[0]); j++ )
	{
		thisLine = &DrvTbl.tcpudp[idx[j]];
		
		// the search order is ASIC -> CANDIDATE
		for( i = NAPT_ASIC_CACHED_IN; i < NAPT_TOTAL_LISTS; i+=2 )
		{
			/* We DO search ALG in this function.
			 * Therefore, fast path must check ALG call-back function to ensure the found entry is in fast path.
			if ( i == NAPT_ALG_IN ) continue;
			 */
			
			SLIST_FOREACH( thisConn, &thisLine->list[i], nextIn )
			{
				/*
				rtlglue_printf( "                                 isTcp=%d  Ext(%08x:%-5d)  Rem(%08x:%-5d)\n",
				                thisConn->tcp, thisConn->insideGlobalIpAddr, thisConn->insideGlobalPort,
				                thisConn->dstIpAddr, thisConn->dstPort );
				*/
				
				if( thisConn->age > 0 && 
				    (thisConn->tcp==1 ?TRUE :FALSE) == isTcp && 
				    thisConn->insideGlobalIpAddr== ExtIp &&
				    thisConn->insideGlobalPort == ExtPort )
				{
					// The intIp and intPort is matched.
					// In addition, check remoteIp and remotePort if necessary.

					// if we want to find a connection with specified IP and port.
					if ( remoteIp != 0 && remotePort != 0 &&
					     thisConn->dstIpAddr == remoteIp &&
					     thisConn->dstPort == remotePort )
					{
						return thisConn; // bingo! the BEST match.
					}
					// else if we want to find a connection without specified IP and port.
					else if ( remoteIp == 0 && remotePort == 0 &&
					          thisConn->dstIpAddr == 0 &&
					          thisConn->dstPort == 0 )
					{
						return thisConn; // bingo! we've got one.
					}
					else if ( allowSP && thisConn->dstIpAddr == 0 && thisConn->dstPort == 0)
					{
						candConn = thisConn;
					}
				}
			}
		}
	}
	
	return candConn;
}

/*
@func rtl8651_tblDrv_naptTcpUdpFlowEntry_s  *	| _rtl8651_addNaptAlgConnection	| Internal use only. Add a ALG connection to software table.
@parm int8 		| assigned	| Use the external IP and port selected by caller.
@parm int8 		| flowType	| Type of connections adding. Possible values: UDP_FLOW, TCP_LISTEN_FLOW, TCP_OUTBOUND_SYN_RCVD_FLOW, TCP_INBOUND_SYN_RCVD_FLOW, TCP_ESTABLISHED_FLOW.
@parm ipaddr_t 	| intIP	| Internal IP.
@parm uint16 	| intPort	| Internal Port.
@parm ipaddr_t *	| extIP	| placeholder for returned External IP
@parm uint16	*	| extPort	| placeholder for returned External port.
@parm ipaddr_t	| dstIP	| Destination IP
@parm uint16		| dstPort	| Destination port
@rvalue Non-NULL	| ALG connection added to software table.
@rvalue NULL	| Failed to add a new ALG connection.
@comm 
For some NAPT flows such as ftp control channel, the forwarding engine takes care of all their traffic. 
Hence it is no need to write such flows into ASIC. 
Returned <p extIP> & <p extIP> derived from classic hashing algorithm
	
Function name implies this is only for ALG. Connection added by this function doesn't affect the collision bit of NAPT table.
This means user should set ALG table to trap packets of added ALG connections. ASIC NAPT table is not responsible for 
trapping them.

Deletion of added ALG flow relies on software aging mechanism. No specific API for ALG NAPT flow deletion

*/
rtl8651_tblDrv_naptTcpUdpFlowEntry_t *
_rtl8651_addNaptAlgConnection( int8 assigned, int8 flowType, 
                               ipaddr_t insideLocalIpAddr, uint16 insideLocalPort, 
                               ipaddr_t *insideGlobalIpAddr, uint16 *insideGlobalPort,
                               ipaddr_t dstIpAddr, uint16 dstPort )
{
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *tempNaptTcpUdpFlowPtr;
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s entry, *pEntry = &entry;
	uint16 inIdx, outIdx;
	uint16 offset;
	uint32 isTcp = (flowType?TRUE:FALSE);
//	int8 isExternal, is_gwip;

	assert( insideGlobalIpAddr != NULL && insideGlobalPort != NULL );

#ifdef ADD_NAPT_DEBUG
	rtlglue_printf("[%s:%d] _rtl8651_addNaptAlgConnection(assigned=%d,flowType=%d,\n  intIp=%d.%d.%d.%d,intPort=%d,\n  extIp=%d.%d.%d.%d,extPort=%d,\n  dstIp=%d.%d.%d.%d,dstPort=%d),.\n",
	               __FILE__,__LINE__,assigned,flowType,NIPQUAD(insideLocalIpAddr),insideLocalPort,NIPQUAD(*insideGlobalIpAddr),*insideGlobalPort,NIPQUAD(dstIpAddr),dstPort);
#endif

	// Check given IP type

//	if (_rtl8651_getIpInfo(insideLocalIpAddr, NULL, &isExternal, &is_gwip)==NULL || isExternal == TRUE || is_gwip == TRUE)
	if (_rtl8651_intNetIpAddr(insideLocalIpAddr)==FALSE)		
	{	
#ifdef ADD_NAPT_DEBUG
		rtlglue_printf( "[%s:%d] _rtl8651_getIpInfo() failed.\n", __FUNCTION__,__LINE__ );
#endif
		return NULL;
	}

	// If the destination IP is local server IP address, this connection cannot be NAPTed.
	if( dstIpAddr && _rtl8651_localServerIpAddr( dstIpAddr ) == TRUE )
	{	
#ifdef ADD_NAPT_DEBUG
		rtlglue_printf( "[%s:%d] _rtl8651_localServerIpAddr() == TRUE.\n", __FUNCTION__,__LINE__ );
#endif
		return NULL;
	}

	// assert enough free entry
	if( CTAILQ_TOTAL( &DrvTbl.freeList.tcpudp ) == 0 )
	{	
#ifdef ADD_NAPT_DEBUG
		rtlglue_printf( "[%s:%d] no free entry.\n", __FUNCTION__,__LINE__ );
#endif
		return NULL;
	}

	//Calulate entry index by hash2
	outIdx = rtl8651_naptTcpUdpTableIndex( (flowType==UDP_FLOW? FALSE: TRUE), 
	                                       insideLocalIpAddr, insideLocalPort, 0, 0 );

	/*
	 * Louis note:
	 *  In the following code, there exists mechanism for non-NAT-friendly protocol, such as SIP/L2TP.
	 *  For those protocol that assists source port cannot be translated by gateway,
	 *    we must utilize 2 ALG connections to implement.
	 *
	 *  When the first packet comes from inside LAN, the ExtPort may be not assigned.
	 *    So, ROME driver can only search in Hash2 fashion.
	 *  After the second packet comes from outside WAN, the ExtPort is assigned.
	 *    So, the ROME driver can search in 'inbound' list.
	 */
	if ( assigned & EXTIPPORT_ASSIGNED )
	{
		/* This is for the incoming packet. */
		if( SUCCESS == _rtl8651_getInboundAlgConnection( (flowType==UDP_FLOW? FALSE: TRUE) , 
		                                                  *insideGlobalIpAddr, *insideGlobalPort,
		                                                  dstIpAddr, dstPort, &pEntry ) )
		{	
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf( "[%s:%d] Connection Duplicate (Inbound).\n", __FUNCTION__,__LINE__ );
#endif
			return NULL;
		}
	}
	else
	{
		/* This is for the outgoing packet. */
		if( SUCCESS == _rtl8651_getOutboundAlgConnection( (flowType==UDP_FLOW? FALSE: TRUE) , 
		                                                  insideLocalIpAddr, insideLocalPort,
		                                                  dstIpAddr, dstPort, &pEntry ) )
		{	
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf( "[%s:%d] Connection Duplicate (Outbound).\n", __FUNCTION__,__LINE__ );
#endif
			return NULL;
		}
	}

	if( !(assigned & EXTIPPORT_ASSIGNED) )
	{
		/* Select an usable external IP address and external port */
		*insideGlobalIpAddr = _rtl8651_getNaptInsideGlobalIpAddr( (flowType==UDP_FLOW? FALSE: TRUE),
		                                                         insideLocalIpAddr, insideLocalPort,
		                                                         dstIpAddr, dstPort );
		// no available external IP
		if ( *insideGlobalIpAddr == 0 )
		{	
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf( "[%s:%d] no available external IP.\n", __FUNCTION__,__LINE__ );
#endif
			return NULL;
		}

		/* The user does not specify ExtPort, use outIdx (hash2). */
		inIdx = outIdx;
			
		// select an external port
		if( _rtl8651_selectNaptPortOffset( inIdx, isTcp, *insideGlobalIpAddr, &offset ) == FAILED ) 
		{	
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf( "[%s:%d] _rtl8651_selectNaptPortOffset() FAILED.\n", __FUNCTION__,__LINE__ );
#endif
			return NULL;
		}
		*insideGlobalPort = offset << RTL8651_TCPUDPTBL_BITS | inIdx;
	}
	else if ( *insideGlobalIpAddr != 0 && *insideGlobalPort != 0 )
	{
		// the external ip and external port are specified.
		pEntry = &entry;
		if( SUCCESS == _rtl8651_getInboundAlgConnection( (flowType==UDP_FLOW? FALSE: TRUE) ,
		                                               *insideGlobalIpAddr, *insideGlobalPort,
		                                               dstIpAddr, dstPort, &pEntry ) )
		{	
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf( "[%s:%d] Duplicate connection found.\n", __FUNCTION__,__LINE__ );
#endif
			return NULL;
		}
		inIdx = *insideGlobalPort & (RTL8651_TCPUDPTBL_SIZE-1);
	}
	else
	{
		// the external ip and port are assigned, but not specified.
#ifdef ADD_NAPT_DEBUG
		rtlglue_printf( "[%s:%d] the external ip and port are assigned, but not specified.\n", __FUNCTION__,__LINE__ );
#endif
		return NULL;
	}

	// Check connection control system
	if ( assigned & DOS_NAPT_ENCOUNT )
	{
		assert( ((assigned & DOS_NAPT_FROMINTERNAL) && insideLocalIpAddr) ||
			    ((!(assigned & DOS_NAPT_FROMINTERNAL)) && dstIpAddr) );

		if (	_rtl8651_dosNaptConnCheck(
			isTcp,
			(assigned & DOS_NAPT_FROMINTERNAL)?insideLocalIpAddr:dstIpAddr,
			(assigned & DOS_NAPT_FROMINTERNAL)?RTL8651_LAN2WAN:RTL8651_WAN2LAN) != SUCCESS)
		{
#ifdef ADD_NAPT_DEBUG
		rtlglue_printf( "[%s:%d] the connection count exceed.\n", __FUNCTION__,__LINE__ );
#endif
			return NULL;
		}
	}

	//Add entry to table	
	tempNaptTcpUdpFlowPtr = CTAILQ_FIRST( &DrvTbl.freeList.tcpudp );
	memset( tempNaptTcpUdpFlowPtr, 0, offsetof(struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s, nextOut ) );
	tempNaptTcpUdpFlowPtr->connectionType= NAPT_TYPE_ALG_CONNECTION;
	tempNaptTcpUdpFlowPtr->dstIpAddr = dstIpAddr;
	tempNaptTcpUdpFlowPtr->dstPort = dstPort;
	tempNaptTcpUdpFlowPtr->insideLocalIpAddr = insideLocalIpAddr;
	tempNaptTcpUdpFlowPtr->insideLocalPort = insideLocalPort;
	tempNaptTcpUdpFlowPtr->insideGlobalIpAddr = *insideGlobalIpAddr;
	tempNaptTcpUdpFlowPtr->insideGlobalPort = *insideGlobalPort;
	tempNaptTcpUdpFlowPtr->hashing = NAPT_HASH2;
	tempNaptTcpUdpFlowPtr->alive = 1;
	tempNaptTcpUdpFlowPtr->fromDrv = 1;
	tempNaptTcpUdpFlowPtr->tcp = (flowType)? 1: 0;
	tempNaptTcpUdpFlowPtr->tcpFlag = flowType? flowType-1:0; //meaningless for UDP
	tempNaptTcpUdpFlowPtr->toProtocolStack = (assigned & TO_PROTOCOL_STACK)?TRUE:FALSE;
	tempNaptTcpUdpFlowPtr->dosEnCount = FALSE;
	tempNaptTcpUdpFlowPtr->dosFromInternal = FALSE;
	//For ALG
	tempNaptTcpUdpFlowPtr->outboundTblIdx = outIdx;
	tempNaptTcpUdpFlowPtr->outListType= NAPT_ALG_OUT;
	tempNaptTcpUdpFlowPtr->inboundTblIdx = inIdx;
	tempNaptTcpUdpFlowPtr->inListType= NAPT_ALG_IN;

	tempNaptTcpUdpFlowPtr->out_rt_t = NULL; 
	tempNaptTcpUdpFlowPtr->in_rt_t = (void *)_rtl8651_getRoutingEntry(insideLocalIpAddr);
	tempNaptTcpUdpFlowPtr->out_macInfo_t = NULL;
	tempNaptTcpUdpFlowPtr->in_macInfo_t = NULL;
	assert(tempNaptTcpUdpFlowPtr->in_rt_t);
	tempNaptTcpUdpFlowPtr->dsid = 0;


	if( flowType == TCP_ESTABLISHED_FLOW )
		tempNaptTcpUdpFlowPtr->age = rtl8651GlobalControl.tcpLongTimeout;
	else
		tempNaptTcpUdpFlowPtr->age = (flowType)?rtl8651GlobalControl.tcpFastTimeout: rtl8651GlobalControl.udpTimeout;

	// link the list
	CTAILQ_REMOVE( &DrvTbl.freeList.tcpudp, tempNaptTcpUdpFlowPtr, next );
	CTAILQ_INSERT_HEAD( &DrvTbl.inuseList.tcpudp, tempNaptTcpUdpFlowPtr, next );
	SLIST_INSERT_HEAD( &DrvTbl.tcpudp[outIdx].list[NAPT_ALG_OUT], tempNaptTcpUdpFlowPtr, nextOut );
	SLIST_INSERT_HEAD( &DrvTbl.tcpudp[inIdx].list[NAPT_ALG_IN], tempNaptTcpUdpFlowPtr, nextIn );

#ifdef CACHE_FIND_NAPT_RESULT
	/* If this connection is in the cache, invalid the cache. */
	if ( 1 )
	{
#ifdef CACHE_FIND_NAPT_DEBUG_RESULT
		rtlglue_printf( "*** addNaptAlgConnection() hit cache! cachedPEntry=%p\n", cachedPEntry );
#endif/*CACHE_FIND_NAPT_DEBUG_RESULT*/
		INVALID_FIND_CACHE();
	}
#endif/*CACHE_FIND_NAPT_RESULT*/

#ifdef CACHE_DOS_COUNT_NAPT_RESULT
	INVALID_DOS_COUNT_CACHE();
#endif

	// dos process
	if ( assigned & DOS_NAPT_ENCOUNT )
	{
		tempNaptTcpUdpFlowPtr->dosEnCount = TRUE;

		if ( isTcp )
		{
			if ( assigned & DOS_NAPT_FROMINTERNAL )
			{
				rtl8651_tblDrvNaptCounter.dosCountedFromInternalTcpCnt++;
			}

			rtl8651_tblDrvNaptCounter.dosCountedTcpCnt++;
		} else
		{
			if ( assigned & DOS_NAPT_FROMINTERNAL )
			{
				rtl8651_tblDrvNaptCounter.dosCountedFromInternalUdpCnt++;
			}

			rtl8651_tblDrvNaptCounter.dosCountedUdpCnt++;
		}

		if ( assigned & DOS_NAPT_FROMINTERNAL )
		{
			tempNaptTcpUdpFlowPtr->dosFromInternal = TRUE;

			_rtl8651_dosSrcConnCtrlUpdateConnection(FALSE, isTcp, insideLocalIpAddr);
		}else
		{
			tempNaptTcpUdpFlowPtr->dosFromInternal = FALSE;

			_rtl8651_dosSrcConnCtrlUpdateConnection(FALSE, isTcp, dstIpAddr);
		}
	}


	rtl8651_tblDrvNaptCounter.drvAddCount++;
	return tempNaptTcpUdpFlowPtr;	
}

/*
@func int32		| _rtl8651_delNaptAlgConnection	| Internal use only. Delete an ALG control connection.
@parm rtl8651_tblDrv_naptTcpUdpFlowEntry_t * | delConn	| pointer to the Connection to be deleted.
@rvalue SUCCESS	| ALG control connection successfully deleted.
@rvalue FAILED	| Failed to delete ALG control connection.
@comm 
Delete an ALG control connection.
*/
int32 _rtl8651_delNaptAlgConnection( rtl8651_tblDrv_naptTcpUdpFlowEntry_t *delConn )
{
	rtl8651DrvNaptLine_t *outLine, *inLine;

	assert( delConn );

	if ( delConn->inListType!=NAPT_ALG_IN ||
	     delConn->outListType!=NAPT_ALG_OUT )
	{
		rtlglue_printf("[%s:%d] assert error! delConn->inListType=%d delConn->inListType=%d\n",
		               __FILE__, __LINE__, delConn->inListType, delConn->outListType );
		return FAILED;
	}

#ifdef CACHE_FIND_NAPT_RESULT
	/* If this connection is in the cache, invalid it. */
	if ( delConn == cachedPEntry )
	{
#ifdef CACHE_FIND_NAPT_DEBUG_RESULT
		rtlglue_printf("-: cache flush (delAlgConnection)...\n" );
#endif/*CACHE_FIND_NAPT_DEBUG_RESULT*/
		INVALID_FIND_CACHE();
	}
#endif/*CACHE_FIND_NAPT_RESULT*/

#ifdef FLUSH_NAPT_OLDEST_CONNECTION2
	if ( candidateToBeFreed == delConn )
	{
		candidateToBeFreed = NULL;
	}
#endif/*FLUSH_NAPT_OLDEST_CONNECTION2*/

	/* Update connection counting cache in dos system */
	if ( delConn->dosEnCount == TRUE )
	{
#ifdef CACHE_DOS_COUNT_NAPT_RESULT
		INVALID_DOS_COUNT_CACHE();
#endif
		_rtl8651_dosSrcConnCtrlUpdateConnection(TRUE, delConn->tcp,
			(delConn->dosFromInternal)?(delConn->insideLocalIpAddr):(delConn->dstIpAddr));

		if ( delConn->tcp )
		{
			if ( delConn->dosFromInternal )
			{
				rtl8651_tblDrvNaptCounter.dosCountedFromInternalTcpCnt--;
			}

			rtl8651_tblDrvNaptCounter.dosCountedTcpCnt--;
		}
		else
		{
			if ( delConn->dosFromInternal )
			{
				rtl8651_tblDrvNaptCounter.dosCountedFromInternalUdpCnt--;
			}

			rtl8651_tblDrvNaptCounter.dosCountedUdpCnt--;
		}
	}

	inLine = &DrvTbl.tcpudp[delConn->inboundTblIdx];
	outLine = &DrvTbl.tcpudp[delConn->outboundTblIdx];
	
	SLIST_REMOVE( &inLine->list[NAPT_ALG_IN], delConn, 
	              rtl8651_tblDrv_naptTcpUdpFlowEntry_s, nextIn );
	SLIST_REMOVE( &outLine->list[NAPT_ALG_OUT], delConn, 
	              rtl8651_tblDrv_naptTcpUdpFlowEntry_s, nextOut );

	//delConn's alive and age all reset to 0 by this memset()...
	memset(delConn, 0, offsetof(struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s, next));
	assert(delConn->alive==0);
	delConn->outboundTblIdx = delConn->outListType = -1;
	delConn->inboundTblIdx = delConn->inListType = -1;
	CTAILQ_REMOVE( &DrvTbl.inuseList.tcpudp, delConn, next );
	CTAILQ_INSERT_HEAD( &DrvTbl.freeList.tcpudp, delConn, next );

	return SUCCESS;
}


/*
@func ipaddr_t	| _rtl8651_getNaptInsideGlobalIpAddr	| Internal use only. Given 5 connection tuples, returns the external IP address to use when doing NAPT.
@parm int8	| isTcp		| TRUE for TCP, FALSE for UDP.
@parm ipaddr_t | internalIP	| Internal IP.
@parm uint16	| internalPort	| Internal Port.
@parm ipaddr_t | destIP	| Destination IP.
@parm uint16	| destPort	| Destination Port.
@rvalue 0	| No external IPs configured for NAPT use yet.
@rvalue non-0 | The IP address selected by driver.
@comm 
Given 5 tuples, let driver find a external  IP to use in NAPT.
@devnote
Although all 5 tuples are given, external IP is determined only by default route now.
 */
ipaddr_t _rtl8651_getNaptInsideGlobalIpAddr(int8 isTcp, ipaddr_t insideLocalIpAddr, uint16 insideLocalPort, ipaddr_t dstIpAddr, uint16 dstPort) 
{
	/* Lookup IP table to find NAPT IP.
	 * In current version, we do NOT consider 5 connection tuples.
	 */
	rtl8651_tblDrv_ipEntry_t *ipEntry;
	int i;

	for( i = 0; i < RTL8651_IPTABLE_SIZE; i++ )
	{
		ipEntry = &DrvTbl.iptbl[i];

		if ( ipEntry->valid==FALSE ) continue;
		if (( ipEntry->type == IP_ATTR_NAPT ) /* && (ipEntry->extip != softNaptIp)*/)
			return ipEntry->extip;
	}

	return 0;
}


/*
@func int32	| _rtl8651_selectNaptPortOffset	|Internal use only. Pick a vacant offset value on given NAPT entry.
@parm uint32	| tblIdx		| Entry index of NAPT table
@parm uint32	| isTcp			| TCP or UDP
@parm ipaddr_t	| wantedIp		| Wanted IP address to use. (not NULL)
@parm uint16 * | offset	| Placeholder for selected offset value on return.
@rvalue SUCCESS	| A vacant offset value was found and returned via <p *offset>.
@rvalue FAILED 	| All offset values inuse. 
@comm 
Given entry index <p tblIdx>, find a unused, available offset for the MSB 6 bits of external port number.
offset values in both ASIC and software table are excluded.
@devnote
External port number 0 is NEVER used nor returned by this function.
*/
static int32 _rtl8651_selectNaptPortOffset(uint32 tblIdx, uint32 isTcp, ipaddr_t wantedIp, uint16 *offset) 
{
	uint32 mask[2]; //an empty bitmap. flatten all possible offset values.
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *thisConn;
	rtl8651DrvNaptLine_t *tcpudpPtr;
	uint32 maxUsed=0, thisOffset;
	int32 i;
	isTcp = (isTcp?TRUE:FALSE);
	tcpudpPtr = &DrvTbl.tcpudp[tblIdx];

	assert( tblIdx < RTL8651_TCPUDPTBL_SIZE );
	assert( wantedIp );

	mask[0] = mask[1] = 0;

	if(tblIdx==0)
		mask[0]=1;	//avoid selecting port 0.

	/* check server port usage */
	{
		rtl8651_tblDrv_networkIntfTable_t *netif_t;
		rtl8651_tblDrv_naptServerPortEntry_t *servPort_t;
		uint32 netIdx;

		if (_rtl8651_extIpAddr(wantedIp, &netIdx) == FALSE ||
		    _rtl8651_isNaptMapping(wantedIp) == FALSE) 
		{
			GET_NETWORK_INTERFACE(extIfName,&netIdx);
			//return TBLDRV_ENOTEXTIP;
		}
		
		netif_t = &DrvTbl.netIface[netIdx];
		assert(netif_t->valid == 1);
		
		SLIST_FOREACH(servPort_t, &netif_t->nextNaptServerPort, nextServerPort) 
		{
			if ( servPort_t->tcp == isTcp &&
			     servPort_t->globalIp == wantedIp )
			{
				int32 firstPort, lastPort, port;

				// the port number that first matches the tblIdx.
				firstPort = ( servPort_t->globalPort & (RTL8651_TCPUDPTBL_SIZE-1) ) + tblIdx;
				if ( firstPort > servPort_t->globalPort ) firstPort += RTL8651_TCPUDPTBL_SIZE;

				// the port number that last matches the tblIdx.
				lastPort = ( servPort_t->endGlobalPort & (RTL8651_TCPUDPTBL_SIZE-1) ) + tblIdx;
				if ( lastPort > servPort_t->endGlobalPort ) lastPort -= RTL8651_TCPUDPTBL_SIZE;
				
				// mask every port that matches the tblIdx.
				for( port = firstPort;
				     port <= lastPort;
				     port += RTL8651_TCPUDPTBL_SIZE )
				{
					thisOffset = ( port >> RTL8651_TCPUDPTBL_BITS );
					mask[thisOffset>>5] |= ( (uint32)1 << (thisOffset&0x1f) );
					if( thisOffset > maxUsed )
						maxUsed = thisOffset;
				}
			}
		}
	}

	// Consider NAPT Connections (including ALG connections).
	for( i = NAPT_ASIC_CACHED_IN; i < NAPT_TOTAL_LISTS; i+=2 )
	{
		SLIST_FOREACH( thisConn, &tcpudpPtr->list[i], nextIn )
		{
			assert( SLIST_NEXT( thisConn, nextIn ) != thisConn );
			if ( thisConn->insideGlobalIpAddr != wantedIp ) continue;
			
			thisOffset = ( thisConn->insideGlobalPort >> RTL8651_TCPUDPTBL_BITS );
			mask[thisOffset>>5] |= ( (uint32)1 << (thisOffset&0x1f) );
			if( thisOffset > maxUsed )
				maxUsed = thisOffset;
		}
	}

	/* Check if start == end, then always use 'start'  */
	if ( rtl8651GlobalControl.l4PortRangeStart == rtl8651GlobalControl.l4PortRangeEnd )
	{
		/* check if ISCLEARED */
		if ( ( mask[rtl8651GlobalControl.l4PortRangeStart>>5] & 
		       ((uint32)1<<(rtl8651GlobalControl.l4PortRangeStart&0x1f) ) ) == 0 )
		{
			*offset = rtl8651GlobalControl.l4PortRangeStart;
			return SUCCESS;
		}
		//port already taken, return FAILED
		return FAILED;
	}
	else
	{
		uint16 candidate = maxUsed + 1;
		uint8 count = 0;

		if ( candidate < rtl8651GlobalControl.l4PortRangeStart ) 
			candidate = rtl8651GlobalControl.l4PortRangeStart;

		do
		{
			if(candidate>rtl8651GlobalControl.l4PortRangeEnd)
				candidate=rtl8651GlobalControl.l4PortRangeStart;
				
			/* check if ISCLEARED */
			if ( ( mask[candidate>>5] & 
			       ((uint32)1<<(candidate&0x1f)) ) == 0 )
			{
				*offset = candidate;
				return SUCCESS;
			}
			candidate++, count++;
		} while ( count < (rtl8651GlobalControl.l4PortRangeEnd- rtl8651GlobalControl.l4PortRangeStart+1) );
	}
	/* no available offset found */
	return FAILED;
}


/*
@func int32		| _rtl8651_NaptSafeSwapToAsic	| This function can safely swap a NAPT connection to ASIC.
@parm struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *	| drvEntry	| pointer to a connection
@comm 
This function can swap a NAPT connection into ASIC.
If ASIC entry is hold by other connection, this function will swap the holding connection and write <p drvEntry> into ASIC entry(s).
An ambiguous Hash 2 connection will NOT be writen to ASIC.
Typically, this function is called by _rtl8651_updateNaptConnection() and _rtl8651_softwareNaptAging().
*/
void _rtl8651_NaptSafeSwapToAsic( rtl8651_tblDrv_naptTcpUdpFlowEntry_t *drvEntry )
{
	rtl8651DrvNaptLine_t *thisLine;
	rtl8651_tblDrv_naptTcpUdpFlowEntry_t *asicCache;

	assert( drvEntry );
	/* This is a pure software, never swap to ASIC. */
	if ( drvEntry->pureSoftware ) return;
	
	// Hash1 and Hash2 have different process.	
	if ( drvEntry->hashing == NAPT_HASH1 ) 
	{
		// Who is using ASIC ?
		assert( drvEntry->inboundTblIdx == drvEntry->outboundTblIdx );
		
		thisLine = &DrvTbl.tcpudp[drvEntry->inboundTblIdx];
		asicCache = SLIST_FIRST(&(thisLine->list[NAPT_ASIC_CACHED_OUT]));
		if( asicCache == NULL ) asicCache = SLIST_FIRST(&(thisLine->list[NAPT_ASIC_CACHED_IN]));

		// If a connection is holding the ASIC entry, first swap it to candidate list.
		if ( asicCache && asicCache != drvEntry )
		{
			if ( asicCache->hashing == NAPT_HASH1 )
			{
				// The holder is a NAPT_HASH1 connection.
				_rtl8651_changeNaptFlowListType( asicCache, NAPT_CANDIDATE_IN, NAPT_CANDIDATE_OUT );
			}
			else
			{
				// The holder is a NAPT_HASH2 connection.
				if ( asicCache->inboundTblIdx == drvEntry->inboundTblIdx )
					_rtl8651_changeNaptFlowListType( asicCache, NAPT_CANDIDATE_IN, asicCache->outListType );
				if ( asicCache->outboundTblIdx == drvEntry->outboundTblIdx )
					_rtl8651_changeNaptFlowListType( asicCache, asicCache->inListType, NAPT_CANDIDATE_OUT );
			}
		}

		// After swap victim connection to software, then, swap candidate into ASIC entry.
		_rtl8651_changeNaptFlowListType( drvEntry, NAPT_ASIC_CACHED_IN, NAPT_ASIC_CACHED_OUT );
	}
	else
	{
		// NAPT_HASH2
		
		int bInboundAmbiguous, bOutboundAmbiguous;
		int bInboundAsicEmpty, bOutboundAsicEmpty;
		_rtl8651_AvoidAmbiguousHash2InAsic( drvEntry, &bInboundAmbiguous, &bOutboundAmbiguous, &bInboundAsicEmpty, &bOutboundAsicEmpty );

		//  inbound entry
		if ( bInboundAmbiguous == FALSE )
		{
			// Who is using inbound ASIC entry.
			thisLine = &DrvTbl.tcpudp[drvEntry->inboundTblIdx];
			asicCache = SLIST_FIRST(&(thisLine->list[NAPT_ASIC_CACHED_OUT]));
			if( asicCache == NULL ) asicCache = SLIST_FIRST(&(thisLine->list[NAPT_ASIC_CACHED_IN]));
			
			// If a connection is holding the ASIC entry, first swap it to candidate list.
			if ( asicCache && asicCache != drvEntry )
			{
				if ( asicCache->hashing == NAPT_HASH1 )
				{
					// The holder is a NAPT_HASH1 connection.
					_rtl8651_changeNaptFlowListType( asicCache, NAPT_CANDIDATE_IN, NAPT_CANDIDATE_OUT );
				}
				else
				{
					// The holder is a NAPT_HASH2 connection.
					if ( asicCache->inboundTblIdx == drvEntry->inboundTblIdx )
						_rtl8651_changeNaptFlowListType( asicCache, NAPT_CANDIDATE_IN, asicCache->outListType );
					if ( asicCache->outboundTblIdx == drvEntry->inboundTblIdx )
						_rtl8651_changeNaptFlowListType( asicCache, asicCache->inListType, NAPT_CANDIDATE_OUT );
				}
			}

			// After swap victim connection to software, then, swap candidate into ASIC entry.
			_rtl8651_changeNaptFlowListType( drvEntry, NAPT_ASIC_CACHED_IN, drvEntry->outListType );
		}
		

		//  outbound entry
		if ( bOutboundAmbiguous == FALSE )
#if 0/* Since holly chhuang has supported multiple nexthop, we do not concern the interface route issue anymore if extIpIdx is availiable. */
		     drvEntry->interfaceRoute == FALSE ) /* If it is an interface route connection, we DO NOT allow outbound entry writen to ASIC! */
#endif/*0*/
		{
			// Who is using outbound ASIC entry.
			thisLine = &DrvTbl.tcpudp[drvEntry->outboundTblIdx];
			asicCache = SLIST_FIRST(&(thisLine->list[NAPT_ASIC_CACHED_OUT]));
			if( asicCache == NULL ) asicCache = SLIST_FIRST(&(thisLine->list[NAPT_ASIC_CACHED_IN]));

			// If a connection is holding the ASIC entry, first swap it to candidate list.
			if ( asicCache && asicCache != drvEntry )
			{
				if ( asicCache->hashing == NAPT_HASH1 )
				{
					// The holder is a NAPT_HASH1 connection.
					_rtl8651_changeNaptFlowListType( asicCache, NAPT_CANDIDATE_IN, NAPT_CANDIDATE_OUT );
				}
				else
				{
					// The holder is a NAPT_HASH2 connection.
					if ( asicCache->inboundTblIdx == drvEntry->outboundTblIdx )
						_rtl8651_changeNaptFlowListType( asicCache, NAPT_CANDIDATE_IN, asicCache->outListType );
					if ( asicCache->outboundTblIdx == drvEntry->outboundTblIdx )
						_rtl8651_changeNaptFlowListType( asicCache, asicCache->inListType, NAPT_CANDIDATE_OUT );
				}
			}
			
			// After swap victim connection to software, then, swap candidate into ASIC entry.
			_rtl8651_changeNaptFlowListType( drvEntry, drvEntry->inListType, NAPT_ASIC_CACHED_OUT );
		}
	}

	/* Louis note that: this counter is the number of attempting swap to ASIC. */
	rtl8651_tblDrvNaptCounter.threadWriteBackCount++;
}


/*
@func int32	| _rtl8651_getUsableExtPortWithIndex	| This program is the front-end of _rtl8651_selectNaptPortOffset(). Typically called by rtl8651_addNaptConnection().
@parm uint32	| idx			| Table index.
@parm uint32	| isTcp			| TCP or UDP
@parm ipaddr_t 	| destIP		| Internal IP.
@rvalue port number	| The spare Port Offset pluses index.
@comm 
This function is used to assign *insideGlobalPort in _rtl8651_addNaptConnection() function.
 */
int32 _rtl8651_getUsableExtPortWithIndex( uint32 idx, uint32 isTcp, ipaddr_t wantedIP, uint16 *ExtPort )
{
	uint16 portOffset;
	
	if ( _rtl8651_selectNaptPortOffset( idx, isTcp, wantedIP, &portOffset ) != SUCCESS )
	{
		return TBLDRV_EGETOFFSETFAIL;
	}
	*ExtPort = ( portOffset << RTL8651_TCPUDPTBL_BITS ) + idx;
	
	return SUCCESS;
}

/*
@func int8	| _rtl8651_getUsableExtIpAndPort	| Given 5 connection tuples, let driver selects a pair of external IP address and port.
@parm int8	| isTcp			| TRUE for TCP, FALSE for UDP.
@parm ipaddr_t 	| internalIP	| Internal IP.
@parm uint16		| internalPort	| Internal Port.
@parm ipaddr_t 	| destIP		| Internal IP.
@parm uint16		| destPort	| Internal Port.
@parm ipaddr_t * 	| extIp		| Destination IP.
@parm uint16 *	| extPort 	| Destination Port.
@rvalue SUCCESS	| A pair of <p extIp> and <p extPort> selected. 
@rvalue FAILED 	| Failed to select either <p extIp> or <p extPort>.
@comm 
Given 5 tuples, let driver select a  pair of external  IP and port to use in NAPT.
If <p destIP> and <p destPort> are both zero, selecte <p extIp> and <p extPort> based on 8651B's liberal hash algorithm, otherwise,
use classic hash algoirithm. Notice that the <p extPort> returned is NOT reserved for caller. The same port number may also be selected by other callers until it is added to software NAPT table.
@devnote
 */
int8 _rtl8651_getUsableExtIpAndPort(int8 isTcp, ipaddr_t sip, uint16 sport, ipaddr_t dip, uint16 dport, ipaddr_t *extip, uint16 *extport) 
{
	uint32 flowTblIdx;
	uint16 offset;
	ipaddr_t extIpSelected;

	assert( extip != NULL && extport != NULL );

	extIpSelected= _rtl8651_getNaptInsideGlobalIpAddr(isTcp, sip, sport, dip, dport);
	if( extIpSelected == 0 )
		return FAILED;
	*extip = extIpSelected;
	
	flowTblIdx = rtl8651_naptTcpUdpTableIndex(isTcp, sip, sport, dip, dport);
	if(_rtl8651_selectNaptPortOffset(flowTblIdx, isTcp, extIpSelected, &offset) == FAILED)
		return FAILED;
	*extport = offset << RTL8651_TCPUDPTBL_BITS | flowTblIdx;
	assert(*extport);
	
	return SUCCESS;
}


/*
@func int32		| _rtl8651_getOutboundNaptFlow	| For forwarding engine. Given 5 tuples in outbound packet, find and return the NAPT flow pointer from software table or ASIC.
@parm int8 		| isTcp	| TRUE for TCP, FALSE for UDP.
@parm ipaddr_t 	| sip		| Internal IP
@parm uint16 	| sport	| Internal port
@parm ipaddr_t	| dip	| Destination IP
@parm uint16		| dport	| Destination port
@parm struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s **		| ppEntry	| pointer to a temp storage from caller
@rvalue SUCCESS					| 	NAPT flow found. 
@rvalue TBLDRV_EENTRYNOTFOUND	| 	Flow not found.
@comm 
Given 5 tuples from output packet, find and get NAPT flow from software table or ASIC if connection exists. This function typically is called by forwarding engine.
If the given 5-tuple cannot find the exactly-matched connection, this function will try to use intIP and intPort to find the partially-matched connection.
*ppEntry should points to an rtl8651_tblDrv_naptTcpUdpFlowEntry_s instacnce on caller's stack. 
If we found the flow in question in table driver,  *ppEntry, on return,points to that flow inside table driver. 
If we found the flow directly from ASIC, then *ppEntry is used to saveASIC snapshot.
If flow not exists on both ASIC and table driver, *ppEntry=NULL is returned
This function is partially-matched.
See exactly-matched: _rtl8651_findNaptConnection().
*/
int32 
_rtl8651_getOutboundNaptFlow( int8 isTcp, ipaddr_t sip, uint16 sport, ipaddr_t dip, uint16 dport,
                              struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s **ppEntry )
{
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *pEntry;
	int32 idx1, idx1_loop;
	int32 idx2;
	int32 ret;
	//assert( ppEntry && *ppEntry );
	//assert( dip != 0 && dport != 0 );

#ifdef CONFIG_RTL865X_ROMEPERF
	rtl8651_romeperfEnterPoint( ROMEPERF_INDEX_NAPT_FIND_OUTBOUND );
#endif

	if ( !(ppEntry /* Louis: we no longer assist this: && *ppEntry */ ) ||
	     !(dip!=0 /* Louis: we no longer assist this: && dport!=0 */ ))
		return FAILED;

#ifdef CACHE_FIND_NAPT_RESULT
#ifdef CACHE_FIND_NAPT_DEBUG_RESULT
	rtlglue_printf("--- getOutbound() -------------------\n" );
	rtlglue_printf("isTcp  = %-5d     %-5d\n", isTcp, cachedIsTcp );
	rtlglue_printf("IntIp  = %x  %x\n",        sip, cachedIntIp );
	rtlglue_printf("IntPort= %-5d     %-5d\n", sport, cachedIntPort );
	rtlglue_printf("RemIp  = %x  %x\n",        dip, cachedRemIp );
	rtlglue_printf("RemPort= %-5d     %-5d\n", dport, cachedRemPort );
	rtlglue_printf("PEntry = %p\n", cachedPEntry );
#endif/*CACHE_FIND_NAPT_DEBUG_RESULT*/

	if ( isTcp==cachedIsTcp && sip==cachedIntIp && sport==cachedIntPort &&
	     dip==cachedRemIp && dport==cachedRemPort )
	{
#ifdef CACHE_FIND_NAPT_DEBUG_RESULT
		rtlglue_printf("+: Hit\n");
#endif/*CACHE_FIND_NAPT_DEBUG_RESULT*/
		*ppEntry = cachedPEntry;
		if ( cachedPEntry ) return SUCCESS; else return TBLDRV_EENTRYNOTFOUND;
	}
#endif/*CACHE_FIND_NAPT_RESULT*/
	
	pEntry = *ppEntry;
	*ppEntry = NULL;

	idx1 = rtl8651_naptTcpUdpTableIndex( isTcp, sip, sport, dip, dport );

	if (enable4WayHash == TRUE)
	{
		// 4-way Hash is enabled.
		int32 i;

		// Try exactly-match first.
		idx1_loop = idx1;
		for( i = 0; i < 4; i++ )
		{
			// Hash 1
			if ( ( pEntry = _rtl8651_findNaptConnection( idx1_loop,  isTcp, sip, sport, dip, dport ) ) ) goto found;
			idx1_loop = WRAPIN_4WAY( idx1_loop, 1 );
		}		
		// Hash 2
		idx2 = rtl8651_naptTcpUdpTableIndex( isTcp, sip, sport, 0, 0 );
		if ( ( pEntry = _rtl8651_findNaptConnection( idx2,  isTcp|ALLOW_SP, sip, sport, dip, dport ) ) ) goto found;
	}
	else
	{
		// 4-way Hash is NOT enabled.
		// Try exactly-match first.
		// use Hash1, then Hash2
		if ( ( pEntry = _rtl8651_findNaptConnection( idx1,  isTcp, sip, sport, dip, dport ) ) ) goto found;
		idx2 = rtl8651_naptTcpUdpTableIndex( isTcp, sip, sport, 0, 0 );
		if ( ( pEntry = _rtl8651_findNaptConnection( idx2,  isTcp|ALLOW_SP, sip, sport, dip, dport ) ) ) goto found;
	}

#ifdef CACHE_FIND_NAPT_RESULT
	/* Cache the connection */
	cachedPEntry = NULL;
#endif/*CACHE_FIND_NAPT_RESULT*/
	
	ret = TBLDRV_EENTRYNOTFOUND;
	goto out;

found:

#if ALLOW_SP==0
	// If this is a Server Port connection, we shall return 'Not found'.
	// Therefore, this connection will pass the slow path.
	// To find the Server Port Connection, use _rtl8651_getServerPortByIntIpPort().
	if ( pEntry->connectionType == NAPT_TYPE_SERVERPORT )
	{
#ifdef CACHE_FIND_NAPT_RESULT
		/* Cache the connection */
		cachedPEntry = NULL;
#endif/*CACHE_FIND_NAPT_RESULT*/
		ret = TBLDRV_EENTRYNOTFOUND;
		goto out;
	}
#endif/*ALLOW_SP==0*/

	// Refresh age value of UDP connection.
	//  (The age value of TCP is updated by _rtl8651_l4TcpMonitorIn/Out().
	if ( pEntry->tcp == FALSE )
		pEntry->age = rtl8651GlobalControl.udpTimeout;
		
	*ppEntry = pEntry;

#ifdef CACHE_FIND_NAPT_RESULT
	/* Cache the connection */
	cachedPEntry = pEntry;
#endif/*CACHE_FIND_NAPT_RESULT*/

	ret = SUCCESS;

out:
#ifdef CACHE_FIND_NAPT_RESULT
	cachedIsTcp = isTcp;
	cachedIntIp = sip;
	cachedIntPort = sport;
	cachedExtIp = 0;
	cachedExtPort = 0;
	cachedRemIp = dip;
	cachedRemPort = dport;
#ifdef CACHE_FIND_NAPT_DEBUG_RESULT
	rtlglue_printf("--- cache recharged getOutbound() -------------------\n" );
	rtlglue_printf("cachedIsTcp  = %-5d\n", cachedIsTcp );
	rtlglue_printf("cachedIntIp  = %x\n", cachedIntIp );
	rtlglue_printf("cachedIntPort= %-5d\n", cachedIntPort );
	rtlglue_printf("cachedRemIp  = %x\n", cachedRemIp );
	rtlglue_printf("cachedRemPort= %-5d\n", cachedRemPort );
	rtlglue_printf("cachedPEntry = %p\n", cachedPEntry );
#endif/*CACHE_FIND_NAPT_DEBUG_RESULT*/
#endif/*CACHE_FIND_NAPT_RESULT*/

#ifdef CONFIG_RTL865X_ROMEPERF
	rtl8651_romeperfExitPoint( ROMEPERF_INDEX_NAPT_FIND_OUTBOUND );
#endif
	
	return ret;
}

/*
@func int32		| _rtl8651_getInboundNaptFlow	| For forwarding engine. Given 5 tuples in inbound packet, find and return the inbound NAPT flow pointer from software table or ASIC.
@parm int8 		| isTcp	| TRUE for TCP, FALSE for UDP.
@parm ipaddr_t 	| gip	| Destination IP in inbound packet. It's one of the gateway's IP.
@parm uint16 	| gport	| Destination port in inbound packet. It's the gateway selected external port.
@parm ipaddr_t	| dip	| Source IP in inbound packet.
@parm uint16		| dport	| Source port in inbound packet.
@parm struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s **		| ppEntry	| pointer to a temp storage from caller
@rvalue SUCCESS					| 	NAPT flow found. 
@rvalue TBLDRV_EENTRYNOTFOUND	| 	Flow not found.
@comm 
Given 5 tuples of an inbound packet, check and get if connection exists. This function typically is called by forwarding engine.
If the given 5-tuple cannot find the exactly-matched connection, this function will try to use extIp and extPort to find the partially-matched connection.
<p *ppEntry> should already points to an struct <p rtl8651_tblDrv_naptTcpUdpFlowEntry_s> instacnce prepared by caller.
If we found the connection in question in table driver,  <p *ppEntry> returns and points to that  connection address
If we found the entry in ASIC, then caller's entry instance storage is used to save this temporary snapshot information. Note that entry read from ASIC WON'T be cached in software table.
This function is partially-matched.
See exactly-matched: _rtl8651_findNaptConnectionByExtIpPort().
 */
int32 
_rtl8651_getInboundNaptFlow( int8 isTcp, ipaddr_t gip, uint16 gport, ipaddr_t dip, uint16 dport, 
                             struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s **ppEntry )
{
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *pEntry;
	int32 ret;
	
	//assert( gip != 0 && gport != 0 );
	//assert( ppEntry && *ppEntry );

#ifdef CONFIG_RTL865X_ROMEPERF
	rtl8651_romeperfEnterPoint( ROMEPERF_INDEX_NAPT_FIND_INBOUND );
#endif

	if (!(gip!=0 /* Louis note: ExtPort is allowed now! For CDRouter. && gport!=0) */ ) ||
	    !(ppEntry /* Louis: we do not assist this: && *ppEntry */ ))
		return FAILED;
	
#ifdef CACHE_FIND_NAPT_RESULT
#ifdef CACHE_FIND_NAPT_DEBUG_RESULT
	rtlglue_printf("--- getInbound() -------------------\n" );
	rtlglue_printf("isTcp  = %-5d     %-5d\n", isTcp, cachedIsTcp );
	rtlglue_printf("ExtIp  = %x  %x\n",        gip, cachedExtIp );
	rtlglue_printf("ExtPort= %-5d     %-5d\n", gport, cachedExtPort );
	rtlglue_printf("RemIp  = %x  %x\n",        dip, cachedRemIp );
	rtlglue_printf("RemPort= %-5d     %-5d\n", dport, cachedRemPort );
	rtlglue_printf("PEntry = %p\n", cachedPEntry );
#endif/*CACHE_FIND_NAPT_DEBUG_RESULT*/

	if ( isTcp==cachedIsTcp && gip==cachedExtIp && gport==cachedExtPort &&
	     dip==cachedRemIp && dport==cachedRemPort )
	{
#ifdef CACHE_FIND_NAPT_DEBUG_RESULT
		rtlglue_printf("+: Hit\n");
#endif/*CACHE_FIND_NAPT_DEBUG_RESULT*/
		*ppEntry = cachedPEntry;
		if ( cachedPEntry ) return SUCCESS; else return TBLDRV_EENTRYNOTFOUND;
	}
#endif/*CACHE_FIND_NAPT_RESULT*/

	pEntry = *ppEntry;
	*ppEntry = NULL;

	/* find by gport */
	if ( ( pEntry = _rtl8651_findNaptConnectionByExtIpPort( isTcp, gip, gport, dip, dport ) ) ) goto found;
	if ( ( pEntry = _rtl8651_findNaptConnectionByExtIpPort( isTcp|ALLOW_SP, gip, gport, 0, 0 ) ) ) goto found;

#ifdef CACHE_FIND_NAPT_RESULT
	/* Cache the connection */
	cachedPEntry = NULL;
#endif/*CACHE_FIND_NAPT_RESULT*/

	ret = TBLDRV_EENTRYNOTFOUND;
	goto out;
			
found:	

#if ALLOW_SP==0
	// If this is a Server Port connection, we shall return 'Not found'.
	// Therefore, this connection will pass the slow path.
	// To find the Server Port Connection, use _rtl8651_getServerPortByExtIpPort().
	if ( pEntry->connectionType == NAPT_TYPE_SERVERPORT )
	{
#ifdef CACHE_FIND_NAPT_RESULT
		/* Cache the connection */
		cachedPEntry = NULL;
#endif/*CACHE_FIND_NAPT_RESULT*/
		ret = TBLDRV_EENTRYNOTFOUND;
		goto out;
	}
#endif/*ALLOW_SP==0*/

	// Refresh age value of UDP connection.
	//  (The age value of TCP is updated by _rtl8651_l4TcpMonitorIn/Out().
	if ( pEntry->tcp == FALSE )
		pEntry->age = rtl8651GlobalControl.udpTimeout;

	*ppEntry = pEntry;

#ifdef CACHE_FIND_NAPT_RESULT
	/* Cache the connection */
	cachedPEntry = pEntry;
#endif/*CACHE_FIND_NAPT_RESULT*/

	ret = SUCCESS;
	
out:
#ifdef CACHE_FIND_NAPT_RESULT
	cachedIsTcp = isTcp;
	cachedIntIp = 0;
	cachedIntPort = 0;
	cachedExtIp = gip;
	cachedExtPort = gport;
	cachedRemIp = dip;
	cachedRemPort = dport;
#ifdef CACHE_FIND_NAPT_DEBUG_RESULT
	rtlglue_printf("--- cache recharged getInbound() -------------------\n" );
	rtlglue_printf("cachedIsTcp  = %-5d\n", cachedIsTcp );
	rtlglue_printf("cachedExtIp  = %x\n", cachedExtIp );
	rtlglue_printf("cachedExtPort= %-5d\n", cachedExtPort );
	rtlglue_printf("cachedRemIp  = %x\n", cachedRemIp );
	rtlglue_printf("cachedRemPort= %-5d\n", cachedRemPort );
	rtlglue_printf("cachedPEntry = %p\n", cachedPEntry );
#endif/*CACHE_FIND_NAPT_DEBUG_RESULT*/
#endif/*CACHE_FIND_NAPT_RESULT*/

#ifdef CONFIG_RTL865X_ROMEPERF
	rtl8651_romeperfExitPoint( ROMEPERF_INDEX_NAPT_FIND_INBOUND );
#endif

	return ret;
}


/*
@func int32		| _rtl8651_updateNaptConnection	| Update flow status to driver table and write back to ASIC, if occupied, right away.
@parm struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *	| drvEntry	| pointer to a already created connection in driver's table
@parm int32 		| newTcpState	| new state of this TCP connection. leave it=0 if it's UDP.
@parm uint32 	| newAge		| new aging timer value of this connection
@rvalue SUCCESS	| 	New status info updated.
@rvalue FAILED	| 	Failed.	
@comm 
Update flow status and last received timestamp with driver table. and if it occupies ASIC, write back to ASIC immediately.
*/
int32 _rtl8651_updateNaptConnection( rtl8651_tblDrv_naptTcpUdpFlowEntry_t *drvEntry, 
                                     int8 tcpstate, uint32 age )
{
	//If drvEntry is an ASIC snapshot, it may be auto-learnt(dynamic) or created by protocol/forwarding engine(static)
	int32 retval = SUCCESS;
	
#ifdef CONFIG_RTL865X_ROMEPERF
	rtl8651_romeperfEnterPoint( ROMEPERF_INDEX_NAPT_UPDATE );
#endif

	assert(drvEntry);
	assert( tcpstate >= UDP_FLOW && tcpstate <= TCP_CLOSED_FLOW );
		
	// update state and age value
	if ( tcpstate != UDP_FLOW ) drvEntry->tcpFlag = tcpstate - 1;
	drvEntry->age = age; //reset timer value by state.

	/* increase packet count */
	drvEntry->pps++;

	/* In pure software mode, NEVER update ASIC state and age. */
	if ( pureSoftFwd == TRUE )
	{
		retval = SUCCESS;
		goto out;
	}
	
	// If pps >= _smartNaptSwappingThreshold, swap To Asic !
	if ( _enableSmartNaptSwappingByLoad )
	{
		/* This is for CDRouter threshold. */
		if ( drvEntry->pps >= _smartNaptSwappingThresholdCdrouter )
		{
			/* We try to swap to ASIC. */
			_rtl8651_NaptSafeSwapToAsic( drvEntry );
			drvEntry->holdForCdrouter = FALSE;
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("[intPort=%d,dstPort=%d] %s pps(%d)>=CdrouterThreshold(%d), swap to ASIC... ",
			               drvEntry->insideLocalPort, drvEntry->dstPort,
			               drvEntry->tcp?"TCP":"UDP",
			               drvEntry->pps, 
			               _smartNaptSwappingThresholdCdrouter );
			if ( drvEntry->inListType==NAPT_ASIC_CACHED_IN )
				rtlglue_printf(" IN:SUCCESS" );
			else
				rtlglue_printf(" IN:FAILED " );

			if ( drvEntry->outListType==NAPT_ASIC_CACHED_OUT )
				rtlglue_printf(" OUT:SUCCESS" );
			else
				rtlglue_printf(" OUT:FAILED " );
			rtlglue_printf("\n");
#endif/*ADD_NAPT_DEBUG*/
		}

		/* This is for heavy load swapping. */
		if ( ( drvEntry->tcp && drvEntry->pps >= _smartNaptSwappingThresholdTcp ) ||
		     ( drvEntry->tcp==FALSE && drvEntry->pps >= _smartNaptSwappingThresholdUdp ) )
		{
			/* If in/out-bound flow is NOT ASIC-accelerated, swap to ASIC. */
			if ( drvEntry->inListType!=NAPT_ASIC_CACHED_IN ||
			     drvEntry->outListType!=NAPT_ASIC_CACHED_OUT )
			{
				_rtl8651_NaptSafeSwapToAsic( drvEntry );
				drvEntry->newlyAddedHash2Udp = FALSE;
#ifdef ADD_NAPT_DEBUG
				rtlglue_printf("[intPort=%d,dstPort=%d] %s pps(%d)>=HeavyThreshold(%d), swap to ASIC... ",
				               drvEntry->insideLocalPort, drvEntry->dstPort,
				               drvEntry->tcp?"TCP":"UDP",
				               drvEntry->pps, 
				               drvEntry->tcp?_smartNaptSwappingThresholdTcp:_smartNaptSwappingThresholdUdp );
				if ( drvEntry->inListType==NAPT_ASIC_CACHED_IN )
					rtlglue_printf(" IN:SUCCESS" );
				else
					rtlglue_printf(" IN:FAILED " );

				if ( drvEntry->outListType==NAPT_ASIC_CACHED_OUT )
					rtlglue_printf(" OUT:SUCCESS" );
				else
					rtlglue_printf(" OUT:FAILED " );
				rtlglue_printf("\n");
#endif/*ADD_NAPT_DEBUG*/
			}
		}
	}

	// if this connection is Server Port, never update the state.
	if ( drvEntry->connectionType == NAPT_TYPE_SERVERPORT )
	{
		// yes, this is a Sevrer Port connection. Do nothing.
		retval = SUCCESS;
		goto out;
	}

	if ( drvEntry->hashing == NAPT_HASH1 )
	{
		assert( drvEntry->outboundTblIdx == drvEntry->inboundTblIdx );
		
		//Read from ASIC if any direction is using ASIC.
		if ( drvEntry->outListType == NAPT_ASIC_CACHED_OUT )
		{
#if 0 /* trap FINed connection cannot improve performance. */
			if ( tcpstate > TCP_ESTABLISHED_FLOW )
			{
				_rtl8651_changeNaptFlowListType( drvEntry, NAPT_CANDIDATE_IN, NAPT_CANDIDATE_OUT );
				pOutbound = pInbound = NULL;

#ifdef ADD_NAPT_DEBUG
				rtlglue_printf("[intPort=%d,dstPort=%d] HASH1 single FINed, swap to SW...\n",
				               drvEntry->insideLocalPort, drvEntry->dstPort );
				if ( drvEntry->inListType==NAPT_ASIC_CACHED_IN )
					rtlglue_printf(" IN:ASIC" );
				else
					rtlglue_printf(" IN:SW " );

				if ( drvEntry->outListType==NAPT_ASIC_CACHED_OUT )
					rtlglue_printf(" OUT:ASIC" );
				else
					rtlglue_printf(" OUT:SW " );
				rtlglue_printf("\n");
#endif/*ADD_NAPT_DEBUG*/
			}
#endif
		}
		else
		{
			/* The connection is not ASIC accelerated.
			 * A possible reason is that it is NOT an established connection.
			 * So, if ther status is to be updated to ESTABLISHED, try to swap to ASIC.
			 *  (Of course, the ASIC must be empty.)
			 */
			if ( tcpstate == TCP_ESTABLISHED_FLOW &&
			     drvEntry->holdForCdrouter==FALSE &&
			     _rtl8651_NaptEntryIsEmpty( drvEntry->outboundTblIdx ) )
			{
				_rtl8651_NaptSafeSwapToAsic( drvEntry );
#ifdef ADD_NAPT_DEBUG
				rtlglue_printf("[intPort=%d,dstPort=%d] HASH1 ESTABLISHED, swap to ASIC...",
				               drvEntry->insideLocalPort, drvEntry->dstPort );
				if ( drvEntry->inListType==NAPT_ASIC_CACHED_IN )
					rtlglue_printf(" IN:SUCCESS" );
				else
					rtlglue_printf(" IN:FAILED " );

				if ( drvEntry->outListType==NAPT_ASIC_CACHED_OUT )
					rtlglue_printf(" OUT:SUCCESS" );
				else
					rtlglue_printf(" OUT:FAILED " );
				rtlglue_printf("\n");
#endif/*ADD_NAPT_DEBUG*/
			}
		}

		/* use FIN timeout */
		if ( tcpstate == TCP_CLOSED_FLOW )
		{
			drvEntry->age = age = rtl8651GlobalControl.tcpFinTimeout;
			retval = SUCCESS;
			goto out;
		}
		
	}
	else if ( drvEntry->hashing == NAPT_HASH2 )
	{
		/* The connection is not ASIC accelerated.
		 * A possible reason is that it is NOT an established connection.
		 * So, if ther status is to be updated to ESTABLISHED, try to swap to ASIC.
		 *  (Of course, the ASIC must be empty.)
		 */
		if ( tcpstate == TCP_ESTABLISHED_FLOW &&
		     drvEntry->holdForCdrouter==FALSE &&
		     ( _rtl8651_NaptEntryIsEmpty( drvEntry->inboundTblIdx ) ||
		       _rtl8651_NaptEntryIsEmpty( drvEntry->outboundTblIdx ) ) )
		{
			_rtl8651_NaptSafeSwapToAsic( drvEntry );
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("[intPort=%d,dstPort=%d] HASH1 ESTABLISHED, swap to ASIC...",
			               drvEntry->insideLocalPort, drvEntry->dstPort );
			if ( drvEntry->inListType==NAPT_ASIC_CACHED_IN )
				rtlglue_printf(" IN:SUCCESS" );
			else
				rtlglue_printf(" IN:FAILED " );

			if ( drvEntry->outListType==NAPT_ASIC_CACHED_OUT )
				rtlglue_printf(" OUT:SUCCESS" );
			else
				rtlglue_printf(" OUT:FAILED " );
			rtlglue_printf("\n");
#endif/*ADD_NAPT_DEBUG*/
		}

		/* use FIN timeout */
		if ( tcpstate == TCP_CLOSED_FLOW )
		{
			drvEntry->age = age = rtl8651GlobalControl.tcpFinTimeout;
			retval = SUCCESS;
			goto out;
		}

		
#ifdef SUPPORT_HASH3
		/* 
		 * Once a man uses ASIC entry, we use other index to avoid collision (for Server Port).
		 */
		if ( drvEntry->inListType == NAPT_CANDIDATE_IN &&
		     drvEntry->outListType == NAPT_CANDIDATE_OUT &&
		     !_rtl8651_NaptEntryIsEmpty( drvEntry->inboundTblIdx ) )
		{
			_rtl8651_changeNaptFlowHashType( drvEntry, NAPT_HASH3 );
		}
		else
#endif
		if ( tcpstate > TCP_ESTABLISHED_FLOW )
		{
			_rtl8651_changeNaptFlowListType( drvEntry, NAPT_CANDIDATE_IN, NAPT_CANDIDATE_OUT );
#ifdef SUPPORT_HASH3
			_rtl8651_changeNaptFlowHashType( drvEntry, NAPT_HASH3 );
#endif

#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("[intPort=%d,dstPort=%d] HASH2 single FINed, swap to SW...\n",
			               drvEntry->insideLocalPort, drvEntry->dstPort );
			if ( drvEntry->inListType==NAPT_ASIC_CACHED_IN )
				rtlglue_printf(" IN:SUCCESS" );
			else
				rtlglue_printf(" IN:FAILED " );

			if ( drvEntry->outListType==NAPT_ASIC_CACHED_OUT )
				rtlglue_printf(" OUT:SUCCESS" );
			else
				rtlglue_printf(" OUT:FAILED " );
			rtlglue_printf("\n");
#endif/*ADD_NAPT_DEBUG*/
		}
	}
#ifdef SUPPORT_HASH3
	else if ( drvEntry->hashing == NAPT_HASH3 )
	{
		if ( tcpstate == TCP_CLOSED_FLOW )
		{
			drvEntry->age = age = rtl8651GlobalControl.tcpFinTimeout;
			retval = SUCCESS;
			goto out;
		}
	}
#endif
	else
	{
		assert( 0 ); /* drvEntry->hashing is neither HASH1 nor HASH2. */
		//_printDrvNaptFlow( drvEntry );
	}

out:

#ifdef CONFIG_RTL865X_ROMEPERF
	rtl8651_romeperfExitPoint( ROMEPERF_INDEX_NAPT_UPDATE );
#endif

	return retval;
}


#if 0
/*
@func int32	| _rtl8651_addHash2_nexthop	| To set SelIPIdx field of hash2 connection
@parm ipaddr_t | nexthop	| The IP address of the nexthop (NULL to use default route)
@parm rtl8651_tblDrv_networkIntfTable_t* | netif_t	| The interface that nexthop belongs to.
@parm ipaddr_t | naptip	| ExtIp to search IP table
@parm uint8* | ipidx	| (Output) Pointer to store SelIPIdx field of L4 TCP/UDP table
@rvalue SUCCESS	| Success, and *ipidx has been pointed to IP table.
@rvalue FAILED	| Error add nexthop table
@comm 
*/
int32 _rtl8651_addHash2_nexthop(ipaddr_t nexthop, rtl8651_tblDrv_networkIntfTable_t *netif_t, ipaddr_t naptip, uint8 *ipidx)
{
#if 1
	/*
	 *  Louis note: In current version, ONLY ONE nexthop is needed.
	 *              Thus, we don't need complex algorithm.
	 *              Finally, holly Chhuang will re-write this code.
	 */
	rtl8651_tblDrv_ipEntry_t *ipEntry_t;
	int32 entry;
	
	if ( naptip == 0 ) return FAILED;
	if ( ipidx == NULL ) return FAILED;
	if ( netif_t == NULL ) return FAILED;
	
	/* Locate the IP Entry */
	for( entry = 0, ipEntry_t = DrvTbl.iptbl; 
	     entry < RTL8651_IPTABLE_SIZE; 
	     entry++, ipEntry_t++ )
	{
		if ( ipEntry_t->valid &&
		     ipEntry_t->type == IP_ATTR_NAPT )
		     /* Louis note: In test code of policy route, naptip may be NOT existed in IP table. */
		     /* && ipEntry_t->extip == naptip */ 
		{
			*ipidx = entry;
			break;
		}
	}
	if ( entry == RTL8651_IPTABLE_SIZE )
		return FAILED;

	return SUCCESS;

#else//1
	rtl8651_tblDrv_ipEntry_t *ipEntry_t = DrvTbl.iptbl;
	rtl8651_tblDrv_pppoeSession_t *session_t;
	rtl8651_tblDrv_routeTable_t *rt_t;
	int32 entry, retval;

	/*
	  * Assumption:
	  * If the nextohp is not specified, use the default route's nexthop as the naptip's
	  * nexthop. If no default route is present, point to a NULL but the ASIC should 
	  * point to a RESERVED nexthop entry.
	  * Note:
	  * If the network interface is a PPPoE interface, the specified naptip will implicitly imply which 
	  * nexthop to be used. Hence in such situation, if nexthop is specified, ignore it!!!
	  * Note XXXX:
	  * This function doesn't support on demand session.
	  */

	assert(naptip && ipidx && netif_t);

	/* Locate the IP Entry */
	for(entry=0; entry<RTL8651_IPTABLE_SIZE; entry++, ipEntry_t++)
		if (IS_ENTRY_VALID(ipEntry_t) && ipEntry_t->type==IP_ATTR_NAPT && ipEntry_t->extip==naptip)
			break;
	if (entry == RTL8651_IPTABLE_SIZE)
		return FAILED;

	
	if (netif_t->linkLayerType != RTL8651_LL_VLAN) {
		/* 
		  * If the network interface is "pppoe" type, always use specified NAPT IP address 
		  * as nexthop. And find its corresponding sessoin.
		  */
		nexthop = naptip;
		SLIST_FOREACH(session_t, &netif_t->psidHead, nextSession) {
			if (session_t->ip_t && session_t->ip_t->ipAddr == naptip)
				break;
		}
		if (!session_t) /* no session was found */
			return FAILED;
	}
	else {
		/* 
		  * If the specified nexhtop is the default route's nexthop, force it select 
		  * section I. Otherwise, the Section II will be selected.
		  */
		rt_t = &DrvTbl.route[RTL8651_ROUTINGTBL_SIZE-1];
		if (IS_ENTRY_VALID(rt_t) && nexthop==rt_t->rt_nhstart_t->nexthop)
			nexthop = 0;
	}


	/* 
	  * Section I: 
	  * If no nexthop is specified, use default route's nexthop. If no default route is present,
	  * use RESERVED nexthop entry.
	  */
	if (nexthop == 0) {
		netif_t = def_netif_t;
		if (!ipEntry_t->nextHop_t) {
			/* If no default route is present, use RESERVED entry */
			if (IS_ENTRY_INVALID(rt_t))
				ipEntry_t->nextHop_t = NULL;
			else 	if (IS_ENTRY_VALID(rt_t) && _rtl8651_addNextHopEntry(NEXTHOP_NAPT_H2, ipEntry_t, netif_t, rt_t->rt_nhstart_t->nexthop, 0, NULL) == FAILED)
			/* This entry is not referenced by NAPT hash 2 */
				return FAILED;			
			ipEntry_t->ref_cnt = 0;
		}
		/* Entry is referenced by NAPT hash 2, but not use default route' nexthop */
		else if (!ipEntry_t->def_flag) 
			return FAILED;
		*ipidx = entry;
		ipEntry_t->ref_cnt ++;	
		ipEntry_t->def_flag = 1;
		return SUCCESS;
	}

	/*
	  * Section II:
	  * The nexthop is specified and is not default route's nexthop.
	  */
	if (ipEntry_t->nextHop_t) {
		assert(ipEntry_t->ref_cnt > 0);
		if (nexthop != ipEntry_t->nextHop_t->nexthop)
			return FAILED; /* conflict */
		*ipidx = entry;
		ipEntry_t->def_flag = 0;
		ipEntry_t->ref_cnt ++;
		return SUCCESS;
	}

	
	if (netif_t->linkLayerType == RTL8651_LL_VLAN)
		retval = _rtl8651_addNextHopEntry(NEXTHOP_NAPT_H2, ipEntry_t, netif_t,  nexthop, 0, NULL);
	else retval = _rtl8651_addNextHopEntry(NEXTHOP_NAPT_H2, ipEntry_t, netif_t, 0, 0, session_t);
	if (retval == FAILED)
		return FAILED;
	ipEntry_t->def_flag = 0;
	ipEntry_t->ref_cnt = 1;
	*ipidx = entry;
	return SUCCESS;
#endif//1
}


/*
@func int32	| _rtl8651_delHash2_nexthop	| De-reference the SelIPIdx field of hash2 connection
@parm ipaddr_t | nexthop	| The IP address of the nexthop (NULL to use default route)
@parm ipaddr_t | naptip	| ExtIp to search IP table
@parm uint8* | ipidx	| Pointer to store SelIPIdx field of L4 TCP/UDP table
@rvalue SUCCESS	| Success, and *ipidx has been pointed to IP table.
@rvalue FAILED	| Error to de-refercence
@comm 
*/
int32 _rtl8651_delHash2_nexthop(ipaddr_t nexthop, ipaddr_t naptip, uint8 *ipidx)
{

#if 1
	/*
	 *  Louis note: In current version, ONLY ONE nexthop is needed.
	 *              Thus, we don't need complex algorithm.
	 *              Finally, holly Chhuang will re-write this code.
	 */
	assert(naptip && ipidx);
#else//1
	rtl8651_tblDrv_ipEntry_t *ipEntry_t = DrvTbl.iptbl;
	rtl8651_tblDrv_routeTable_t *rt_t;
	uint32 entry;

	/*
	  * If the nexthop is not specified, it means that try to reset all naptip's nexhtop.
	  * Otherwise, try to reset the naptip's nexthop which uses the specified nexthop
	  */

	
	assert(naptip && ipidx);

	/* Locate the IP Entry */
	for(entry=0; entry<RTL8651_IPTABLE_SIZE; entry++, ipEntry_t++)
		if (IS_ENTRY_VALID(ipEntry_t) && ipEntry_t->type==IP_ATTR_NAPT && ipEntry_t->extip==naptip)
			break;
	if (entry == RTL8651_IPTABLE_SIZE)
		return FAILED;

	/* If PPPoE, the nexthop is set to naptip */
	if (!ipEntry_t->nextHop_t)
		return FAILED;
	if (ipEntry_t->nextHop_t->nextHopType != RTL8651_LL_VLAN) 
		nexthop = naptip;
	else {
		/* 
		  * If the specified nexhtop is the default route's nexthop, force it select 
		  * section I. Otherwise, the Section II will be selected.
		  */
		rt_t = &DrvTbl.route[RTL8651_ROUTINGTBL_SIZE-1];
		if (IS_ENTRY_VALID(rt_t) && nexthop==rt_t->rt_nhstart_t->nexthop)
			nexthop = 0;
	}
	
	if (!nexthop && !ipEntry_t->def_flag)
		return FAILED;
	if (nexthop && ipEntry_t->def_flag)
		return FAILED;
	if ((--ipEntry_t->ref_cnt) <= 0) {
		if (_rtl8651_delNextHopEntry(NEXTHOP_NAPT_H2, ipEntry_t) == FAILED)
			return FAILED;
	}
	*ipidx = 0;
#endif//1
	return SUCCESS;
}
#endif/*0*/


/*
@func int32	| _rtl8651_setl4NaptFindEmptyAsicRetry	| Set retry time in _rtl8651_FindEmptyASICEntry().
@parm uint16 | retry	| Retry time in _rtl8651_FindEmptyASICEntry().
@rvalue SUCCESS			| 
@rvalue TBLDRV_EINVALIDINPUT	| retry value is too big.
@comm 
*/
int32 _rtl8651_setl4NaptFindEmptyAsicRetry( uint16 retry )
{
	if ( retry > RTL8651_TCPUDPTBL_SIZE ) return TBLDRV_EINVALIDINPUT;
	
	rtlglue_drvMutexLock();//Lock resource
	rtl8651GlobalControl.l4NaptFindEmptyAsicRetry = retry;
	rtlglue_drvMutexUnlock();//Unlock resource
	
	return SUCCESS;
}

/*
@func int32		| _rtl8651_NaptEntryIsInUsed	| Internal function only. Return if the ASIC NAPT entry is in use ?
@parm uint16 	| tblIndex	| The index in the DrvTbl.tcpudp.
@rdesc	Returns TRUE if the ASIC entry is in use.
@rvalue FALSE	| The ASIC entry is empty.
@comm 
This internal function returns if the specified ASIC entry is used (Hardware-Accelerated)
*/
static int32 
_rtl8651_NaptEntryIsInUse( uint16 tblIndex )
{
	// get from SW list
	rtl8651DrvNaptLine_t *line;
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *asicCached;
	
	line = &DrvTbl.tcpudp[tblIndex];

	// Get who is holding ASIC entry.
	asicCached = SLIST_FIRST(&(line->list[NAPT_ASIC_CACHED_IN]));
	if ( asicCached == NULL ) asicCached = SLIST_FIRST(&(line->list[NAPT_ASIC_CACHED_OUT]));

	return ( asicCached != NULL ) ?TRUE :FALSE;
}

/*
@func int32		| _rtl8651_NaptEntryIsEmpty	| Internal function only. Return if the ASIC NAPT entry is empty ?
@parm uint16 	| tblIndex	| The index in the DrvTbl.tcpudp.
@rdesc	Returns TRUE if the ASIC entry is empty.
@rvalue FALSE	| The ASIC entry is in use.
@comm 
This internal function returns if the specified ASIC entry is empty.
*/
static int32 
_rtl8651_NaptEntryIsEmpty( uint16 tblIndex )
{
	return !_rtl8651_NaptEntryIsInUse( tblIndex );
}

// front-end of _rtl8651_setNaptEntry()
// This function will dereference the nexthop count in IP table.
void _rtl8651_derefHash2NexthopCount( uint16 tblIndex, rtl8651DrvNaptLine_t *naptLine, 
                                      rtl8651_tblDrv_naptTcpUdpFlowEntry_t *entry,
                                      int32 cleanIn, int32 cleanOut )
{
	// The 'selExtIPIdx' field must be obtained from ASIC entry.exit
	rtl8651_tblDrv_nextHopEntry_t *def_next;
	ipaddr_t nexthop = (ipaddr_t)NULL;
	int32 retval;

	if ( ( def_next = _rtl8651_getDefaultNextHop() ) ) nexthop = def_next->nexthop;

	// Only Hash 2 connection needs to dereference.
	if ( entry->hashing == NAPT_HASH2 )
	{
		if ( entry->inboundTblIdx == tblIndex && 
		     entry->inListType == NAPT_ASIC_CACHED_IN &&
		     cleanIn )
		{
			retval = _rtl8651_delHash2FromNexthop( entry->interfaceRoute? entry->dstIpAddr: 0, entry->insideGlobalIpAddr );
			assert( retval == SUCCESS );
			entry->inExtIpIdx = -1;
		}

		if ( entry->outboundTblIdx == tblIndex && 
		     entry->outListType == NAPT_ASIC_CACHED_OUT &&
		     cleanOut )
		{
			retval = _rtl8651_delHash2FromNexthop( entry->interfaceRoute? entry->dstIpAddr: 0, entry->insideGlobalIpAddr );
			assert( retval == SUCCESS );
			entry->outExtIpIdx = -1;
		}
	}

	_rtl8651_setNaptEntry( tblIndex, naptLine, NULL, FALSE );
}


#ifdef SUPPORT_HASH3
/*
int32 | _rtl8651_changeNaptFlowHashType	| Change the hash type of napt flow.
struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *	| thisFlow	| NAPT connection.
uint8 | newHashType | new Hash Type
SUCCESS	| 	<p thisFlow> changes the type successfully.
FAILED	| 	<p thisFlow> failed to change the type.
comm 
For the weakness of our ROME Driver for server port application,
all connections congest in the same entry.
We can change the hash type from Hash2 to Hash3.
*/

static int32 _rtl8651_changeNaptFlowHashType( struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s * thisFlow,
                                              uint32 newHashType )
{
	rtl8651DrvNaptLine_t *orgInLine, *newInLine;
	rtl8651DrvNaptLine_t *orgOutLine, *newOutLine;
	uint16 idx1, idx3;

	if ( thisFlow->hashing == NAPT_HASH3 ) return FAILED;
	if ( thisFlow->inListType == NAPT_ASIC_CACHED_IN ) return FAILED;
	if ( thisFlow->outListType == NAPT_ASIC_CACHED_OUT ) return FAILED;
	
	assert( newHashType == NAPT_HASH3 );
	assert( thisFlow->hashing == NAPT_HASH2 );
	if ( thisFlow->tcp ) assert( thisFlow->tcpFlag >= TCP_ESTABLISHED_FLOW );
	assert( thisFlow->inListType == NAPT_CANDIDATE_IN );
	assert( thisFlow->outListType == NAPT_CANDIDATE_OUT );
	assert( thisFlow->dstIpAddr && thisFlow->dstPort );

	orgInLine = &DrvTbl.tcpudp[thisFlow->inboundTblIdx];
	orgOutLine = &DrvTbl.tcpudp[thisFlow->outboundTblIdx];
	idx1 = rtl8651_naptTcpUdpTableIndex( thisFlow->tcp, 
	                                     thisFlow->insideLocalIpAddr, thisFlow->insideLocalPort,
	                                     thisFlow->dstIpAddr, thisFlow->dstPort );
	idx3 = rtl8651_naptTcpUdpTableIndex( thisFlow->tcp, 
	                                     thisFlow->insideGlobalIpAddr, thisFlow->insideGlobalPort,
	                                     thisFlow->dstIpAddr, thisFlow->dstPort );
	newInLine = &DrvTbl.tcpudp[idx3];
	newOutLine = &DrvTbl.tcpudp[idx1];

	/* Remove from original list */
	SLIST_REMOVE( &orgInLine->list[thisFlow->inListType], thisFlow, rtl8651_tblDrv_naptTcpUdpFlowEntry_s, nextIn );
	SLIST_REMOVE( &orgOutLine->list[thisFlow->outListType], thisFlow, rtl8651_tblDrv_naptTcpUdpFlowEntry_s, nextOut );

	/* Insert to new list */
	thisFlow->inboundTblIdx = idx3;
	thisFlow->outboundTblIdx = idx1;
	SLIST_INSERT_HEAD( &newInLine->list[thisFlow->inListType], thisFlow, nextIn );
	SLIST_INSERT_HEAD( &newOutLine->list[thisFlow->outListType], thisFlow, nextOut );

	thisFlow->hashing = newHashType;

	return SUCCESS;
}
#endif/*SUPPORT_HASH3*/


/*
int32 | _rtl8651_changeNaptIpAndPort	| Change the IP and port of the existed connection.
struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *	| thisFlow	| NAPT connection.
ipaddr_t | newIntIp | new internal IP address
uint16 | newIntPort | new internal port
ipaddr_t | newExtIp | new external IP address
uint16 | newExtPort | new external port
ipaddr_t | newRemIp | new remote IP address
uint16 | newRemPort | new remote port
SUCCESS	| 	<p thisFlow> change successfully.
FAILED	| 	<p thisFlow> failed to change.
comm 
ALG sometimes needs to change the external port after the connection has been added to table.
Therefore, we must update its hash type and hash index (including inbound and outbound).
*/

int32 _rtl8651_changeNaptIpAndPort( struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s * thisFlow,
                                    ipaddr_t newIntIp, uint16 newIntPort,
                                    ipaddr_t newExtIp, uint16 newExtPort,
                                    ipaddr_t newRemIp, uint16 newRemPort
                                    )
{
	ipaddr_t orgIntIp;
	uint16 orgIntPort;
	ipaddr_t orgExtIp;
	uint16 orgExtPort;
	ipaddr_t orgRemIp;
	uint16 orgRemPort;
	rtl8651DrvNaptLine_t *orgInLine, *newInLine;
	rtl8651DrvNaptLine_t *orgOutLine, *newOutLine;
	int32 newIdx1, newIdx2;
	int32 orgInListType, orgOutListType;

	orgIntIp = thisFlow->insideLocalIpAddr;
	orgIntPort = thisFlow->insideLocalPort;
	orgExtIp = thisFlow->insideGlobalIpAddr;
	orgExtPort = thisFlow->insideGlobalPort;
	orgRemIp = thisFlow->dstIpAddr;
	orgRemPort = thisFlow->dstPort;
	orgInLine = &DrvTbl.tcpudp[thisFlow->inboundTblIdx];
	orgOutLine = &DrvTbl.tcpudp[thisFlow->outboundTblIdx];

	/* If any direction is in ASIC, trap to software. */
	orgInListType = thisFlow->inListType;
	orgOutListType = thisFlow->outListType;
	if ( orgInListType==NAPT_ASIC_CACHED_IN || orgOutListType==NAPT_ASIC_CACHED_OUT )
	{
		_rtl8651_changeNaptFlowListType( thisFlow, NAPT_CANDIDATE_IN, NAPT_CANDIDATE_OUT );
	}
	
	/* Remove from original list */
	SLIST_REMOVE( &orgInLine->list[thisFlow->inListType], thisFlow, rtl8651_tblDrv_naptTcpUdpFlowEntry_s, nextIn );
	SLIST_REMOVE( &orgOutLine->list[thisFlow->outListType], thisFlow, rtl8651_tblDrv_naptTcpUdpFlowEntry_s, nextOut );

	thisFlow->insideLocalIpAddr = newIntIp;
	thisFlow->insideLocalPort = newIntPort;
	thisFlow->insideGlobalIpAddr = newExtIp;
	thisFlow->insideGlobalPort = newExtPort;
	thisFlow->dstIpAddr = newRemIp;
	thisFlow->dstPort = newRemPort;

	newIdx1 = rtl8651_naptTcpUdpTableIndex( thisFlow->tcp, 
	                                        newIntIp, newIntPort,
	                                        newRemIp, newRemPort );
	newIdx2 = rtl8651_naptTcpUdpTableIndex( thisFlow->tcp, 
	                                        newIntIp, newIntPort,
	                                        0, 0 );

	if ( newIdx1 == ( newExtPort & (RTL8651_TCPUDPTBL_SIZE-1) ) )
	{
		/* outIdx==inIdx, use Hash1 */
		thisFlow->hashing = NAPT_HASH1;
		thisFlow->inboundTblIdx = newIdx1;
		thisFlow->outboundTblIdx = newIdx1;
		thisFlow->selEIdx = 0;
	}
	else
	{
		/* outIdx!=inIdx, use Hash2 */
		thisFlow->hashing = NAPT_HASH2;
		thisFlow->inboundTblIdx = newExtPort & (RTL8651_TCPUDPTBL_SIZE-1);
		thisFlow->outboundTblIdx = newIdx2;
		thisFlow->selEIdx = newIdx2;
	}

	newInLine = &DrvTbl.tcpudp[thisFlow->inboundTblIdx];
	newOutLine = &DrvTbl.tcpudp[thisFlow->outboundTblIdx];
		
	/* Insert to new list */
	SLIST_INSERT_HEAD( &newInLine->list[thisFlow->inListType], thisFlow, nextIn );
	SLIST_INSERT_HEAD( &newOutLine->list[thisFlow->outListType], thisFlow, nextOut );

	/* recover its status */
	_rtl8651_changeNaptFlowListType( thisFlow, orgInListType, orgOutListType );

	return SUCCESS;
}


char* strNaptFlowType[] =
{
	"AsicOut",
	"AsicIn",
	"AlgOut",
	"AlgIn",
	"CandOut",
	"CandIn",
};
/*
int32 | _rtl8651_changeNaptFlowListType	| Change the list type of napt flow.
struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *	| thisFlow	| NAPT connection.
uint8 | inListType | inbound list type to use
uint8 | outListType | outbound list type to use
SUCCESS	| 	<p thisFlow> changes the type successfully.
FAILED	| 	<p thisFlow> failed to change the type.
comm 
You can specify the new list type of the existed NAPT flow for the following purposes:
1. NAPT_ASIC_CACHED_IN/NAPT_ASIC_CACHED_OUT: make a candidate flow to use ASIC to accelerate.
2. NAPT_CANDIDATE_IN/NAPT_CANDIDATE_OUT: make a ASIC-accelerated flow to be appended to candidate list.
If there already exists a connection in the ASIC, and the new type is ASIC, return FAILED.
@devnote
For the symmetric connection, we force the in/out list type change together.
*/

int32 _rtl8651_changeNaptFlowListType( struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s * thisFlow,
                                              uint8 inListType, uint8 outListType )
{
	uint8 bSymmetric;
	uint8 bInboundNoChange;
	uint8 bOutboundNoChange;
	rtl8651DrvNaptLine_t *inLine;
	rtl8651DrvNaptLine_t *outLine;
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *asicIn;
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *asicOut;
	int32 ret;

#ifdef ADD_NAPT_DEBUG
	rtlglue_printf("[%s():%d] tcp=%d  in:[%s]->[%s] out:[%s]->[%s]\n    Int(%d.%d.%d.%d:%d) Ext(%d.%d.%d.%d:%d) Rem(%d.%d.%d.%d:%d).\n",
	               __FUNCTION__,__LINE__, thisFlow->tcp,
	               strNaptFlowType[thisFlow->inListType], strNaptFlowType[inListType],
	               strNaptFlowType[thisFlow->outListType], strNaptFlowType[outListType],
	               NIPQUAD(thisFlow->insideLocalIpAddr), thisFlow->insideLocalPort,
	               NIPQUAD(thisFlow->insideGlobalIpAddr), thisFlow->insideGlobalPort,
	               NIPQUAD(thisFlow->dstIpAddr), thisFlow->dstPort
	               );
#endif/*ADD_NAPT_DEBUG*/

	/* This is a pure software, never swap to ASIC. */
	if ( thisFlow->pureSoftware &&
	     (inListType==NAPT_ASIC_CACHED_IN||outListType==NAPT_ASIC_CACHED_OUT) ) return FAILED;

#if 0/* Since holly chhuang has supported multiple nexthop, we do not concern the interface route issue anymore if extIpIdx is availiable. */
	/* if the is an interface Route connection, it CANNOT be ASIC-accelerated. */
	if ( thisFlow->interfaceRoute &&
	     thisFlow->hashing == NAPT_HASH2 &&
	     outListType==NAPT_ASIC_CACHED_OUT ) return FAILED;
#endif/*0*/

#if 1
	/* We ONLY consider HASH1 is symmetric. 
	 * Even HASH2 has the same inbound/outbound index, we don't treat it as symmetric.
	 * It is because we hope HASH2 can be ASIC-accelerated asymmetrically.
	 */
	bSymmetric = (thisFlow->hashing == NAPT_HASH1);
#else
	bSymmetric = ( thisFlow->inboundTblIdx == thisFlow->outboundTblIdx );
	// Assert the following conditions are symmetric mode:
	// 1. Hash1 connection
	// 2. Hash2 connection and bi-dir connection
	if ( bSymmetric )
	{
		assert( ( thisFlow->hashing == NAPT_HASH1 ) ||
		        ( thisFlow->hashing == NAPT_HASH2 && ( thisFlow->inboundTblIdx == thisFlow->outboundTblIdx ) ) );
	}
#endif//1

	// first check if the new type and the old type is equal ?
	bInboundNoChange = ( thisFlow->inListType == inListType );
	bOutboundNoChange = ( thisFlow->outListType == outListType );
	
	if ( bSymmetric )
	{
		// In the symmetric mode, the different type of in/out is non-sense.

		// no change
		if ( bInboundNoChange == TRUE && bOutboundNoChange == TRUE )
		{
			return SUCCESS;
		}
		else if ( bInboundNoChange == FALSE && bOutboundNoChange == FALSE ) 
		{
			// in/out are changed
		}
		else if ( bInboundNoChange == FALSE && bOutboundNoChange == TRUE )
		{
			// only inbound chnage,
			// oubound must change, too
			bOutboundNoChange = FALSE;
			outListType = inListType & 0xfe;
		}
		else
		{
			// only outbound change,
			// inbound must change, too
			bInboundNoChange = FALSE;
			inListType = outListType | 0x01;
		}
		
		assert( ( inListType >> 1 ) == ( outListType >> 1 ) );
	}
	else
	{
		// no change
		if ( bInboundNoChange == TRUE && bOutboundNoChange == TRUE ) return SUCCESS;
	}

	// the new type and the old type are different.
	inLine = &DrvTbl.tcpudp[thisFlow->inboundTblIdx];
	outLine = &DrvTbl.tcpudp[thisFlow->outboundTblIdx];

	// Get who is holding ASIC entry.
	asicIn = SLIST_FIRST(&(inLine->list[NAPT_ASIC_CACHED_IN]));
	if ( asicIn == NULL ) asicIn = SLIST_FIRST(&(inLine->list[NAPT_ASIC_CACHED_OUT]));
	asicOut = SLIST_FIRST(&(outLine->list[NAPT_ASIC_CACHED_OUT]));
	if ( asicOut == NULL ) asicOut = SLIST_FIRST(&(outLine->list[NAPT_ASIC_CACHED_IN]));

	// Only two type can exchang: ASIC_CACHED and CANDIDATE.
	if ( !( ( thisFlow->inListType == NAPT_ASIC_CACHED_IN || thisFlow->inListType == NAPT_CANDIDATE_IN ) &&
	        ( thisFlow->outListType == NAPT_ASIC_CACHED_OUT || thisFlow->outListType == NAPT_CANDIDATE_OUT ) ) )
	{
/*	
		rtlglue_printf("[%s:%d] assert error! thisFlow->inListType=%d thisFlow->outListType=%d\n",
		               __FILE__, __LINE__, thisFlow->inListType, thisFlow->outListType );
*/
		return FAILED;
	}
	if ( !( ( inListType == NAPT_ASIC_CACHED_IN || inListType == NAPT_CANDIDATE_IN ) &&
	        ( outListType == NAPT_ASIC_CACHED_OUT || outListType == NAPT_CANDIDATE_OUT ) ) )
	{
/*
		rtlglue_printf("[%s:%d] assert error! inListType=%d outListType=%d\n",
		               __FILE__, __LINE__, inListType, outListType );
*/
		return FAILED;
	}

	// All checks have done, change the type now.
	if ( bSymmetric )
	{
		assert( thisFlow->hashing == NAPT_HASH1 );
		/* In symmetric mode, inbound and outbound are equal. */

		if ( bInboundNoChange == FALSE && bOutboundNoChange == FALSE )
		{
			/* if the new type is ASIC, but ASIC is not empty, ignore it. */
			if ( inListType == NAPT_ASIC_CACHED_IN && asicIn != NULL )
			{
				/* The ASIC entry is in-used, skip it. */
			}
			else 
			{
				/* The new type is NOT ASIC, or ASIC is free. */

				/* Sync state and age if ASIC state is newer than software.
				 * Note that only HASH1 connection needs to sync.
				 * Because HASH2 ASIC entry does NOT maintain TCP state information.
				 */
				if ( thisFlow->hashing == NAPT_HASH1 &&
				     asicIn != NULL &&
				     inListType != NAPT_ASIC_CACHED_IN &&
				     thisFlow->inListType == NAPT_ASIC_CACHED_IN )
				{
					rtl8651DrvNaptLine_t *thisLine;
					struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s entry;
					struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *pEntry=&entry;
					
					thisLine = &DrvTbl.tcpudp[thisFlow->inboundTblIdx];
					ret = _rtl8651_getNaptEntry( thisFlow->inboundTblIdx, thisLine, &pEntry, TRUE);
					assert( ret == SUCCESS );

					if ( pEntry->tcpFlag > thisFlow->tcpFlag )
					{	
						/* rtlglue_printf("[%d] state updated from %d to %d because of _rtl8651_changeNaptFlowListType()\n",thisFlow->inboundTblIdx,thisFlow->tcpFlag+1,pEntry->tcpFlag+1); */
						thisFlow->tcpFlag = pEntry->tcpFlag;
						thisFlow->age = *tcpAgeTimer[thisFlow->tcpFlag+1];
					}
				}
			
				SLIST_REMOVE( &inLine->list[thisFlow->inListType], thisFlow, rtl8651_tblDrv_naptTcpUdpFlowEntry_s, nextIn );
				SLIST_REMOVE( &outLine->list[thisFlow->outListType], thisFlow, rtl8651_tblDrv_naptTcpUdpFlowEntry_s, nextOut );
				if ( inListType == NAPT_ASIC_CACHED_IN ) assert( SLIST_EMPTY( &inLine->list[inListType] ) );
				if ( outListType == NAPT_ASIC_CACHED_OUT ) assert( SLIST_EMPTY( &outLine->list[outListType] ) );
				SLIST_INSERT_HEAD( &inLine->list[inListType], thisFlow, nextIn );
				SLIST_INSERT_HEAD( &outLine->list[outListType], thisFlow, nextOut );

				if ( inListType == NAPT_ASIC_CACHED_IN )
				{
					thisFlow->inListType = inListType;
					thisFlow->outListType = outListType;
					// Write to ASIC entry
					ret = _rtl8651_setNaptEntry( thisFlow->inboundTblIdx, inLine, thisFlow, TRUE );
					assert( ret == SUCCESS ); /* findHash2Nexthop() FAILED. */
				}
				else
				{
					/* Clear ASIC entry
					 * HASH2 connection must de-reference Nexthop count. */
					if ( thisFlow->hashing == NAPT_HASH2 )
						_rtl8651_derefHash2NexthopCount( thisFlow->inboundTblIdx, inLine, thisFlow, TRUE, TRUE );
					else
						_rtl8651_setNaptEntry( thisFlow->inboundTblIdx, inLine, NULL, FALSE );

					thisFlow->inListType = inListType;
					thisFlow->outListType = outListType;
				}
			}
		}
		else
		{
			assert( 0 ); // something wrong .... you need to fix the code.
		}
	}
	else
	{
		// In asymmetric mode, inbound and outbound are different.
		if ( bInboundNoChange == FALSE )
		{
			// if the new type is ASIC, but ASIC is not empty, ignore it.
			if ( inListType == NAPT_ASIC_CACHED_IN && asicIn != NULL )
			{
				// The ASIC entry is in-used, skip it.
			}
			else 
			{
				// change thisFlow's inbound list type
				SLIST_REMOVE( &inLine->list[thisFlow->inListType], thisFlow, rtl8651_tblDrv_naptTcpUdpFlowEntry_s, nextIn );
				if ( inListType == NAPT_ASIC_CACHED_IN ) assert( SLIST_EMPTY( &inLine->list[inListType] ) );
				SLIST_INSERT_HEAD( &inLine->list[inListType], thisFlow, nextIn );

				if ( inListType == NAPT_ASIC_CACHED_IN )
				{
					thisFlow->inListType = inListType;
					// Write to ASIC entry

					if ( thisFlow->inboundTblIdx == thisFlow->outboundTblIdx &&
					     inListType == outListType )
					{
						/* Since the index of inbound and outbound are identical,
						 * We do NOT update ASIC entry right away.
						 * Instead, we leave it for later.
						 */
						assert( bOutboundNoChange == FALSE );
						assert( !( outListType == NAPT_ASIC_CACHED_OUT && asicOut != NULL ) );
					}
					else
					{
						ret = _rtl8651_setNaptEntry( thisFlow->inboundTblIdx, inLine, thisFlow, TRUE );
						assert( ret == SUCCESS ); /* findHash2Nexthop() FAILED. */
					}
				}
				else
				{
					// Clear ASIC entry
					_rtl8651_derefHash2NexthopCount( thisFlow->inboundTblIdx, inLine, thisFlow, TRUE, FALSE );

					thisFlow->inListType = inListType;
				}
			}
		}
		
		if ( bOutboundNoChange == FALSE )
		{
			// if the new type is ASIC, but ASIC is not empty, ignore it.
			if ( outListType == NAPT_ASIC_CACHED_OUT && asicOut != NULL )
			{
				// The ASIC entry is in-used, skip it.
			}
			else
			{
				// change thisFlow's outbound list type
				SLIST_REMOVE( &outLine->list[thisFlow->outListType], thisFlow, rtl8651_tblDrv_naptTcpUdpFlowEntry_s, nextOut );
				if ( outListType == NAPT_ASIC_CACHED_OUT ) assert( SLIST_EMPTY( &outLine->list[outListType] ) );
				SLIST_INSERT_HEAD( &outLine->list[outListType], thisFlow, nextOut );

				if ( outListType == NAPT_ASIC_CACHED_OUT )
				{
					thisFlow->outListType = outListType;
					
					// Write to ASIC entry
					ret = _rtl8651_setNaptEntry( thisFlow->outboundTblIdx, outLine, thisFlow, TRUE );
					assert( ret == SUCCESS ); /* findHash2Nexthop() FAILED. */
				}
				else
				{
					// Clear ASIC entry
					_rtl8651_derefHash2NexthopCount( thisFlow->outboundTblIdx, outLine, thisFlow, FALSE, TRUE );

					thisFlow->outListType = outListType;
				}
			}
		}
	}

	return SUCCESS;
}


/*
@func int32		| _rtl8651_AppendToCandidateList	| Internal function only. Append the entry to candidate list.
@parm uint16 	| tblIndex	| The index in the DrvTbl.tcpudp.
@parm rtl8651_tblDrv_naptTcpUdpFlowEntry_s 	| *tmp	| The temporal storage which will be added.
@parm rtl8651_tblDrv_naptTcpUdpFlowEntry_s 	| **ppConn	| Pointer to store the pointer of added connection.
@rdesc	Always returns SUCCESS.
@comm 
Allocate an entry from tcpudp.freelist, and copy temporal storage into the new entry.
Then, insert it to inuselist and tcpudp[].list[NAPT_CANDIDATE_*].
*/
static int32 
_rtl8651_AppendToCandidateList( uint16 tblIndex, 
                                struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *tmp )
{
	
	tmp->inListType = NAPT_CANDIDATE_IN;
	tmp->outListType = NAPT_CANDIDATE_OUT;
	tmp->inboundTblIdx = tblIndex;
	tmp->outboundTblIdx = tblIndex;

	/* this may cause bug ! comment it!
	if ( tmp->hashing == NAPT_HASH2 && tmp->inboundTblIdx == tmp->outboundTblIdx ) tmp->tcpFlag &= 5; // bi-direct
	*/
	
	SLIST_INSERT_HEAD( &DrvTbl.tcpudp[tblIndex].list[NAPT_CANDIDATE_IN], tmp, nextIn );
	SLIST_INSERT_HEAD( &DrvTbl.tcpudp[tblIndex].list[NAPT_CANDIDATE_OUT], tmp, nextOut );

	return SUCCESS;
}

/*
@func int32		| _rtl8651_WriteToAsicNaptEntry	| Internal function only. Write to ASIC NAPT entry directly.
@parm uint16 	| tblIndex	| The index in the DrvTbl.tcpudp.
@parm uint16 	| dir	| 0 for OUT, and 1 for IN.
@parm rtl8651_tblDrv_naptTcpUdpFlowEntry_s 	| *tmp	| The temporal storage which will be added.
@rdesc	Returns SUCCESS if done.
@rvalue FAILED	| The software cannot write to ASIC.
@comm 
Allocate an entry from tcpudp.freelist, and copy temporal storage into the new entry.
Then, insert it to inuselist and tcpudp[].list[NAPT_ASIC_CACHED_*].
However, we will never write to ASIC directly. Instead, we append it to Candidate list only.
The maintaining thread will write to ASIC.
*/
static int32 
_rtl8651_WriteToAsicNaptEntry( uint16 tblIndex,
                               struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *tmp )
{
	rtl865x_tblAsicDrv_naptTcpUdpParam_t e;

	/* Louis note that pure software connection should NOT call _rtl8651_WriteToAsicNaptEntry(). */
	assert( tmp->pureSoftware == FALSE );

	// ASIC Cache list always contains ONLY one entry.
	if ( SLIST_FIRST( &DrvTbl.tcpudp[tblIndex].list[NAPT_ASIC_CACHED_IN] ) != NULL ) return FAILED;
	if ( SLIST_FIRST( &DrvTbl.tcpudp[tblIndex].list[NAPT_ASIC_CACHED_OUT] ) != NULL ) return FAILED;

	memset( &e, 0 ,sizeof( e ) );
	
	e.ageSec = tmp->age;
	e.insideLocalIpAddr = tmp->insideLocalIpAddr;
	e.insideLocalPort = tmp->insideLocalPort;
	e.isStatic = tmp->isStatic;
	e.isTcp = tmp->tcp;
	e.offset = tmp->insideGlobalPort >> RTL8651_TCPUDPTBL_BITS;

	if ( tmp->hashing == NAPT_HASH1 )
	{
		e.isCollision = 0;
		e.isCollision2 = 0;
		e.isValid = 1;
		e.isDedicated = 0;
		e.selEIdx = 0; // FIXME: Static==1, Nextop Table Index. See 8.3.6
		e.selExtIPIdx = 0;
		e.tcpFlag = tmp->tcpFlag;
	}
	else if ( tmp->hashing == NAPT_HASH2 )
	{
		e.isCollision = 0;
		e.isCollision2 = 0;
		e.isValid = 1;
		e.isDedicated = 1;
		e.selEIdx = tblIndex;

		if ( tmp->inExtIpIdx < 0 )
		{
			tmp->inExtIpIdx = _rtl8651_addHash2ToNexthop( tmp->interfaceRoute? tmp->dstIpAddr: 0, tmp->insideGlobalIpAddr );
			if ( tmp->inExtIpIdx == FAILED )
				return FAILED;
		}
		/* Since ClearAsicNaptEntry() will de-reference twice, we must add twice in symmetric mode. */
		if ( tmp->outExtIpIdx < 0 )
		{
			tmp->outExtIpIdx = _rtl8651_addHash2ToNexthop( tmp->interfaceRoute? tmp->dstIpAddr: 0, tmp->insideGlobalIpAddr );
			if ( tmp->outExtIpIdx == FAILED )
			{
				_rtl8651_delHash2FromNexthop( tmp->interfaceRoute? tmp->dstIpAddr: 0, tmp->insideGlobalIpAddr );
				tmp->inExtIpIdx = -1;
				return FAILED;
			}
		}
		assert( tmp->inExtIpIdx == tmp->outExtIpIdx );
		e.selExtIPIdx = tmp->inExtIpIdx;
		
		if ( tmp->connectionType == NAPT_TYPE_SERVERPORT ) e.tcpFlag |= 4;
		e.tcpFlag |= 0;
		e.tcpFlag |= 0;
	}
	else
	{
		assert( 0 );
	}
	
	if ( rtl8651_setAsicNaptTcpUdpTable( TRUE, tblIndex, &e ) == SUCCESS )
	{
		// link driver table ( ASIC Cached )
		tmp->inListType = NAPT_ASIC_CACHED_IN;
		tmp->outListType = NAPT_ASIC_CACHED_OUT;
		tmp->inboundTblIdx = tblIndex;
		tmp->outboundTblIdx = tblIndex;

		/* this may cause bug ! comment it!
		if ( tmp->hashing == NAPT_HASH2 && tmp->inboundTblIdx == tmp->outboundTblIdx ) tmp->tcpFlag &= 5; // bi-direct
		*/

		SLIST_INSERT_HEAD( &DrvTbl.tcpudp[tblIndex].list[NAPT_ASIC_CACHED_IN], tmp, nextOut );
		SLIST_INSERT_HEAD( &DrvTbl.tcpudp[tblIndex].list[NAPT_ASIC_CACHED_OUT], tmp, nextOut );

		return SUCCESS;
	}
	else
	{
		return FAILED;
	}
}

/*
@func int32		| _rtl8651_HalfHardwareAccelerated	| Internal function only.
@parm uint16 	| inIdx	| The index for in-entry, may be assigned by user. Hardware-Accelerated.
@parm uint16 	| outIdx	| The index for out-entry, fixed by hash2(). Software-Handled.
@parm rtl8651_tblDrv_naptTcpUdpFlowEntry_s 	| *tmp	| The temporal storage which will be added.
@rdesc	Returns SUCCESS if done.
@rvalue FAILED	| The software cannot write to ASIC.
@comm 
Half-Hardware-Accelerated.
*/
static int32
_rtl8651_HalfHardwareAccelerated( uint16 inIdx, uint16 outIdx, 
                                  struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *tmp )
{
	rtl865x_tblAsicDrv_naptTcpUdpParam_t e;
	
	// ASIC Cache list always contains ONLY one entry.
	if ( SLIST_FIRST( &DrvTbl.tcpudp[inIdx].list[NAPT_ASIC_CACHED_IN] ) != NULL ) return FAILED;
	if ( SLIST_FIRST( &DrvTbl.tcpudp[inIdx].list[NAPT_ASIC_CACHED_OUT] ) != NULL ) return FAILED;
	
	memset( &e, 0 ,sizeof( e ) );
	
	e.ageSec = tmp->age;
	e.insideLocalIpAddr = tmp->insideLocalIpAddr;
	e.insideLocalPort = tmp->insideLocalPort;
	e.isStatic = tmp->isStatic;
	e.isTcp = tmp->tcp;
	e.offset = tmp->insideGlobalPort >> RTL8651_TCPUDPTBL_BITS;

	assert( tmp->inExtIpIdx < 0 );
	tmp->inExtIpIdx = _rtl8651_addHash2ToNexthop( tmp->interfaceRoute? tmp->dstIpAddr: 0, tmp->insideGlobalIpAddr );
	if ( tmp->inExtIpIdx == FAILED )
		return FAILED;
	e.selExtIPIdx = tmp->inExtIpIdx;

	e.isCollision = 0;
	e.isCollision2 = 0;
	e.isValid = 1;
	e.isDedicated = 1;
	e.selEIdx = inIdx;

	if ( tmp->connectionType == NAPT_TYPE_SERVERPORT ) e.tcpFlag |= 4;
	e.tcpFlag |= 2;
	e.tcpFlag |= 0;
	
	if ( rtl8651_setAsicNaptTcpUdpTable( TRUE, inIdx, &e ) == SUCCESS )
	{
		// link to driver table ( ASIC Cached )
		tmp->inListType = NAPT_ASIC_CACHED_IN;
		tmp->outListType = NAPT_CANDIDATE_OUT;
		tmp->inboundTblIdx = inIdx;
		tmp->outboundTblIdx = outIdx;
		assert( inIdx != outIdx ); // Half Hareware-Accelerated must not bi-direct
		assert( SLIST_EMPTY( &DrvTbl.tcpudp[inIdx].list[NAPT_ASIC_CACHED_IN] ) );
		SLIST_INSERT_HEAD( &DrvTbl.tcpudp[inIdx].list[NAPT_ASIC_CACHED_IN], tmp, nextIn );
		SLIST_INSERT_HEAD( &DrvTbl.tcpudp[outIdx].list[NAPT_CANDIDATE_OUT], tmp, nextOut );

		return SUCCESS;
	}
	else
	{
		return FAILED;
	}
}

/*
@func int32		| _rtl8651_Hash2TryHardwareAccelerated	| Internal function only.
@parm uint16 	| inIdx	| The index for in-entry, may be assigned by user. Hardware-Accelerated.
@parm uint16 	| outIdx	| The index for out-entry, fixed by hash2(). Software-Handled.
@parm rtl8651_tblDrv_naptTcpUdpFlowEntry_s 	| *tmp	| The temporal storage which will be added.
@rdesc	Returns SUCCESS if done.
@rvalue FAILED	| The software cannot write to ASIC.
@comm 
This is L07.
Half-Hardware-Accelerated.
*/
static int32
_rtl8651_Hash2TryHardwareAccelerated( uint16 inIdx, uint16 outIdx, 
                                  struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *tmp, uint8 bPortReused )
{
	rtl865x_tblAsicDrv_naptTcpUdpParam_t e;

	if ( inIdx == outIdx )
	{
		// Symmetric
		// We note that this section is a backup code.
		// In the normal case, the L01 and L05 section will be executed first.
		// However, if they are disbled with some reason, this section will take over.
		
		if ( tmp->pureSoftware ) goto L070; /* pure Software, DO NOT write to ASIC. */

		/* For CDRouter */
		if ( tmp->holdForCdrouter )
		{
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("[%s:%d] for CDRouter TCP-long-timeout test, goto L070.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
			goto L070;
		}
		
		/* For CDRouter */
		if ( tmp->tcp==FALSE )
		{
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("[%s:%d] UDP, for CDRouter, goto L070.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
			tmp->newlyAddedHash2Udp = TRUE;
			goto L070;	
		}

		if ( SLIST_FIRST( &DrvTbl.tcpudp[inIdx].list[NAPT_ASIC_CACHED_IN] ) == NULL &&
		     SLIST_FIRST( &DrvTbl.tcpudp[inIdx].list[NAPT_ASIC_CACHED_OUT] ) == NULL )
		{
			// this ASIC entry is empty
			if ( bPortReused == TRUE ) goto L070;
			
			memset( &e, 0, sizeof( e ) );
			e.ageSec = tmp->age;
			e.insideLocalIpAddr = tmp->insideLocalIpAddr;
			e.insideLocalPort = tmp->insideLocalPort;
			e.isStatic = tmp->isStatic;
			e.isTcp = tmp->tcp;
			e.offset = tmp->insideGlobalPort >> RTL8651_TCPUDPTBL_BITS;

			assert( tmp->inExtIpIdx == -1 );
			assert( tmp->outExtIpIdx == -1 );
			tmp->inExtIpIdx = _rtl8651_addHash2ToNexthop( tmp->interfaceRoute? tmp->dstIpAddr: 0, tmp->insideGlobalIpAddr );
			if ( tmp->inExtIpIdx == FAILED )
			{
#ifdef ADD_NAPT_DEBUG
				rtlglue_printf("[%s:%d] _rtl8651_addHash2ToNexthop()==FAILED, goto L070.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
				goto L070;
			}
			/* Since ClearAsicNaptEntry() will de-reference twice, we must add twice in symmetric mode. */
			tmp->outExtIpIdx = _rtl8651_addHash2ToNexthop( tmp->interfaceRoute? tmp->dstIpAddr: 0, tmp->insideGlobalIpAddr );
			if ( tmp->outExtIpIdx == FAILED )
			{
#ifdef ADD_NAPT_DEBUG
				rtlglue_printf("[%s:%d] _rtl8651_addHash2ToNexthop()==FAILED, goto L070.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
				goto L070;
			}
			assert( tmp->inExtIpIdx == tmp->outExtIpIdx );
			e.selExtIPIdx = tmp->inExtIpIdx;

			e.isCollision = 0;
			e.isCollision2 = 0;
			e.isValid = 1;
			e.isDedicated = 1;
			e.selEIdx = inIdx;

			if ( tmp->connectionType == NAPT_TYPE_SERVERPORT ) e.tcpFlag |= 4;
			e.tcpFlag |= 0; // bi-dir
			e.tcpFlag |= 0; // in-bound
			
			if ( rtl8651_setAsicNaptTcpUdpTable( TRUE, inIdx, &e ) == SUCCESS )
			{
				// success, link to driver table
				tmp->inListType = NAPT_ASIC_CACHED_IN;
				tmp->inboundTblIdx = inIdx;
				tmp->outListType = NAPT_ASIC_CACHED_OUT;
				tmp->outboundTblIdx = outIdx;

				/* this may cause bug ! comment it!
				tmp->tcpFlag &= 5; // bi-direct
				*/

				assert( SLIST_EMPTY( &DrvTbl.tcpudp[inIdx].list[NAPT_ASIC_CACHED_IN] ) );
				assert( SLIST_EMPTY( &DrvTbl.tcpudp[outIdx].list[NAPT_ASIC_CACHED_OUT] ) );
				SLIST_INSERT_HEAD( &DrvTbl.tcpudp[inIdx].list[NAPT_ASIC_CACHED_IN], tmp, nextIn );
				SLIST_INSERT_HEAD( &DrvTbl.tcpudp[outIdx].list[NAPT_ASIC_CACHED_OUT], tmp, nextOut );
#ifdef ADD_NAPT_DEBUG
				rtlglue_printf("[%s:%d] addNaptConnection() F09.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
			}
			else
			{
				// write failed, append to candidate list.
				goto L070;
			}
				
		}
		else
		{
			// already exists a connection in the ASIC, append to candidate list.
			L070:
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("[%s:%d] addNaptConnection() L070.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
			tmp->inListType = NAPT_CANDIDATE_IN;
			tmp->inboundTblIdx = inIdx;
			tmp->outListType = NAPT_CANDIDATE_OUT;
			tmp->outboundTblIdx = outIdx;

			/* de-refer the IP count */
			if ( tmp->inExtIpIdx >= 0 )
			{
				_rtl8651_delHash2FromNexthop( tmp->interfaceRoute? tmp->dstIpAddr: 0, tmp->insideGlobalIpAddr );
				tmp->inExtIpIdx = -1;
			}
			if ( tmp->outExtIpIdx >= 0 )
			{
				_rtl8651_delHash2FromNexthop( tmp->interfaceRoute? tmp->dstIpAddr: 0, tmp->insideGlobalIpAddr );
				tmp->outExtIpIdx = -1;
			}
				
			/* this may cause bug ! comment it!
			tmp->tcpFlag &= 5; // bi-direct
			*/
			
			SLIST_INSERT_HEAD( &DrvTbl.tcpudp[inIdx].list[NAPT_CANDIDATE_IN], tmp, nextIn );
			SLIST_INSERT_HEAD( &DrvTbl.tcpudp[outIdx].list[NAPT_CANDIDATE_OUT], tmp, nextOut );
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("[%s:%d] addNaptConnection() F10.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
		}

	}
	else
	{
		//
		// Asymmetric
		//

		//
		// First, write In-entry.
		//
		if ( tmp->pureSoftware ) goto L071; /* pure Software, DO NOT write to ASIC. */

		/* For CDRouter */
		if ( tmp->holdForCdrouter )
		{
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("[%s:%d] for CDRouter TCP-long-timeout test, goto L071.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
			goto L071;
		}
				
		/* For CDRouter */
		if ( tmp->tcp==FALSE )
		{
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("[%s:%d] UDP, for CDRouter, goto L071.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
			tmp->newlyAddedHash2Udp = TRUE;
			goto L071;	
		}
		
		if ( SLIST_FIRST( &DrvTbl.tcpudp[inIdx].list[NAPT_ASIC_CACHED_IN] ) == NULL &&
		     SLIST_FIRST( &DrvTbl.tcpudp[inIdx].list[NAPT_ASIC_CACHED_OUT] ) == NULL )
		{
			// this ASIC entry is empty
			if ( bPortReused == TRUE ) goto L071;
			
			memset( &e, 0, sizeof( e ) );
			e.ageSec = tmp->age;
			e.insideLocalIpAddr = tmp->insideLocalIpAddr;
			e.insideLocalPort = tmp->insideLocalPort;
			e.isStatic = tmp->isStatic;
			e.isTcp = tmp->tcp;
			e.offset = tmp->insideGlobalPort >> RTL8651_TCPUDPTBL_BITS;

			tmp->inExtIpIdx = _rtl8651_addHash2ToNexthop( tmp->interfaceRoute? tmp->dstIpAddr: 0, tmp->insideGlobalIpAddr );
			if ( tmp->inExtIpIdx == FAILED )
			{
#ifdef ADD_NAPT_DEBUG
				rtlglue_printf("[%s:%d] _rtl8651_addHash2ToNexthop()==FAILED, goto L071.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
				goto L071;
			}
			e.selExtIPIdx = tmp->inExtIpIdx;

			e.isCollision = 0;
			e.isCollision2 = 0;
			e.isValid = 1;
			e.isDedicated = 1;
			e.selEIdx = inIdx;

			if ( tmp->connectionType == NAPT_TYPE_SERVERPORT ) e.tcpFlag |= 4;
			e.tcpFlag |= 2; // uni
			e.tcpFlag |= 0; // in-bound
			
			if ( rtl8651_setAsicNaptTcpUdpTable( TRUE, inIdx, &e ) == SUCCESS )
			{
				// success, link to driver table
				tmp->inListType = NAPT_ASIC_CACHED_IN;
				tmp->inboundTblIdx = inIdx;
				assert( SLIST_EMPTY( &DrvTbl.tcpudp[inIdx].list[NAPT_ASIC_CACHED_IN] ) );
				SLIST_INSERT_HEAD( &DrvTbl.tcpudp[inIdx].list[NAPT_ASIC_CACHED_IN], tmp, nextIn );
#ifdef ADD_NAPT_DEBUG
				rtlglue_printf("[%s:%d] addNaptConnection() F11.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
			}
			else
			{
				// write failed, append to candidate list.
				goto L071;
			}
				
		}
		else
		{
			// already exists a connection in the ASIC, append to candidate list.
			L071:

			/* De-refer IP count */
			if ( tmp->inExtIpIdx >= 0 )
			{
				_rtl8651_delHash2FromNexthop( tmp->interfaceRoute? tmp->dstIpAddr: 0, tmp->insideGlobalIpAddr );
				tmp->inExtIpIdx = -1;
			}

			tmp->inListType = NAPT_CANDIDATE_IN;
			tmp->inboundTblIdx = inIdx;
			SLIST_INSERT_HEAD( &DrvTbl.tcpudp[inIdx].list[NAPT_CANDIDATE_IN], tmp, nextIn );
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("[%s:%d] addNaptConnection() F12.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
		}

		//
		// Second, write out-entry.
		//
		
		if ( tmp->pureSoftware ) goto L072; /* pure Software, DO NOT write to ASIC. */

		/* For CDRouter */
		if ( tmp->holdForCdrouter )
		{
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("[%s:%d] for CDRouter TCP-long-timeout test, goto L072.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
			goto L072;
		}
				
		/* For CDRouter */
		if ( tmp->tcp==FALSE )
		{
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("[%s:%d] UDP, for CDRouter, goto L072.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
			tmp->newlyAddedHash2Udp = TRUE;
			goto L072;
		}
		
		if ( SLIST_FIRST( &DrvTbl.tcpudp[outIdx].list[NAPT_ASIC_CACHED_IN] ) == NULL &&
		     SLIST_FIRST( &DrvTbl.tcpudp[outIdx].list[NAPT_ASIC_CACHED_OUT] ) == NULL )
		{
			// this ASIC entry is empty
						
			memset( &e, 0, sizeof( e ) );
			e.ageSec = tmp->age;
			e.insideLocalIpAddr = tmp->insideLocalIpAddr;
			e.insideLocalPort = tmp->insideLocalPort;
			e.isStatic = tmp->isStatic;
			e.isTcp = tmp->tcp;
			e.offset = tmp->insideGlobalPort >> RTL8651_TCPUDPTBL_BITS;

			tmp->outExtIpIdx = _rtl8651_addHash2ToNexthop( tmp->interfaceRoute? tmp->dstIpAddr: 0, tmp->insideGlobalIpAddr );
			if ( tmp->outExtIpIdx == FAILED )
			{
#ifdef ADD_NAPT_DEBUG
				rtlglue_printf("[%s:%d] _rtl8651_addHash2ToNexthop()==FAILED, goto L072.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
				goto L072;
			}
			e.selExtIPIdx = tmp->outExtIpIdx;

			e.isCollision = 0;
			e.isCollision2 = 0;
			e.isValid = 1;
			e.isDedicated = 1;
			e.selEIdx = inIdx;

			if ( tmp->connectionType == NAPT_TYPE_SERVERPORT ) e.tcpFlag |= 4;
			e.tcpFlag |= 2;
			e.tcpFlag |= 1;
			
			if ( rtl8651_setAsicNaptTcpUdpTable( TRUE, outIdx, &e ) == SUCCESS )
			{
				// success, link to driver table
				tmp->outListType = NAPT_ASIC_CACHED_OUT;
				tmp->outboundTblIdx = outIdx;
				assert( SLIST_EMPTY( &DrvTbl.tcpudp[outIdx].list[NAPT_ASIC_CACHED_OUT] ) );
				SLIST_INSERT_HEAD( &DrvTbl.tcpudp[outIdx].list[NAPT_ASIC_CACHED_OUT], tmp, nextOut );
#ifdef ADD_NAPT_DEBUG
				rtlglue_printf("[%s:%d] addNaptConnection() F13.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
			}
			else
			{
				// write failed, append to candidate list.
				goto L072;
			}
				
		}
		else
		{
			// already exists a connection in the ASIC, append to candidate list.
			L072:

			/* De-refer IP count */
			if ( tmp->outExtIpIdx >= 0 )
			{
				_rtl8651_delHash2FromNexthop( tmp->interfaceRoute? tmp->dstIpAddr: 0, tmp->insideGlobalIpAddr );
				tmp->outExtIpIdx = -1;
			}
				
			tmp->outListType = NAPT_CANDIDATE_OUT;
			tmp->outboundTblIdx = outIdx;
			SLIST_INSERT_HEAD( &DrvTbl.tcpudp[outIdx].list[NAPT_CANDIDATE_OUT], tmp, nextOut );
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("[%s:%d] addNaptConnection() F14.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
		}
	}

#ifdef SUPPORT_HASH3
#if 1
	/*
	 * If it is a normal NAPT connection (typically DMZ, not Server Port),
	 *   and there are too many connections wating or using this ASIC entry,
	 *   use HASH3 to distribute connection.
	 */
	if ( tmp->connectionType == NAPT_TYPE_NORMAL &&
	     tmp->inListType == NAPT_CANDIDATE_IN &&
	     tmp->outListType == NAPT_CANDIDATE_OUT )
	{
		int32 threshold = 10; /* no more 10 connections. */
		struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *track;

		SLIST_FOREACH( track, &DrvTbl.tcpudp[tmp->inboundTblIdx].list[NAPT_CANDIDATE_IN], nextIn )
		{
			if ( (threshold--) <= 0 )
			{
				_rtl8651_changeNaptFlowHashType( tmp, NAPT_HASH3 );
				break;
			}
		}
	}
#else
	/*
	 * If it is a normal NAPT connection (typically DMZ, not Server Port),
	 *   and there is any entry wating or using this ASIC entry,
	 *   use HASH3 to distribute connection.
	 */
	if ( tmp->connectionType == NAPT_TYPE_NORMAL &&
	     tmp->inListType == NAPT_CANDIDATE_IN &&
	     tmp->outListType == NAPT_CANDIDATE_OUT &&
	     ( !SLIST_EMPTY( &DrvTbl.tcpudp[tmp->inboundTblIdx].list[NAPT_ASIC_CACHED_IN] ) ||
	       !SLIST_EMPTY( &DrvTbl.tcpudp[tmp->outboundTblIdx].list[NAPT_ASIC_CACHED_OUT] ) ||
	       !SLIST_EMPTY( &DrvTbl.tcpudp[tmp->inboundTblIdx].list[NAPT_CANDIDATE_IN] ) ||
	       !SLIST_EMPTY( &DrvTbl.tcpudp[tmp->outboundTblIdx].list[NAPT_CANDIDATE_OUT] ) ) )
	{
		_rtl8651_changeNaptFlowHashType( tmp, NAPT_HASH3 );
	}
#endif
#endif

	return SUCCESS;
}

static int32 _rtl8651_delFlowFromMNQueue(rtl8651_tblDrvAclRule_t *rule, uint32 GroupId);
/*
@func int32		| _rtl8651_delNaptConnection	| Internal use only. Delete an NAPT connection.
@parm rtl8651_tblDrv_naptTcpUdpFlowEntry_t * | delEntry	| pointer to the Connection to be deleted.
@rvalue SUCCESS	| NAPT connection successfully deleted.
@rvalue FAILED	| Failed to delete NAPT connection.
@comm 
Delete an NAPT connection. If either or both directions of the flows of this connection has been written into ASIC, 
clear it(them). Don't try to refill with candidate connections. Let maintenance thread do the job.
*/
int32 _rtl8651_delNaptConnection( rtl8651_tblDrv_naptTcpUdpFlowEntry_t *delConn ) 
{
	rtl8651DrvNaptLine_t *outLine, *inLine;
	
#ifdef CONFIG_RTL865X_ROMEPERF
	rtl8651_romeperfEnterPoint(ROMEPERF_INDEX_NAPT_DEL);
#endif

	assert( delConn );
	assert( delConn->alive );
	assert( delConn->inboundTblIdx>=0 && delConn->inboundTblIdx<RTL8651_TCPUDPTBL_SIZE );
	assert( delConn->outboundTblIdx>=0 && delConn->outboundTblIdx<RTL8651_TCPUDPTBL_SIZE );
	assert( delConn->outListType==NAPT_ASIC_CACHED_OUT ||
	        delConn->outListType==NAPT_ALG_OUT ||
	        delConn->outListType==NAPT_CANDIDATE_OUT );
	assert( delConn->inListType==NAPT_ASIC_CACHED_IN ||
	        delConn->inListType==NAPT_ALG_IN ||
	        delConn->inListType==NAPT_CANDIDATE_IN );



/* delete ALG Guaranteed/Spare Class QoS */
{
	int i;
	rtl8651_tblDrvAclRule_t rule;
	memset(&rule,0,sizeof(rule));
	rule.actionType_ = RTL8651_ACL_DROP;
	
	rule.srcIpAddr_		= delConn->insideLocalIpAddr;
	rule.srcIpAddrMask_	= 0xffffffff;
	if(delConn->tcp)
	{
		rule.ruleType_ = RTL8651_ACL_TCP;
		rule.tcpSrcPortUB_	= delConn->insideLocalPort;
		rule.tcpSrcPortLB_	= delConn->insideLocalPort;
		rule.tcpDstPortUB_	= 65535;
		rule.tcpDstPortLB_	= 0;	
	}
	else
	{
		rule.ruleType_= RTL8651_ACL_UDP;
		rule.udpSrcPortUB_	= delConn->insideLocalPort;
		rule.udpSrcPortLB_		= delConn->insideLocalPort;
		rule.udpDstPortUB_	= 65535;
		rule.udpDstPortLB_		= 0;			
	}

	for(i=0;i<32;i++)
	{
		if(alg_qosqid[0][i]!=0)
		{
			int r;			
			r=_rtl8651_delFlowFromMNQueue(&rule, alg_qosqid[0][i]);	
			
		}
	}

}



#ifdef ADD_NAPT_DEBUG
	rtlglue_printf("delNaptConnection(intPort=%d,dstPort=%d)\n",
	               delConn->insideLocalPort,
	               delConn->dstPort );
#endif/* ADD_NAPT_DEBUG */

	/////////////////////////////
	// Clear outbound ASIC entry if it is ASIC-accelerated.
	/////////////////////////////
	outLine = &DrvTbl.tcpudp[delConn->outboundTblIdx];
	//printfByPolling("removing on outbound line %d.\n ", delConn->outboundTblIdx);
	SLIST_REMOVE( &outLine->list[delConn->outListType], delConn,
	              rtl8651_tblDrv_naptTcpUdpFlowEntry_s, nextOut );

	if( delConn->outListType == NAPT_ASIC_CACHED_OUT )
	{
		//force clear the ASIC entry
		assert( delConn->nextOut.sle_next==NULL ); // only one entry in ASIC cache
		_rtl8651_derefHash2NexthopCount( delConn->outboundTblIdx, outLine, delConn, FALSE, TRUE );
		SLIST_INIT( &outLine->list[NAPT_ASIC_CACHED_OUT] );
		assert( !SLIST_FIRST( &outLine->list[NAPT_ASIC_CACHED_OUT] ) );
	}

#ifdef CACHE_FIND_NAPT_RESULT
	/* If this connection is in the cache, invalid it. */
	if ( delConn == cachedPEntry )
	{
#ifdef CACHE_FIND_NAPT_DEBUG_RESULT
		rtlglue_printf("-: cache flush (delNaptConnection)...\n" );
#endif/*CACHE_FIND_NAPT_DEBUG_RESULT*/
		INVALID_FIND_CACHE();
	}
#endif/*CACHE_FIND_NAPT_RESULT*/

#ifdef FLUSH_NAPT_OLDEST_CONNECTION2
	if ( candidateToBeFreed == delConn )
	{
		candidateToBeFreed = NULL;
	}
#endif/*FLUSH_NAPT_OLDEST_CONNECTION2*/

	/* Update connection counting cache in dos system */
	if ( delConn->dosEnCount == TRUE )
	{
#ifdef CACHE_DOS_COUNT_NAPT_RESULT
		INVALID_DOS_COUNT_CACHE();
#endif
		_rtl8651_dosSrcConnCtrlUpdateConnection(TRUE, delConn->tcp,
			(delConn->dosFromInternal)?(delConn->insideLocalIpAddr):(delConn->dstIpAddr));

		if ( delConn->tcp )
		{
			if ( delConn->dosFromInternal )
			{
				rtl8651_tblDrvNaptCounter.dosCountedFromInternalTcpCnt--;
			}

			rtl8651_tblDrvNaptCounter.dosCountedTcpCnt--;
		}
		else
		{
			if ( delConn->dosFromInternal )
			{
				rtl8651_tblDrvNaptCounter.dosCountedFromInternalUdpCnt--;
			}

			rtl8651_tblDrvNaptCounter.dosCountedUdpCnt--;
		}
	}

	/////////////////////////////
	// Clear inbound ASIC entry if it is ASIC-accelerated.
	/////////////////////////////
	inLine = &DrvTbl.tcpudp[delConn->inboundTblIdx];
	//printfByPolling("removing on inbound line %d", delConn->inboundTblIdx);
	SLIST_REMOVE( &inLine->list[delConn->inListType], delConn, 
	              rtl8651_tblDrv_naptTcpUdpFlowEntry_s, nextIn );

	if( delConn->inListType == NAPT_ASIC_CACHED_IN ) 
	{
		//force clear the ASIC entry
		assert(delConn->nextIn.sle_next==NULL);
		_rtl8651_derefHash2NexthopCount( delConn->inboundTblIdx, inLine, delConn, TRUE, FALSE );
		assert( !SLIST_FIRST( &inLine->list[NAPT_ASIC_CACHED_IN] ) );
	}
		
	////////////////////////////////////////
	//Remove software pointers that links with 'delConn'
	////////////////////////////////////////
	//delConn's alive and age all reset to 0 by this memset()...
	memset( delConn, 0, offsetof( struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s, next ) );
	assert(delConn->alive==0);
	delConn->outboundTblIdx = delConn->outListType = -1;
	delConn->inboundTblIdx = delConn->inListType = -1;
	CTAILQ_REMOVE(&DrvTbl.inuseList.tcpudp, delConn, next);
	CTAILQ_INSERT_HEAD(&DrvTbl.freeList.tcpudp, delConn, next);

#ifdef CONFIG_RTL865X_ROMEPERF
	rtl8651_romeperfExitPoint(ROMEPERF_INDEX_NAPT_DEL);
#endif

	return SUCCESS;
}


#if 0
/*
@func int32		| _rtl8651_refreshNaptConnectionAge	| Internal use only. Refresh all TCP/UDP/ICMP NAPT connections max aging time.
@rvalue SUCCESS	| Always returns SUCCESS.
@comm 
Reset aging time of all ICMP/UDP/TCP NAPT connections in software and ASIC. If remaining aging timer longer than given max value, 
set to given max age time..
*/
static int32 _rtl8651_refreshNaptConnectionAge(uint32 maxage) {
	rtl8651DrvNaptLine_t *thisLine;
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s  entry, *pEntry=&entry, *thisFlow;
	uint32  flowTblIdx;
	if(!naptIpCount)
		return FAILED;

 	//refresh all connections in driver table
	thisFlow = CTAILQ_FIRST(&DrvTbl.inuseList.tcpudp);
	while(thisFlow){
		if(thisFlow->age>maxage)
			thisFlow->age=maxage;
		thisFlow = CTAILQ_NEXT(thisFlow, next);
	}
 	//refresh all connections in ASIC table
	for(flowTblIdx=0; flowTblIdx<RTL8651_TCPUDPTBL_SIZE; flowTblIdx++){
		pEntry=&entry;
		thisLine=&DrvTbl.tcpudp[flowTblIdx];
		_rtl8651_getNaptEntry(flowTblIdx, thisLine, &pEntry, TRUE);
		assert(pEntry);
		if(pEntry->alive&&pEntry->age>maxage){
			pEntry->age=maxage;
			_rtl8651_setNaptEntry(flowTblIdx, thisLine, pEntry, TRUE);	
		}
	}
	return SUCCESS;
}
#endif/*0*/


/*
@func int32		| _rtl8651_flushNaptConnection	| Internal use only. Flush all TCP/UDP/ICMP NAPT connections
@rvalue SUCCESS	| Always returns SUCCESS.
@comm 
Delete ALL ICMP/UDP/TCP NAPT connections currently in table.
*/
static int32 _rtl8651_flushNaptConnection(void) {
	uint32 flowTblIdx, i=CTAILQ_TOTAL(&DrvTbl.inuseList.tcpudp);
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s * tempNaptTcpUdpFlowPtr;
	rtl8651_tblDrv_naptIcmpFlowEntry_t * tempNaptIcmpFlowPtr;
	rtl8651DrvNaptLine_t *tcpudpPtr;

 	//release all connections in driver table
	tempNaptTcpUdpFlowPtr = CTAILQ_FIRST(&DrvTbl.inuseList.tcpudp);
	while(tempNaptTcpUdpFlowPtr){
		//if connection in software, clear in software. If connection in ASIC, clear ASIC content.
		_rtl8651_delNaptConnection(tempNaptTcpUdpFlowPtr);
		tempNaptTcpUdpFlowPtr = CTAILQ_FIRST(&DrvTbl.inuseList.tcpudp);
	}
	
	//Reinitialize whole NAPT lines in table
	for(i=0,tcpudpPtr=&DrvTbl.tcpudp[0];i<RTL8651_TCPUDPTBL_SIZE;i++,tcpudpPtr++) {
		int j;
		//reinit all software lists
		for(j=NAPT_ASIC_CACHED_OUT; j < NAPT_TOTAL_LISTS; j++)
			SLIST_INIT(&tcpudpPtr->list[j]);
	}

	/* flush ASIC Napt TCPUDP flows */
	for(flowTblIdx=0; flowTblIdx<RTL8651_TCPUDPTBL_SIZE; flowTblIdx++)
		convert_setAsicNaptTcpUdpTable(TRUE, 0, 0, flowTblIdx, 0, RTL8651_DYNAMIC_NAPT_ENTRY, FALSE, 0, FALSE, FALSE, FALSE);	


	for(flowTblIdx=0; flowTblIdx<RTL8651_ICMPTBL_SIZE; flowTblIdx++) {
		while(DrvTbl.icmp[flowTblIdx]) {
			tempNaptIcmpFlowPtr = DrvTbl.icmp[flowTblIdx];
			DrvTbl.icmp[flowTblIdx] = tempNaptIcmpFlowPtr->next;
			tempNaptIcmpFlowPtr->next = DrvTbl.freeList.icmp;
			DrvTbl.freeList.icmp = tempNaptIcmpFlowPtr;
		}
	}
	for(flowTblIdx=0; flowTblIdx<RTL8651_ICMPTBL_SIZE; flowTblIdx++) {
		rtl8651NaptIcmpBackwardCollisionCountTable[flowTblIdx] = 0;
		rtl8651NaptIcmpFlowNonAsicNumberTable[flowTblIdx] = 0;
		rtl8651NaptIcmpFlowNumberTable[flowTblIdx] = 0;
	}
	rtl8651ExistedNaptIcmpFlows = 0;


// yjlou note: We no longer support ICMP ASIC-accelerated.
#if 0
	/* flush ASIC Napt ICMP flows */
	for(flowTblIdx=0; flowTblIdx<RTL8651_ICMPTBL_SIZE; flowTblIdx++)
		convert_setAsicNaptIcmpTable(TRUE, 0, 0, flowTblIdx, RTL8651_DYNAMIC_NAPT_ENTRY, FALSE, 0, FALSE, FALSE);
#endif

	return SUCCESS;
}


#ifdef FLUSH_NAPT_CONNECTION_WHEN_MORE_THAN_LIMIT
#ifdef FLUSH_NAPT_OLDEST_CONNECTION2
#else/*FLUSH_NAPT_OLDEST_CONNECTION2*/
/*
@func int32		| _rtl8651_flushNaptFINedConnection	| Internal use only. Flush FINed TCP/UDP NAPT connections
@parm uint32  | NumWantedToBeFreed	| Number of connection wanted to be freed
@rvalue int32	| #NumWantedToBeFreed are already freed.
@comm 
When a connection is double-FINed, we no longer need to maintain it (even if a fast timeout is not reached).
This function can recycle #NumWantedToBeFreed connections.
*/
static int32 _rtl8651_flushNaptFINedConnection( int32 NumWantedToBeFreed ) 
{
	int32 numFreed; 
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s * tempNaptTcpUdpFlowPtr;

	numFreed = 0;
 	//release FINed connections in driver table
	tempNaptTcpUdpFlowPtr = CTAILQ_FIRST(&DrvTbl.inuseList.tcpudp);
	while( NumWantedToBeFreed > 0 &&
	       tempNaptTcpUdpFlowPtr )
	{
		if ( tempNaptTcpUdpFlowPtr->tcpFlag == TCP_CLOSED_FLOW-1 )
		{
			_rtl8651_delNaptConnection(tempNaptTcpUdpFlowPtr);
			NumWantedToBeFreed--;
			numFreed++;
			tempNaptTcpUdpFlowPtr = CTAILQ_FIRST(&DrvTbl.inuseList.tcpudp);
		}
		else
		{
			tempNaptTcpUdpFlowPtr = CTAILQ_NEXT( tempNaptTcpUdpFlowPtr, next );
		}
	}
	
	return numFreed;
}
#endif/*FLUSH_NAPT_OLDEST_CONNECTION2*/


#ifdef FLUSH_NAPT_OLDEST_CONNECTION2
/*
@func int32		| _rtl8651_flushNaptOldestConnection2	| Internal use only. Flush TCP/UDP NAPT connections that has minimum age value.
@parm uint32  | NumWantedToBeFreed	| Number of connection wanted to be freed
@rvalue int32	| #NumWantedToBeFreed are already freed.
@comm 
Consider NAPT table is run out, a new connection is coming.
We must find out some connections to be deleted, even they are not in CLOSED state.
The rule is "delete the connections that have small age value".
This function is the faster version of _rtl8651_flushNaptOldestConnection().
_rtl8651_softwareNaptAging() always findout a connection that has minimum age value.
So that this function can delete immediately.
This function can recycle #NumWantedToBeFreed connections.
*/
static int32 _rtl8651_flushNaptOldestConnection2( int32 NumWantedToBeFreed ) 
{
	int32 numFreed; 
	int i;
	int retval;

	numFreed = 0;
	
	for( i = 0; i < NumWantedToBeFreed; i++ )
	{
		/* To improve performance, we no longer use _rtl8651_softwareNaptAging() to find oldest connection. */
#if 1
		if ( candidateToBeFreed == NULL ) break;
#else
	 	/* If no candidateToBeFreed is selected, call _rtl8651_softwareNaptAging() to choose one. */
		if ( candidateToBeFreed == NULL )
		{
			_rtl8651_softwareNaptAging( 0 );

			/* If still empty, return. */
			if ( candidateToBeFreed == NULL ) break;
		}
#endif
		
		/* Deleting entry found in software table immediately */
		if ( candidateToBeFreed->connectionType == NAPT_TYPE_ALG_CONNECTION )
		{
			retval = _rtl8651_delNaptAlgConnection( candidateToBeFreed );
			assert( retval==SUCCESS );
		}
		else
		{
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("_rtl8651_flushNaptOldestConnection2() deleted ExtPort=%d\n", candidateToBeFreed->insideGlobalPort );
#endif/*ADD_NAPT_DEBUG*/
			retval = _rtl8651_delNaptConnection( candidateToBeFreed );
			assert( retval==SUCCESS );
		}

		candidateToBeFreed = NULL;
		
		numFreed++;
	}
	
	return numFreed;
}
#endif/*FLUSH_NAPT_OLDEST_CONNECTION2*/
#endif/*FLUSH_NAPT_CONNECTION_WHEN_MORE_THAN_LIMIT*/


#if 0
/*
@func int32		| _rtl8651_flushNaptOldestConnection	| Internal use only. Flush TCP/UDP NAPT connections that has short age.
@parm uint32  | NumWantedToBeFreed	| Number of connection wanted to be freed
@rvalue int32	| #NumWantedToBeFreed are already freed.
@comm 
Consider NAPT table is run out, a new connection is coming.
We must find out some connections to be deleted, even they are not in CLOSED state.
The rule is "delete connections with smallest age".
This function can recycle #NumWantedToBeFreed connections.
We note that the algorithm for large #NumWantedToBeFreed is not efficient.
*/
static int32 _rtl8651_flushNaptOldestConnection( int32 NumWantedToBeFreed ) 
{
	int32 numFreed; 
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *tempNaptTcpUdpFlowPtr;
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *toBeDeleted;
	int i;
	int retval;

	numFreed = 0;
	
	for( i = 0; i < NumWantedToBeFreed; i++ )
	{
	 	// traverse connections in driver table
	 	tempNaptTcpUdpFlowPtr = CTAILQ_FIRST(&DrvTbl.inuseList.tcpudp);
	 	toBeDeleted = tempNaptTcpUdpFlowPtr;
		if ( tempNaptTcpUdpFlowPtr == NULL ) break;
		
		while( tempNaptTcpUdpFlowPtr )
		{
			/* If we found a older connection. */
			if ( tempNaptTcpUdpFlowPtr->age < toBeDeleted->age )
			{
				toBeDeleted = tempNaptTcpUdpFlowPtr;
			}
			tempNaptTcpUdpFlowPtr = CTAILQ_NEXT( tempNaptTcpUdpFlowPtr, next );
		}

		/* Deleting entry found in software table immediately */
		if ( toBeDeleted->connectionType == NAPT_TYPE_ALG_CONNECTION )
		{
			retval = _rtl8651_delNaptAlgConnection( toBeDeleted );
			assert( retval==SUCCESS );
		}
		else
		{
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("flushNaptOldestConnection() deleted ExtPort=%d\n", toBeDeleted->insideGlobalPort );
#endif/*ADD_NAPT_DEBUG*/
			retval = _rtl8651_delNaptConnection( toBeDeleted );
			assert( retval==SUCCESS );
		}
		numFreed++;
	}
	
	return numFreed;
}
#endif/*0*/


#ifdef FLUSH_NAPT_CONNECTION_WHEN_MORE_THAN_LIMIT
/*
@func int32		| _rtl8651_flushNaptHeadConnection	| Internal use only. Flush TCP/UDP NAPT connections that inserted for long time.
@parm uint32  | NumWantedToBeFreed	| Number of connection wanted to be freed
@rvalue int32	| #NumWantedToBeFreed are already freed.
@comm 
Consider NAPT table is run out, a new connection is coming.
We must find out some connections to be deleted, even they are not in CLOSED state.
The rule is "always delete the connections at the head of inuse list".
This function is the faster version of _rtl8651_flushNaptOldestConnection().
This function can recycle #NumWantedToBeFreed connections.
*/
static int32 _rtl8651_flushNaptHeadConnection( int32 NumWantedToBeFreed ) 
{
	int32 numFreed; 
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *tempNaptTcpUdpFlowPtr;
	int i;
	int retval;

	numFreed = 0;
	
	for( i = 0; i < NumWantedToBeFreed; i++ )
	{
	 	// find the last one in the list
	 	tempNaptTcpUdpFlowPtr = CTAILQ_FIRST(&DrvTbl.inuseList.tcpudp);
		if ( tempNaptTcpUdpFlowPtr == NULL ) break;
		
		/* Deleting entry found in software table immediately */
		if ( tempNaptTcpUdpFlowPtr->connectionType == NAPT_TYPE_ALG_CONNECTION )
		{
			retval = _rtl8651_delNaptAlgConnection( tempNaptTcpUdpFlowPtr );
			assert( retval==SUCCESS );
		}
		else
		{
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("_rtl8651_flushNaptHeadConnection() deleted ExtPort=%d\n", tempNaptTcpUdpFlowPtr->insideGlobalPort );
#endif/*ADD_NAPT_DEBUG*/
			retval = _rtl8651_delNaptConnection( tempNaptTcpUdpFlowPtr );
			assert( retval==SUCCESS );
		}
		numFreed++;
	}
	
	return numFreed;
}
#endif/*FLUSH_NAPT_CONNECTION_WHEN_MORE_THAN_LIMIT*/


/*
@func void	| _rtl8651_removeNaptConnectionByExtIP	| Internal use only. Flush all NAPT connections with <p sip> as internal IP.
@parm ipaddr_t  | sip	| The internal source IP. All connections originated from this host would be deleted.
@rdesc	<p _rtl8651_removeNaptConnectionByExtIP()> has no return value.
@comm 
Given a internal IP <p sip>, remove all NAPT connections originated from this host.
This would traverse whole software table and also ASIC NAPT table exactly once.
*/

void _rtl8651_removeNaptConnectionByExtIP( ipaddr_t ExtIp ) 
{
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *naptflow;
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *nextFlow;
	uint32 flowTblIdx;
	rtl8651_tblDrv_naptIcmpFlowEntry_t * tempNaptIcmpFlowPtr;

 	// flush specified ExtIp in table
	naptflow = CTAILQ_FIRST(&DrvTbl.inuseList.tcpudp);
	while(naptflow)
	{
		nextFlow = CTAILQ_NEXT( naptflow, next );;
		if ( naptflow->insideGlobalIpAddr == ExtIp )
			_rtl8651_delNaptConnection(naptflow);
		naptflow = nextFlow;
	}

	/* chhuang: Flush ICMP: Do not distinguish the specified NAPT IP address, just flush all! */
	for(flowTblIdx=0; flowTblIdx<RTL8651_ICMPTBL_SIZE; flowTblIdx++) {
		while(DrvTbl.icmp[flowTblIdx]) {
			tempNaptIcmpFlowPtr = DrvTbl.icmp[flowTblIdx];
			DrvTbl.icmp[flowTblIdx] = tempNaptIcmpFlowPtr->next;
			tempNaptIcmpFlowPtr->next = DrvTbl.freeList.icmp;
			DrvTbl.freeList.icmp = tempNaptIcmpFlowPtr;
		}
	}
	for(flowTblIdx=0; flowTblIdx<RTL8651_ICMPTBL_SIZE; flowTblIdx++) {
		rtl8651NaptIcmpBackwardCollisionCountTable[flowTblIdx] = 0;
		rtl8651NaptIcmpFlowNonAsicNumberTable[flowTblIdx] = 0;
		rtl8651NaptIcmpFlowNumberTable[flowTblIdx] = 0;
	}
	rtl8651ExistedNaptIcmpFlows = 0;
}


/*
@func void	| _rtl8651_removeNaptConnectionBySrcIP	| Internal use only. Flush all NAPT connections with <p sip> as internal IP.
@parm ipaddr_t  | sip	| The internal source IP. All connections originated from this host would be deleted.
@rdesc	<p _rtl8651_removeNaptConnectionBySrcIP()> has no return value.
@comm 
Given a internal IP <p sip>, remove all NAPT connections originated from this host.
This would traverse whole software table and also ASIC NAPT table exactly once.
*/

void _rtl8651_removeNaptConnectionBySrcIP( ipaddr_t sip ) 
{
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *naptflow;
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *nextFlow;
	rtl8651_tblDrv_naptIcmpFlowEntry_t * tempNaptIcmpFlowPtr;
	uint32 flowTblIdx;

	// flush specified ExtIp in table
	naptflow = CTAILQ_FIRST(&DrvTbl.inuseList.tcpudp);
	while(naptflow)
	{
		nextFlow = CTAILQ_NEXT( naptflow, next );;
		if ( naptflow->insideLocalIpAddr == sip )
			_rtl8651_delNaptConnection(naptflow);
		naptflow = nextFlow;
	}

	/* chhuang: Flush ICMP: Do not distinguish the specified NAPT IP address, just flush all! */
	for(flowTblIdx=0; flowTblIdx<RTL8651_ICMPTBL_SIZE; flowTblIdx++) {
		while(DrvTbl.icmp[flowTblIdx]) {
			tempNaptIcmpFlowPtr = DrvTbl.icmp[flowTblIdx];
			DrvTbl.icmp[flowTblIdx] = tempNaptIcmpFlowPtr->next;
			tempNaptIcmpFlowPtr->next = DrvTbl.freeList.icmp;
			DrvTbl.freeList.icmp = tempNaptIcmpFlowPtr;
		}
	}
	for(flowTblIdx=0; flowTblIdx<RTL8651_ICMPTBL_SIZE; flowTblIdx++) {
		rtl8651NaptIcmpBackwardCollisionCountTable[flowTblIdx] = 0;
		rtl8651NaptIcmpFlowNonAsicNumberTable[flowTblIdx] = 0;
		rtl8651NaptIcmpFlowNumberTable[flowTblIdx] = 0;
	}
	rtl8651ExistedNaptIcmpFlows = 0;
}


/*
@func void	| _rtl8651_removeNaptConnectionByIntIPandPort	| Internal use only. Flush all NAPT connections with <p intIp> as internal IP and <p intPort> as internal port.
@parm int8  | isTcp	| TRUE for TCP, FALSE for UDP.
@parm ipaddr_t  | intIp	| The internal IP.
@parm uint16  | intPort	| The internal Port.
@rdesc	<p _rtl8651_removeNaptConnectionByIntIPandPort()> returns the number of connections deleted.
@comm 
Given a pair of {isTcp,intIp,intPort}, those connections will be deleted.
In original, this is designed for Netmeeting ALG.
Note that, this function will not delete ICMP flow.
*/
int32 _rtl8651_removeNaptConnectionByIntIPandPort( int8 isTcp, ipaddr_t intIp, uint16 intPort ) 
{
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *thisFlow;
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *nextFlow;
	int32 delNo;

	delNo = 0;
	// flush specified ExtIp in table
	thisFlow = CTAILQ_FIRST( &DrvTbl.inuseList.tcpudp );
	while( thisFlow )
	{
		nextFlow = CTAILQ_NEXT( thisFlow, next );;
		if ( thisFlow->tcp == isTcp &&
		     thisFlow->insideLocalIpAddr == intIp &&
		     thisFlow->insideLocalPort == intPort )
		{
			delNo++;
			_rtl8651_delNaptConnection( thisFlow );
		}
		thisFlow = nextFlow;
	}

	return delNo;
}

/*
@func void	| _rtl8651_removeNaptConnectionByIP	| Internal use only. Flush all NAPT connections with <p ip> as internal IP or destination IP.
@parm ipaddr_t  | ip	| The internal IP.
@rdesc	<p _rtl8651_removeNaptConnectionByIP()> returns the number of connections deleted.
@comm 
Given a ip address, those connections will be deleted.
In original, this is designed for DoS Jail system.
Note that, this function will not delete ICMP flow.
*/
int32 _rtl8651_removeNaptConnectionByIP( ipaddr_t ip ) 
{
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *thisFlow;
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *nextFlow;
	int32 delNo;

	delNo = 0;

	thisFlow = CTAILQ_FIRST( &DrvTbl.inuseList.tcpudp );
	while( thisFlow )
	{
		nextFlow = CTAILQ_NEXT( thisFlow, next );;
		if ( thisFlow->insideLocalIpAddr == ip ||
		     thisFlow->dstIpAddr== ip )
		{
			delNo++;
			_rtl8651_delNaptConnection( thisFlow );
		}
		thisFlow = nextFlow;
	}

	return delNo;
}

/*
@func void	| _rtl8651_countNaptConnectionByIP	| Internal use only. Count number of all NAPT connections with <p ip> as internal IP or destination IP.
@parm ipaddr_t  | ip	| The internal IP.
@rdesc	<p _rtl8651_countNaptConnectionByIP()> returns the result of counting.
@comm 
Given a ip address, those connections will be deleted.
In original, this is designed for DoS Jail system.
Note that, this function will not delete ICMP flow.
*/
int32 _rtl8651_countNaptConnectionByIP( ipaddr_t ip, uint32 *tcpCnt, uint32 *udpCnt )
{
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *thisFlow;
	uint32 tcp;
	uint32 udp;

	tcp = 0;
	udp = 0;

#ifdef CACHE_DOS_COUNT_NAPT_RESULT
	if ( ip == cachedDosCntIp )
	{
		tcp = cachedDosTcpCnt;
		udp = cachedDosUdpCnt;
		goto setResult;
	}
#endif

	thisFlow = CTAILQ_FIRST( &DrvTbl.inuseList.tcpudp );
	while( thisFlow )
	{
		if ( thisFlow->dosEnCount == TRUE )
		{
			if (( thisFlow->dosFromInternal && thisFlow->insideLocalIpAddr == ip ) ||
				( !thisFlow->dosFromInternal && thisFlow->dstIpAddr == ip ))
			{
				if ( thisFlow->tcp )
				{
					tcp ++;
				}
				else
				{
					udp ++;
				}
			}
		}
		thisFlow = CTAILQ_NEXT( thisFlow, next );
	}

#ifdef CACHE_DOS_COUNT_NAPT_RESULT
setResult:
#endif

	if (tcpCnt)
	{
		*tcpCnt = tcp;
	}
	if (udpCnt)
	{
		*udpCnt = udp;
	}

	return SUCCESS;
}


/*
@func int32		| _rtl8651_FindEmptyASICEntry	| Internal function only. Find an empty ASIC entry.
@parm uint16 	| tblIndex	| The index in the DrvTbl.tcpudp.
@rdesc	Returns SUCCESS if found, and stores the found entry in <p tblIndex>.
@rvalue FAILED	| Cannot find empty entry in reasonable trials.
@comm 
This function will try for n times to find an empty entry.
*/
static int32 
_rtl8651_FindEmptyASICEntry( uint16 *tblIndex )
{
	uint16 i;
	// FIXME: Should be implemented by freelist, not retry mode
	for( i = 0; i < rtl8651GlobalControl.l4NaptFindEmptyAsicRetry; i++ ) 
	{
		if ( _rtl8651_NaptEntryIsEmpty( *tblIndex ) )
		{
			return SUCCESS;
		}

		*tblIndex = ( (*tblIndex) + 1 ) & ( RTL8651_TCPUDPTBL_SIZE - 1 );
	}
	return FAILED;
}


/* just for debug */
void _dump_naptEntry( rtl8651_tblDrv_naptTcpUdpFlowEntry_t* pe )
{
	rtlglue_printf("[%s] %s Int:%d.%d.%d.%d:%d Ext:%d.%d.%d.%d:%d Rem:%d.%d.%d.%d:%d\n",
   			       pe->hashing==NAPT_HASH1?"H1":"H2",
   			       pe->tcp?"TCP":"UDP",
   			       pe->insideLocalIpAddr>>24, (pe->insideLocalIpAddr&0x00ff0000) >> 16,
   			       (pe->insideLocalIpAddr&0x0000ff00)>>8, pe->insideLocalIpAddr&0x000000ff,
   			       pe->insideLocalPort, 
   			       pe->insideGlobalIpAddr>>24, (pe->insideGlobalIpAddr&0x00ff0000) >> 16,
   			       (pe->insideGlobalIpAddr&0x0000ff00)>>8, pe->insideGlobalIpAddr&0x000000ff,
   			       pe->insideGlobalPort, 
   			       pe->dstIpAddr>>24, (pe->dstIpAddr&0x00ff0000) >> 16,
   			       (pe->dstIpAddr&0x0000ff00)>>8, pe->dstIpAddr&0x000000ff,
   			       pe->dstPort );

	rtlglue_printf("    %s age(%d) tcpflag(%d) inList:%s outList:%s\n",
			        (pe->connectionType==NAPT_TYPE_NORMAL)?"NOR":"SP",
			        pe->age, pe->tcpFlag,
			        pe->inListType==NAPT_ASIC_CACHED_IN?"ASIC_IN":
			        pe->inListType==NAPT_CANDIDATE_IN?"CAND_IN":
			        pe->inListType==NAPT_ALG_IN?"ALG_IN":"Uknown",
			        pe->outListType==NAPT_ASIC_CACHED_OUT?"ASIC_OUT":
			        pe->outListType==NAPT_CANDIDATE_OUT?"CAND_OUT":
			        pe->outListType==NAPT_ALG_OUT?"ALG_OUT":"Uknown" );
}

/* just for debug */
void _dump_naptLine( uint16 inIdx, uint16 outIdx )
{
	rtl8651DrvNaptLine_t *thisNaptLine;
	rtl8651_tblDrv_naptTcpUdpFlowEntry_t *thisConn;
	uint32 i;
	
	rtlglue_printf( "Dump NaptLine -----\n" );
	thisNaptLine = &DrvTbl.tcpudp[inIdx];
	for( i = NAPT_ASIC_CACHED_IN; i < NAPT_TOTAL_LISTS; i += 2 )
	{
		rtlglue_printf( "idx[%d],list[%d]:\n", inIdx, i );
		SLIST_FOREACH( thisConn, &thisNaptLine->list[i], nextIn )
		{
			_dump_naptEntry( thisConn );
		}
	}
	thisNaptLine = &DrvTbl.tcpudp[outIdx];
	for( i = NAPT_ASIC_CACHED_OUT; i < NAPT_TOTAL_LISTS; i += 2 )
	{
		rtlglue_printf( "idx[%d],list[%d]:\n", outIdx, i );
		SLIST_FOREACH( thisConn, &thisNaptLine->list[i], nextOut )
		{
			_dump_naptEntry( thisConn );
		}
	}
	rtlglue_printf( "-------------------\n" );
}


/*
@func int8 | _rtl8651_AvoidAmbiguousHash2InAsic | This function is used to avoid the ambiguous situation.
@param rtl8651_tblDrv_naptTcpUdpFlowEntry_t | newConn | The connection wantted to write to ASIC (maybe Hash1 or Hash2).
@param int* | bInboundAmbiguous | Return TRUE if the ExtPort is shared and ambiguous with other connections.
@param int* | bOutboundAmbiguous | Return TRUE if the IntIp/Port is shared and ambiguous with other connections.
@param int* | bInboundAsicEmpty | Return TRUE if the inbound ASIC entry is free.
@param int* | bOutboundAsicEmpty | Return TRUE if the outbound ASIC entry is free.
@rvalue SUCCESS |
@comm
This function is used to avoid the ambiguous situations.
Typically, this is called by _rtl8651_addNaptConnection() and maintain thread
  when they want to change Hash2 connection into ASIC.
The reason is that the ASIC does NOT verify the packets that are matched Hash 2 entry in ASIC.
Consider the following cases:

For inbound flow:
	If two connections share one ExtPort, but have different IntIp/Port, we call it inbound ambiguous.
	If the <p newConn> is ambiguous with the current ASIC entry,
	  the bInboundAmbiguous is set to TRUE, which means <p newConn> CANNOT be writen to ASIC.
	  Further more, if the ASIC entry is hold by a Hash2 connection, swap it to Candidate list.
	If the <p newConn> is ambiguous with the connection in Candidate or ALG list,
	  the bInboundAmbiguous is set to TRUE, which means <p newConn> CANNOT be writen to ASIC.

For outbound flow:
	If two connections share one IntIp/Port, but have different ExtIp/Port, we call it outbound ambiguous.
	If the <p newConn> is ambiguous with the current ASIC entry,
	  the bOutboundAmbiguous is set to TRUE, which means <p newConn> CANNOT be writen to ASIC.
	  Further more, if the ASIC entry is hold by a Hash2 connection, swap it to Candidate list.
	If the <p newConn> is ambiguous with the connection in Candidate or ALG list,
	  the bOutboundAmbiguous is set to TRUE, which means <p newConn> CANNOT be writen to ASIC.

Reference to 8651B Enhancement Specification Ver0.15, 11.2.3, the figures of ASIC behavior.

We note that, before this function is called:
1. If the ASIC is free and the ExtPort is not shared, Hash 2 connection can use it.
   (Because only one connection exists, and never ambiguous).
2. If the ASIC is in-used, Hash 2 connection would be queued in candidate list first.

*/
int32 _rtl8651_AvoidAmbiguousHash2InAsic( rtl8651_tblDrv_naptTcpUdpFlowEntry_t *newConn,
                                          int *bInboundAmbiguous, int *bOutboundAmbiguous,
                                          int *bInboundAsicEmpty, int *bOutboundAsicEmpty )
{
	rtl8651DrvNaptLine_t *thisNaptLine;
	rtl8651_tblDrv_naptTcpUdpFlowEntry_t *thisConn;
	uint16 inIdx, outIdx;
	int32 i;

	*bInboundAmbiguous = *bOutboundAmbiguous = FALSE;
	*bInboundAsicEmpty = *bOutboundAsicEmpty = TRUE; // we guest the ASIC entry is free.
	
	// traverse inbound list
	inIdx = newConn->inboundTblIdx;
	thisNaptLine = &DrvTbl.tcpudp[inIdx];
	for( i = NAPT_ASIC_CACHED_IN; i < NAPT_TOTAL_LISTS; i += 2 )
	{
		SLIST_FOREACH( thisConn, &thisNaptLine->list[i], nextIn )
		{
			if ( i == NAPT_ASIC_CACHED_IN ) *bInboundAsicEmpty = FALSE; // The ASIC entry is in-used.
			if ( thisConn == newConn ) continue; // newConn itself
			if ( thisConn->tcp == newConn->tcp &&
			     thisConn->insideGlobalPort == newConn->insideGlobalPort )
			{
				// yes, this connection is sharing ExtPort with newConn.
				if ( thisConn->insideLocalIpAddr != newConn->insideLocalIpAddr ||
				     thisConn->insideLocalPort != newConn->insideLocalPort )
				{
					// the ASIC cannot tell who is who.
					*bInboundAmbiguous = TRUE;

					// ambiguous with ASIC entry
					if ( i == NAPT_ASIC_CACHED_IN )
					{
						if ( thisConn->hashing == NAPT_HASH2 )
						{
							// Hash2 connection holds ASIC entry, swap to candidate.
							_rtl8651_changeNaptFlowListType( thisConn, NAPT_CANDIDATE_IN, thisConn->outListType );
							*bInboundAsicEmpty = TRUE; // The ASIC entry is free again.
						}
					}
					
					break;
				}
			}
		}
		if ( *bInboundAmbiguous == TRUE ) break; // break again
	}


	// traverse outbound list
	outIdx = newConn->outboundTblIdx;
	thisNaptLine = &DrvTbl.tcpudp[outIdx];
	for( i = NAPT_ASIC_CACHED_OUT; i < NAPT_TOTAL_LISTS; i += 2 )
	{
		SLIST_FOREACH( thisConn, &thisNaptLine->list[i], nextOut )
		{
			if ( i == NAPT_ASIC_CACHED_OUT ) *bOutboundAsicEmpty = FALSE; // The ASIC entry is in-used.
			if ( thisConn == newConn ) continue; // newConn itself
			if ( thisConn->tcp == newConn->tcp &&
			     thisConn->insideLocalIpAddr == newConn->insideLocalIpAddr &&
			     thisConn->insideLocalPort == newConn->insideLocalPort )
			{
				// yes, this connection is sharing IntPort with newConn.
				if ( thisConn->insideGlobalIpAddr != newConn->insideGlobalIpAddr ||
				     thisConn->insideGlobalPort != newConn->insideGlobalPort )
				{
					// the ASIC cannot tell who is who.
					*bOutboundAmbiguous = TRUE;

					// ambiguous with ASIC entry
					if ( i == NAPT_ASIC_CACHED_OUT )
					{
						if ( thisConn->hashing == NAPT_HASH2 )
						{
							// Hash2 connection holds ASIC entry, swap to candidate.
							_rtl8651_changeNaptFlowListType( thisConn, thisConn->inListType, NAPT_CANDIDATE_OUT );
							*bOutboundAsicEmpty = TRUE; // The ASIC entry is free again.
						}
					}

					if ( newConn->outListType == NAPT_ASIC_CACHED_OUT &&
					     newConn->hashing == NAPT_HASH2 )
					{
						/* If the newConn is a HASH2 connection and in ASIC, it will cause ambigious.
						 * We note that newConn may NOT be a new connection.
						 * newConn may be an existed connection.
						 */
						_rtl8651_changeNaptFlowListType( newConn, newConn->inListType, NAPT_CANDIDATE_OUT );
						*bOutboundAsicEmpty = TRUE; // The ASIC entry is free again.
					}
					
					break;
				}
				else if ( thisConn->insideGlobalIpAddr == newConn->insideGlobalIpAddr &&
				          thisConn->insideGlobalPort == newConn->insideGlobalPort )
				{
					/* The IntIp/IntPort/ExtIp/ExtPort of thisConn and newConn are identical.
					 * Therefore, the ASIC cannot tell who is who.
					 * Consider any one RemIp is interface route, ASIC will use wrong DMAC to forward packets.
					 * Further, the ASIC will also use wrong DMAC in multiple nexthop configuration.
					 * So, we DO NOT permit to write them to ASIC.
					 */
					_rtl8651_changeNaptFlowListType( newConn, newConn->inListType, NAPT_CANDIDATE_OUT );
					*bOutboundAmbiguous = TRUE;
					*bOutboundAsicEmpty = TRUE; // The ASIC entry is free again.
				}
			}
		}
		if ( *bOutboundAmbiguous == TRUE ) break; // break again
	}

#if 0
	// The symmetric hash2 connection CANNOT use ASIC to accelerate 
	//   if it causes ambiguous in either inbound or outbound entry.
	if ( newConn->hashing == NAPT_HASH2 && 
	     inIdx == outIdx )
	{
		if ( *bInboundAmbiguous == TRUE || *bOutboundAmbiguous == TRUE )
		{
			*bInboundAmbiguous = *bOutboundAmbiguous = TRUE;
		}
	}
#endif//0

	return SUCCESS;
}


/*
@func int8		| _rtl8651_addNaptConnection	| Internal function only. Add a NAPT connection to software table or ASIC, depends on the extPort selected and/or hash algorithm used.
@parm int8 		| fromDrv	| TRUE if requested from forwarding engine. FALSE if requested from user protocol stack.
@parm int8 		| assigned	| TRUE if <p extIP> and <p extPort> already assigned. FALSE if <p extIP> and <p extPort> can be selected freely by driver.
@parm int8 		| flowType	| Type of connections adding. Possible values: UDP_FLOW, TCP_LISTEN_FLOW, TCP_OUTBOUND_SYN_RCVD_FLOW, TCP_INBOUND_SYN_RCVD_FLOW, TCP_ESTABLISHED_FLOW.
@parm ipaddr_t 	| intIP	| Internal IP.
@parm uint16 	| intPort	| Internal Port.
@parm ipaddr_t *	| extIP	| External IP. If <p assigned> is TRUE, <p *extIP> must not be NULL.
@parm uint16	*	| extPort	| External port.  If <p assigned> is FALSE, <p *extIP> must not be NULL.
@parm ipaddr_t	| dstIP	| Destination IP
@parm uint16		| dstPort	| Destination port
@rdesc	Returns SUCCESS if connection added successfully. Returns one of the following eroor codes when error occured.
@rvalue TBLDRV_ENOTINTIP		| 	internal IP <p intIP> does not belong to any of the configured network interfaces.
@rvalue TBLDRV_EDSTIPISLS	| 	Destination IP <p dstIP> belongs to a configured local server. should use routing not NAPT.
@rvalue TBLDRV_ENOTEXTIP		| 	External IP address <p extIP> given is not one of the configured WAN IPs.
@rvalue TBLDRV_ENOFREEBUF	| 	Out of buffer space. Can't add any more.
@rvalue TBLDRV_EINVALIDEXTPORT | External port <p extPort> given already inuse.
@rvalue TBLDRV_EDRVNAPTEXIST | 	Connection already exists.
@rvalue TBLDRV_ENOTPERMIT	| 	Assigned external IP, port pair doesn't match with hash result. Forbid this new entry add.
@rvalue TBLDRV_EGETOFFSETFAIL | 	Can't allocate a valid offset value when selecting a <p extPort>.
@comm 
This internal function adds a new NAPT connection into table driver. Driver uses both calculated hash keys to determine which ASIC hash algorithm to use.
No matter <p extIp> and <p extPort>
*/

/**************************************************************************
 *    API for legacy code
 **************************************************************************/
int32 _rtl8651_addNaptConnection(int8 fromDrv, int16 assigned, int8 flowType, ipaddr_t insideLocalIpAddr, uint16 insideLocalPort, 
			ipaddr_t *insideGlobalIpAddr, uint16 *insideGlobalPort, ipaddr_t dstIpAddr, uint16 dstPort) 
{
#if 0
	return _rtl8651_addAndGetNaptConnection(fromDrv,assigned,flowType,insideLocalIpAddr,insideLocalPort,
	                                      insideGlobalIpAddr,insideGlobalPort,dstIpAddr,dstPort, NULL );
}


/**************************************************************************
 *    API for new code (faster)
 **************************************************************************/
int32 _rtl8651_addAndGetNaptConnection(int8 fromDrv, int16 assigned, int8 flowType, ipaddr_t insideLocalIpAddr, uint16 insideLocalPort, 
			ipaddr_t *insideGlobalIpAddr, uint16 *insideGlobalPort, ipaddr_t dstIpAddr, uint16 dstPort,
			struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s **ppe ) 
{
#endif
	uint16 idx1, idx2; // index for Hash1 and Hash2
	uint16 inIdx, outIdx; // index for incoming port and outgoing slots(not port)
	uint32 isTcp = (flowType==UDP_FLOW? FALSE: TRUE);
	uint16 foundIdx;
	uint8 bPortReused = FALSE; // TRUE if the want-to-add port is shared with other connection.
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *pe; // temporal storage
	int8 isExternal, is_gwip;
	uint32 interfaceRoute = FALSE;

#ifdef CONFIG_RTL865X_ROMEPERF
	rtl8651_romeperfEnterPoint(ROMEPERF_INDEX_NAPT_ADD);
#endif

	assert((assigned & DONT_CHECK_INTIP) || insideLocalIpAddr);
	//assert(insideLocalPort);

	//Must have inside Global IP address and Global port for storage
	assert(insideGlobalIpAddr);
	assert(insideGlobalPort);

	/*
	 * If this connection is not added by ROME driver, we should suspect it.
	 * Set DO_REDUNDANCY_CHECK bit for later check.
	 */
	if ( fromDrv == FALSE ) assigned |= DO_REDUNDANCY_CHECK;

	idx1 = rtl8651_naptTcpUdpTableIndex( isTcp, insideLocalIpAddr, insideLocalPort, dstIpAddr, dstPort );
	idx2 = rtl8651_naptTcpUdpTableIndex( isTcp, insideLocalIpAddr, insideLocalPort, 0, 0 );

#ifdef ADD_NAPT_DEBUG
	rtlglue_printf("[%s:%d] addNaptConnection(fromDrv:%d,assigned=%d,flowType=%d,\n  intIp=%d.%d.%d.%d,intPort=%d,\n  extIp=%d.%d.%d.%d,extPort=%d,\n  dstIp=%d.%d.%d.%d,dstPort=%d), idx1=%d, idx2=%d.\n",
	               __FILE__,__LINE__,fromDrv,assigned,flowType,NIPQUAD(insideLocalIpAddr),insideLocalPort,NIPQUAD(*insideGlobalIpAddr),*insideGlobalPort,NIPQUAD(dstIpAddr),dstPort,idx1,idx2);
#endif/*ADD_NAPT_DEBUG*/


	/*************************************************************************
	 * The following code is to check if the user specified the existing connection ?
	 * The following code is to check if the newly adding port is duplicate with the added port.
	 ************************************************************************/
	
	rtl8651_tblDrvNaptCounter.addAttempts++;

	//Check given IP address type. insideLocalIpAddr must be on LAN interface,

#ifdef CACHE_DOS_COUNT_NAPT_RESULT
	INVALID_DOS_COUNT_CACHE();
#endif

#ifdef CONFIG_RTL865X_ROMEPERF
	rtl8651_romeperfEnterPoint(ROMEPERF_INDEX_NAPT_ADD_1);
#endif
	if ((!(assigned & DONT_CHECK_INTIP)) && (_rtl8651_intNetIpAddr(insideLocalIpAddr)==FALSE))
//	if ((!(assigned & DONT_CHECK_INTIP)) && (_rtl8651_getIpInfo(insideLocalIpAddr, NULL, &isExternal, &is_gwip)==NULL || isExternal==TRUE || is_gwip == TRUE))
	{
#ifdef ADD_NAPT_DEBUG
		rtlglue_printf("[%s:%d] addNaptConnection() returns TBLDRV_ENOTINTIP.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
		return TBLDRV_ENOTINTIP;
	}

#ifdef CONFIG_RTL865X_ROMEPERF
	rtl8651_romeperfExitPoint(ROMEPERF_INDEX_NAPT_ADD_1);
#endif

#ifdef CONFIG_RTL865X_ROMEPERF
	rtl8651_romeperfEnterPoint(ROMEPERF_INDEX_NAPT_ADD_2);
#endif
	// Check dest IP. packets to Local server should not do NAPT.
	if( dstIpAddr && _rtl8651_localServerIpAddr( dstIpAddr ) == TRUE )
	{
#ifdef ADD_NAPT_DEBUG
		rtlglue_printf("[%s:%d] addNaptConnection() returns TBLDRV_EDSTIPISLS.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
		return TBLDRV_EDSTIPISLS;
	}
#ifdef CONFIG_RTL865X_ROMEPERF
	rtl8651_romeperfExitPoint(ROMEPERF_INDEX_NAPT_ADD_2);
#endif

#ifdef CONFIG_RTL865X_ROMEPERF
	rtl8651_romeperfEnterPoint(ROMEPERF_INDEX_NAPT_ADD_3);
#endif
	//insideGlobalIpAddr must be on WAN interface
	if( assigned & EXTIPPORT_ASSIGNED )
	{
		if (_rtl8651_getIpInfo(*insideGlobalIpAddr, NULL, &isExternal, &is_gwip) == NULL || isExternal == FALSE || is_gwip == FALSE)
		{
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("[%s:%d] addNaptConnection() returns TBLDRV_EINVEXTIP.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
			return TBLDRV_EINVEXTIP;
		}
	}
#ifdef CONFIG_RTL865X_ROMEPERF
	rtl8651_romeperfExitPoint(ROMEPERF_INDEX_NAPT_ADD_3);
#endif

#if 0 /* Louis note: ExtPort is allowed now! For CDRouter. */
	// the user should not assign non-sense port number
	if ( assigned & EXTIPPORT_ASSIGNED )
	{
		if ( *insideGlobalPort == 0 )
		{
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("[%s:%d] addNaptConnection() returns TBLDRV_EINVL4PORTNUM.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
			return TBLDRV_EINVL4PORTNUM;
		}
	}
#endif/*0*/

	// this is non-sense IP and Port.
	if ( dstIpAddr != 0 && dstPort == 0 )
	{
#ifdef ADD_NAPT_DEBUG
		rtlglue_printf("[%s:%d] addNaptConnection() returns TBLDRV_EINVL4PORTNUM.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
		return TBLDRV_EINVL4PORTNUM;
	}

	// check if "duplicate" connection is existed ?
	// Note that the following two connections are considered different.
	//   int     ext   remote
	//    A ----- B ----- C
	//    A ----- B
	if ( ( assigned & DO_REDUNDANCY_CHECK ) == FALSE )
	{
		/* If this is fwdEngine's request, we trust it had traversed.
		 * For performance issue, we no longer traverse again. 
		 */
	}
	else if ( dstIpAddr != 0 && dstPort != 0 )
	{
#ifdef CONFIG_RTL865X_ROMEPERF
		rtl8651_romeperfEnterPoint(ROMEPERF_INDEX_NAPT_ADD_4);
#endif

#if ALLOW_SP>0
		// if dip/dport are provided, just traversal idx1 and idx2.
		if( _rtl8651_findNaptConnection( idx1, isTcp, insideLocalIpAddr, insideLocalPort,
		                                              dstIpAddr, dstPort ) )
		{
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("[%s:%d] addNaptConnection() returns TBLDRV_EDRVNAPTEXIST.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
			return TBLDRV_EDRVNAPTEXIST; //entry already exist in driver table.
		}
		
		if( ( assigned & EXTIPPORT_ASSIGNED ) &&
		    _rtl8651_findNaptConnectionByExtIpPort( isTcp, *insideGlobalIpAddr, *insideGlobalPort,
		                                            dstIpAddr, dstPort ) )
		{
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("[%s:%d] addNaptConnection() returns TBLDRV_EDRVNAPTEXIST.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
			return TBLDRV_EINVALIDEXTPORT; //entry already exist in driver table.
		}
#else/*ALLOW_SP>0*/
		/* If we allow server port, the following code will cause error in test code. */
		rtl8651_tblDrv_naptTcpUdpFlowEntry_t *pDummy;
		
		if ( _rtl8651_getOutboundNaptFlow( isTcp, insideLocalIpAddr, insideLocalPort, 
		                                   dstIpAddr, dstPort, &pDummy ) == SUCCESS )
		{
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("[%s:%d] addNaptConnection() returns TBLDRV_EDRVNAPTEXIST.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
			return TBLDRV_EDRVNAPTEXIST; //entry already exist in driver table.
		}
#endif/*ALLOW_SP>0*/

#ifdef CONFIG_RTL865X_ROMEPERF
		rtl8651_romeperfExitPoint(ROMEPERF_INDEX_NAPT_ADD_4);
#endif
	}
	else
	{
		// if dip/dport are not present, we must traversal all the table.
		//   (because we do not know the idx1 of the connection).
#ifdef CONFIG_RTL865X_ROMEPERF
		rtl8651_romeperfEnterPoint(ROMEPERF_INDEX_NAPT_ADD_5);
#endif
#if 1
		struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *tmp;

 		if( assigned & EXTIPPORT_ASSIGNED )
		{
			for( tmp = CTAILQ_FIRST(&DrvTbl.inuseList.tcpudp);
			     tmp;
		 	     tmp = CTAILQ_NEXT( tmp, next ) )
		 	{
				/* If the user specifies the ExtIp/ExtPort,
				 *   then we must detect the duplicated connection as followed.
				 *
				 *   Int    Ext
				 *   A,B====A,B
				 */
		 		if ( tmp->tcp==isTcp &&
		 		     tmp->insideLocalIpAddr==insideLocalIpAddr &&
		 		     tmp->insideLocalPort==insideLocalPort && 
		 		     tmp->insideGlobalIpAddr==*insideGlobalIpAddr &&
		 		     tmp->insideGlobalPort==*insideGlobalPort )
		 		{
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] addNaptConnection() returns TBLDRV_EDRVNAPTEXIST.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
					return TBLDRV_EDRVNAPTEXIST; //entry already exist in driver table.
		 		}
			}
		}
 		else
 		{
 			/* Since the user does not specify the ExtIp/ExtPort,
 			 * It is always OK because _rtl8651_getUsableExtIpAndPort() will avoid this problem.
 			 */
	 	}
#else/*1*/
		uint16 i;
		for( i = 0; i < RTL8651_TCPUDPTBL_SIZE; i++ )
		{
			// Note that: dip/dport are NULL.
			if( _rtl8651_findNaptConnection( i, isTcp, insideLocalIpAddr, insideLocalPort,
			                                           dstIpAddr, dstPort ) )
			{
#ifdef ADD_NAPT_DEBUG
				rtlglue_printf("[%s:%d] addNaptConnection() returns TBLDRV_EDRVNAPTEXIST.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
				return TBLDRV_EDRVNAPTEXIST; //entry already exist in driver table.
			}
		}
#endif/*1*/
#ifdef CONFIG_RTL865X_ROMEPERF
		rtl8651_romeperfExitPoint(ROMEPERF_INDEX_NAPT_ADD_5);
#endif
	}

	/*
	 * If the user specified the gport, then the following four conditions are denied. 
	 * They will cause ambiguous situations.
	 *
	 *         internal    external        remote
	 *
	 *          A --\             
	 * Case 1        >====== gport ========= A,B
	 *          B --/
	 *
	 *
	 * Case 2   A,B ======== gport ========= A,B
	 *
	 *
	 *          A --\             
	 * Case 3        >====== gport
	 *          B --/
	 *
	 *
	 * Case 4   A,B ======== gport
	 *
	 *
	 * We note that Case 2 and Case 4 have been filtered by "duplicate connection check".
	 * Therefore, the following code just check Case 1 and Case 3.
	 */
	// If the user specified the gport, check if the gport is in use ?
#ifdef CONFIG_RTL865X_ROMEPERF
	rtl8651_romeperfEnterPoint(ROMEPERF_INDEX_NAPT_ADD_6);
#endif
	if( assigned & EXTIPPORT_ASSIGNED )
	{
		/* If this is fwdEngine's request, we trust it had traversed.
		 * For performance issue, we no longer traverse again. 
		 */
		if ( ( assigned & DO_REDUNDANCY_CHECK ) &&
		     _rtl8651_findNaptConnectionByExtIpPort( isTcp, *insideGlobalIpAddr, *insideGlobalPort,
	                                                        dstIpAddr, dstPort ) != NULL )
		{
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("[%s:%d] addNaptConnection() returns TBLDRV_EINVALIDEXTPORT.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
			return TBLDRV_EINVALIDEXTPORT;
		}

		/* Check if extPort is shared. */
		if ( _rtl8651_ExtPortIsShared( isTcp, *insideGlobalIpAddr, *insideGlobalPort, *insideGlobalPort ) )
		{
			bPortReused = TRUE;
		}
	}
#ifdef CONFIG_RTL865X_ROMEPERF
	rtl8651_romeperfExitPoint(ROMEPERF_INDEX_NAPT_ADD_6);
#endif
	
	/*************************************************************************
	 * All right, all the check has done in above.
	 *************************************************************************/
	// decide using which insideGlobalIpAddr
	if ( (assigned & EXTIPPORT_ASSIGNED) &&
	     *insideGlobalIpAddr != 0 )
	{
		// The insideGlobalIpAddr has been assigned. Check if it is equal to Policy Route.
		// FIXME: Ask policy route to get gip, and compare gip and insideGlobalIpAddr.
		//        If the policy route is disbled, compare with def_napt_t.
	}
	else
	{
		ipaddr_t defExtIp;

		defExtIp = _rtl8651_getNaptInsideGlobalIpAddr( isTcp, insideLocalIpAddr, insideLocalPort,
		                                                     dstIpAddr, dstPort );
		
		if ( dstIpAddr != 0 )
		{
			// The dstIpAddr is known, therefore we can estimate the insideGlobalIpAddr.
			// FIXME: Should ask policy route
			if ( defExtIp )
			{
				*insideGlobalIpAddr = defExtIp;
			}
			else
			{
				// No default route, error.
#ifdef ADD_NAPT_DEBUG
				rtlglue_printf("[%s:%d] addNaptConnection() returns TBLDRV_EDEFAULTROUTENOTFOUND.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
				return TBLDRV_EDEFAULTROUTENOTFOUND;
			}
		}
		else
		{
			// The dstIpAddr is unknown, for UPnP.
			// We assume the insideGlobalIpAddr is the NAPT IP address.
			if ( defExtIp )
			{
				*insideGlobalIpAddr = defExtIp;
			}
			else
			{
				// No default route, error.
#ifdef ADD_NAPT_DEBUG
				rtlglue_printf("[%s:%d] addNaptConnection() returns TBLDRV_EDEFAULTROUTENOTFOUND.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
				return TBLDRV_EDEFAULTROUTENOTFOUND;
			}
		}
		
#ifdef ADD_NAPT_DEBUG
		rtlglue_printf("[%s:%d] ExtIp is selected: %d.%d.%d.%d\n", __FILE__,__LINE__,NIPQUAD(*insideGlobalIpAddr));
#endif/*ADD_NAPT_DEBUG*/
	}

	/* Patch for interface route (nexthop's lack)
	 * Holly chhuang will recover them!
	 */
#ifdef CONFIG_RTL865X_ROMEPERF
		rtl8651_romeperfEnterPoint(ROMEPERF_INDEX_NAPT_ADD_7);
#endif
	if ( dstIpAddr != 0 )
	{
		rtl8651_tblDrv_routeTable_t *rt;
		
		rt = _rtl8651_getRoutingEntry( dstIpAddr );
		if ( rt == NULL )
		{
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("[%s:%d] addNaptConnection() returns TBLDRV_EDEFAULTROUTENOTFOUND.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
			return TBLDRV_EDEFAULTROUTENOTFOUND;
		}

		interfaceRoute = rt->ifrt_flag;

		/* dstIPAddr is default gateway, it is NOT interface route ! */
		if ( interfaceRoute )
		{
			if ( DrvTbl.route[rtl8651_tblDrvPara.routingTableSize-1].valid )
			{
				/* yes, we have got default route. */
				if ( dstIpAddr == DrvTbl.route[rtl8651_tblDrvPara.routingTableSize-1].nextHop )
				{
					interfaceRoute = FALSE;
				}
			}
			else
			{
				/* NO default route, it is interfaceRoute. */
			}
		}
		
		/* rtlglue_printf("_rtl8651_getRoutingEntry(%08x), interfaceRoute=%d\n",dstIpAddr, interfaceRoute); */
	}
#ifdef CONFIG_RTL865X_ROMEPERF
		rtl8651_romeperfExitPoint(ROMEPERF_INDEX_NAPT_ADD_7);
#endif

	// Check connection control system
	if ( assigned & DOS_NAPT_ENCOUNT )
	{
		assert( ((assigned & DOS_NAPT_FROMINTERNAL) && insideLocalIpAddr) ||
			    ((!(assigned & DOS_NAPT_FROMINTERNAL)) && dstIpAddr) );

		if (	_rtl8651_dosNaptConnCheck(
			isTcp,
			(assigned & DOS_NAPT_FROMINTERNAL)?insideLocalIpAddr:dstIpAddr,
			(assigned & DOS_NAPT_FROMINTERNAL)?RTL8651_LAN2WAN:RTL8651_WAN2LAN) != SUCCESS)
		{
#ifdef ADD_NAPT_DEBUG
		rtlglue_printf( "[%s:%d] the connection count exceed.\n", __FUNCTION__,__LINE__ );
#endif
			return TBLDRV_ENAPTCONNECTIONFULL;
		}
	}

#ifdef CONFIG_RTL865X_ROMEPERF
		rtl8651_romeperfEnterPoint(ROMEPERF_INDEX_NAPT_ADD_8);
#endif
	//Check we still have free connection slots.
	if ( ( pe = CTAILQ_FIRST(&DrvTbl.freeList.tcpudp) ) == NULL )
	{
		#ifdef DROP_NEW_CONNECTION_WHEN_NAT_FULL
		if(rtl8651_dropNewConnWhenNatFull==TRUE)
		{
			return TBLDRV_ENOFREEBUFFER;//No free flow entry available
		}
		#endif
		
#ifdef FLUSH_NAPT_CONNECTION_WHEN_MORE_THAN_LIMIT

#ifdef FLUSH_NAPT_OLDEST_CONNECTION2
		if ( _rtl8651_flushNaptOldestConnection2( 1 ) >= 1 )
		{
			/* Yes, we success */
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("flush Oldest Connection2(1) SUCCESS! addSuceeded=%d.\n",rtl8651_tblDrvNaptCounter.addSucceeded);
#endif/*ADD_NAPT_DEBUG*/
			pe = CTAILQ_FIRST(&DrvTbl.freeList.tcpudp);
			assert ( pe );
		}
#else/*FLUSH_NAPT_OLDEST_CONNECTION2*/
		/* Try to recycle FINed and older connections. */
		if ( _rtl8651_flushNaptFINedConnection( 1 ) >= 1 )
		{
			/* Yes, we success */
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("flush FINed Connection(1) SUCCESS! addSuceeded=%d.\n",rtl8651_tblDrvNaptCounter.addSucceeded);
#endif/*ADD_NAPT_DEBUG*/
			pe = CTAILQ_FIRST(&DrvTbl.freeList.tcpudp);
			assert ( pe );
		}
#endif/*FLUSH_NAPT_OLDEST_CONNECTION2*/
		else if ( _rtl8651_flushNaptHeadConnection( 1 ) >= 1 )
		{
			/* Yes, we success */
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("flush Connection(1) at tail of inuse list SUCCESS! addSuceeded=%d.\n",rtl8651_tblDrvNaptCounter.addSucceeded);
#endif/*ADD_NAPT_DEBUG*/
			pe = CTAILQ_FIRST(&DrvTbl.freeList.tcpudp);
			assert ( pe );
		}
#if 0
		else if ( _rtl8651_flushNaptOldestConnection( 1 ) >= 1 )
		{
			/* Yes, we success */
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("flush Oldest Connection(1) SUCCESS! addSuceeded=%d.\n",rtl8651_tblDrvNaptCounter.addSucceeded);
#endif/*ADD_NAPT_DEBUG*/
			pe = CTAILQ_FIRST(&DrvTbl.freeList.tcpudp);
			assert ( pe );
		}
#endif/*0*/
		else
		{
			/* Ouch! we failed .... */
#ifdef ADD_NAPT_DEBUG
			rtlglue_printf("[%s:%d] addNaptConnection() returns TBLDRV_ENOFREEBUFFER.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
			return TBLDRV_ENOFREEBUFFER;//No free flow entry available
		}

#else/*FLUSH_NAPT_CONNECTION_WHEN_MORE_THAN_LIMIT*/

#ifdef ADD_NAPT_DEBUG
		rtlglue_printf("[%s:%d] addNaptConnection() returns TBLDRV_ENOFREEBUFFER.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
		return TBLDRV_ENOFREEBUFFER;//No free flow entry available

#endif/*FLUSH_NAPT_CONNECTION_WHEN_MORE_THAN_LIMIT*/
	}
	CTAILQ_REMOVE( &DrvTbl.freeList.tcpudp, pe, next );
	CTAILQ_INSERT_TAIL( &DrvTbl.inuseList.tcpudp, pe, next );


#if EXTPORT_PPPoE_UDP_PATCH==1 /* cfliu's patch */
	if(!isTcp){
		rtl8651_tblDrv_networkIntfTable_t *iface;
		rtl8651_tblDrv_arpEntry_t *arp;
		rtl8651_tblDrv_filterDbTableEntry_t *l2;
		//Route lookup...
		rtl8651_tblDrv_routeTable_t *rt = _rtl8651_getRoutingEntry( dstIpAddr );
		if(!rt||!rt->netif_t)
			goto no_extPortPatch;
		//Route found...
		iface=rt->netif_t;
		if(iface->linkLayerType!=RTL8651_LL_PPPOE&&iface->linkLayerType!=RTL8651_LL_WIPUNNUMBERED&&
		   iface->linkLayerType!=RTL8651_LL_LIPUNNUMBERED)
			goto no_extPortPatch;	
		//Routes to a PPPoE based WAN
		arp= _rtl8651_getArpEntry(NULL, insideLocalIpAddr, FALSE);
		if(!arp ||!arp->macInfo)
			goto no_extPortPatch;
		//MAC found by source IP
		l2=arp->macInfo;
		if(l2->linkId==0)
			goto no_extPortPatch;
		//It's from a WLAN station

		//ASIC has a bug for masquerading UDP flow from a WLAN station to a PPPoE based WAN,
		//don't write it to ASIC... keep it in software.
		assigned|=PURE_SW;
	}
no_extPortPatch:
#endif
#ifdef CONFIG_RTL865X_ROMEPERF
		rtl8651_romeperfExitPoint(ROMEPERF_INDEX_NAPT_ADD_8);
#endif


#ifdef CONFIG_RTL865X_ROMEPERF
		rtl8651_romeperfEnterPoint(ROMEPERF_INDEX_NAPT_ADD_9);
#endif
	// common entry content.
	/* Louis: this can save about 1000 CPU cycles.
	 * memset( pe, 0, offsetof(struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s, nextOut ) );
	 */
	pe->insideLocalIpAddr = insideLocalIpAddr;
	pe->insideLocalPort = insideLocalPort;
	pe->insideGlobalIpAddr = *insideGlobalIpAddr;
	// pe->insideGlobalPort will be set later
	pe->dstIpAddr = dstIpAddr;
	pe->dstPort = dstPort;
	pe->alive = TRUE;
	pe->tcp = isTcp;
	pe->hashing = 0;
	pe->tcpFlag = flowType ?flowType-1 :0; // UDP is set to 0
	pe->quiet = 0;
	pe->fromDrv = fromDrv;
	pe->col1 = TRUE;
	pe->col2 = TRUE;
	pe->isStatic = TRUE;
	pe->asicSnapshot = FALSE;
	/* In the following condition, we must set in pure software mode.
	 *   1.fwdEngine specifies it is pure softrware connection (with PURE_SW), such as ALG.
	 *   2.extIp is equal to softNaptIp, which is the 2nd PPPoE IP.
	 *   3.865xB B-cut, and the dstIP address is not specified. It is buggy when interface route.
	 */
	pe->pureSoftware = (assigned & PURE_SW)?TRUE:FALSE ||
	                   (softNaptIp!=0&&softNaptIp==pe->insideGlobalIpAddr) ||
	                   (RtkHomeGatewayChipRevisionID==1&&dstIpAddr==0);
	pe->interfaceRoute = interfaceRoute;
	pe->newlyAddedHash2Udp = FALSE;
	pe->holdForCdrouter = (assigned&DONT_BIRD_CDROUTER)
	                      ?FALSE/* DO NOT bird CDRouter, write to ASIC if possible. */
	                      :TRUE; /* We enable this option for CDRouter by default. */
#ifndef RTL865XB_URL_FILTER
	pe->urlFiltered = FALSE;
#endif
	pe->connectionType = NAPT_TYPE_NORMAL;
	pe->toProtocolStack = (assigned & TO_PROTOCOL_STACK)?TRUE:FALSE;
	pe->dosEnCount = 0;
	pe->dosFromInternal = 0;
	pe->out_rt_t = NULL; 
	pe->in_rt_t = (void *)_rtl8651_getRoutingEntry(insideLocalIpAddr);
	pe->out_macInfo_t = NULL;
	pe->in_macInfo_t = NULL;
	pe->dsid = 0;
	pe->inExtIpIdx = -1; /* Not allocated yet. */
	pe->outExtIpIdx = -1; /* Not allocated yet. */
	assert(pe->in_rt_t);
	
	// update timeout value
	if( flowType == TCP_ESTABLISHED_FLOW )
		pe->age = rtl8651GlobalControl.tcpLongTimeout;
	else
		pe->age = (flowType) ?rtl8651GlobalControl.tcpFastTimeout :rtl8651GlobalControl.udpTimeout;
#ifdef CONFIG_RTL865X_ROMEPERF
		rtl8651_romeperfExitPoint(ROMEPERF_INDEX_NAPT_ADD_9);
#endif

#ifdef CONFIG_RTL865X_ROMEPERF
		rtl8651_romeperfEnterPoint(ROMEPERF_INDEX_NAPT_ADD_10);
#endif
	/* according to the input argument to act differently. */
	if ( dstIpAddr != 0 && dstPort != 0 )
	{
		// The user gave 7-tuple (with dip/dport), and our strategy is Hash1 --> Hash2.

		if ( (assigned & EXTIPPORT_ASSIGNED) /* Louis note: ExtPort is allowed now! For CDRouter. &&
		     *insideGlobalPort != 0 */ )
		{
			// The user specified gport, try Hash1 first, then Hash2.
			inIdx = *insideGlobalPort & (RTL8651_TCPUDPTBL_SIZE-1);
			outIdx = idx1;

			if ( inIdx == outIdx )
			{
				// It is possible that user-spicified gport is equal to hash1 index.
				// Therefore, we can try to use Hardware-Accelerator.

				if ( pe->pureSoftware ) /* pure Software, DO NOT write to ASIC. */
				{
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] pe->pureSoftware==TRUE, goto L01.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
					goto L01;
				}
				
				/* For CDRouter */
				if ( pe->holdForCdrouter )
				{
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] for CDRouter TCP-long-timeout test, goto L01.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
					goto L01;
				}
				
				/* For CDRouter */
				if ( isTcp==FALSE )
				{
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] UDP, for CDRouter, goto L01.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
					pe->newlyAddedHash2Udp = TRUE;
					goto L01;
				}

				if ( _rtl8651_NaptEntryIsEmpty( idx1 ) )
				{
					if ( bPortReused == TRUE )
					{
#ifdef ADD_NAPT_DEBUG
						rtlglue_printf("[%s:%d] bPortReused==TRUE, goto L01.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
						goto L01;
					}
					inIdx = *insideGlobalPort & (RTL8651_TCPUDPTBL_SIZE-1);
					outIdx = idx1;
					pe->hashing = NAPT_HASH1;
					pe->insideGlobalPort = *insideGlobalPort;
					if ( isTcp && flowType < TCP_ESTABLISHED_FLOW ) /* trap to CPU to modify MSS */
						goto L01;
					if ( _rtl8651_WriteToAsicNaptEntry( idx1, pe ) == FAILED ) goto L01;
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] addNaptConnection() F01.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
				}
				else
				{
					L01:
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] addNaptConnection() L01.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
					// Cannot to use Hardware-Accelerated. Append to Candidate List.
					
					inIdx = *insideGlobalPort & (RTL8651_TCPUDPTBL_SIZE-1);
					outIdx = idx1;
					pe->hashing = NAPT_HASH1;
					pe->insideGlobalPort = *insideGlobalPort;
					_rtl8651_AppendToCandidateList( idx1, pe );
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] addNaptConnection() F02.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
				}
			}
			else
			{
				// The in/out entries are different index. Use Hash2 only.
				outIdx = idx2;
				goto L07;
			}
		}
		else
		{
			/* The user did not specify gport, try Hash1 first, then Hash2. */

			if (enable4WayHash == TRUE)
			{
				/* 4-way hash enabled */
				int i, idx_loop;
				int found = FALSE;

				/* try all possibility to find empty ASIC entry. */
				idx_loop = idx1;
				for( i = 0; i < 4; i++ )
				{
#if 1
					/* Louis note:
					 *   Only check ASIC entry is not enough, because candidate will race with it.
					 *   Therefore, a better way is to scan these 4 lists.
					 */
					rtl8651DrvNaptLine_t *line;
	
					line = &DrvTbl.tcpudp[idx_loop];

					if ( SLIST_FIRST(&(line->list[NAPT_ASIC_CACHED_IN]))==NULL &&
					     SLIST_FIRST(&(line->list[NAPT_ASIC_CACHED_OUT]))==NULL &&
					     SLIST_FIRST(&(line->list[NAPT_CANDIDATE_IN]))==NULL &&
					     SLIST_FIRST(&(line->list[NAPT_CANDIDATE_OUT]))==NULL )
					{
						found = TRUE;
						break;
					}
#else
					// Find if any empty entry exists ? 
					if ( _rtl8651_NaptEntryIsEmpty( idx_loop ) )
					{
						found = TRUE;
						break;
					}
#endif/*1*/
					// next idx
					idx_loop = WRAPIN_4WAY( idx_loop, 1 );
				}

#ifdef ADD_NAPT_DEBUG
				rtlglue_printf("[%s:%d] idx1=%d, 4-way hash %s(%d)\n",__FILE__,__LINE__,idx1, found?"found":"not found", idx_loop );
#endif/*ADD_NAPT_DEBUG*/

				idx1 = idx_loop; /* idx1 had been changed for extPort. */

				if ( pe->pureSoftware ) /* pure Software, DO NOT write to ASIC. */
				{
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] pe->pureSoftware==TRUE, goto L04.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
					goto L04;
				}
				
				/* For CDRouter */
				if ( pe->holdForCdrouter )
				{
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] for CDRouter TCP-long-timeout test, goto L04.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
					goto L04;
				}
				
				/* For CDRouter */
				if ( isTcp==FALSE )
				{
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] UDP, for CDRouter, goto L04.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
					pe->newlyAddedHash2Udp = TRUE;
					goto L04;
				}

				if ( found == FALSE )
				{
					// no empty ASIC entry found
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] Typical case NO empty ASIC entry found, goto L02.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
					goto L02;
				}
				else
				{
					// yes, we've got empty ASIC entry !
					if ( TBLDRV_EGETOFFSETFAIL == _rtl8651_getUsableExtPortWithIndex( idx_loop, isTcp, *insideGlobalIpAddr, insideGlobalPort ) ) goto err_offset;
					inIdx = idx_loop;
					outIdx = idx_loop;
					pe->hashing = NAPT_HASH1;
					pe->insideGlobalPort = *insideGlobalPort;
					if ( isTcp && flowType < TCP_ESTABLISHED_FLOW ) /* trap to CPU to modify MSS */
					{
#ifdef ADD_NAPT_DEBUG
						rtlglue_printf("[%s:%d] trap TO CPU to modify MSS, goto L04.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
						goto L04;
					}
					if ( _rtl8651_WriteToAsicNaptEntry( idx_loop, pe ) == FAILED ) goto L02;
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] addNaptConnection() F03.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
				}
			}
			else
			{
				// 1-way hash
				// Typical Case. Try Hash1.
				if ( pe->pureSoftware ) /* pure Software, DO NOT write to ASIC. */
				{
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] pe->pureSoftware==TRUE, goto L04.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
					goto L04;
				}
				
				/* For CDRouter */
				if ( pe->holdForCdrouter )
				{
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] for CDRouter TCP-long-timeout test, goto L04.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
					goto L04;
				}
				
				/* For CDRouter */
				if ( isTcp==FALSE )
				{
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] UDP, for CDRouter, goto L04.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
					pe->newlyAddedHash2Udp = TRUE;
					goto L04;
				}
				
				if ( _rtl8651_NaptEntryIsEmpty( idx1 ) )
				{
					if ( TBLDRV_EGETOFFSETFAIL == _rtl8651_getUsableExtPortWithIndex( idx1, isTcp, *insideGlobalIpAddr, insideGlobalPort ) ) goto err_offset;
					inIdx = idx1;
					outIdx = idx1;
					pe->hashing = NAPT_HASH1;
					pe->insideGlobalPort = *insideGlobalPort;
					if ( isTcp && flowType < TCP_ESTABLISHED_FLOW ) /* trap to CPU to modify MSS */
					{
#ifdef ADD_NAPT_DEBUG
						rtlglue_printf("[%s:%d] trap to CPU to modify MSS, goto L04.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
						goto L04;
					}
					if ( _rtl8651_WriteToAsicNaptEntry( idx1, pe ) == FAILED ) goto L02;
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] addNaptConnection() F04.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
				}
				else
				{
					L02:
					// Try Hash2.
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] addNaptConnection() L02.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
					if ( _rtl8651_NaptEntryIsEmpty( idx2 ) )
					{
						goto L03;
					}
					else
					{
						// The ASIC entries of idx1 and idx2 are in use.
						// Try to separate in/out entry (Semi-Hardware-Accelerated).
						
						foundIdx = idx2; // initial index
						if ( _rtl8651_FindEmptyASICEntry( &foundIdx ) == SUCCESS )
						{
							// Semi-Hardware-Accelerated

							// In this case, the port will NEVER be re-used !
							if ( bPortReused == TRUE ) assert( 0 );
						
							inIdx = foundIdx;
							outIdx = idx2;
							if ( TBLDRV_EGETOFFSETFAIL == _rtl8651_getUsableExtPortWithIndex( foundIdx, isTcp, *insideGlobalIpAddr, insideGlobalPort ) ) goto err_offset;
							pe->insideGlobalPort = *insideGlobalPort;
							if ( flowType ) pe->tcpFlag |= 2; // uni-dir
							pe->hashing = NAPT_HASH2;
							if ( _rtl8651_HalfHardwareAccelerated( inIdx, outIdx, pe ) == FAILED ) goto L04;
#ifdef ADD_NAPT_DEBUG
							rtlglue_printf("[%s:%d] addNaptConnection() F05.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
						}
						else
						{
							L04:
							// Finally, we had tried Hash1-HA, Hash2-HA, Semi-HA.
							// We have no way to go, append to candidate list.
#ifdef ADD_NAPT_DEBUG
							rtlglue_printf("[%s:%d] addNaptConnection() L04.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/

							inIdx = idx1;
							outIdx = idx1;
							if ( TBLDRV_EGETOFFSETFAIL == _rtl8651_getUsableExtPortWithIndex( idx1, isTcp, *insideGlobalIpAddr, insideGlobalPort ) ) goto err_offset;
							pe->insideGlobalPort = *insideGlobalPort;

							inIdx = *insideGlobalPort & (RTL8651_TCPUDPTBL_SIZE-1);
							if ( flowType ) pe->tcpFlag &= ~2; // uni-dir
							pe->hashing = NAPT_HASH1;
							_rtl8651_AppendToCandidateList( idx1, pe );
#ifdef ADD_NAPT_DEBUG
							rtlglue_printf("[%s:%d] addNaptConnection() F06.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
						}
					}
				}
			}
		}
	}
	else
	{
		// The user gave 5-tuple (without dip/dport), and the only way is Hash2.
		
		if ( (assigned & EXTIPPORT_ASSIGNED) /* Louis note: ExtPort is allowed now! For CDRouter. &&
		     *insideGlobalPort != 0 */ )
		{
			// The user specified gport, this is Server Port.
			inIdx = *insideGlobalPort & (RTL8651_TCPUDPTBL_SIZE-1);
			outIdx = idx2;
#ifdef ADD_FINITE_HASH2_SP
			/* In this mode, 5-tuple connection will be aged out BY DEFAULT. */
#else/*ADD_FINITE_HASH2_SP*/
			pe->connectionType = NAPT_TYPE_SERVERPORT;
#endif/*ADD_FINITE_HASH2_S*/

			if ( inIdx == outIdx )
			{
				// Inbound and outbound of HASH2 are the same.
				// Therefore, we can try to use Hardware-Accelerator.

				if ( pe->pureSoftware ) /* pure Software, DO NOT write to ASIC. */
				{
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] pe->pureSoftware==TRUE, goto L05.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
					goto L05;
				}
				
				if ( _rtl8651_NaptEntryIsEmpty( idx2 ) )
				{
					// Using Hash2 Hareware-Accelerator
					if ( bPortReused == TRUE )
					{
#ifdef ADD_NAPT_DEBUG
						rtlglue_printf("[%s:%d] bPortReused==TRUE, goto L05.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
						goto L05;
					}
					
					/* For CDRouter */
					if ( pe->holdForCdrouter )
					{
#ifdef ADD_NAPT_DEBUG
						rtlglue_printf("[%s:%d] for CDRouter TCP-long-timeout test, goto L05.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
						goto L05;
					}
				
					/* For CDRouter */
					if ( isTcp==FALSE )
					{
#ifdef ADD_NAPT_DEBUG
						rtlglue_printf("[%s:%d] UDP, for CDRouter, goto L05.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
						pe->newlyAddedHash2Udp = TRUE;
						goto L05;
					}

					inIdx = *insideGlobalPort & (RTL8651_TCPUDPTBL_SIZE-1);
					outIdx = idx2;

					pe->hashing = NAPT_HASH2;
					if ( flowType ) pe->tcpFlag &= ~2; // bi-dir
					pe->insideGlobalPort = *insideGlobalPort;
					if ( _rtl8651_WriteToAsicNaptEntry( idx2,pe ) == FAILED )
					{
#ifdef ADD_NAPT_DEBUG
						rtlglue_printf("[%s:%d] WriteToAsicNaptEntry()==FAILED, and goto L05.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
						goto L05;
					}
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] addNaptConnection() F07.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
				}
				else
				{
					L05:
					// Cannot to use Hardware-Accelerated. Append to Candidate List.
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] addNaptConnection() L05.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
					
					inIdx = *insideGlobalPort & (RTL8651_TCPUDPTBL_SIZE-1);
					outIdx = idx2;

					pe->hashing = NAPT_HASH2;
					if ( flowType ) pe->tcpFlag &= ~2; // bi-dir
					pe->insideGlobalPort = *insideGlobalPort;
					_rtl8651_AppendToCandidateList( idx2, pe );
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] addNaptConnection() F08.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
				}
			}
			else
			{
				inIdx = *insideGlobalPort & (RTL8651_TCPUDPTBL_SIZE-1);
				outIdx = idx2;

				L07:
#ifdef ADD_NAPT_DEBUG
				rtlglue_printf("[%s:%d] addNaptConnection() L07.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
				pe->hashing = NAPT_HASH2;
				if ( flowType ) pe->tcpFlag |= 2; // uni-dir
				pe->insideGlobalPort = *insideGlobalPort;
				if ( _rtl8651_Hash2TryHardwareAccelerated( inIdx, outIdx, pe, bPortReused ) != SUCCESS )
				{
					goto err_L07; /* the most posibility is addHash2_nexthop() error. */
				}
			}
		}
		else
		{
			if ( pe->pureSoftware ) /* pure Software, DO NOT write to ASIC. */
			{
#ifdef ADD_NAPT_DEBUG
				rtlglue_printf("[%s:%d] pe->pureSoftware==TRUE, goto L06.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
				goto L06;
			}
			
			if ( _rtl8651_NaptEntryIsEmpty( idx2 ) )
			{
				L03:
				// Using Hash2 Hareware-Accelerator
#ifdef ADD_NAPT_DEBUG
				rtlglue_printf("[%s:%d] addNaptConnection() L03.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/

				// In this case, the port will NEVER be re-used !
				if ( bPortReused == TRUE ) assert( 0 );

				if ( TBLDRV_EGETOFFSETFAIL == _rtl8651_getUsableExtPortWithIndex( idx2, isTcp, *insideGlobalIpAddr, insideGlobalPort ) ) goto err_offset;
				inIdx = idx2;
				outIdx = idx2;
				pe->hashing = NAPT_HASH2;
				pe->tcpFlag &= ~2; // bi-dir
				pe->insideGlobalPort = *insideGlobalPort;

				/* For CDRouter */
				if ( pe->holdForCdrouter )
				{
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] for CDRouter TCP-long-timeout test, goto L06.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
					goto L06;
				}
				
				/* For CDRouter */
				if ( isTcp==FALSE )
				{
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] UDP, for CDRouter, goto L06.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
					pe->newlyAddedHash2Udp = TRUE;
					goto L06;
				}

				if ( _rtl8651_WriteToAsicNaptEntry( idx2, pe ) == FAILED )
				{
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] WriteToAsicNaptEntry()==FAILED, goto L06.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
					goto L06;
				}
#ifdef ADD_NAPT_DEBUG
				rtlglue_printf("[%s:%d] addNaptConnection() F15.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
			}
			else
			{
				// idx2 ASIC entry is in use.
				// Try to separate in/out entry (Semi-Hardware-Accelerated).
				
				foundIdx = idx2; // initial index
				if ( _rtl8651_FindEmptyASICEntry( &foundIdx ) == SUCCESS )
				{
					// Semi-Hardware-Accelerated

					// In this case, the port will NEVER be re-used !
					if ( bPortReused == TRUE ) assert( 0 );
					
					inIdx = foundIdx;
					outIdx = idx2;
					if ( TBLDRV_EGETOFFSETFAIL == _rtl8651_getUsableExtPortWithIndex( foundIdx, isTcp, *insideGlobalIpAddr, insideGlobalPort ) ) goto err_offset;
					pe->insideGlobalPort = *insideGlobalPort;
					if ( flowType ) pe->tcpFlag |= 2; // uni-dir
					pe->hashing = NAPT_HASH2;
					if ( _rtl8651_HalfHardwareAccelerated( inIdx, outIdx, pe ) == FAILED ) goto L06;
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] addNaptConnection() F16.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
				}
				else
				{
					L06:
					// Finally, we had tried Hash2-HA, Semi-HA.
					// We have no way to go, append to candidate list.
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] addNaptConnection() L06.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/

					inIdx = idx2;
					outIdx = idx2;
					if ( TBLDRV_EGETOFFSETFAIL == _rtl8651_getUsableExtPortWithIndex( idx2, isTcp, *insideGlobalIpAddr, insideGlobalPort ) ) goto err_offset;
					pe->hashing = NAPT_HASH2;
					if ( flowType ) pe->tcpFlag &= ~2; // bi-dir
					pe->insideGlobalPort = *insideGlobalPort;
					_rtl8651_AppendToCandidateList( idx2, pe );
#ifdef ADD_NAPT_DEBUG
					rtlglue_printf("[%s:%d] addNaptConnection() F17.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
				}
			}
		}
	}
#ifdef CONFIG_RTL865X_ROMEPERF
	rtl8651_romeperfExitPoint(ROMEPERF_INDEX_NAPT_ADD_10);
#endif

#ifdef CONFIG_RTL865X_ROMEPERF
	rtl8651_romeperfEnterPoint(ROMEPERF_INDEX_NAPT_ADD_11);
#endif
	/* After setting Hash2 connection, tcpFlag had been overwrited.
	 * So, we must recover it. Therefore, the fwd engine has the state of this connection. 
	 */
	pe->tcpFlag = flowType ?flowType-1 :0; // UDP is set to 0

	/*
	 * Ok, the newly-added connection has been added into ASIC or appended to CANDIDATE list.
	 * However, if the gport (ExtPort) is re-used, we must prevent Hash2 entry holding ASIC entry.
	 *
	 * Louis note that consider the following condition:
	 *   Connection1's inbound/outbound is index K.
	 *   Connection2's inbound is index K+1, but outbound is index K.
	 * In this condition (typically DMZ FTP active mode), bPortReused is FALSE.
	 * But both connection cannot write to ASIC !!!
	 * Therefore, we always check ambiguous rather than bPortReused == TRUE.
	 */
	if ( 1 /* bPortReused */ )
	{
#if 1
#else
		rtl8651_tblDrv_naptTcpUdpFlowEntry_t *thisConn;
#endif
		int bInboundAmbiguous, bOutboundAmbiguous;
		int bInboundAsicEmpty, bOutboundAsicEmpty;

		/* If ambiguous, this function will swap the Hash2 ASIC entry to candidate list, and
		 *    return whether the newly-added connection is ambiguous with others.
		 * bInboundAmbiguous will be set TRUE if the newly-added connection will be ambiguous with
		 *    the other connections in the inbound entry line.
		 * bOutboundAmbiguous will be set TRUE if the newly-added connection will be ambiguous with
		 *    the other connections in the outbound entry line.
		 * bInboundAsicEmpty will be set TRUE if the inbound ASIC entry is free.
		 * bOutboundAsicEmpty will be set TRUE if the outbound ASIC entry is free.
		 */
		_rtl8651_AvoidAmbiguousHash2InAsic( pe, &bInboundAmbiguous, &bOutboundAmbiguous, &bInboundAsicEmpty, &bOutboundAsicEmpty );

#ifdef ADD_NAPT_DEBUG
		rtlglue_printf("[%s():%d] AvoidAmbiguous(pe->ExtPort:%d,inA:%d,outA:%d,inE:%d,outE:%d)\n",
		               __FUNCTION__,__LINE__,
		               pe->insideGlobalPort,bInboundAmbiguous,bOutboundAmbiguous,bInboundAsicEmpty, bOutboundAsicEmpty);
#endif

#if 1
		/*
		 * Search candidate connections will drop down performance when many connections collises in one entry.
		 * Therefore, I prefer do not search candidate connections and left to be determined by pps rate.
		 */
#else
		/* After _rtl8651_AvoidAmbiguousHash2InAsic(), ASIC entry may be empty.
		 * Search candidate list to fill ASIC if the ASIC entry is free.
		 * The order is :
		 *   [inIdx].Candidate_in
		 *   [inIdx].Candidate_out
		 *   [outIdx].Candidate_in
		 *   [outIdx].Candidate_out
		 * We note that: if the ASIC is not empty, the following change is ignored.
		 */
		if ( bInboundAsicEmpty == TRUE )
		{
			SLIST_FOREACH( thisConn, &DrvTbl.tcpudp[inIdx].list[NAPT_CANDIDATE_IN], nextIn )
			{
				_rtl8651_AvoidAmbiguousHash2InAsic( thisConn, &bInboundAmbiguous, &bOutboundAmbiguous, &bInboundAsicEmpty, &bOutboundAsicEmpty );
				if ( thisConn->hashing == NAPT_HASH1 ) 
				{
					if ( thisConn->tcpFlag==TCP_ESTABLISHED_FLOW-1 )
					{
						_rtl8651_changeNaptFlowListType( thisConn, NAPT_ASIC_CACHED_IN, NAPT_ASIC_CACHED_OUT );
						bInboundAsicEmpty = FALSE;
						break;
					}
				}
				else if ( thisConn->hashing == NAPT_HASH2 && bInboundAmbiguous == FALSE &&
				          thisConn->newlyAddedHash2Udp==FALSE/*For CDRouter, newly-added UDP HASH2 connection cannot be ASIC-accelerated.*/ &&
				          thisConn->holdForCdrouter==FALSE /* For CDRouter, newly-added connection cannot be ASIC-accelerated. */ )
				{
					_rtl8651_changeNaptFlowListType( thisConn, NAPT_ASIC_CACHED_IN, thisConn->outListType );
					bInboundAsicEmpty = FALSE;
					break;
				}
			}
		}
		if ( bInboundAsicEmpty == TRUE )
		{
			SLIST_FOREACH( thisConn, &DrvTbl.tcpudp[inIdx].list[NAPT_CANDIDATE_OUT], nextOut )
			{
				_rtl8651_AvoidAmbiguousHash2InAsic( thisConn, &bInboundAmbiguous, &bOutboundAmbiguous, &bInboundAsicEmpty, &bOutboundAsicEmpty );
				if ( thisConn->hashing == NAPT_HASH1 )
				{
					if ( thisConn->tcpFlag==TCP_ESTABLISHED_FLOW-1 )
					{
						_rtl8651_changeNaptFlowListType( thisConn, NAPT_ASIC_CACHED_IN, NAPT_ASIC_CACHED_OUT );
						bInboundAsicEmpty = FALSE;
						break;
					}
				}
				else if ( thisConn->hashing == NAPT_HASH2 && bOutboundAmbiguous == FALSE &&
				          thisConn->newlyAddedHash2Udp==FALSE/*For CDRouter, newly-added connection cannot be ASIC-accelerated.*/ &&
				          thisConn->holdForCdrouter==FALSE /* For CDRouter, newly-added connection cannot be ASIC-accelerated. */ )
				{
					_rtl8651_changeNaptFlowListType( thisConn, thisConn->inListType, NAPT_ASIC_CACHED_OUT );
					bInboundAsicEmpty = FALSE;
					break;
				}
			}
		}
		if ( bOutboundAsicEmpty == TRUE )
		{
			SLIST_FOREACH( thisConn, &DrvTbl.tcpudp[outIdx].list[NAPT_CANDIDATE_IN], nextIn )
			{
				_rtl8651_AvoidAmbiguousHash2InAsic( thisConn, &bInboundAmbiguous, &bOutboundAmbiguous, &bInboundAsicEmpty, &bOutboundAsicEmpty );
				if ( thisConn->hashing == NAPT_HASH1 )
				{
					if ( thisConn->tcpFlag==TCP_ESTABLISHED_FLOW-1 )
					{
						_rtl8651_changeNaptFlowListType( thisConn, NAPT_ASIC_CACHED_IN, NAPT_ASIC_CACHED_OUT );
						bOutboundAsicEmpty = FALSE;
						break;
					}
				}
				else if ( thisConn->hashing == NAPT_HASH2 && bInboundAmbiguous == FALSE &&
				          thisConn->newlyAddedHash2Udp==FALSE/*For CDRouter, newly-added connection cannot be ASIC-accelerated.*/ &&
				          thisConn->holdForCdrouter==FALSE /* For CDRouter, newly-added connection cannot be ASIC-accelerated. */ )
				{
					_rtl8651_changeNaptFlowListType( thisConn, NAPT_ASIC_CACHED_IN, thisConn->outListType );
					bOutboundAsicEmpty = FALSE;
					break;
				}
			}
		}
		if ( bOutboundAsicEmpty == TRUE )
		{
			SLIST_FOREACH( thisConn, &DrvTbl.tcpudp[outIdx].list[NAPT_CANDIDATE_OUT], nextOut )
			{
				_rtl8651_AvoidAmbiguousHash2InAsic( thisConn, &bInboundAmbiguous, &bOutboundAmbiguous, &bInboundAsicEmpty, &bOutboundAsicEmpty );
				if ( thisConn->hashing == NAPT_HASH1 )
				{
					if ( thisConn->tcpFlag==TCP_ESTABLISHED_FLOW-1 )
					{
						_rtl8651_changeNaptFlowListType( thisConn, NAPT_ASIC_CACHED_IN, NAPT_ASIC_CACHED_OUT );
						bOutboundAsicEmpty = FALSE;
						break;
					}
				}
				else if ( thisConn->hashing == NAPT_HASH2 && bOutboundAmbiguous == FALSE &&
				          thisConn->newlyAddedHash2Udp==FALSE/*For CDRouter, newly-added connection cannot be ASIC-accelerated.*/ &&
				          thisConn->holdForCdrouter==FALSE /* For CDRouter, newly-added connection cannot be ASIC-accelerated. */ )
				{
					_rtl8651_changeNaptFlowListType( thisConn, thisConn->inListType, NAPT_ASIC_CACHED_OUT );
					bOutboundAsicEmpty = FALSE;
					break;
				}
			}
		}
		// If there is NO connection can be ASIC-accelerated, fine, it is OK.
#endif/*1*/
	}
#ifdef CONFIG_RTL865X_ROMEPERF
	rtl8651_romeperfExitPoint(ROMEPERF_INDEX_NAPT_ADD_11);
#endif

	// dos process
	if ( assigned & DOS_NAPT_ENCOUNT )
	{
		pe->dosEnCount = TRUE;

		if ( isTcp )
		{
			if ( assigned & DOS_NAPT_FROMINTERNAL)
			{
				rtl8651_tblDrvNaptCounter.dosCountedFromInternalTcpCnt++;
			}

			rtl8651_tblDrvNaptCounter.dosCountedTcpCnt++;
		} else
		{
			if ( assigned & DOS_NAPT_FROMINTERNAL)
			{
				rtl8651_tblDrvNaptCounter.dosCountedFromInternalUdpCnt++;
			}

			rtl8651_tblDrvNaptCounter.dosCountedUdpCnt++;
		}

		if ( assigned & DOS_NAPT_FROMINTERNAL )
		{
			pe->dosFromInternal = TRUE;

			_rtl8651_dosSrcConnCtrlUpdateConnection(FALSE, isTcp, insideLocalIpAddr);
		}else
		{
			pe->dosFromInternal = FALSE;

			_rtl8651_dosSrcConnCtrlUpdateConnection(FALSE, isTcp, dstIpAddr);
		}
	}

	// update counter
	rtl8651_tblDrvNaptCounter.addSucceeded++;
	if ( fromDrv ) 
		rtl8651_tblDrvNaptCounter.drvAddCount++;
	else
		rtl8651_tblDrvNaptCounter.protoAddCount++;

	rtl8651_tblDrvNaptCounter.naptUnitAddCnt++;
	
#if 0
	if ( ppe ) *ppe = pe;
#endif

#ifdef CACHE_FIND_NAPT_RESULT
	/* If this connection is in the cache, invalid the cache. */
	if ( 1 )
	{
#ifdef CACHE_FIND_NAPT_DEBUG_RESULT
		rtlglue_printf( "*** addNaptConnection() hit cache! cachedPEntry=%p\n", cachedPEntry );
#endif/*CACHE_FIND_NAPT_DEBUG_RESULT*/
		INVALID_FIND_CACHE();
	}
#endif/*CACHE_FIND_NAPT_RESULT*/
	
#ifdef CONFIG_RTL865X_ROMEPERF
	rtl8651_romeperfExitPoint(ROMEPERF_INDEX_NAPT_ADD);
#endif

return SUCCESS;

/* exit for _rtl8651_getUsableExtPortWithIndex() error. */
err_offset:

#ifdef ADD_NAPT_DEBUG
	rtlglue_printf("[%s:%d] addNaptConnection() returns TBLDRV_EGETOFFSETFAIL.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
	/* we must give back the entry */
	CTAILQ_REMOVE( &DrvTbl.inuseList.tcpudp, pe, next );
	CTAILQ_INSERT_HEAD( &DrvTbl.freeList.tcpudp, pe, next );
#if 0
	if ( ppe ) *ppe = NULL;
#endif

	return TBLDRV_EGETOFFSETFAIL;

/* exit for L07 error. Posible error: addHash2_nexthop() error. */
err_L07:

#ifdef ADD_NAPT_DEBUG
	rtlglue_printf("[%s:%d] addNaptConnection() returns TBLDRV_EADDHASH2NEXTHOP.\n",__FILE__,__LINE__);
#endif/*ADD_NAPT_DEBUG*/
	/* we must give back the entry */
	CTAILQ_REMOVE( &DrvTbl.inuseList.tcpudp, pe, next );
	CTAILQ_INSERT_HEAD( &DrvTbl.freeList.tcpudp, pe, next );
#if 0
	if ( ppe ) *ppe = NULL;
#endif

	return TBLDRV_EADDHASH2NEXTHOP;

}
#if 0
/*
@func void	| _rtl8651_refillOrClearAsicNaptEntry	| Internal use only. Pick a candidate flow for napt line number <p tblIdx>. If no candidates pickable, reinitialize this ASIC entry.
@parm ipaddr_t  | sip	| The internal source IP. All connections originated from this host would be deleted.
@parm struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s * | selectedRefillFlow | selected flow to refill.
@rdesc	<p _rtl8651_refillOrClearAsicNaptEntry()> has no return value.
@comm 
Refill this ASIC entry with a connection selected from driver table. When selecting a candidate flow, prefer inbound flows than outbound flows since inbound flows are usually download links.
If there are no more pending connections, reset the ASIC entry. This function assumes 
ASIC entry IS INDEED UNUSED. Caller should check this before using this API.
@devnote
The preference bias for inbound flow may need to configurable on a per napt line basis.
*/
 void _rtl8651_refillOrClearAsicNaptEntry(uint32 tblIdx,struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s * selectedRefillFlow){
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s  * candidateFlow;	
	rtl8651DrvNaptLine_t *line=&DrvTbl.tcpudp[tblIdx];
	int8 assigned=FALSE;
	int32 inbound=FALSE;
	int32 retval;
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s entry;
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *pEntry=&entry;
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *victimFlow;
	
	//find a candidate flow to refill ASIC entry
	//Prefers inbound flow than outbound flow since inbound flow are usually for downlink.
	
	if(!selectedRefillFlow){
		//find in inbound list first since these are downlink which needs ASIC acceleration the most.
		SLIST_FOREACH(candidateFlow, &line->list[NAPT_CANDIDATE_IN], nextIn){	
			if(candidateFlow->alive) {
				if(selectedRefillFlow == NULL || (candidateFlow->fromDrv && !selectedRefillFlow->fromDrv))
					selectedRefillFlow = candidateFlow; //first possible candidate or.. prefer entries added by driver than protocol stack.

			}
		}
		if(!selectedRefillFlow){
			//Nothing replaceable in inbound list. Find from outbound list.
			SLIST_FOREACH(candidateFlow, &line->list[NAPT_CANDIDATE_OUT], nextOut){	
				if( candidateFlow->alive) {
					if(selectedRefillFlow == NULL ||(candidateFlow->fromDrv && !selectedRefillFlow->fromDrv))
						selectedRefillFlow = candidateFlow; //first possible candidate..or prefer entries added by driver than protocol stack.
			        }	
			}			
		}else
			inbound=TRUE;
	}else{
		assigned=inbound=TRUE;
	}
	if(selectedRefillFlow) {
		//If a TCP entry is not yet established and system in pure software mode or auto learn closed
		//just clear entry, dont' refill
		if(selectedRefillFlow->tcp&&selectedRefillFlow->tcpFlag <TCP_ESTABLISHED_FLOW-1&&(pureSoftFwd||!naptAutoLearn)){
			//rtlglue_printf("Can't refill entry %d. state=%d\n", tblIdx, selectedRefillFlow->tcpFlag);
			goto clear;
		}
		if(inbound){
			if(selectedRefillFlow->outboundTblIdx==selectedRefillFlow->inboundTblIdx){
				SLIST_REMOVE(&line->list[NAPT_CANDIDATE_OUT], selectedRefillFlow, rtl8651_tblDrv_naptTcpUdpFlowEntry_s, nextOut);
				selectedRefillFlow->outListType = NAPT_ASIC_CACHED_OUT;
			}		
			SLIST_REMOVE(&line->list[NAPT_CANDIDATE_IN], selectedRefillFlow, rtl8651_tblDrv_naptTcpUdpFlowEntry_s, nextIn);
			selectedRefillFlow->inListType = NAPT_ASIC_CACHED_IN;
		}else{
			assert(selectedRefillFlow->outboundTblIdx!=selectedRefillFlow->inboundTblIdx);
			SLIST_REMOVE(&line->list[NAPT_CANDIDATE_OUT], selectedRefillFlow, rtl8651_tblDrv_naptTcpUdpFlowEntry_s, nextOut);
			selectedRefillFlow->outListType = NAPT_ASIC_CACHED_OUT;
		}

		/* 
		 * Before the selected refill flow swaps to ASIC, 
		 *   we must sync the TCP state again if there exists a flow in ASIC entry.
		 */
		 victimFlow = SLIST_FIRST( &line->list[NAPT_ASIC_CACHED_OUT] );
		 if ( victimFlow == NULL )
		 	victimFlow = SLIST_FIRST( &line->list[NAPT_ASIC_CACHED_OUT] );

		/* Sync state and age if ASIC state is newer than software.
		 * Note that only HASH1 connection needs to sync.
		 * Because HASH2 ASIC entry does NOT maintain TCP state information.
		 */
		if ( victimFlow->hashing == NAPT_HASH1 &&
		     victimFlow != NULL )
		{
		 	/* victim flow is using ASIC entry */
			retval = _rtl8651_getNaptEntry( tblIdx, line, &pEntry, TRUE );
			assert( retval == SUCCESS );
			assert( pEntry );
			if ( pEntry->alive )
			{
				/* The ASIC entry contains a TCP flow, update the TCP state. */
				if ( victimFlow->tcpFlag == pEntry->tcpFlag )
				{
					/* rtlglue_printf( "victimFlow->tcpFlag(%d) == pEntry->tcpFlag(%d)\n", victimFlow->tcpFlag, pEntry->tcpFlag ); */
				}
				else if ( victimFlow->tcpFlag < pEntry->tcpFlag )
				{
					/* rtlglue_printf( "victimFlow->tcpFlag(%d) < pEntry->tcpFlag(%d), update SW state.\n", victimFlow->tcpFlag, pEntry->tcpFlag ); */
					victimFlow->age = pEntry->age;
					victimFlow->tcpFlag = pEntry->tcpFlag;
				}
				else /* victimFlow->tcpFlag > pEntry->tcpFlag */
				{
					/* rtlglue_printf( "victimFlow->tcpFlag(%d) > pEntry->tcpFlag(%d), it is OK.\n", victimFlow->tcpFlag, pEntry->tcpFlag ); */
				}
			}
			else
			{
				/* The ASIC entry is not alive (not valid), it is NOT we expected. */
				rtlglue_printf( "victim Flow claims owning ASIC entry, but in fact, the ASIC entry is empty.\n" );
				assert( 0 );
			}
		}

		//Replace ASIC entry with selected flow.
		if(_rtl8651_setNaptEntry(tblIdx, line, selectedRefillFlow, assigned)!=SUCCESS){
			//Failed to write to ASIC, roll back. Entry would be refilled next time.
			if(inbound){
				if(selectedRefillFlow->outboundTblIdx==selectedRefillFlow->inboundTblIdx){
					SLIST_INSERT_HEAD(&line->list[NAPT_CANDIDATE_OUT], selectedRefillFlow, nextOut);
					selectedRefillFlow->outListType = NAPT_CANDIDATE_OUT;
				}
				SLIST_INSERT_HEAD(&line->list[NAPT_CANDIDATE_IN], selectedRefillFlow, nextIn);
				selectedRefillFlow->inListType = NAPT_CANDIDATE_IN;
			}else{
				SLIST_INSERT_HEAD(&line->list[NAPT_CANDIDATE_OUT], selectedRefillFlow, nextOut);
				selectedRefillFlow->outListType = NAPT_CANDIDATE_OUT;
			}
		}else{
			//write to ASIC successfully
			if(inbound){
				if(selectedRefillFlow->outboundTblIdx==selectedRefillFlow->inboundTblIdx){
					SLIST_INSERT_HEAD(&line->list[NAPT_ASIC_CACHED_OUT], selectedRefillFlow, nextOut);
					SLIST_INSERT_HEAD(&line->list[NAPT_ASIC_CACHED_IN], selectedRefillFlow, nextIn);
				}else{
					SLIST_INIT(&line->list[NAPT_ASIC_CACHED_OUT]);
					SLIST_INSERT_HEAD(&line->list[NAPT_ASIC_CACHED_IN], selectedRefillFlow, nextIn);
				}
			}else{
				SLIST_INIT(&line->list[NAPT_ASIC_CACHED_IN]);
				SLIST_INSERT_HEAD(&line->list[NAPT_ASIC_CACHED_OUT], selectedRefillFlow, nextOut);
			}
			rtl8651_tblDrvNaptCounter.threadWriteBackCount++;
			//rtlglue_printf("line %d refilled. Total refilled %d\n", tblIdx, rtl8651_tblDrvNaptCounter.threadWriteBackCount);
		}
	} else{ 	
clear:	
		_rtl8651_setNaptEntry( tblIdx, line, NULL, FALSE);//force clear the entry
		SLIST_INIT(&line->list[NAPT_ASIC_CACHED_OUT]);
		SLIST_INIT(&line->list[NAPT_ASIC_CACHED_IN]);

	}
}
#endif/*0*/



/*
@func void	| rtl8651_updateAsicNaptTable	| Housekeeping function. Replace expired ASIC NAPT entries with software candidate flows. Also synchronize dynamic status information of each ASIC entry with software table.
@parm uint32 index	| ASIC index (traverse from tblDrv.tcpudp.inuse list)
@rdesc	<p rtl8651_updateAsicNaptTable()> has no return value.
@comm 
This function must be called by maintenance thread to synchronize asic table entries with software table periodically,
If an ASIC entry is unused, select a candidate flow from software table to ASIC to accelerate forwarding spped of selected flow.
*/
void _rtl8651_updateAsicNaptTable(uint32 index)
{
	rtl8651DrvNaptLine_t *thisLine;
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s entry;
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *pEntry=&entry;
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *asicCache=NULL;
	uint32 asicTcpState, swTcpState;
	
	thisLine = &DrvTbl.tcpudp[index];

	asicCache = SLIST_FIRST(&(thisLine->list[NAPT_ASIC_CACHED_OUT]));

	if( asicCache == NULL ) 
		asicCache = SLIST_FIRST(&(thisLine->list[NAPT_ASIC_CACHED_IN]));

	assert(asicCache);
	// read the ASIC entry
	_rtl8651_getNaptEntry(index, thisLine, &pEntry, TRUE);
	assert( pEntry );

	// If software has cached the connection,
	//   it means the ASIC must hold the connection, too.
	if ( asicCache != NULL && pEntry->alive == FALSE )
	{
		/*assert( 0 );*/ // Something wrong, we need to check ....
		/* ASIC is aged out, but, software is NOT aged out. */
		return;
	}

	//software has cached this ASIC entry, so sync dynamic status info between ASIC and software

	/* Only TCP connection has state information */
	if ( asicCache->tcp )
	{
		if ( asicCache->hashing == NAPT_HASH1 )
		{
			/* HASH1 */
			asicTcpState = pEntry->tcpFlag+1;
			swTcpState = asicCache->tcpFlag+1;
		}
		else if ( asicCache->hashing == NAPT_HASH2 ||
		          asicCache->hashing == NAPT_HASH3 )
		{	
			/* HASH2, ASIC does NOT maintain TCP state.
			 * Thus, asicTcpState is got from software.
			 */
			asicTcpState = asicCache->tcpFlag+1;
			swTcpState = asicCache->tcpFlag+1;
		}
		else
		{
			assert( 0 ); // neither HASH1 nor HASH2
			return;
		}
	}
	else
	{
		asicTcpState = 0;
		swTcpState = 0;
	}

	/* copy out_rt_t to pEntry, therefore, _rtl8651_setNaptEntry() can limit the ASIC age. */
	pEntry->out_rt_t = asicCache->out_rt_t;
	
	/*
	 *  We must check ASIC is either idle or busy.
	 */
	{
		rtl8651_tblDrv_routeTable_t *rt;
		rtl8651_tblDrv_pppTable_t *ppp = NULL;

		rt = (rtl8651_tblDrv_routeTable_t*) asicCache->out_rt_t;
		if ( rt && rt->nextHopType != RTL8651_LL_VLAN )
		{
			rtl8651_tblDrv_nextHopEntry_t *nexthop;

			switch( rt->process )
			{
				case RT_NEXTHOP:
					nexthop = rt->rt_nhstart_t;
					if ( nexthop )
					{
						ppp = (rtl8651_tblDrv_pppTable_t*)nexthop->nexthop_t;
					}
					break;
				case RT_SESSION:
					ppp = rt->rt_nhSession_t;
					break;
			}
		}

		{
			uint32 ageFull; /* the age value meaned that packets are passed thru by ASIC. */

			if ( asicCache->tcp )
				ageFull = rtl8651AsicAge.tcpLongTimeout; /* we assume only established connection will use ASIC entry. */
			else
				ageFull = rtl8651AsicAge.udpTimeout;

			/*
			 * If ASIC age is larger or equal than ageFull,
			 *   it means the packets are passed thru by ASIC.
			 */
			if ( ageFull != 0 &&
			     pEntry->age >= ageFull )
			{
				/* refill software age */
				if ( asicCache->tcp )
					asicCache->age = *tcpAgeTimer[asicTcpState]; /* TCP */
				else
					asicCache->age = rtl8651GlobalControl.udpTimeout; /* UDP */
			}
		}
	}
	
	/* If it is a single-FINed connection, swap to SW. */
	if ( asicTcpState > TCP_ESTABLISHED_FLOW || 
	     swTcpState > TCP_ESTABLISHED_FLOW )
	{
		_rtl8651_changeNaptFlowListType( asicCache, NAPT_CANDIDATE_IN, NAPT_CANDIDATE_OUT );
		return;
	}
		
	if( asicCache->hashing == NAPT_HASH1 )
	{
		// yjlou asserts for testing 4-way hashing. If those values are not equal, 4-way ability has something wrong.
		assert( asicCache->inboundTblIdx == index );
		assert( asicCache->outboundTblIdx == index );
	
		if(asicTcpState == swTcpState){
			uint32 maxAge=0;
			//if auto learn is off, asic will reset age to long timeout upon every pkt rcvd. 
			//Adjust that value to per state config value.
			if(asicTcpState>1){
				maxAge=*tcpAgeTimer[asicTcpState];
				if(maxAge&&pEntry->age>maxAge)
					pEntry->age=*tcpAgeTimer[asicTcpState];
			}else
				maxAge=rtl8651GlobalControl.udpTimeout;
		
			//same state
			if(asicCache->age!=pEntry->age){
				if(asicCache->age>pEntry->age){//flow traffic goes through software. Just overwrite ASIC's age info
					/* In period sync mode, we do sync ASIC in rtl8651_updateAsicNaptTable. */
					if(asicCache->tcp)
					{
						pEntry->age=asicCache->age; 
					}
					else
					{
						if(asicCache->age>=rtl8651AsicAge.udpTimeout)
						{
							pEntry->age= rtl8651AsicAge.udpTimeout-1; 
						}
						else
						{
							pEntry->age=asicCache->age;
						}
					} 
					_rtl8651_setNaptEntry(index, thisLine, pEntry, TRUE);
				}else if(asicCache->age<3){
					//flow traffic goes through ASIC. Don't update it until software entry is about to be deleted.
					asicCache->age=pEntry->age-1; 
				}
			}
		}else if(asicTcpState>swTcpState){
			if(asicTcpState==TCP_INBOUND_SYN_RCVD_FLOW&&swTcpState==TCP_OUTBOUND_SYN_RCVD_FLOW){
				//both asic and driver goes to established state
				asicCache->age=pEntry->age = rtl8651GlobalControl.tcpLongTimeout;
				asicCache->tcpFlag = pEntry->tcpFlag = TCP_ESTABLISHED_FLOW-1;
				_rtl8651_setNaptEntry(index, thisLine, pEntry, TRUE);//force write
			}else if(asicTcpState==TCP_INBOUND_FIN_RCVD_FLOW&&swTcpState==TCP_OUTBOUND_FIN_RCVD_FLOW){
				//both asic and driver goes to closed state
				asicCache->age=pEntry->age = rtl8651GlobalControl.tcpFastTimeout;
				asicCache->tcpFlag = pEntry->tcpFlag = TCP_CLOSED_FLOW-1;
				_rtl8651_setNaptEntry(index, thisLine, pEntry, TRUE);//force write
			}else{
				asicCache->age=pEntry->age;
				asicCache->tcpFlag=pEntry->tcpFlag;
			}
		}else{
			if(swTcpState==TCP_INBOUND_SYN_RCVD_FLOW&&asicTcpState==TCP_OUTBOUND_SYN_RCVD_FLOW){
				//both asic and driver goes to established state
				asicCache->age=pEntry->age = rtl8651GlobalControl.tcpLongTimeout;
				asicCache->tcpFlag = pEntry->tcpFlag = TCP_ESTABLISHED_FLOW-1;
				_rtl8651_setNaptEntry(index, thisLine, pEntry, TRUE);//force write
			}else if(swTcpState==TCP_INBOUND_FIN_RCVD_FLOW&&asicTcpState==TCP_OUTBOUND_FIN_RCVD_FLOW){
				//both asic and driver goes to closed state
				asicCache->age=pEntry->age = rtl8651GlobalControl.tcpFastTimeout;
				asicCache->tcpFlag = pEntry->tcpFlag = TCP_CLOSED_FLOW-1;
				_rtl8651_setNaptEntry(index, thisLine, pEntry, TRUE);//force write
			}else{
				pEntry->age=asicCache->age;
				pEntry->tcpFlag=asicCache->tcpFlag;
				_rtl8651_setNaptEntry(index, thisLine, pEntry, TRUE);				
			}
		}
	}
	else if ( asicCache->hashing == NAPT_HASH2 ||
	          asicCache->hashing == NAPT_HASH3 ) 
	{
		/*
		 * When using hash 2, there is NO state information in ASIC.
		 */
		rtl8651DrvNaptLine_t *line = &DrvTbl.tcpudp[index];
		rtl8651_tblDrv_naptTcpUdpFlowEntry_t *itEntry;
		
		/* Consider the following conditions:
		 *  1. ASIC is almost aged out, but software still has longer age time:
		 *     --> Re-charge ASIC age time.
		 *  2. Software is almost aged out, but ASIC still hss longer age time:
		 *     --> Re-charge software age time.
		 */
		if ( asicCache->tcpFlag != TCP_CLOSED_FLOW-1 ) /* NEVER update age of closed connection. */
		{
			if ( pEntry->age < 3 )
			{
				// ASIC is almost aged out, re-charge ASIC if software age is longer.
				if ( asicCache->age > pEntry->age )
				{
					/* In period sync mode, we do sync ASIC in rtl8651_updateAsicNaptTable. */
					pEntry->age = asicCache->age-1;
					_rtl8651_setNaptEntry(index, thisLine, pEntry, TRUE);//force write
				}
			}
			if ( asicCache->age < 3 )
			{
				// Software is almost aged out, re-charge software if ASIC age is longer.
				if ( pEntry->age > asicCache->age )
				{
					asicCache->age = pEntry->age-1;
				}
			}
		}

		// To forward the arrival packet, the ASIC only checks the following fields:
		//   1.inbound packet: verify ExtPort (Offset)
		//   2.outbound packet: verify IntIp/Port
		// Those packets will NOT trap to CPU in normal case.
		// However, we allow the ExtPort is shared, and the IntIp/Port is duplicable.
		// So that we also update the 'age' fields of the sharing connections in candidate list.
		
		// If it is a inbound or bi-direct flow in ASIC, 
		//   update the age field of the connections holding same ExtPort.
		if ( asicCache->inboundTblIdx == index )
		{
			int i;
			for( i = NAPT_ASIC_CACHED_IN; i < NAPT_TOTAL_LISTS; i += 2 )
			{
				SLIST_FOREACH( itEntry, &line->list[i], nextIn )
				{
					if ( itEntry->tcp == asicCache->tcp &&
					     itEntry->tcpFlag == TCP_ESTABLISHED_FLOW-1 &&
					     itEntry->insideGlobalPort == asicCache->insideGlobalPort )
					{
						/* If software has lower age, refill with ASIC age. */
						if ( itEntry->age < pEntry->age )
						{
							itEntry->age = pEntry->age;
						}
					}
				}
			}
		}

		// If it is a outbound or bi-direct flow in ASIC,
		//   update the age value of the connections holding same IntIp/Port.
		if ( asicCache->outboundTblIdx == index )
		{
			int i;
			for( i = NAPT_ASIC_CACHED_OUT; i < NAPT_TOTAL_LISTS; i += 2 )
			{
				SLIST_FOREACH( itEntry, &line->list[i], nextOut )
				{
					if ( itEntry->tcp == asicCache->tcp &&
					     itEntry->tcpFlag == TCP_ESTABLISHED_FLOW-1 &&
					     itEntry->insideLocalIpAddr == asicCache->insideLocalIpAddr &&
					     itEntry->insideLocalPort == asicCache->insideLocalPort )
					{
						/* If software has lower age, refill with ASIC age. */
						if ( itEntry->age < pEntry->age )
						{
							itEntry->age = pEntry->age;
						}
					}
				}
			}
		}
	}
	else
	{
		assert(0); /* neither HASH1 nor HASH2 */
	}
}




#define MIN_FAST_TIMEOUT_VALUE	2
void _rtl8651_naptTimerAdaptiveAdjust(int8 enabled, uint32 threshold, uint32 secPassed){
	uint32 currentFastTimeoutValue, newFastTimeoutValue;
	if(threshold)
		_adaptiveTimeoutThreshold=threshold;
	if(enabled){
		_enableAdaptiveNAPTfastTimeout=1;
		currentFastTimeoutValue=rtl8651GlobalControl.tcpFinTimeout;
		if(rtl8651_tblDrvNaptCounter.naptUnitAddCnt>_adaptiveTimeoutThreshold){
			if(currentFastTimeoutValue>MIN_FAST_TIMEOUT_VALUE &&_userConfigFastTimeoutValue>MIN_FAST_TIMEOUT_VALUE){
				//we have too many connection create requests in last measured unit time,
				//then may imply we are under heavy loading benchmark test
				//adaptively adjust TCP fast timeout value
				newFastTimeoutValue=(currentFastTimeoutValue>>2); //multiplicative decrease
				newFastTimeoutValue=(newFastTimeoutValue<MIN_FAST_TIMEOUT_VALUE)?
					MIN_FAST_TIMEOUT_VALUE:newFastTimeoutValue;
				rtl8651GlobalControl.tcpFinTimeout=newFastTimeoutValue;
				/*rtl8651_setAsicNaptTcpFastTimeout(newFastTimeoutValue);*/
			}
		}else{
			if(currentFastTimeoutValue < _userConfigFastTimeoutValue){
				newFastTimeoutValue=currentFastTimeoutValue+1; //
				newFastTimeoutValue=(newFastTimeoutValue>_userConfigFastTimeoutValue)?
					_userConfigFastTimeoutValue:newFastTimeoutValue;
				rtl8651GlobalControl.tcpFinTimeout=newFastTimeoutValue;
				/*rtl8651_setAsicNaptTcpFastTimeout(newFastTimeoutValue);*/
			}
		}

		/* 'secPassed==0' means this function is called for setting threshold, instead of timeUpdate(). */
		if ( secPassed > 0 )
		{
			if ( rtl8651_tblDrvNaptCounter.maxNaptUnitAddCnt < ( rtl8651_tblDrvNaptCounter.naptUnitAddCnt/secPassed ) )
			{
				rtl8651_tblDrvNaptCounter.maxNaptUnitAddCnt = rtl8651_tblDrvNaptCounter.naptUnitAddCnt/secPassed;
			}
			rtl8651_tblDrvNaptCounter.naptUnitAddCnt=0;
		}
	}else{
		_enableAdaptiveNAPTfastTimeout=0;
		rtl8651GlobalControl.tcpFinTimeout=_userConfigFastTimeoutValue;
		/*rtl8651_setAsicNaptTcpFastTimeout(_userConfigFastTimeoutValue);*/
	}
}


/*
@func void | _rtl8651_naptSmartFlowSwapByPPS	| Set the threshold of smart swapping.
@parm int8	| enabled | TRUE or FALSE
@parm uint32	| threshold | Packet count
@rvalue NONE	
@comm 
*/
void _rtl8651_naptSmartFlowSwapByPPS(int8 enabled, uint32 threshold)
{
	if(enabled)
	{
		_enableSmartNaptSwappingByLoad = TRUE;
		_smartNaptSwappingThresholdTcp = threshold;
		_smartNaptSwappingThresholdUdp = threshold;
	}
	else
	{
		_enableSmartNaptSwappingByLoad = FALSE;
		_smartNaptSwappingThresholdTcp = threshold;
		_smartNaptSwappingThresholdUdp = threshold;
	}
	return;	
}


/*
@func void | _rtl8651_naptSmartFlowSwapByPPSTcp	| Set the threshold of smart swapping of TCP flow.
@parm uint32	| threshold | Packet count
@rvalue NONE	
@comm 
*/
void _rtl8651_naptSmartFlowSwapByPPSTcp( uint32 threshold )
{
	_smartNaptSwappingThresholdTcp = threshold;
	return;	
}


/*
@func void | _rtl8651_naptSmartFlowSwapByPPSUdp	| Set the threshold of smart swapping of TCP flow.
@parm uint32	| threshold | Packet count
@rvalue NONE	
@comm 
*/
void _rtl8651_naptSmartFlowSwapByPPSUdp( uint32 threshold )
{
	_smartNaptSwappingThresholdUdp = threshold;
	return;	
}


/*
@func void | _rtl8651_naptSmartFlowSwapByPPSCdrouter	| Set the threshold of smart swapping of TCP flow.
@parm uint32	| threshold | Packet count
@rvalue NONE	
@comm 
*/
void _rtl8651_naptSmartFlowSwapByPPSCdrouter( uint32 threshold )
{
	_smartNaptSwappingThresholdCdrouter = threshold;
	return;	
}


/*
@func void | _rtl8651_softwareNaptAging	| Housekeeping function. Retire expired software connections.
@parm uint32	| secPassed	| time passed (in seconds) since last call.
@rdesc	<p _rtl8651_softwareNaptAging()> has no return value.
@comm 
Scan all inuse NAPT flows, retire aged out flows.
If smart flow swapping is enabled, software flows which can be accelerated by ASIC would be swapped to ASIC if their arriving rate faster than predefined threshold value.
*/
void _rtl8651_softwareNaptAging(uint32 secPassed)
{
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s  * thisFlow, *nextFlow;
 	for( thisFlow = CTAILQ_FIRST(&DrvTbl.inuseList.tcpudp); 
 	     thisFlow; 
 	     thisFlow = nextFlow )
 	{
		nextFlow = CTAILQ_NEXT(thisFlow, next);
		assert(thisFlow->alive);
		if (thisFlow->age > secPassed)
		{
			thisFlow->age -= secPassed;
		}
		else if(thisFlow->fromDrv)
		{
			// the age is out.
			int32 retval = SUCCESS;
			//printfByPolling("removing entry %d\n", thisFlow->outboundTblIdx);
			thisFlow->age=0;
			/* Connections configured by protocol stack NEVER got expired, except ALG connections */
			if ( thisFlow->connectionType == NAPT_TYPE_ALG_CONNECTION )
			{
				retval = _rtl8651_delNaptAlgConnection( thisFlow );
			}
			else
			{
				if ( thisFlow->connectionType == NAPT_TYPE_SERVERPORT )
				{
					// Server Port connection will be never aged out.
				}
				else
				{
					retval = _rtl8651_delNaptConnection( thisFlow );
				}
			}
			assert( retval == SUCCESS );
			rtl8651_tblDrvNaptCounter.drvAgeoutCount++;

			continue; /* Conncection deleted, next one */
		}
		else
		{
			// This connection has aged out.
			// However, it is added by protocol stack.
			// Before the PS delete it, we will NOT delete it.
			thisFlow->age=0;

			// So, do nothing here.
		}

#if 0 /* Louis notes: only _rtl8651_updateNaptConnection() needs to check pps value. Because only it increases pps value. */		
		// If the Packets Per Second is more than the threshold, swap to ASIC.		
		if( _enableSmartNaptSwappingByLoad )
		{
			if ( ( thisFlow->tcp && thisFlow->pps >= _smartNaptSwappingThresholdTcp ) ||
			     ( thisFlow->tcp==FALSE && thisFlow->pps >= _smartNaptSwappingThresholdUdp ) )
			{
				_rtl8651_NaptSafeSwapToAsic( thisFlow );
				thisFlow->newlyAddedHash2Udp = FALSE;
#ifdef ADD_NAPT_DEBUG
				if ( thisFlow->tcp )
					rtlglue_printf("[%s:%d] TCP PPS reaches threshold(%d). Swap to ASIC.\n",__FILE__,__LINE__,_smartNaptSwappingThresholdTcp);
				else
					rtlglue_printf("[%s:%d] UDP PPS reaches threshold(%d). Swap to ASIC.\n",__FILE__,__LINE__,_smartNaptSwappingThresholdUdp);
#endif/*ADD_NAPT_DEBUG*/
			}
		}
#endif/*0*/

#ifdef FLUSH_NAPT_OLDEST_CONNECTION2
		/* If the age is smaller than candidateToBeFreed, choose this connection as candidateToBeFreed. */
		if ( candidateToBeFreed == NULL )
		{
			candidateToBeFreed = thisFlow;
		}
		else 
		{
			register uint32 flag1, flag2;

			flag1 = thisFlow->tcp?thisFlow->tcpFlag:TCP_ESTABLISHED_FLOW-1;
			flag2 = candidateToBeFreed->tcp?candidateToBeFreed->tcpFlag:TCP_ESTABLISHED_FLOW-1;
			
			if ( flag1 >= flag2 &&
			     thisFlow->age < candidateToBeFreed->age )
			{
				candidateToBeFreed = thisFlow;
			}
		}
#endif/*FLUSH_NAPT_OLDEST_CONNECTION2*/

		// reset PPS
		thisFlow->pps=0; 
		
		/* If this packet is ASIC-accelerated, update ASIC table. */
		if ( thisFlow->outListType==NAPT_ASIC_CACHED_OUT)	
			_rtl8651_updateAsicNaptTable(thisFlow->outboundTblIdx);
		if(thisFlow->inboundTblIdx!=thisFlow->outboundTblIdx &&
		   thisFlow->inListType==NAPT_ASIC_CACHED_IN)
			_rtl8651_updateAsicNaptTable(thisFlow->inboundTblIdx);

	}

}


/*
@func int32		| rtl8651_addNaptConnection	| Add a NAPT connection.
@parm int8 		| assigned	| TRUE if <p extIP> and <p extPort> already assigned. FALSE if <p extIP> and <p extPort> can be selected freely by driver.
@parm int8 		| flowType	| Type of connections adding. Possible values: UDP_FLOW, TCP_LISTEN_FLOW, TCP_OUTBOUND_SYN_RCVD_FLOW, TCP_INBOUND_SYN_RCVD_FLOW, TCP_ESTABLISHED_FLOW.
@parm ipaddr_t 	| intIP	| Internal IP.
@parm uint16 	| intPort	| Internal Port.
@parm ipaddr_t *	| extIP	| External IP. If <p assigned> is TRUE, <p *extIP> must not be NULL.
@parm uint16	*	| extPort	| External port.  If <p assigned> is FALSE, <p *extIP> must not be NULL.
@parm ipaddr_t	| dstIP	| Destination IP
@parm uint16		| dstPort	| Destination port
@rdesc	Returns SUCCESS if connection added successfully. Returns one of the following eroor codes when error occured.
@rvalue TBLDRV_ENOTINTIP		| 	internal IP <p intIP> does not belong to any of the configured network interfaces.
@rvalue TBLDRV_EDSTIPISLS	| 	Destination IP <p dstIP> belongs to a configured local server. should use routing not NAPT.
@rvalue TBLDRV_ENOTEXTIP		| 	External IP address <p extIP> given is not one of the configured WAN IPs.
@rvalue TBLDRV_ENOFREEBUFFER	| 	Out of buffer space. Can't add any more.
@rvalue TBLDRV_EDRVNAPTEXIST | 	Connection already exists.
@rvalue TBLDRV_ENOTPERMIT	| 	Assigned external IP, port pair doesn't match with hash result. Forbid this new entry add.
@rvalue TBLDRV_EGETOFFSETFAIL | 	Can't allocate a valid offset value when selecting a <p extPort>.
@comm 
Exported function to write a new NAPT entry for 3rd party protocol stack or control protocols such as UPnP.
This function adds a new NAPT connection into table driver. Table driver would try to write it into ASIC whenever possible.
CAUTION: Connections, once successfully added, never released by table driver. Caller MUST use <p rtl8651_delNaptConnection()> to delete the connection.
*/

int32 rtl8651_addNaptConnection(int16 assigned, int8 flowType, ipaddr_t insideLocalIpAddr, uint16 insideLocalPort, 
                                 ipaddr_t *insideGlobalIpAddr, uint16 *insideGlobalPort, ipaddr_t dstIpAddr, uint16 dstPort )
{
	uint32 result;

	rtlglue_drvMutexLock();//Lock resource

	//Add request from protocol stack. call internal NAPT add function. 
	result = _rtl8651_addNaptConnection(FALSE, assigned, flowType, insideLocalIpAddr, insideLocalPort, insideGlobalIpAddr, insideGlobalPort, dstIpAddr, dstPort);

	rtlglue_drvMutexUnlock();//Unlock resource
	return result;
}


/*
@func int32 | rtl8651_delNaptConnection	| Delete an NAPT connection.
@parm int8 		| isTcp	| TRUE for TCP, FALSE for non-TCP connection
@parm ipaddr_t 	| intIP	| Internal IP.
@parm uint16 	| intPort	| Internal Port.
@parm ipaddr_t	| dstIP	| Destination IP
@parm uint16		| dstPort	| Destination port
@rdesc	Returns SUCCESS if connection deleted successfully. Returns one of the following eroor codes when error occured.
@rvalue TBLDRV_ENOTINTIP		| 	internal IP <p intIP> does not belong to any of the configured network interfaces.
@rvalue TBLDRV_EDSTIPISLS	| 	Destination IP <p dstIP> belongs to a configured local server. should use routing not NAPT.
@rvalue TBLDRV_EENTRYNOTFOUND	| 	Connection not found inside table driver. Can't delete.
@comm 
This function deletes an NAPT connection previously added by <p rtl8651_addNaptConnection()>.
All connections MUST be deleted using this function explicitly or the connection would never be freed even after connection terminated.
We note that this function is called by Protocol Stack in typical case.
So that we will NOT remove the connecttion instantly.
		 */
int32 rtl8651_delNaptConnection( int8 isTcp, ipaddr_t insideLocalIpAddr, uint16 insideLocalPort, 
		                         ipaddr_t dstIpAddr, uint16 dstPort )
{
	uint16 idx1, idx1_loop, idx2;
	int32 ret = SUCCESS;
	struct rtl8651_tblDrv_naptTcpUdpFlowEntry_s *deletedEntry;
//	int8 isExternal, is_gwip;

	rtlglue_drvMutexLock();//Lock resource

	idx1 = rtl8651_naptTcpUdpTableIndex( isTcp, insideLocalIpAddr, insideLocalPort, dstIpAddr, dstPort );
	idx2 = rtl8651_naptTcpUdpTableIndex( isTcp, insideLocalIpAddr, insideLocalPort, 0, 0 );


	if (_rtl8651_intNetIpAddr(insideLocalIpAddr)==FALSE) 
	{
//	if (_rtl8651_getIpInfo(insideLocalIpAddr, NULL, &isExternal, &is_gwip) == NULL || isExternal==TRUE || is_gwip==TRUE) {
		ret= TBLDRV_ENOTINTIP;//Local ip address not internal IP address
		goto failed;
	}
      	
	// If the dstIpAddr is local server IP address, this connection could never be NAPTed.
	if( _rtl8651_localServerIpAddr(dstIpAddr) == TRUE )
	{
		ret = TBLDRV_EDSTIPISLS;
		goto failed;
	}

	//find in software table. Use entry index hashed from hash1 algorithm. 
	if (enable4WayHash == TRUE)
	{
		// 4-way hash enabled
		int i;
		idx1_loop = idx1;
		for( i = 0; i < 4; i++ )
		{
			deletedEntry = _rtl8651_findNaptConnection( idx1_loop, isTcp, insideLocalIpAddr, insideLocalPort,
	                                                                      dstIpAddr, dstPort );
			idx1_loop = WRAPIN_4WAY( idx1_loop, 1 );
	     	if ( deletedEntry ) break; // found
		}
	}
	else
	{
		// 1-way hash
		deletedEntry = _rtl8651_findNaptConnection( idx1, isTcp, insideLocalIpAddr, insideLocalPort,
	                                                             dstIpAddr, dstPort );
	}

	// Not found, try hash2 algorithm.		
	if( deletedEntry == NULL ||
	    deletedEntry->fromDrv ) 
	{
		deletedEntry = _rtl8651_findNaptConnection( idx2, isTcp, insideLocalIpAddr,  insideLocalPort,
		                                                         dstIpAddr, dstPort );
		
		if( deletedEntry == NULL ||
		    deletedEntry->fromDrv ) 
		{
			ret= TBLDRV_EENTRYNOTFOUND;//Entry to be deleted is not found
			goto failed;
		}
	}

	//connection found but it was established by forwarding engine. Don't allow user to delete it now.
	if( deletedEntry->fromDrv == TRUE )
	{
		ret= TBLDRV_EENTRYNOTFOUND;
		goto failed;
	}

	//Deleting entry found in software table immediately
	if ( deletedEntry->connectionType == NAPT_TYPE_ALG_CONNECTION )
	{
		ret = _rtl8651_delNaptAlgConnection( deletedEntry );
	}
	else
	{
		ret=_rtl8651_delNaptConnection(deletedEntry);
	}
	if(ret==SUCCESS)
		rtl8651_tblDrvNaptCounter.protoDelCount++;
failed:
	rtlglue_drvMutexUnlock();//Unlock resource
	return ret;
}


/*
@func int32 | rtl8651_flushAllNaptConnections	| Delete all ICMP/UDP/TCP NAPT connections.
@rdesc	<p rtl8651_flushAllNaptConnections()> has no return value.
@comm 
Flush whole NAPT table. Both software and ASIC entries are cleared.
*/
int32 rtl8651_flushAllNaptConnections(void) {
	int32 retval;
	rtlglue_drvMutexLock();//Lock resource
	retval = _rtl8651_flushNaptConnection();
	rtlglue_drvMutexUnlock();//Unlock resource
	return retval;
}

#define RTL8651_NAPT_ICMP_APIS

//
// This function is used to scan the whole table to ensure the ExtId is not in-used.
//
uint8 _rtl8651_NaptIcmpExtIdIsInUsed( ipaddr_t ExtIp, uint16 ExtId )
{
	rtl8651_tblDrv_naptIcmpFlowEntry_t *tmpIcmpEntry;
	uint32 i;

	// yjlou note: The user may have allocated the ExtId, but store the flow in different index entry.
	//             Therefore, we must scan the whole table to find out the flow entry.
	for( i = 0; i < RTL8651_ICMPTBL_SIZE; i++ )
	{
		tmpIcmpEntry = DrvTbl.icmp[i];
		for(; tmpIcmpEntry; tmpIcmpEntry=tmpIcmpEntry->next)
			if (tmpIcmpEntry->insideGlobalId == ExtId) {
				if ( tmpIcmpEntry->insideGlobalIpAddr == ExtIp )
				{
					return TRUE;
				}
			}
	}

	return FALSE;
}



static int32 _rtl8651_selectNaptIcmpFlowOffset(uint32 flowTblIdx, uint32 HashIdx, ipaddr_t ExtIp, uint16 *offset) {
	int8 isValid;
	uint32 mask[2]; //an empty bitmap. flatten all possible offset values.
	rtl8651_tblDrv_naptIcmpFlowEntry_t * trackNaptIcmpFlowPtr;

	mask[0] = mask[1] =0;

	/* Read offset of ASIC entry, should avoid using this value */
	rtl8651_getAsicNaptIcmpOffset(flowTblIdx, offset, &isValid);

	assert(*offset<64);//rtl8651 offset should ranged from 0 to 63
	if(isValid == TRUE)
		mask[*offset>>5] |= ((uint32)1)<<(*offset&0x1f);

	//Do we have a pending/collision entry on hand? should avoid using those values
	trackNaptIcmpFlowPtr = DrvTbl.icmp[flowTblIdx];
	while(trackNaptIcmpFlowPtr) {

		// No matter this flow entry can be writen to ASIC or not, the offset CANNOT be re-used.
		//if(trackNaptIcmpFlowPtr->canAsic == 1) {
		if ( 1 )
		{
			*offset = trackNaptIcmpFlowPtr->insideGlobalId>>RTL8651_TCPUDPTBL_BITS;
			mask[*offset>>5] |= ((uint32)1)<<(*offset&0x1f);
		}
		trackNaptIcmpFlowPtr = trackNaptIcmpFlowPtr->next;
	}

	/* now check which offset value within configured range we can use */	
	for(*offset = 0; *offset < 64; (*offset)++)
	{
		//Entry not occupied
		// yjlou note: The user may have allocated the ExtId, but store the flow in different index entry.
		//             Therefore, we must call _rtl8651_NaptIcmpExtIdIsInUsed() to ensure this ExtId is not in-used.
		uint16 TryExtPort = ( *offset << RTL8651_TCPUDPTBL_BITS ) + HashIdx;
		
		if( ( ( mask[*offset>>5] & ((uint32)1)<<(*offset&0x1f)) ) == 0 &&
		    FALSE == _rtl8651_NaptIcmpExtIdIsInUsed( ExtIp, TryExtPort ) )
			break;
	}
	if(*offset == 64)
		return FAILED;//Unable to get feasible offset
	return SUCCESS;
}


int8 _rtl8651_getIcmpUsableExtIpAndID(ipaddr_t sip, uint16 sID, ipaddr_t dip, ipaddr_t *gip, uint16 *gID) {
	uint32 flowTblIdx, flowHashingIdx;
	uint16 offset;

	if (gip == NULL || gID == NULL)
		return FAILED;

	*gip = _rtl8651_getNaptInsideGlobalIpAddr(FALSE, sip, sID, dip, 0);
	if (*gip==0)
		return FAILED;
		
	flowHashingIdx = rtl8651_naptIcmpTableIndex(sip, sID, dip, &flowTblIdx);
	if (_rtl8651_selectNaptIcmpFlowOffset(flowTblIdx, flowHashingIdx, *gip, &offset ) == FAILED)
		return FAILED;
	*gID = offset<<10 | ((uint16)flowHashingIdx);
	return SUCCESS;
}


static void _rtl8651_removeIcmpFlowBySrcIP(ipaddr_t sip) {
	rtl8651_tblDrv_naptIcmpFlowEntry_t *naptflow;
	ipaddr_t asicLocalIpAddr;
	uint32 flowTblIdx;
	int8 asicIsValid;

	for(flowTblIdx=0; flowTblIdx<RTL8651_ICMPTBL_SIZE; flowTblIdx++) {
		/* handle table driver entry */
		naptflow = DrvTbl.icmp[flowTblIdx];
		while(naptflow) {
			if (naptflow->insideLocalIpAddr == sip) {
				naptflow->age = 0;
				_rtl8651_delNaptIcmpFlow(flowTblIdx, naptflow);
			}
			naptflow = naptflow->next;
		}
		/* handle asic entry */
		convert_getAsicNaptIcmpTable(FALSE, flowTblIdx, &asicLocalIpAddr, NULL, NULL, NULL, NULL, NULL, NULL, &asicIsValid);

// yjlou note: We no longer support ICMP ASIC-accelerated.
#if 0
		if (asicIsValid == TRUE && asicLocalIpAddr == sip)
			_rtl8651_refillNaptIcmpFlowOccupiedEntry(flowTblIdx, NULL);
#endif
	}
}


static rtl8651_tblDrv_naptIcmpFlowEntry_t * _rtl8651_getNaptIcmpFlowEntry(uint32 flowTblIdx, ipaddr_t insideLocalIpAddr, uint16 insideLocalId) {
	rtl8651_tblDrv_naptIcmpFlowEntry_t * trackNaptIcmpFlowPtr;
	
	trackNaptIcmpFlowPtr = DrvTbl.icmp[flowTblIdx];
	while(trackNaptIcmpFlowPtr) {
		if(trackNaptIcmpFlowPtr->insideLocalIpAddr == insideLocalIpAddr && 
		   trackNaptIcmpFlowPtr->insideLocalId == insideLocalId) {
			return trackNaptIcmpFlowPtr;//Entry found
		}
		trackNaptIcmpFlowPtr = trackNaptIcmpFlowPtr->next;
	}
	return NULL;
}

static int32 _rtl8651_delNaptIcmpFlowEntry(uint32 flowTblIdx, rtl8651_tblDrv_naptIcmpFlowEntry_t * deletedEntry) {
	rtl8651_tblDrv_naptIcmpFlowEntry_t * trackEntry;

	if(deletedEntry == NULL || DrvTbl.icmp[flowTblIdx] == NULL)
		return FAILED;//No entry to delete
	/*
	 * If the 'protoDel' field is turned on, then remove it from ASIC and driver
	 * table; otherwise, only remove the entry from ASIC table and the protocol
	 * stack should take care of entry deletion.
	 */	
	if (deletedEntry->protoDel == 1) {
		if(DrvTbl.icmp[flowTblIdx] == deletedEntry)
			DrvTbl.icmp[flowTblIdx] = deletedEntry->next;
		else {
			trackEntry = DrvTbl.icmp[flowTblIdx];
			while(trackEntry->next) {
				if(trackEntry->next == deletedEntry) {
					trackEntry->next = deletedEntry->next;
					break;
				}
				trackEntry = trackEntry->next;
			}
			if(trackEntry == NULL)
				return FAILED;//No matching entry found.
		}
		deletedEntry->next = DrvTbl.freeList.icmp;
		DrvTbl.freeList.icmp = deletedEntry;
		rtl8651ExistedNaptIcmpFlows--;
	}
	rtl8651NaptIcmpFlowNumberTable[flowTblIdx]--;
	if(deletedEntry->canAsic == 0)
		rtl8651NaptIcmpFlowNonAsicNumberTable[flowTblIdx]--;
	deletedEntry->alive = 0;
	return SUCCESS;
}


rtl8651_tblDrv_naptIcmpFlowEntry_t * 
_rtl8651_getOutsideNaptIcmpEntry(ipaddr_t gip, uint16 gID, ipaddr_t dip, int8 isRefresh, int8 needFastFin) {
	rtl8651_tblDrv_naptIcmpFlowEntry_t *tmpIcmpEntry;
	uint32 flowTblIdx = gID&(RTL8651_ICMPTBL_SIZE-1);

	// yjlou note: The user may have assigned the ExtId, but store the flow in different index entry.
	//             Therefore, we must scan the whole table to find out the flow entry.
	uint32 bFound = FALSE;
	for( flowTblIdx = 0; flowTblIdx < RTL8651_ICMPTBL_SIZE; flowTblIdx++ )
	{
		tmpIcmpEntry = DrvTbl.icmp[flowTblIdx];
		for(; tmpIcmpEntry; tmpIcmpEntry=tmpIcmpEntry->next)
		{
			if ( tmpIcmpEntry->age == 0 || !tmpIcmpEntry->alive ) continue;/* This connection is aged out. */
			if ( tmpIcmpEntry->insideGlobalId == gID ) {
			
				// yjlou note: No more consider fromAsic/toAsic.
				//if (tmpIcmpEntry->fromAsic == 1) 
				//{
				//	bFound = TRUE;
				//	break;
				//}
				//if (tmpIcmpEntry->fromAsic == 0 && tmpIcmpEntry->dstIpAddr == dip &&
				//tmpIcmpEntry->insideGlobalIpAddr == gip)
				
				if ( tmpIcmpEntry->dstIpAddr == dip &&
				     tmpIcmpEntry->insideGlobalIpAddr == gip)
				{
					bFound = TRUE;
					break;
				}
			}
		}
		if ( bFound == TRUE ) break;
	}
	if (tmpIcmpEntry != (rtl8651_tblDrv_naptIcmpFlowEntry_t *)NULL) {
		if (isRefresh == TRUE) {
			if (needFastFin == TRUE)
				tmpIcmpEntry->age = rtl8651GlobalControl.icmpFastTimeout;
			else
				tmpIcmpEntry->age = rtl8651GlobalControl.icmpTimeout;
		}
		return tmpIcmpEntry;
	}

	// yjlou note: We no longer support ICMP ASIC-accelerated.
	return tmpIcmpEntry;

	/* Try to read ASIC entry to driver table */
	if(DrvTbl.freeList.icmp) {
		ipaddr_t asicInsideLocalIp;
		uint32 asicAgeSec;
		uint16 asicInsideLocalId, asicInsideGlobalId, asicCount;
		int8 isAsicStatic, isAsicCollision, isAsicValid;

		convert_getAsicNaptIcmpTable(FALSE, flowTblIdx, &asicInsideLocalIp, &asicInsideLocalId, &asicInsideGlobalId, &asicAgeSec, &isAsicStatic, &asicCount, &isAsicCollision, &isAsicValid);
		if(isAsicValid == TRUE && (asicInsideGlobalId&0xfc1f) == (gID&0xfc1f)) {
			tmpIcmpEntry = DrvTbl.freeList.icmp;
			DrvTbl.freeList.icmp = tmpIcmpEntry->next;
			assert(tmpIcmpEntry != DrvTbl.freeList.icmp);
			tmpIcmpEntry->next = DrvTbl.icmp[flowTblIdx];
			DrvTbl.icmp[flowTblIdx] = tmpIcmpEntry;
			assert(tmpIcmpEntry->next != DrvTbl.icmp[flowTblIdx]);
			rtl8651NaptIcmpFlowNumberTable[flowTblIdx]++;
			rtl8651ExistedNaptIcmpFlows++;
			tmpIcmpEntry->canAsic = 1;
			tmpIcmpEntry->insideLocalIpAddr = asicInsideLocalIp;
			tmpIcmpEntry->insideLocalId = asicInsideLocalId;
			tmpIcmpEntry->insideGlobalId= asicInsideGlobalId;
			tmpIcmpEntry->insideGlobalIpAddr = 0;
			tmpIcmpEntry->dstIpAddr = 0;
			tmpIcmpEntry->age = asicAgeSec;
			tmpIcmpEntry->count = asicCount;
			tmpIcmpEntry->fromAsic = 1;
			tmpIcmpEntry->fromDrv = 0;
			tmpIcmpEntry->toAsic = 1;
			tmpIcmpEntry->staticEntry = 0; /* should be dynamic entry */
			tmpIcmpEntry->protoDel = 1;
			tmpIcmpEntry->alive = 1;
			rtl8651_tblDrvIcmpCounter.drvAddCount++;
			return tmpIcmpEntry;
		}
	}
	return (rtl8651_tblDrv_naptIcmpFlowEntry_t *)NULL;
}



rtl8651_tblDrv_naptIcmpFlowEntry_t *
_rtl8651_getInsideNaptIcmpEntry(ipaddr_t sip, uint16 sID, ipaddr_t dip, int8 isRefresh, int8 needFastFin) {
	rtl8651_tblDrv_naptIcmpFlowEntry_t *tmpIcmpEntry;
	uint32 flowTblIdx;
	
	rtl8651_naptIcmpTableIndex(sip, sID, dip, &flowTblIdx);
	tmpIcmpEntry = DrvTbl.icmp[flowTblIdx];
	for(; tmpIcmpEntry; tmpIcmpEntry=tmpIcmpEntry->next)
	{
		if ( tmpIcmpEntry->age == 0 || !tmpIcmpEntry->alive ) continue;/* This connection is aged out. */
		if ((tmpIcmpEntry->insideLocalIpAddr == sip) &&
			(tmpIcmpEntry->dstIpAddr == dip) &&
			(tmpIcmpEntry->insideLocalId == sID)) {
			if (isRefresh == TRUE)
			{
				if (needFastFin == TRUE)
					tmpIcmpEntry->age = rtl8651GlobalControl.icmpFastTimeout;
				else
					tmpIcmpEntry->age = rtl8651GlobalControl.icmpTimeout;
			}
			return tmpIcmpEntry;
		}
	}
	return (rtl8651_tblDrv_naptIcmpFlowEntry_t *)NULL;
}


#if 0
static rtl8651_tblDrv_naptIcmpFlowEntry_t * 
_rtl8651_getAsicNaptIcmpDynamicFlowToTable(uint32 flowTblIdx) {
	ipaddr_t asicInsideLocalIp;
	uint32 asicAgeSec;
	uint16 asicInsideLocalId, asicInsideGlobalId, asicCount;
	int8 asicEntryType, isAsicCollision, isAsicValid;
	rtl8651_tblDrv_naptIcmpFlowEntry_t * asicNaptIcmpFlowPtr;
	
	convert_getAsicNaptIcmpTable(FALSE, flowTblIdx, &asicInsideLocalIp, &asicInsideLocalId, &asicInsideGlobalId, &asicAgeSec, &asicEntryType, &asicCount, &isAsicCollision, &isAsicValid);
	if(isAsicValid == FALSE) {//Remove from ASIC entry
		asicNaptIcmpFlowPtr = NULL;
	}
	else {//Entry valid
		asicNaptIcmpFlowPtr = _rtl8651_getNaptIcmpFlowEntry(flowTblIdx, asicInsideLocalIp, asicInsideLocalId);
		assert(asicNaptIcmpFlowPtr == NULL || asicNaptIcmpFlowPtr->toAsic == 1);
		if(asicNaptIcmpFlowPtr) {
			/* Update aging time */
		 	asicNaptIcmpFlowPtr->age = asicAgeSec;
			assert(asicNaptIcmpFlowPtr->toAsic == 1);
			assert(asicNaptIcmpFlowPtr->canAsic == 1);
			return asicNaptIcmpFlowPtr;//Entry is duplicated no further process is required
		}
		/*
		 * Otherwise, the entry was added by ASIC auto-learn.
		 */
		assert(asicEntryType== RTL8651_DYNAMIC_NAPT_ENTRY);//Should not get static entry but not in driver table
		if(DrvTbl.freeList.icmp == NULL)
			return NULL;//Entry exist but not storage to store it.
		asicNaptIcmpFlowPtr = DrvTbl.freeList.icmp;
		DrvTbl.freeList.icmp = asicNaptIcmpFlowPtr->next;
		asicNaptIcmpFlowPtr->next = DrvTbl.icmp[flowTblIdx];
		DrvTbl.icmp[flowTblIdx] = asicNaptIcmpFlowPtr;
		rtl8651NaptIcmpFlowNumberTable[flowTblIdx]++;
		rtl8651ExistedNaptIcmpFlows++;
		asicNaptIcmpFlowPtr->canAsic = 1;
		asicNaptIcmpFlowPtr->insideLocalIpAddr = asicInsideLocalIp;
		asicNaptIcmpFlowPtr->insideLocalId = asicInsideLocalId;
		asicNaptIcmpFlowPtr->insideGlobalId= asicInsideGlobalId;
		asicNaptIcmpFlowPtr->age = asicAgeSec;
		asicNaptIcmpFlowPtr->count = asicCount;
		asicNaptIcmpFlowPtr->fromAsic = 1;
		asicNaptIcmpFlowPtr->fromDrv = 0;
		asicNaptIcmpFlowPtr->toAsic = 1;
		asicNaptIcmpFlowPtr->staticEntry = 0; /* should be dynamic entry */
		asicNaptIcmpFlowPtr->protoDel = 1;
		asicNaptIcmpFlowPtr->alive = 1;
		rtl8651_tblDrvIcmpCounter.drvAddCount++;
	}
	return asicNaptIcmpFlowPtr;
}
#endif

int8 _rtl8651_addNaptIcmpCandidateFlow(int8 fromDrv, ipaddr_t insideLocalIpAddr, uint16 insideLocalId, 
			ipaddr_t insideGlobalIpAddr, uint16 insideGlobalId, ipaddr_t dstIpAddr) {
	uint32 flowTblIdx, asicAgeSec;
	ipaddr_t asicInsideLocalIp;
	uint16 asicInsideLocalId, asicInsideGlobalId, asicCount;
	int8 asicEntryType, asicIsColl, asicIsValid;
	rtl8651_tblDrv_naptIcmpFlowEntry_t * tempNaptIcmpFlowPtr, 
									   * trackNaptIcmpFlowPtr;

	if (_rtl8651_isNaptMapping(insideGlobalIpAddr) == FALSE && softNaptIp!=insideGlobalIpAddr)
		return FAILED;
	rtl8651_naptIcmpTableIndex(insideLocalIpAddr, insideLocalId, dstIpAddr, &flowTblIdx);
	if(DrvTbl.freeList.icmp == NULL) {
		rtl8651_tblDrvIcmpCounter.addFailure++;
		return TBLDRV_ENOFREEBUFFER;//No free flow entry available
	}
	tempNaptIcmpFlowPtr = DrvTbl.freeList.icmp;
	DrvTbl.freeList.icmp = tempNaptIcmpFlowPtr->next;
	tempNaptIcmpFlowPtr->next = DrvTbl.icmp[flowTblIdx];
	DrvTbl.icmp[flowTblIdx] = tempNaptIcmpFlowPtr;
	rtl8651ExistedNaptIcmpFlows++;
	rtl8651NaptIcmpFlowNumberTable[flowTblIdx]++;
	tempNaptIcmpFlowPtr->insideLocalIpAddr = insideLocalIpAddr;
	tempNaptIcmpFlowPtr->insideLocalId = insideLocalId;
	tempNaptIcmpFlowPtr->insideGlobalId = insideGlobalId;
	tempNaptIcmpFlowPtr->insideGlobalIpAddr = insideGlobalIpAddr;
	tempNaptIcmpFlowPtr->dstIpAddr = dstIpAddr;
	tempNaptIcmpFlowPtr->age = rtl8651GlobalControl.icmpTimeout;
	tempNaptIcmpFlowPtr->canAsic = 1;
	tempNaptIcmpFlowPtr->fromAsic = 0;
	tempNaptIcmpFlowPtr->fromDrv = fromDrv==TRUE? 1: 0;
	tempNaptIcmpFlowPtr->staticEntry = 1;
	tempNaptIcmpFlowPtr->protoDel = fromDrv==TRUE? 1: 0;
	tempNaptIcmpFlowPtr->alive = 1;
	tempNaptIcmpFlowPtr->toAsic = 0;
	tempNaptIcmpFlowPtr->count = 0;
	tempNaptIcmpFlowPtr->dsid = 0;
	if (fromDrv == FALSE)
		rtl8651_tblDrvIcmpCounter.protoAddCount++;
	else rtl8651_tblDrvIcmpCounter.drvAddCount++;
	
	if(rtl8651NaptIcmpBackwardCollisionCountTable[flowTblIdx] > 0) 
		return SUCCESS; //Entry is collided means no change to installed to ASIC
	if(rtl8651NaptIcmpFlowNumberTable[flowTblIdx] - rtl8651NaptIcmpFlowNonAsicNumberTable[flowTblIdx] == 1) {//First can forward by ASIC entry
		if(convert_setAsicNaptIcmpTable(FALSE, tempNaptIcmpFlowPtr->insideLocalIpAddr, tempNaptIcmpFlowPtr->insideLocalId, tempNaptIcmpFlowPtr->insideGlobalId, tempNaptIcmpFlowPtr->age, tempNaptIcmpFlowPtr->staticEntry==1? RTL8651_STATIC_NAPT_ENTRY: RTL8651_DYNAMIC_NAPT_ENTRY, tempNaptIcmpFlowPtr->count, rtl8651NaptIcmpFlowNumberTable[flowTblIdx]> 1 || rtl8651NaptIcmpFlowNonAsicNumberTable[flowTblIdx]>0 ||rtl8651NaptIcmpBackwardCollisionCountTable[flowTblIdx]>0? TRUE: FALSE, TRUE) == SUCCESS)
			tempNaptIcmpFlowPtr->toAsic = 1;
		else { 
			//chhuang: set collison bit
			// NOTE:
			//		It may happen that the entry is immediately timeout after the first try-write. In
			//		such situation, we may consider that the entry is still valid; but in fact, it is
			//		timeout already.
			//		TO SOVE this problem, DO NOT CARE this fact, once a NULL is returned, force write!
			convert_getAsicNaptIcmpTable(FALSE, flowTblIdx, &asicInsideLocalIp, &asicInsideLocalId, &asicInsideGlobalId, &asicAgeSec, &asicEntryType, &asicCount, &asicIsColl, &asicIsValid);
			if (asicIsValid == TRUE)
				convert_setAsicNaptIcmpTable(TRUE, asicInsideLocalIp, asicInsideLocalId, asicInsideGlobalId, asicAgeSec, asicEntryType, asicCount, TRUE, asicIsValid);
			else {
				convert_setAsicNaptIcmpTable(TRUE, tempNaptIcmpFlowPtr->insideLocalIpAddr, tempNaptIcmpFlowPtr->insideLocalId, tempNaptIcmpFlowPtr->insideGlobalId, tempNaptIcmpFlowPtr->age, tempNaptIcmpFlowPtr->staticEntry==1? RTL8651_STATIC_NAPT_ENTRY: RTL8651_DYNAMIC_NAPT_ENTRY, tempNaptIcmpFlowPtr->count, rtl8651NaptIcmpFlowNumberTable[flowTblIdx]> 1 || rtl8651NaptIcmpFlowNonAsicNumberTable[flowTblIdx]>0 ||rtl8651NaptIcmpBackwardCollisionCountTable[flowTblIdx]>0? TRUE: FALSE, TRUE);
				tempNaptIcmpFlowPtr->toAsic = 1;
			}
		}
	}
	else if(rtl8651NaptIcmpFlowNumberTable[flowTblIdx] == 2) {
		trackNaptIcmpFlowPtr = DrvTbl.icmp[flowTblIdx];
		while(trackNaptIcmpFlowPtr) {
			if(trackNaptIcmpFlowPtr->toAsic == 1)
				break;
			trackNaptIcmpFlowPtr = trackNaptIcmpFlowPtr->next;
		}
		if(trackNaptIcmpFlowPtr) //Entry previous filled into ASIC
			convert_setAsicNaptIcmpTable(TRUE, trackNaptIcmpFlowPtr->insideLocalIpAddr, trackNaptIcmpFlowPtr->insideLocalId, trackNaptIcmpFlowPtr->insideGlobalId, trackNaptIcmpFlowPtr->age, trackNaptIcmpFlowPtr->staticEntry==1? RTL8651_STATIC_NAPT_ENTRY: RTL8651_DYNAMIC_NAPT_ENTRY, trackNaptIcmpFlowPtr->count, rtl8651NaptIcmpFlowNumberTable[flowTblIdx]> 1 || rtl8651NaptIcmpFlowNonAsicNumberTable[flowTblIdx]>0 ||rtl8651NaptIcmpBackwardCollisionCountTable[flowTblIdx]>0? TRUE: FALSE, TRUE);
	}
	return SUCCESS;//Entry either write to ASIC or update ASIC information
}

int32 _rtl8651_addNaptIcmpNonCandidateFlow(int8 fromDrv, ipaddr_t insideLocalIpAddr, uint16 insideLocalId, 
			ipaddr_t insideGlobalIpAddr, uint16 insideGlobalId, ipaddr_t dstIpAddr) {
	uint32 flowTblIdx;



	rtl8651_tblDrv_naptIcmpFlowEntry_t * tempNaptIcmpFlowPtr, * asicNaptIcmpFlowPtr;
	uint32 i;

	if (_rtl8651_isNaptMapping(insideGlobalIpAddr) == FALSE && softNaptIp!=insideGlobalIpAddr)
		return FAILED;
		
	rtl8651_naptIcmpTableIndex(insideLocalIpAddr, insideLocalId, dstIpAddr, &flowTblIdx);

	/* Check if user specified ID is in use */
	/* yjlou note: we must scan the whole table to ensure the external ID is not in-used. */
	for( i = 0; i < RTL8651_ICMPTBL_SIZE; i++ )
	{
		tempNaptIcmpFlowPtr = DrvTbl.icmp[i];
		for(; tempNaptIcmpFlowPtr; tempNaptIcmpFlowPtr = tempNaptIcmpFlowPtr->next)
		{
			if ( tempNaptIcmpFlowPtr->insideGlobalIpAddr == insideGlobalIpAddr &&
			     tempNaptIcmpFlowPtr->insideGlobalId == insideGlobalId )
				return(TBLDRV_EEXTIDINUSE);
		}
	}
	if(DrvTbl.freeList.icmp == NULL) {
		rtl8651_tblDrvIcmpCounter.addFailure++;
		return TBLDRV_ENOFREEBUFFER;//No free flow entry available
	}
	tempNaptIcmpFlowPtr = DrvTbl.freeList.icmp;
	DrvTbl.freeList.icmp = tempNaptIcmpFlowPtr->next;
	tempNaptIcmpFlowPtr->next = DrvTbl.icmp[flowTblIdx];
	DrvTbl.icmp[flowTblIdx] = tempNaptIcmpFlowPtr;
	rtl8651ExistedNaptIcmpFlows++;
	rtl8651NaptIcmpFlowNumberTable[flowTblIdx]++;
	rtl8651NaptIcmpFlowNonAsicNumberTable[flowTblIdx]++;
	tempNaptIcmpFlowPtr->insideLocalIpAddr = insideLocalIpAddr;
	tempNaptIcmpFlowPtr->insideLocalId = insideLocalId;
	tempNaptIcmpFlowPtr->insideGlobalId = insideGlobalId;
	tempNaptIcmpFlowPtr->insideGlobalIpAddr = insideGlobalIpAddr;
	tempNaptIcmpFlowPtr->dstIpAddr = dstIpAddr;
	tempNaptIcmpFlowPtr->age = rtl8651GlobalControl.icmpTimeout;
	tempNaptIcmpFlowPtr->canAsic = 0;
	tempNaptIcmpFlowPtr->fromAsic = 0;
	tempNaptIcmpFlowPtr->fromDrv = fromDrv==TRUE? 1: 0;
	tempNaptIcmpFlowPtr->staticEntry = 1;
	tempNaptIcmpFlowPtr->protoDel = fromDrv==TRUE? 1: 0;
	tempNaptIcmpFlowPtr->alive = 1;
	tempNaptIcmpFlowPtr->count = 0;
	tempNaptIcmpFlowPtr->toAsic = 0;
	tempNaptIcmpFlowPtr->dsid = 0;
	rtl8651_tblDrvIcmpCounter.diffHashCount++;
	if (fromDrv == FALSE)
		rtl8651_tblDrvIcmpCounter.protoAddCount++;
	
	if(rtl8651NaptIcmpBackwardCollisionCountTable[flowTblIdx] == 0) {//Only when non-backward collided need to fill collision bit
		asicNaptIcmpFlowPtr = DrvTbl.icmp[flowTblIdx];
		while(asicNaptIcmpFlowPtr) {
			if(asicNaptIcmpFlowPtr->toAsic == 1)
				break;
			asicNaptIcmpFlowPtr = asicNaptIcmpFlowPtr->next;
		}

// yjlou note: We no longer support ICMP ASIC-accelerated.
#if 0
		if(asicNaptIcmpFlowPtr)
			convert_setAsicNaptIcmpTable(TRUE, asicNaptIcmpFlowPtr->insideLocalIpAddr, asicNaptIcmpFlowPtr->insideLocalId, asicNaptIcmpFlowPtr->insideGlobalId, asicNaptIcmpFlowPtr->age, asicNaptIcmpFlowPtr->staticEntry==1? RTL8651_STATIC_NAPT_ENTRY: RTL8651_DYNAMIC_NAPT_ENTRY, asicNaptIcmpFlowPtr->count, rtl8651NaptIcmpFlowNumberTable[flowTblIdx]> 1 || rtl8651NaptIcmpFlowNonAsicNumberTable[flowTblIdx]>0 || rtl8651NaptIcmpBackwardCollisionCountTable[flowTblIdx]>0? TRUE: FALSE, TRUE);
		else {
			if(convert_setAsicNaptIcmpTable(FALSE, 0, 0, flowTblIdx, 0, FALSE, 0, TRUE, FALSE) == FAILED) {
				convert_getAsicNaptIcmpTable(FALSE, flowTblIdx, &asicInsideLocalIp, &asicInsideLocalId, &asicInsideGlobalId, &asicAgeSec, &asicEntryType, &asicCount, &asicIsColl, &asicIsValid);
				if (asicIsValid == TRUE)
					convert_setAsicNaptIcmpTable(TRUE, asicInsideLocalIp, asicInsideLocalId, asicInsideGlobalId, asicAgeSec, asicEntryType, asicCount, TRUE, asicIsValid);
				else //If unable to write normally and unable to get valid ASIC entry, just force write to collide the entry
					convert_setAsicNaptIcmpTable(TRUE, 0, 0, flowTblIdx, 0, FALSE, 0, TRUE, FALSE);
			}
		}
#endif//0
	}
	flowTblIdx = insideGlobalId & (RTL8651_ICMPTBL_SIZE-1);
	rtl8651NaptIcmpBackwardCollisionCountTable[flowTblIdx]++;

// yjlou note: We no longer support ICMP ASIC-accelerated.
#if 0
	if(convert_setAsicNaptIcmpTable(FALSE, 0, 0, flowTblIdx, 0, FALSE, 0, TRUE, FALSE) == FAILED) {
		asicNaptIcmpFlowPtr = _rtl8651_getAsicNaptIcmpDynamicFlowToTable(flowTblIdx);
		asicNaptIcmpFlowPtr->toAsic = 0;
		convert_setAsicNaptIcmpTable(TRUE, 0, 0, flowTblIdx, 0, FALSE, 0, TRUE, FALSE);
	}
#endif//0

	return SUCCESS;//Entry either write to ASIC or update ASIC information
}

/*
@func int32 | rtl8651_addNaptIcmpFlow | Create a NAPT ICMP flow.
@parm int8 | assigned | See comments.
@parm ipaddr_t | insideLocalIpAddr | Local(pricate) IP address.
@parm uint16 | insideLocalId | Local ICMP ID.
@parm ipaddr_t * | insideGlobalIpAddr | If <p assigned> is FALSE, this parameter is assigned an External IP address selected by table driver. If is TRUE, the External IP address is selected by users and will force table driver uses user-assigned IP address as external IP address.
@parm uint16 * | insideGlobalId | If <p assigned> is FALSE, this parameter is assigned an External Port number by table driver. If is TRUE, the External Port number is selected by users and will force table driver uses user-assigned Port number as external Port number.
@parm ipaddr_t | dstIpAddr | Destination IP address.
@rvalue SUCCESS | A new NAPT ICMP entry was created successfully.
@rvalue TBLDRV_EINVALIDINPUT | Invalid input parameter.
@rvalue TBLDRV_EINVEXTIP | No valid external IP address found.
@rvalue TBLDRV_ENOTINTIP | Specified internal IP address doesn't belong to internal interface.
@rvalue TBLDRV_ENOTEXTIP | Specified external IP address doesn't belong to external interface.
@rvalue TBLDRV_EDSTIPISLS | NAPT DST IP is Local Server.
@rvalue TBLDRV_EENTRYALREADYEXIST | The specified entry already exists.
@rvalue TBLDRV_EEXTIDINUSE | NAPT ICMP external ID is in use.
@rvalue TBLDRV_ENOFREEBUFFER | Out of memory buffer.
@comm
The <p rtl8651_addNaptIcmpFlow()> creates a ICMP entry in the ASIC ICMP table. It provides two configuration for use. One is that uses do not care the External IP and External Port selection. Hence the parameter <p assigned> can be set to FALSE
to let table driver auto select. Another is that uses select External IP and External Port by themself. Hence <p assigned> can be set to TRUE to prevent from table driver auto selection.
*/
int32 rtl8651_addNaptIcmpFlow(int8 assigned, ipaddr_t insideLocalIpAddr, uint16 insideLocalId, 
			ipaddr_t *insideGlobalIpAddr, uint16 *insideGlobalId, ipaddr_t dstIpAddr) {
	rtl8651_tblDrv_naptIcmpFlowEntry_t * tempNaptIcmpFlowPtr;
	uint32 flowTblIdx, flowHashingIdx, result;
	uint16 offset;

	if(insideGlobalIpAddr == NULL || insideGlobalId == NULL)	 
		return TBLDRV_EINVALIDINPUT;//No IP and ID for storage
	rtlglue_drvMutexLock();//Lock resource
	 
	if(_rtl8651_intNetIpAddr(insideLocalIpAddr) == FALSE) {
		rtlglue_drvMutexUnlock();//Unlock resource	 
		return TBLDRV_ENOTINTIP;//Inside local ip address does not belong to configured network interface
	}
	if(assigned == TRUE && _rtl8651_extIpAddr(*insideGlobalIpAddr, NULL) == FALSE) {
		rtlglue_drvMutexUnlock();//Unlock resource
		return TBLDRV_ENOTEXTIP;//Inside global ip address does not belong to external IP interface IP addresses
	}
	if(_rtl8651_localServerIpAddr(dstIpAddr) == TRUE) {
		rtlglue_drvMutexUnlock();//Unlock resource
		return TBLDRV_EDSTIPISLS;//Local server should not trigger NAT operation
	}

	flowHashingIdx = rtl8651_naptIcmpTableIndex(insideLocalIpAddr, insideLocalId, dstIpAddr, &flowTblIdx);
	assert(flowHashingIdx < RTL8651_TCPUDPTBL_SIZE);//Hashing result is 10-bit instead of matching table size

	// If the user does NOT specific the ExtIp and ExtId, 
	//   we should allocate those for him.
	if(assigned == FALSE) {

		*insideGlobalIpAddr = _rtl8651_getNaptInsideGlobalIpAddr(FALSE, insideLocalIpAddr, insideLocalId, dstIpAddr, 0);
		if ( *insideGlobalIpAddr == 0 )
		{
			rtlglue_drvMutexUnlock();//Unlock resource
			return TBLDRV_EINVEXTIP;
		}
			
		if( FAILED == _rtl8651_selectNaptIcmpFlowOffset( flowTblIdx, flowHashingIdx, *insideGlobalIpAddr, &offset ) ) 
		{
			rtlglue_drvMutexUnlock();//Unlock resource
			return TBLDRV_EDSTIPISLS;//Available ID not found
		}
		*insideGlobalId = offset<<10 | flowHashingIdx;

	}
	
	if ((tempNaptIcmpFlowPtr=_rtl8651_getNaptIcmpFlowEntry(flowTblIdx, insideLocalIpAddr, insideLocalId))) {
		/*
		 * After protocol stack removing and Beofer entry aging out, if protocol stack
		 * adds the same entry again, handle it!
		 */
		if (tempNaptIcmpFlowPtr->protoDel == 1 && tempNaptIcmpFlowPtr->age > 0) {
			tempNaptIcmpFlowPtr->age = rtl8651GlobalControl.icmpTimeout;

// yjlou note: We no longer support ICMP ASIC-accelerated.
#if 0
			if (tempNaptIcmpFlowPtr->toAsic == 1)
				convert_setAsicNaptIcmpTable(TRUE, tempNaptIcmpFlowPtr->insideLocalIpAddr, tempNaptIcmpFlowPtr->insideLocalId, tempNaptIcmpFlowPtr->insideGlobalId, tempNaptIcmpFlowPtr->age, (tempNaptIcmpFlowPtr->staticEntry==1?RTL8651_STATIC_NAPT_ENTRY: RTL8651_DYNAMIC_NAPT_ENTRY), tempNaptIcmpFlowPtr->count, rtl8651NaptIcmpFlowNumberTable[flowTblIdx]> 1 || rtl8651NaptIcmpFlowNonAsicNumberTable[flowTblIdx]>0 || rtl8651NaptIcmpBackwardCollisionCountTable[flowTblIdx]>0? TRUE: FALSE, TRUE);
#endif//0
			tempNaptIcmpFlowPtr->protoDel = 0;
			*insideGlobalId = tempNaptIcmpFlowPtr->insideGlobalId;
			*insideGlobalIpAddr = tempNaptIcmpFlowPtr->insideGlobalIpAddr;
			rtl8651_tblDrvIcmpCounter.protoAddCount++;
			rtlglue_drvMutexUnlock();//Unlock resource
			return SUCCESS;
		}
		rtlglue_drvMutexUnlock();//Unlock resource
		return TBLDRV_EENTRYALREADYEXIST; /* Duplicate entry */
	}

	result = _rtl8651_addNaptIcmpNonCandidateFlow(FALSE, insideLocalIpAddr, insideLocalId, *insideGlobalIpAddr, *insideGlobalId, dstIpAddr);
	/*
	// yjlou note: We no longer support ICMP ASIC-accelerated in 8650B.
	if(flowTblIdx != (*insideGlobalId&(RTL8651_ICMPTBL_SIZE-1)) || *insideGlobalIpAddr != _rtl8651_getNaptInsideGlobalIpAddr(FALSE, insideLocalIpAddr, insideLocalId, dstIpAddr, 0)) //Unable to configure into ASIC
		result = _rtl8651_addNaptIcmpNonCandidateFlow(FALSE, insideLocalIpAddr, insideLocalId, *insideGlobalIpAddr, *insideGlobalId, dstIpAddr);
	else 
		result = _rtl8651_addNaptIcmpCandidateFlow(FALSE, insideLocalIpAddr, insideLocalId, *insideGlobalIpAddr, *insideGlobalId, dstIpAddr);
	*/

	rtlglue_drvMutexUnlock();//Unlock resource
	return result;
}

#if 0
static void _rtl8651_refillNaptIcmpFlowOccupiedEntry(uint32 flowTblIdx, rtl8651_tblDrv_naptIcmpFlowEntry_t *delEntry) {
	rtl8651_tblDrv_naptIcmpFlowEntry_t * trackNaptIcmpFlowPtr, * asicNaptIcmpFlowPtr;
	ipaddr_t asicInsideLocalIp;
	uint32 asicAgeSec;
	uint16 asicInsideLocalId, asicInsideGlobalId, asicCount;
	int8 asicEntryType, asicIsColl, asicIsValid, drvToAsic = FALSE;

	if (rtl8651NaptIcmpBackwardCollisionCountTable[flowTblIdx] > 0)
		return; //chhuang: bwcoll count>0 => nothing to do
	asicNaptIcmpFlowPtr = NULL;//Find candidate entry to fill into ASIC
	if(rtl8651NaptIcmpFlowNumberTable[flowTblIdx] - rtl8651NaptIcmpFlowNonAsicNumberTable[flowTblIdx] > 0) {
		trackNaptIcmpFlowPtr = DrvTbl.icmp[flowTblIdx];
		while(trackNaptIcmpFlowPtr) {//Count entry number
			if(trackNaptIcmpFlowPtr->canAsic == 1 && trackNaptIcmpFlowPtr->age > 0) {
				if(asicNaptIcmpFlowPtr == NULL)
					asicNaptIcmpFlowPtr = trackNaptIcmpFlowPtr;
				else if(trackNaptIcmpFlowPtr->fromAsic > asicNaptIcmpFlowPtr->fromAsic)
					asicNaptIcmpFlowPtr = trackNaptIcmpFlowPtr;
				else if (trackNaptIcmpFlowPtr->fromAsic ==  asicNaptIcmpFlowPtr->fromAsic && trackNaptIcmpFlowPtr->fromDrv > asicNaptIcmpFlowPtr->fromDrv)
					asicNaptIcmpFlowPtr = trackNaptIcmpFlowPtr;
			}
			if (trackNaptIcmpFlowPtr->toAsic == 1)
				drvToAsic = TRUE;
			trackNaptIcmpFlowPtr = trackNaptIcmpFlowPtr->next;
		}
	}
	if (delEntry != NULL) {
		convert_getAsicNaptIcmpTable(FALSE, flowTblIdx, &asicInsideLocalIp, &asicInsideLocalId, &asicInsideGlobalId, &asicAgeSec, &asicEntryType, &asicCount, &asicIsColl, &asicIsValid);
		if (asicIsValid == TRUE) {
			if (delEntry->insideLocalIpAddr != asicInsideLocalIp ||
				delEntry->insideLocalId != asicInsideLocalId ||
				(delEntry->insideGlobalId&0xfc1f) != (asicInsideGlobalId&0xfc1f)) {
				convert_setAsicNaptIcmpTable(TRUE, asicInsideLocalIp, asicInsideLocalId, asicInsideGlobalId, asicAgeSec, asicEntryType, asicCount, (rtl8651NaptIcmpFlowNumberTable[flowTblIdx]>0&&drvToAsic==FALSE) || (rtl8651NaptIcmpFlowNumberTable[flowTblIdx]>1&&drvToAsic==TRUE) || rtl8651NaptIcmpFlowNonAsicNumberTable[flowTblIdx]>0 || rtl8651NaptIcmpBackwardCollisionCountTable[flowTblIdx]>0? TRUE: FALSE, asicIsValid);
				return;
			}
		}
	}
	if(asicNaptIcmpFlowPtr) {
		convert_setAsicNaptIcmpTable(TRUE, asicNaptIcmpFlowPtr->insideLocalIpAddr, asicNaptIcmpFlowPtr->insideLocalId, asicNaptIcmpFlowPtr->insideGlobalId, asicNaptIcmpFlowPtr->age, asicNaptIcmpFlowPtr->staticEntry==1? RTL8651_STATIC_NAPT_ENTRY: RTL8651_DYNAMIC_NAPT_ENTRY, asicNaptIcmpFlowPtr->count, rtl8651NaptIcmpFlowNumberTable[flowTblIdx]>1 || rtl8651NaptIcmpFlowNonAsicNumberTable[flowTblIdx] > 0 || rtl8651NaptIcmpBackwardCollisionCountTable[flowTblIdx]>0? TRUE: FALSE, TRUE);
		/*
		 * If the entry is from ASIC, write it into ASIC and remove it from driver table
		 */
		if (asicNaptIcmpFlowPtr->fromAsic == 1)
			_rtl8651_delNaptIcmpFlowEntry(flowTblIdx, asicNaptIcmpFlowPtr);
	 	else asicNaptIcmpFlowPtr->toAsic = 1;		
		//asicNaptIcmpFlowPtr->toAsic = 1;
		rtl8651_tblDrvIcmpCounter.threadWriteBackCount++;
	} else convert_setAsicNaptIcmpTable(TRUE, 0, 0, flowTblIdx, 0, FALSE, 0, rtl8651NaptIcmpFlowNumberTable[flowTblIdx]>1 || rtl8651NaptIcmpFlowNonAsicNumberTable[flowTblIdx] > 0 || rtl8651NaptIcmpBackwardCollisionCountTable[flowTblIdx]>0? TRUE: FALSE, FALSE);
}
#endif

static int32 _rtl8651_delNaptIcmpFlow(uint32 flowTblIdx, rtl8651_tblDrv_naptIcmpFlowEntry_t *delEntry) {




	rtl8651_tblDrv_naptIcmpFlowEntry_t *tmpDrvEntry;

	if (delEntry == (rtl8651_tblDrv_naptIcmpFlowEntry_t *)0)
		return FAILED;
	
	_rtl8651_delNaptIcmpFlowEntry(flowTblIdx, delEntry);
	rtl8651_tblDrvIcmpCounter.drvDelCount++;
	if(delEntry->toAsic == 1) {
		assert(delEntry->canAsic == 1);
		assert(rtl8651NaptIcmpBackwardCollisionCountTable[flowTblIdx] == 0);
	}
	if(delEntry->toAsic == 1 || delEntry->canAsic == 0)

// yjlou note: We no longer support ICMP ASIC-accelerated.
#if 0
		_rtl8651_refillNaptIcmpFlowOccupiedEntry(flowTblIdx, delEntry);
#else
		;
#endif
	else {
		/*
		 * Otherwise, the deleted entry is not written to ASIC. Beside remove it from
		 * internal structure, it is also needed to update the collison bit in the ASIC 
		 * table.
		 * NOTE:
		 *		In this case, we assume there is an entry in the ASIC table. Hence try
		 *		to read ASIC table and update its collison bit. If we find that the 
		 *		ASIC table entry is invalid, just do nothing. Because the maintenance
		 *		thread will take care of writting entry in the driver back to ASIC table.
		 */
		tmpDrvEntry = DrvTbl.icmp[flowTblIdx];
		while(tmpDrvEntry && tmpDrvEntry->toAsic != 1)
			tmpDrvEntry = tmpDrvEntry->next;

// yjlou note: We no longer support ICMP ASIC-accelerated.
#if 0
		convert_getAsicNaptIcmpTable(FALSE, flowTblIdx, &asicInsideLocalIp, &asicInsideLocalId, &asicInsideGlobalId, &asicAgeSec, &asicEntryType, &asicCount, &asicIsColl, &asicIsValid);
		if (asicIsValid == TRUE) {
			if (tmpDrvEntry)
				convert_setAsicNaptIcmpTable(TRUE, asicInsideLocalIp, asicInsideLocalId, asicInsideGlobalId, asicAgeSec, asicEntryType, asicCount, rtl8651NaptIcmpFlowNumberTable[flowTblIdx]> 1 || rtl8651NaptIcmpFlowNonAsicNumberTable[flowTblIdx]>0 || rtl8651NaptIcmpBackwardCollisionCountTable[flowTblIdx]>0? TRUE: FALSE, asicIsValid);
			else 
				convert_setAsicNaptIcmpTable(TRUE, asicInsideLocalIp, asicInsideLocalId, asicInsideGlobalId, asicAgeSec, asicEntryType, asicCount, rtl8651NaptIcmpFlowNumberTable[flowTblIdx]> 0 || rtl8651NaptIcmpFlowNonAsicNumberTable[flowTblIdx]>0 || rtl8651NaptIcmpBackwardCollisionCountTable[flowTblIdx]>0? TRUE: FALSE, asicIsValid);
		}			
#endif//0
	}
	if(delEntry->canAsic==0) {//This flow will backward collision other entry, process the collided flow entry
		assert(delEntry->toAsic == 0);
		flowTblIdx = delEntry->insideGlobalId & (RTL8651_ICMPTBL_SIZE-1);
		rtl8651NaptIcmpBackwardCollisionCountTable[flowTblIdx]--;

// yjlou note: We no longer support ICMP ASIC-accelerated.
#if 0
		if(rtl8651NaptIcmpBackwardCollisionCountTable[flowTblIdx] == 0) 
			_rtl8651_refillNaptIcmpFlowOccupiedEntry(flowTblIdx, NULL);
#endif
	}
	return SUCCESS;
}

/*
@func int32 | rtl8651_delNaptIcmpFlow | Remove a NAPT ICMP entry.
@parm ipaddr_t | insideLocalIpAddr | Specify local(private) IP address.
@parm uint16 | insideLocalId | Specify local ICMP ID.
@parm ipaddr_t | dstIpAddr | Destination IP address.
@rvalue SUCCESS | A NAPT ICMP entry was successfully removed.
@rvalue TBLDRV_ENOTINTIP | Specified internal IP address doesn't belong to internal interface.
@rvalue TBLDRV_EDSTIPISLS | NAPT DST IP is Local Server.
@rvalue TBLDRV_EENTRYNOTFOUND | The specified entry was not found.
*/
int32 rtl8651_delNaptIcmpFlow(ipaddr_t insideLocalIpAddr, uint16 insideLocalId, ipaddr_t dstIpAddr) {
	uint32 flowTblIdx;
	rtl8651_tblDrv_naptIcmpFlowEntry_t * deletedEntry, * preDelEntry;

	rtlglue_drvMutexLock();//Lock resource
	if(_rtl8651_intNetIpAddr(insideLocalIpAddr) == FALSE) {
		rtlglue_drvMutexUnlock();//Unlock resource
		return TBLDRV_ENOTINTIP;//Local ip address not internal IP address
	}
	if(_rtl8651_localServerIpAddr(dstIpAddr) == TRUE) {
		rtlglue_drvMutexUnlock();//Unlock resource
		return TBLDRV_EDSTIPISLS;
	}
	rtl8651_naptIcmpTableIndex(insideLocalIpAddr, insideLocalId, dstIpAddr, &flowTblIdx);
	
	deletedEntry = DrvTbl.icmp[flowTblIdx];
	for(preDelEntry=NULL; deletedEntry; preDelEntry=deletedEntry, deletedEntry=deletedEntry->next) {
		if (deletedEntry->fromAsic == 0 && deletedEntry->fromDrv == 0 &&
			deletedEntry->insideLocalIpAddr == insideLocalIpAddr &&
			deletedEntry->insideLocalId == insideLocalId &&
			deletedEntry->dstIpAddr == dstIpAddr)
				break;
	}
	if(deletedEntry == NULL) {
		rtlglue_drvMutexUnlock();//Unlock resource
		return TBLDRV_EENTRYNOTFOUND;//Sepcified entry not found
	}
	/*
	 * If the entry was already remove by protocol stack, just return FAILED
	 * to indicate it.
	 */
	if (deletedEntry->protoDel == 1) {
		rtlglue_drvMutexUnlock();//Unlock resource
		return TBLDRV_EENTRYNOTFOUND;//Entry to be deleted is not found
	}
	deletedEntry->protoDel = 1;
	//if (tempNaptIcmpFlowPtr->age == 0)
	//	_rtl8651_delNaptIcmpFlow(flowTblIdx, tempNaptIcmpFlowPtr);
	if (deletedEntry->age == 0) {
		/* Only remove the entry from driver table */
		if (preDelEntry == NULL)
			DrvTbl.icmp[flowTblIdx] = deletedEntry->next;
		else preDelEntry->next = deletedEntry->next;
		deletedEntry->next = DrvTbl.freeList.icmp;
		DrvTbl.freeList.icmp = deletedEntry;
		rtl8651ExistedNaptIcmpFlows --;
	}
	rtl8651_tblDrvIcmpCounter.protoDelCount++;
	rtlglue_drvMutexUnlock();//Unlock resource
	return SUCCESS;
}

//This API try to take advantage of ASIC acceleration capability
/*
@func void	| rtl8651_updateAsicIcmpTable	| Housekeeping function. Replace expired ASIC ICMP entries with software candidate flows. Also synchronize dynamic status information of each ASIC entry with software table.
@rdesc	<p rtl8651_updateAsicIcmpTable()> has no return value.
@comm 
This function must be called by maintenance thread to synchronize asic table entries with software table periodically,
If an ASIC entry is unused, select a candidate flow from software table to ASIC to accelerate forwarding spped of selected flow.
*/
void rtl8651_updateAsicIcmpTable(void) {
#if 0 //disable by tony: all the icmp entry is collision.
	rtl8651_tblDrv_naptIcmpFlowEntry_t *tmpNaptPtr, *delNaptPtr;
	static uint32 flowTblIdx = RTL8651_ICMPTBL_SIZE;
	uint32 asicAgeSec;
	ipaddr_t asicInsideLocalIp;
	uint16 asicInsideLocalId, asicInsideGlobalId, asicCount;
	int8 asicEntryType, asicIsColl, asicIsValid;

	rtlglue_drvMutexLock();//Lock resource
	flowTblIdx = ((flowTblIdx+1)&(RTL8651_ICMPTBL_SIZE-1));
	convert_getAsicNaptIcmpTable(FALSE, flowTblIdx, &asicInsideLocalIp, &asicInsideLocalId, &asicInsideGlobalId, &asicAgeSec, &asicEntryType, &asicCount, &asicIsColl, &asicIsValid);

	//If NAPT entry exists and the NAPT IP is used by PPPoE Session,
	//refresh the PPPoE Session aging time to avoid auto hang-up.
	if (asicIsValid == TRUE || DrvTbl.icmp[flowTblIdx]) {
		if (pppoeHangUp.pppoe_t)
		{
			softpppoeHangUp[pppoeHangUp.pppoe_t->objectId].last_received=_sysUpSeconds;						
		}			
	}
	
	/*
	 * If the ASIC table entry is invalid, try to write an entry in the driver 
	 * table to the ASIC table.
	 */
	if(asicIsColl == TRUE && asicIsValid == FALSE)
// yjlou note: We no longer support ICMP ASIC-accelerated.
#if 0
		_rtl8651_refillNaptIcmpFlowOccupiedEntry(flowTblIdx, NULL);
#else
	;
#endif
	else if (asicIsValid == TRUE) {
		tmpNaptPtr = DrvTbl.icmp[flowTblIdx];
		while(tmpNaptPtr) {
			if (tmpNaptPtr->toAsic == 1) {
				if (tmpNaptPtr->insideLocalIpAddr == asicInsideLocalIp &&
					tmpNaptPtr->insideLocalId == asicInsideLocalId &&
					(tmpNaptPtr->insideGlobalId&0xfc1f) == (asicInsideGlobalId&0xfc1f)) {
					/* entry match! SYNC! */
					tmpNaptPtr->age = asicAgeSec;
				} else {
					/* Otherwise, the entry in the ASIC is aging out */
					tmpNaptPtr->age = 0;
					delNaptPtr = tmpNaptPtr;
					tmpNaptPtr = tmpNaptPtr->next;
					_rtl8651_delNaptIcmpFlow(flowTblIdx, delNaptPtr);
					delNaptPtr->toAsic = 0;
					rtl8651_tblDrvIcmpCounter.syncFailure++;
					continue;
				}
			}
			tmpNaptPtr = tmpNaptPtr->next;
		}
	}
	rtlglue_drvMutexUnlock();//Unlock resource
#endif	
}


static void _rtl8651_timeUpdateNaptIcmpTable(uint32 secPassed) {
	uint32 flowTblIdx;
	rtl8651_tblDrv_naptIcmpFlowEntry_t * trackNaptIcmpFlowPtr, * deletedNaptIcmpFlowPtr;

	for(flowTblIdx = 0; flowTblIdx < RTL8651_ICMPTBL_SIZE; flowTblIdx++) {
		trackNaptIcmpFlowPtr = DrvTbl.icmp[flowTblIdx];
		while(trackNaptIcmpFlowPtr) {
			/* If the entry is already aging out, IGNORE! */
			//if (trackNaptIcmpFlowPtr->age == 0 && trackNaptIcmpFlowPtr->protoDel == 0) {
			if (trackNaptIcmpFlowPtr->alive == 0) {
				trackNaptIcmpFlowPtr = trackNaptIcmpFlowPtr->next;
				continue;
			}
			/* Entry is in aging state, COUNTING! */
			if (trackNaptIcmpFlowPtr->age > secPassed) {
				trackNaptIcmpFlowPtr->age -= secPassed;
				deletedNaptIcmpFlowPtr = NULL;
			} else {
				trackNaptIcmpFlowPtr->age = 0;
				deletedNaptIcmpFlowPtr = trackNaptIcmpFlowPtr;
			}
			trackNaptIcmpFlowPtr = trackNaptIcmpFlowPtr->next;
			if (deletedNaptIcmpFlowPtr) {
				//if (deletedNaptIcmpFlowPtr->protoDel == 1)
				//	_rtl8651_delNaptIcmpFlow(flowTblIdx, deletedNaptIcmpFlowPtr);	
				//else deletedNaptIcmpFlowPtr->toAsic = 0;
				_rtl8651_delNaptIcmpFlow(flowTblIdx, deletedNaptIcmpFlowPtr);
				deletedNaptIcmpFlowPtr->toAsic = 0;
				
			}
		}
	}
}



#define RTL8651_ACL_API
/*=======================================================================================
 * RTL8651 ACL APIs:
 *	1. Static Functions
 *		. _rtl8651_setAsicAclDefaultRule()
 *		. _rtl8651_tryAddServerPortToAcl()
 *		. _rtl8651_tryAddAlgToAcl()
 *		. _rtl8651_arrangeAllACL()
 *		. _rtl8651_sameAclRule()
 *	2. Internal APIs
 *		. _rtl8651_aclLookUp()
 *	3. External APIs
 *		. rtl8651_addAclRule()
 *		. rtl8651_delAclRule()
 *		. rtl8651_addAclRuleByIndexing()
 *		. rtl8651_delAclRuleByIndexing()
 *		. rtl8651_flushAclRule()
 *		. rtl8651_setDefaultAcl()
 *=======================================================================================*/
#define ACL_MODE_NONE				0x00
#define ACL_MODE_STACK				0x01
#define ACL_MODE_INDEXING			0x02

#define RTL8651_SET_DEFAULT_ACL_TOCPU(aclIdx) 		_rtl8651_setAsicAclDefaultRule((aclIdx), RTL8651_ACL_CPU)
#define RTL8651_SET_DEFAULT_ACL_DROPALL(aclIdx) 	_rtl8651_setAsicAclDefaultRule((aclIdx), RTL8651_ACL_DROP)
#define RTL8651_SET_DEFAULT_ACL_PERMITALL(aclIdx)   _rtl8651_setAsicAclDefaultRule((aclIdx), RTL8651_ACL_PERMIT)
#define RTL8651_TCP_ACL_RULE(tcpRule, action, sip, sipM, dip, dipM, TOS, tosM, flag, flagM,\
		spUB, spLB, dpUB, dpLB)\
{\
	(tcpRule)->actionType_      = action;\
	(tcpRule)->ruleType_        = RTL8651_ACL_TCP;\
	(tcpRule)->dstIpAddr_       = dip;\
	(tcpRule)->dstIpAddrMask_   = dipM;\
	(tcpRule)->srcIpAddr_       = sip;\
	(tcpRule)->srcIpAddrMask_   = sipM;\
	(tcpRule)->tos_             = TOS;\
	(tcpRule)->tosMask_         = tosM;\
	(tcpRule)->tcpFlag_	    = flag;\
	(tcpRule)->tcpFlagMask_	    = flagM;\
	(tcpRule)->tcpSrcPortUB_    = spUB;\
	(tcpRule)->tcpSrcPortLB_    = spLB;\
	(tcpRule)->tcpDstPortUB_    = dpUB;\
	(tcpRule)->tcpDstPortLB_    = dpLB;\
	}

#define RTL8651_UDP_ACL_RULE(udpRule, action, sip, sipM, dip, dipM, TOS, tosM,\
		spUB, spLB, dpUB, dpLB)\
{\
	(udpRule)->actionType_      = action;\
	(udpRule)->ruleType_        = RTL8651_ACL_UDP;\
	(udpRule)->dstIpAddr_       = dip;\
	(udpRule)->dstIpAddrMask_   = dipM;\
	(udpRule)->srcIpAddr_       = sip;\
	(udpRule)->srcIpAddrMask_   = sipM;\
	(udpRule)->tos_             = TOS;\
	(udpRule)->tosMask_         = tosM;\
	(udpRule)->udpSrcPortUB_    = spUB;\
	(udpRule)->udpSrcPortLB_    = spLB;\
	(udpRule)->udpDstPortUB_    = dpUB;\
	(udpRule)->udpDstPortLB_    = dpLB;\
}

#define RTL8651_ALG_TCPSERVER		0x01
#define RTL8651_ALG_TCPCLIENT		0x02
#define RTL8651_ALG_UDPSERVER		0x04
#define RTL8651_ALG_UDPCLIENT		0x08
#define RTL8651_SERVERPORT_TCP		0x01
#define RTL8651_SERVERPORT_UDP		0x02


static void _rtl8651_setAsicAclDefaultRule(uint32 aclIdx, int8 actionType) {
	_rtl8651_tblDrvAclRule_t rule;

	bzero(&rule, sizeof(_rtl8651_tblDrvAclRule_t));
	memset(&rule.dstMac_, 0, sizeof(ether_addr_t));
	memset(&rule.dstMacMask_, 0, sizeof(ether_addr_t));
	memset(&rule.srcMac_, 0, sizeof(ether_addr_t));
	memset(&rule.srcMacMask_, 0, sizeof(ether_addr_t));
	rule.typeLen_ = 0;
	rule.typeLenMask_ = 0;
	if (actionType == RTL8651_ACL_L34_DROP) {
		rule.actionType_ = RTL8651_ACL_PERMIT;
		rule.pktOpApp = RTL8651_ACLTBL_ONLY_L2;
	}else
		rule.actionType_ = actionType;
	rule.ruleType_ = RTL8651_ACL_MAC;
	rtl8651_setAsicAclRule(aclIdx, &rule);
}


static uint32 _rtl8651_acl_PPTP_L2TP(uint16 vid, uint32 aclStart, uint32 aclEnd)
{
	rtl8651_tblDrv_vlanTable_t *vlan_t;
	_rtl8651_tblDrvAclRule_t rule;
	ether_addr_t *mac, *macMask;
	int32 retval;
	uint32 count=0;


	if (aclStart > aclEnd || aclEnd>=RTL8651_ACLTBL_SIZE)	
		return 0;

	vlan_t = &DrvTbl.vlan[rtl8651_vlanTableIndex(vid)];	
	bzero(&rule, sizeof(_rtl8651_tblDrvAclRule_t));
	rule.ruleType_		= RTL8651_ACL_MAC;
	rule.actionType_	= RTL8651_ACL_CPU;
	if (vid == tunnel.loopBackVid) {
		/* The VLAN created by rome driver, add a acl to trap source MAC=gatewawy MAC 
		  * This virtual VLAN's mac is real WAN mac.
		  */
		mac = &rule.srcMac_;
		macMask = &rule.srcMacMask_;
	}
	else {
		/* real WAN */
		mac = &rule.dstMac_;
		macMask = &rule.dstMacMask_;
	}
	rtl8651_memcpy(mac, &vlan_t->macAddr, sizeof(ether_addr_t));
	memset(macMask, 0xff, sizeof(ether_addr_t));
	retval = rtl8651_setAsicAclRule(aclStart, &rule);
	assert(retval == SUCCESS);
	count ++;
	if (vid == tunnel.wanVid) 
	{
		RTL8651_SET_DEFAULT_ACL_PERMITALL(1);
		vlan_t->inAclStart = 0;
		vlan_t->inAclEnd = 1;
		RTL8651_SETASICVLAN(vlan_t);
		count ++;
	}
	return count;
}

/*  multicast packet will error in PPPoE interface (known chip version: 8650B-Ver.B), so we must trap it */
uint32 _rtl8651_addAclRuleToTrapPPPoEMulticastPkt( rtl8651_tblDrv_networkIntfTable_t *netif, uint32 aclStart, uint32 aclEnd)
{
	_rtl8651_tblDrvAclRule_t rule;

	assert((netif->linkLayerType == RTL8651_LL_PPPOE) || (netif->linkLayerType == RTL8651_LL_LIPUNNUMBERED));

	if (aclStart > aclEnd)
		return 0;  /* we don't add any acl rule */

	/* trap all dip==multicast ip (class D) packet to CPU */
	memset(&rule, 0, sizeof(_rtl8651_tblDrvAclRule_t));
	RTL8651_IP_ACL_RULE(&rule, RTL8651_ACL_CPU, 0, 0, 0xe0000000, 0xf0000000, 0, 0, 0, 0, 0, 0);
	rtl8651_setAsicAclRule(aclStart, &rule);
	return 1; /* we add 1 acl rule */

}


static uint32 _rtl8651_tryAddAlgToAcl(uint16 vid, uint32 aclStart, uint32 aclEnd) {
	rtl8651_tblDrv_algEntry_t *trackAlgPtr;
	rtl8651_tblDrv_vlanTable_t *vlanPtr;
	_rtl8651_tblDrvAclRule_t rule;
	uint16 _spUB, _spLB, _dpUB, _dpLB;
	uint32 servFlag, count = 0;

	assert(aclEnd < RTL8651_ACLTBL_SIZE);
	if (aclStart > aclEnd)	return 0;
	vlanPtr = &DrvTbl.vlan[rtl8651_vlanTableIndex(vid)];
	SLIST_FOREACH(trackAlgPtr, &DrvTbl.inuseList.alg, nextAlgEntry) {
		if (trackAlgPtr->toAsic == 1)
			continue;
			
		servFlag = 0;
		if (trackAlgPtr->tcpServer == 1)	servFlag |= RTL8651_ALG_TCPSERVER;
		if (trackAlgPtr->tcpClient == 1)	servFlag |= RTL8651_ALG_TCPCLIENT;
		if (trackAlgPtr->udpServer == 1)	servFlag |= RTL8651_ALG_UDPSERVER;
		if (trackAlgPtr->udpClient == 1)	servFlag |= RTL8651_ALG_UDPCLIENT;
		while (servFlag) {
			if(servFlag & (RTL8651_ALG_TCPSERVER | RTL8651_ALG_TCPCLIENT)) {
				if(	((servFlag & RTL8651_ALG_TCPSERVER) && vlanPtr->internal == 1) || 
					((servFlag & RTL8651_ALG_TCPCLIENT) && vlanPtr->internal == 0)){
					_spUB = trackAlgPtr->endPort;
					_spLB = trackAlgPtr->startPort;
					_dpUB = 0xffff;
					_dpLB = 0x0;
				} else {
					_spUB = 0xffff;
					_spLB = 0x0;
					_dpUB = trackAlgPtr->endPort;
					_dpLB = trackAlgPtr->startPort;
				}
				RTL8651_TCP_ACL_RULE(
					&rule, RTL8651_ACL_CPU, 0, 0, 0, 0, 0x0, 0, 0, 0,
					_spUB,
					_spLB,
					_dpUB,
					_dpLB
				);
				if(servFlag & RTL8651_ALG_TCPSERVER)
					servFlag &= ~RTL8651_ALG_TCPSERVER;
				else servFlag &= ~RTL8651_ALG_TCPCLIENT;
			}
			else {
				assert(servFlag & (RTL8651_ALG_UDPSERVER | RTL8651_ALG_UDPCLIENT));
				if(	((servFlag & RTL8651_ALG_UDPSERVER) && vlanPtr->internal == 1) || 
					((servFlag & RTL8651_ALG_UDPCLIENT) && vlanPtr->internal == 0)){
					_spUB = trackAlgPtr->endPort;
					_spLB = trackAlgPtr->startPort;
					_dpUB = 0xffff;
					_dpLB = 0x0;
				} else {
					_spUB = 0xffff;
					_spLB = 0x0;
					_dpUB = trackAlgPtr->endPort;
					_dpLB = trackAlgPtr->startPort;
				}
				RTL8651_UDP_ACL_RULE(
					&rule, RTL8651_ACL_CPU, 0, 0, 0, 0, 0, 0,
					_spUB,
					_spLB,
					_dpUB,
					_dpLB
				);
				if(servFlag & RTL8651_ALG_UDPSERVER)
					servFlag &= ~RTL8651_ALG_UDPSERVER;
				else servFlag &= ~RTL8651_ALG_UDPCLIENT;
			}
			rtl8651_setAsicAclRule(aclStart, &rule);
			aclStart++; count++;
			if (aclStart > aclEnd)
				return count;
		}
	}
	return count;
}

static uint32 _rtl8651_addAclForLanSideIpUnMacFilter(int8 isAdd, rtl8651_tblDrv_networkIntfTable_t *netif_t, uint32 aclStart, uint32 aclEnd)
{
	uint32 counter = 0;
	_rtl8651_tblDrvAclRule_t aclRule;
	rtl8651_tblDrv_arpEntry_t *arp_t;
	rtl8651_tblDrv_macACEntry_t *mfRule;

	assert(aclEnd < RTL8651_ACLTBL_SIZE);

	if (aclStart > aclEnd)
		return 0;

	/* If this function is disabled, always return 0 */
	if (DrvTbl.macACInfo->enable == FALSE)
		return 0;
		
	CTAILQ_FOREACH(mfRule, &DrvTbl.macACInfo->macAccessControl, nextMAC) {

		ether_addr_t macMask = { { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff } };

#if 0	/* debug */
		rtlglue_printf("========= found a mac filter rule....mac: %x.%x.%x.%x.%x.%x\n",
			mfRule->mac.octet[0], mfRule->mac.octet[1], mfRule->mac.octet[2],
			mfRule->mac.octet[3], mfRule->mac.octet[4], mfRule->mac.octet[5]);
#endif
		if (memcmp(&(mfRule->macMask), &macMask, sizeof(ether_addr_t)) == 0) {	/* Currently, only support single mac address. */
			
			arp_t = _rtl8651_getArpEntryByMac(NULL, &(mfRule->mac), FALSE);
			
			/* If arp lookup successful and ASIC ip table has the corresponding LS entry */
			if (arp_t && (_rtl8651_getIPTableEntry(arp_t->ipAddr, 0, IP_ATTR_LS) != NULL)) {
				counter++;

				if (isAdd == TRUE) {
					/* set a source filter ACL rule */
					bzero(&aclRule, sizeof(_rtl8651_tblDrvAclRule_t));
					memcpy(&(aclRule.srcFilterMac_), &(mfRule->mac), sizeof(ether_addr_t));
					memset(&(aclRule.srcFilterMacMask_), 0xff, 6);
					aclRule.srcFilterIgnoreL3L4_ = TRUE;
					aclRule.ruleType_ = RTL8651_ACL_SRCFILTER;
					aclRule.actionType_ = mfRule->actionType;
					rtl8651_setAsicAclRule(aclStart, &aclRule);
				}
			}
		}
	}
	return counter;

}

static uint32 _rtl8651_addAclForIpRangeRule(int8 isAdd, int8 insuff,_rtl8651_tblDrvAclRule_t *inputRule, uint32 aclStart, uint32 aclEnd)
{
	int i, msbDiff, srcIpRange;
	uint32 counter = 0;
	ipaddr_t lowIp, highIp, tmpIp, bitString;
	_rtl8651_tblDrvAclRule_t rule;

	assert(aclEnd < RTL8651_ACLTBL_SIZE);
	if (aclStart > aclEnd)
		return 0;
	
	rtl8651_memcpy(&rule, inputRule, sizeof(_rtl8651_tblDrvAclRule_t));
	
	/* Pre-process: find out the valid ip range. */
	if (inputRule->srcIpAddrStart_ == 0 && inputRule->srcIpAddrEnd_ == 0) {
		lowIp = inputRule->dstIpAddrStart_;
		highIp = inputRule->dstIpAddrEnd_;
		srcIpRange = FALSE;
	}
	else {
		lowIp = inputRule->srcIpAddrStart_;
		highIp = inputRule->srcIpAddrEnd_;
		srcIpRange = TRUE;
	}
	if ((lowIp == 0 && highIp == 0) || (lowIp > highIp))	// No acl rule will be added.
		return 0;

	/* 	
		Convert ip range to subnet
	    	(1) find out the max significant bit of difference of lowIp and highIp. 
	*/
	bitString = lowIp ^ highIp;
	for (i = 31; i > 0 && (bitString >> i) == 0; i--);
	msbDiff = i;

	if (insuff) {
		if (isAdd == TRUE && srcIpRange == TRUE) {
			bzero(&rule, sizeof(_rtl8651_tblDrvAclRule_t));
			rule.srcIpAddrMask_ = convPrefix(32 - (msbDiff + 1));
			rule.srcIpAddr_ = (lowIp & rule.srcIpAddrMask_);
			rule.ruleType_ = RTL8651_ACL_IP;
			rule.actionType_ = RTL8651_ACL_CPU;
			rtl8651_setAsicAclRule(aclStart, &rule);
		}
		else if (isAdd == TRUE) {
			bzero(&rule, sizeof(_rtl8651_tblDrvAclRule_t));
			rule.dstIpAddrMask_ = convPrefix(32 - (msbDiff + 1));
			rule.dstIpAddr_ = (lowIp & rule.dstIpAddrMask_);
			rule.ruleType_ = RTL8651_ACL_IP;
			rule.actionType_ = RTL8651_ACL_CPU;
			rtl8651_setAsicAclRule(aclStart, &rule);
		}
		
		return 1;
	}

	/* (2) Scan low ip bit string from LSB to calculate the subnets. */
	while(1) {
		if (isAdd == TRUE && aclStart > aclEnd)
			return counter;
		for (i = 0; i <= msbDiff && ((lowIp >> i) & 0x01) == 0; i++);
		if (i == msbDiff)	// don't handle this case, leave it to "high ip bit string scan process"
			break;
		if (i == msbDiff + 1) {	// special case: check if this ip range is a "complete subnet" or not
			if (lowIp - 1 + (0x01 << i) == highIp) {	// yes, it is a complete subnet
				if (isAdd == TRUE && srcIpRange == TRUE) {
					rule.srcIpAddr_ = lowIp;
					rule.srcIpAddrMask_ = convPrefix(32 - i);
					rtl8651_setAsicAclRule(aclStart, &rule);
				}
				else if (isAdd == TRUE) {
					rule.dstIpAddr_ = lowIp;
					rule.dstIpAddrMask_ = convPrefix(32 - i);
					rtl8651_setAsicAclRule(aclStart, &rule);
				}
				aclStart++;				
			}
			else {	// no, it is not a complete subnet.
				if (isAdd == TRUE && srcIpRange == TRUE) {
					rule.srcIpAddr_ = lowIp;
					rule.srcIpAddrMask_ = convPrefix(32 - i + 1);
					rtl8651_setAsicAclRule(aclStart, &rule);
				}
				else if (isAdd == TRUE) {
					rule.dstIpAddr_ = lowIp;
					rule.dstIpAddrMask_ = convPrefix(32 - i + 1);
					rtl8651_setAsicAclRule(aclStart, &rule);
				}
				aclStart++;
			}
			counter++;
			break;
		}
		if (isAdd == TRUE && srcIpRange == TRUE) {
			rule.srcIpAddr_ = lowIp;
			rule.srcIpAddrMask_ = convPrefix(32 - i);
			rtl8651_setAsicAclRule(aclStart, &rule);
		}
		else if (isAdd == TRUE) {
			rule.dstIpAddr_ = lowIp;
			rule.dstIpAddrMask_ = convPrefix(32 - i);
			rtl8651_setAsicAclRule(aclStart, &rule);
		}
		aclStart++;
		counter++;
		lowIp = lowIp + (0x01 << i);
	}

	/* (3) Scan highIp bit string from LSB to calculate the subnets. */
	while(1) {
		if (isAdd == TRUE && aclStart > aclEnd)
			return counter;
		for (i = 0; i <= msbDiff && ((highIp >> i) & 0x01) == 1; i++);
		if (i == msbDiff)	// don't handle this case, "low ip bit string scan process" has taken it over.
         		break;
      		if (i == msbDiff + 1) { // special case: check if this ip range is a "complete subnet" or not
         		if (highIp + 1 - (0x01 << i) == lowIp);	// yes, it is a complete subnet. but "low ip bit string scan process has taken it over.
         		else {								// no, it is not a complete subnet.
            			tmpIp = highIp - (0x01 << (i - 1)) + 1;
				if (isAdd == TRUE && srcIpRange == TRUE) {
					rule.srcIpAddr_ = tmpIp;
					rule.srcIpAddrMask_ = convPrefix(32 - i + 1);
					rtl8651_setAsicAclRule(aclStart, &rule);
				}
				else if (isAdd == TRUE) {
					rule.dstIpAddr_ = tmpIp;
					rule.dstIpAddrMask_ = convPrefix(32 - i + 1);
					rtl8651_setAsicAclRule(aclStart, &rule);
				}
				aclStart++;
				counter++;
			}
         		break;
      		}
      		tmpIp = highIp - (0x01 << i) + 1;
		if (isAdd == TRUE && srcIpRange == TRUE) {
			rule.srcIpAddr_ = tmpIp;
			rule.srcIpAddrMask_ = convPrefix(32 - i);
			rtl8651_setAsicAclRule(aclStart, &rule);
		}
		else if (isAdd == TRUE) {
			rule.dstIpAddr_ = tmpIp;
			rule.dstIpAddrMask_ = convPrefix(32 - i);
			rtl8651_setAsicAclRule(aclStart, &rule);
		}
		aclStart++;
		counter++;
      		highIp = highIp - (0x01 << i);
	}

	return counter;
}

static int32 _rtl8651_addAclForSrcMacDstPortRule(rtl8651_tblDrv_networkIntfTable_t *netif_t, _rtl8651_tblDrvAclRule_t *inputRule, uint32 aclStart)
{
	rtl8651_tblDrv_arpEntry_t *arp_t;
	_rtl8651_tblDrvAclRule_t rule;
	ether_addr_t macAddr, zero_mac, max_mac;
	int8 isTcp;

	memset(&zero_mac, 0, sizeof(ether_addr_t));
	memset(&max_mac, 0xff, sizeof(ether_addr_t));
	memset(&rule, 0, sizeof(_rtl8651_tblDrvAclRule_t));
#if 0
	printk("%x.%x.%x.%x.%x.%x\n", inputRule->un_ty.L3L4.is.tcp._l2srcMac.octet[0], 
		inputRule->un_ty.L3L4.is.tcp._l2srcMac.octet[1], 
		inputRule->un_ty.L3L4.is.tcp._l2srcMac.octet[2],
		inputRule->un_ty.L3L4.is.tcp._l2srcMac.octet[3],
		inputRule->un_ty.L3L4.is.tcp._l2srcMac.octet[4],
		inputRule->un_ty.L3L4.is.tcp._l2srcMac.octet[5]);
	printk("memcmp: %d\n", memcmp(&inputRule->tcpl2srcMac_, &zero_mac, sizeof(ether_addr_t)));
#endif

	if (inputRule->ruleType_ == RTL8651_ACL_TCP && memcmp(&inputRule->tcpl2srcMac_, &zero_mac, sizeof(ether_addr_t)) && inputRule->tcpDstPortUB_ != 0 && inputRule->tcpDstPortLB_ != 0)
		isTcp = TRUE;
	else if (inputRule->ruleType_ == RTL8651_ACL_UDP && memcmp(&inputRule->udpl2srcMac_, &zero_mac, sizeof(ether_addr_t)) && inputRule->udpDstPortUB_ != 0 && inputRule->udpDstPortLB_ != 0)
		isTcp = FALSE;
	else
		return FAILED;
	
	/* Lookup ARP table to search the src IP of this src Mac*/
	macAddr = (isTcp ? inputRule->tcpl2srcMac_ : inputRule->udpl2srcMac_);
	arp_t = _rtl8651_getArpEntryByMac(netif_t, &macAddr, FALSE);

	/* If arplookup successful */
	if (arp_t) {
		//printk("arp_t->ipAddr: %u.%u.%u.%u\n", NIPQUAD(arp_t->ipAddr));
		memset(&rule, 0, sizeof(_rtl8651_tblDrvAclRule_t));
		rule.ruleType_     	= inputRule->ruleType_;		// RTL8651_ACL_TCP or RTL8651_ACL_UDP
	 	rule.actionType_   	= inputRule->actionType_;
		rule.srcIpAddr_	= arp_t->ipAddr;				// The ip address we lookup in S/W ARP table.
		rule.srcIpAddrMask_	= 0xffffffff;
		if (isTcp) {
			rule.tcpSrcPortUB_	= inputRule->tcpSrcPortUB_;
			rule.tcpSrcPortLB_	= inputRule->tcpSrcPortLB_;
		 	rule.tcpDstPortUB_ = inputRule->tcpDstPortUB_;
	 		rule.tcpDstPortLB_ 	= inputRule->tcpDstPortLB_;			
		}
		else {
			rule.udpSrcPortUB_ = inputRule->udpSrcPortUB_;
			rule.udpSrcPortLB_	 = inputRule->udpSrcPortLB_;
			rule.udpDstPortUB_ = inputRule->udpDstPortUB_;
			rule.udpDstPortLB_	 = inputRule->udpDstPortLB_;
		}
	 	rtl8651_setAsicAclRule(aclStart, &rule);
	}
	else {	/* If arp lookup failed */
		memset(&rule, 0, sizeof(_rtl8651_tblDrvAclRule_t));
		RTL8651_MAC_ACL_RULE(&rule, RTL8651_ACL_CPU, &macAddr, &max_mac, &zero_mac, &zero_mac, 0, 0);
		rtl8651_setAsicAclRule(aclStart, &rule);
	}
	return SUCCESS;
}


static void _rtl8651_addL2PermitRule(uint32 aclIdx)
{
	_rtl8651_tblDrvAclRule_t rule;

	bzero(&rule, sizeof(_rtl8651_tblDrvAclRule_t));
	rule.actionType_ = RTL8651_ACL_PERMIT;
	rule.pktOpApp = RTL8651_ACLTBL_ONLY_L2;
	rule.ruleType_ = RTL8651_ACL_MAC;
	rtl8651_setAsicAclRule(aclIdx, &rule);
}

void _rtl8651_arrangeAllACL(void) {
	rtl8651_tblDrv_vlanTable_t *vlanPtr;
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	_rtl8651_tblDrvAclRule_t *trackAcl;
	uint32 i , algCount=0, aclIndex = 0, addCount=0, patchCount=0, natCount=0, dosCount=0, ipRangeCount = 0, mfCount = 0;
	uint32 inAclStart, inAclEnd, outAclStart, outAclEnd, inCount, outCount;
	#if 0	// implementation of ip range ACL has been modified.
	int8 vlanIpRangeRuleIngressExist[RTL8651_VLAN_NUMBER];
	int8 vlanIpRangeRuleEgressExist[RTL8651_VLAN_NUMBER];
	#endif

	/*   This flag is intended to be passed into the function which adds ACL rules without estimating. 
	  *   If the function adds ACL rules successfully, this flag will be assigned to '1'.
	  *   Otherwise, this flag will be assigned to '0'.
	  *   For example:
	  *   If _rtl8651_addAclRuleForSipBlock() wants to add 10 ACL rules, but it failed adding 10 ACL rules.
	  *   (Maybe it just adds 8 ACL rules successfully) 
	  *   This flag will be assigned to '0'.
	  */
	int8 enoughFlag;



	/* Initial Phase */

#if 0	// implementation of ip range ACL has been modified.
	for(i=0;i<RTL8651_VLAN_NUMBER;i++)
	{
		vlanIpRangeRuleIngressExist[i] = 0;
		vlanIpRangeRuleEgressExist[i] = 0;
	}
#endif
#ifdef RTL865XB_ACLDB
	/* always clear ACL DB */
	_rtl8651_initACLDB();
#endif/*RTL865XB_ACLDB*/

	netIfPtr = &DrvTbl.netIface[0];
	for(i=0; i<rtl8651_tblDrvPara.networkIntfTableSize; i++,netIfPtr++) {
		/*
		  *	ASIC ACL indexing: Before setting acl index, initiate it with "RTL8651_ACLTBL_SIZE+RTL8651_ACLTBL_RESERV_SIZE"
		  *   and make sure it will be checked in software even the ASIC's reason bit is applied.
		  */
		CTAILQ_FOREACH(trackAcl, &netIfPtr->inAclHead, nextRule)
		{
			trackAcl->aclIdx = RTL8651_ACLTBL_SIZE+RTL8651_ACLTBL_RESERV_SIZE;
		}
	}

	{
		/* init total ACL budget for unnumber patch */
		rtl8651_tblDrv_ipUnnumbered_t *ipUnnumber_t;

		rtl8651_ipUnnumberGlobalCtrlBlock.ip_unnumber_ACL_budget = IP_UNNUMBER_MAX_ASIC_ACL_PATCH;

		SLIST_FOREACH(ipUnnumber_t, &DrvTbl.inuseList.ipUnnumber, nextIpUnnumberEntry)
			ipUnnumber_t->ip_as_ACL_budget = 0;
	}

	/* PPTP/L2TP patch */
	if (tunnel.valid == 1)
		aclIndex = _rtl8651_acl_PPTP_L2TP(tunnel.wanVid, 0, RTL8651_ACLTBL_SIZE-1);


	/*
	 * Start to fill ACL rules to ASIC ACL Table:
	 *    1. If WLAN acceleration is enabled, add all wireless stations's MAC to ACL.
	 *    2. Add PPTP/L2TP rules	 
	 *    3. Add PPPoE Passthru rules
	 *    4. Add DoS rules
	 * 	5. ALG 
	 *    6. Server Port has high priority if they need to be added to ACL
	 *    7. Lan side IP unnumbered rules
	 *    8. The user-add ACL rule has middle priority
	 *    9. The system-add ACL rule has the lowest priority
	 */
	netIfPtr = &DrvTbl.netIface[0];
	for(i=0; i<rtl8651_tblDrvPara.networkIntfTableSize; i++,netIfPtr++) {
		if (netIfPtr->valid == 0 || netIfPtr->linkLayerType == RTL8651_LL_NONE)
			continue;

             if ( tunnel.valid == 1 && netIfPtr->vid == tunnel.wanVid )
             {
             	vlanPtr = &DrvTbl.vlan[(tunnel.loopBackVid& (8 -1))];
		}
		else
		{  
			vlanPtr = &DrvTbl.vlan[rtl8651_vlanTableIndex(netIfPtr->vid)];
		}		
		inCount = outCount = 0;
		inAclStart = inAclEnd = outAclStart = outAclEnd = 0;

		inAclStart = aclIndex; 


                if (tunnel.valid == 1 && netIfPtr->vid == tunnel.wanVid)
                {
                        addCount = _rtl8651_acl_PPTP_L2TP(vlanPtr->vid , aclIndex, RTL8651_ACLTBL_SIZE - 3);
                        aclIndex += addCount;
                        inCount += addCount;
                        if (addCount < 1)
                                goto noAclEntry_forIngress_;
                }
		

		/* 
		  * (0) Rate Limit rule ALWAYS in the head 
		  *      - Currently, we ignore table overflow issue. If no empty acl entry to add new rate limit rule (table overflow),
		  *	     the added rate limit rule would not work. We do not want to solve this issue because it will cause software
		  *	     overhead and assume that the table overflow would not occur.
		  *      - The software do not take care of rate limit, it is hardware's duty. Hence for those packets which
		  *	     are trapped to CPU to work correctly, we put all rate limit rules in the head of each network interface.
		  *	   - Note: For Counter mode, the rate limit rule need to put in the Egress Rule instead of HERE.
		  *
		  *
		  */
		{
			CTAILQ_FOREACH(trackAcl, &netIfPtr->inRateLimitHead, nextRule) {
				uint32 pktOp;
				/*   When we want to add a new rule to "RTL8651_ACLTBL_SIZE - 2" entry,
				  *   it means that this new rule can't be added in ASIC ACL table due to insufficient space.
				  *   Therefore, we leave it in the software, and 
				  *   (1) add a "L2 permit" ACL rule.
				  *   (2) add an ACL rule to let all pkts to CPU.
				  */
				if (aclIndex >= (RTL8651_ACLTBL_SIZE - 2))
					goto noAclEntry_forIngress_;

				pktOp = trackAcl->pktOpApp;
				trackAcl->pktOpApp = RTL8651_ACLTBL_L3_AND_L4;
				rtl8651_setAsicAclRule(aclIndex, trackAcl);
				trackAcl->pktOpApp = pktOp;								
				aclIndex++; inCount++;
			}

			/* For M-N Queue Model */
			CTAILQ_FOREACH(trackAcl, &netIfPtr->mnqHead, nextRule) {
				uint32 pktOp;
								
				if (aclIndex >= (RTL8651_ACLTBL_SIZE - 2))
					goto noAclEntry_forIngress_;
					
				pktOp = trackAcl->pktOpApp;
				trackAcl->pktOpApp = RTL8651_ACLTBL_L3_AND_L4;
				rtl8651_setAsicAclRule(aclIndex, trackAcl);
				trackAcl->pktOpApp = pktOp;
				aclIndex++; inCount++;
			}

		}

#ifdef CONFIG_RTL865X_IPSEC
		{
			if(vlanPtr->internal)
			{
				enoughFlag = 1;	// Assume there's enough space for adding SipBlock ACL rules.
				/*   trap outbound traffic at internal VLAN 
				  *   Parameter "RTL8651_ACLTBL_SIZE - 3" means that this function cannot add over the position "RTL8651_ACLTBL_SIZE - 3".
				  *   Position "RTL8651_ACLTBL_SIZE - 2" is used for L2 permit rule.
				  *   Position "RTL8651_ACLTBL_SIZE - 1" is used for all pkts trapping to CPU rule.
				  */				
				addCount = _rtl8651_tryToAddAclRuleForIpsecOutbound(vlanPtr->vid, aclIndex, RTL8651_ACLTBL_SIZE - 3, &enoughFlag);				
			}
			else
			{
				enoughFlag = 1;	// Assume there's enough space for adding SipBlock ACL rules.
				/* trap inbound traffic at external VLAN */
				addCount = _rtl8651_tryToAddAclRuleForIpsecInbound(vlanPtr->vid, aclIndex, RTL8651_ACLTBL_SIZE - 3, &enoughFlag);
			}
			aclIndex += addCount;
			inCount += addCount;
			if (enoughFlag == 0)	// Not enough space for adding IPSec ACL rules.
				goto noAclEntry_forIngress_;			
		}
#endif/*CONFIG_RTL865X_IPSEC*/

		{
			if (aclIndex >= (RTL8651_ACLTBL_SIZE - 2))
				goto noAclEntry_forIngress_;

			enoughFlag = 1;	// Assume there's enough space for adding SipBlock ACL rules.
			addCount = _rtl8651_tryToAddAclRuleForDns(aclIndex, RTL8651_ACLTBL_SIZE - 3, &enoughFlag);
			aclIndex += addCount;
			inCount += addCount;
			if (enoughFlag == 0)	// Not enough space for adding IPSec ACL rules.
				goto noAclEntry_forIngress_;
		}

#ifdef RTL865XB_URL_FILTER
		if(_urlFilter.urlFilterCount >= 0){
			if(vlanPtr->internal){
				_rtl8651_tblDrvAclRule_t rule;

				if (aclIndex >= (RTL8651_ACLTBL_SIZE - 2))
					goto noAclEntry_forIngress_;

				bzero(&rule, sizeof(_rtl8651_tblDrvAclRule_t));
				rule.actionType_ = RTL8651_ACL_CPU;
				rule.ruleType_ = RTL8651_ACL_IP;
				rule.ipHttpFilter_=rule.ipHttpFilterM_=1;
 				rule.pktOpApp = RTL8651_ACLTBL_ONLY_L4;
				rtl8651_setAsicAclRule(aclIndex, &rule);
				_urlFilterAclRuleNum=aclIndex;
				aclIndex++; inCount++;
			}
		}else
			_urlFilterAclRuleNum=-1;

#endif

		/* <------------------- drop case -------------------> */
		/* (1) source ip blocking */
		{
			enoughFlag = 1;	// Assume there's enough space for adding SipBlock ACL rules.
			addCount=_rtl8651_addAclRuleForSipBlock(vlanPtr->vid, aclIndex, RTL8651_ACLTBL_SIZE - 3, &enoughFlag);
			aclIndex += addCount;
			
			inCount += addCount;
			if (enoughFlag == 0)	// Not enough space for adding SipBlock ACL rules.
				goto noAclEntry_forIngress_;			
		}

		/* (2) domain blocking */
		{
			enoughFlag = 1;	/* Assume there's enough space for adding SipBlock ACL rules. */
			addCount = _rtl8651_tryToAddAclRuleForDnsDomainBlocking(vlanPtr, aclIndex, RTL8651_ACLTBL_SIZE-3, &enoughFlag);
			aclIndex += addCount;

			inCount += addCount;
			if (enoughFlag == 0)	/* Not enough space for adding SipBlock ACL rules. */
				goto noAclEntry_forIngress_;
		}


#if 1//CONFIG_RTL865XB_EXP_PPPOE_PASSTHRU
		/*
		 *  FOR CDRouter
		 *  If we want to drop unknown PPPoE Discovery message, we must add ACL to trap.
		 */
		if ( ( rtl8651_l2protoPassThrough & PASSTHRU_PPPOE ) &&
		     rtl8651_drvDropUnknownPppoePADT )
		{
			_rtl8651_tblDrvAclRule_t rule;
			int32 retval;

			if (aclIndex >= (RTL8651_ACLTBL_SIZE - 2))
				goto noAclEntry_forIngress_;

			// add ACL to trap 0x8863 in every VLAN
			bzero(&rule, sizeof(_rtl8651_tblDrvAclRule_t));
			rule.ruleType_              = RTL8651_ACL_MAC;
			rule.actionType_            = RTL8651_ACL_CPU;
			rule.un_ty.MAC._typeLen     = 0x8863;
			rule.un_ty.MAC._typeLenMask = 0xffff;

			retval = rtl8651_setAsicAclRule(aclIndex, &rule);
			if (retval == SUCCESS)
			{
				aclIndex += 1;
				inCount += 1;
			}
		}
#else//CONFIG_RTL865XB
		/* (4) add patch rule for PPPoE passthru */
		if ( rtl8651_l2protoPassThrough & PASSTHRU_PPPOE )
		{
			int ExpectedCount = 2; // 0x8863 and 0x8864
			
			addCount = _rtl8651_acl_PPPoE_Passthru( aclIndex, RTL8651_ACLTBL_SIZE-1, netIfPtr );
			aclIndex += addCount;
			inCount += addCount;
			passthruCount += addCount;
			if ( addCount < ExpectedCount ) 
			{
				RTL8651_SET_DEFAULT_ACL_TOCPU(RTL8651_ACLTBL_SIZE-1);
				inAclEnd = RTL8651_ACLTBL_SIZE;
				goto noAclEntry__;
			}
		}
#endif//CONFIG_RTL865XB


#if 1//CONFIG_RTL865XB_EXP_IPV6_PASSTHRU
#else//CONFIG_RTL865XB
		/* (4.5) add patch rule for IPv6 passthru */
		if ( rtl8651_drvIpv6PassthruEnable )
		{
			int ExpectedCount = 1; // this is 0x86DD.
			
			addCount = _rtl8651_acl_IPv6_Passthru( aclIndex, RTL8651_ACLTBL_SIZE-1, netIfPtr );
			aclIndex += addCount;
			inCount += addCount;
			passthruCount += addCount;
			if ( addCount < ExpectedCount ) 
			{
				RTL8651_SET_DEFAULT_ACL_TOCPU(RTL8651_ACLTBL_SIZE-1);
				inAclEnd = RTL8651_ACLTBL_SIZE;
				goto noAclEntry__;
			}
		}
#endif//CONFIG_RTL865XB


		/* (5) DoS patch rule */
		dosCount = _rtl8651_addAclRuleForDoS(netIfPtr->vid, 0, RTL8651_ACLTBL_SIZE - 3, FALSE);
		addCount = _rtl8651_addAclRuleForDoS(netIfPtr->vid, aclIndex, RTL8651_ACLTBL_SIZE - 3, TRUE);
		aclIndex += addCount;
		
		inCount += addCount;
		assert(addCount <= dosCount);
		assert(aclIndex <= (RTL8651_ACLTBL_SIZE - 2));
		if (addCount < dosCount)
			goto noAclEntry_forIngress_;

		
		/* (6) Fill ALG to Ingress rule */
		algCount = _rtl8651_algToAclCount();
		addCount = _rtl8651_tryAddAlgToAcl(netIfPtr->vid, aclIndex, RTL8651_ACLTBL_SIZE - 3);
		aclIndex += addCount; 
		
		inCount += addCount;
		assert(addCount <= algCount);
		assert(aclIndex <= (RTL8651_ACLTBL_SIZE - 2));
		if (addCount < algCount)
			goto noAclEntry_forIngress_;

#if 1
		/* (7) add ACL to trap soft Napt IP: note we assume one WAN and one LAN configuration */
		if (vlanPtr->internal == 0 && softNaptIp) {
			addCount = _rtl8651_addSoftNaptToAcl(aclIndex, RTL8651_ACLTBL_SIZE - 3);
			aclIndex += addCount;
			
			inCount += addCount;
			assert(addCount <= 1);
			if (addCount < 1) {
				goto noAclEntry_forIngress_;
			}
		}

		/* (7) Policy Route */
		if (vlanPtr->internal == 1 && TAILQ_FIRST(&DrvTbl.inuseList.policyRoute)) { /* Only apply to internal VLAN */
			enoughFlag = 1;	// Assume there's enough space for adding policy route ACL rules.
			addCount = _rtl8651_addPolicyRouteToAcl(aclIndex, RTL8651_ACLTBL_SIZE - 3, &enoughFlag);
			aclIndex += addCount;
			inCount += addCount;
			if (enoughFlag == 0)	// Not enough space for adding ACL rules.
				goto noAclEntry_forIngress_;
		}

#endif
		/* (7.5) pactch: trap multicast packet for PPPoE/Ip-unnumbered interface */
		if ((netIfPtr->linkLayerType == RTL8651_LL_PPPOE) || (netIfPtr->linkLayerType == RTL8651_LL_LIPUNNUMBERED))
		{
			addCount = _rtl8651_addAclRuleToTrapPPPoEMulticastPkt(netIfPtr, aclIndex, RTL8651_ACLTBL_SIZE - 3);
			aclIndex += addCount;
			inCount += addCount;
			assert(aclIndex <= (RTL8651_ACLTBL_SIZE - 2));
			if (addCount < 1)
				goto noAclEntry_forIngress_;
		}

		/* (8) Add ACL rule to patch Lan Side IP-Unnumbered */
		if (netIfPtr->linkLayerType == RTL8651_LL_LIPUNNUMBERED && (lanSideIpUnnumberedLan == netIfPtr || lanSideIpUnnumberedWan == netIfPtr)) {
			natCount = _rtl8651_addAclForLanSideIpUnnumbered(FALSE, netIfPtr, aclIndex, RTL8651_ACLTBL_SIZE-3);
			if (natCount > 0) {
				addCount = _rtl8651_addAclForLanSideIpUnnumbered(TRUE, netIfPtr, aclIndex, RTL8651_ACLTBL_SIZE-3);
				aclIndex += addCount;
				inCount += addCount;
				assert(addCount <= natCount);
				assert(aclIndex <= (RTL8651_ACLTBL_SIZE -2));
				if (addCount < natCount)
					goto noAclEntry_forIngress_;
			}
		}

#if 0
		patchCount = _rtl8651_tryAddAclForMultipleNextHop(FALSE, netIfPtr->vid, aclIndex, RTL8651_ACLTBL_SIZE-1);
		/* (9) Fill user-add ACL rule to Ingress rule */
		CTAILQ_FOREACH(trackAcl, &netIfPtr->inAclHead, nextRule) {
			if (aclIndex >= RTL8651_ACLTBL_SIZE) {
				RTL8651_SET_DEFAULT_ACL_TOCPU(RTL8651_ACLTBL_SIZE-1);
				inAclEnd = RTL8651_ACLTBL_SIZE;
				goto noAclEntry__;
			}
			/* 
			 * This is the limiation of multiple Nexthop(multiple PPPoE Session).
			 * If a multiple nexthop route is added, some ACL patch rules will
			 * be added automatically for multiple nexthop usage. But in order
			 * to let added acl rules work correctly, NO permit rule is allowed.
			 * Hence we stealthily modify existed permit rule to toCPU rule.
			 */
			if (patchCount > 0) {
				if (trackAcl->actionType_ == RTL8651_ACL_PERMIT) {
					trackAcl->actionType_ = RTL8651_ACL_CPU;
					rtl8651_setAsicAclRule(aclIndex, trackAcl);
					trackAcl->actionType_ = RTL8651_ACL_PERMIT;
				}
			} else rtl8651_setAsicAclRule(aclIndex, trackAcl);
			aclIndex++; inCount++;
		}
#else
		/* (9) Fill user-add ACL rule to Ingress rule */
		CTAILQ_FOREACH(trackAcl, &netIfPtr->inAclHead, nextRule) {
			if (trackAcl->matchType_ == RTL8651_ACL_GENERIC) {
				if (aclIndex >= (RTL8651_ACLTBL_SIZE - 2))
					goto noAclEntry_forIngress_;

				if (trackAcl->isIPRange_ == 1) 
				#if 1	// chenyl (2005/03/02): Edward: IP Range ACL is only for L34 process 
				{
					/* Convert ip range to subnets & try to write them to ASIC ACL table */
					ipRangeCount = _rtl8651_addAclForIpRangeRule(FALSE, FALSE, trackAcl, aclIndex, RTL8651_ACLTBL_SIZE - 3);
					if ((aclIndex + ipRangeCount) > (RTL8651_ACLTBL_SIZE - 2) && netIfPtr->defAclAction == RTL8651_ACL_PERMIT)
						/* There's no enough space to write this ip range ACL to ASIC. Write an ACL to trap*/
						addCount = _rtl8651_addAclForIpRangeRule(TRUE, TRUE, trackAcl, aclIndex, RTL8651_ACLTBL_SIZE - 3);
					else
						addCount = _rtl8651_addAclForIpRangeRule(TRUE, FALSE, trackAcl, aclIndex, RTL8651_ACLTBL_SIZE - 3);
					
					aclIndex += addCount; 	
					inCount += addCount;
					assert(addCount <= ipRangeCount);
					if (aclIndex >= RTL8651_ACLTBL_SIZE - 2)
						goto noAclEntry_forIngress_;
					
					#if 0
					if (vlanIpRangeRuleIngressExist[(netIfPtr->vid & (RTL8651_VLAN_NUMBER-1))] == 0)
					{
						_rtl8651_tblDrvAclRule_t rangeAcl;
						ether_addr_t zero_mac, max_mac;
						uint16 ethType;
						rtl8651_tblDrv_vlanTable_t *vlan = &(DrvTbl.vlan[(netIfPtr->vid)&(RTL8651_VLAN_NUMBER-1)]);
						
						memset(&zero_mac, 0, sizeof(ether_addr_t));
						memset(&max_mac, 0xff, sizeof(ether_addr_t));
						ethType = (netIfPtr->linkLayerType == RTL8651_LL_PPPOE || ((netIfPtr->linkLayerType == RTL8651_LL_LIPUNNUMBERED || netIfPtr->linkLayerType == RTL8651_LL_WIPUNNUMBERED) && vlan->internal == FALSE))?0x8864:0x0800;
						
						memset(&rangeAcl, 0, sizeof(_rtl8651_tblDrvAclRule_t));
						RTL8651_MAC_ACL_RULE(&rangeAcl, RTL8651_ACL_CPU, &zero_mac, &zero_mac, &(vlan->macAddr), &max_mac, ethType, 0xffff);
						rtl8651_setAsicAclRule(aclIndex, &rangeAcl);

						aclIndex++; inCount++;

						vlanIpRangeRuleIngressExist[(netIfPtr->vid & (RTL8651_VLAN_NUMBER-1))] = 1;
					}
					#endif
				} 
				/* For srcMac & dstPort ACL rule setting */
				else if (_rtl8651_addAclForSrcMacDstPortRule(netIfPtr, trackAcl, aclIndex) == SUCCESS) {
					aclIndex++; 
					inCount++;
				}
				else
				#else
					goto Ingress_End_;
				#endif
				{
					if (_rtl8651_multiple_dsid == 1) {		/* multiple sessions configured */
						/*
							** Limitation:
							When multiple sessions configured, 
							ASIC ACL table has no knowledge about "an ACL rule corresponds to which session".

							** Workaround:
							Table driver has to config this user ACL rule to ASIC with action "toCPU".
							If a packet match this user ACL rule, it will be trapped to CPU.
							Next, fwdeng will judge this packet belongs to which session and check the following ACL.

							Think further regarding of pktOpApp...

							- If pktOpApp of this user ACL rule is "all", table driver will split it to 2 ACL rules:
							   => one with pktOpApp = "L2 or L3" without changing action type.
							   => one with pktOpApp = "L4" with changing action type to "toCPU".

							- If pktOpApp of this user ACL rule is "L3 or L4", table driver will split it to 2 ACL rules:
							   => one with pktOpApp = "L3" without changing action type.
							   => one with pktOpApp = "L4" with changing action type to "toCPU".

							- If pktOpApp of this user ACL rule is "L4", table driver will change its action type to "toCPU".
							
							- If pktOpApp of this user ACL rule is "L2" / "L3" / "L2 or L3", do we need to change its action type to "toCPU" ?
							   => no, because packets matching these kind of ACL have no relationship with concept "session".
							         For example, LAN to LAN packets matching this "L2" ACL rule do not have to be classified to which session they belong.
						*/
						uint32 tmpPktOpApp = (trackAcl->pktOpApp & 0x7);
						uint32 tmpActionType = (trackAcl->actionType_ & 0xf);
						
						if (trackAcl->pktOpApp == RTL8651_ACLTBL_BACKWARD_COMPATIBLE) { /* Case I */
							/* (1) pktOpApp = "L2 or L3" without changing action type */
							trackAcl->pktOpApp = RTL8651_ACLTBL_L2_AND_L3;
							rtl8651_setAsicAclRule(aclIndex, trackAcl);
							aclIndex++; inCount++;

							if (aclIndex >= (RTL8651_ACLTBL_SIZE - 2)) {
								trackAcl->pktOpApp = tmpPktOpApp;
								trackAcl->actionType_ = tmpActionType;
								goto noAclEntry_forIngress_;
							}
							
							/* (2) pktOpApp = "L4" with changing action type to "toCPU" */
							trackAcl->pktOpApp = RTL8651_ACLTBL_ONLY_L4;
							trackAcl->actionType_=RTL8651_ACL_DROP_LOG;
							rtl8651_setAsicAclRule(aclIndex, trackAcl);
							aclIndex++; inCount++;
						}
						else if (trackAcl->pktOpApp == RTL8651_ACLTBL_L3_AND_L4) {	/* Case II */
							/* (1) pktOpApp = "L3" without changing action type */
							trackAcl->pktOpApp = RTL8651_ACLTBL_ONLY_L3;
							rtl8651_setAsicAclRule(aclIndex, trackAcl);
							aclIndex++; inCount++;

							if (aclIndex >= (RTL8651_ACLTBL_SIZE - 2)) {
								trackAcl->pktOpApp = tmpPktOpApp;
								trackAcl->actionType_ = tmpActionType;
								goto noAclEntry_forIngress_;
							}
							
							/* (2) pktOpApp = "L4" with changing action type to "toCPU" */
							trackAcl->pktOpApp = RTL8651_ACLTBL_ONLY_L4;
							trackAcl->actionType_=RTL8651_ACL_DROP_LOG;
							rtl8651_setAsicAclRule(aclIndex, trackAcl);
							aclIndex++; inCount++;
						}
						else if (trackAcl->pktOpApp == RTL8651_ACLTBL_ONLY_L4) {	/* Case III */
							/* change its action type to "toCPU" */
							trackAcl->actionType_=RTL8651_ACL_DROP_LOG;
							rtl8651_setAsicAclRule(aclIndex, trackAcl);
							aclIndex++; inCount++;
						}
						else {	/* Case IV */
							/* without modification */
							rtl8651_setAsicAclRule(aclIndex, trackAcl);
							aclIndex++; inCount++;
						}
						trackAcl->pktOpApp = tmpPktOpApp;
						trackAcl->actionType_ = tmpActionType;
					}
					else {	/* not multiple sessions */
						rtl8651_setAsicAclRule(aclIndex, trackAcl);
						aclIndex++; inCount++;
					}
				}
			}
		}
#endif

		// shliu: Currently, this code segment is useless,
		// because _rtl8651_tryAddAclForMultipleNextHop() is an empty function when holy chhuang manages this function.

		/* (10) ACL patch rule for multiple nexthop/PPPoe */
		addCount = _rtl8651_tryAddAclForMultipleNextHop(TRUE, netIfPtr->vid, aclIndex, RTL8651_ACLTBL_SIZE-3);	
		aclIndex += addCount;
		inCount += addCount;
		assert(addCount <= patchCount);
		assert(aclIndex <= (RTL8651_ACLTBL_SIZE -2));
		if (addCount < patchCount)
			goto noAclEntry_forIngress_;


		/* (11) ACL patch rule for trapping tcp/syn packets at wan */
		/* Louis modified: trap all TCP syn/fin/rst packet at both WAN/LAN. */
#ifdef TRAP_TCP_SYN_FIN_RST
		if ( 1 ) 
		{
			addCount = _rtl8651_tryAddAclToTrapTcpSynFinRst(aclIndex, RTL8651_ACLTBL_SIZE-3);	
			aclIndex += addCount;
			inCount += addCount;
			assert(addCount <= 3);
			assert(aclIndex <= (RTL8651_ACLTBL_SIZE - 2));
			if (addCount < 3)
				goto noAclEntry_forIngress_;
		}
#else
		if (vlanPtr->internal == 0) {
			addCount = _rtl8651_tryAddAclForWanSynTrap(aclIndex, RTL8651_ACLTBL_SIZE - 3);
			aclIndex += addCount;
			inCount += addCount;
			assert(addCount <= 1);
			assert(aclIndex <= (RTL8651_ACLTBL_SIZE - 2));
			if (addCount < 1)
				goto noAclEntry_forIngress_;
		}
#endif


		/* (12) Policy-Based QoS */
		addCount = _rtl8651_tryAddAclForPolicyBasedQos(netIfPtr, aclIndex, RTL8651_ACLTBL_SIZE - 3);
		aclIndex += addCount;
		inCount += addCount;
		assert(addCount <= CTAILQ_TOTAL(&netIfPtr->policyQosHead));
		assert(aclIndex <= (RTL8651_ACLTBL_SIZE - 2));
		if (addCount < CTAILQ_TOTAL(&netIfPtr->policyQosHead))
			goto noAclEntry_forIngress_;

		/* (13) M-N Queue Model: add a permit rule with high priority */
		CTAILQ_FOREACH(trackAcl, &netIfPtr->mnqHead, nextRule) {
			uint32 action;
			uint32 pktOp;
			if (aclIndex >= (RTL8651_ACLTBL_SIZE - 2))
				goto noAclEntry_forIngress_;

#define DIFFSERV		1
#if DIFFSERV
			if (DrvTbl.rateLimit[trackAcl->rlIndex].mark == 1)
			{
				/* Set those rules to "to CPU acton" */
				action = trackAcl->actionType_;
				pktOp = trackAcl->pktOpApp;

				trackAcl->actionType_ = RTL8651_ACL_CPU;
				trackAcl->pktOpApp = RTL8651_ACLTBL_L3_AND_L4;

				rtl8651_setAsicAclRule(aclIndex, trackAcl);
				_rtl8651_registerACLDB(aclIndex, _rtl8651_markDiffserv, &(DrvTbl.rateLimit[trackAcl->rlIndex].dscp));
				aclIndex++; inCount++;
				trackAcl->actionType_ = action;
				trackAcl->pktOpApp = pktOp;
			}
			else
			{
				/* Modify those rules with priority set to PERMIT acton. */
				action = trackAcl->actionType_;
				pktOp = trackAcl->pktOpApp;				
				trackAcl->actionType_ = RTL8651_ACL_PERMIT;
				trackAcl->pktOpApp = RTL8651_ACLTBL_L3_AND_L4;				
				rtl8651_setAsicAclRule(aclIndex, trackAcl);
				aclIndex++; inCount++;
				trackAcl->actionType_ = action;
				trackAcl->pktOpApp = pktOp;
			}
#else
			/* Modify those rules with priority set to PERMIT acton. */
			action = trackAcl->actionType_;
			trackAcl->actionType_ = RTL8651_ACL_PERMIT;
			rtl8651_setAsicAclRule(aclIndex, trackAcl);
			aclIndex++; inCount++;
			trackAcl->actionType_ = action;
#endif
		}
		
		
		/* 
		 * According to ACL default mode to add a special rule:
		 *   1. aclIndex < RTL8651_ACLTBL_SIZE
		 *		=> default DROP: no rule should be added
		 *		=> default PERMIT: a PERMIT ALL rule should be added
		 *		=> default TOCPU: a TOCPU rule should be added
		 *	 2. aclIndex == RTL8651_ACLTBL_SIZE
		 *		=> default DROP: no rule should be added
		 *		=> default PERMIT: a PERMIT ALL rule hsould be added to the last entry
		 *		=> default TOCPU: a TOCPU rule should be added to the last entry
		 *	 3. aclIndex > RTL8651_ACLTBL_SIZE
		 *		=> never occur
		 */
//Ingress_End_:

		/* It's time to add default ACL for ingress rules of this VLAN (network interface). */

		/*   There are some user/driver added ingress ACL rules. */
		if (inCount > 0) {
			if (netIfPtr->defAclAction == RTL8651_ACL_DROP)
				inAclEnd = aclIndex - 1;
			else if (aclIndex < (RTL8651_ACLTBL_SIZE - 2)) {
				if (mnQ_enable==TRUE && vlanPtr->internal == 1 &&
//				    DrvTbl.rateLimit[RTL8651_RATELIMITTBL_SIZE-1].groupId == 3) {
				    DrvTbl.rateLimit[RTL8651_RATELIMITTBL_SIZE-1].queueType == 3) {
					_rtl8651_tblDrvAclRule_t rule;
					bzero(&rule,sizeof(_rtl8651_tblDrvAclRule_t));
					rule.actionType_ 	= RTL8651_ACL_DROP_RATE_EXCEED_BPS;
					rule.ruleType_ 		= RTL8651_ACL_IP;
					rule.rlIndex		= RTL8651_RATELIMITTBL_SIZE - 1;
					rule.pktOpApp= RTL8651_ACLTBL_L3_AND_L4;
					rtl8651_setAsicAclRule(aclIndex, &rule);
					aclIndex++;

					if (aclIndex >= (RTL8651_ACLTBL_SIZE - 2))
						goto noAclEntry_forIngress_;
				}
				_rtl8651_setAsicAclDefaultRule(aclIndex, netIfPtr->defAclAction); 
			aclIndex++;
				inAclEnd = aclIndex - 1;
		}
			else
				goto noAclEntry_forIngress_;

		}
		/*   No any user or driver added ingress ACL rules. 
		  *   Assign the reserved ingress ACL rules for the ingress ACL range of this VLAN. 
		  */
		else {
			switch(netIfPtr->defAclAction) {
				case RTL8651_ACL_PERMIT:
					inAclStart = inAclEnd = RTL8651_ACLTBL_PERMIT_ALL;
					break;
				case RTL8651_ACL_DROP:
					inAclStart = inAclEnd = RTL8651_ACLTBL_DROP_ALL;
					break;
				case RTL8651_ACL_CPU:
					inAclStart = inAclEnd = RTL8651_ACLTBL_ALL_TO_CPU;
					break;
				default: assert(0);
			}
		}

		outAclStart = aclIndex;
		/* (-1) Rate limit rule on egress side of a network interface. */
		CTAILQ_FOREACH(trackAcl, &netIfPtr->outRateLimitHead, nextRule) {
			uint32 pktOp;

			if (trackAcl->isRateLimitCounter_ == 1)	 /* Don't process rate limit counter rule here. */	
				continue;

			if (aclIndex >= (RTL8651_ACLTBL_SIZE - 2))
				goto noAclEntry_forEgress_;

			pktOp = trackAcl->pktOpApp;
			trackAcl->pktOpApp = RTL8651_ACLTBL_L3_AND_L4;
			rtl8651_setAsicAclRule(aclIndex, trackAcl);
			trackAcl->pktOpApp = pktOp;								
			aclIndex++; outCount++;
		}

		
		/* (0) Patch for MAC filter in Ip unnumbered case */
		if (netIfPtr->linkLayerType == RTL8651_LL_LIPUNNUMBERED && lanSideIpUnnumberedWan == netIfPtr) {
			mfCount = _rtl8651_addAclForLanSideIpUnMacFilter(FALSE, netIfPtr, aclIndex, RTL8651_ACLTBL_SIZE - 3);
			if (mfCount > 0) {
				addCount = _rtl8651_addAclForLanSideIpUnMacFilter(TRUE, netIfPtr, aclIndex, RTL8651_ACLTBL_SIZE - 3);
				aclIndex += addCount;
				outCount += addCount;
				assert(addCount <= mfCount);
				assert(aclIndex <= (RTL8651_ACLTBL_SIZE  - 2));
				if (addCount < mfCount)
					goto noAclEntry_forEgress_;
			}
		}

		/* (1) Fill user-defined Egress rule */
		CTAILQ_FOREACH(trackAcl, &netIfPtr->outAclHead, nextRule) {
			if (aclIndex >= (RTL8651_ACLTBL_SIZE - 2))
				goto noAclEntry_forEgress_;

			if (trackAcl->isIPRange_ == 1) 
			#if 1	// chenyl (2005/03/02): Edward: IP Range ACL is only for L34 process 
			{
				/* Convert ip range to subnets & try to write them to ASIC ACL table */
				ipRangeCount = _rtl8651_addAclForIpRangeRule(FALSE, FALSE, trackAcl, aclIndex, RTL8651_ACLTBL_SIZE - 3);
				if ((aclIndex + ipRangeCount) > (RTL8651_ACLTBL_SIZE - 2))
					/* There's no enough space to write this ip range ACL to ASIC. Write an ACL to trap. */
					addCount = _rtl8651_addAclForIpRangeRule(TRUE, TRUE, trackAcl, aclIndex, RTL8651_ACLTBL_SIZE - 3);
				else
					addCount = _rtl8651_addAclForIpRangeRule(TRUE, FALSE, trackAcl, aclIndex, RTL8651_ACLTBL_SIZE - 3);
				
				aclIndex += addCount; 	
				outCount += addCount;
				assert(addCount <= ipRangeCount);
				if (aclIndex >= RTL8651_ACLTBL_SIZE - 2)
					goto noAclEntry_forEgress_;

				#if 0
				if (vlanIpRangeRuleEgressExist[(netIfPtr->vid & (RTL8651_VLAN_NUMBER-1))] == 0)
				{
					_rtl8651_tblDrvAclRule_t rangeAcl;
					ether_addr_t zero_mac, max_mac;
					uint16 ethType;
					rtl8651_tblDrv_vlanTable_t *vlan = &(DrvTbl.vlan[(netIfPtr->vid)&(RTL8651_VLAN_NUMBER-1)]);
					
					memset(&zero_mac, 0, sizeof(ether_addr_t));
					memset(&max_mac, 0xff, sizeof(ether_addr_t));
					ethType = (netIfPtr->linkLayerType == RTL8651_LL_PPPOE || ((netIfPtr->linkLayerType == RTL8651_LL_LIPUNNUMBERED || netIfPtr->linkLayerType == RTL8651_LL_WIPUNNUMBERED) && vlan->internal == FALSE))?0x8864:0x0800;
					
					memset(&rangeAcl, 0, sizeof(_rtl8651_tblDrvAclRule_t));
					RTL8651_MAC_ACL_RULE(&rangeAcl, RTL8651_ACL_CPU, &zero_mac, &zero_mac, &(vlan->macAddr), &max_mac, ethType, 0xffff);
					rtl8651_setAsicAclRule(aclIndex, &rangeAcl);

					aclIndex++; outCount++;

					vlanIpRangeRuleEgressExist[(netIfPtr->vid & (RTL8651_VLAN_NUMBER-1))] = 1;
				}
				#endif
			} else
			#else
				goto Egress_End_;
			#endif
			{
				if (_rtl8651_multiple_dsid == 1) {		/* multiple sessions configured */
					/*
						** Limitation:
						When multiple sessions configured, 
						ASIC ACL table has no knowledge about "an ACL rule corresponds to which session".

						** Workaround:
						Currrently, traffic belongs to non-default-session is forwarded by fwdeng.
						Therefore, we cannot write egress ACL rule of non-default-session to ASIC, 
						because direct-Tx won't apply ASIC egress ACL rules.

						Table driver writes two kinds of ACL rules to ASIC:
						(1) ACL rules belonging to default-session.
						(2) ACL rules belonging to non-default-session and operation layer covering "L2" / "L3" / "L2 or L3".
					*/
					if (trackAcl->dsid != 0 /* not default session */) {
						if (trackAcl->pktOpApp == RTL8651_ACLTBL_BACKWARD_COMPATIBLE) {
							trackAcl->pktOpApp = RTL8651_ACLTBL_L2_AND_L3;
							rtl8651_setAsicAclRule(aclIndex, trackAcl);
							aclIndex++; outCount++;
						}
						#if 0
						/* === Currently, we don't allow users to config "pktOpApp" in egress side. For future use === */
						else if (trackAcl->pktOpApp == RTL8651_ACLTBL_L3_AND_L4) {
							trackAcl->pktOpApp = RTL8651_ACLTBL_ONLY_L3;
							rtl8651_setAsicAclRule(aclIndex, trackAcl);
							aclIndex++; outCount++;
						}
						else if (trackAcl->pktOpApp == RTL8651_ACLTBL_ONLY_L4);
						else {
							rtl8651_setAsicAclRule(aclIndex, trackAcl);
							aclIndex++; outCount++;		
						}
						/* ============================================================ */
						#endif
					}
					else {	/* default session */
						rtl8651_setAsicAclRule(aclIndex, trackAcl);
						aclIndex++; outCount++;
					}
				}
				else {	/* not multiple sessions configured */
					rtl8651_setAsicAclRule(aclIndex, trackAcl);
					aclIndex++; outCount++;
				}
			}
		}

		/* (11) Fill Rate Limit based Counter */
		{
			CTAILQ_FOREACH(trackAcl, &netIfPtr->outRateLimitHead, nextRule) {
				if (trackAcl->isRateLimitCounter_ == 0)	/* don't process other rate limit rule here */
					continue;
				if (aclIndex >= (RTL8651_ACLTBL_SIZE - 2))
					goto noAclEntry_forEgress_;
				
				rtl8651_setAsicAclRule(aclIndex, trackAcl);
				aclIndex++; outCount++;
			}
		}

		/*   Set the default egress ACL rule.
		  *   According to ACL default mode to add a special rule 
		  */
		if (outCount > 0) {
			if (netIfPtr->defAclAction == RTL8651_ACL_DROP)
				outAclEnd = aclIndex - 1;
			else if (aclIndex < (RTL8651_ACLTBL_SIZE - 2)) {
				_rtl8651_setAsicAclDefaultRule(aclIndex, netIfPtr->defAclAction);
				outAclEnd = aclIndex;
				aclIndex++;
		}
			else
				goto noAclEntry_forEgress_;
		}
		/*   No any user or driver added egress ACL rules. 
		  *   Assign the reserved ACL rules for the egress ACL range of this VLAN. 
		  */
		else {
			switch(netIfPtr->defAclAction) {
				case RTL8651_ACL_PERMIT:
					outAclStart = outAclEnd = RTL8651_ACLTBL_PERMIT_ALL;
					break;
				case RTL8651_ACL_DROP:
					outAclStart = outAclEnd = RTL8651_ACLTBL_DROP_ALL;
					break;
				case RTL8651_ACL_CPU:
					outAclStart = outAclEnd = RTL8651_ACLTBL_ALL_TO_CPU;
					break;
				default: assert(0);
			}
		}
		
		goto end__;

/*   When program execution flow comes here, it means that...
  *   There's still some ingress ACL rules but they cannot be added into ASIC ACL table.
  */
noAclEntry_forIngress_:		

		_rtl8651_addL2PermitRule(RTL8651_ACLTBL_SIZE - 2);
				RTL8651_SET_DEFAULT_ACL_TOCPU(RTL8651_ACLTBL_SIZE-1);

		/* Handling ingress ACL starting & ending position. */
		inAclEnd = RTL8651_ACLTBL_SIZE - 1;

		/* Handling egress ACL starting & ending position. */
		if (CTAILQ_TOTAL(&netIfPtr->outAclHead) > 0) {	// There are user egress ACL rules.
			outAclStart = RTL8651_ACLTBL_SIZE - 2;
			outAclEnd = RTL8651_ACLTBL_SIZE - 1;
		}
		else {	// No user egress ACL rules.
			switch(netIfPtr->defAclAction) {
				case RTL8651_ACL_PERMIT:
					outAclStart = outAclEnd = RTL8651_ACLTBL_PERMIT_ALL;
					break;
				case RTL8651_ACL_DROP:
					outAclStart = outAclEnd = RTL8651_ACLTBL_DROP_ALL;
					break;
				case RTL8651_ACL_CPU:
					outAclStart = outAclEnd = RTL8651_ACLTBL_ALL_TO_CPU;
					break;
				default: assert(0);
			}			
		}
		goto end__;

/*   When program execution flow comes here, it means that...
  *   There's still some egress ACL rules but they cannot be added into ASIC ACL table.
  */		
noAclEntry_forEgress_:

		_rtl8651_addL2PermitRule(RTL8651_ACLTBL_SIZE - 2);
		RTL8651_SET_DEFAULT_ACL_TOCPU(RTL8651_ACLTBL_SIZE - 1);

		/* No need to handle ingress ACL starting & ending position. */
		outAclEnd = RTL8651_ACLTBL_SIZE - 1;
	
end__:
		/* Set ingress ACL rule starting & ending position of this VLAN. */
			vlanPtr->inAclStart = inAclStart;
		vlanPtr->inAclEnd = inAclEnd;

		/* Set egress ACL rule starting & ending position of this VLAN. */
			vlanPtr->outAclStart = outAclStart;
		vlanPtr->outAclEnd = outAclEnd;
		
		RTL8651_SETASICVLAN(vlanPtr);
	}


	/* For Pseudo-VLAN */
	for(i=0, vlanPtr=DrvTbl.vlan; i<RTL8651_VLAN_NUMBER; i++, vlanPtr++) {
		if (vlanPtr->valid == 0 || vlanPtr->fvlan_t == NULL)
			continue;
		/* Otherwise is Sub VLAN */
		vlanPtr->inAclStart 	= vlanPtr->fvlan_t->inAclStart;
		vlanPtr->inAclEnd		= vlanPtr->fvlan_t->inAclEnd;
		vlanPtr->outAclStart	= vlanPtr->fvlan_t->outAclStart;
		vlanPtr->outAclEnd		= vlanPtr->fvlan_t->outAclEnd;
		RTL8651_SETASICVLAN(vlanPtr);
	}

#if 1/*CONFIG_RTL865XB_EXP_PPPOE_PASSTHRU*/
		/*
		 *  If PPPoE Passthru is enabled, we must sync passthru's ACL with WAN's.
		 */
		if ( ( rtl8651_l2protoPassThrough & PASSTHRU_PPPOE ) &&
		     rtl8651_gPPPoEExternalVlanIndex != RTL8651_VLAN_NUMBER &&
		     rtl8651_gPPPoEPassthruVlanID != RTL8651_PPPOEPASSTHRU_VLANID_IS_NOT_SET )
		{
			int32 retval;
			
			vlanPtr = &DrvTbl.vlan[rtl8651_vlanTableIndex(rtl8651_gPPPoEPassthruVlanID)];
			vlanPtr->inAclStart = DrvTbl.vlan[rtl8651_gPPPoEExternalVlanIndex].inAclStart;
			vlanPtr->inAclEnd = DrvTbl.vlan[rtl8651_gPPPoEExternalVlanIndex].inAclEnd;
			vlanPtr->outAclStart = DrvTbl.vlan[rtl8651_gPPPoEExternalVlanIndex].outAclStart;
			vlanPtr->outAclEnd = DrvTbl.vlan[rtl8651_gPPPoEExternalVlanIndex].outAclEnd;
			retval = RTL8651_SETASICVLAN( vlanPtr );
			assert( retval == SUCCESS );
		}
#endif/*CONFIG_RTL865XB*//*CONFIG_RTL865XB_EXP_PPPOE_PASSTHRU*/

}




#ifdef RTL865X_TEST
//
//  Only for PPPoE Passthru testing code to call _rtl8651_arrangeAllACL().
//
void for_pppoe_passthru_testcode_rtl8651_arrangeAllACL(void)
{
	_rtl8651_arrangeAllACL();
}

//
//  Only for IPv6 Passthru testing code to call _rtl8651_arrangeAllACL().
//
void for_ipv6_passthru_testcode_rtl8651_arrangeAllACL(void)
{
	_rtl8651_arrangeAllACL();
}
#endif//RTL865X_TEST






/*=======================================
  * ACL
  *=======================================*/
#define RTL865X_ACL_FUNCTION


static int8 
_rtl8651_sameAclRule(_rtl8651_tblDrvAclRule_t *rule1, rtl8651_tblDrvAclRule_t *rule2) {


	if (rule1->actionType_ != rule2->actionType_ || rule1->ruleType_ != rule2->ruleType_)
		return FALSE;

	switch(rule1->ruleType_) {
	case RTL8651_ACL_MAC:
		 if (rule1->typeLen_ != rule2->typeLen_ || rule1->typeLenMask_ != rule2->typeLenMask_)
		 	return FALSE;
		 if (memcmp(&rule1->dstMac_, &rule2->dstMac_, sizeof(ether_addr_t)) || 
		 	 memcmp(&rule1->dstMacMask_, &rule2->dstMacMask_, sizeof(ether_addr_t)) ||
			 memcmp(&rule1->srcMac_, &rule2->srcMac_, sizeof(ether_addr_t)) ||
			 memcmp(&rule1->srcMacMask_, &rule2->srcMacMask_, sizeof(ether_addr_t)) )
			 return FALSE;
		 return TRUE;
	case RTL8651_ACL_IP:
		 if (rule1->ipProto_ != rule2->ipProto_ || rule1->ipProtoMask_ != rule2->ipProtoMask_ ||
			rule1->ipFlag_ != rule2->ipFlag_ || rule1->ipFlagMask_ != rule2->ipFlagMask_)
			return FALSE; 
		break;
			
	case RTL8651_ACL_ICMP:
		 if (rule1->icmpType_ != rule2->icmpType_ || rule1->icmpTypeMask_ != rule2->icmpTypeMask_ ||
			rule1->icmpCode_ != rule2->icmpCode_ || rule1->icmpCodeMask_ != rule2->icmpCodeMask_)
			return FALSE; 
		 break;

	case RTL8651_ACL_IGMP:
		 if(rule1->igmpType_ != rule2->igmpType_ || rule1->igmpTypeMask_ != rule2->igmpTypeMask_)
		 	return FALSE; 
		 break;
	case RTL8651_ACL_TCP:
		 if(rule1->tcpFlag_ != rule2->tcpFlag_ || rule1->tcpFlagMask_ != rule2->tcpFlagMask_ ||
			rule1->tcpSrcPortUB_ != rule2->tcpSrcPortUB_ || rule1->tcpSrcPortLB_ != rule2->tcpSrcPortLB_ ||
			rule1->tcpDstPortUB_ != rule2->tcpDstPortUB_ || rule1->tcpDstPortLB_ != rule2->tcpDstPortLB_)
		 	return FALSE; 
		 break;
	case RTL8651_ACL_UDP:
		printk("%d %d, %d %d ,%d %d ,%d %d \n",rule1->udpSrcPortUB_,rule2->udpSrcPortUB_ 
			, rule1->udpSrcPortLB_, rule2->udpSrcPortLB_ 
			,rule1->udpDstPortUB_ , rule2->udpDstPortUB_ 
			, rule1->udpDstPortLB_ ,rule2->udpDstPortLB_);
		
		 if(rule1->udpSrcPortUB_ != rule2->udpSrcPortUB_ || rule1->udpSrcPortLB_ != rule2->udpSrcPortLB_ ||
			rule1->udpDstPortUB_ != rule2->udpDstPortUB_ || rule1->udpDstPortLB_ != rule2->udpDstPortLB_)
			return FALSE;
		 break;
	default: return FALSE; /* Unknown rule type */
	
	}
	/* Compare common part */
	if (rule1->srcIpAddr_ != rule2->srcIpAddr_ || rule1->srcIpAddrMask_ != rule2->srcIpAddrMask_ ||
		rule1->dstIpAddr_ != rule2->dstIpAddr_ || rule1->dstIpAddrMask_ != rule2->dstIpAddrMask_ ||
		rule1->tos_ != rule2->tos_ || rule1->tosMask_ != rule2->tosMask_ )
		return FALSE;
	return TRUE;				
}


static int32 
_rtl8651_addAclRule(uint32 sessionID, rtl8651_tblDrv_networkIntfTable_t *netif_t, int8 Ingress, rtl8651_tblDrvAclRule_t *rule, int8 flag) 
{
	_rtl8651_tblDrvAclRule_t *aclRule;
	uint32 dsid;
	
	assert(rule && netif_t);
	
	if (  (rule->actionType_!=RTL8651_ACL_DROP) &&  (rule->actionType_!=RTL8651_ACL_PERMIT) &&
  	      (rule->actionType_!=RTL8651_ACL_DROP_LOG) &&  (rule->actionType_!=RTL8651_ACL_CPU) &&
  	      (rule->actionType_!=RTL8651_ACL_DROP_NOTIFY) )
  	      	return TBLDRV_EINVALACTIONTYPE;

	/* Egress rule is disallowed to add this type of ACL */
	if (rule->actionType_ == RTL8651_ACL_DROP_NOTIFY && Ingress == FALSE)
		return TBLDRV_EINVALACTIONTYPE;

	if (aclMode == ACL_MODE_INDEXING)
		return TBLDRV_EACLMODEISNOTALLOWED;
	if ((aclRule = CTAILQ_FIRST(&DrvTbl.freeList.aclRule)) == NULL) 
		return TBLDRV_ENOFREEBUFFER;
	dsid=_rtl8651_SessionIDToDSID(sessionID);
	CTAILQ_REMOVE(&DrvTbl.freeList.aclRule, aclRule, nextRule);
	bzero(aclRule, sizeof(_rtl8651_tblDrvAclRule_t)-sizeof(aclRule->nextRule));
	rtl8651_memcpy(aclRule, rule,sizeof(rtl8651_tblDrvAclRule_t));
	aclRule->dsid = dsid; /* don't know if it's a right place here yet 2004/1/19 */
	aclRule->isIPRange_ = (flag == 0x01)? 1: 0;
	aclRule->isRateLimitCounter_ = 0;

	/*
	  *	ASIC ACL indexing: Before setting acl index, initiate it with "RTL8651_ACLTBL_SIZE+RTL8651_ACLTBL_RESERV_SIZE"
	  *   and make sure it will be checked in software even the ASIC's reason bit is applied.
	  */
	aclRule->aclIdx = RTL8651_ACLTBL_SIZE+RTL8651_ACLTBL_RESERV_SIZE;

	/* for dsid support, 2004/1/19 orlando
	 * do not drop by hardware in multiple pppoe/napt case
	 * always trap to CPU to do the drop, to further check if dsid matches!!
	 * assume rtl8651a_addPppoeSession() performed before this step
	 */
	if ( _rtl8651_multiple_dsid == 1 && aclRule->actionType_==RTL8651_ACL_DROP) {
		if(_rtl8651_acl_drop_nolog[dsid] == 0)
			_rtl8651_acl_drop_nolog[dsid] = 1;
	}

	if (Ingress == TRUE) 
		CTAILQ_INSERT_TAIL(&netif_t->inAclHead, aclRule, nextRule);
	else CTAILQ_INSERT_TAIL(&netif_t->outAclHead, aclRule, nextRule);
	if ((CTAILQ_TOTAL(&netif_t->inAclHead)+CTAILQ_TOTAL(&netif_t->outAclHead)) == 1)
		aclMode = ACL_MODE_STACK;

	_rtl8651_arrangeAllACL();
	
	return SUCCESS;	
}


static int32 
_rtl8651_delAclRule(uint32 sessionID, rtl8651_tblDrv_networkIntfTable_t *netif_t, int8 Ingress, rtl8651_tblDrvAclRule_t *rule)
{
	_rtl8651_tblDrvAclRule_t *delAcl;
	CTAILQ_HEAD(_aclStruct, _rtl8651_tblDrvAclRule_s) *aclHead;

	uint32 dsid;
	

	assert(netif_t && rule);

	if (aclMode == ACL_MODE_INDEXING) 
		return TBLDRV_EACLMODEISNOTALLOWED;
	dsid=_rtl8651_SessionIDToDSID(sessionID);
	if (Ingress == TRUE)
		aclHead = (struct _aclStruct *)&netif_t->inAclHead;
	else aclHead = (struct _aclStruct *)&netif_t->outAclHead;
	CTAILQ_FOREACH_REVERSE(delAcl, aclHead, _aclStruct, nextRule)
		if (_rtl8651_sameAclRule(delAcl, rule) == TRUE)
			break;
	if (delAcl == NULL) 
		return TBLDRV_EENTRYNOTFOUND;
	CTAILQ_REMOVE(aclHead, delAcl, nextRule);
	delAcl->dsid = dsid; /* sort of clean this field, but not of great use i guess 2004/1/19 orlando */
	CTAILQ_INSERT_HEAD(&DrvTbl.freeList.aclRule, delAcl, nextRule);
	if ((CTAILQ_TOTAL(&netif_t->inAclHead)+CTAILQ_TOTAL(&netif_t->outAclHead)) == 0)
		aclMode = ACL_MODE_NONE;
	_rtl8651_arrangeAllACL();
	return SUCCESS;
}


/* 
	This macro is to convert "operation layer" to ASIC ACL "pktOpApp".
	For example, if the operation layer of a certain packet is 3 (routing),
	then it is converted to (010) in binary form.

	reference: see ASIC ACL definition of pktOpApp field.
*/
#define _RTL8651B_PKTOPAPP_BIT(optLayer) (0x01 << (optLayer - 2))


/* Parameter "optLayer" is a determined in FWDENG. */
_rtl8651_tblDrvAclRule_t *
_rtl8651_aclLookUp(uint32 dsid, rtl8651_tblDrv_networkIntfTable_t *netIfPtr, int8 Ingress, uint32 startIdx, int8 *ether_hdr, int8 *ip_hdr, int8 *defPermit, uint32 matchType, uint32 optLayer) {
	struct ether_header *ehdr = (struct ether_header *)ether_hdr;
	struct ip *iphdr = (struct ip *)ip_hdr;
	_rtl8651_tblDrvAclRule_t *rule;
	CTAILQ_HEAD(_aclStruct, _rtl8651_tblDrvAclRule_s) *aclHead;
	struct icmp *icmphdr;
	struct igmp *igmphdr;
	struct tcphdr *tcphdr;
	struct udphdr *udphdr;
	ipaddr_t ip_s=0, ip_d=0;
	uint16 aclCount;



	assert(ehdr != NULL);
	assert((optLayer == _RTL8651_OPERATION_OTHER) || ((optLayer <= _RTL8651_OPERATION_LAYER4) && (optLayer >= _RTL8651_OPERATION_LAYER2)));
		
	if (netIfPtr == NULL) {
		/* If no network itnerface attached, always permit all */
		if (defPermit != NULL) *defPermit = TRUE;
		return (_rtl8651_tblDrvAclRule_t *)NULL;
	}
	if (iphdr != (struct ip *)NULL) {
		ip_s = PKTGET_UINT32_UNALIGNED(&iphdr->ip_src);
		ip_d = PKTGET_UINT32_UNALIGNED(&iphdr->ip_dst);
	}
	if (Ingress == TRUE)
		aclHead = (struct _aclStruct *)&netIfPtr->inAclHead;
	else aclHead = (struct _aclStruct *)&netIfPtr->outAclHead;

	CTAILQ_FOREACH(rule, aclHead, nextRule) {

		if (rule->aclIdx < startIdx)
			continue;

		/* 
			Ingress user ACL rules have been divided into some categories 
			by "matchType_" field. If the matchType_ of an ingress ACL rule does not conform 
			to parameter "matchType", we skipped it.
		*/
		if (Ingress == TRUE && rule->matchType_ != matchType)
			continue;

		/* 2005.09.25 Check applicability on packet operation.*/
		if (rule->pktOpApp !=  RTL8651_ACLTBL_BACKWARD_COMPATIBLE) {
			/* 
				When this ACL rule has pktOpApp defined, skip it if...
				(1) No operation layer determined by FWDENG, or
				(2) If "the decision of this packet" does not match "pktOpApp of this rule"
			*/
			if ((optLayer == _RTL8651_OPERATION_OTHER) || ((_RTL8651B_PKTOPAPP_BIT(optLayer) & rule->pktOpApp) == 0))
				continue;
		}

		switch (rule->ruleType_)
		{
		case RTL8651_ACL_MAC:
			{
			 ether_addr_t srcMac, dstMac;

			 dstMac.octet[0]= ehdr->ether_dhost[0] & rule->dstMacMask_.octet[0];
			 dstMac.octet[1]= ehdr->ether_dhost[1] & rule->dstMacMask_.octet[1];
			 dstMac.octet[2]= ehdr->ether_dhost[2] & rule->dstMacMask_.octet[2];
			 dstMac.octet[3]= ehdr->ether_dhost[3] & rule->dstMacMask_.octet[3];
			 dstMac.octet[4]= ehdr->ether_dhost[4] & rule->dstMacMask_.octet[4];
			 dstMac.octet[5]= ehdr->ether_dhost[5] & rule->dstMacMask_.octet[5];
			 srcMac.octet[0]= ehdr->ether_shost[0] & rule->srcMacMask_.octet[0];
			 srcMac.octet[1]= ehdr->ether_shost[1] & rule->srcMacMask_.octet[1];
			 srcMac.octet[2]= ehdr->ether_shost[2] & rule->srcMacMask_.octet[2];
			 srcMac.octet[3]= ehdr->ether_shost[3] & rule->srcMacMask_.octet[3];
			 srcMac.octet[4]= ehdr->ether_shost[4] & rule->srcMacMask_.octet[4];
			 srcMac.octet[5]= ehdr->ether_shost[5] & rule->srcMacMask_.octet[5];
			 if (((ehdr->ether_type & rule->typeLenMask_) == rule->typeLen_) &&
			 	(memcmp(dstMac.octet, rule->dstMac_.octet, 6) == 0) &&
			 	(memcmp(srcMac.octet, rule->srcMac_.octet, 6) == 0) &&
			 	(rule->dsid==dsid))
				return (_rtl8651_tblDrvAclRule_t *)rule;
			}
			break;

		case RTL8651_ACL_IP:
			{
			 if (iphdr == (struct ip *)NULL)
			 	break;
			 if (((iphdr->ip_p & rule->ipProtoMask_) != rule->ipProto_) ||
				(((ntohs(iphdr->ip_off)>>13) & rule->ipFlagMask_) != rule->ipFlag_))
			 		break;
			}
			goto l3l4_shared;

		case RTL8651_ACL_ICMP:
			{/* Check if the packet is an ICMP packet */
			 if (iphdr == (struct ip *)NULL || iphdr->ip_p != IPPROTO_ICMP || (ntohs(iphdr->ip_off) & IP_OFFMASK))
				break;
			 /* Start to match ICMP rule */
			 icmphdr = (struct icmp *)(ip_hdr + ((iphdr->ip_vhl  & 0x0f)<<2));
			 if (((icmphdr->icmp_type & rule->icmpTypeMask_) != rule->icmpType_) ||
				((icmphdr->icmp_code & rule->icmpCodeMask_) != rule->icmpCode_))
					break;
			}
			goto l3l4_shared;

		case RTL8651_ACL_IGMP:
			{/* Check if the packet is an IGMP packet */
			 if (iphdr == (struct ip *)NULL || iphdr->ip_p != IPPROTO_IGMP || (ntohs(iphdr->ip_off) & IP_OFFMASK))
				break;
			 /* Start to match IGMP rule */
			 igmphdr = (struct igmp *)(ip_hdr + ((iphdr->ip_vhl  & 0x0f)<<2));
			 if (((igmphdr->igmp_type & rule->igmpTypeMask_) != rule->igmpType_))
					break;
			}
			goto l3l4_shared;

		case RTL8651_ACL_TCP:
			{
			/* For srcMac & destPort ACL rule */
			ether_addr_t srcMac;

			 srcMac.octet[0]= ehdr->ether_shost[0] & rule->tcpl2srcMacMask_.octet[0];
			 srcMac.octet[1]= ehdr->ether_shost[1] & rule->tcpl2srcMacMask_.octet[1];
			 srcMac.octet[2]= ehdr->ether_shost[2] & rule->tcpl2srcMacMask_.octet[2];
			 srcMac.octet[3]= ehdr->ether_shost[3] & rule->tcpl2srcMacMask_.octet[3];
			 srcMac.octet[4]= ehdr->ether_shost[4] & rule->tcpl2srcMacMask_.octet[4];
			 srcMac.octet[5]= ehdr->ether_shost[5] & rule->tcpl2srcMacMask_.octet[5];

			/* Check if the packet is a TCP packet */
			 if (iphdr == (struct ip *)NULL || iphdr->ip_p != IPPROTO_TCP || (ntohs(iphdr->ip_off) & IP_OFFMASK))
				break;			 

			 /* Start to match TCP rule */
			 tcphdr = (struct tcphdr *)(ip_hdr + ((iphdr->ip_vhl  & 0x0f)<<2));
			 if ( (memcmp(srcMac.octet, rule->tcpl2srcMac_.octet, 6) != 0) || ((tcphdr->th_flags & rule->tcpFlagMask_) != rule->tcpFlag_) ||
				 !(ntohs(tcphdr->th_sport)>=rule->tcpSrcPortLB_ && ntohs(tcphdr->th_sport) <= rule->tcpSrcPortUB_) ||
				 !(ntohs(tcphdr->th_dport)>=rule->tcpDstPortLB_ && ntohs(tcphdr->th_dport) <= rule->tcpDstPortUB_) )
					break;
			}
			goto l3l4_shared;

		case RTL8651_ACL_UDP:
			{
			/* For srcMac & destPort ACL rule */
			ether_addr_t srcMac;

			 srcMac.octet[0]= ehdr->ether_shost[0] & rule->udpl2srcMacMask_.octet[0];
			 srcMac.octet[1]= ehdr->ether_shost[1] & rule->udpl2srcMacMask_.octet[1];
			 srcMac.octet[2]= ehdr->ether_shost[2] & rule->udpl2srcMacMask_.octet[2];
			 srcMac.octet[3]= ehdr->ether_shost[3] & rule->udpl2srcMacMask_.octet[3];
			 srcMac.octet[4]= ehdr->ether_shost[4] & rule->udpl2srcMacMask_.octet[4];
			 srcMac.octet[5]= ehdr->ether_shost[5] & rule->udpl2srcMacMask_.octet[5];

			/* Check if the packet is an UDP packet */
			 if (iphdr == (struct ip *)NULL || iphdr->ip_p != IPPROTO_UDP || (ntohs(iphdr->ip_off) & IP_OFFMASK))
				break;
			 /* Start to match UDP rule */
			 udphdr = (struct udphdr *)(ip_hdr + ((iphdr->ip_vhl  & 0x0f)<<2));
			 if ((memcmp(srcMac.octet, rule->udpl2srcMac_.octet, 6) != 0) || !(ntohs(udphdr->uh_sport)>=rule->udpSrcPortLB_ && ntohs(udphdr->uh_sport)<=rule->udpSrcPortUB_) ||
				 !(ntohs(udphdr->uh_dport)>=rule->udpDstPortLB_ && ntohs(udphdr->uh_dport)<=rule->udpDstPortUB_))
					break;
l3l4_shared:
			if (((iphdr->ip_tos & rule->tosMask_) == rule->tos_) && (rule->dsid==dsid)) {
				if (rule->isIPRange_ == 1) {	
					/* If both start and end are zero,  we consider it is match*/
					if ((rule->srcIpAddrStart_==0 && rule->srcIpAddrEnd_==0) || 
					    (rule->srcIpAddrStart_<=ip_s && rule->srcIpAddrEnd_>=ip_s)) { //source match
						if ((rule->dstIpAddrStart_==0 && rule->dstIpAddrEnd_==0) ||
						    (rule->dstIpAddrStart_<=ip_d && rule->dstIpAddrEnd_>=ip_d)) //destination match
						    	return (_rtl8651_tblDrvAclRule_t *)rule;
					}
				}
				else {
					if ((( ip_s & rule->srcIpAddrMask_) == rule->srcIpAddr_) && ((ip_d & rule->dstIpAddrMask_) == rule->dstIpAddr_) )
						return (_rtl8651_tblDrvAclRule_t *)rule;
				}
			}

//			 if ( (( ip_s & rule->srcIpAddrMask_) == rule->srcIpAddr_) &&
//			      ((ip_d & rule->dstIpAddrMask_) == rule->dstIpAddr_) &&
//			      ((iphdr->ip_tos & rule->tosMask_) == rule->tos_) &&
//			      (rule->dsid==dsid)) {
//			    	return (_rtl8651_tblDrvAclRule_t *)rule;
//			}
			}
			break;

		case RTL8651_ACL_IFSEL:
			{/* IFSEL rule */
			 if ((rule->gidxSel_ & (1<<(ip_s&0x3)))
			 	&& (rule->dsid==dsid))
			 	return (_rtl8651_tblDrvAclRule_t *)rule;
			}
			break;

		default: assert(0);
		}
	}
	
	/* If no rule is matched, two cases may happen:
	 * Case 1: mismatch due to all rule mismatch
	 *		   => use default action
	 * Case 2: mismatch due to no any acl rule
	 *		   => use permit all
	 */
	if (defPermit != (int8 *)NULL) {
		if (Ingress == TRUE) {
			aclCount = CTAILQ_TOTAL(&netIfPtr->inAclHead);
		} else aclCount = CTAILQ_TOTAL(&netIfPtr->outAclHead);
		if (aclCount > 0) {
			if (netIfPtr->defAclAction == RTL8651_ACL_PERMIT)
				*defPermit = TRUE;
			else *defPermit = FALSE;
		} else *defPermit = TRUE; /* If no rule, always permit all */
	}
	return (_rtl8651_tblDrvAclRule_t *)NULL;
}




/*
@func int32 | rtl8651_addAclRule | Add an ACL rule in ACL STACK Mode.
@parm int8 * | ifName | Network interface name.
@parm int8 | Ingress | TRUE for Ingress rule and FALSE for Egress rule.
@parm rtl8651_tblDrvAclRule_t * | rule | ACL rule.
@rvalue SUCCESS | An ACL rule was successfully added.
@rvalue TBLDRV_EACLRULEISNULL | The specified ACL rule is NULL.
@rvalue TBLDRV_EINVALACTIONTYPE | Unknown ACL rule action type.
@rvalue TBLDRV_EACLMODEISNOTALLOWED | The current ACL operation mode is not STACK Mode.
@rvalue TBLDRV_ENOFREEBUFFER | Out of memory buffer.
@rvalue TBLDRV_EINVNETIFNAME | The specified network interface name was not found.
@rvalue TBLDRV_EACLRULE_MATCHTYPEINCORRECT | Incorrect match type.
@rvalue TBLDRV_EACLEGRESSRULE_NOTSUPPORTPKTOP | Egress ACL not support packet operation
@comm
Add a new ACL rule from the tail of the specified network interface's ACL rule. The RTL8651 supports two ACL operation modes - STACK Mode & Indexing Mode.
These two modes cannot coexist in the system. Hence if the current operatoin mode is Indexing mode, the API will return fail.
*/
int32 rtl8651a_addAclRule(uint32 SessionID,int8 * ifName, int8 Ingress, rtl8651_tblDrvAclRule_t *rule) {
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	int32 retval;
	if (rule == NULL)	return TBLDRV_EACLRULEISNULL;
	if ((rule->matchType_ != RTL8651_ACL_GENERIC) && (rule->matchType_ != RTL8651_ACL_L4NEWFLOW))
		return TBLDRV_EACLRULE_MATCHTYPEINCORRECT;
	if ((Ingress == FALSE) && (rule->pktOpApp != RTL8651_ACLTBL_BACKWARD_COMPATIBLE))
		return TBLDRV_EACLEGRESSRULE_NOTSUPPORTPKTOP;
	
	rtlglue_drvMutexLock();
	if ((netif_t=GET_NETWORK_INTERFACE(ifName, NULL)))
		retval = _rtl8651_addAclRule(SessionID, netif_t, Ingress, rule, 0x00);
	else retval = TBLDRV_EINVNETIFNAME;
	rtlglue_drvMutexUnlock();
	return retval;
}

/*
@func int32 | rtl8651_delAclRule | Remove an ACL rule when ACL operation mode is STACK mode.
@parm int8 * | ifName | Network interface name.
@parm int8 | Ingress | TRUE for Ingress rule and FALSE for Egress rule.
@parm rtl8651_tblDrvAclRule_t * | rule | ACL rule.
@rvalue TBLDRV_EACLRULEISNULL | The specified ACL rule is NULL.
@rvalue TBLDRV_EACLMODEISNOTALLOWED | The current ACL operation mode is not STACK Mode.
@rvalue TBLDRV_EINVNETIFNAME | Network interface name was not found.
@rvalue TBLDRV_EENTRYNOTFOUND | The specified ACL rule was not found.
@comm
According the ACL rule content to remove a rule. Currently the RTL8651 supports two ACL operation modes - STACK Mode and Indexing Mode.
These two modes can not coexit in the system. If the current ACL operation mode is Indexing Mode, this API will return fail.
*/
int32 rtl8651a_delAclRule(uint32 SessionID,int8 *ifName, int8 Ingress, rtl8651_tblDrvAclRule_t *rule) {
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	int32 retval;
	if (rule == NULL)	return TBLDRV_EACLRULEISNULL;
	rtlglue_drvMutexLock();
	if ((netif_t = GET_NETWORK_INTERFACE(ifName, NULL)) )
		retval = _rtl8651_delAclRule(SessionID, netif_t, Ingress, rule);
	else retval = TBLDRV_EINVNETIFNAME;
	rtlglue_drvMutexUnlock();
	return retval;
}

/*
@func int32 | rtl8651a_addAclRuleExt | Add an IP range ACL rule when ACL operation mode is STACK mode.
@parm uint32 | pppoeId | Specify PPPoE ID.
@parm int8 * | ifName | Network interface name.
@parm int8 | Ingress | TRUE for Ingress rule and FALSE for Egress rule.
@parm rtl8651_tblDrvAclRule_t * | rule | ACL rule.
@rvalue SUCCESS | An ACL rule was successfully added.
@rvalue TBLDRV_EACLRULEISNULL | The specified ACL rule is NULL.
@rvalue TBLDRV_EINVALACTIONTYPE | Unknown ACL rule action type.
@rvalue TBLDRV_EACLMODEISNOTALLOWED | The current ACL operation mode is not STACK Mode.
@rvalue TBLDRV_ENOFREEBUFFER | Out of memory buffer.
@rvalue TBLDRV_EINVNETIFNAME | The specified network interface name was not found.
@comm
Add a new IP range ACL rule from the tail of the specified network interface's ACL rule. 
The RTL8651 supports two ACL operation modes - STACK Mode & Indexing Mode.
These two modes cannot coexist in the system. Hence if the current operatoin mode is Indexing mode, the API will return fail.
*/
int32 rtl8651a_addAclRuleExt(uint32 sessionID,int8 * ifName, int8 Ingress, rtl8651_tblDrvAclRule_t *rule) 
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	int32 retval;
	if (rule == NULL)	return TBLDRV_EACLRULEISNULL;

	rtlglue_drvMutexLock();
	if ((netif_t=GET_NETWORK_INTERFACE(ifName, NULL)))
		retval = _rtl8651_addAclRule(sessionID, netif_t, Ingress, rule, 0x01);
	else retval = TBLDRV_EINVNETIFNAME;
	rtlglue_drvMutexUnlock();
	return retval;
}

/*
@func int32 | rtl8651a_delAclRuleExt | Remove an IP range ACL rule when ACL operation mode is STACK mode.
@parm uint32 | pppoeId | Specify PPPoE ID.
@parm int8 * | ifName | Network interface name.
@parm int8 | Ingress | TRUE for Ingress rule and FALSE for Egress rule.
@parm rtl8651_tblDrvAclRule_t * | rule | ACL rule.
@rvalue TBLDRV_EACLRULEISNULL | The specified ACL rule is NULL.
@rvalue TBLDRV_EACLMODEISNOTALLOWED | The current ACL operation mode is not STACK Mode.
@rvalue TBLDRV_EINVNETIFNAME | Network interface name was not found.
@rvalue TBLDRV_EENTRYNOTFOUND | The specified ACL rule was not found.
@comm
According the ACL rule content to remove a rule. Currently the RTL8651 supports two ACL operation modes - STACK Mode and Indexing Mode.
These two modes can not coexit in the system. If the current ACL operation mode is Indexing Mode, this API will return fail.
*/
int32 rtl8651a_delAclRuleExt(uint32 sessionID,int8 *ifName, int8 Ingress, rtl8651_tblDrvAclRule_t *rule) 
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	int32 retval;
	if (rule == NULL)	return TBLDRV_EACLRULEISNULL;
	rtlglue_drvMutexLock();
	if ((netif_t = GET_NETWORK_INTERFACE(ifName, NULL)) )
		retval = _rtl8651_delAclRule(sessionID, netif_t, Ingress, rule);
	else retval = TBLDRV_EINVNETIFNAME;
	rtlglue_drvMutexUnlock();
	return retval;
}




/*
@func int32 | rtl8651_addAclRuleByIndexing | Add an ACL rule by specifying rule ID.
@parm uint16 | rIdx | Rule ID.
@parm int8 * | ifName | Network interface name.
@parm int8 | Ingress | TRUE for Ingress rule and FALSE for Egress rule.
@parm rtl8651_tblDrvAclRule_t * | rule | ACL rule.
@rvalue SUCCESS | An ACL rule was successfully added.
@rvalue TBLDRV_EACLRULEISNULL | The specified ACL rule is NULL.
@rvalue TBLDRV_EINVALACTIONTYPE | Unknown ACL action type.
@rvalue TBLDRV_EACLMODEISNOTALLOWED | Current ACL operation mode is not Indexing Mode.
@rvalue TBLDRV_EENTRYALREADYEXIST | Rule ID already exists.
@rvalue TBLDRV_ENOFREEBUFFER | Out of memory buffer.
@comm
According to the ACL rule indexing to add a ACL rule. The RTL8651 supports two ACL operation modes - STACK Mode and Indexing Mode. 
These two modes cannot coexist in the system. Hence if the current ACL operation mode is not Index mode, this API 
will forbid the rule to be added.
*/
int32 rtl8651_addAclRuleByIndexing(uint16 rIdx, int8 *ifName, int8 Ingress, rtl8651_tblDrvAclRule_t *rule) {
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	_rtl8651_tblDrvAclRule_t *trackAcl, *addAcl;
	CTAILQ_HEAD(_aclStruct, _rtl8651_tblDrvAclRule_s) *aclHead;
	int32 retval = SUCCESS;
	

	if ( rule == NULL)
		return TBLDRV_EACLRULEISNULL;
	if ( (rule->actionType_!=RTL8651_ACL_DROP) &&  (rule->actionType_!=RTL8651_ACL_PERMIT) &&
  	     (rule->actionType_!=RTL8651_ACL_DROP_LOG) &&  (rule->actionType_!=RTL8651_ACL_CPU) &&
  	     (rule->actionType_!=RTL8651_ACL_DROP_NOTIFY) )
		return TBLDRV_EINVALACTIONTYPE;	
	rtlglue_drvMutexLock();
	
	if (aclMode == ACL_MODE_STACK) {
		retval = TBLDRV_EACLMODEISNOTALLOWED;
		goto out;
	}
	 /* Allocate Rule Space */
	if ((addAcl = CTAILQ_FIRST(&DrvTbl.freeList.aclRule)) == NULL) {
		retval = TBLDRV_ENOFREEBUFFER;
		goto out;
	}
	/* Get Network interface by interface name */
	if ((netIfPtr = GET_NETWORK_INTERFACE(ifName, NULL)) == NULL) {
		retval = TBLDRV_EINVNETIFNAME;
		goto out;
	}
	if (Ingress == TRUE) 
		aclHead = (struct _aclStruct *)&netIfPtr->inAclHead;
	else aclHead = (struct _aclStruct *)&netIfPtr->outAclHead;
	CTAILQ_FOREACH(trackAcl, aclHead, nextRule) {
		/* Duplicate Index checking */
		LOCK_ERROR_CHECK((trackAcl->aclIdx == rIdx), TBLDRV_EENTRYALREADYEXIST);
		/* Find a proper place to add new entry */
		if (rIdx < trackAcl->aclIdx)
			break;
	}
	CTAILQ_REMOVE(&DrvTbl.freeList.aclRule, addAcl, nextRule);
	bzero(addAcl, sizeof(_rtl8651_tblDrvAclRule_t)-sizeof(addAcl->nextRule));
	rtl8651_memcpy(addAcl, rule, sizeof(rtl8651_tblDrvAclRule_t));
	addAcl->aclIdx = rIdx;
	if (trackAcl == (_rtl8651_tblDrvAclRule_t *) NULL)
		CTAILQ_INSERT_TAIL(aclHead, addAcl, nextRule);
	else CTAILQ_INSERT_BEFORE(aclHead, trackAcl, addAcl, nextRule);
	if ((CTAILQ_TOTAL(&netIfPtr->inAclHead)+CTAILQ_TOTAL(&netIfPtr->outAclHead)) == 1)
		aclMode = ACL_MODE_INDEXING;
	_rtl8651_arrangeAllACL();

out:
	rtlglue_drvMutexUnlock();
	return retval;
}

/*
@func int32 | rtl8651_delAclRuleByIndexing | Remove an ACL rule by specifying the rule ID.
@parm uint16 | rIdx | Rule ID.
@parm int8 * | ifName | Network interface name.
@parm int8 | Ingress | TRUE for Ingress rule and FALSE for Egress rule.
@rvalue SUCCESS | The specified ACL rule was removed successfully.
@rvalue TBLDRV_EACLMODEISNOTALLOWED | The system ACL operates in STACK Mode, hence Indexing mode is disallowed.
@rvalue TBLDRV_EINVNETIFNAME | Network interface was not found.
@rvalue TBLDRV_EENTRYNOTFOUND | The specified ACL ID was not found.
@comm
Currently RTL8651 supports two ACL operation modes - STACK Mode and Indexing Mode. These two mode cannot coexist in the system.
*/
int32 rtl8651_delAclRuleByIndexing(uint16 rIdx, int8 *ifName, int8 Ingress) {
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	_rtl8651_tblDrvAclRule_t *delAcl;
	CTAILQ_HEAD(_aclStruct, _rtl8651_tblDrvAclRule_s) *aclHead;
	int32 retval = SUCCESS;
	
	rtlglue_drvMutexLock();//Lock resource
	if (aclMode == ACL_MODE_STACK) {
		retval = TBLDRV_EACLMODEISNOTALLOWED;
		goto out;
	}
	/* Get network interface by interface name */
	if ((netIfPtr = GET_NETWORK_INTERFACE(ifName, NULL)) == NULL) {
		retval = TBLDRV_EINVNETIFNAME;
		goto out;
	}
	if (Ingress == TRUE)
		aclHead = (struct _aclStruct *)&netIfPtr->inAclHead;
	else aclHead = (struct _aclStruct *)&netIfPtr->outAclHead;
	CTAILQ_FOREACH(delAcl, aclHead, nextRule) {
		if (delAcl->aclIdx == rIdx)
			break;
	}
	if (delAcl == NULL) {
		retval = TBLDRV_EENTRYNOTFOUND;
		goto out;
	}
	CTAILQ_REMOVE(aclHead, delAcl, nextRule);
	CTAILQ_INSERT_HEAD(&DrvTbl.freeList.aclRule, delAcl, nextRule);
	if ((CTAILQ_TOTAL(&netIfPtr->inAclHead)+CTAILQ_TOTAL(&netIfPtr->outAclHead)) == 0)
		aclMode = ACL_MODE_NONE;
	_rtl8651_arrangeAllACL();
	
out:
	rtlglue_drvMutexUnlock();
	return SUCCESS;
}

/*
@func int32 | rtl8651_flushAclRule | Flush the specified Network interface's Ingress/Egress ACL rule.
@parm int8 | ifName | Network interface name.
@parm int8 | Ingress | Specify Ingress or Egress rule to be flushed.
@rvalue SUCCESS | The Specified ACL rules were flushed.
@rvalue TBLDRV_EINVNETIFNAME | The specified network interface name was not found.
*/
int32 rtl8651a_flushAclRule(uint32 SessionID,int8 *ifName, int8 Ingress) {
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	_rtl8651_tblDrvAclRule_t *delAcl;
	CTAILQ_HEAD(_aclStruct, _rtl8651_tblDrvAclRule_s) *aclHead;
	int32 retval = SUCCESS;
	uint32 dsid;

	
	rtlglue_drvMutexLock();
	dsid=_rtl8651_SessionIDToDSID(SessionID);
	/* Get Network interface by interface name */
	if ((netIfPtr = GET_NETWORK_INTERFACE(ifName, NULL)) == NULL) {
		retval = TBLDRV_EINVNETIFNAME;
		goto out;
	}
	if (Ingress == TRUE)
		aclHead = (struct _aclStruct *)&netIfPtr->inAclHead;
	else aclHead = (struct _aclStruct *)&netIfPtr->outAclHead;

	for( delAcl = CTAILQ_FIRST(aclHead);
	     delAcl;
	     )
	{
		if (delAcl->dsid==dsid)
		{
			/* Yes, we want to delete this rule ! */
			_rtl8651_tblDrvAclRule_t *next;
			next = CTAILQ_NEXT(delAcl, nextRule); /* first, keep the next rule in aclHead. */
			CTAILQ_REMOVE(aclHead, delAcl, nextRule);
			CTAILQ_INSERT_HEAD(&DrvTbl.freeList.aclRule, delAcl, nextRule); /* chain to free list */
			delAcl = next; /* continue to the next rule in aclHead. */
		}
		else
		{
			delAcl = CTAILQ_NEXT(delAcl, nextRule); /* try next rule. */
		}
	}
	
	assert(_rtl8651_acl_drop_nolog);
	_rtl8651_acl_drop_nolog[dsid]=0;
	_rtl8651_arrangeAllACL();

out:
	rtlglue_drvMutexUnlock();
	return retval;
}


/*
@func int32 | rtl8651_setDefaultAcl | Set default ACL rule action. If all rule is mismatch, the default rule will alway be matched.
@parm int8 | ifName | Network interface name.
@parm int8 | defAction | The default action. It would be the following values: RTL8651_ACL_CPU(If mismatch, trap packet to CPU), RTL8651_ACL_DROP(if mismatch, drop packet), RTL8651_ACL_PERMIT(if mismatch, permit packet).
@rvalue SUCCESS | The ACL default action was set successfully.
@rvalue TBLDRV_EINVALIDINPUT | Unknown default action type or specified network interface is NULL.
@rvalue TBLDRV_EINVNETIFNAME | The specified network interface name was not found.
@comm
Set the specified network interface's ACL to PERMIT, DROP or toCPU if rule mismatch. Note: If no ACL rule in the ACL table, default action always PERMIT. Once any ACL rule was added, the user-configure default action will work.
*/
int32 rtl8651a_setDefaultAcl(uint32 SessionID,int8 *ifName, int8 defAction) {
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
//	uint32 dsid;
//	dsid=_rtl8651_SessionIDToDSID(SessionID);

	UNLOCK_ERROR_CHECK(
		( (defAction != RTL8651_ACL_CPU) &&
		  (defAction != RTL8651_ACL_DROP) &&
		  (defAction != RTL8651_ACL_PERMIT) &&
		  (defAction != RTL8651_ACL_L34_DROP)
		), TBLDRV_EINVALIDINPUT
	);
	rtlglue_drvMutexLock();
	/* Get Network interface by interface name */
	netif_t = GET_NETWORK_INTERFACE(ifName, NULL);
	LOCK_ERROR_CHECK((netif_t==NULL), TBLDRV_EINVNETIFNAME);
	netif_t->defAclAction = defAction;
	_rtl8651_arrangeAllACL();
	rtlglue_drvMutexUnlock();
	return SUCCESS;
}

/*
@func int32 | _rtl8651_getDefaultAcl | Get default ACL rule action. If all rule is mismatch, the default rule will alway be matched.
@parm int8 | ifName | Network interface name.
@parm int8 | defAction | The default action. It would be the following values: RTL8651_ACL_CPU(If mismatch, trap packet to CPU), RTL8651_ACL_DROP(if mismatch, drop packet), RTL8651_ACL_PERMIT(if mismatch, permit packet).
@rvalue SUCCESS | The ACL default action was set successfully.
@rvalue TBLDRV_EINVALIDINPUT | Unknown default action type or specified network interface is NULL.
@rvalue TBLDRV_EINVNETIFNAME | The specified network interface name was not found.
@comm
Get the specified network interface's ACL to PERMIT, DROP or toCPU if rule mismatch. Note: If no ACL rule in the ACL table, default action always PERMIT. Once any ACL rule was added, the user-configure default action will work.
*/
int32 _rtl8651_getDefaultAcl(int8 *ifName, int8 * defAction) {
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	//rtlglue_drvMutexLock();
	/* Get Network interface by interface name */
	netif_t = GET_NETWORK_INTERFACE(ifName, NULL);
	UNLOCK_ERROR_CHECK((netif_t==NULL), TBLDRV_EINVNETIFNAME);
	//netif_t->defAclAction = defAction;
	*defAction = netif_t->defAclAction;
	//rtlglue_drvMutexUnlock();
	return SUCCESS;
}


#define RTL8651_ALG_API
/*=======================================================================================
 * RTL8651 ALG APIs:
 *	1. Static Functions
 *		. _rtl8651_algToAclCount()
 *		. _rtl8651_delAlgEntryFromAsic()
 *		. _rtl8651_addAlgEntryToAsic()
 *		. _rtl8651_arrangeAsicAlg()
 *	2. Internal APIs
 *		. _rtl8651_getAlgEntry()
 *	3. External APIs
 *		. rtl8651_addALGRule()
 *		. rtl8651_delALGRule()
 *=======================================================================================*/
static uint32 _rtl8651_algToAclCount(void) {
	rtl8651_tblDrv_algEntry_t *trackAlgPtr;
	uint32 count = 0;

	SLIST_FOREACH(trackAlgPtr, &DrvTbl.inuseList.alg, nextAlgEntry) {
		if (trackAlgPtr->toAsic == 1)		continue;
		if (trackAlgPtr->tcpServer == 1)	count++;
		if (trackAlgPtr->tcpClient == 1) 	count++;
		if (trackAlgPtr->udpServer == 1)	count++;
		if (trackAlgPtr->udpClient == 1)	count++;
	}
	return count;
}


static int32 _rtl8651_delAlgEntryFromAsic(rtl8651_tblDrv_algEntry_t *algPtr) {
	rtl8651_tblDrv_algAsicArrangementEntry_t *asicAlgPtr;
	int32 algIdx, count = 0;

 	asicAlgPtr=&AsicTableUsage.alg[0];
	for(algIdx=0; algIdx<RTL8651_ALGTBL_SIZE; algIdx++, asicAlgPtr++)
		if (asicAlgPtr->valid == 1 && 
			asicAlgPtr->port >= algPtr->startPort && asicAlgPtr->port <= algPtr->endPort) {
			asicAlgPtr->valid = 0;
			rtl8651_delAsicAlg(algIdx);
		count++;
	}
	algPtr->toAsic = 0;
	return count;
}

static int32 _rtl8651_addAlgEntryToAsic(rtl8651_tblDrv_algEntry_t *algPtr) {
	rtl8651_tblDrv_algAsicArrangementEntry_t *asicAlgPtr;
	uint16 port = algPtr->startPort;
	int32 algIdx;

	asicAlgPtr=&AsicTableUsage.alg[0];
	for(algIdx=0; algIdx<RTL8651_ALGTBL_SIZE&&port<=algPtr->endPort; algIdx++, asicAlgPtr++)
		if (asicAlgPtr->valid == 0) {
			asicAlgPtr->port = port;
			asicAlgPtr->valid = 1;
			convert_setAsicAlg(algIdx, port);
			port++;
		}
	algPtr->toAsic = 1;
	return (port - algPtr->startPort);
}


static void _rtl8651_arrangeAsicAlg(void) {
	rtl8651_tblDrv_algEntry_t *trackAlgPtr, *maxAlgEntry=NULL, *minAclEntry=NULL;
	uint32 count = 0, maxAlgCount, minAclCount, cnt;

	/* After the loop, the following information will be derived:
	 *	1. The entry with maximal port range in the ASIC ALG table
	 *	2. How many ASIC ALG entry were occupied.
	 *	3. The entry with minimal port range in the ACL table
	 */
	SLIST_FOREACH(trackAlgPtr, &DrvTbl.inuseList.alg, nextAlgEntry) {
		if (trackAlgPtr->toAsic == 1) {
			count += (trackAlgPtr->endPort-trackAlgPtr->startPort+1);
			maxAlgEntry = trackAlgPtr;
		}
		else if (minAclEntry == NULL)
			minAclEntry = trackAlgPtr;
	}


	/* Four cases should be handled individually:
	 *	1. maxAlgEntry == NULL && minAclEntry == NULL
	 *	   it implies the ALG table is empty, do nothing!
	 *	2. maxAlgEntry == NULL && minAclEntry != NULL
	 *	   no ALG entry in the ALG table but some are merged to ACL table,
	 *	   try to move ALG entry in the ACL table to ALG table
	 *	3. maxAlgEntry != NULL && minAclEntry == NULL
	 *	   do nothing!!
	 *	4. maxAlgEntry != NULL && minAclEntry != NULL
	 *	   if minAclEntry can not add to ALG table, try to swap out maxAlgEntry
	 *	PS: minAclEntry is the minimal range in the ACL table while
	 *		maxAlgEntry is the maximal range in the ALG table.
	 */
	 
	if (minAclEntry == (rtl8651_tblDrv_algEntry_t *)NULL) 
		return; /* Case 1 and Case 3 */
	
		
	minAclCount = (minAclEntry->endPort - minAclEntry->startPort + 1);
	if (maxAlgEntry == (rtl8651_tblDrv_algEntry_t *)NULL) { /* Case 2 */
		assert(count == 0);
		assert(minAclCount > 0);
		if (minAclCount > RTL8651_ALGTBL_SIZE) 
			return;
		cnt = _rtl8651_addAlgEntryToAsic(minAclEntry);
		assert(cnt == minAclCount);
	} else { /* Case 4 */
		assert(count > 0);
		if (RTL8651_ALGTBL_SIZE - count >= minAclCount) {
			cnt = _rtl8651_addAlgEntryToAsic(minAclEntry);
			assert(cnt == minAclCount);
			return;
		}
		maxAlgCount = (maxAlgEntry->endPort - maxAlgEntry->startPort + 1);
		if (maxAlgCount > minAclCount) {
			/* swap out maxAlgEntry */
			cnt = _rtl8651_delAlgEntryFromAsic(maxAlgEntry);
			assert(cnt == maxAlgCount);
			cnt = _rtl8651_addAlgEntryToAsic(minAclEntry);
			assert(cnt == minAclCount);
		}
	}
}


rtl8651_tblDrv_algEntry_t *_rtl8651_getAlgEntry(int8 isTcp, int8 *isServer, int8 *isClient, uint16 sport, uint16 eport) {
	rtl8651_tblDrv_algEntry_t *tmpAlgEntry;
	uint32 flag = 0;

	/* Parameter Check */
	if (isServer == FALSE && isClient == FALSE)
		return (rtl8651_tblDrv_algEntry_t *)0;
	
	/* Go though ALG list */
	SLIST_FOREACH(tmpAlgEntry, &DrvTbl.inuseList.alg, nextAlgEntry) {
		if (tmpAlgEntry->startPort == sport && tmpAlgEntry->endPort == eport) {
			flag = (isTcp==TRUE)? (tmpAlgEntry->tcpClient|tmpAlgEntry->tcpServer) :
								(tmpAlgEntry->udpClient|tmpAlgEntry->udpServer);
			if (flag) {
				*isServer = (isTcp == TRUE)? tmpAlgEntry->tcpServer : tmpAlgEntry->udpServer;
				*isClient = (isTcp == TRUE)? tmpAlgEntry->tcpClient : tmpAlgEntry->udpClient;
				return tmpAlgEntry;
			}
		}
	}
	return (rtl8651_tblDrv_algEntry_t *)0;
}

/*
@func int32 | rtl8651_addALGRule | Add a ALG port or port range to ASIC ALG Table.
@parm uint16 | startPort | start port number in the specified port range.
@parm uint16 | endPort | ending port number in the specified port range.
@parm int8 | isTcp | TRUE is for TCP Protocol, FALSE is for UDP Protocol.
@parm int8 | isServer | If the ALG rule is used for Server, TRUE is specified.
@parm int8 | isClient | If the ALG rule is used for Client, TRUE is specified.
@rvalue SUCCESS | A ALG Rule was added successfully.
@rvalue TBLDRV_EINVALALGATTR | Neither Server nor Client was specified to an entry.
@rvalue TBLDRV_EINVALPORTRANGE | Invalid port range was specified.
@rvalue TBLDRV_EENTRYALREADYEXIST | ALG rule already exists.
@rvalue TBLDRV_EALGPORTRANGEOVERLAP | The new ALG rule overlaps the port range of the one already in existence.
@rvalue TBLDRV_ENOFREEBUFFER | Out of ALG rule buffer.
@comm
Protocol Stack can add a ALG rule to trap packets which their L4 port number matchs the added ALG rule.
*/
int32 rtl8651_addALGRule(uint16 startPort, uint16 endPort, int8 isTcp, int8 isServer, int8 isClient) {
	rtl8651_tblDrv_algEntry_t *trackAlgPtr, *tmpAlgPtr, *algPtr;
	
	UNLOCK_ERROR_CHECK((isServer==FALSE&&isClient==FALSE), TBLDRV_EINVALALGATTR);
	UNLOCK_ERROR_CHECK((startPort>endPort), TBLDRV_EINVALPORTRANGE);
	rtlglue_drvMutexLock();//Lock resource	
	/* Check overlap range! If found, reject the request */
	SLIST_FOREACH(trackAlgPtr, &DrvTbl.inuseList.alg, nextAlgEntry) {
		if (trackAlgPtr->startPort == startPort && trackAlgPtr->endPort == endPort)
			break;
		LOCK_ERROR_CHECK(
			( (trackAlgPtr->startPort <= startPort && trackAlgPtr->endPort >= startPort)||
			  (trackAlgPtr->startPort <= endPort && trackAlgPtr->endPort >= endPort) ||	
			  (trackAlgPtr->startPort <= startPort && trackAlgPtr->endPort >= endPort) ||
			  (trackAlgPtr->startPort >= startPort && trackAlgPtr->endPort <= endPort)
			),
			TBLDRV_EALGPORTRANGEOVERLAP
		);
	}
	/* Allocate alg space */
	if (trackAlgPtr == (rtl8651_tblDrv_algEntry_t *) NULL) {
		LOCK_ERROR_CHECK((DrvTbl.freeList.alg.slh_first==NULL), TBLDRV_ENOFREEBUFFER);
		trackAlgPtr = SLIST_FIRST(&DrvTbl.freeList.alg);
		SLIST_REMOVE_HEAD(&DrvTbl.freeList.alg, nextAlgEntry);
		trackAlgPtr->nextAlgEntry.sle_next = NULL;
		/* Insert to inuseList in ascending order(startPort-endPort) */
		algPtr = (rtl8651_tblDrv_algEntry_t *)NULL;
		SLIST_FOREACH(tmpAlgPtr, &DrvTbl.inuseList.alg, nextAlgEntry) {
			if (tmpAlgPtr->endPort-tmpAlgPtr->startPort > endPort-startPort)
				break;
			algPtr = tmpAlgPtr;
		}
		if (algPtr == (rtl8651_tblDrv_algEntry_t *)NULL) /* Insert to head */
			SLIST_INSERT_HEAD(&DrvTbl.inuseList.alg, trackAlgPtr, nextAlgEntry);
		else SLIST_INSERT_AFTER(algPtr, trackAlgPtr, nextAlgEntry);
		trackAlgPtr->startPort = startPort;
		trackAlgPtr->endPort = endPort;
		trackAlgPtr->tcpServer = 0;
		trackAlgPtr->tcpClient = 0;
		trackAlgPtr->udpServer = 0;
		trackAlgPtr->udpClient = 0;
		trackAlgPtr->toAsic = 0;
		_rtl8651_arrangeAsicAlg();
	}
	
	/* Configure trapping property */
	if(isTcp == TRUE) {
		LOCK_ERROR_CHECK(
			((isServer==TRUE&&trackAlgPtr->tcpServer== 1) ||
			 (isClient==TRUE&&trackAlgPtr->tcpClient== 1)
			), 
			TBLDRV_EENTRYALREADYEXIST 
		);
		if(isServer == TRUE) trackAlgPtr->tcpServer = 1;
		if(isClient == TRUE) trackAlgPtr->tcpClient = 1;
	} 
	else {
		assert(isTcp == FALSE);
		LOCK_ERROR_CHECK(
			((isServer == TRUE && trackAlgPtr->udpServer == 1) ||
			 (isClient == TRUE && trackAlgPtr->udpClient == 1)
			),
			TBLDRV_EENTRYALREADYEXIST
		);
		if(isServer == TRUE) trackAlgPtr->udpServer = 1;
		if(isClient == TRUE) trackAlgPtr->udpClient = 1;
	}
	SLIST_FOREACH(tmpAlgPtr, &DrvTbl.inuseList.alg, nextAlgEntry)
		if (tmpAlgPtr->toAsic == 0)
			_rtl8651_arrangeAllACL();
	rtlglue_drvMutexUnlock();//Unlock resource
	return SUCCESS;
}

/*
@func int32 | rtl8651_delALGRule | Remove a ALG rule from ASIC ALG Table.
@parm uint16 | startPort | start port number in the specified port range.
@parm uint16 | endPort | ending port number in the specified port range.
@parm int8 | isTcp | TRUE is for TCP Protocol, FALSE is for UDP Protocol.
@parm int8 | isServer | If the ALG rule is used for Server, TRUE is specified.
@parm int8 | isClient | If the ALG rule is used for Client, TRUE is specified.
@rvalue SUCCESS | The specified ALG rule was removed from ASIC ALG Table successfully.
@rvalue TBLDRV_EINVALALGATTR | Neither Server nor Client was specified to an ALG entry.
@rvalue TBLDRV_EINVALPORTRANGE | Invalid port specified.
@rvalue TBLDRV_ERANGEOVERLAP | The new ALG rule overlaps the port range of the one already in existence.
@rvalue TBLDRV_EENTRYNOTFOUND | ALG rule not found.
@comm
This API remove the specified ALG rule from ASIC ALG Table.
*/
int32 rtl8651_delALGRule(uint16 startPort, uint16 endPort, int8 isTcp, int8 isServer, int8 isClient) {
	rtl8651_tblDrv_algEntry_t *delAlgPtr;
	
	UNLOCK_ERROR_CHECK((isServer==FALSE&&isClient==FALSE), TBLDRV_EINVALALGATTR);
	UNLOCK_ERROR_CHECK((startPort>endPort), TBLDRV_EINVALPORTRANGE);
	rtlglue_drvMutexLock();//Lock resource
	SLIST_FOREACH(delAlgPtr, &DrvTbl.inuseList.alg, nextAlgEntry) {
		if (delAlgPtr->startPort == startPort && delAlgPtr->endPort == endPort)
			break;
		LOCK_ERROR_CHECK(
			( (delAlgPtr->startPort <= startPort && delAlgPtr->endPort >= startPort) ||
			  (delAlgPtr->startPort <= endPort && delAlgPtr->endPort >= endPort) ||	
			  (delAlgPtr->startPort <= startPort && delAlgPtr->endPort >= endPort) ||
			  (delAlgPtr->startPort >= startPort && delAlgPtr->endPort <= endPort)
			),
			TBLDRV_ERANGEOVERLAP
		);
	}
	LOCK_ERROR_CHECK((delAlgPtr==NULL), TBLDRV_EENTRYNOTFOUND);
	if(isTcp == TRUE) {
		LOCK_ERROR_CHECK(
			( (delAlgPtr->tcpServer == 0 && isServer == TRUE) ||
			  (delAlgPtr->tcpClient == 0 && isClient == TRUE)
			),
			TBLDRV_EENTRYNOTFOUND
		);
		if(isServer == TRUE) delAlgPtr->tcpServer = 0;
		if(isClient == TRUE) delAlgPtr->tcpClient = 0;
	}
	else {
		assert(isTcp == FALSE);
		LOCK_ERROR_CHECK(
			( (delAlgPtr->udpServer == 0 && isServer == TRUE) ||
			  (delAlgPtr->udpClient == 0 && isClient == TRUE)
			),
			TBLDRV_EENTRYNOTFOUND
		);
		if(isServer == TRUE) delAlgPtr->udpServer = 0;
		if(isClient == TRUE) delAlgPtr->udpClient = 0;
	}
	if (delAlgPtr->tcpServer == 0 && delAlgPtr->tcpClient == 0 &&
		delAlgPtr->udpServer == 0 && delAlgPtr->udpClient == 0) {
		SLIST_REMOVE(&DrvTbl.inuseList.alg, delAlgPtr, rtl8651_tblDrv_algEntry_s, nextAlgEntry);
		SLIST_INSERT_HEAD(&DrvTbl.freeList.alg, delAlgPtr, nextAlgEntry);
		_rtl8651_delAlgEntryFromAsic(delAlgPtr);
		_rtl8651_arrangeAsicAlg();
	}
	_rtl8651_arrangeAllACL();
	rtlglue_drvMutexUnlock();//Unlock resource
	return SUCCESS;
}


#define RTL8651_DoS_APIs
/*=======================================================================================
 * RTL8651 Deny Of Service APIs:
 *	1. Static Functions
 *
 *	2. Internal APIs
 *
 *	3. External APIs
 *
 *=======================================================================================*/
typedef struct rtl8651_DoS_Table_s {
	int32 (*p_dosfunc)(uint16, uint32, uint32, int8);
/*	uint32 curtype; */
	uint32 vlanMask;
	uint32 dostype;
} rtl8651_DoS_Table_t;

/* ACL rules about DOS check is L3 and L4 (because in forwarding engine, DoS is checked for routing process, too.) */
#define SET_DOSACL_MATCHLAYER(rule) \
	rule.pktOpApp = RTL8651_ACLTBL_L3_AND_L4;

/* chenyl (2004/7/26):
	only allow one attack item in one entry now,
	de-multiplex is done in rtl8651_dos.c
*/
rtl8651_DoS_Table_t rtl8651_Dos_Table[] =
{
	{ _rtl8651_DoS_tcpSynFlood,		0x00, RTL8651_TCP_SYNFLOOD/*|RTL8651_TCP_SYNWITHDATA|RTL8651_TCP_LANDATTACK */}, /* TCP Syn Flood, Syn with data, land Attack */
	{ _rtl8651_DoS_tcpFinFlood,		0x00, RTL8651_TCP_STEALTHFIN }, /* TCP Fin Flood (StealthFIN) */
	{ _rtl8651_DoS_udpEchoChargen,	0x00, RTL8651_UDP_ECHOCHARGENBOMB },  /* UDP echo & chargen */
	{ _rtl8651_DoS_fullXmasScan,	0x00, RTL8651_TCP_FULLXMASSCAN }, /* Tcp Full Xmas Scan */
	{ _rtl8651_DoS_fullSapuScan,		0x00, RTL8651_TCP_FULLSAPUSCAN }, /* Full Sapu Scan */
	{ _rtl8651_DoS_finScan,			0x00, RTL8651_TCP_FINSCAN }, /* FIN Scan */
	{ _rtl8651_DoS_synFinScan,		0x00, RTL8651_TCP_SYNFINSCAN }, /* SynFin Scan */
	{ _rtl8651_DoS_nullScan,			0x00, RTL8651_TCP_NULLSCAN },  /* NULL Scan */
	{ _rtl8651_DoS_nmapXmasScan,	0x00, RTL8651_TCP_NMAPXMASSCAN }, /* Nmap Xmas Scan */
	{ _rtl8651_DoS_vecnaScan,		0x00, RTL8651_TCP_VECNASCAN }, /* Vecna Scan */
	{ _rtl8651_DoS_ipSpoofing,		0x00, RTL8651_IP_SPOOFING }, /* IP Spoofing Attack */
	{ _rtl8651_DoS_udpFlood,		0x00, RTL8651_UDP_FLOOD }, /* UDP Flood Attack */
	{ _rtl8651_DoS_icmpFlood,		0x00, /* RTL8651_ICMP_SMURF|*/RTL8651_ICMP_FLOOD }, /* ICMP smurf, ICMP flood Attack */
	{_rtl8651_DoS_portScan,			0x00, RTL8651_TCPUDP_PORTSCAN},	/* tcp syn/fin/ack port scan, udp port scan */
	{ NULL,							0x00, 0x00 },
};


static int32 _rtl8651_DoS_tcpSynFlood(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr) {
 	_rtl8651_tblDrvAclRule_t rule;

 	/* Layer 4 => TCP */
 	/* Trap all TCP packets with SYN bit set to CPU */
	assert((aclStart<=aclEnd) && (aclEnd<RTL8651_ACLTBL_SIZE));
	if (wr == TRUE) {
		memset(&rule, 0, sizeof(_rtl8651_tblDrvAclRule_t));
		rule.ruleType_     = RTL8651_ACL_TCP;
	 	rule.actionType_   = RTL8651_ACL_CPU;
	 	rule.tcpSYN_       = 0x01;
	 	rule.tcpFlagMask_  = rule.tcpFlag_; /* SYN bit set */
	 	rule.tcpSrcPortUB_ = 0xffff;
	 	rule.tcpSrcPortLB_ = 0x00;
	 	rule.tcpDstPortUB_ = 0xffff;
	 	rule.tcpDstPortLB_ = 0x00;
		SET_DOSACL_MATCHLAYER(rule);
	 	rtl8651_setAsicAclRule(aclStart, &rule);
 	}
	return 1;
}

static int32 _rtl8651_DoS_tcpFinFlood(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr) {
 	_rtl8651_tblDrvAclRule_t rule;

 	/* Layer 4 => TCP */
 	/* Trap all TCP packets with FIN bit set to CPU */
	assert((aclStart<=aclEnd) && (aclEnd<RTL8651_ACLTBL_SIZE));
	if (wr == TRUE) {
		memset(&rule, 0, sizeof(_rtl8651_tblDrvAclRule_t));
		rule.ruleType_     = RTL8651_ACL_TCP;
	 	rule.actionType_   = RTL8651_ACL_CPU;
	 	rule.tcpFIN_       = 0x01;
	 	rule.tcpFlagMask_  = rule.tcpFlag_; /* FIN bit set */
	 	rule.tcpSrcPortUB_ = 0xffff;
	 	rule.tcpSrcPortLB_ = 0x00;
	 	rule.tcpDstPortUB_ = 0xffff;
	 	rule.tcpDstPortLB_ = 0x00;
		SET_DOSACL_MATCHLAYER(rule);
	 	rtl8651_setAsicAclRule(aclStart, &rule);
 	}
	return 1;
}


static int32 _rtl8651_DoS_icmpFlood(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr) {
	_rtl8651_tblDrvAclRule_t rule;

	/* Layer 3 => ICMP */
	/* Trap all ICMP echo request packets to CPU */
	assert((aclStart<=aclEnd) && (aclEnd<RTL8651_ACLTBL_SIZE));
	if (wr == TRUE) {
		memset(&rule, 0, sizeof(_rtl8651_tblDrvAclRule_t));
		rule.ruleType_     = RTL8651_ACL_ICMP;
	 	rule.actionType_   = RTL8651_ACL_CPU;
		rule.icmpType_     = ICMP_ECHO;
		rule.icmpTypeMask_ = 0xff;
		SET_DOSACL_MATCHLAYER(rule);
		rtl8651_setAsicAclRule(aclStart, &rule);
	}
	return 1;
}

static int32 _rtl8651_DoS_udpFlood(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr) {
	_rtl8651_tblDrvAclRule_t rule;

 	/* Layer 3 => IP */
 	/* Trap all UDP packets to cPU */
	assert((aclStart<=aclEnd) && (aclEnd<RTL8651_ACLTBL_SIZE));
	if (wr == TRUE) {
		memset(&rule, 0, sizeof(_rtl8651_tblDrvAclRule_t));
		rule.ruleType_    = RTL8651_ACL_IP;
	 	rule.actionType_  = RTL8651_ACL_CPU;
		rule.ipProto_     = IPPROTO_UDP;
		rule.ipProtoMask_ = 0xff;
		SET_DOSACL_MATCHLAYER(rule);
		rtl8651_setAsicAclRule(aclStart, &rule);
	}
	return 1;
}

static int32 _rtl8651_DoS_ipSpoofing(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr) {
	rtl8651_tblDrv_vlanTable_t *vlan_t = &DrvTbl.vlan[rtl8651_vlanTableIndex(vid)];
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_ipIntfEntry_t *ipintf_t;
	_rtl8651_tblDrvAclRule_t rule;
	int32 vidx, count = 0;
	
	/* Layer 3 => IP */
	/* For each external LAN, if an inbound packet with source IP address belongs
	 * to an Internal VLAN, trap it to CPU.
	 */
	if (vlan_t->internal == 1)
		return 0; /* Only external vlan is allowed */
	assert((aclStart<=aclEnd) && (aclEnd<RTL8651_ACLTBL_SIZE));
	vlan_t = &DrvTbl.vlan[0];
	for(vidx=0; vidx<RTL8651_VLAN_NUMBER; vidx++, vlan_t++) {
		if (vlan_t->valid == 0 || vlan_t->internal == 0 || vlan_t->netif_t == 0)
			continue; /* filter out external VLAN and L2 vlan */

		netif_t = vlan_t->netif_t;
		SLIST_FOREACH(ipintf_t, &netif_t->IpHead, nextIp) {
			if (ipintf_t->lanSideExternal == 1)
				continue; /* filter out external interface */
			if (wr == TRUE) {
				RTL8651_IP_ACL_RULE(
					&rule, 
					RTL8651_ACL_CPU, 
					(ipintf_t->ipAddr.slh_first->ipAddr)&ipintf_t->ipMask, 
					ipintf_t->ipMask, 
					0, 0, 0, 0, 0, 0, 0, 0
				);
				SET_DOSACL_MATCHLAYER(rule);
				rtl8651_setAsicAclRule(aclStart, &rule);
				aclStart++;
				if (aclStart > aclEnd)
					return ++count;
			}
			count++;
		}
	}
	return count;
}

static int32 _rtl8651_DoS_udpEchoChargen(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr) {
	_rtl8651_tblDrvAclRule_t rule;
	uint16 port[] = { 7, 17, 19 };	
	int32 count=0, pidx;

	/* Layer 4: UDP */
	/* Trap all UDP packets with port=[7, 17, 19] set to CPU */
	assert((aclStart<=aclEnd) && (aclEnd<RTL8651_ACLTBL_SIZE));
	if (wr == TRUE) {
		for(pidx=0; pidx<3; pidx++) {
			RTL8651_UDP_ACL_RULE(
				&rule, 
				RTL8651_ACL_CPU, 
				0, 0, 0, 0, 0, 0, 0xffff, 0x00, 
				port[pidx], port[pidx]
			);
			SET_DOSACL_MATCHLAYER(rule);
			rtl8651_setAsicAclRule(aclStart, &rule);
			aclStart++; count++;
			if (aclStart > aclEnd)
				return count;
		}
	}
	return 3;
}


static int32 _rtl8651_DoS_fullXmasScan(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr) {
	_rtl8651_tblDrvAclRule_t rule;

	/* Layer 4: TCP */
	/* Trap All TCP packets with all TCP flag set */
	assert((aclStart<=aclEnd) && (aclEnd<RTL8651_ACLTBL_SIZE));
	if (wr == TRUE) {
		memset(&rule, 0, sizeof(_rtl8651_tblDrvAclRule_t));
		rule.ruleType_     = RTL8651_ACL_TCP;
		rule.actionType_   = RTL8651_ACL_CPU;
		rule.tcpSrcPortUB_ = 0xffff;
		rule.tcpSrcPortLB_ = 0x00;
		rule.tcpDstPortUB_ = 0xffff;
		rule.tcpDstPortLB_ = 0x00;
		rule.tcpFlagMask_  = 0xff;
		rule.tcpFlag_      = 0xff;
		SET_DOSACL_MATCHLAYER(rule);
		rtl8651_setAsicAclRule(aclStart, &rule);
	}
	return 1;
}

static int32 _rtl8651_DoS_fullSapuScan(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr) {
	_rtl8651_tblDrvAclRule_t rule;

	/* Layer 4: TCP */
	/* Trap All TCP packets with only 4 bits[TH_URG|TH_ACK|TH_PUSH|TH_SYN] TCP flag set */
	assert((aclStart<=aclEnd) && (aclEnd<RTL8651_ACLTBL_SIZE));
	if (wr == TRUE) {
		memset(&rule, 0, sizeof(_rtl8651_tblDrvAclRule_t));
		rule.ruleType_     = RTL8651_ACL_TCP;
		rule.actionType_   = RTL8651_ACL_CPU;
		rule.tcpSrcPortUB_ = 0xffff;
		rule.tcpSrcPortLB_ = 0x00;
		rule.tcpDstPortUB_ = 0xffff;
		rule.tcpDstPortLB_ = 0x00;
		rule.tcpFlagMask_  = 0xff;
		rule.tcpURG_       = 0x01;
		rule.tcpACK_	   = 0x01;
		rule.tcpPSH_	   = 0x01;
		rule.tcpSYN_	   = 0x01;
		SET_DOSACL_MATCHLAYER(rule);
		rtl8651_setAsicAclRule(aclStart, &rule);
	}
	return 1;
}

static int32 _rtl8651_DoS_finScan(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr) {
 	_rtl8651_tblDrvAclRule_t rule;

 	/* Layer 4 => TCP */
	assert((aclStart<=aclEnd) && (aclEnd<RTL8651_ACLTBL_SIZE));
	if (wr == TRUE) {
		memset(&rule, 0, sizeof(_rtl8651_tblDrvAclRule_t));
		rule.ruleType_     = RTL8651_ACL_TCP;
	 	rule.actionType_   = RTL8651_ACL_CPU;
	 	rule.tcpFIN_       = 0x01;
	 	rule.tcpFlagMask_  = 0xff; /* only FIN bit set */
	 	rule.tcpSrcPortUB_ = 0xffff;
	 	rule.tcpSrcPortLB_ = 0x00;
	 	rule.tcpDstPortUB_ = 0xffff;
	 	rule.tcpDstPortLB_ = 0x00;
		SET_DOSACL_MATCHLAYER(rule);
	 	rtl8651_setAsicAclRule(aclStart, &rule);
 	}
	return 1;
}

static int32 _rtl8651_DoS_synFinScan(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr) {
	_rtl8651_tblDrvAclRule_t rule;
	
	/* Layer 4 => TCP */
	/* Trap TCP packets with only SYN and FIN flags set to CPU */
	assert((aclStart<=aclEnd) && (aclEnd<RTL8651_ACLTBL_SIZE));
	if (wr == TRUE) {
		memset(&rule, 0, sizeof(_rtl8651_tblDrvAclRule_t));
		rule.ruleType_     = RTL8651_ACL_TCP;
	 	rule.actionType_   = RTL8651_ACL_CPU;
	 	rule.tcpFIN_       = 0x01;
	 	rule.tcpSYN_	   = 0x01;
	 	rule.tcpFlagMask_  = 0xff; /* only FIN & SYN bits set */
	 	rule.tcpSrcPortUB_ = 0xffff;
	 	rule.tcpSrcPortLB_ = 0x00;
	 	rule.tcpDstPortUB_ = 0xffff;
	 	rule.tcpDstPortLB_ = 0x00;
		SET_DOSACL_MATCHLAYER(rule);
	 	rtl8651_setAsicAclRule(aclStart, &rule);
 	}
	return 1;
}

static int32 _rtl8651_DoS_nullScan(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr) {
	_rtl8651_tblDrvAclRule_t rule;

	/* Layer 4: TCP */
	/* Trap TCP packets with null flag set */
	assert((aclStart<=aclEnd) && (aclEnd<RTL8651_ACLTBL_SIZE));
	if (wr == TRUE) {
		memset(&rule, 0, sizeof(_rtl8651_tblDrvAclRule_t));
		rule.ruleType_     = RTL8651_ACL_TCP;
	 	rule.actionType_   = RTL8651_ACL_CPU;
	 	rule.tcpFlag_	   = 0x00;
	 	rule.tcpFlagMask_  = 0xff; 
	 	rule.tcpSrcPortUB_ = 0xffff;
	 	rule.tcpSrcPortLB_ = 0x00;
	 	rule.tcpDstPortUB_ = 0xffff;
	 	rule.tcpDstPortLB_ = 0x00;
		SET_DOSACL_MATCHLAYER(rule);
	 	rtl8651_setAsicAclRule(aclStart, &rule);
 	}
	return 1;
}

static int32 _rtl8651_DoS_nmapXmasScan(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr) {
	_rtl8651_tblDrvAclRule_t rule;

	/* Layer 4: TCP */
	assert((aclStart<=aclEnd) && (aclEnd<RTL8651_ACLTBL_SIZE));
	if (wr == TRUE) {
		memset(&rule, 0, sizeof(_rtl8651_tblDrvAclRule_t));
		rule.ruleType_     = RTL8651_ACL_TCP;
	 	rule.actionType_   = RTL8651_ACL_CPU;
		rule.tcpFIN_	   = 0x01;
		rule.tcpPSH_	   = 0x01;
		rule.tcpURG_	   = 0x01;
	 	rule.tcpFlagMask_  = 0xff; 
	 	rule.tcpSrcPortUB_ = 0xffff;
	 	rule.tcpSrcPortLB_ = 0x00;
	 	rule.tcpDstPortUB_ = 0xffff;
	 	rule.tcpDstPortLB_ = 0x00;
		SET_DOSACL_MATCHLAYER(rule);
	 	rtl8651_setAsicAclRule(aclStart, &rule);
 	}
	return 1;
}

static int32 _rtl8651_DoS_vecnaScan(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr) {
	_rtl8651_tblDrvAclRule_t rule;
	int32 i, count = 0;

	/* Layer 4: TCP */
	assert((aclStart<=aclEnd) && (aclEnd<RTL8651_ACLTBL_SIZE));
	if (wr == TRUE) {
		memset(&rule, 0, sizeof(_rtl8651_tblDrvAclRule_t));
		rule.ruleType_     = RTL8651_ACL_TCP;
	 	rule.actionType_   = RTL8651_ACL_CPU;
	 	rule.tcpFlagMask_  = 0xff; 
	 	rule.tcpSrcPortUB_ = 0xffff;
	 	rule.tcpSrcPortLB_ = 0x00;
	 	rule.tcpDstPortUB_ = 0xffff;
	 	rule.tcpDstPortLB_ = 0x00;
		for(i=0; i<5; i++) {
			switch(i) {
			case 0: rule.tcpURG_ = 0x01; break;
			case 1: rule.tcpPSH_ = 0x01; break;
			case 2: rule.tcpURG_ = 0x01; rule.tcpFIN_ = 0x01; break;
			case 3: rule.tcpPSH_ = 0x01; rule.tcpFIN_ = 0x01; break;
			case 4: rule.tcpURG_ = 0x01; rule.tcpPSH_ = 0x01; break;
			default: assert(0);
			}
			SET_DOSACL_MATCHLAYER(rule);
			rtl8651_setAsicAclRule(aclStart, &rule);
			count++; aclStart++;
			if (aclStart> aclEnd)
				return count;
			rule.tcpFlag_ = 0x00;
		}	 	
 	} else count = 5;
	return count;
}

static int32 _rtl8651_DoS_portScan(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr)
{
 	_rtl8651_tblDrvAclRule_t rule;

 	/* Layer 4 => TCP */
 	/* Trap all TCP packets with RST bit set to CPU */
	assert((aclStart<=aclEnd) && (aclEnd<RTL8651_ACLTBL_SIZE));
	if (wr == TRUE) {
		memset(&rule, 0, sizeof(_rtl8651_tblDrvAclRule_t));
		rule.ruleType_		= RTL8651_ACL_TCP;
	 	rule.actionType_	= RTL8651_ACL_CPU;
	 	rule.tcpRST_		= 0x01;
	 	rule.tcpFlagMask_	= rule.tcpFlag_; /* RST bit set */
	 	rule.tcpSrcPortUB_	= 0xffff;
	 	rule.tcpSrcPortLB_	= 0x00;
	 	rule.tcpDstPortUB_	= 0xffff;
	 	rule.tcpDstPortLB_	= 0x00;
		SET_DOSACL_MATCHLAYER(rule);
	 	rtl8651_setAsicAclRule(aclStart, &rule);
 	}
	return 1;
}

static int32 _rtl8651_addAclRuleForDoS(uint16 vid, uint32 aclStart, uint32 aclEnd, int8 wr) {
	rtl8651_DoS_Table_t *dos_t;
	int32 count=0, addCount=0;

//	assert(aclEnd<RTL8651_ACLTBL_SIZE);
//	if (aclStart > aclEnd)
//		return 0;
	for(dos_t=&rtl8651_Dos_Table[0]; dos_t->p_dosfunc; dos_t++)
	{
		if (dos_t->vlanMask)
		{
			int32 i;
			rtl8651_tblDrv_vlanTable_t *vlanp;
			int32 addAcl = FALSE;

			for (i = 0 ; i < RTL8651_VLAN_NUMBER ; i ++)
			{
				if (!((1<<i)&dos_t->vlanMask))
					continue;
				vlanp = &DrvTbl.vlan[i];
				if (vlanp->vid == vid)
				{
					addAcl = TRUE;
					goto add;
				}
			}
add:
			if (addAcl == TRUE)
			{
				addCount = (dos_t->p_dosfunc)(vid, aclStart, aclEnd, wr);
				count+=addCount;
				if (wr == TRUE) {
					aclStart += addCount;
					if (aclStart > aclEnd)
						return count;
				}
			}
		}
	}
	return count;
}

/*
	when enable one dos item, its vlanMask will be updated to "vlanMask"
	if enable item, but the vlanMask == 0, we disable this item
	if disable item, then we disable this item regardless of "vlanMask"
*/
int32 _rtl8651_enableDoS(uint32 dostype, int8 enable, uint32 vlanMask) {
	rtl8651_DoS_Table_t *dos_t;
	uint32 i, mask, vlanBit;

	/* get the mask of vlan */
	vlanBit = 0;
	for (i = 0 ; i < RTL8651_VLAN_NUMBER ; i ++)
		vlanBit = (vlanBit << 1) + 1;
	mask = vlanMask & vlanBit;

	for(dos_t=&rtl8651_Dos_Table[0]; dos_t->p_dosfunc; dos_t++)
		if (dostype == dos_t->dostype) {
			if ((enable == TRUE) && mask && (dos_t->vlanMask != mask))
			{
				dos_t->vlanMask = mask;
				_rtl8651_arrangeAllACL();
			}
			if (((enable == FALSE) || (enable == TRUE && !mask)) && (dos_t->vlanMask))
			{
				dos_t->vlanMask = 0;
				_rtl8651_arrangeAllACL();
			}
			return SUCCESS;
		}
	return FAILED;
}


#define RTL8651_MISC_APIs
 void rtl8651_updatePortStatus(uint32 port, int8 linkUp) {
	//cfliu: 
	//All Layer 3/4 related tasks to be done when Port link status change such as
	//ARP,L2,NAPT  table flushing ....should be in this function...
	
	rtlglue_drvMutexLock();
	//remove all arp entries learned from this port.
	if(linkUp == FALSE) 
		_rtl8651_removeArpAndNaptFlowWhenLinkDown(port,NULL);

	rtlglue_drvMutexUnlock();
	return;
}
 
/*=======================================
  * CLASS A/B patching
  *=======================================*/
#define RTL865X_PATCH_FOR_ASIC_ARP_ENTRY_LIMITATION

int32 _rtl8651_ipClassMonitoring(ipaddr_t dip, rtl8651_tblDrv_arpEntry_t *arp_t)
{
	ipClassABMonitorTable_t *entry_t;
	uint32 index;

	
	assert(arp_t && dip);

	entry_t = ipClassABMonitorTable;
	for(index=0; index<ipClassCount; index++, entry_t++) {
		if (dip == entry_t->dip) {
			entry_t->load ++;
			//If one entry become the candidate of routing acceleration, just add one proper route into hardware routing table.
			if(entry_t->load == (IP_CLASS_THRESHOLD+1))
				_rtl8651_ipClassTimeUpdate();
			return SUCCESS;
		}
	}
	/* Allocate new entry */
	if (ipClassCount < IP_CLASS_MAX_ENTRY) {
		entry_t = &ipClassABMonitorTable[ipClassCount++];
		entry_t->dip 	 = dip;
		entry_t->arp_t = arp_t;
		entry_t->load 	 = 1;
		return SUCCESS;
	}
	return FAILED;
}



void _rtl8651_ipClassTimeUpdate(void)
{
	rtl8651_tblDrv_routeTable_t *l3Entry_t;
	ipClassABMonitorTable_t *entry_t, *select_t;
	rtl865x_tblAsicDrv_routingParam_t asic_l3;
	rtl8651_tblDrv_filterDbTableEntry_t *macInfo;
	uint32 max_l3;
	uint32 index, count;
	int32 retval;


	/* If no connection is under monitoring or out of usable routing entry, do nothing!! */
	if (ipClassCount == 0 || rtEntry == (RTL8651_ROUTINGTBL_SIZE-1))
		return;
	/* Initial local variable */
	max_l3 = RTL8651_ROUTINGTBL_SIZE - rtEntry - 1;
	count = 0;

	while (max_l3>0 && count<ipClassCount) {
		/* In the monitoring list, find a maximal one */
		entry_t = select_t = ipClassABMonitorTable;
		for(index=0; index<ipClassCount; index++, entry_t++) {
			if (entry_t->load > select_t->load)
				select_t = entry_t;
		}
		/* If the load of the select one is greater than THRESHOLD, write it to ASIC */
		if (select_t->load > IP_CLASS_THRESHOLD) {
			l3Entry_t = DrvTbl.route; max_l3 --;
			while (cur_rt_t->valid) {	// This will not cause infinite loop because there must be a free(invalid) routing entry here.
				if (--cur_rt_t == DrvTbl.route)
					cur_rt_t = &DrvTbl.route[RTL8651_ROUTINGTBL_SIZE-2];	
			}
			assert(select_t->arp_t->macInfo);
			macInfo = select_t->arp_t->macInfo;
			asic_l3.ipAddr			= select_t->dip;
			asic_l3.ipMask			= 0xffffffff;
			asic_l3.process			= RT_L2;
			asic_l3.vidx			= select_t->arp_t->vid;
			asic_l3.nextHopRow		= rtl8651_filterDbIndex(&macInfo->macAddr);
			asic_l3.nextHopColumn	= macInfo->asicPos;
			retval = rtl8651_setAsicRouting( L3_TABLE_INDEX(cur_rt_t), &asic_l3);
			assert(retval == SUCCESS);
			cur_rt_t->ipAddr = select_t->dip;	
			if (--cur_rt_t == DrvTbl.route)
				cur_rt_t = &DrvTbl.route[RTL8651_ROUTINGTBL_SIZE-2];	
			select_t->load = 0;
		}
		else 	break;
		count ++;
		
	}

	/* Reset the monitoring table */
	ipClassCount = 0;
	bzero(ipClassABMonitorTable, sizeof(ipClassABMonitorTable_t)*ipClassCount);
}








/*=======================================
  * QoS
  *=======================================*/
#define RTL865X_QOS_FUNCTION

static uint32 _rtl8651_tryAddAclForPolicyBasedQos(rtl8651_tblDrv_networkIntfTable_t *netif_t, uint32 aclStart, uint32 aclEnd)
{
	_rtl8651_tblDrvPolicy_t *policy_t;
	_rtl8651_tblDrvAclRule_t rule;
	uint32 count=0;

	assert(aclEnd < RTL8651_ACLTBL_SIZE);
	if (aclStart > aclEnd)	
		return 0;
	bzero(&rule, sizeof(_rtl8651_tblDrvAclRule_t));
	TAILQ_FOREACH(policy_t, &netif_t->policyQosHead, nextPolicy_t) {
		switch(policy_t->proto)
		{
		case RTL8651_POLICY_TCP:
			rule.ruleType_ 		= RTL8651_ACL_TCP;
			rule.tcpSrcPortUB_	= policy_t->sport_end;
			rule.tcpSrcPortLB_	= policy_t->sport_start;
			rule.tcpDstPortUB_	= policy_t->dport_end;
			rule.tcpDstPortLB_	= policy_t->dport_start;
			goto commL3;
			
		case RTL8651_POLICY_UDP:
			rule.ruleType_  = RTL8651_ACL_UDP;
			rule.udpSrcPortUB_	= policy_t->sport_end;
			rule.udpSrcPortLB_	= policy_t->sport_start;
			rule.udpDstPortUB_	= policy_t->dport_end;
			rule.udpDstPortLB_	= policy_t->dport_start;
			goto commL3;
			
		case RTL8651_POLICY_IP:
			rule.ruleType_ = RTL8651_ACL_IP;
commL3:
			rule.srcIpAddr_	= policy_t->sip;
			rule.srcIpAddrMask_= policy_t->sipMask;
			rule.dstIpAddr_	= policy_t->dip;
			rule.dstIpAddrMask_= policy_t->dipMask;
			break;
			
		default: assert(0);

		}
		rule.priority_ = 1;
		rule.actionType_ = RTL8651_ACL_PERMIT;
		rtl8651_setAsicAclRule(aclStart, &rule);
		aclStart++; count++;
		if (aclStart > aclEnd)
			return count;
	}
	return count;		
}


/*
@func int32 | rtl8651_setFlowControl | Set Flow Control.
@parm uint32 | port | Physical port number.
@parm int8 | enable | TRUE: Enable flow control, FALSE: Disable flow control.
@rvalue TBLDRV_EINVALIDPORT | Invalid Port Number.
@rvalue SUCCESS | Success API call.
@comm
This API is used to set a port's flow control. 
*/

int32 rtl8651_setFlowControl(uint32 port, int8 enable)
{
	int32 retval;
	
	if (port >= RTL8651_PORT_NUMBER && port != RTL8651_CPU_PORT)
		return TBLDRV_EINVALIDPORT;
	rtlglue_drvMutexLock();
	retval = rtl8651_setAsicFlowControlRegister(port, enable);
	rtlglue_drvMutexUnlock();
	return retval;
}


/*
@func int32 | rtl8651_setQueueScheduling | Set High/Low Queue scheduling.
@parm uint32 | type | Specify which packet scheduling will be used. Currently, only RTL8651_QOS_RR is supported.
@parm uint32 | weight | Specify high/low queue wieght. RTL8651_RR_H4_L1: 4:1, RTL8651_RR_H8_L1: 8:1, RTL8651_RR_H16_L1: 16:1, RTL8651_RR_H_FIRST: Always high queue first.
@rvalue TBLDRV_EINVALIDINPUT | Invalid Input parameter.
@rvalue SUCCESS | Success API call.
@comm
This API is used to set queue scheduling and the high/low queue weight. Currently we only support round-robin packet scheduling.
*/
int32 rtl8651_setQueueScheduling(uint32 type, uint32 weight)
{
	int32 retval=TBLDRV_EINVALIDINPUT;
	
	if (type != RTL8651_QOS_RR || weight > 0x03)
		return TBLDRV_EINVALIDINPUT;
	rtlglue_drvMutexLock();
	retval = rtl8651_setAsicHLQueueWeight(weight);	
	rtlglue_drvMutexUnlock();
	return retval;
}


/*
@func int32 | rtl8651_setPortPriorityQueue | Set Port's priority.
@parm uint32 | port | Physical port number.
@parm int8 | highPriority | TRUE: high, FALSE: low.
@rvalue TBLDRV_EINVALIDINPUT | Invalid input parameter.
@rvalue SUCCESS | Success API call.
@comm
If a port is set to have high priority, packets received from the port will have high priority when send it to output queue.
*/
int32 rtl8651_setPortPriorityQueue(uint32 port, int8 highPriority)
{
	int32 retval = TBLDRV_EINVALIDINPUT;
	
	if (port > 0x05)
		return TBLDRV_EINVALIDINPUT;
	rtlglue_drvMutexLock();
	retval = rtl8651_setAsicPortPriority(port, highPriority);
	rtlglue_drvMutexUnlock();
	return retval;
}



int32 rtl8651_setDiffServDSCP(uint32 dscp, int8 highPriority)
{
	int32 retval = FAILED;
	
	if (dscp > 63)
		return TBLDRV_EINVALIDINPUT;
	rtlglue_drvMutexLock();
	retval = rtl8651_setAsicDiffServReg(dscp, highPriority);
	rtlglue_drvMutexUnlock();
	return retval;
}


/*
@func int32 | rtl8651_addPolicyBasedQoS | Add a policy-based QoS.
@parm int8 * | ifName | Network interface name.
@parm rtl8651_tblDrvPolicy_t * |policy_t | Policy rule.
@parm int8 | highPriority | TRUE: high priority, FALSE: low priority.
@rvalue SUCCESS | Success API call.
@rvalue TBLDRV_EINVALIDINPUT | Invalid Input paramter.
@rvalue TBLDRV_EENTRYNOTFOUND | The specified entry was not found.
@rvalue TBLDRV_ENOFREEBUFFER | Out of memory.
@comm
If a packet match a rule added by this API, the packet will have high/low priority when send it to output queue.
*/

int32 rtl8651_addPolicyBasedQoS(int8 *ifName, rtl8651_tblDrvPolicy_t *policy_t, int8 highPriority)
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	_rtl8651_tblDrvPolicy_t *qosPolicy_t;
	int32 retval = SUCCESS;

	if (policy_t == NULL || ifName == NULL || policy_t->proto < 0x01 || policy_t->proto > 0x03)	
		return TBLDRV_EINVALIDINPUT;
	if (highPriority == FALSE) //if low priority, do nothing
		return SUCCESS;
	rtlglue_drvMutexLock();
	if ((netif_t=GET_NETWORK_INTERFACE(ifName, NULL)) == NULL) {
		retval = TBLDRV_EENTRYNOTFOUND;
		goto out;
	}
	if ((qosPolicy_t=TAILQ_FIRST(&DrvTbl.freeList.freePolicy)) == NULL) {
		retval = TBLDRV_ENOFREEBUFFER;
		goto out;
	}
	bzero(qosPolicy_t, sizeof(_rtl8651_tblDrvPolicy_t)-sizeof(qosPolicy_t->nextPolicy_t));
	rtl8651_memcpy(qosPolicy_t, policy_t, sizeof(rtl8651_tblDrvPolicy_t));
	TAILQ_REMOVE(&DrvTbl.freeList.freePolicy, qosPolicy_t, nextPolicy_t);
	CTAILQ_INSERT_TAIL(&netif_t->policyQosHead, qosPolicy_t, nextPolicy_t);
	_rtl8651_arrangeAllACL();

out:
	rtlglue_drvMutexUnlock();
	return retval;
}


/*
@func int32 | rtl8651_delPolicyBasedQoS | Remove Policy-Based QoS rule.
@parm init8 * | ifName | Network interface name.
@parm rtl8651_tblDrvPolicy_t * | Specify the Policy-Based QoS rule.
@rvalue SUCCESS | Success.
@rvalue TBLDRV_EENTRYNOTFOUND | The specified rule was not found.
@comm
Remove a Policy-Based rule added by rtl8651_addPolicyBasedQoS API.
*/
int32 rtl8651_delPolicyBasedQoS(int8 *ifName, rtl8651_tblDrvPolicy_t *policy_t)
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	_rtl8651_tblDrvPolicy_t *qosPolicy_t;
	int32 retval = SUCCESS;
	
	rtlglue_drvMutexLock();
	if ((netif_t=GET_NETWORK_INTERFACE(ifName, NULL)) == NULL) {
		retval = TBLDRV_EENTRYNOTFOUND;
		goto out;
	}
	CTAILQ_FOREACH(qosPolicy_t, &netif_t->policyQosHead, nextPolicy_t) {
		if (!memcmp(qosPolicy_t, policy_t, sizeof(rtl8651_tblDrvPolicy_t)-sizeof(ipaddr_t))) {
			CTAILQ_REMOVE(&netif_t->policyQosHead, qosPolicy_t, nextPolicy_t);
			TAILQ_INSERT_HEAD(&DrvTbl.freeList.freePolicy, qosPolicy_t, nextPolicy_t);
			break;
		}
	}
	_rtl8651_arrangeAllACL();
out:
	rtlglue_drvMutexUnlock();
	return retval;
}


/*
@func int32 | rtl8651_flushPolicyBasedQoS | Flush all Policy-Based QoS rules.
@rvalue SUCCESS | Success.
@comm
*/
int32 rtl8651_flushPolicyBasedQoS(void)
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	_rtl8651_tblDrvPolicy_t *qosPolicy_t;
	int32 entry;
	
	rtlglue_drvMutexLock();
	netif_t = DrvTbl.netIface;
	for(entry=0; entry<rtl8651_tblDrvPara.networkIntfTableSize; entry++, netif_t++) {
		if (netif_t->valid == 0)
			continue;
		while ((qosPolicy_t=CTAILQ_FIRST(&netif_t->policyQosHead))) {
			CTAILQ_REMOVE(&netif_t->policyQosHead, qosPolicy_t, nextPolicy_t);
			TAILQ_INSERT_HEAD(&DrvTbl.freeList.freePolicy, qosPolicy_t, nextPolicy_t);
		}
	}
	_rtl8651_arrangeAllACL();
	rtlglue_drvMutexUnlock();
	return SUCCESS;
}









/*=======================================
  * Network interface
  *=======================================*/
#define RTL865X_NETWORK_INTERFACE_APIs


static int32 _rtl8651_specifyNetworkIntfLinkLayerType(rtl8651_tblDrv_networkIntfTable_t *netif_t, uint32 llType, uint16 vid) 
{
	rtl8651_tblDrv_networkIntfTable_t *net_t=DrvTbl.netIface;
	rtl8651_tblDrvSession_t *psession_t;
	rtl8651_tblDrv_pppTable_t *pppEntry_t;
	rtl8651_tblDrv_vlanTable_t *vlan_t = &DrvTbl.vlan[rtl8651_vlanTableIndex(vid)];
	uint32 index;


	assert(netif_t);
	
	/* Check to see if VLAN MAC allocated */
	if (vlan_t->macAddrNumber == 0)
		return TBLDRV_EVLANNOMAC;
	/* check VLAN */
	if (vlan_t->vid != vid || vlan_t->valid == 0) 
		return TBLDRV_EINVALIDVLANID;
	/* Check to see if the link layer type has been specified */
	if (netif_t->linkLayerType != 0)
		return TBLDRV_ENETIFLLTYPEALREADYSET;
	/* Make sure the VLAN isn't referenced */
//	if (llType != RTL8651_LL_VLAN) {
//		for(index=0; index<rtl8651_tblDrvPara.networkIntfTableSize; index++, net_t++) {
//			if (net_t->valid && net_t->linkLayerType==RTL8651_LL_VLAN && net_t->vid == vid)
//				return TBLDRV_ENETIFLLTYPEALREADYSET;
//		}
//	}
	

	switch(llType)	{
	
	case RTL8651_LL_VLAN:
		/* If promiscuous mode is enabled, forbid to bind Network interface to VLAN */
		if (vlan_t->promiscuous == 1) 
			return TBLDRV_EVLANPROMISSET;//promiscuous mode is enabled
		/* Make sure the VLAN isn't referenced */
		for(index=0; index<rtl8651_tblDrvPara.networkIntfTableSize; index++, net_t++) {
			if (net_t->valid && net_t->vid == vid)
				return TBLDRV_ENETIFLLTYPEALREADYSET;
		}
		/* Specified vlan should not be used */
		for(index=0, pppEntry_t=DrvTbl.pppoe; index<RTL8651_PPPOE_NUMBER; index++, pppEntry_t++)
			if(pppEntry_t->valid == 1 && pppEntry_t->vid == vlan_t->vid)
				return TBLDRV_EVLANISREFERENCEDBYPPPOE;
		break;


	case RTL8651_LL_PPTP:
	case RTL8651_LL_L2TP:
	case RTL8651_LL_PPPOE:
		for(index=0; index<rtl8651_tblDrvPara.networkIntfTableSize; index++, net_t++) {
			if (net_t->valid && net_t->linkLayerType!= llType&& net_t->vid == vid)
				return TBLDRV_ENETIFLLTYPEALREADYSET;
		}
		if (llType != RTL8651_LL_PPPOE)
			break;
			
		/*
		  * Note:
		  * Due to the API, rtl8651_addPppoeSession(sessionId, vid), the network interface, session and
		  * vid can not be bound together immediately, we need to finish it here!
		  */
		  
		/* Make sure the VLAN is not referenced by LL_VLAN, LL_PPTP and LL_L2TP */
		pppEntry_t = &DrvTbl.pppoe[RTL8651_PPPOETBL_SIZE];
		for(index=0; index<RTL8651_PPPOETBL_SIZE; index++, pppEntry_t++) {
			if (pppEntry_t->valid && pppEntry_t->vid == vid)
				return TBLDRV_ESESSIONISREFERENCED;
		}
		if (netif_t->flag == 1)
			break;
		/* Bind PPPoE sessions associate with specified VLAN ID to Network Interface */
		for(index=0,pppEntry_t=DrvTbl.pppoe; index<RTL8651_PPPOE_NUMBER; index++,pppEntry_t++) {
			if (pppEntry_t->valid == 1 && pppEntry_t->vid == vlan_t->vid) {
				CTAILQ_FOREACH(psession_t, &netif_t->sessionHead, nextSession)
					if (psession_t->session_t == pppEntry_t)
						break; 
				if (psession_t != NULL)
					continue; /* already bind to network interface */
				/* bind to network interface */
				assert(TAILQ_FIRST(&DrvTbl.freeList.freeSession) != NULL);
				psession_t = TAILQ_FIRST(&DrvTbl.freeList.freeSession);
				TAILQ_REMOVE(&DrvTbl.freeList.freeSession, psession_t, nextSession);
				CTAILQ_INSERT_TAIL(&netif_t->sessionHead, psession_t, nextSession);
				pppEntry_t->netif_t = netif_t;
				psession_t->session_t = pppEntry_t;
				psession_t->ip_t = NULL;
			}
		}
		break;
	
	default: assert(0);
	}
	
	/* If Network Interface is bound to VLAN, set its initial value */
	netif_t->vid = vlan_t->vid;
	netif_t->linkLayerType = llType;
	netif_t->defAclAction = RTL8651_ACL_PERMIT;
	
#ifdef CONFIG_RTL865XB_EXP_INVALID
	SLIST_INSERT_HEAD(&vlan_t->netHead, netif_t, nextNet);
#else
	vlan_t->netif_t = netif_t;
#endif
	return SUCCESS;


}

static int32 _rtl8651_removeNetworkIntfLinkLayerType(rtl8651_tblDrv_networkIntfTable_t *netif_t) 
{
	rtl8651_tblDrv_vlanTable_t *vlan_t;
	rtl8651_tblDrvSession_t *psession_t;


	/*
	  * This function resets the specified network interface link-layer type. Three things to do:
	  * (1) reset the specified network interface link-layer type to zero.
	  * (2) detach PPPoE session which attached to it.
	  * (3) the specified network interface should be detached from the VLAN which it attached to
	  */


	assert(netif_t);
	vlan_t = &DrvTbl.vlan[rtl8651_vlanTableIndex(netif_t->vid)];
	
	/* If the network interface is not VLAN type, remove all sessions whcih attached to it */
	if (netif_t->linkLayerType != RTL8651_LL_VLAN) {
		while((psession_t=CTAILQ_FIRST(&netif_t->sessionHead))) {
			CTAILQ_REMOVE(&netif_t->sessionHead, psession_t, nextSession);
			TAILQ_INSERT_HEAD(&DrvTbl.freeList.freeSession, psession_t, nextSession);
			psession_t->session_t->netif_t = NULL;
		}
	}	
	netif_t->linkLayerType 	= 0;
	netif_t->vid			= 0;
	vlan_t->ipAttached 		= 0;
	vlan_t->netif_t 		= NULL;
	RTL8651_SETASICVLAN(vlan_t);
	//SLIST_REMOVE(&vlan_t->netHead, netif_t, rtl8651_tblDrv_networkIntfTable_s, nextNet);
	return SUCCESS;
}
 


/*
@func int32 | rtl8651_specifyNetworkIntfLinkLayerType | Bind networking interface to a VLAN.
@parm int8 * | ifName | Network interface name.
@parm uint32 | llType | Specify link layer type, which includes RTL8651_LL_VLAN, RTL8651_LL_PPPOE, RTL8651_LL_PPTP, RTL8651_LL_L2TP.
@parm uint16 | vid | VLAN ID.
@rvalue TBLDRV_EUNKNOWLINKLAYERTYPE | Unknow link layer type.
@rvalue TBLDRV_EINVNETIFNAME | Invalid network interface name.
@rvalue TBLDRV_EINVALIDVLANID | Invalid VLAN ID.
@rvalue TBLDRV_EVLANPROMISSET | If VLAN promisuous mode is set, the VLAN cannot attatch any network interface.
@rvalue TBLDRV_EVLANISREFERENCEDBYPPPOE | If VLAN is referenced by PPPoE, the VLAN cannot attach any netwrok interface.
@rvalue TBLDRV_ENETIFLLTYPEALREADYSET | Link layer of the specified network itnerface type already set.
@rvalue TBLDRV_EVLANNOMAC | The specified VLAN has no MAC address allocated.
@comm
For a Layer-3 or Layer-4 switch, the VLAN should have a network interface attached. Hence, if this API is success, the 
specified VLAN will change to Layer-3 or Layer-4.
*/

int32 rtl8651_specifyNetworkIntfLinkLayerType(int8 * ifName, uint32 llType, uint16 vid) 
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	int32 retval;

	if (llType < 0x01 || llType > 0x06) {
		return TBLDRV_EUNKNOWLINKLAYERTYPE;
	}
	if (vid < 1 || vid > 4094) {
		return TBLDRV_EINVALIDVLANID;
	}
	rtlglue_drvMutexLock();
	if ((netif_t = GET_NETWORK_INTERFACE(ifName, NULL)) == NULL) {
		retval = TBLDRV_EINVNETIFNAME;
		goto out;
	}
	retval = _rtl8651_specifyNetworkIntfLinkLayerType(netif_t, llType, vid);

out:
	rtlglue_drvMutexUnlock();
	return retval;
}


/*
@func int32 | rtl8651_removeNetworkIntfLinkLayerType | Rmove link layer type and detach network interface from a VLAN.
@parm int8 * | ifName | Network interface name.
@rvalue TBLDRV_EINVNETIFNAME | Invalid network interface name.
@rvalue TBLDRV_ECANNOTREMOVEIPUNNUMBER | Cannot remove IP Unnumbered network interface. Use <p rtl8651_delIpUnnumbered()> first.
@rvalue TBLDRV_ENETHASNOLLTYPESPECIFY | The Network interface has no link layer type specified.
@rvalue TBLDRV_ENETIFREFERENCEDBYIPIF | The network interface is being referenced by IP interface.
@rvalue TBLDRV_ENETISEXTERNAL | The specified network interface is an external network interface. Hence it cannot be removed.
@comm
The <p rtl8651_removeNetworkIntfLinkLayerType()> removes an existing network interface. The network interface must not be referenced.
*/

int32 rtl8651_removeNetworkIntfLinkLayerType(int8 * ifName) {
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	int32 retval;

	rtlglue_drvMutexLock();//Lock resource
	if ((netif_t = GET_NETWORK_INTERFACE(ifName, NULL)) == NULL) {
		retval = TBLDRV_EINVNETIFNAME;
		goto out;
	}
	/* Not allow non-specified network link-layer type */
	if (netif_t->linkLayerType == 0) {
		retval = TBLDRV_ENETHASNOLLTYPESPECIFY;
		goto out;
	}
	if (netif_t->linkLayerType == RTL8651_LL_LIPUNNUMBERED) {
		retval = TBLDRV_ECANNOTREMOVEIPUNNUMBER;
		goto out;
	}
	/* If the removed vlan is an external interface, forbid the request */
	if (DrvTbl.vlan[rtl8651_vlanTableIndex(netif_t->vid)].internal == 0)  {
		retval = TBLDRV_ENETISEXTERNAL;
		goto out;
	}
	if (SLIST_FIRST(&netif_t->IpHead) || CTAILQ_FIRST(&netif_t->sessionHead) ||SLIST_FIRST(&netif_t->nextNat) ||
	    CTAILQ_TOTAL(&netif_t->inAclHead) > 0 || CTAILQ_TOTAL(&netif_t->outAclHead) > 0) {
		retval = TBLDRV_ENETIFREFERENCEDBYIPIF;
		goto out;
	}
	retval = _rtl8651_removeNetworkIntfLinkLayerType(netif_t);

out:
	rtlglue_drvMutexUnlock();
	return retval;
}


/*=======================================
  * Session-Based 
  *=======================================*/
#define RTL865X_SESSION_BASED_APIs

#if defined(CONFIG_RTL865X_PPTPL2TP)||defined(CONFIG_RTL865XB_PPTPL2TP)
static void _rtl8651_MiiForPptpL2tp(uint16 vid, rtl8651_tblDrv_vlanTable_t *vlan_t)
{
	uint16 lpvid = (vlan_t - DrvTbl.vlan) + (RTL8651_VLAN_NUMBER<<8);
	int32 retval;

	if (tunnel.valid == 1)
		return;
	_rtl8651_addVlan(lpvid);
	vlan_t->internal			= 0;
	vlan_t->ipAttached			= 1;
#ifdef CONFIG_RTL865XB_PPTPL2TP
	vlan_t->memberPortMask		= (1<<CONFIG_RTL865XB_PPTPL2TP_PORT);//0x80;
	vlan_t->untagPortMask		= (1<<CONFIG_RTL865XB_PPTPL2TP_PORT);//0x80;	
#endif
	vlan_t->macAddrNumber		= 1;
	vlan_t->macAddr			= DrvTbl.vlan[rtl8651_vlanTableIndex(vid)].macAddr;
	RTL8651_SETASICVLAN(vlan_t);

	tunnel.valid				= 1;

#ifdef CONFIG_RTL865XB_PPTPL2TP	
	tunnel.loopbackPort			= CONFIG_RTL865XB_PPTPL2TP_PORT;
	rtl8651_setEthernetPortLinkStatus(CONFIG_RTL865XB_PPTPL2TP_PORT, TRUE);
#endif

	tunnel.loopBackVid			= lpvid;
	tunnel.wanPort			= 0xff; /* Invalid port, currently */
	tunnel.wanVid				= vid;

	retval = _rtl8651_setPvid(tunnel.loopbackPort, tunnel.loopBackVid);
	assert(retval == SUCCESS);

	/* rearrange multicast port status */
	_rtl8651_arrangeMulticastPortStatus();

	_rtl8651_arrangeAllACL();
}
#endif




static int32 _rtl8651_freePPPEntry(rtl8651_tblDrv_pppTable_t *pppEntry_t)
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrvSession_t *session_t;

	
	assert(pppEntry_t);
	pppEntry_t->valid = 0;
	/* Unbind PPPoE Session and Network Interface */
	netif_t = pppEntry_t->netif_t;
	session_t = pppEntry_t->session_t;
	CTAILQ_REMOVE(&netif_t->sessionHead, session_t, nextSession);
	TAILQ_INSERT_HEAD(&DrvTbl.freeList.freeSession, session_t, nextSession);
	//SHOULD remove it in the new model, at the mean time, this doing may have bugs occur
	_rtl8651_dsid--;
	if (pppEntry_t->type == RTL8651_PPPOE) {
		if (_rtl8651_pppoe_count>0){
			_rtl8651_pppoe_count--;
			if(_rtl8651_pppoe_count<=1)
				_rtl8651_multiple_dsid=0;
		}
	}
	return SUCCESS;
}


static rtl8651_tblDrv_pppTable_t *
_rtl8651_allocatePPPEntry(uint32 type, uint32 sessionId, rtl8651_tblDrv_networkIntfTable_t *netif_t)
{
	rtl8651_tblDrv_pppTable_t *session_t;
	uint32 totalEntry, entry;

	assert(netif_t && type > 0x0 && type < 0x4 );
	assert(netif_t);

	totalEntry = (type==RTL8651_PPPOE)? RTL8651_PPPOETBL_SIZE: ((type==RTL8651_PPTP)? rtl8651_tblDrvPara.pptpSize: rtl8651_tblDrvPara.l2tpSize);
	session_t = (type==RTL8651_PPPOE)? DrvTbl.pppoe: ((type==RTL8651_PPTP)? DrvTbl.pptp: DrvTbl.l2tp);
	for(entry=0; entry<totalEntry; entry++, session_t++) {
		if (session_t->valid == 0) {
			bzero(session_t, sizeof(rtl8651_tblDrv_pppTable_t));
			session_t->sessionId	= sessionId;
			session_t->netif_t		= netif_t;
			session_t->vid		= netif_t->vid;
			session_t->session_t	= NULL;
			session_t->type		= type;
			session_t->valid		= 1;
			session_t->initiated	= 0;
			session_t->mss		= 1500 - 40 - 8;
			session_t->dsid		= _rtl8651_dsid++; //SHOULD remove it in the new model
			//SHOULD remove it in the new model
			if (type == RTL8651_PPPOE) {
				_rtl8651_pppoe_count++;
				if (_rtl8651_pppoe_count>1)
					_rtl8651_multiple_dsid=1; /* set flag */
			}
			return session_t;
		}
	} 
	return (rtl8651_tblDrv_pppTable_t *)NULL;
}


rtl8651_tblDrv_pppTable_t *_rtl8651_getPPPInfo(uint32 sessionId, ipaddr_t ipaddr)
{
	rtl8651_tblDrv_pppTable_t *session_t=DrvTbl.pppoe;
	int32 entry, totalEntry;

	totalEntry = RTL8651_PPPOETBL_SIZE + rtl8651_tblDrvPara.pptpSize + rtl8651_tblDrvPara.l2tpSize;
	for(entry=0; entry<totalEntry; entry++, session_t++) {
		if (session_t->valid == 0)
			continue;
		if (ipaddr) {
			if (session_t->session_t && session_t->session_t->ip_t && session_t->session_t->ip_t->ipAddr == ipaddr)
				return session_t;
		}
		else if (session_t->sessionId == sessionId) 
			return session_t;

#if 0			
		if (sessionId) {
			if (session_t->sessionId == sessionId) 
				return session_t;
		} else if (session_t->session_t && session_t->session_t->ip_t && session_t->session_t->ip_t->ipAddr == ipaddr)
			return session_t;
#endif
	}
	return (rtl8651_tblDrv_pppTable_t *)NULL;
}



int32 _rtl8651_addSession(uint32 type, uint32 sessionId, int8 *ifName, uint16 vid)
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t=NULL, *net_t = DrvTbl.netIface;
	rtl8651_tblDrvSession_t *ps_t, *session_t=NULL;
	rtl8651_tblDrv_pppTable_t *pppEntry_t;
	rtl8651_tblDrv_vlanTable_t *vlan_t;
	int32 entry, handleCase, retval;
	uint8 llType;

	/*
	  * Assumption:
	  * A vlan is allowed to attach more than one same type network interface(It is disallowed in 8651),
	  * but the network interface here should be PPPoE, PPTP and L2TP type.
	  *
	  * This extended API is used to replace or patch the original rtl8651_addPppoeSession() API and 
	  * has the following abilities:
	  * 1) It allow existing PPPoE session to be moved to another network interface. If a PPPoE
	  *     session is removed from the original network interface, a new network interface will
	  *     be created automatically and the removed PPPoE session will be attached to it.
	  * 2) If no specified PPPoE session exists, this API will create one and attach to the specified
	  *     network interface automatically.
	  * 3) If only one PPPoE session exists in a netwrok interface, this API will forbid the specified PPPoE
	  *     session to be moved to another network interface.
	  * 
	  */


	UNLOCK_ERROR_CHECK((vid < 1 || vid > 4094), TBLDRV_EINVALIDVLANID);
	UNLOCK_ERROR_CHECK((ifName==NULL), TBLDRV_EINVNETIFNAME);

	/* The specified VLAN should exist and have PPPoE network interface attached. */
	vlan_t = &DrvTbl.vlan[rtl8651_vlanTableIndex(vid)];
	if (vlan_t->valid == 0) {
		return TBLDRV_EINVALIDVLANID;
	}
	/* If specified VLAN promiscuous mode is turned on, forbid */
	if (vlan_t->promiscuous == 1) { 
		return TBLDRV_EFORBIDPROMISCINPPPOE;
	}
	/* Make sure the specified Session ID not exists */
	if (_rtl8651_getPPPInfo(sessionId, 0) != NULL) {
		return TBLDRV_EENTRYALREADYEXIST;
	}
	llType = (type==RTL8651_PPPOE)? RTL8651_LL_PPPOE: ((type==RTL8651_PPTP)? RTL8651_LL_PPTP: RTL8651_LL_L2TP);
	for(entry=0; entry<rtl8651_tblDrvPara.networkIntfTableSize; entry++, net_t++) {
		if (!net_t->valid)
			continue;
		if (!_strncmp(ifName, net_t->ifName, RTL8651_NETINTF_NAME_LEN))
			netif_t = net_t;		
		CTAILQ_FOREACH(ps_t, &net_t->sessionHead, nextSession) {
			assert(ps_t->session_t);
			if (ps_t->session_t->sessionId == sessionId)
				session_t = ps_t;
		}
		/* The specified VLAN should be PPPoE, PPTP L2TP type  */
		if (net_t->vid == vid && net_t->linkLayerType != llType) 
			return TBLDRV_EINVALIDVLANID;
	}

	/* Make sure the network interface is PPPoE Type */
	if (netif_t && netif_t->linkLayerType != llType) {
		return TBLDRV_EUNKNOWLINKLAYERTYPE;
	}
	/* Make sure the specified network interface and session should be the same VLAN */
	if (netif_t && session_t && netif_t->vid!=session_t->session_t->netif_t->vid) {
		return TBLDRV_EINVALIDVLANID;
	}
	if (netif_t && netif_t->vid != vid)  {
		return TBLDRV_EINVALIDVLANID;
	}
		
	
	/* 
	  * Find the specified network interface and PPPoE ID. Consider following cases:
	  * (1) netif_t == NULL && session_t == NULL
	  *	   Create a network interface and PPPoE session. Bind them together.
	  * (2) netif_t == NULL && session_t != NULL
	  *  	   The specified session already exists in a network interface, create a new
	  *	   network interface and move the session to it.
	  * (3) netif_t != NULL && session_t == NULL
	  *	   Only occurs when adding session to the original network interface.
	  * (4) netif_t != NULL && session_t != NULL
	  *	   If netif_t already has session, borbid it!
	  *	   If there is only a session_t in a network interface, borbid it!
	 */
	if (!netif_t  && !session_t) 
		handleCase = 1; /* Case 1 */
	else if (!netif_t && session_t) 
		handleCase = 2;  /* Case 2 */
	else if (netif_t && !session_t)
		handleCase = 3; /* Case 3 */
	else  handleCase = 4; /* Case 4 */


	/* Try to allocate session space */
	if (handleCase == 1 || handleCase == 3) {
		if (TAILQ_FIRST(&DrvTbl.freeList.freeSession) == NULL) 
			return TBLDRV_ENOFREEBUFFER;
	}
	/* Try to create a network interface if needed */
	if (!netif_t && (netif_t=_rtl8651_addNetworkInterface(ifName))) { 
		netif_t->flag = 1;
		retval = _rtl8651_specifyNetworkIntfLinkLayerType(netif_t, llType, vid);
		assert(retval == SUCCESS);
	} 
	/* Try to create a new Session if needed */
	if (session_t == NULL) {
		session_t = TAILQ_FIRST(&DrvTbl.freeList.freeSession);
		if ((pppEntry_t=_rtl8651_allocatePPPEntry(type, sessionId, netif_t)) != NULL) {
			TAILQ_REMOVE(&DrvTbl.freeList.freeSession, session_t, nextSession);
			session_t->session_t = pppEntry_t;
			pppEntry_t->session_t = session_t;
			session_t->ip_t = NULL;
		} 
		else { /* rollback: invalidate network interface  */
			if (handleCase == 1 || handleCase == 2) 
				netif_t->valid = 0;
			return TBLDRV_ENOFREEBUFFER;
		}
	}
//	if ((netif_t->linkLayerType!=RTL8651_LL_PPTP&& netif_t->linkLayerType!=RTL8651_LL_L2TP) && 
//	     SLIST_FIRST(&netif_t->IpHead) != NULL) {
//		return TBLDRV_ENETIFREFERENCEDBYIPIF; 
//	}
	assert(session_t && netif_t);

	switch (handleCase)
	{
	case 1:
		CTAILQ_INSERT_TAIL(&netif_t->sessionHead, session_t, nextSession);
		session_t->session_t->netif_t = netif_t;		
		break;
		
	case 2:
		/* Make sure the session's network interface has more than one session */
		net_t = session_t->session_t->netif_t;
		if (CTAILQ_TOTAL(&netif_t->sessionHead) <= 1) { /* Rollback network interface */
			netif_t->valid = 0;
			return TBLDRV_ECANNOTMOVESESSION;
		}
		else {
			CTAILQ_REMOVE(&net_t->sessionHead, session_t, nextSession);
			CTAILQ_INSERT_TAIL(&netif_t->sessionHead, session_t, nextSession);
			session_t->session_t->netif_t = netif_t;
		}
		break;

	case 3:
		if (CTAILQ_FIRST(&netif_t->sessionHead) > 0 && netif_t->flag == 1) { /* Rollback pppoe Session */
			TAILQ_INSERT_HEAD(&DrvTbl.freeList.freeSession, session_t, nextSession);
			session_t->session_t->valid = 0;
			return TBLDRV_EALREADYHAVESESSION;
		}
		else {
			CTAILQ_INSERT_TAIL(&netif_t->sessionHead, session_t, nextSession);
			session_t->session_t->netif_t = netif_t;
		}
		break;

	case 4:
		if ((net_t=session_t->session_t->netif_t) == netif_t)
			break;
		if (CTAILQ_TOTAL(&netif_t->sessionHead) > 0){ 
			return TBLDRV_EALREADYHAVESESSION;
		}
		if (CTAILQ_TOTAL(&net_t->sessionHead)  <= 1) {
			return TBLDRV_ECANNOTMOVESESSION;
		}	
		CTAILQ_INSERT_TAIL(&netif_t->sessionHead, session_t, nextSession);
		session_t->session_t->netif_t = netif_t;
		break;

	default: assert(0);

	}
	_rtl8651_updateDefaultRoute();

#if EXTPORT_PPPoE_UDP_PATCH==2
	/* To patch bug of ASIC, if any one of WAN is PPPoE, enable VLAN tag of extension port. */
	{
		rtl8651_tblDrv_pppTable_t *session_t=DrvTbl.pppoe;
		rtl8651_tblDrv_vlanTable_t *vlanPtr;
		int32 entry;
		int32 bPppoeSessionExist = FALSE;

		/* The first part of DrvTbl.pppoe is PPPoE. The other is PPTP and L2TP. */
		for(entry=0; entry<RTL8651_PPPOETBL_SIZE; entry++, session_t++) 
		{
			if (session_t->valid == 0)
				continue;
			else
				bPppoeSessionExist = TRUE;
		}

		/* If any PPPoE session exists, traverse all VLAN. */
		if ( RtkHomeGatewayChipRevisionID==0x1 && bPppoeSessionExist )
		{
			uint32 i;

			for( i = 0, vlanPtr = DrvTbl.vlan; i < RTL8651_VLAN_NUMBER; i++, vlanPtr++ )
			{
				if (vlanPtr->valid == 0 )
					continue;

				/* We only enable internal VLAN's VLAN tag. */
				if ( vlanPtr->internal )
				{
					if ( vlanPtr->memberPortMask & RTL8651_PORT_EXT1 )
						rtl8651_setVlanPortUntag( vlanPtr->vid, RTL8651_PORT_EXT1, FALSE );
					if ( vlanPtr->memberPortMask & RTL8651_PORT_EXT2 )
						rtl8651_setVlanPortUntag( vlanPtr->vid, RTL8651_PORT_EXT2, FALSE );
					if ( vlanPtr->memberPortMask & RTL8651_PORT_EXT3 )
						rtl8651_setVlanPortUntag( vlanPtr->vid, RTL8651_PORT_EXT3, FALSE );
				}
			}
		}
	}
#endif/*EXTPORT_PPPoE_UDP_PATCH==2*/

	return SUCCESS;
}


static int32 _rtl8651_delSession(uint32 sessionId)
{
	rtl8651_tblDrv_pppTable_t *pppEntry_t;

	rtl8651_tblDrv_routeTable_t *l3_t=DrvTbl.route;
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	int32 entry, retval;
	
	
	if ((pppEntry_t=_rtl8651_getPPPInfo(sessionId, 0)) == NULL) {
		return TBLDRV_EENTRYNOTFOUND;
	}
	/* Forbid in-use session */
	if (pppEntry_t->initiated == 1) {
		return TBLDRV_EPPPOEISINUSE;
	}
	/* If the specified session is referencd by routing entry,  forbid! */
	for(entry=0; entry<rtl8651_tblDrvPara.routingTableSize; entry++, l3_t++) {
		if (l3_t->valid == 0 || l3_t->nextHopType == RTL8651_LL_VLAN)
			continue;
		
		if (l3_t->rt_nhSession_t == pppEntry_t)
			return TBLDRV_EPPPOEISREFERENCEDBYRT;
	}
	_rtl8651_freePPPEntry(pppEntry_t);
	netif_t = pppEntry_t->netif_t;
	/* If the network interface is created due to session being added, remove it! */
	if (netif_t->flag == 1) {
		retval = _rtl8651_removeNetworkIntfLinkLayerType(netif_t);
		assert(retval == SUCCESS);
		netif_t->valid = 0;
	}
	_rtl8651_updateDefaultRoute();

#if EXTPORT_PPPoE_UDP_PATCH==2
	/* To patch bug of ASIC, if no WAN is PPPoE, recover VLAN tag of extension port. */
	{
		rtl8651_tblDrv_pppTable_t *session_t=DrvTbl.pppoe;
		rtl8651_tblDrv_vlanTable_t *vlanPtr;
		int32 entry;
		int32 bPppoeSessionExist = FALSE;

		/* The first part of DrvTbl.pppoe is PPPoE. The other is PPTP and L2TP. */
		for(entry=0; entry<RTL8651_PPPOETBL_SIZE; entry++, session_t++) 
		{
			if (session_t->valid == 0)
				continue;
			else
				bPppoeSessionExist = TRUE;
		}

		/* If no PPPoE session exists, traverse all VLAN. */
		if ( RtkHomeGatewayChipRevisionID== 1 && bPppoeSessionExist == FALSE )
		{
			uint32 i;

			for( i = 0, vlanPtr = DrvTbl.vlan; i < RTL8651_VLAN_NUMBER; i++, vlanPtr++ )
			{
				if (vlanPtr->valid == 0 )
					continue;

				/* We only tag internal VLAN's VLAN tag. */
				if ( vlanPtr->internal )
				{
					if ( vlanPtr->memberPortMask & RTL8651_PORT_EXT1 )
						rtl8651_setVlanPortUntag( vlanPtr->vid, RTL8651_PORT_EXT1, TRUE );
					if ( vlanPtr->memberPortMask & RTL8651_PORT_EXT2 )
						rtl8651_setVlanPortUntag( vlanPtr->vid, RTL8651_PORT_EXT2, TRUE );
					if ( vlanPtr->memberPortMask & RTL8651_PORT_EXT3 )
						rtl8651_setVlanPortUntag( vlanPtr->vid, RTL8651_PORT_EXT3, TRUE );
				}
			}
		}
	}
#endif/*EXTPORT_PPPoE_UDP_PATCH==2*/

	return SUCCESS;
}


static int32 _rtl8651_bindSession(ipaddr_t ipAddr, uint32 sessionId)
{

	rtl8651_tblDrv_pppTable_t *pppEntry_t;
	rtl8651_tblDrv_ipIntfIpAddrEntry_t *ip_t;
	rtl8651_tblDrv_ipIntfEntry_t *ipintf_t;
	int32 entry;

	
	/* Find the specified session */
	if ((pppEntry_t = _rtl8651_getPPPInfo(sessionId, 0)) == NULL)
		return TBLDRV_ESESSIONNOTFOUND;

#if 0		
	/* Check to see if the specified IP address has been added */	
	netif_t = pppEntry_t->netif_t;
	assert(netif_t->linkLayerType != RTL8651_LL_VLAN);
	SLIST_FOREACH(ipintf_t, &netif_t->IpHead, nextIp) {
		SLIST_FOREACH(ip_t, &ipintf_t->ipAddr, nextIpAddr)
			if (ip_t->ipAddr == ipAddr)
				goto found_;
	}
#else

	{
		//for old model 
		rtl8651_tblDrv_networkIntfTable_t *netif_t = DrvTbl.netIface;
		for(entry=0; entry<rtl8651_tblDrvPara.networkIntfTableSize; entry++, netif_t++) {
			if (netif_t->valid == 0)
				continue;
			SLIST_FOREACH(ipintf_t, &netif_t->IpHead, nextIp) {
				SLIST_FOREACH(ip_t, &ipintf_t->ipAddr, nextIpAddr)
					if (ip_t->ipAddr == ipAddr)
						goto found_;
			}
		}
	}
#endif
	return TBLDRV_EINVALIDINPUT;

found_:

//	if (netIfPtr1->vid != netIfPtr2->vid) {
//		LOCK_ERROR_CHECK(
//			( (netIfPtr1->linkLayerType != RTL8651_LL_LIPUNNUMBERED) ||
//			  (netIfPtr2->linkLayerType != RTL8651_LL_LIPUNNUMBERED)
//			), TBLDRV_EINVALIDINPUT
//		);
//	} else assert(netIfPtr1->linkLayerType != RTL8651_LL_LIPUNNUMBERED);	
	pppEntry_t->session_t->ip_t = ip_t;
	return SUCCESS;
}


static void _rtl8651_unbindSession(rtl8651_tblDrv_ipIntfIpAddrEntry_t *ipAddr, rtl8651_tblDrv_pppTable_t *pppEntry_t) 
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t = DrvTbl.netIface;
	rtl8651_tblDrvSession_t *session_t;
	uint32 entry;

	
	assert( ipAddr || pppEntry_t);	
	for(entry=0; entry<rtl8651_tblDrvPara.networkIntfTableSize; entry++, netif_t++) {
		if (netif_t->valid == 1) { 
			CTAILQ_FOREACH(session_t, &netif_t->sessionHead, nextSession) {
				if (ipAddr && session_t->ip_t == ipAddr) {
					//session_t->session_t->session_t = NULL;
					session_t->ip_t = NULL;
					break;
				}
				if (pppEntry_t && pppEntry_t->session_t == session_t) {
					//pppEntry_t->session_t = NULL;
					session_t->ip_t = NULL;
					break;
				}
			}
	        }
       } 

}




static int32 _rtl8651_resetSessionProperty(rtl8651_tblDrv_pppTable_t *pppEntry_t)
{
	rtl8651_tblDrv_routeTable_t *l3Entry_t;
	uint32 entry;
	uint8 llType=0;


	assert(pppEntry_t);
	switch (pppEntry_t->type) 
	{
	case RTL8651_PPPOE:
		break;
	case RTL8651_PPTP:	llType = RTL8651_LL_PPTP; break;
	case RTL8651_L2TP:	llType = RTL8651_LL_L2TP; break;
	default: assert(0);
	}
	pppEntry_t->initiated	= 0;
	
	/* Make sure the session is not referenced by routing entry */
	for(entry=0, l3Entry_t=DrvTbl.route; entry<rtl8651_tblDrvPara.routingTableSize; entry++, l3Entry_t++) {
		if (l3Entry_t->valid == 0)
			continue;
		if (l3Entry_t->netif_t->linkLayerType == llType && l3Entry_t->rt_nhSession_t == pppEntry_t) {
//			mbrMask = (1 << tunnel.wanPort);
//			_rtl8651_modifyVlanRefFilterDatabaseEntry(0, &l3Entry_t->rt_nhSession_t->macInfo->macAddr, &mbrMask, RTL8651_FORWARD_MAC, NULL, L2_FORCE_STATIC);
			_rtl8651_updateL3Entry(l3Entry_t);				
		}
	}
	return SUCCESS;
}


int32 _rtl8651_getSessionMac(uint32 sessionId, ether_addr_t *macAddr)
{
	rtl8651_tblDrv_pppTable_t *pppEntry_t;
	rtl8651_tblDrv_vlanTable_t *vlan_t;

	if ((pppEntry_t=_rtl8651_getPPPInfo(sessionId, 0)) == NULL)
		return TBLDRV_EINVALIDPPPOEID;
	vlan_t = &DrvTbl.vlan[rtl8651_vlanTableIndex(pppEntry_t->vid)];
	if (vlan_t->macAddrNumber > 0) {
		*macAddr = vlan_t->macAddr;
		return SUCCESS;
	}
	else return TBLDRV_EVLANNOMAC;	
}



/*
@func int32 | rtl8651_addSession | Add a session-based connection.
@parm uint32 | type | Only following types are allowed: RTL8651_PPPOE, RTL8651_PPTP, RTL8651_L2TP.
@parm uint32 | sessionId | Session ID.
@parm int8 * | ifName | Network interface name.
@parm uint16 | vid | Vlan ID.
@rvalue TBLDRV_ECANNOTUSETHISVID | The specified VLAN ID is already in use.
@rvalue TBLDRV_ENEEDLOOPBACKSET | Need Loop Back port set for PPTP/L2TP.
@rvalue TBLDRV_EINVALIDVLANID | Invalid VLAN ID.
@rvalue TBLDRV_EINVNETIFNAME | Invalid Netwrok Interface Name.
@rvalue TBLDRV_EFORBIDPROMISCINPPPOE | If promiscuous mode is enabled, Session cannot be added.
@rvalue TBLDRV_EENTRYALREADYEXIST | Duplicate Session ID assignment.
@rvalue TBLDRV_EUNKNOWLINKLAYERTYPE | Unknown network interface's link-layer type.
@rvalue TBLDRV_ENOFREEBUFFER | Out of memory.
@rvalue TBLDRV_ECANNOTMOVESESSION | Cannot move the specified session to another network interface.
@rvalue TBLDRV_EALREADYHAVESESSION | The specified netwrok interface already has session attached.
@rvalue SUCCESS | Success.
@comm
This API is used to add a session-based connection such as PPPoE, PPTP, L2TP connection. Before dial-up a session-based connection,
this API should be called to add a session to user-specified netwrok interface and VLAN.
*/
int32 rtl8651_addSession(uint32 type, uint32 sessionId, int8 *ifName, uint16 vid)
{
	rtl8651_tblDrv_vlanTable_t *vlan_t=DrvTbl.vlan;
	int32 retval, entry;
	
	rtlglue_drvMutexLock();
	if (tunnel.valid == 1 && tunnel.wanVid != vid) {
		retval = TBLDRV_ECANNOTUSETHISVID;
		goto out;
	}

	for(entry=0; entry<RTL8651_VLAN_NUMBER; entry++, vlan_t++) 
		if (vlan_t->valid == 0)
			break;
	if (entry == RTL8651_VLAN_NUMBER)
		vlan_t = NULL;
	if ((retval = _rtl8651_addSession(type, sessionId, ifName, vid)) == SUCCESS) {
#if defined(CONFIG_RTL865X_PPTPL2TP)||defined(CONFIG_RTL865XB_PPTPL2TP)		
		if (type == RTL8651_PPTP || type == RTL8651_L2TP)
			_rtl8651_MiiForPptpL2tp(vid, vlan_t);
#endif		
	}

out:
	rtlglue_drvMutexUnlock();
	return retval;
}


/*
@func int32 | rtl8651_delSession | Remove a session-based connection.
@parm uint32 | sessionId | Session ID.
@rvalue SUCCESS | Success.
@rvalue TBLDRV_EENTRYNOTFOUND | The specified session ID was not found.
@rvalue TBLDRV_EPPPOEISINUSE | The specified session is still in use. Hand up it and reset its property first.
@rvalue TBLDRV_EPPPOEISREFERENCEDBYRT | The session is referenced by routing entry. Remove the route first.
@comm
Only a session is hung up and property reset, the session can be removed.
*/
int32 rtl8651_delSession(uint32 sessionId)
{
	rtl8651_tblDrv_pppTable_t*pppEntry_t;
	int32 retval, entry;
	rtlglue_drvMutexLock();
	if ((retval = _rtl8651_delSession(sessionId)) == SUCCESS) {
		pppEntry_t = DrvTbl.pptp;
		for(entry=0; entry<rtl8651_tblDrvPara.pptpSize+rtl8651_tblDrvPara.l2tpSize; entry++, pppEntry_t++)
			if (pppEntry_t->valid == 1)
				break;
		if (entry == rtl8651_tblDrvPara.pptpSize+rtl8651_tblDrvPara.l2tpSize) {
			tunnel.valid = 0;
			DrvTbl.vlan[rtl8651_vlanTableIndex(tunnel.loopBackVid)].valid = 0;
			rtl8651_delAsicVlan(tunnel.loopBackVid);
		}
	}
	rtlglue_drvMutexUnlock();
	return retval;
}


/*
@func int32 | rtl8651_bindSession | Bind a IP address with a session.
@parm ipaddr_t | ipAddr | IP address given by session dial-up.
@parm uint32 | sessionId | Session ID.
@rvalue TBLDRV_ESESSIONNOTFOUND | Session ID was not found.
@rvalue TBLDRV_EINVALIDINPUT | Invalid Input parameter.
@rvaue SUCCESS | Success.
@comm
When a session dial up, the session will be given a IP address by ISP, the IP address should be bound with the session through this API.
*/
int32 rtl8651_bindSession(ipaddr_t ipAddr, uint32 sessionId)
{
	int32 retval;
	rtlglue_drvMutexLock();
	retval = _rtl8651_bindSession(ipAddr, sessionId);	
	rtlglue_drvMutexUnlock();
	return retval;
}

/*
@func int32 | rtl8651_getSessionMac | Get a Session's MAC address.
@parm uint32 | sessionId | Session ID.
@parm ether_addr_t * | macAddr |Pointer to MAC address.
@rvalue TBLDRV_EINVALIDPPPOEID | The specified session ID was not found.
@rvalue TBLDRV_EVLANNOMAC | The session has no MAC Address found.
@rvalue SUCCESS | Success.
@comm
Given an existed session-based connection ID, this API will return the session's MAC Address.
*/
int32 rtl8651_getSessionMac(uint32 sessionId, ether_addr_t *macAddr)
{
	int32 retval;
	rtlglue_drvMutexLock();
	retval = _rtl8651_getSessionMac(sessionId, macAddr);
	rtlglue_drvMutexUnlock();
	return retval;
}


/*
int32 rtl8651_setSessionMtu(uint32 sessionId, uint32 mtu)
{
	rtl8651_tblDrv_pppTable_t *pppEntry_t;
	int32 retval = SUCCESS;
	
	rtlglue_drvMutexLock();
	if ((pppEntry_t = _rtl8651_getPPPInfo(sessionId)) == NULL) {
		retval TBLDRV_ESESSIONNOTFOUND;
		goto out;
	}
	pppEntry_t->mss = mtu - 40;
out:
	rtlglue_drvMutexUnlock();
	return retval;
}
*/


/*=======================================
  * PPPOE APIs 
  *=======================================*/
#define RTL865X_PPPOE_APIs


/*
@func int32 | rtl8651_addPppoeSession | Create a PPPoE.
@parm uint32 | pppoeId | PPPoE ID.
@parm uint16 | vid | VLAN ID.
@rvalue SUCCESS | A PPPoE was added successfully.
@rvalue TBLDRV_EINVALIDVLANID | Invalid VLAN ID.
@rvalue TBLDRV_EFORBIDPROMISCINPPPOE | If promiscuous mode was set, PPPoE cannot be created.
@rvalue TBLDRV_EVLANSHOULDBEPPPOE | The specified VLAN should be PPPoE Type.
@rvalue TBLDRV_EPPPOEALREADYEXISTS | The specified PPPoE already exists.
@rvalue TBLDRV_EOUTOFVLANMACADDR | Out of VLAN MAC address.
*/
int32 rtl8651_addPppoeSession(uint32 sessionId, uint16 vid) 
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	int32 retval, entry;
	int8 *ifName=NULL;
	rtlglue_drvMutexLock();
	netif_t = DrvTbl.netIface;
	for(entry=0; entry<rtl8651_tblDrvPara.networkIntfTableSize; entry++, netif_t++) {
		if (netif_t->valid && netif_t->vid == vid && netif_t->flag == 0) {
			ifName = netif_t->ifName;
			break;
		}
	}
	retval = _rtl8651_addSession(RTL8651_PPPOE, sessionId, ifName, vid);
	rtlglue_drvMutexUnlock();
	return retval;
}


/*
@func int32 | rtl8651_delPppoeSession | Remove a PPPoE.
@parm uint32 | pppoeId | PPPoE ID.
@rvalue SUCCESS | The specified PPPoE was removed successfully.
@rvalue TBLDRV_EINVALIDPPPOEID | Invalid PPPoE ID.
@rvalue TBLDRV_EPPPOEISINUSE | The specified PPPoE is in use.
@rvalue TBLDRV_EPPPOEISREFERENCEDBYRT | PPPoE is referenced by routing.
*/
int32 rtl8651_delPppoeSession(uint32 sessionId)
{
	int32 retval;
	rtlglue_drvMutexLock();
	retval = _rtl8651_delSession(sessionId);
	rtlglue_drvMutexUnlock();
	return retval;	
}


/*
@func int32 | rtl8651_setPppoeSessionProperty | Set a PPPoE Session's property.
@parm uint32 | pppoeId | Specify PPPoE ID.
@parm uint16 | sid | PPPoE Session ID which is given by ISP.
@parm ether_addr_t * | macAddr | ISP MAC address.
@parm uint32 | port | Logical port number.
@rvalue SUCCESS | The specified PPPoE session was set successfully.
@rvalue TBLDRV_EINVALIDSESSIONID | Invalid PPPoE Session ID.
@rvalue TBLDRV_EINVALIDPPPOEID | Invalid PPPoE ID.
@rvalue TBLDRV_EPROPERTUALREADYSET | The specified PPPoE property was already set to another PPPoE ID.
@rvalue TBLDRV_EAGGREGATORHASNOPORT | Not allow aggregator without port member.
@rvalue TBLDRV_EPPPOEHASPROPERTYSET | The specified PPPoE ID already has property set.
@rvalue TBLDRV_EPPPOECANNOTADDL2ENTRY | Cannot add L2 entry for the specified PPPoE.
*/
int32 rtl8651_setPppoeSessionProperty(uint32 sessionId, uint16 sid, ether_addr_t * macAddr, uint32 port)
{
	rtl8651_tblDrv_pppTable_t *pppEntry_t, *tracker_t;
	rtl8651_tblDrv_routeTable_t *rt_t = DrvTbl.route;
	rtl865x_tblAsicDrv_pppoeParam_t asic_pppoe;
	int32 entry, retval=SUCCESS;
	uint32 vidx;
	uint8 buf[24];
	
	/* RFC2516 p.4 define 0xffff MUST NOT be used */
	if (sid == 0xffff) {
		return TBLDRV_EINVALIDSESSIONID;
	}
	rtlglue_drvMutexLock();
	/* Find the specified Session ID */
	if ((pppEntry_t=_rtl8651_getPPPInfo(sessionId, 0)) == NULL) {
		retval = TBLDRV_EINVALIDPPPOEID;
		goto out;
	}
	if (pppEntry_t->type != RTL8651_PPPOE) {
		retval = TBLDRV_EINVALIDINPUT;
		goto out;
	}
	/* Check whether the property already configured for other pppoe object */
	tracker_t = DrvTbl.pppoe;
	for(entry=0; entry<RTL8651_PPPOE_NUMBER; entry++, tracker_t++) {
		if (tracker_t->valid && tracker_t->sessionId == sessionId)
			continue;
		if (tracker_t->initiated && tracker_t->pppoeSessionId_ == sid && 
		    !memcmp(&tracker_t->macInfo->macAddr, macAddr, sizeof(ether_addr_t))) 	{
			retval = TBLDRV_EPROPERTUALREADYSET;
			goto out;
		}
	}
#if 0	
	//ignore the port parameter.
	if (DrvTbl.aggregator[port].ethernetPortMask == 0) {
		retval = TBLDRV_EAGGREGATORHASNOPORT;
		goto out;
	}	
#endif	
	/* Forbid initiated session overwritten */
	if (pppEntry_t->initiated == 1) {
		retval = TBLDRV_EPPPOEHASPROPERTYSET;
		goto out;
	}
	vidx = rtl8651_vlanTableIndex(pppEntry_t->vid);
	if (_rtl8651_addVlanRefFilterDatabaseEntry(RTL8651_L2_TYPEI, DrvTbl.vlan[vidx].fid, pppEntry_t->vid, macAddr, RTL8651_FORWARD_MAC, 1<<port)) {
		retval = TBLDRV_EPPPOECANNOTADDL2ENTRY;
		goto out;
	}
	/*uint8 buf[24];*/
	 ether_ntoa_r(macAddr, buf);
	/* Set PPPoE Property */
	pppEntry_t->initiated 			= 1;
	pppEntry_t->pppoeSessionId_ 	= sid;
	pppEntry_t->macInfo			= _rtl8651_getVlanFilterDatabaseEntry(pppEntry_t->vid, macAddr);
	asic_pppoe.sessionId			= sid;
#ifdef CONFIG_RTL865XB_EXP_INVALID
	asic_pppoe.age				= 7;
#endif /* CONFIG_RTL865XB_EXP_INVALID */
	rtl8651_setAsicPppoe(pppEntry_t-DrvTbl.pppoe,  &asic_pppoe);
	/* Update routing entry */
	for(entry=0; entry<rtl8651_tblDrvPara.routingTableSize; entry++, rt_t++)
		if (rt_t->valid && rt_t->rt_nhSession_t == pppEntry_t)
			_rtl8651_updateL3Entry(rt_t);
 	_rtl8651_updateDefaultRoute();
	_rtl8651_updateNextHop_NHIDX(NULL, pppEntry_t);

 out:
	rtlglue_drvMutexUnlock();
	return retval;


}


/*
@func int32 | rtl8651_resetPppoeSessionProperty | Rest the PPPoE session property.
@parm uint32 | pppoeId | PPPoE ID.
@rvalue SUCCESS | The specified PPPoE was reset successfully.
@rvalue TBLDRV_EINVALIDPPPOEID | Invalid PPPoE ID.
*/
int32 rtl8651_resetPppoeSessionProperty(uint32 sessionId)
{
	rtl8651_tblDrv_routeTable_t *rt_t = DrvTbl.route;
	rtl8651_tblDrv_pppTable_t *pppEntry_t;
	int32 retval = SUCCESS;
	uint32 vidx, entry;


	rtlglue_drvMutexLock();
	if ((pppEntry_t=_rtl8651_getPPPInfo(sessionId, 0)) == NULL) {
		retval = TBLDRV_EINVALIDPPPOEID;
		goto out;
	}
	if (pppEntry_t->initiated== 0) 
		goto out;
	pppEntry_t->initiated= 0;

	vidx = rtl8651_vlanTableIndex(pppEntry_t->vid);
	retval = _rtl8651_delVlanRefFilterDatabaseEntry(RTL8651_L2_TYPEI, DrvTbl.vlan[vidx].fid, &(pppEntry_t->macInfo->macAddr));
	assert(retval == SUCCESS);
	pppEntry_t->macInfo = NULL;
	/* Try to unbind IP address and PPPoE Session */
	_rtl8651_unbindSession(NULL, pppEntry_t);
	for(entry=0; entry<rtl8651_tblDrvPara.routingTableSize; entry++, rt_t++)
		if (rt_t->valid && rt_t->rt_nhSession_t == pppEntry_t)
			_rtl8651_updateL3Entry(rt_t);
	_rtl8651_updateDefaultRoute();
	_rtl8651_updateNextHop_NHIDX(NULL, pppEntry_t);

out:
	rtlglue_drvMutexUnlock();
	return retval;
}


/*
@func int32 | rtl8651_bindPppoeSession | Bind PPPoE Session and IP address.
@parm ipaddr_t | ipaddr | PPPoE IP address.
@parm uint32 | pppoeId | PPPoE ID.
@rvalue SUCCESS | The PPPoE IP and Session ID were bound successfully.
@rvalue TBLDRV_EINVALIDINPUT | Invalid input parameter.
@comm
When a PPPoE dials up, a PPPoE IP address and PPPoE Session ID will be assigned by ISP. They should be bound together to Table Driver.
*/
int32 rtl8651_bindPppoeSession(ipaddr_t ipAddr, uint32 sessionId)
{
	int32 retval;
	rtlglue_drvMutexLock();
	retval = _rtl8651_bindSession(ipAddr, sessionId);
	rtlglue_drvMutexUnlock();
	return retval;
}


/*
@func int32 | rtl8651_getPppoeSessionMac | Get the source MAC address which the specified PPPoE uses.
@parm uint32 | pppoeId | Specify PPPoE ID.
@parm ether_addr_t * | macAddr | Source MAC address.
@rvalue SUCCESS | Success.
@rvalue TBLDRV_EINVALIDPPPOEID | Invalid PPPoE ID was specified.
*/
int32 rtl8651_getPppoeSessionMac(uint32 sessionId, ether_addr_t * macAddr) 
{
	int32 retval;
	rtlglue_drvMutexLock();
	retval = _rtl8651_getSessionMac(sessionId, macAddr);
	rtlglue_drvMutexUnlock();
	return retval;
}




/*=======================================
  * PPTP APIs
  *=======================================*/
#define RTL865X_PPTP_APIs

/*
@func int32 | rtl8651_setPptpProperty | Set PPTP's Property.
@parm uint32 | sessionId | Session ID. Zero is not allowed.
@parm uint16 | callId | PPTP call ID.
@parm uint16 | peerCallId | Peer PPTP call ID.
@parm ipaddr_t | s_ipaddr | PPTP Server IP address.
@Parm ipaddr_t | gw_ipaddr | Default gateway IP address.
@rvalue TBLDRV_ESESSIONNOTFOUND | The specified Session ID is not found.
@rvalue TBLDRV_ENOARPFOUND | We have no  PPTP Server's ARP.
@rvalue SUCCESS | Success API call.
@comm
Set PPTP's property. If a PPTP session is dialed up, this API should be called to add the PPTP session's property.
Note that the parameter <p gw_ipaddr> is set only when the PPTP server isn't in the same subnet with our DUT.
If PPTP server is in the same subnet with us, this parameter MUST be set to 0.
*/
int32 rtl8651_setPptpProperty(uint32 sessionId, uint16 callId, uint16 peerCallId, ipaddr_t s_ipaddr, ipaddr_t gw_ipaddr)
{
	rtl8651_tblDrv_pppTable_t *pppEntry_t;
	rtl8651_tblDrv_arpEntry_t *arp_t;
	rtl8651_tblDrv_routeTable_t *rt_t;
	int32 retval = SUCCESS;
	uint32 portMask, entry;
	
	rtlglue_drvMutexLock();

	if ((pppEntry_t = _rtl8651_getPPPInfo(sessionId, 0)) == NULL) {
		retval = TBLDRV_ESESSIONNOTFOUND;
		goto out;
	}
	if ((arp_t=_rtl8651_getArpEntry(pppEntry_t->netif_t, s_ipaddr, FALSE)) == NULL) {
		if(gw_ipaddr && (arp_t = _rtl8651_getArpEntry(pppEntry_t->netif_t, gw_ipaddr, FALSE)) != NULL);
		else {
			retval = TBLDRV_ENOARPFOUND;
			goto out;
		}
	}
	if ((pppEntry_t->macInfo=arp_t->macInfo) == NULL) {		
		retval = TBLDRV_ENOARPFOUND;
		goto out;
	}
	retval = _rtl8651_addVlanRefFilterDatabaseEntry(RTL8651_L2_TYPEI, DrvTbl.vlan[rtl8651_vlanTableIndex(arp_t->vid)].fid, 
		arp_t->vid, &arp_t->macInfo->macAddr, RTL8651_FORWARD_MAC, arp_t->macInfo->memberPortMask);
	if (retval != SUCCESS) {
		retval = TBLDRV_ENOARPFOUND;
		goto out;
	}
	pppEntry_t->initiated		= 1;
	pppEntry_t->callId_			= callId;
	pppEntry_t->peerCallId_		= peerCallId;
	pppEntry_t->pptpServerIp_	= s_ipaddr;
	pppEntry_t->pptpDefaultGwIp_ = gw_ipaddr;
	portMask					= arp_t->macInfo->memberPortMask;
	assert(tunnel.valid == 1);
	tunnel.wanPort			= 0;
	while (portMask > 1) {
		portMask = portMask >> 1;
		tunnel.wanPort ++;
	}
	_rtl8651_updateNextHop_NHIDX(NULL, pppEntry_t);
	
	/* If routing entry references, update route */
	rt_t = DrvTbl.route;
	for(entry=0; entry<rtl8651_tblDrvPara.routingTableSize; entry++, rt_t++) {
		if (rt_t->valid == 0)
			continue;
		if (rt_t->netif_t->linkLayerType == RTL8651_LL_PPTP && rt_t->rt_nhSession_t == pppEntry_t) 
			_rtl8651_updateL3Entry(rt_t);
	}
	_rtl8651_updateDefaultRoute();

	/* When a PPTP/L2TP session is dialed up, we remove the interface route of "outer subnet". 
	     Therefore, packets destined to "outer subnet" will be treated as packets destined to "inner subnet". */
	if (gw_ipaddr)
		_rtl8651_rmPPTPL2TPInterfaceRoute(gw_ipaddr);
	else
		_rtl8651_rmPPTPL2TPInterfaceRoute(s_ipaddr);
	//_rtl8651_updateNextHop_NHIDX(NULL, pppEntry_t);
	
out:
	rtlglue_drvMutexUnlock();
	return retval;
}


/*
@func int32 | rtl8651_resetPptpProperty | Reset PPTP Property.
@parm uint32 | sessionId | Session ID. Zero is not allowed.
@rvalue TBLDRV_ESESSIONNOTFOUND | The specified Session ID is not found.
@rvalue SUCCESS | Success API call.
@comm
After a PPTP session is hung up, this API should be called to reset the PPTP's property.
*/
int32 rtl8651_resetPptpProperty(uint32 sessionId)
{
	//rtl8651_tblDrv_filterDbTableEntry_t *macInfo;
	rtl8651_tblDrv_pppTable_t *pppEntry_t;
	ipaddr_t pptpServerIp_tmp, pptpDefaultGwIp_tmp;
	int32 retval;
	rtlglue_drvMutexLock();
	if ((pppEntry_t = _rtl8651_getPPPInfo(sessionId, 0)) == NULL) 
	{
		rtlglue_drvMutexUnlock();
		return TBLDRV_ESESSIONNOTFOUND;
	}

	//printk("pptp server ip: %u.%u.%u.%u\n", NIPQUAD(pppEntry_t->pptpServerIp_));
	pptpServerIp_tmp = pppEntry_t->pptpServerIp_;
	pptpDefaultGwIp_tmp = pppEntry_t->pptpDefaultGwIp_;
	
	//for auto-reconnect or dial-on-demand
	pppEntry_t->initiated		= 0;
	pppEntry_t->callId_		= 0;
	pppEntry_t->peerCallId_	= 0;
	pppEntry_t->pptpServerIp_	= 0;
	pppEntry_t->pptpDefaultGwIp_ = 0;
	pppEntry_t->delta_			= 0;
	pppEntry_t->lastReceiveSeq_	= 0;	
	if (pppEntry_t->type == RTL8651_PPTP)
		retval = _rtl8651_resetSessionProperty(pppEntry_t);
	else retval = TBLDRV_ESESSIONNOTFOUND;
	if (tunnel.wanPort != 0xff) {
		uint32 mbrMask = (1<<tunnel.wanPort);
		pppEntry_t->macInfo->memberPortMask = mbrMask;
		_rtl8651_modifyVlanRefFilterDatabaseEntry(0, &pppEntry_t->macInfo->macAddr, &mbrMask, RTL8651_FORWARD_MAC, NULL, L2_FORCE_STATIC);
	}
	_rtl8651_delVlanRefFilterDatabaseEntry(
			RTL8651_L2_TYPEI, 
			DrvTbl.vlan[rtl8651_vlanTableIndex(pppEntry_t->vid)].fid, 
			&pppEntry_t->macInfo->macAddr
	);	
	_rtl8651_updateDefaultRoute();

	/* When a PPTP/L2TP session is hung up, we add an interface route for "outer subnet". */
	if (pptpDefaultGwIp_tmp)
		_rtl8651_addPPTPL2TPInterfaceRoute(pppEntry_t, pptpDefaultGwIp_tmp);
	else
		_rtl8651_addPPTPL2TPInterfaceRoute(pppEntry_t, pptpServerIp_tmp);
	_rtl8651_updateNextHop_NHIDX(NULL, pppEntry_t);
	tunnel.wanPort			= 0xff;
	rtlglue_drvMutexUnlock();
	return retval;
}


/*=======================================
  * L2TP APIs 
  *=======================================*/
#define RTL865X_L2TP_APIs


/*
@func int32 | rtl8651_setL2tpProperty | Set L2TP's Property.
@parm uint32 | sessionId | Session ID. Zero is not allowed.
@parm uint16 | tunnelId | Tunnel ID got from L2TP handshake.
@parm ipaddr_t | s_ipaddr | L2TP Server IP address.
@Parm ipaddr_t | gw_ipaddr | Default gateway IP address.
@rvalue TBLDRV_ESESSIONNOTFOUND | The specified Session ID is not found.
@rvalue TBLDRV_ENOARPFOUND | We have no  L2TP Server's ARP.
@rvalue SUCCESS | Success API call.
@comm
Set L2TP's property. If a L2TP session is dialed up, this API should be called to add the L2TP session's property.
Note that the parameter <p gw_ipaddr> is set only when the L2TP server isn't in the same subnet with our DUT.
If L2TP server is in the same subnet with us, this parameter MUST be set to 0.
*/
int32 rtl8651_setL2tpProperty(uint32 sessionId, uint16 tunnelId, uint16 sid, ipaddr_t s_ipaddr, ipaddr_t gw_ipaddr)
{
	rtl8651_tblDrv_pppTable_t *pppEntry_t;
	rtl8651_tblDrv_arpEntry_t *arp_t;
	rtl8651_tblDrv_routeTable_t *rt_t;
	int32 retval = SUCCESS;
	uint32 portMask, entry;
	
	rtlglue_drvMutexLock();
	if ((pppEntry_t = _rtl8651_getPPPInfo(sessionId, 0)) == NULL) {
		retval = TBLDRV_ESESSIONNOTFOUND;
		goto out;
	}
	if ((arp_t=_rtl8651_getArpEntry(pppEntry_t->netif_t, s_ipaddr, FALSE)) == NULL) {
		if(gw_ipaddr && (arp_t = _rtl8651_getArpEntry(pppEntry_t->netif_t, gw_ipaddr, FALSE)) != NULL);
		else {
			retval = TBLDRV_ENOARPFOUND;
			goto out;
		}
	}
	if ((pppEntry_t->macInfo=arp_t->macInfo) == NULL) {
		retval = TBLDRV_ENOARPFOUND;
		goto out;
	}
	retval = _rtl8651_addVlanRefFilterDatabaseEntry(RTL8651_L2_TYPEI, DrvTbl.vlan[rtl8651_vlanTableIndex(arp_t->vid)].fid, 
		arp_t->vid, &arp_t->macInfo->macAddr, RTL8651_FORWARD_MAC, arp_t->macInfo->memberPortMask);
	if (retval != SUCCESS) {
		retval = TBLDRV_ENOARPFOUND;
		goto out;
	}
	pppEntry_t->initiated		= 1;
	pppEntry_t->tunnelId_		= tunnelId;
	pppEntry_t->l2tpSessionId_	= sid;
	pppEntry_t->l2tpServerIp_	= s_ipaddr;
	pppEntry_t->l2tpDefaultGwIp_ = gw_ipaddr;
	pppEntry_t->l2tpOffset2_	= 0;
	portMask					= arp_t->macInfo->memberPortMask;
	
	assert(tunnel.valid == 1);
	tunnel.wanPort			= 0;
	
	while (portMask > 1) {
		portMask = portMask >> 1;
		tunnel.wanPort ++;
	}
	_rtl8651_updateNextHop_NHIDX(NULL, pppEntry_t);

	/* If routing entry references, update route */
	rt_t = DrvTbl.route;
	for(entry=0; entry<rtl8651_tblDrvPara.routingTableSize; entry++, rt_t++) {
		if (rt_t->valid == 0)
			continue;
		if (rt_t->netif_t->linkLayerType == RTL8651_LL_L2TP && rt_t->rt_nhSession_t == pppEntry_t) 
			_rtl8651_updateL3Entry(rt_t);
	}
	_rtl8651_updateDefaultRoute();

	/* 	When a PPTP/L2TP session is dialed up, we remove the interface route of "outer subnet". 
     		Therefore, packets destined to "outer subnet" will be treated as packets destined to "inner subnet". */
     if (gw_ipaddr)
	 	_rtl8651_rmPPTPL2TPInterfaceRoute(gw_ipaddr);
     else
		_rtl8651_rmPPTPL2TPInterfaceRoute(s_ipaddr);
	 //	_rtl8651_updateNextHop_NHIDX(NULL, pppEntry_t);
	
out:
	rtlglue_drvMutexUnlock();
	return retval;
}


/*
@func int32 | rtl8651_resetL2tpProperty | Reset L2TP Property.
@parm uint32 | sessionId | Session ID. Zero is not allowed.
@rvalue TBLDRV_ESESSIONNOTFOUND | The specified Session ID is not found.
@rvalue SUCCESS | Success API call.
@comm
After a L2TP session is hung up, this API should be called to reset the L2TP's property.
*/
int32 rtl8651_resetL2tpProperty(uint32 sessionId)
{
	rtl8651_tblDrv_pppTable_t *pppEntry_t;
	ipaddr_t l2tpServerIp_tmp, l2tpDefaultGwIp_tmp;
	int32 retval;
	
	rtlglue_drvMutexLock();
	if ((pppEntry_t = _rtl8651_getPPPInfo(sessionId, 0)) == NULL) 
	{
		rtlglue_drvMutexUnlock();
		return TBLDRV_ESESSIONNOTFOUND;
	}

	l2tpServerIp_tmp = pppEntry_t->l2tpServerIp_;
	l2tpDefaultGwIp_tmp = pppEntry_t->l2tpDefaultGwIp_;

	if (pppEntry_t->type == RTL8651_L2TP)
		retval = _rtl8651_resetSessionProperty(pppEntry_t);
	else retval = TBLDRV_ESESSIONNOTFOUND;
	if (tunnel.wanPort != 0xff) {
		uint32 mbrMask = (1<<tunnel.wanPort);
		pppEntry_t->macInfo->memberPortMask = mbrMask;
		_rtl8651_modifyVlanRefFilterDatabaseEntry(0, &pppEntry_t->macInfo->macAddr, &mbrMask, RTL8651_FORWARD_MAC, NULL, L2_FORCE_STATIC);
	}
	_rtl8651_delVlanRefFilterDatabaseEntry(
			RTL8651_L2_TYPEI, 
			DrvTbl.vlan[rtl8651_vlanTableIndex(pppEntry_t->vid)].fid, 
			&pppEntry_t->macInfo->macAddr
	);
	_rtl8651_updateDefaultRoute();
	/* When a PPTP/L2TP session is hung up, we add an interface route for "outer subnet". */
	if (l2tpDefaultGwIp_tmp)
		_rtl8651_addPPTPL2TPInterfaceRoute(pppEntry_t, l2tpDefaultGwIp_tmp);
	else
		_rtl8651_addPPTPL2TPInterfaceRoute(pppEntry_t, l2tpServerIp_tmp);
	_rtl8651_updateNextHop_NHIDX(NULL, pppEntry_t);
	tunnel.wanPort			= 0xff;
	rtlglue_drvMutexUnlock();

	return retval;
}








/*=======================================
  * IP TABLE Internal APIs 
  *=======================================*/
#define RTL865X_IP_TABLE_APIs


static void _rtl8651_updateIPTableEntry(rtl8651_tblDrv_ipEntry_t *entry_t)
{
	rtl865x_tblAsicDrv_extIntIpParam_t asic;
	int32 retval;

	if (rtl8651_getAsicExtIntIpTable(IP_TABLE_INDEX(entry_t), &asic) == FAILED)
		return;
	asic.nhIndex = (entry_t->nextHop_t) ? entry_t->nextHop_t->entryIndex: 0;
	retval = rtl8651_setAsicExtIntIpTable(IP_TABLE_INDEX(entry_t), &asic);
	assert(retval == SUCCESS);
}


static void _rtl8651_ipTbl_setDefNexthop(rtl8651_tblDrv_routeTable_t *rt_t)
{
	rtl8651_tblDrv_ipEntry_t *entry_t = DrvTbl.iptbl;
	int32 entry, retval;
	int8 flag;

	if (rt_t && rt_t->nextHopType != RTL8651_LL_VLAN)
		return;
	for(entry=0; entry<RTL8651_IPTABLE_SIZE; entry++, entry_t++) {
		flag = (entry_t->type==IP_ATTR_NAPT)? NEXTHOP_NAPT_H2: NEXTHOP_NAT;
		if (rt_t && entry_t->def_flag == 1) {
			retval = _rtl8651_addNextHopEntry(flag, entry_t, rt_t->netif_t, rt_t->nextHop, 0, NULL);
			assert(retval == SUCCESS);
		}
		if (!rt_t && entry_t->def_flag == 1 && entry_t->nextHop_t) {
			retval = _rtl8651_delNextHopEntry(flag, entry_t);
			assert(retval == SUCCESS);
		}
	}
}

#define RTL8651_GIDXREG_NUM	8
void _rtl8651_arrangeAlliptbl_patch(void)
{
	rtl8651_tblDrv_ipEntry_t *ipEntry_t = DrvTbl.iptbl;
	rtl865x_tblAsicDrv_extIntIpParam_t asic_ip;
	uint32 entry, validIpEntryNum = 0, regValue = 0, ipidx_candidate = 0;
	int32 retval;

	/* all the following patch is performed when default napt ip exists. */
	if (def_napt_t)
	{
		/* (1) Traverse whole ip table entries recorded in driver, and write them to ASIC ip tables. */
		for (entry = 0; entry < RTL8651_IPTABLE_SIZE; entry++, ipEntry_t++)
		{
			retval = rtl8651_delAsicExtIntIpTable(entry);
			assert(retval == SUCCESS);
			
			if (ipEntry_t->valid == 1)
			{
				validIpEntryNum++;
				
				/* find out the candidate ip table index */
				if ((ipEntry_t->type == IP_ATTR_NAPT) && (ipEntry_t->extip == def_napt_t->extip))
					ipidx_candidate |= entry;
				
				bzero(&asic_ip, sizeof(rtl865x_tblAsicDrv_extIntIpParam_t));
				asic_ip.extIpAddr	= ipEntry_t->extip;
				asic_ip.intIpAddr	= ipEntry_t->intip;
				asic_ip.localPublic	= (ipEntry_t->type == IP_ATTR_LS)? TRUE: FALSE;
				asic_ip.nat		= (ipEntry_t->type == IP_ATTR_NAT)? TRUE: FALSE;
				asic_ip.nhIndex	= ipEntry_t->nextHop_t ? ipEntry_t->nextHop_t->entryIndex: 0;
				retval = rtl8651_setAsicExtIntIpTable(entry, &asic_ip);
				assert(retval == SUCCESS);
			}
		}
		
		/* (2) If the candidate ip table entry does not exist, create one because gidx register will reference it. */
		if (validIpEntryNum != 0 /* ip table in driver is not empty */ && DrvTbl.iptbl[ipidx_candidate].valid == 0)
		{
			retval = rtl8651_getAsicExtIntIpTable(IP_TABLE_INDEX(def_napt_t) , &asic_ip);
			assert(retval == SUCCESS);
			retval = rtl8651_setAsicExtIntIpTable(ipidx_candidate, &asic_ip);
			assert(retval == SUCCESS);
		}
		
		/* (3) update gidx register */
		for (entry = 0; entry < RTL8651_IPTABLE_SIZE; entry++)
			_gidxRegister[entry] = ipidx_candidate;
		
		for (entry = 0; entry < RTL8651_GIDXREG_NUM; entry++)
			regValue |= (ipidx_candidate << (3 * entry));		
		retval = rtl8651_setAsicGidxRegister(regValue);
		assert(retval == SUCCESS);	
	}
}

static int32 _rtl8651_delIpEntry(int8 attr, ipaddr_t extip, ipaddr_t intip) 
{
	rtl8651_tblDrv_ipEntry_t *ipEntry_t=DrvTbl.iptbl;
	int32 entry, retval;

	/* Assumption: intip is treated as nexthop when the attr is IP_ATTR_NAPT.  */
	
	for(entry=0; entry<RTL8651_IPTABLE_SIZE; entry++, ipEntry_t++) {
		if (ipEntry_t->valid == 0 || ipEntry_t->extip!=extip)
			continue;	
		switch (attr)
		{
		case IP_ATTR_LS:		goto found;
		case IP_ATTR_NAT:		if (ipEntry_t->intip == intip)
								goto found;
		case IP_ATTR_NAPT:
			if (ipEntry_t->nextHop_t && ipEntry_t->nextHop_t->nextHopType == RTL8651_LL_VLAN) {
				if (intip == 0) { /* nexthop is default route's nexthop */
					if (ipEntry_t->def_flag != 1) 
						break;
				} else if (ipEntry_t->nextHop_t->nexthop != intip) 
					break;
			}
			if (--ipEntry_t->ref_cnt == 0) {
				_rtl8651_delNextHopEntry(NEXTHOP_NAPT_H2, ipEntry_t);
				goto found;
			}
			return SUCCESS;
		
		default: assert(0);
		}
	}
found:

	if (entry == RTL8651_IPTABLE_SIZE)
		return TBLDRV_EENTRYNOTFOUND;

	ipEntry_t->valid = 0; /* Invalidate the entry */
	retval = rtl8651_delAsicExtIntIpTable(entry);
	assert(retval == SUCCESS);

	if (attr == IP_ATTR_LS)
		localServerCount--;
	else if (attr == IP_ATTR_NAT)
		natIpCount--;
	else naptIpCount--;
	if (naptIpCount+natIpCount+localServerCount == 0) 
		rtl8651_setAsicOperationLayer(3);
	assert(natIpCount+naptIpCount+localServerCount <= RTL8651_IPTABLE_SIZE);
	_rtl8651_updateGidxRegister();		
	_rtl8651_RearrangeHWTrafficCounter();

	/* Patch for "patch for interface route problem" */
	_rtl8651_arrangeAlliptbl_patch();

	return SUCCESS;
}



rtl8651_tblDrv_ipEntry_t *
_rtl8651_addIpEntry(ipaddr_t extip, ipaddr_t intip, int8 attr, rtl8651_tblDrv_networkIntfTable_t *netif_t) 
{
	rtl8651_tblDrv_ipEntry_t *ipEntry_t=NULL, *track_t = DrvTbl.iptbl;
	rtl865x_tblAsicDrv_extIntIpParam_t asic_ip;
	rtl8651_tblDrv_pppTable_t *ppp_t;
	//rtl8651_tblDrvSession_t *session_t;
	int32 entry, retval;

	/* Assumption: 
	 * Three types for parameter attr:
	 * 1. IP_ATTR_NAT		: Only add NAT mapping to IP Table
	 * 2. IP_ATTR_LS		: Only add LS to IP Table
	 * 3. IP_ATTR_NAPT		: Beside add to IP Table, also need to add a nexthop entry and assoicate it with 
	 *					  IP entry. 
	 *					  Note : the parameter, intip will be intepreted as nexthop IP Address
	 *					  	     in this type. If intip is zero, always use default route's nexthop as its
	 *						     nexthop.
	 */

	/* Reject duplicate entry: if NAPT, not apply */
	for(entry=0; entry<RTL8651_IPTABLE_SIZE; entry++, track_t++)  {
		if (track_t->valid == 0) {
			if (ipEntry_t == NULL) ipEntry_t = track_t; /* find an empty entry */
			//chhuang:
			//  We disable the following code. Note that the configure sequence shuold be 
			//  1. add NAPT IP address
			//  2. add NAT/LS
			//  This is because that the NAPT IP should be ranged from index 0 to index 7

			if (ipEntry_t && attr==IP_ATTR_LS && (ipEntry_t-DrvTbl.iptbl)<2)
				ipEntry_t = NULL;
			continue;
		}

		if (attr == IP_ATTR_NAPT) {
			/* Same NAPT IP is specified, if same nexthop, we do not need to add a new entry */
			if (track_t->extip == extip && track_t->nextHop_t) {
				if (track_t->nextHop_t->nextHopType == RTL8651_LL_VLAN) {
					if (intip == 0) { /* use default route's nexthop */
						if (track_t->def_flag == 0)
							continue;
					} else if (intip != track_t->nextHop_t->nexthop)
							continue;
				}
				track_t->ref_cnt ++;
				return track_t;
			}
		} else if (track_t->extip == extip)
			return NULL; /* Duplicate external IP found */

#if 0	// RTL865xB d-cut testing hash-2
		// ------------- Testing --------------
		if (track_t->extip == extip)
			return track_t;
		// ------------- End of testing ---------
#endif

	}
	if (ipEntry_t == NULL)
		return NULL; /* Out of IP entry */

	/* An free entry was found for new one */
	ipEntry_t->extip		= extip;
	ipEntry_t->intip		= (attr==IP_ATTR_NAPT)? 0 :intip;
	ipEntry_t->type			= attr;
	ipEntry_t->valid		= 1;
	ipEntry_t->ref_cnt		= 1;
	ipEntry_t->def_flag 	= 0;
	ipEntry_t->nextHop_t	= NULL;
	
	if (attr == IP_ATTR_NAPT) {
		if (netif_t->linkLayerType == RTL8651_LL_VLAN) 
			ppp_t = NULL;
		else if ((ppp_t = _rtl8651_getPPPInfo(0, extip)) == NULL) {
				ipEntry_t->valid = 0;
				return NULL;
		}
		if (_rtl8651_addNextHopEntry(NEXTHOP_NAPT_H2, ipEntry_t, netif_t, intip, extip, ppp_t) == FAILED) {
			ipEntry_t->valid = 0;
			return NULL;
		}	
	} 
	

	/* Set asic */
	bzero(&asic_ip, sizeof(rtl865x_tblAsicDrv_extIntIpParam_t));
	asic_ip.extIpAddr	= extip;
	asic_ip.intIpAddr	= ipEntry_t->intip;
	asic_ip.localPublic	= (attr == IP_ATTR_LS)? TRUE: FALSE;
	asic_ip.nat			= (attr == IP_ATTR_NAT)? TRUE: FALSE;
	asic_ip.nhIndex		= ipEntry_t->nextHop_t ? ipEntry_t->nextHop_t->entryIndex: 0;
	retval = rtl8651_setAsicExtIntIpTable(IP_TABLE_INDEX(ipEntry_t), &asic_ip);
	assert(retval == SUCCESS);

	if (attr == IP_ATTR_NAPT)
		naptIpCount++;
	else if (attr == IP_ATTR_LS)
		localServerCount++;
	else if (attr == IP_ATTR_NAT)
		natIpCount++;
	if (natIpCount+naptIpCount+localServerCount >= 1) 
		rtl8651_setAsicOperationLayer(4);
	assert(natIpCount+naptIpCount+localServerCount <= RTL8651_IPTABLE_SIZE);
	_rtl8651_updateGidxRegister();
	_rtl8651_RearrangeHWTrafficCounter();

	/* Patch for "patch for interface route problem" */
	_rtl8651_arrangeAlliptbl_patch();

	return ipEntry_t;	
}


rtl8651_tblDrv_ipEntry_t  *
_rtl8651_getIPTableEntry(ipaddr_t extip, ipaddr_t intip, uint8 attr)
{
	rtl8651_tblDrv_ipEntry_t *asic_t = DrvTbl.iptbl;
	uint32 entry=0;

	/*
	  *  IP_ATTR_LS:	get Local Server entry, the intip is ignored.
	  *  IP_ATTR_NAT:	get NAT entry, the intip and extip should have value.
	  *  IP_ATTR_NAPT:	get NAPT entry, the intip is ignored.
	  *  IP_ATTR_ALL:	if extip!=0, use it to find entry; otherwise use 'intip' to find entry
	  */
		
	for(entry=0; entry<RTL8651_IPTABLE_SIZE; entry++, asic_t++) {
		switch (attr)
		{
		case IP_ATTR_NAPT:
			if ((asic_t->extip == extip)&&(asic_t->type==IP_ATTR_NAPT))
				return asic_t;
			break;
		case IP_ATTR_NAT:
			if ((asic_t->extip == extip) && (asic_t->intip == intip)&&(asic_t->type==IP_ATTR_NAT))
				return asic_t;
			break;
		case IP_ATTR_LS:
			if ((asic_t->extip == extip)&&(asic_t->type==IP_ATTR_LS))
				return asic_t;
			break;
		case IP_ATTR_ALL:
			if (extip) {
				if (extip == asic_t->extip)
					return asic_t;
					}
			if (intip) {
				if (intip == asic_t->intip)
					return asic_t;
				}
			break;
		default: assert(0);
			}
	}
	return (rtl8651_tblDrv_ipEntry_t *)NULL;
}






/*=======================================
  * Multicast Table APIs
  *=======================================*/
#define RTL865X_MULTICASE_TABLE_APIs

void inline _rtl8651_setASICMulticastPortStatus(void) {
	uint32 index;

	for (index=0; index<RTL8651_PORT_NUMBER+rtl8651_totalExtPortNum; index++) {
		rtl8651_setAsicMulticastPortInternal(index, (rtl8651_externalMulticastPortMask&(1<<index))?FALSE:TRUE);
	}
}

void _rtl8651_arrangeMulticastPortStatus(void) {

	uint32 index;

	for (index=0; index<RTL8651_PORT_NUMBER+rtl8651_totalExtPortNum; index++) {
		/*
		  *	Check all vlans with this port is their memberport:
		  *
		  *		Internal: one of vlan is internal or this port is not in any vlan
		  *		External: all of vlans are external
		  */
		uint8 vlanCnt;
		uint8 i;
		rtl8651_tblDrv_vlanTable_t *vlanPtr;

		vlanCnt = 0;
		for(i=0,vlanPtr=&DrvTbl.vlan[0]; i<RTL8651_VLAN_NUMBER; i++, vlanPtr++) {
			if (!(vlanPtr->memberPortMask & (1<<index)) || !vlanPtr->valid)
				continue;
			vlanCnt ++;
			if (vlanPtr->internal)
				goto isInternal;
		}
		if (vlanCnt) { /* external */
			rtl8651_externalMulticastPortMask |= (1<<index);
		} else {
	isInternal:
			rtl8651_externalMulticastPortMask &= ~(1<<index);
		}
	}
	_rtl8651_setASICMulticastPortStatus();
}

/*
@func int32 | rtl8651_addExternalMulticastPort | Set the specific port to be the external port for IP-Multicast system
@parm uint32 | extMPort | External Port
@rvalue SUCCESS | The specified ports was successfully changed to external.
*/
int32 rtl8651_addExternalMulticastPort(uint32 extMPort)
{
	rtl8651_externalMulticastPortMask |= (1<<extMPort);
	_rtl8651_setASICMulticastPortStatus();

	return SUCCESS;
}

/*
@func int32 | rtl8651_delExternalMulticastPort | Set the specific port back to be the internal port for IP-Multicast system
@parm uint32 | extMPort | External Port
@rvalue SUCCESS | The specified ports was successfully changed to external.
*/
int32 rtl8651_delExternalMulticastPort(uint32 extMPort)
{
	rtl8651_externalMulticastPortMask &= ~(1<<extMPort);
	_rtl8651_setASICMulticastPortStatus();

	return SUCCESS;
}


#define RTL865X_MULTICAST_TABLE_AGE				35
#define RTL865X_MULTICAST_TABLE_ASIC_FULL_AGE	35
#define RTL865X_MULTICAST_TABLE_ASIC_AGE			10	/* we would always write this value into ASIC */
#define RTL865X_MULTICAST_SWAP_THRESHOLD		300
#define SINGLE_BITMASK_ISSET(x) ((x & (x - 1)) == 0)	/* chenyl: only 1 bit is set in vlanMask */

static inline void _rtl8651_patchPppoeWeak(rtl8651_tblDrv_mCast_t *mCast_t)
{
	uint8 idx;
	rtl8651_tblDrv_vlanTable_t *	 vptr;

	/* patch: keep cache in software if one vlan's interface is pppoe */
	vptr = DrvTbl.vlan;
	for (idx = 0 ; idx < RTL8651_VLAN_NUMBER ; idx ++, vptr ++) {
		if ((1<<idx) & mCast_t->vlanMask) {
			rtl8651_tblDrv_networkIntfTable_t *netif;
			assert(vptr->valid);
			if (((netif = _rtl8651_getNetworkIntfEntryByVid(vptr->vid)) != NULL) &&
				(netif->linkLayerType == RTL8651_LL_PPPOE)) {
				mCast_t->flag |= RTL865X_MULTICAST_PPPOEPATCH_CPUBIT;
				return;
			}
		}
	}
	mCast_t->flag &= ~RTL865X_MULTICAST_PPPOEPATCH_CPUBIT;
}

#ifdef RTL865xB_MCAST_CUTCD_TTL_PATCH
/* re-select Multicast entry to write to ASIC in order to fix the problem for 865xB cut C/D */
#define MIN(a,b) ((a>b)?b:a)
typedef struct __rtl8651_mcastAsicDipUsage_s {
	ipaddr_t	dip;
	uint8	refcnt;
	uint8	ipEntryIdx;
} __rtl8651_mcastAsicDipUsage_t;
void _rtl8651_arrangeAllMulticast(void)
{
	__rtl8651_mcastAsicDipUsage_t asicMcastEntryDip[RTL8651_IPMULTICASTTBL_SIZE];
	int32 totalEmptyIpEntryCnt = 0;
	int32 totalWanToLanDipCnt = 0;
	int32 patchIpEntryIdx = 0;
	int32 idx;
	/*
		We only patch for 865xB cut C/D
	*/
	if (!	((RtkHomeGatewayChipName[strlen(RtkHomeGatewayChipName) - 1] == 'B') &&
		  (RtkHomeGatewayChipRevisionID >= 0x02) &&
		  (RtkHomeGatewayChipRevisionID <= 0x03)))
	{
		return;
	}

	/*
		Step 1: We check how many empty IP entries can be used to patch this problem.
	*/
	for ( idx = 0 ; idx < RTL8651_IPTABLE_SIZE ; idx ++ )
	{
		rtl865x_tblAsicDrv_extIntIpParam_t dummy;
		if ( rtl8651_getAsicExtIntIpTable( idx, &dummy ) != SUCCESS )
		{
			totalEmptyIpEntryCnt ++;
		}
	}

	RTL_DEBUG(	RTL_MSG_MCAST,
				"Total Empty IP Entry : %d",
				totalEmptyIpEntryCnt);

	/*
		Step 2: Summarise the Mcast-IP address requirement for existing HW-forwarded WAN->LAN multicast flow.
	*/
	bzero(asicMcastEntryDip, (sizeof(__rtl8651_mcastAsicDipUsage_t) * RTL8651_IPMULTICASTTBL_SIZE));

	for ( idx = 0 ; idx < RTL8651_IPMULTICASTTBL_SIZE ; idx ++ )
	{
		rtl8651_tblDrv_mCast_t *mCast_t;

		TAILQ_FOREACH(mCast_t, &DrvTbl.inuseList.mCastTbl[idx], nextMCast)
		{
			if (	( mCast_t->inAsic == TRUE /* In ASIC now */) &&
				( mCast_t->ipEntry_t == NULL /* WAN->LAN */))
			{
				int32 dipIdx;
				/* find if the DIP exists in previous entry */
				for ( dipIdx = 0 ; dipIdx < RTL8651_IPMULTICASTTBL_SIZE ; dipIdx ++ )
				{
					if ( asicMcastEntryDip[dipIdx].dip == mCast_t->dip )
					{	/* use same DIP , add its reference count */
						asicMcastEntryDip[dipIdx].refcnt ++;

						RTL_DEBUG(	RTL_MSG_MCAST,
									"ASIC Mcast DIP (%u.%u.%u.%u) RefCnt : %d",
									NIPQUAD(asicMcastEntryDip[dipIdx].dip),
									asicMcastEntryDip[dipIdx].refcnt);
						
						break;
					}
					if ( asicMcastEntryDip[dipIdx].dip == 0 )
					{	/* not found */
						break;
					}
				}
				if ( dipIdx >= RTL8651_IPMULTICASTTBL_SIZE )
				{	/* impossible ! */
					RTL_BUG("Total DIP count can NOT exceed ASIC multicast table size.");
				} else
				{
					if ( asicMcastEntryDip[dipIdx].dip == 0 )
					{
						asicMcastEntryDip[dipIdx].dip = mCast_t->dip;
						asicMcastEntryDip[dipIdx].refcnt = 1;
						totalWanToLanDipCnt ++;

						RTL_DEBUG(	RTL_MSG_MCAST,
									"ASIC Mcast DIP (%u.%u.%u.%u) RefCnt : %d",
									NIPQUAD(asicMcastEntryDip[dipIdx].dip),
									asicMcastEntryDip[dipIdx].refcnt);

					}
				}
				goto step2NextIdx;
			}
		}
step2NextIdx:
		;
	}

	/*
		Step 3:
			Choose what entry can be patched using IP Entry

			If { total WAN->LAN multicast entry count } > { empty IP entry count},
			we need to choice some of them.

			If { total WAN->LAN multicast entry count } <= { empty IP entry count},
			All entries can be choosed.
	*/
	RTL_DEBUG(	RTL_MSG_MCAST,
				"Total Empty IP table entry cnt: %d -- Total dip need patch: %d %s",
				totalEmptyIpEntryCnt,
				totalWanToLanDipCnt,
				(totalEmptyIpEntryCnt < totalWanToLanDipCnt)?"\n\t=> [Must Trap]":"");

	{
		__rtl8651_mcastAsicDipUsage_t *currMax = NULL;
		int32 remainPatchCnt = MIN(totalEmptyIpEntryCnt, totalWanToLanDipCnt);

		/*
			Design Note:
				Now we use asicMcastEntryDip[] to choose the DIPs with MAX reference count to patch.
				In step 2, all DIPs have been gotten and we wrote them into asicMcastEntryDip[].
				Now, we define that:
				1. asicMcastEntryDip[].dip == 0 --> Empty entry.
				2. asicMcastEntryDip[].dip != 0, asicMcastEntryDip[].refcnt > 0   --> Entry not be selected
				3. asicMcastEntryDip[].dip != 0, asicMcastEntryDip[].refcnt == 0 --> Entry is selected
		*/

		while ( remainPatchCnt > 0 )
		{
			currMax = NULL;
			for ( idx = 0 ; idx < RTL8651_IPMULTICASTTBL_SIZE ; idx ++ )
			{
				if (	( asicMcastEntryDip[idx].dip != 0 /* non-empty entry */ ) &&
					( asicMcastEntryDip[idx].refcnt > 0 /* Not be selected yet */))
				{
					if (	( currMax == NULL ) ||
						( asicMcastEntryDip[idx].refcnt > currMax->refcnt ))
					{
						currMax = &asicMcastEntryDip[idx];
					}
				}
			}
			if ( currMax == NULL )
			{	/* impossible */
				RTL_BUG("least one HW used DIP entry must be found. (make sure this at step 1)");
			} else
			{
				currMax->refcnt = 0;
				remainPatchCnt --;
			}
		}
	}
	/*
		Step 4:
			Modify IP Entries to patch mcast entry.
	*/
	{
		int32 ipEntryIdx = 0, nextPatchDipIdx = 0;
		int32 remainPatchCnt = MIN(totalEmptyIpEntryCnt, totalWanToLanDipCnt);

		while ( remainPatchCnt > 0 )
		{
			rtl865x_tblAsicDrv_extIntIpParam_t dummy;

			if ( ipEntryIdx >= RTL8651_IPTABLE_SIZE )
			{
				RTL_BUG("least one HW used DIP entry must be found. (make sure this at step 1)");
			}

			/* find empty IP Table Entry to patch */
			if ( rtl8651_getAsicExtIntIpTable( ipEntryIdx, &dummy ) != SUCCESS )
			{
				for ( idx = nextPatchDipIdx ; idx < RTL8651_IPMULTICASTTBL_SIZE ; idx ++ )
				{
					if (	( asicMcastEntryDip[idx].dip != 0 ) &&
						( asicMcastEntryDip[idx].refcnt == 0 ))
					{
						/* patch IP Entry */
						rtl865x_tblAsicDrv_extIntIpParam_t ipEntry_t;

						bzero(&ipEntry_t, sizeof(ipEntry_t));
						ipEntry_t.extIpAddr = asicMcastEntryDip[idx].dip;
						rtl8651_setInvalidAsicExtIntIpTable( ipEntryIdx, &ipEntry_t );
						asicMcastEntryDip[idx].ipEntryIdx = ipEntryIdx;

						RTL_DEBUG(	RTL_MSG_MCAST,
									"@ Avaliable IP Entry (0x%x) for %u.%u.%u.%u",
									ipEntryIdx,
									NIPQUAD(asicMcastEntryDip[idx].dip));


						nextPatchDipIdx = idx + 1;
						remainPatchCnt --;
						goto nextIpEntryPatch;
					}
				}
				/* normal process MUST match one of entry and jump to "nextIpEntryPatch" */
				RTL_BUG("For all empty IP entry, we would always can find a DIP to patch. (make sure at step 1/2).");
			}
nextIpEntryPatch:
			ipEntryIdx ++;
		}

		patchIpEntryIdx = ipEntryIdx;	/* for Step6's patch */
	}
	/*
		Step 5:
			Modify Multicast Entries to apply the patch.
	*/
	{
		for ( idx = 0 ; idx < RTL8651_IPMULTICASTTBL_SIZE ; idx ++ )
		{
			rtl8651_tblDrv_mCast_t *mCast_t;

			TAILQ_FOREACH(mCast_t, &DrvTbl.inuseList.mCastTbl[idx], nextMCast)
			{
				if (	( mCast_t->inAsic == TRUE /* In ASIC */ ) &&
					( mCast_t->ipEntry_t == NULL /* WAN->LAN */ ) )
				{
					int mcastDipIdx;
					/* check if WAN->LAN ASIC entry can be patched */
					for ( mcastDipIdx = 0 ; mcastDipIdx < RTL8651_IPMULTICASTTBL_SIZE ; mcastDipIdx ++ )
					{
						/* can be patched */
						if (	( asicMcastEntryDip[mcastDipIdx].dip == mCast_t->dip ) &&
							( asicMcastEntryDip[mcastDipIdx].refcnt == 0 ))
						{
							rtl865x_tblAsicDrv_multiCastParam_t asic_mcast;

							RTL_DEBUG(	RTL_MSG_MCAST,
										"Mcast Entry (%d) %u.%u.%u.%u - Patched IP idx (0x%x)",
										idx,
										NIPQUAD(mCast_t->dip),
										asicMcastEntryDip[mcastDipIdx].ipEntryIdx);


							/* update MCAST table for this patch */
							bzero(&asic_mcast, sizeof(asic_mcast));
							rtl8651_memcpy(&asic_mcast, mCast_t, (uint32)&(((rtl8651_tblDrv_mCast_t *)0)->ipEntry_t));
							asic_mcast.extIdx = asicMcastEntryDip[mcastDipIdx].ipEntryIdx;
							rtl8651_setAsicIpMulticastTable(&asic_mcast);
							/* we don't need to modify this entry's position */
							break;
						}
					}

					RTL_NOTICE(	RTL_MSG_MCAST,
									"[TRAP] => Mcast Entry (%d) %u.%u.%u.%u",
									idx,
									NIPQUAD(mCast_t->dip));


					/* this ASIC entry CAN NOT be patched ! ==> Choose other entry, or trap it. */
					if ( mcastDipIdx >= RTL8651_IPMULTICASTTBL_SIZE )
					{
						rtl8651_tblDrv_mCast_t *swMcast_t = NULL;
						rtl8651_tblDrv_mCast_t *select_t = NULL;
						__rtl8651_mcastAsicDipUsage_t *swMcastPatch = NULL;
						__rtl8651_mcastAsicDipUsage_t *selectPatch = NULL;

						TAILQ_FOREACH(swMcast_t, &DrvTbl.inuseList.mCastTbl[idx], nextMCast)
						{
							if (	(swMcast_t != mCast_t) &&
								(swMcast_t->cpu == 0) &&
								((swMcast_t->flag & RTL865X_MULTICAST_PPPOEPATCH_CPUBIT) == 0))
							{
								swMcastPatch = NULL;
								if ( swMcast_t->ipEntry_t == NULL /* for WAN->LAN entry, we need to check it's DIP can be patched or not */ )
								{
									for ( mcastDipIdx = 0 ; mcastDipIdx < RTL8651_IPMULTICASTTBL_SIZE ; mcastDipIdx ++ )
									{
										if (	( asicMcastEntryDip[mcastDipIdx].dip == mCast_t->dip ) &&
											( asicMcastEntryDip[mcastDipIdx].refcnt == 0 ))
										{
											swMcastPatch = &(asicMcastEntryDip[mcastDipIdx]);
											break;
										}
									}
									if ( swMcastPatch == NULL )
									{
										goto nextMcastIdxEntry;
									}
								}

								/*
									Here : the swMcast_t is ...
										1. Forwarded via SW
										2. can be forwarded via HW
										3. can be patched if its WAN->LAN entry, the patched DIP is pointed by "wMcastPatch"
										==> It can we written into HW
								*/
								if (	( select_t == NULL ) ||
									( swMcast_t->count > select_t->count  ) )
								{
									select_t = swMcast_t;
									selectPatch = swMcastPatch;
								}

							}
nextMcastIdxEntry:
							;
						}

						/* newly HW entry of this MCAST index has been selected */

						/* First : ALWAYS remove mCast_t due to we can NOT patch it */
						mCast_t->inAsic = FALSE;

						/* Second : Write new entry into ASIC if we can */
						if ( select_t )
						{
							/* we write new entry into ASIC */
							rtl865x_tblAsicDrv_multiCastParam_t asic_mcast;

							select_t->inAsic = TRUE;
							select_t->age = RTL865X_MULTICAST_TABLE_ASIC_AGE;
							bzero(&asic_mcast, sizeof(rtl865x_tblAsicDrv_multiCastParam_t));
							rtl8651_memcpy(&asic_mcast, select_t, (uint32)&(((rtl8651_tblDrv_mCast_t *)0)->ipEntry_t));
							if ( select_t->ipEntry_t )
							{
								asic_mcast.extIdx = IP_TABLE_INDEX( select_t->ipEntry_t );
							} else
							{
								if ( selectPatch == NULL )
								{
									RTL_BUG("selectPatch must be found at above code.");
								}
								asic_mcast.extIdx = selectPatch->ipEntryIdx;
							}
							rtl8651_setAsicIpMulticastTable( &asic_mcast );

							RTL_DEBUG(	RTL_MSG_MCAST,
										"[Replace] => %s Mcast Entry (%d) %u.%u.%u.%u - Patched IP idx (0x%x)",
										( select_t->ipEntry_t )?"(LAN->WAN)":"(WAN->LAN)",
										idx,
										NIPQUAD(select_t->dip),
										asic_mcast.extIdx);

							TAILQ_REMOVE(&DrvTbl.inuseList.mCastTbl[idx], select_t, nextMCast);
							TAILQ_INSERT_HEAD(&DrvTbl.inuseList.mCastTbl[idx], select_t, nextMCast);
						} else
						{
							RTL_DEBUG(	RTL_MSG_MCAST,
										"[Empty] => Mcast Entry (%d)",
										idx);

							rtl8651_delAsicIpMulticastTable( idx );
						}
					}

					/* check next idx */
					goto step5NextIdx;
				}
			}
step5NextIdx:
			;
		}
	}
	/*
		Step 6:
			If there are some additional empty IP entries can be used.
			We check if there is any flow can be written into ASIC.
	*/
	if ( totalEmptyIpEntryCnt > totalWanToLanDipCnt )
	{
		int32 remainEmptyIpEntryCnt = totalEmptyIpEntryCnt - totalWanToLanDipCnt;
		int32 currPatchIpEntryIdx;

		for ( idx = 0 ; idx < RTL8651_IPMULTICASTTBL_SIZE ; idx ++ )
		{
			rtl8651_tblDrv_mCast_t *mCast_t = NULL;
			rtl8651_tblDrv_mCast_t *select_t = NULL;

			TAILQ_FOREACH(mCast_t, &DrvTbl.inuseList.mCastTbl[idx], nextMCast)
			{
				if ( mCast_t->inAsic == TRUE )
				{
					break;
				}
			}
			/* HW entry exists at this index */
			if ( mCast_t )
			{
				continue;
			}

			/* No HW entry for this index ==> try to swap one */
			currPatchIpEntryIdx = -1;	/* means empty */
			TAILQ_FOREACH(mCast_t, &DrvTbl.inuseList.mCastTbl[idx], nextMCast)
			{
				if (	( mCast_t->cpu == 0 ) &&
					(( mCast_t->flag & RTL865X_MULTICAST_PPPOEPATCH_CPUBIT) == 0 ))
				{
					if (	( select_t == NULL ) ||
						( mCast_t->count > select_t->count)	)
					{
						if ( select_t->ipEntry_t == NULL )
						{
							int32 mcastDipIdx;

							/* check if exist DIP patch can be used, or use new IP entry to patch */
							for ( mcastDipIdx = 0 ; mcastDipIdx < RTL8651_IPMULTICASTTBL_SIZE ; mcastDipIdx ++ )
							{
								if (	( asicMcastEntryDip[mcastDipIdx].dip == mCast_t->dip ) &&
									( asicMcastEntryDip[mcastDipIdx].refcnt == 0 ))
								{
									currPatchIpEntryIdx = asicMcastEntryDip[mcastDipIdx].ipEntryIdx;
									break;
								}
							}

							/* If this entry need patch but no any additional ip entry for it, ignore */
							if (	( mcastDipIdx >= RTL8651_IPMULTICASTTBL_SIZE ) &&
								( remainEmptyIpEntryCnt == 0 ))
							{
								continue;
							}
						}
						/* select entry */
						select_t = mCast_t;
					}
				}
			}

			/*
				Here :
					We have selected one entry.
					If it is
						1. LAN->WAN entry, no patch need.
						2. WAN->LAN entry, dip is patched. => The patch IpEntry IDX is stored in "currPatchIpEntryIdx"
						3. WAN->LAN entry, need additional dip patch => We would have empty entry for patch.
				
			*/
			if ( select_t )
			{
				rtl865x_tblAsicDrv_multiCastParam_t asic_mcast;

				RTL_DEBUG(	RTL_MSG_MCAST,
							"@ Idx (%d): entry selected -- DIP %u.%u.%u.%u %s",
							idx,
							NIPQUAD(select_t->dip),
							((select_t->ipEntry_t==NULL)&&(currPatchIpEntryIdx<0))?
								"(Need additional patch)":
								"");

				if ( select_t->ipEntry_t == NULL )
				{
					/* we patch it using additional empty IP entry */
					if ( currPatchIpEntryIdx < 0 )
					{
						int32 ipEntryIdx;

						for ( ipEntryIdx =  patchIpEntryIdx ; ipEntryIdx < RTL8651_IPTABLE_SIZE ; ipEntryIdx ++ )
						{
							rtl865x_tblAsicDrv_extIntIpParam_t dummy;

							/* find empty IP Table Entry to patch */
							if ( rtl8651_getAsicExtIntIpTable( ipEntryIdx, &dummy ) != SUCCESS )
							{
								break;
							}
						}
						if ( ipEntryIdx >= RTL8651_IPTABLE_SIZE )
						{
							RTL_BUG("There must have additional empty IP table. (make sure at start of step 6)");
						} else
						{	/* patch IP Entry */
							rtl865x_tblAsicDrv_extIntIpParam_t ipEntry_t;

							RTL_DEBUG(	RTL_MSG_MCAST,
										"@ Use IP Entry (0x%x) to patch DIP %u.%u.%u.%u",
										ipEntryIdx,
										NIPQUAD(select_t->dip));


							bzero(&ipEntry_t, sizeof(ipEntry_t));
							ipEntry_t.extIpAddr = select_t->dip;
							rtl8651_setInvalidAsicExtIntIpTable( ipEntryIdx, &ipEntry_t );
							currPatchIpEntryIdx = ipEntryIdx;
							patchIpEntryIdx = ipEntryIdx + 1;
							remainEmptyIpEntryCnt --;
						}
					}
				}

				/* add multicast entry and add patch */
				select_t->inAsic = TRUE;
				select_t->age = RTL865X_MULTICAST_TABLE_ASIC_AGE;
				bzero( &asic_mcast, sizeof( rtl865x_tblAsicDrv_multiCastParam_t ) );
				rtl8651_memcpy(&asic_mcast, select_t, (uint32)&(((rtl8651_tblDrv_mCast_t *)0)->ipEntry_t));
				if ( select_t->ipEntry_t )
				{
					asic_mcast.extIdx = IP_TABLE_INDEX( select_t->ipEntry_t );
				} else
				{
					asic_mcast.extIdx = currPatchIpEntryIdx;
				}
				rtl8651_setAsicIpMulticastTable( &asic_mcast );
				TAILQ_REMOVE( &DrvTbl.inuseList.mCastTbl[idx], select_t, nextMCast );
				TAILQ_INSERT_HEAD( &DrvTbl.inuseList.mCastTbl[idx], select_t, nextMCast );

			}
		}
	}

}
#endif

/* re-select Multicast entry to ASIC for the index ""entryIndex */
static void _rtl8651_arrangeMulticast(uint32 entryIndex)
{
	rtl865x_tblAsicDrv_multiCastParam_t asic_mcast;
	rtl8651_tblDrv_mCast_t *mCast_t, *select_t=NULL;
	int32 retval;
	
	TAILQ_FOREACH(mCast_t, &DrvTbl.inuseList.mCastTbl[entryIndex], nextMCast) {
		if ((mCast_t->cpu == 0) && !(mCast_t->flag & RTL865X_MULTICAST_PPPOEPATCH_CPUBIT)) { /* Ignore cpu=1 */
			if (select_t) {
				if (mCast_t->count > select_t->count)
					select_t = mCast_t;
			} else select_t = mCast_t;
		}
		mCast_t->inAsic = FALSE;	/* reset "inAsic" bit */
	} 
	if (select_t) {
		select_t->inAsic = TRUE;
		select_t->age = RTL865X_MULTICAST_TABLE_ASIC_AGE;
		bzero(&asic_mcast, sizeof(rtl865x_tblAsicDrv_multiCastParam_t));
		rtl8651_memcpy(&asic_mcast, select_t, (uint32)&(((rtl8651_tblDrv_mCast_t *)0)->ipEntry_t));
		if (select_t->ipEntry_t)
			asic_mcast.extIdx = IP_TABLE_INDEX(select_t->ipEntry_t);
		retval = rtl8651_setAsicIpMulticastTable(&asic_mcast);
		assert(retval == SUCCESS);
		TAILQ_REMOVE(&DrvTbl.inuseList.mCastTbl[entryIndex], select_t, nextMCast);
		TAILQ_INSERT_HEAD(&DrvTbl.inuseList.mCastTbl[entryIndex], select_t, nextMCast);

		#ifdef RTL865xB_MCAST_CUTCD_TTL_PATCH
		/* we only check HW mcast entry when new entry written into HW */
		_rtl8651_arrangeAllMulticast();
		#endif
	}
	else 	rtl8651_delAsicIpMulticastTable(entryIndex);
}




void _rtl8651_timeUpdateMulticast(uint32 secPassed)
{

	rtl8651_tblDrv_mCast_t *mCast_t, *tmp_t;
	uint32 entry;

	/* Note: Entris in the ASIC do not aging */

	/* check for HW packet forwarding */
	for (entry=0; entry< rtl8651_tblDrvPara.mCastTblSize; entry ++)
	{
		int32 idx;
		rtl865x_tblAsicDrv_multiCastParam_t mAsicCast_t;
		for ( idx = 0 ; idx < RTL8651_IPMULTICASTTBL_SIZE ; idx ++ )
		{
			if (rtl8651_getAsicIpMulticastTable(idx, &mAsicCast_t) == SUCCESS)
			{
				if (	(mAsicCast_t.mbr & rtl8651_externalMulticastPortMask) &&
					(mAsicCast_t.age > RTL865X_MULTICAST_TABLE_ASIC_AGE))
				{	/* this ASIC entry has traffic in past interval */
					rtl8651_tblDrv_ipEntry_t *ipEntry_t;
					ipEntry_t = &DrvTbl.iptbl[mAsicCast_t.extIdx];
					if ((mAsicCast_t.extIdx < RTL8651_IPTABLE_SIZE) && ipEntry_t->valid)
					{
						rtl8651_tblDrv_networkIntfTable_t *netif_t;
						if (((netif_t = _rtl8651_getIpInfo(	ipEntry_t->extip,
														NULL,
														NULL,
														NULL)) != NULL) &&
							(netif_t->valid))
						{
							if (netif_t->linkLayerType == RTL8651_LL_VLAN)
							{
								/*
									For VLAN type wan.
									=> Assumption: we assume there would only ONE external VLAN-type network interface.
								*/
								RTL_INFO(	RTL_MSG_SESSION_HANGUP,
											"Record traffic for (%s) - Dsid (%d)",
											netif_t->ifName,
											0);

								_rtl8651_recordTraffic( 0 /* update default session*/, TRUE /* to WAN */);
							} else
							{
								/* For session-based LL types */
								rtl8651_tblDrvSession_t *psession_t;
								CTAILQ_FOREACH(psession_t, &netif_t->sessionHead, nextSession)
								{
									if (psession_t->session_t && psession_t->session_t->valid == TRUE)
									{
										RTL_INFO(	RTL_MSG_SESSION_HANGUP,
													"Record traffic for (%s) - Dsid (%d)",
													netif_t->ifName,
													_rtl8651_SessionIDToDSID(psession_t->session_t->sessionId));

										_rtl8651_recordTraffic(_rtl8651_SessionIDToDSID(psession_t->session_t->sessionId), TRUE /* to WAN */);
									}
								}
							}
						}
					}
				}
			}
		}
	}

	/* check to Aging and HW swapping */
	for (entry=0; entry< rtl8651_tblDrvPara.mCastTblSize; entry++) {
		mCast_t = TAILQ_FIRST(&DrvTbl.inuseList.mCastTbl[entry]);
		while (mCast_t) {
			if (mCast_t->inAsic == TRUE) {
				/* Entry is in the ASIC */
				if (mCast_t->age <= secPassed) { /* try to swap other flow */
					mCast_t->age = RTL865X_MULTICAST_TABLE_AGE;
					_rtl8651_arrangeMulticast(entry);
				} else mCast_t->age -= secPassed;
			} else {
				/* Entry is not in the ASIC */
				if (mCast_t->age <= secPassed) { /* aging out */
					tmp_t = TAILQ_NEXT(mCast_t, nextMCast);
					TAILQ_REMOVE(&DrvTbl.inuseList.mCastTbl[entry], mCast_t, nextMCast);
					TAILQ_INSERT_HEAD(&DrvTbl.freeList.freeMultiCast, mCast_t, nextMCast);
					mCast_t = tmp_t;
					continue;
				} else mCast_t->age -= secPassed;
			}
			mCast_t->count = 0;
			mCast_t = TAILQ_NEXT(mCast_t, nextMCast);
		}
	}
}

int32 _rtl8651_addMulticast(ipaddr_t mAddr, ipaddr_t sip, uint16 svid, uint16 sport, uint32 dvidMask, uint32 mbr, ipaddr_t extIp, int8 toCpu, uint8 flag)
{

	rtl8651_tblDrv_mCast_t *mCast_t, *empty_t;
	uint32 entry = rtl8651_ipMulticastTableIndex(sip, mAddr);


	TAILQ_FOREACH(mCast_t, &DrvTbl.inuseList.mCastTbl[entry], nextMCast) {
		if (mCast_t->sip==sip && mCast_t->dip==mAddr && mCast_t->svid==svid && mCast_t->port==sport)
			break;
	}
	if (mCast_t == NULL) {
		if ((empty_t=TAILQ_FIRST(&DrvTbl.freeList.freeMultiCast)) == NULL)
			return FAILED;
		TAILQ_REMOVE(&DrvTbl.freeList.freeMultiCast, empty_t, nextMCast);
		TAILQ_INSERT_TAIL(&DrvTbl.inuseList.mCastTbl[entry], empty_t, nextMCast);
		mCast_t = empty_t;
		mCast_t->sip			= sip;
		mCast_t->dip			= mAddr;
		mCast_t->svid		= svid;
		mCast_t->port		= sport;
		mCast_t->vlanMask 	= 0;
		mCast_t->mbr		= 0;
		mCast_t->count		= 0;
		mCast_t->inAsic		= FALSE;
	}
	mCast_t->ipEntry_t		= (extIp)?_rtl8651_getIPTableEntry(extIp, 0, IP_ATTR_ALL):NULL;
	mCast_t->vlanMask		|= dvidMask;
	mCast_t->mbr			|= mbr;
	mCast_t->age			= RTL865X_MULTICAST_TABLE_AGE;
	mCast_t->cpu			= (toCpu==TRUE? 1: 0);
	mCast_t->flag			= flag;
	if (extIp)
		mCast_t->flag |= RTL865X_MULTICAST_EXTIP_SET;
	else
		mCast_t->flag &= ~RTL865X_MULTICAST_EXTIP_SET;

	_rtl8651_patchPppoeWeak(mCast_t);
	_rtl8651_arrangeMulticast(entry);
	return SUCCESS;	
}

int32 _rtl8651_addMulticastMember(ipaddr_t mcast_addr, uint32 vlanMask, uint32 mbr)
{

	rtl8651_tblDrv_mCast_t *mCast_t;
	uint32 entry, idx = 0, srcVlanMask, mbr_add;

	assert(SINGLE_BITMASK_ISSET(vlanMask));

	for (entry=0; entry< rtl8651_tblDrvPara.mCastTblSize; entry++) {
		TAILQ_FOREACH(mCast_t, &DrvTbl.inuseList.mCastTbl[entry], nextMCast) {
			if (mCast_t->dip != mcast_addr)
				continue;

		/* filter source vlan/port */
		srcVlanMask = 1<<rtl8651_vlanTableIndex(mCast_t->svid);
		mbr_add = mbr;
		if (srcVlanMask == vlanMask) {
			mbr_add = mbr & ~(1<<mCast_t->port);
			if (mbr_add == 0)
				continue;
		}
			
#ifndef RTL8651_MCAST_ALWAYS2UPSTREAM
			if (mCast_t->flag & RTL865X_MULTICAST_UPLOADONLY)
			{	/* remove upload term*/
				assert(vlanMask && mbr_add);
				mCast_t->flag &= ~RTL865X_MULTICAST_UPLOADONLY;
				mCast_t->mbr	  = 0;
				mCast_t->vlanMask = 0;
				/* we assume multicast member will NEVER in External interface, so we remove
				     external ip now */
				mCast_t->flag &= ~RTL865X_MULTICAST_EXTIP_SET;
				mCast_t->ipEntry_t = 0;
			}
#endif /* RTL8651_MCAST_ALWAYS2UPSTREAM */
			mCast_t->mbr 		|= mbr_add;
			mCast_t->vlanMask	|= vlanMask;

			_rtl8651_patchPppoeWeak(mCast_t);
			_rtl8651_arrangeMulticast(entry);
			idx = entry;
		}
	}

	if (idx) {
		return SUCCESS;
	}
	return FAILED;
}

int32 _rtl8651_delMulticast(ipaddr_t mcast_addr, uint32 vlanMask, uint32 mbr)
{

	rtl8651_tblDrv_mCast_t *mCast_t, *temp_t;
	uint32 entry;

	/*
		vlanMask == 0 : remove any entry with mcast address is mcast_addr,
		vlanMask != 0 : remove any entry with vlanMask and member port mbr
	*/

	for(entry=0; entry<rtl8651_tblDrvPara.mCastTblSize; entry++) {
		/* If both vlanMask==0 and mbr==0, imply removing mcast_addr group */
		if (!mbr && !vlanMask) {
			mCast_t = TAILQ_FIRST(&DrvTbl.inuseList.mCastTbl[entry]);
			while (mCast_t) {
				if (!mcast_addr || mCast_t->dip == mcast_addr) {
					temp_t = TAILQ_NEXT(mCast_t, nextMCast);
					TAILQ_REMOVE(&DrvTbl.inuseList.mCastTbl[entry], mCast_t, nextMCast);
					TAILQ_INSERT_HEAD(&DrvTbl.freeList.freeMultiCast, mCast_t, nextMCast);
					mCast_t = temp_t;
				}
				else mCast_t = TAILQ_NEXT(mCast_t, nextMCast);
			}
			_rtl8651_arrangeMulticast(entry);
			continue;
		}
		
again:		
		TAILQ_FOREACH(mCast_t, &DrvTbl.inuseList.mCastTbl[entry], nextMCast) {
			if (mcast_addr && mCast_t->dip != mcast_addr)
				continue;
			if (vlanMask) {

				assert(SINGLE_BITMASK_ISSET(vlanMask));

				if ((mCast_t->vlanMask & vlanMask) == 0) {
					/* for this flow, it don't include this vlanMask */
					continue;
				}

				{
					uint32 other_vlanMask;

					other_vlanMask = mCast_t->vlanMask & ~vlanMask;

					/* other vlans are included in this multicast entry */
					if (other_vlanMask)
					{
						rtl8651_tblDrv_vlanTable_t *	 vptr;
						uint32 idx;

						vptr = DrvTbl.vlan;
						for (idx = 0 ; idx < RTL8651_VLAN_NUMBER ; idx ++, vptr ++) {
							/* check if the target "mbr" is belong to other destination vlan, too */
							if ((1<<idx) & other_vlanMask) {
								/* member port confuse! so we MUST remove this entry and re-generate it in forwarding engine slow-path */
								if ((mbr & mCast_t->mbr) & vptr->memberPortMask)
									goto remove_entry;
							}
							/* check if the target "vlanMask" need to be removed from Multicast flow entry */
							if ((1<<idx) & vlanMask) {
								if (((mCast_t->mbr & ~mbr) & vptr->memberPortMask) == 0) {
									mCast_t->vlanMask &= ~vlanMask;
								}
							}
						}
					}

remove_mbr:
					mCast_t->mbr &= ~mbr;
					if (mCast_t->mbr == 0) {
remove_entry:
						TAILQ_REMOVE(&DrvTbl.inuseList.mCastTbl[entry], mCast_t, nextMCast);
						TAILQ_INSERT_HEAD(&DrvTbl.freeList.freeMultiCast, mCast_t, nextMCast);
						_rtl8651_arrangeMulticast(entry);
						goto again;
					}
				}

			}else
			{	/* only remove mbr from mCast_t */
				goto remove_mbr;
			}
			_rtl8651_patchPppoeWeak(mCast_t);
		}
		_rtl8651_arrangeMulticast(entry);
	}

	return SUCCESS;
}


rtl8651_tblDrv_mCast_t *_rtl8651_findMCast(ipaddr_t mAddr, ipaddr_t sip, uint16 svid, uint16 sport)
{
	rtl8651_tblDrv_mCast_t *mCast_t;
	uint32 entry = rtl8651_ipMulticastTableIndex(sip, mAddr);

	TAILQ_FOREACH(mCast_t, &DrvTbl.inuseList.mCastTbl[entry], nextMCast) {
		if (mCast_t->dip==mAddr && mCast_t->sip==sip && mCast_t->svid==svid && mCast_t->port==sport)
		{
#ifndef RTL865X_TEST
			if (mCast_t->inAsic == FALSE) {
#endif /* RTL865X_TEST */
				mCast_t->age = RTL865X_MULTICAST_TABLE_AGE;
				mCast_t->count ++;
				if (mCast_t->count > RTL865X_MULTICAST_SWAP_THRESHOLD) {
					// swap
					if ((mCast_t->cpu == 0) && !(mCast_t->flag & RTL865X_MULTICAST_PPPOEPATCH_CPUBIT)) {
						_rtl8651_arrangeMulticast(entry);
					}
				}
#ifndef RTL865X_TEST
			}
#endif /* RTL865X_TEST */
			return mCast_t;
		}
	}
	return (rtl8651_tblDrv_mCast_t *)NULL;	
}

int32 _rtl8651_delMulticastUpStream(ipaddr_t mcast_addr, ipaddr_t sip, uint16 svid, uint16 sport)
{

	rtl8651_tblDrv_mCast_t *mCast_t;
	uint32 entry;
	for(entry=0; entry<rtl8651_tblDrvPara.mCastTblSize; entry++) {
		TAILQ_FOREACH(mCast_t, &DrvTbl.inuseList.mCastTbl[entry], nextMCast) {
			if ((!mcast_addr || mCast_t->dip == mcast_addr) && (!sip || mCast_t->sip==sip) && (!svid || mCast_t->svid==svid) && mCast_t->port==sport) {
				TAILQ_REMOVE(&DrvTbl.inuseList.mCastTbl[entry], mCast_t, nextMCast);
				TAILQ_INSERT_HEAD(&DrvTbl.freeList.freeMultiCast, mCast_t, nextMCast);
				_rtl8651_arrangeMulticast(entry);
				return SUCCESS;
			}
		}
	}
	return FAILED;
}


int32 _rtl8651_setMGroupAttribute(ipaddr_t groupIp, int8 toCpu)
{
	rtl8651_tblDrv_mCast_t *mCast_t;
	uint32 entry;

	for(entry=0; entry<rtl8651_tblDrvPara.mCastTblSize; entry++) {
		TAILQ_FOREACH(mCast_t, &DrvTbl.inuseList.mCastTbl[entry], nextMCast) {
			if (mCast_t->dip == groupIp)
				mCast_t->cpu = (toCpu==TRUE? 1: 0);
		}
		_rtl8651_arrangeMulticast(entry);
	}
	return SUCCESS;
}

int32 _rtl8651_delMulticastByVid(uint32 vid)
{
	rtl8651_tblDrv_vlanTable_t *	svlan = &DrvTbl.vlan[rtl8651_vlanTableIndex(vid)];
	uint32 sportMask = svlan->memberPortMask;
	uint16 sport;

	/* delete all upstream related to vid */
	sport = 0;
	while (sportMask) {
		if (sportMask & 1)
			_rtl8651_delMulticastUpStream(0, 0, vid, sport);
		sportMask = sportMask >> 1;
		sport ++;
	}
	/* delete all downstream related to vid. FIXME: we assume there are no VLAN overlap in single port */
	_rtl8651_delMulticast(0, (1<<(vid & (RTL8651_VLAN_NUMBER-1))), svlan->memberPortMask);

	return FAILED;
}

int32 _rtl8651_delMulticastByPort(uint32 port)
{
	uint32 portMask = 1<<port;
	uint32 vlanMask = 0;

	/* delete all upstream related to this port */
	_rtl8651_delMulticastUpStream(0, 0, 0, port);

	/* determine how many vlan has this member port */
	{
		uint8 i;
		rtl8651_tblDrv_vlanTable_t *vlanPtr;
		for(i=0,vlanPtr=&DrvTbl.vlan[0]; i<RTL8651_VLAN_NUMBER; i++, vlanPtr++) {
			if (vlanPtr->memberPortMask == portMask)
				vlanMask |= 1<<i;
		}
	}

	/* delete all downstream related to vid */
	{
		rtl8651_tblDrv_mCast_t *mCast_t;
		uint32 entry;

		for(entry=0; entry<rtl8651_tblDrvPara.mCastTblSize; entry++) {		
	again:		
			TAILQ_FOREACH(mCast_t, &DrvTbl.inuseList.mCastTbl[entry], nextMCast) {
				if (!(mCast_t->mbr & portMask))
					continue;
				{	/* check ambiguous, if this port is belong to 2 of vlans in mCast_t's vlan mask, remove it  */
					uint32 mask = mCast_t->vlanMask & vlanMask;
					if (mask) {
						uint32 tmp = (~mask) + 1;
						if (mask != (mask & tmp))
							goto remove;	/* there are least 2 vlans in mCast's vlanMask has this member port */
					}
				}
				mCast_t->mbr 	&= ~portMask;
				mCast_t->vlanMask&= ~vlanMask;
				if ((mCast_t->mbr == 0) || (mCast_t->vlanMask == 0)) {
	remove:
					TAILQ_REMOVE(&DrvTbl.inuseList.mCastTbl[entry], mCast_t, nextMCast);
					TAILQ_INSERT_HEAD(&DrvTbl.freeList.freeMultiCast, mCast_t, nextMCast);
					_rtl8651_arrangeMulticast(entry);
					goto again;
				}
				_rtl8651_patchPppoeWeak(mCast_t);
			}
			_rtl8651_arrangeMulticast(entry);
		}	
	}

	return FAILED;
}


/*=======================================
  * MAC Access Control
  *=======================================*/
#define RTL865X_MAC_ACCESS_CONTROL_APIs

int32 _rtl8651_isMacAccessControlPermit(ether_addr_t *sMac)
{
	rtl8651_tblDrv_macACEntry_t *aclRule;
	ether_addr_t mac;
	int32 index;

	/* If this function is disabled, always return TRUE */
	if (DrvTbl.macACInfo->enable == FALSE)
		return TRUE;
		
	CTAILQ_FOREACH(aclRule, &DrvTbl.macACInfo->macAccessControl, nextMAC) {
		for(index=0; index<6; index++)
			mac.octet[index] = (sMac->octet[index] & aclRule->macMask.octet[index]);
		if (memcmp(&mac, &aclRule->mac, 6) == 0) {
			if (aclRule->actionType == RTL8651_ACL_PERMIT)
				return TRUE;
			else return FALSE;
		}
	}
	/* Check default action */
	return (int32)DrvTbl.macACInfo->def_permit;
}


void _rtl8651_arrangeNATbyMacAccessControl(void)
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t = DrvTbl.netIface;
	rtl8651_tblDrv_natEntry_t *nat_t;
	rtl8651_tblDrv_arpEntry_t *arp_t;
	uint32 entry;
	
	for(entry=0; entry<rtl8651_tblDrvPara.networkIntfTableSize; entry++, netif_t++) {
		if (netif_t->valid == 0)
			continue;
		SLIST_FOREACH(nat_t, &netif_t->nextNat, nextNatEntry) {
			if ((arp_t=_rtl8651_getArpEntry(NULL, nat_t->localIp, FALSE)) &&
			     _rtl8651_isMacAccessControlPermit(&arp_t->macInfo->macAddr)==FALSE )
				_rtl8651_delIpEntry(IP_ATTR_NAT, nat_t->globalIp, nat_t->localIp);
			else _rtl8651_addIpEntry(nat_t->globalIp, nat_t->localIp, IP_ATTR_NAT, netif_t);
		}
	}
}


void _rtl8651_arrangeSPbyMacAccessControl(void)
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t = DrvTbl.netIface;
	rtl8651_tblDrv_serverPortAsicArrangementEntry_t *asic_t;
	rtl8651_tblDrv_naptServerPortEntry_t *sp_t;
	rtl8651_tblDrv_arpEntry_t *arp_t;
	uint32 entry, index;

	for(entry=0; entry<rtl8651_tblDrvPara.networkIntfTableSize; entry++, netif_t++) {
		if (netif_t->valid == 0)
			continue;
		SLIST_FOREACH(sp_t, &netif_t->nextNaptServerPort, nextServerPort) {

			arp_t=_rtl8651_getArpEntry(NULL, sp_t->localIp, FALSE);

			if ( (arp_t) && (_rtl8651_isMacAccessControlPermit(&arp_t->macInfo->macAddr)==FALSE ))
			{
				if (sp_t->toAsic == 1) {
					AsicTableUsage.serverPort[sp_t->asicTablePos].valid = 0;
					rtl8651_delAsicServerPortTable(sp_t->asicTablePos);
					sp_t->toAsic = 0;
				}
			}
			else {
				if ( sp_t->toAsic == 0 &&
				     sp_t->localPort==sp_t->endLocalPort &&
#ifdef SERVERPORT_PARK_TCPUDP
				     sp_t->tcp &&
				     sp_t->udp
#else/*SERVERPORT_PARK_TCPUDP*/
				     1
#endif/*SERVERPORT_PARK_TCPUDP*/
				     )
				{
					asic_t=&AsicTableUsage.serverPort[0];
					for(index=0; index<RTL8651_SERVERPORTTBL_SIZE; index++,asic_t++) {
						if(asic_t->valid == 0) {
							sp_t->toAsic		= 1;
							sp_t->asicTablePos	= index;
							asic_t->valid 		= 1;
							asic_t->servPort_t	= sp_t;
							convert_setAsicServerPortTable(index, sp_t->globalIp, sp_t->globalPort, sp_t->localIp, sp_t->localPort);
							break;
						}
					}
				}
			}
		}
	}
}

/*
@func int32 | rtl8651_addMacAccessControl | Add a MAC Filtering rule.
@parm ether_addr_t * | sMac | Pointer to a MAC Address.
@parm ether_addr_t * | macMask | Pointer to MAC Address Mask.
@parm int8 | isPermit | TRUE: if match the rule, permit the packet, FALSE: if match the rule, drop the packet.
@rvalue TBLDRV_ENOFREEBUFFER | Out of memory.
@rvallue TBLDRV_EINVALIDINPUT | Both paramters <p sMAc> and <p macMask> can not be NULL.
@rvalue SUCCESS | Success API call.
@comm
Add a new MAC Filtering rule. Whether a  rule match or not depends on the following rule:
         (pkt.srcMac & macMask) == sMac
Note that this API only provides source MAC address filtering.
*/
int32 rtl8651_addMacAccessControl(ether_addr_t *sMac, ether_addr_t *macMask, int8 isPermit)
{
	rtl8651_tblDrv_macACEntry_t *aclRule;

	int32 retval = SUCCESS, index;
	
	if (sMac == NULL || macMask == NULL)
		return TBLDRV_EINVALIDINPUT;
	rtlglue_drvMutexLock();
	if ((aclRule=CTAILQ_FIRST(&DrvTbl.freeList.freeMacAccessControl)) == NULL) {
		retval = TBLDRV_ENOFREEBUFFER;
		goto out;
	}
	bzero(aclRule, (uint32)&(((rtl8651_tblDrv_macACEntry_t *)0)->nextMAC) );
	aclRule->actionType	= (isPermit==TRUE)? RTL8651_ACL_PERMIT: RTL8651_ACL_DROP;
	aclRule->macMask		= *macMask;
	for(index=0; index<6; index++)
		aclRule->mac.octet[index] = (sMac->octet[index] & macMask->octet[index]);
	CTAILQ_REMOVE(&DrvTbl.freeList.freeMacAccessControl, aclRule, nextMAC);
	CTAILQ_INSERT_HEAD(&DrvTbl.macACInfo->macAccessControl, aclRule, nextMAC);
	_rtl8651_arrangeNATbyMacAccessControl();
	_rtl8651_arrangeSPbyMacAccessControl();
	
out:
	rtlglue_drvMutexUnlock();
	return retval;
}


/*
@func int32 | rtl8651_delMacAccessControl | Remove a MAC Filtering rule.
@parm ether_addr_t * | Pointer to a MAC Address.
@parm ether_addr_t * | Pointer to a MAC Address Mask.
@rvalue SUCCESS | Success API call.
@rvalue TBLDRV_EENTRYNOTFOUND | The specified entry was not found.
@comm
The usage of <p sMac> and <p macMask> can be seen in the description of rtl8651_addMacAccessControl API.
*/

int32 rtl8651_delMacAccessControl(ether_addr_t *sMac, ether_addr_t *macMask)
{
	rtl8651_tblDrv_macACEntry_t *aclRule;

	int32 retval = TBLDRV_EENTRYNOTFOUND, index;
	ether_addr_t mac;

	if (sMac == NULL || macMask == NULL)
		return TBLDRV_EINVALIDINPUT;
	rtlglue_drvMutexLock();
	CTAILQ_FOREACH(aclRule, &DrvTbl.macACInfo->macAccessControl, nextMAC) {
		for(index=0; index<6; index++)
			mac.octet[index] = sMac->octet[index] & macMask->octet[index];
		if (!memcmp(&mac, &aclRule->mac, 6) && !memcmp(macMask, &aclRule->macMask, 6) ) {
			CTAILQ_REMOVE(&DrvTbl.macACInfo->macAccessControl, aclRule, nextMAC);
			CTAILQ_INSERT_HEAD(&DrvTbl.freeList.freeMacAccessControl, aclRule, nextMAC);
			_rtl8651_arrangeNATbyMacAccessControl();
			_rtl8651_arrangeSPbyMacAccessControl();
			retval = SUCCESS;
			break;
		}
	}
	rtlglue_drvMutexUnlock();
	return retval;
}


/*
@func int32 | rtl8651_flushMacAccessControl | Flush all MAC Filtering rule.
@rvalue SUCCESS | Success API call.
@comm
This API will flush all MAC Filtering rules.
*/
int32 rtl8651_flushMacAccessControl(void)
{
	rtl8651_tblDrv_macACEntry_t *aclRule;

	
	rtlglue_drvMutexLock();
	while ((aclRule=CTAILQ_FIRST(&DrvTbl.macACInfo->macAccessControl))) {
		CTAILQ_REMOVE(&DrvTbl.macACInfo->macAccessControl, aclRule, nextMAC);
		CTAILQ_INSERT_HEAD(&DrvTbl.freeList.freeMacAccessControl, aclRule, nextMAC);
	}
	_rtl8651_arrangeNATbyMacAccessControl();
	_rtl8651_arrangeSPbyMacAccessControl();
	rtlglue_drvMutexUnlock();
	return SUCCESS;
}


/*
@func int32 | rtl8651_setMacAccessControlDefaultPermit | Set Default MAC Filtering rule.
@parm int8 | isPermit | TRUE: default rule is permit, FALSE: default rule is drop.
@rvalue SUCCESS | Success API call.
@comm
If packet doesn't match any MAC Filtering rule, the action decision(permit or drop) depends on the defule rule setting.
*/
int32 rtl8651_setMacAccessControlDefaultPermit(int8 isPermit)
{
	rtlglue_drvMutexLock();
	DrvTbl.macACInfo->def_permit = (isPermit==TRUE)? TRUE: FALSE;
	_rtl8651_arrangeNATbyMacAccessControl();
	_rtl8651_arrangeSPbyMacAccessControl();
	rtlglue_drvMutexUnlock();
	return SUCCESS;
}


/*
@func int32 | rtl8651_enableMacAccessControl | Enable/Disable MAC Filtering.
@parm int8 | enable | TRUE: enable, FALSE: disable
@rvalue SUCCESS | Success API call.
@comm
If MAC Filtering is enabled, packets match MAC rules will be dropped/permitted.
*/
int32 rtl8651_enableMacAccessControl(int8 enable)
{
	rtlglue_drvMutexLock();
	DrvTbl.macACInfo->enable = (enable==TRUE)? TRUE: FALSE;
	_rtl8651_arrangeNATbyMacAccessControl();
	_rtl8651_arrangeSPbyMacAccessControl();
	rtlglue_drvMutexUnlock();
	return SUCCESS;
}




/*=======================================
  * Netwrok Interface MTU APIs
  *=======================================*/
#define RTL865X_MTU_APIs


int32 rtl8651_installMTUCallBackFunction(int32 (*p_handler)(struct rtl_pktHdr *))
{
	rtlglue_drvMutexLock();
	p_mtuHandler = p_handler;
	rtlglue_drvMutexUnlock();
	return SUCCESS;
}


/*
@func void	| rtl8651_setPppoeMtu	| Set the MTU value of PPPoE, Multiple PPPoE, Unnumbered PPPoE connection mode.
@parm uint32 | pppoeObjId | PPPoE object ID number.
@parm uint16 | mtu | MTU value 
@comm 
When PPPoE Dialed, Rome Driver can't get the MTU value from PPPoE daemon.
We must assign the MTU value by this function.
@xref  <p rtl8651_setMtu>
*/
void rtl8651_setPppoeMtu(uint32 pppObjId,uint16 mtu)
{	
	rtl8651_tblDrv_pppTable_t *ppp_t;
	rtl8651_tblDrv_networkIntfTable_t *netif_t, *net_t=NULL;
	rtl8651_tblDrv_vlanTable_t *vlan_t;
	uint16 minMtu, entry;

	rtlglue_drvMutexLock();

	pppoe_mss[pppObjId]=mtu-40;

	ppp_t = _rtl8651_getPppoeSessionByDsid(pppObjId);
	if (ppp_t) {
		/* set mtu of this session & net_if */
		ppp_t->mss = mtu - 40;
		assert(ppp_t->netif_t);
#if 0
		/* for new model : 1 network interface maps 1 session */
		ppp_t->netif_t->mtu = mtu;
#else
		/* for old model : 1 network interface maps multi-sessions, we choose minimum mtu to set netif's */
		if (ppp_t->netif_t->mtu > mtu)
			ppp_t->netif_t->mtu = mtu;
#endif
		/* get the minimal mtu */
		net_t = ppp_t->netif_t;
		minMtu = mtu;
		netif_t = DrvTbl.netIface;
		for(entry=0; entry<rtl8651_tblDrvPara.networkIntfTableSize; entry++, netif_t++) {
			if (netif_t->valid == 0 || netif_t->vid != ppp_t->vid)
				continue;
			if (netif_t->mtu < minMtu) {
				minMtu = netif_t->mtu;
				net_t = netif_t;
			}
		}
		vlan_t = &DrvTbl.vlan[rtl8651_vlanTableIndex(ppp_t->vid)];
		if ((net_t->linkLayerType == RTL8651_LL_PPPOE)
			|| (net_t->linkLayerType == RTL8651_LL_LIPUNNUMBERED && vlan_t->internal == FALSE))
	    		vlan_t->mtu = minMtu + 8;
		else vlan_t->mtu = minMtu;

		RTL8651_SETASICVLAN(vlan_t);
	}

	rtlglue_drvMutexUnlock();
}


/*
@func void		| rtl8651_setMtu	| Set the MTU value of Static, DHCPC connection mode.
@parm uint16 | mtu | MTU value 
@comm 
By default, the MTU value is 1500, if we want to set to another vaule, 
we must assign the MTU value by this function. This API will modify whole system's MTU value to the user specified value.
@xref  <p rtl8651_setPppoeMtu>
*/

void rtl8651_setMtu(uint16 mtu)
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_vlanTable_t *vlan_t;
	int32 entry;

	rtlglue_drvMutexLock();
	rtl8651_mss=mtu-40;
	netif_t = DrvTbl.netIface;
	/* set all network interface's MTU to the user specified value */
	for(entry=0; entry<rtl8651_tblDrvPara.networkIntfTableSize; entry++, netif_t++) {
		if (netif_t->valid == 0)
			continue;
		vlan_t = &DrvTbl.vlan[rtl8651_vlanTableIndex(netif_t->vid)];
		netif_t->mtu = mtu;
		vlan_t->mtu = mtu;
		RTL8651_SETASICVLAN(vlan_t);
	}
	rtlglue_drvMutexUnlock();
}

/*
@func void		| rtl8651_getMtu	| Get the MTU value of Static, DHCPC connection mode.
@parm uint16 * | mtu | Point to the variable to store the MTU value 
@comm 
@xref  <p rtl8651_getPppoeMtu>
*/
void rtl8651_getMtu(uint16 * mtu)
{
	* mtu = rtl8651_mss + 40;
}


/* 
@func int32 | rtl8651_setNetMtu | Set a network interface's MTU.
@parm int8 * | ifName | Network interface Name.
@parm uint16 | mtu | MTU vlaue.
@comm
This API is used to set user specified network interface's MTU. If the network interface has more than one PPPoE attached,
then the network interface's all PPPoE MTU will be modified.
*/
int32 rtl8651_setNetMtu(int8 *ifName, uint16 mtu)
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t, *net_t;
	rtl8651_tblDrv_vlanTable_t *vlan_t;
	rtl8651_tblDrvSession_t *ppp_t;
	int32 retval=SUCCESS, entry;
	
	rtlglue_drvMutexLock();

#ifdef CONFIG_RTL865X_IPSEC
	/* Louis patch for IPSEC */
	if ( !strcmp( ifName, "ipsec0" ) )
	{
		rtl865x_ipsecMtu = mtu;
		goto out;
	}
#endif
	
	if ((netif_t=GET_NETWORK_INTERFACE(ifName, NULL)) == NULL) {
		retval = TBLDRV_EINVNETIFNAME;
		goto out;
	}
	netif_t->mtu = mtu;
	vlan_t = &DrvTbl.vlan[rtl8651_vlanTableIndex(netif_t->vid)];

	/* process for PPPOE */
	if (netif_t->linkLayerType != RTL8651_LL_VLAN) {
		CTAILQ_FOREACH(ppp_t, &netif_t->sessionHead, nextSession) {
			if (ppp_t->session_t) {
				ppp_t->session_t->mss = mtu - 40;
				pppoe_mss[_rtl8651_SessionIDToDSID(ppp_t->session_t->sessionId)] = mtu -40;
			}
		}
	}

	//only for old model: rtl8651_mss in old model is seemed to be wan's mss, hence is the specified 
	//network interface's vlan is an external vlan, we set rtl8651_mss.
	if (vlan_t->internal == 0)
		rtl8651_mss=mtu-40;

	net_t = netif_t;
	/* get the minimal mtu */
	netif_t = DrvTbl.netIface;
	for(entry=0; entry<rtl8651_tblDrvPara.networkIntfTableSize; entry++, netif_t++) {
		if (netif_t->valid == 0 || netif_t->vid != vlan_t->vid)
			continue;
		if (netif_t->mtu < mtu)
			mtu = netif_t->mtu;
	}
	if ((net_t->linkLayerType == RTL8651_LL_PPPOE)
		|| (net_t->linkLayerType == RTL8651_LL_LIPUNNUMBERED && vlan_t->internal == FALSE))
    		vlan_t->mtu = mtu + 8;
	else vlan_t->mtu = mtu;
	RTL8651_SETASICVLAN(vlan_t);
	
out:
	rtlglue_drvMutexUnlock();
	return retval;
}


/*
@func int32 | rtl8651_getNetMtu | Get a network interface's MTU.
@parm int8 * | ifName | Network interface name.
@parm uint16 * | mtu | returned MTU value.
@rvalue SUCCESS | TBLDRV_EINVALIDINPUT | Invalid Input Paramter.
@rvalue TBLDRV_EINVNETIFNAME | Invalid network interface name.
@rvalue SUCCESS | Success.
*/

int32 rtl8651_getNetMtu(int8 *ifName, uint16 *mtu)
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	int32 retval = SUCCESS;
	
	if (!ifName || !mtu)
		return TBLDRV_EINVALIDINPUT;
	rtlglue_drvMutexLock();
	if ((netif_t=GET_NETWORK_INTERFACE(ifName, NULL)) == NULL) {
		retval = TBLDRV_EINVNETIFNAME;
		goto out;
	}
	*mtu  = netif_t->mtu;
out:
	rtlglue_drvMutexUnlock();
	return retval;
}





/*=======================================
  * Rate Limit  APIs
  *=======================================*/
#define RTL865X_RATE_LIMIT_APIs

/* Note: MN-queue mechanism should be transparent to this compile flag.
*
*  This compile flag is defined to refill rate limit table token by software. 
*  It is only applied to RTL865xB b-cut IC.
*  That is, when you use RTL865xB c/d-cut IC, even you define this compile flag,
*  rate limit table token is still refilled by hardware.
*/

#ifdef CONFIG_RTL865XB_BCUT_RLREFILL_BYSW
void _rtl8651_rateLimitTimer(void)
{
	/* Refilling token by software only applies to RTL865xB b-cut and before */
	if (RtkHomeGatewayChipName[strlen(RtkHomeGatewayChipName) - 1] == 'B' && RtkHomeGatewayChipRevisionID < 0x02)
	{
		rtl865x_tblAsicDrv_rateLimitParam_t asic_rl;
		rtl8651_tblDrv_rateLimit_t *rl_t;
		uint32 entry;
				
		rl_t = DrvTbl.rateLimit;
		for (entry = 0; entry < RTL8651_RATELIMITTBL_SIZE; entry++, rl_t++)
		{
			/* User has configured this rate limit entry */
			if (rl_t->queueType == 0 /* not MN-queue */ && rl_t->groupId >= RATELIMIT_GROUP_INTERNALRESV /* not internal use */)
			{				
				bzero(&asic_rl, sizeof(rtl865x_tblAsicDrv_rateLimitParam_t));
				if (rtl8651_getAsicRateLimitTable(entry, &asic_rl) == SUCCESS)	/* This ASIC entry is valid */
				{					
					/* Software will refill tokens every 10ms. */
					asic_rl.token += (rl_t->rate / 100);	/* Precision Issue here */
					if (asic_rl.token > rl_t->burst)
						asic_rl.token = rl_t->burst;	
				}
				else		/* This ASIC entry is in-valid. Should not happen!!! => However, it does happen in b-cut few times. Does not happen in d-cut. */
				{
					bzero(&asic_rl, sizeof(rtl865x_tblAsicDrv_rateLimitParam_t));
					asic_rl.token			= (rl_t->rate / 100);
					if (asic_rl.token > rl_t->burst)
						asic_rl.token = rl_t->burst;					
				}
				
				/* update rate limit entry */
				asic_rl.maxToken		= rl_t->burst;
				asic_rl.refill_number	= 0;
				asic_rl.t_intervalUnit	= 0x3f; 
				asic_rl.t_remainUnit	= 0x3f;

				rtl8651_setAsicRateLimitTable(entry, &asic_rl);
			}
		}/* End of ASIC rate limit table traverse. */
	}
}
#endif

static int32 _rtl8651_delRateLimitGroup(uint32 GroupId)
{
	rtl8651_tblDrv_rateLimit_t *rl_t;
	int32  entry;

	assert(GroupId);
	/* Find the specified entry */
	for(entry=0, rl_t=DrvTbl.rateLimit; entry<RTL8651_RATELIMITTBL_SIZE; entry++, rl_t++)  {
		if (rl_t->groupId == GroupId) {
			if (rl_t->ref_count > 0) { /* The entry should not be referenced */
				return TBLDRV_ERLENTRYISREFERENCE;
			}
			rl_t->groupId = 0;  /* remove the entry */
			rtl8651_delAsicRateLimitTable(RATELIMIT_TABLE_INDEX(rl_t));
			return SUCCESS;
		}
	}
	return TBLDRV_EENTRYNOTFOUND;
}


static int32 _rtl8651_delRateLimitRule(rtl8651_tblDrvAclRule_t *rule, uint32 GroupId)
{
	_rtl8651_tblDrvAclRule_t *rl_rule;
	rtl8651_tblDrv_rateLimit_t *rl_t;
	CTAILQ_HEAD(_aclStruct, _rtl8651_tblDrvAclRule_s);
	int32 entry;

	/* Find out the specified rate limit group ID */
	rl_t = DrvTbl.rateLimit;
	for(entry=0; entry<RTL8651_RATELIMITTBL_SIZE; entry++, rl_t++) 
		{
	
		if (rl_t->groupId == GroupId)
			break;
		}
	if (entry == RTL8651_RATELIMITTBL_SIZE) 
		return TBLDRV_ENOGROUPIDFOUND;
	if (rule->actionType_ == RTL8651_ACL_DROP) {
		if (rl_t->isByteCount == TRUE)
			rule->actionType_ = RTL8651_ACL_DROP_RATE_EXCEED_BPS;
		else rule->actionType_ = RTL8651_ACL_DROP_RATE_EXCEED_PPS;
	}
	else {
		if (rl_t->isByteCount == TRUE)
			rule->actionType_ = RTL8651_ACL_LOG_RATE_EXCEED_BPS;
		else rule->actionType_ = RTL8651_ACL_LOG_RATE_EXCEED_PPS;
	}

	if (rule->isEgressRateLimitRule_)
	{
		/* Find the specified rule */
		CTAILQ_FOREACH_REVERSE(rl_rule, &rl_t->netif_t->outRateLimitHead, _aclStruct, nextRule) {

			if (_rtl8651_sameAclRule(rl_rule, rule) == TRUE) {
				rl_t->ref_count --;
				CTAILQ_REMOVE(&rl_t->netif_t->outRateLimitHead, rl_rule, nextRule);
				CTAILQ_INSERT_HEAD(&DrvTbl.freeList.aclRule, rl_rule, nextRule);
				_rtl8651_arrangeAllACL();
				return SUCCESS;
			}
		}
	}
	else
	{
        	/* Find the specified rule */
	        CTAILQ_FOREACH_REVERSE(rl_rule, &rl_t->netif_t->inRateLimitHead, _aclStruct, nextRule) {
	
        	        if (_rtl8651_sameAclRule(rl_rule, rule) == TRUE) {
				rl_t->ref_count --;
				CTAILQ_REMOVE(&rl_t->netif_t->inRateLimitHead, rl_rule, nextRule);
				CTAILQ_INSERT_HEAD(&DrvTbl.freeList.aclRule, rl_rule, nextRule);
				_rtl8651_arrangeAllACL();
				return SUCCESS;
	                }
        	}
	}
	return TBLDRV_EENTRYNOTFOUND;
}





//static int32 _rtl8651_addRateLimitGroup(uint32 GroupId, int8 *ifName, uint32 rate, uint32 burst, int8 isByteCount, int32 ratio, uint32 *index)
static int32 _rtl8651_addRateLimitGroup(uint32 *GroupId, int8 *ifName, uint32 rate, uint32 burst, int8 isByteCount, int32 ratio, int8 queueType)
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_rateLimit_t *rl_t, *freeEntry_t;
	rtl865x_tblAsicDrv_rateLimitParam_t asic_rl;
	int32 entry;
	
//	printk("AddRateLimitGroup: <id, rate, burst, ratio> = <%u, %u, %u, %u>\n", *GroupId, rate, burst, ratio);

	if ((netif_t=GET_NETWORK_INTERFACE(ifName, NULL)) == NULL) 
		return TBLDRV_EINVNETIFNAME;

	/* 
	  * <ghhuang> 
	  * First check whether the specified index already maps to an existing queue.
	  * 	If it is, do nothing and return.
	  *	If not, set the specified rate limit entry.
	  */

	rl_t = DrvTbl.rateLimit;
	freeEntry_t = NULL;
	for(entry=0; entry<RTL8651_RATELIMITTBL_SIZE; entry++, rl_t++) {
		if (freeEntry_t == NULL && rl_t->groupId == 0) { /* find an empty entry */
			freeEntry_t = rl_t;
			break;
		}
		if (rl_t->groupId == *GroupId)  { /* check duplicate entry */
//		if (!ratio && rl_t->groupId == *GroupId)  { /* check duplicate entry */
//			printk("1: Index %u  <=>  Entry %u [%u]\n", *GroupId, entry, rl_t->groupId);			
			*GroupId = entry;
			return SUCCESS;
		}
//			return TBLDRV_EDUPGROUPID;
	}
	if (freeEntry_t == NULL) 
		return TBLDRV_ENOFREEBUFFER;

	/* For RTL865xB c-cut and after that */
	if (RtkHomeGatewayChipName[strlen(RtkHomeGatewayChipName) - 1] == 'B' && RtkHomeGatewayChipRevisionID >= 0x02)
	{
		/*
		* <ghhuang>
		* In 8650B Cut-C, differentiated granularity is scaled by 1/10000.
		* Therefore, value 1 means 1/10000 (sec)
		* Confirmed by <jblin>
		*
	 	* The interval is set to around 1 ms. (2004/1/25)
	 	*/
		freeEntry_t->queueType	= queueType;
		freeEntry_t->groupId 		= *GroupId;

		if (ratio == 0)
		{
			/* Find a vaue within 1~7 that leads to higher precision */
			int i, val = 1, max = 0, remain;
			for (i = 1; i <= 7; i++)
			{
				remain = (rate*i) % 10000;
				remain = (remain > 5000) ? (remain - 5000) : (5000 - remain); /* center on 10000 / 2 */

				if (remain > max)
				{
					max = remain;
					val = i;
				}
			}

			freeEntry_t->addToken = ((rate*val)%10000) > 5000? (rate*val)/10000+1 : (rate*val)/10000;
			freeEntry_t->interval	= val;
		}
		else
		{
			freeEntry_t->addToken = 0;
			freeEntry_t->interval	= 0x3f;
		}

		freeEntry_t->maxToken	= (ratio == 0) ? (rate < 15360 ? 15360: rate) : 0;
		freeEntry_t->isByteCount	= isByteCount;
		freeEntry_t->netif_t		= netif_t;
		freeEntry_t->ref_count		= 0;
		freeEntry_t->ratio			= ratio;
	}
	/* For RTL865xB b-cut and after that */
	else
	{
		#ifdef CONFIG_RTL865XB_BCUT_RLREFILL_BYSW
			/* This code segment should not affect "MN-queue" and "rate limit counter" mechanisms. */
			if (*GroupId >= RATELIMIT_GROUP_INTERNALRESV)
			{
				freeEntry_t->rate		= rate;	/* Record the user-config rate constraint. */
				freeEntry_t->burst		= burst;
				freeEntry_t->addToken		= 0;
				freeEntry_t->interval		= 0x3f;
				freeEntry_t->maxToken		= (ratio == 0)? burst: 0;
			}
			else	/* internal use, currently for rate limit counter */
			{
				freeEntry_t->addToken   = (rate * 5) / 3;
				freeEntry_t->interval   = 1;
				freeEntry_t->maxToken   = (burst * 5) / 3;
			}
			freeEntry_t->queueType		= queueType;
			freeEntry_t->groupId		= *GroupId;
			freeEntry_t->isByteCount	= isByteCount;
			freeEntry_t->netif_t		= netif_t;
			freeEntry_t->ref_count		= 0;
			freeEntry_t->ratio		= ratio;
		#else
			/* 
		  	* Designer said: The time unit used to achieve rate limit is 1.67s (5/3), hence here we change 
		  	* the time unit to 1 sec.
		  	* If parameter ratio != 0: this rate limit entry is used  to M-N Queue. In this case, we set the
		  	* refill time to its maximal value to disable ASIC refilling mechanism. Instead, refilling token is taken
		  	* over by pure software.
		  	*/
			freeEntry_t->queueType	= queueType;
			freeEntry_t->groupId 		= *GroupId;
			freeEntry_t->addToken		= (ratio == 0) ? ((rate * 5) / 3) : 0 ;
			freeEntry_t->interval		= (ratio == 0) ? 1: 0x3f; /* one second */
			freeEntry_t->maxToken	= (ratio == 0) ? ((burst * 5) / 3) : 0 ;
			freeEntry_t->isByteCount	= isByteCount;
			freeEntry_t->netif_t		= netif_t;
			freeEntry_t->ref_count		= 0;
			freeEntry_t->ratio			= ratio;
		#endif
	}

	/*Set ASIC entry */
	bzero(&asic_rl, sizeof(rtl865x_tblAsicDrv_rateLimitParam_t));
	asic_rl.maxToken			= freeEntry_t->maxToken;
	asic_rl.refill_number		= freeEntry_t->addToken;
	asic_rl.t_intervalUnit		= freeEntry_t->interval; 
	asic_rl.t_remainUnit		= freeEntry_t->interval;
	asic_rl.token				= freeEntry_t->addToken;
	rtl8651_setAsicRateLimitTable(RATELIMIT_TABLE_INDEX(freeEntry_t), &asic_rl);
//	if (index) *index = RATELIMIT_TABLE_INDEX(freeEntry_t);

	/* Pass out real entry number */
	*GroupId = entry;

	return SUCCESS;
}

int32 _rtl8651_addRateLimitRule(rtl8651_tblDrvAclRule_t *rule, uint32 GroupId, uint8 isCounter)
{
	rtl8651_tblDrv_rateLimit_t *rl_t;
	_rtl8651_tblDrvAclRule_t *rl_rule;
	int32 retval=SUCCESS, entry;

	/* Note: this API ignores actionType_.  The real actionType_ will depend on rate limit entry */
	
	if ( rule == NULL ||  !GroupId || 
	    (rule->actionType_!=RTL8651_ACL_DROP && rule->actionType_!=RTL8651_ACL_DROP_LOG)) 
		return TBLDRV_EINVALIDINPUT;
	
	rl_t = DrvTbl.rateLimit;
	for(entry=0; entry<RTL8651_RATELIMITTBL_SIZE; entry++, rl_t++) 
		if (rl_t->groupId == GroupId)
			break;
	if (entry == RTL8651_RATELIMITTBL_SIZE) {
		retval = TBLDRV_ENOGROUPIDFOUND;
		goto out;
	}
	if ((rl_rule=CTAILQ_FIRST(&DrvTbl.freeList.aclRule)) == NULL) {
		retval = TBLDRV_ENOFREEBUFFER;
		goto out;
	}
	CTAILQ_REMOVE(&DrvTbl.freeList.aclRule, rl_rule, nextRule);
	bzero(rl_rule, sizeof(_rtl8651_tblDrvAclRule_t));
	rtl8651_memcpy(rl_rule, rule, sizeof(rtl8651_tblDrvAclRule_t));

	/* Judge whether this rule is ingress rate limit rule or egress. */
	if (rule->isEgressRateLimitRule_)
	{
		CTAILQ_INSERT_TAIL(&rl_t->netif_t->outRateLimitHead, rl_rule, nextRule);
		rl_rule->isRateLimitCounter_ = (isCounter == TRUE)? 1: 0;
	}
	else 
	{
		CTAILQ_INSERT_TAIL(&rl_t->netif_t->inRateLimitHead, rl_rule, nextRule);
	}

	rl_rule->rlIndex = RATELIMIT_TABLE_INDEX(rl_t);

	/* 
	  * Disable the following rule becaue of IC's bug:
	  * pktOptApp field of ACL rule can not work when applying to IP fragment packet. Hence
	  * always set to 0.
	  */
	/* rl_rule->pktOpApp = 0x06; */ /* apply this rule to only L3 and L4 */
	if (rule->actionType_ == RTL8651_ACL_DROP) {
		if (rl_t->isByteCount == TRUE)
			rl_rule->actionType_ = RTL8651_ACL_DROP_RATE_EXCEED_BPS;
		else rl_rule->actionType_ = RTL8651_ACL_DROP_RATE_EXCEED_PPS;
	}
	else {
		if (rl_t->isByteCount == TRUE)
			rl_rule->actionType_ = RTL8651_ACL_LOG_RATE_EXCEED_BPS;
		else rl_rule->actionType_ = RTL8651_ACL_LOG_RATE_EXCEED_PPS;
	}
	rl_t->ref_count ++;
	_rtl8651_arrangeAllACL();

out:
	return retval;	
}

int32 _rtl8651_flushRateLimitGroup(int8* ifName, uint8 flushCtrlGroup)
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	_rtl8651_tblDrvAclRule_t *rl_rule;
	uint32 GroupId;
	int32 retval=SUCCESS;
	
	if ((netif_t=GET_NETWORK_INTERFACE(ifName, NULL)) == NULL) {
		retval = TBLDRV_EINVNETIFNAME;
		goto out;
	}
	/* remove rules one by one */
	while ((rl_rule=CTAILQ_FIRST(&netif_t->inRateLimitHead))) {
		GroupId = DrvTbl.rateLimit[rl_rule->rlIndex].groupId;

		/*
			We don't flush internal used rate limit table entries (GroupId < RATELIMIT_GROUP_INTERNALRESV
			is for internal used) if flushCtrlGroup == 0.
		*/
		if (flushCtrlGroup == FALSE && GroupId < RATELIMIT_GROUP_INTERNALRESV)
			continue;

		_rtl8651_delRateLimitRule((rtl8651_tblDrvAclRule_t *)rl_rule, GroupId);
		_rtl8651_delRateLimitGroup(GroupId);
	}

        /* remove rules one by one */
        while ((rl_rule=CTAILQ_FIRST(&netif_t->outRateLimitHead))) {
                GroupId = DrvTbl.rateLimit[rl_rule->rlIndex].groupId;

                /*
                        We don't flush internal used rate limit table entries (GroupId < RATELIMIT_GROUP_INTERNALRESV
                        is for internal used) if flushCtrlGroup == 0.
                */
                if (flushCtrlGroup == FALSE && GroupId < RATELIMIT_GROUP_INTERNALRESV)
                        continue;

                _rtl8651_delRateLimitRule((rtl8651_tblDrvAclRule_t *)rl_rule, GroupId);
                _rtl8651_delRateLimitGroup(GroupId);
        }

out:
	return retval;;
}


int32 rtl8651_addRateLimitGroup(uint32 GroupId, int8 *ifName, uint32 rate, uint32 burst, int8 isByteCount)
{
	int32 retval;
	
	if (ifName == NULL || GroupId < RATELIMIT_GROUP_INTERNALRESV)
		return TBLDRV_EINVALIDINPUT;
	rtlglue_drvMutexLock();
	retval = _rtl8651_addRateLimitGroup(&GroupId, ifName, rate, burst, isByteCount, 0, 0);
	rtlglue_drvMutexUnlock();
	return retval;
}

int32 rtl8651_delRateLimitGroup(uint32 GroupId)
{
	int32 retval;
	if (GroupId == 0)
		return TBLDRV_EINVALIDINPUT;
	rtlglue_drvMutexLock();
	retval = _rtl8651_delRateLimitGroup(GroupId);
	rtlglue_drvMutexUnlock();
	return retval;	
}


int32 rtl8651_addRateLimitRule(rtl8651_tblDrvAclRule_t *rule, uint32 GroupId)
{
	int32 retval;

	rtlglue_drvMutexLock();
	retval = _rtl8651_addRateLimitRule(rule, GroupId, FALSE);
	rtlglue_drvMutexUnlock();

	return retval;	
}


int32 rtl8651_delRateLimitRule(rtl8651_tblDrvAclRule_t *rule, uint32 GroupId)
{
	int32 retval;
	if (rule == NULL || (rule->actionType_ != RTL8651_ACL_DROP && rule->actionType_!=RTL8651_ACL_DROP_LOG) || !GroupId) 
		return TBLDRV_EINVALIDINPUT;
	rtlglue_drvMutexLock();
	retval = _rtl8651_delRateLimitRule(rule, GroupId);
	rtlglue_drvMutexUnlock();
	return retval;	
}


int32 rtl8651_flushRateLimitGroup(int8* ifName) 
{
	int32 retval;
	rtlglue_drvMutexLock();
	retval = _rtl8651_flushRateLimitGroup(ifName, FALSE);	/* don't flush control group when user call this API */
	rtlglue_drvMutexUnlock();
	return retval;;
}






/*=======================================
  * M-N Queue  APIs
  *=======================================*/
#define RTL865X_M_N_QUEUE_APIs

#define RATE_TBL_SYNCHK	0
#define DQM	0

#define RTL865X_MNQUEUE_UPDATE_PERIOD	100
#define RTL865X_MNQUEUE_MIN_MAXTOKEN	15360
#define RTL865X_MNQUEUE_MIN_RATE			200 /* 200 * 8 = 1600-bit/sec */
#define RTL865X_MNQUEUE_GREATE_CHANGE_THRESHOLD	2
#define RTL865X_MNQUEUE_MAXPKTLEN		1518
#define RTL865X_MNQUEUE_EAGERBITS			7
#define RTL865X_MNQUEUE_EAGEREXPANDBITS		3

void printRateLine(char * dispString, uint32 rate) {
	if(rate > 125000)
		printk("\t%s is %3u.%03u Mbps\n", dispString, rate/125000, (rate%125000)/125);
	else if(rate > 125)
		printk("\t%s is %3u.%03u Kbps\n", dispString, rate/125, (rate%125));
	else
		printk("\t%s is     %3u  bps\n", dispString, rate);
}

void printRateString(uint32 rate) {
	if(rate > 125000)
		printk(" %3u.%03u Mbps ", rate/125000, (rate%125000)/125);
	else if(rate > 125)
		printk(" %3u.%03u Kbps ", rate/125, (rate%125));
	else
		printk("     %3u  bps ", rate);
}

void _rtl8651_mnQueueTimer(void)
{
	rtl865x_tblAsicDrv_rateLimitParam_t asic_rl, asic_rl1 = { 0, 0, 0x3F, 0x3F, 0};
	rtl8651_tblDrv_rateLimit_t *rl_t;
	uint32 entry; /* Used to track the rate limit table */
	/* The operation of timer is divided into two phases, the fast one is operated every time (10ms) timer function is called. Its mission is to 
		update the token based on previous decision. The slower one working frequency is expected to be once every second which do
		much complex decision and caluclate the way to update token every 10ms*/
	static uint32 updateState = 0; 
	
	uint32	mVRate,
			nAvailableRate, 
			nConsumedRate,
			nRatioSummary, 
			nConsumedRatio,
			nQueueNeedProcessedNumber;

 	if (mnQ_enable == FALSE)
 		return;

	/* (Fast update) Refill token with each.  */
	for(entry=0, rl_t = DrvTbl.rateLimit; entry<RTL8651_RATELIMITTBL_SIZE; entry++, rl_t++) {
		if (rl_t->queueType) { /* 1: M queue's Group ID, 2: N queue's Group ID, 3 default queue's Group ID (Best effort) */
			rtl8651_getAsicRateLimitTable(entry, &asic_rl);	/* read entry from ASIC */
			if(rl_t->lastConfigToken < RTL865X_MNQUEUE_MAXPKTLEN || asic_rl.token < RTL865X_MNQUEUE_MAXPKTLEN)
				rl_t->eagerCounter = 1<<(RTL865X_MNQUEUE_EAGERBITS+RTL865X_MNQUEUE_EAGEREXPANDBITS);
			else 
				rl_t->eagerCounter = ((rl_t->eagerCounter<<RTL865X_MNQUEUE_EAGERBITS) - rl_t->eagerCounter) >> RTL865X_MNQUEUE_EAGERBITS;
			if(rl_t->lastConfigToken > 0xFFFFFF) /* Due to token only have 24-bit field, recover those token not stored in ASIC */
				asic_rl.token += (rl_t->lastConfigToken - 0xFFFFFF);
			else if(rl_t->lastConfigToken < asic_rl.token) {
#if RATE_TBL_SYNCHK
				printk("Group %d addToken %u(%x) remained token %u(%x)\n", entry, rl_t->lastConfigToken, rl_t->lastConfigToken, asic_rl.token, asic_rl.token);
				printk("Other info: MAX %u Remain %u Interval %u Add %u\n", asic_rl.maxToken, asic_rl.t_intervalUnit, asic_rl.t_remainUnit, asic_rl.refill_number);
#endif
				/* Patch ASIC unclarified issue, treat this case as token not used */
				asic_rl.token = rl_t->lastConfigToken;
			}
			/* accumulate flow used bytes */
			rl_t->accumulatedToken += (rl_t->lastConfigToken -asic_rl.token);
			/* increment the token with slice */
			rl_t->lastConfigToken = asic_rl.token + rl_t->sliceIncrementToken;
			if(updateState==0)
				rl_t->lastConfigToken += (rl_t->rateMappedToken%100);
			if(rl_t->lastConfigToken > rl_t->maxToken) 
				rl_t->lastConfigToken = rl_t->maxToken;

			if(rl_t->lastConfigToken > 0xFFFFFF) {/* Due to token only have 24-bit field, if token number exceed 0xFFFFFF, saturate it only */
				asic_rl1.maxToken	= 0xFFFFFF;
				asic_rl1.token = 0xFFFFFF;
			} else {
				asic_rl1.maxToken = rl_t->lastConfigToken;
				asic_rl1.token = rl_t->lastConfigToken;
			}
			rtl8651_setAsicRateLimitTable(entry, &asic_rl1);	
#if RATE_TBL_SYNCHK
			rtl8651_getAsicRateLimitTable((entry+1)%32, &asic_rl);	/* read entry from ASIC */
			rtl8651_getAsicRateLimitTable(entry, &asic_rl);	/* read entry from ASIC */
			if(asic_rl1.token < asic_rl.token || asic_rl1.maxToken != asic_rl.maxToken ) {
				printk("Configure group %d with token %u(%x) but read back is %u(%x)\n\tMax token %u(%x) but read back is %u(%x)\n", 
					entry, asic_rl1.token, asic_rl1.token, asic_rl.token, asic_rl.token,
					asic_rl1.maxToken, asic_rl1.maxToken, asic_rl.maxToken, asic_rl.maxToken);
			}
#endif 

		}
	}

	if(++updateState >= RTL865X_MNQUEUE_UPDATE_PERIOD) {/* Every second computes the next step to do */
		/* (1) Calculate vRate and decides M queue token */
		mnqueue_ctrl.mVRate = mnqueue_ctrl.mEVRate = mnqueue_ctrl.nVRate = 0;
		mnqueue_ctrl.eagerNQueueNumber = 0;
		nRatioSummary = nQueueNeedProcessedNumber = 0;
		for(entry=0, rl_t = DrvTbl.rateLimit; entry<RTL8651_RATELIMITTBL_SIZE; entry++, rl_t++) {
			/* (1.1) Calculate vRate */
			if (rl_t->queueType) { /* 1: M queue's Group ID, 2: N queue's Group ID, 3 default queue's Group ID (Best effort) */
				rl_t->vRate = (((rl_t->vRate << 3) - rl_t->vRate)>>3) + (rl_t->accumulatedToken>>3);

#if DQM
				printk("Group %2d vRate is", entry);
				printRateString(rl_t->vRate);
				printk("\n");
#endif
			}
			/* (1.2) Decides M queue token and prepare control information for following N queue process */
			if (rl_t->queueType== 1) {
				mnqueue_ctrl.mVRate += rl_t->vRate;
				mnqueue_ctrl.mEVRate += rl_t->rateMappedToken;
			}
			else if (rl_t->queueType == 2 || rl_t->queueType == 3) { 
				mnqueue_ctrl.nVRate += rl_t->vRate;
				if(rl_t->eagerCounter)
					mnqueue_ctrl.eagerNQueueNumber++;
				rl_t->nQueueNeedProcess = 1; 
				nRatioSummary += rl_t->ratio;
				nQueueNeedProcessedNumber++;
			}
			rl_t->accumulatedToken = 0;
		}
		/* (2) Process N queue */
		mVRate = mnqueue_ctrl.mVRate + (mnqueue_ctrl.mVRate>>2);/* For M queues, not used bandwidth keep 1/2^2 for M queue traffic to get more */
		mVRate = (mVRate > mnqueue_ctrl.mEVRate)?  mnqueue_ctrl.mEVRate: mVRate;/* Total M queues only allow to use their total bandwidth*/
		nAvailableRate = mnqueue_ctrl.wanBandwidth > mVRate?mnqueue_ctrl.wanBandwidth - mVRate: 0;
		mnqueue_ctrl.nAvailableRate = nAvailableRate;

		/* 2.1 If used bandwidth larger than fair share bandwdith, it should be eager as well */
		for(entry=0, rl_t = DrvTbl.rateLimit; entry<RTL8651_RATELIMITTBL_SIZE; entry++, rl_t++) 
			if (rl_t->queueType == 2 || rl_t->queueType == 3) {
				rl_t->rateMappedToken = nAvailableRate * rl_t->ratio / nRatioSummary;
				if(rl_t->eagerCounter == 0 && rl_t->vRate > rl_t->rateMappedToken) {
					rl_t->eagerCounter = 1<<RTL865X_MNQUEUE_EAGERBITS;
					mnqueue_ctrl.eagerNQueueNumber++;
				}
			}

		/* 2.2 If part of N queues are eager, provide bandwdith to no eager queues */
		if(mnqueue_ctrl.eagerNQueueNumber >0 && mnqueue_ctrl.eagerNQueueNumber <= mnqueue_ctrl.nQueueNumber) {
			nConsumedRate = nConsumedRatio = 0;
			for(entry=0, rl_t = DrvTbl.rateLimit; entry<RTL8651_RATELIMITTBL_SIZE; entry++, rl_t++) 
				if (rl_t->eagerCounter == 0 && (rl_t->queueType == 2 || rl_t->queueType == 3)) { 
					if(rl_t->vRate + (rl_t->vRate>>4) < rl_t->rateMappedToken)
						rl_t->rateMappedToken = rl_t->vRate + (rl_t->vRate>>4);
					if(rl_t->rateMappedToken < RTL865X_MNQUEUE_MIN_RATE)
						rl_t->rateMappedToken = RTL865X_MNQUEUE_MIN_RATE;
					rl_t->maxToken = rl_t->rateMappedToken > RTL865X_MNQUEUE_MIN_MAXTOKEN? rl_t->rateMappedToken : RTL865X_MNQUEUE_MIN_MAXTOKEN;
					nConsumedRate += rl_t->rateMappedToken;
					nConsumedRatio += rl_t->ratio;
					rl_t->nQueueNeedProcess = 0;
					rl_t->sliceIncrementToken = rl_t->rateMappedToken /100;
				}
			nAvailableRate = (nAvailableRate>nConsumedRate)?(nAvailableRate-nConsumedRate):0;
			nRatioSummary -= nConsumedRatio;
		}
		/* 2.3 If part of N queues are eager, provide bandwdith to eager queues. If no N queue is eager, system provides bandwidth here. */
		if (nRatioSummary)	// chenyl: prevent from "divide by 0"
			for(entry=0, rl_t = DrvTbl.rateLimit; entry<RTL8651_RATELIMITTBL_SIZE; entry++, rl_t++) {
				if (rl_t->nQueueNeedProcess == 1 && (rl_t->queueType == 2 || rl_t->queueType == 3)) { 
					rl_t->rateMappedToken = nAvailableRate * rl_t->ratio / nRatioSummary;
					if(rl_t->rateMappedToken < RTL865X_MNQUEUE_MIN_RATE)
						rl_t->rateMappedToken = RTL865X_MNQUEUE_MIN_RATE;
					rl_t->maxToken = rl_t->rateMappedToken > RTL865X_MNQUEUE_MIN_MAXTOKEN? rl_t->rateMappedToken : RTL865X_MNQUEUE_MIN_MAXTOKEN;
					rl_t->nQueueNeedProcess = 0;
					rl_t->sliceIncrementToken = rl_t->rateMappedToken /100;
				}
			}
		updateState = 0;
	}
}

void _rtl8651_mnQueue_init(void) {
	rtl8651_tblDrv_rateLimit_t *rl_t;
	uint32 entry, mRatioSummary, nRatioSummary; /* Update MN queue control information*/

	mnqueue_ctrl.nQueueNumber = 0;
	for(entry=0, rl_t = DrvTbl.rateLimit, mRatioSummary = nRatioSummary = 0; entry<RTL8651_RATELIMITTBL_SIZE; entry++, rl_t++) {
		if (rl_t->queueType == 1) 
			mRatioSummary += rl_t->ratio;
		else if (rl_t->queueType == 2 || rl_t->queueType == 3) {
			nRatioSummary += rl_t->ratio;
			mnqueue_ctrl.nQueueNumber++;
		}
		rl_t->vRate = rl_t->accumulatedToken = rl_t->eagerCounter = 0;
	}
	for(entry=0, rl_t = DrvTbl.rateLimit; entry<RTL8651_RATELIMITTBL_SIZE; entry++, rl_t++) {
		if (rl_t->queueType == 1) {
			rl_t->rateMappedToken = rl_t->ratio * mnqueue_ctrl.wanBandwidth / 100;
			rl_t->maxToken = rl_t->rateMappedToken > RTL865X_MNQUEUE_MIN_MAXTOKEN? rl_t->rateMappedToken : RTL865X_MNQUEUE_MIN_MAXTOKEN;
		} else if (rl_t->queueType == 2 || rl_t->queueType == 3) {
			rl_t->rateMappedToken = (100 - mRatioSummary)*mnqueue_ctrl.wanBandwidth/100 * rl_t->ratio/nRatioSummary;
			rl_t->maxToken = rl_t->rateMappedToken > RTL865X_MNQUEUE_MIN_MAXTOKEN? rl_t->rateMappedToken : RTL865X_MNQUEUE_MIN_MAXTOKEN;
		}
		rl_t->sliceIncrementToken = rl_t->rateMappedToken /100;
	}

}

void rtl8651_dumpMNQueueSystemInfo(void) {
	rtl8651_tblDrv_rateLimit_t *rl_t;
	uint32 entry; /* Used to track the rate limit table */
	printk("MNQueue System information:\n");
	printk("\tWAN bandwidth %u ", mnqueue_ctrl.wanBandwidth);
	if(mnqueue_ctrl.wanBandwidth > 125000)
		printk("(%3u.%03u Mbps)", mnqueue_ctrl.wanBandwidth/125000, (mnqueue_ctrl.wanBandwidth%125000)/125);
	else if(mnqueue_ctrl.wanBandwidth > 125)
		printk("(%3u.%03u Kbps", mnqueue_ctrl.wanBandwidth/125, (mnqueue_ctrl.wanBandwidth%125));
	else
		printk("(%3u      bps)", mnqueue_ctrl.wanBandwidth);
	printk("\n\tM queue rate %u ", mnqueue_ctrl.mVRate);
	if(mnqueue_ctrl.mVRate > 125000)
		printk("(%3u.%03u Mbps)", mnqueue_ctrl.mVRate/125000, (mnqueue_ctrl.mVRate%125000)/125);
	else if(mnqueue_ctrl.mVRate > 125)
		printk("(%3u.%03u Kbps", mnqueue_ctrl.mVRate/125, (mnqueue_ctrl.mVRate%125));
	else
		printk("(%3u      bps)", mnqueue_ctrl.mVRate);
	printk("\n\tM queue expect rate %u ", mnqueue_ctrl.mEVRate);
	if(mnqueue_ctrl.mEVRate > 125000)
		printk("(%3u.%03u Mbps)", mnqueue_ctrl.mEVRate/125000, (mnqueue_ctrl.mEVRate%125000)/125);
	else if(mnqueue_ctrl.mEVRate > 125)
		printk("(%3u.%03u Kbps", mnqueue_ctrl.mEVRate/125, (mnqueue_ctrl.mEVRate%125));
	else
		printk("(%3u      bps)", mnqueue_ctrl.mEVRate);
	printk("\n\tN queue rate %u ", mnqueue_ctrl.nVRate);
	if(mnqueue_ctrl.nVRate > 125000)
		printk("(%3u.%03u Mbps)", mnqueue_ctrl.nVRate/125000, (mnqueue_ctrl.nVRate%125000)/125);
	else if(mnqueue_ctrl.nVRate > 125)
		printk("(%3u.%03u Kbps", mnqueue_ctrl.nVRate/125, (mnqueue_ctrl.nVRate%125));
	else
		printk("(%3u      bps)", mnqueue_ctrl.nVRate);
	printk("\n\tN queue available bandwidth %u ", mnqueue_ctrl.nAvailableRate);
	if(mnqueue_ctrl.nAvailableRate > 125000)
		printk("(%3u.%03u Mbps)", mnqueue_ctrl.nAvailableRate/125000, (mnqueue_ctrl.nAvailableRate%125000)/125);
	else if(mnqueue_ctrl.nAvailableRate > 125)
		printk("(%3u.%03u Kbps", mnqueue_ctrl.nAvailableRate/125, (mnqueue_ctrl.nAvailableRate%125));
	else
		printk("(%3u      bps)", mnqueue_ctrl.nAvailableRate);
	printk("\n");
	printk("Eager queue number %u total N queue number %u\n", mnqueue_ctrl.eagerNQueueNumber, mnqueue_ctrl.nQueueNumber);
	for(entry=0, rl_t = DrvTbl.rateLimit; entry<RTL8651_RATELIMITTBL_SIZE; entry++, rl_t++)
		if (rl_t->queueType) { /* 1: M queue's Group ID, 2: N queue's Group ID, 3 default queue's Group ID (Best effort) */
			printk("Group %u maxToken %u, rateMappedToken %u, vRate %u\n", entry, rl_t->maxToken, rl_t->rateMappedToken, rl_t->vRate);
			printk("\tlastConfigToken %u, sliceIncrementToken %u\n", rl_t->lastConfigToken, rl_t->sliceIncrementToken);
			printk("\taccumulatedToken %u, groupId %u, queueType %u\n", rl_t->accumulatedToken, rl_t->groupId, rl_t->queueType);
			printk("\tref_count %u, ratio %u, eagerCounter %u\n", rl_t->ref_count, rl_t->ratio, rl_t->eagerCounter); 
			if(rl_t->vRate > 125000)
				printk("\tvirtual rate is %3u.%03u Mbps ", rl_t->vRate/125000, (rl_t->vRate%125000)/125);
			else if(rl_t->vRate > 125)
				printk("\tvirtual rate is %3u.%03u Kbps ", rl_t->vRate/125, (rl_t->vRate%125));
			else
				printk("\tvirtual rate is %3u      bps ", rl_t->vRate);
			if(rl_t->rateMappedToken > 125000)
				printk("Allowed rate is %3u.%03u Mbps\n", rl_t->rateMappedToken/125000, (rl_t->rateMappedToken%125000)/125);
			else if(rl_t->rateMappedToken > 125)
				printk("Allowed rate is %3u.%03u Kbps\n", rl_t->rateMappedToken/125, (rl_t->rateMappedToken%125));
			else
				printk("Allowed rate is %3u      bps\n", rl_t->rateMappedToken);
		}
		
}

static void _rtl8651_flushMNQueue(void) {
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	_rtl8651_tblDrvAclRule_t *rl_rule;
	int32 entry;

	netif_t = DrvTbl.netIface;
	for(entry=0; entry<rtl8651_tblDrvPara.networkIntfTableSize; entry++, netif_t++) {
		if (netif_t->valid == 0)
			continue;
		while( (rl_rule = CTAILQ_FIRST(&netif_t->mnqHead)) ) {
			DrvTbl.rateLimit[rl_rule->rlIndex].groupId = 0;
			rtl8651_delAsicRateLimitTable(rl_rule->rlIndex);
			DrvTbl.rateLimit[rl_rule->rlIndex].ratio = 0;
			CTAILQ_REMOVE(&netif_t->mnqHead, rl_rule, nextRule);
			CTAILQ_INSERT_HEAD(&DrvTbl.freeList.aclRule, rl_rule, nextRule);
		}
	}
	_rtl8651_arrangeAllACL();
}


static int32 _rtl8651_delFlowFromMNQueue(rtl8651_tblDrvAclRule_t *rule, uint32 GroupId)
{
	_rtl8651_tblDrvAclRule_t *rl_rule;
	rtl8651_tblDrv_rateLimit_t *rl_t;
	CTAILQ_HEAD(_aclStruct, _rtl8651_tblDrvAclRule_s);
	int32 entry;

	/* Find out the specified rate limit group ID */
	rl_t = DrvTbl.rateLimit;

	for(entry=0; entry<RTL8651_RATELIMITTBL_SIZE; entry++, rl_t++) 
		if (rl_t->groupId == GroupId)
			break;
	if (entry == RTL8651_RATELIMITTBL_SIZE) 
		return TBLDRV_ENOGROUPIDFOUND;
	if (rule->actionType_ == RTL8651_ACL_DROP) {
		if (rl_t->isByteCount == TRUE)
			rule->actionType_ = RTL8651_ACL_DROP_RATE_EXCEED_BPS;
		else rule->actionType_ = RTL8651_ACL_DROP_RATE_EXCEED_PPS;
	}
	else {
		if (rl_t->isByteCount == TRUE)
			rule->actionType_ = RTL8651_ACL_LOG_RATE_EXCEED_BPS;
		else rule->actionType_ = RTL8651_ACL_LOG_RATE_EXCEED_PPS;
	}


	/* Find the specified rule */
	CTAILQ_FOREACH_REVERSE(rl_rule, &rl_t->netif_t->mnqHead, _aclStruct, nextRule) {
		if (_rtl8651_sameAclRule(rl_rule, rule) == TRUE) {
			rl_t->ref_count --;
			CTAILQ_REMOVE(&rl_t->netif_t->mnqHead, rl_rule, nextRule);
			CTAILQ_INSERT_HEAD(&DrvTbl.freeList.aclRule, rl_rule, nextRule);
			_rtl8651_arrangeAllACL();

			if (rl_t->ref_count == 0) { /* The entry should be removed */
				rl_t->groupId = 0;  /* remove the entry */
				rl_t->queueType = 0; /* inform MNQ timer not to update this */
				rtl8651_delAsicRateLimitTable(entry);	
			}
			return SUCCESS;
		}
	}

	_rtl8651_mnQueue_init();
	
	return TBLDRV_EENTRYNOTFOUND;
}


static int32 _rtl8651_addFlowToMNQueue(int8 *ifName, rtl8651_tblDrvAclRule_t *rule, uint32 ratio, uint32 qidx, int8 isMqueue)
{
	rtl8651_tblDrv_rateLimit_t *default_rl_t = &DrvTbl.rateLimit[RTL8651_RATELIMITTBL_SIZE-1], *rl_t;
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	_rtl8651_tblDrvAclRule_t *rl_rule;
	int32 retval=SUCCESS;
	uint32 flowRatio, queueType;
	uint32 entry, mRatioSummary;

	/* Only to create a rate limit entry */
	if (rule == NULL)
		return SUCCESS;

	/* For M+N+1 Queue, the last one entry of Rate Limit Table is used as default class flow */
	if (default_rl_t->queueType!= 3)
		return TBLDRV_EINVALIDINPUT;
	if (rule->actionType_!=RTL8651_ACL_DROP && rule->actionType_!=RTL8651_ACL_DROP_LOG)
		return TBLDRV_EINVALIDINPUT;
	if ((netif_t=GET_NETWORK_INTERFACE(ifName, NULL)) == NULL) 
		return TBLDRV_EINVNETIFNAME;

	/* Make sure there still has enough entry */
	if ((rl_rule=CTAILQ_FIRST(&DrvTbl.freeList.aclRule)) == NULL) 
		return TBLDRV_ENOFREEBUFFER;

	if(isMqueue == TRUE) {
		for(entry=0, rl_t = DrvTbl.rateLimit, mRatioSummary = 0; entry<RTL8651_RATELIMITTBL_SIZE; entry++, rl_t++) {
			if (rl_t->queueType == 1) 
				mRatioSummary += rl_t->ratio;
		}
		if(mRatioSummary + ratio > 100)
		{
			return TBLDRV_EINVALIDINPUT;
		}
	}
	flowRatio = (isMqueue==TRUE) ?(ratio * mnqueue_ctrl.wanBandwidth / 100)  : 0; 
	/* M Queue: queueType=1, N Queue: queueType = 2 */
	queueType = (isMqueue==TRUE)? 1: 2;

	if ((retval=_rtl8651_addRateLimitGroup(&qidx, ifName, flowRatio, flowRatio, TRUE, ratio, queueType)) != SUCCESS)
		return retval;	

	CTAILQ_REMOVE(&DrvTbl.freeList.aclRule, rl_rule, nextRule);
	bzero(rl_rule, sizeof(_rtl8651_tblDrvAclRule_t));
	rtl8651_memcpy(rl_rule, rule, sizeof(rtl8651_tblDrvAclRule_t));
	CTAILQ_INSERT_TAIL(&netif_t->mnqHead, rl_rule, nextRule);
	if (rule->actionType_ == RTL8651_ACL_DROP)
		rl_rule->actionType_ = RTL8651_ACL_DROP_RATE_EXCEED_BPS;
	else rl_rule->actionType_ = RTL8651_ACL_LOG_RATE_EXCEED_BPS;
	rl_rule->rlIndex = qidx;
	if (isMqueue == TRUE)
		rl_rule->priority_ = 1; /* Add to M queue */
	mnQ_enable = TRUE;

	/* Edward: Update rate when flow belong to M queue */
	rl_t = &DrvTbl.rateLimit[qidx];	

	/* 0110: bug fix */
	rl_t->ref_count++;
	rl_t->mark = 0;		/* <ghhuang>: Clear to 0 for correctness */

	_rtl8651_mnQueue_init();

	/* Update default Class flow */
	_rtl8651_arrangeAllACL();

	return SUCCESS;
}

int32 rtl8651_addFlowToUnlimitedQueue(int8 *ifName, rtl8651_tblDrvAclRule_t *rule, uint32 ratio, uint32 queueID)
{
	int32 retval;

	if (ifName == NULL || ratio == 0 || ratio > 100 || queueID < RATELIMIT_GROUP_INTERNALRESV)
		return TBLDRV_EINVALIDINPUT;
	rtlglue_drvMutexLock(); 	
	retval = _rtl8651_addFlowToMNQueue(ifName, rule, ratio, queueID, TRUE);
	rtlglue_drvMutexUnlock();
	return retval;
}



int32 rtl8651_addFlowToLimitedQueue(int8 *ifName, rtl8651_tblDrvAclRule_t *rule, uint32 ratio, uint32 queueID)
{
	int32 retval;

	if (ifName == NULL  || ratio == 0 || ratio > 100 || queueID < RATELIMIT_GROUP_INTERNALRESV)
		return TBLDRV_EINVALIDINPUT;
	rtlglue_drvMutexLock();
	retval = _rtl8651_addFlowToMNQueue(ifName, rule, ratio, queueID, FALSE);	
	rtlglue_drvMutexUnlock();

	return retval;
}

int32 rtl8651_delFlowFromMNQueue(rtl8651_tblDrvAclRule_t *rule, uint32 GroupId)
{
	int32 retval;
	if (rule == NULL || (rule->actionType_ != RTL8651_ACL_DROP && rule->actionType_!=RTL8651_ACL_DROP_LOG) || !GroupId) 
		return TBLDRV_EINVALIDINPUT;
	rtlglue_drvMutexLock();
	retval = _rtl8651_delFlowFromMNQueue(rule, GroupId);
	rtlglue_drvMutexUnlock();
	return retval;	
}

int32 rtl8651_getMNQueueEntry(rtl8651_tblDrvAclRule_t *rule,uint32 GroupId,uint32 index)
{
	rtl8651_tblDrv_rateLimit_t *rl_t;
	int32  entry;
	
	/* Find the specified entry */
	for(entry=0, rl_t=DrvTbl.rateLimit; entry<RTL8651_RATELIMITTBL_SIZE; entry++, rl_t++)  {
		if (rl_t->groupId == GroupId) {

			_rtl8651_tblDrvAclRule_t asic_acl;
			rtl865x_tblAsicDrv_vlanParam_t asic_vlan;
			uint32 start, end,match_idx=0;
			uint16 vid;


			memset(rule,0,sizeof(*rule));

			for(vid=0; vid<4095; vid++ ) 
			{
				/* Read VLAN Table */
				if (rtl8651_getAsicVlan(vid, &asic_vlan) == FAILED)
					continue;

				if(!asic_vlan.internal) continue;
	

				start = asic_vlan.inAclStart; end = asic_vlan.inAclEnd;
				for( ; start<=end; start++) 
				{
					if (rtl8651_getAsicAclRule(start, &asic_acl) == FAILED)
					assert(0);

					if(asic_acl.actionType_!=12) continue; /* drop rate exceed bps */
				
					if(entry!=asic_acl.rlIndex) continue; /* check GroupID */

					if(match_idx==index) 	
					{						
						memcpy((void*)rule,(void*)&asic_acl,sizeof(*rule));
						return SUCCESS;
					}
					match_idx++;
				}
			}			
		}
	}
	return TBLDRV_EENTRYNOTFOUND;	

}



int32 rtl8651_setUpstreamBandwidth(int8 *ifName, uint32 bps)
{
	rtl8651_tblDrv_rateLimit_t *default_rl_t = &DrvTbl.rateLimit[RTL8651_RATELIMITTBL_SIZE-1];
	rtl865x_tblAsicDrv_rateLimitParam_t asic_rl = { 0, 0, 0x3f, 0x3f, 0 }; 
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	int32 retval=SUCCESS;

	
	/*
	  * The ifName parameter is used to extend N+M queuing system to apply to multiple
	  * WAN. But currently, the multiple WAN is not supported.
	  */
	if (bps > (100 * 1000*1000) ) /* bps <= 100Mbps */
		return TBLDRV_EINVALIDINPUT;
	rtlglue_drvMutexLock();		
	/* Check if the specified network interface is an external interface */
	if ((netif_t=GET_NETWORK_INTERFACE(ifName, NULL)) == NULL) {
		retval = TBLDRV_EINVNETIFNAME;
		goto out;
	}
	/* If bps == 0 , disable M-N-1 Queue */
	if (bps == 0) {
		mnQ_enable = FALSE;
		default_rl_t->groupId = 0;
		_rtl8651_flushMNQueue();
		goto out;
	}

	mnqueue_ctrl.wanBandwidth = (bps >> 3); /* unit: N bytes/sec */

	/* Update default rate limit flow */
	if (default_rl_t->queueType!= 0 && default_rl_t->queueType!= 3) {
		retval = TBLDRV_EINVALIDINPUT;
		goto out;
	}
	if (default_rl_t->queueType== 0) {
		bzero(default_rl_t, sizeof(rtl8651_tblDrv_rateLimit_t));
		default_rl_t->queueType= 3;
		rtl8651_setAsicRateLimitTable(RTL8651_RATELIMITTBL_SIZE-1, &asic_rl);
	}
	default_rl_t->ratio = 1;

	_rtl8651_mnQueue_init();
	
	mnQ_enable = TRUE;
	rtl8651_setAsicHLQueueWeight(RTL8651_RR_H_FIRST); /* Always high queue first */
	_rtl8651_arrangeAllACL();
	
out:
	rtlglue_drvMutexUnlock();
	return retval;
}

/*
 *	<ghhuang>:
 *    In addition to maintain compatibility without modifying old API interfaces, 
 *    this function is newly added.  Currently it sets DSCP field of each MN Queue
 *    according to given priority=>dscp mapping.
 *
 */
int32 rtl8651_setDscpToMNQueue(uint32 queueID, uint32 queueDSCP)
{
	rtl8651_tblDrv_rateLimit_t *rl_t;
	uint32 entry;

	rtlglue_drvMutexLock();
	
	for(entry=0, rl_t=DrvTbl.rateLimit; entry<RTL8651_RATELIMITTBL_SIZE; entry++, rl_t++) {
		if (rl_t->groupId == queueID)  {
			rl_t->dscp = (uint16) queueDSCP;
			rl_t->mark = 1;

			/* Update default Class flow */
			_rtl8651_arrangeAllACL();

			rtlglue_drvMutexUnlock();
			return SUCCESS;
		}
	}

	rtlglue_drvMutexUnlock();
	return FAILED;
}


#define DIFFSERV		1
#if DIFFSERV

/*
 *  <ghhuang>:
 *  This function marks given packet with rule-specified DSCP.
 */
int _rtl8651_markDiffservPkt(_rtl8651_tblDrvAclRule_t *rule, struct ip *pip)
{
        uint8 dscp = DrvTbl.rateLimit[rule->rlIndex].dscp;
        uint8 oldTos;

        oldTos = pip->ip_tos;
        pip->ip_tos = dscp;
        _RTL8651_DATA_ADJUST_CHECKSUM(dscp, oldTos, pip->ip_sum);

        return SUCCESS;
}


/*
 *  <ghhuang>:
 *  This function matches given packet with rules of MNQ.
 *  If matched, then return the matched rule.
 *  If not, return NULL.
 */
_rtl8651_tblDrvAclRule_t * _rtl8651_matchDiffservACL(uint32 dsid, struct ip *iphdr)
{
	CTAILQ_HEAD(_aclStruct, _rtl8651_tblDrvAclRule_s) *aclHead;
	struct tcphdr *tcphdr;
	_rtl8651_tblDrvAclRule_t *rule;
	ipaddr_t ip_s, ip_d;	

//	printk("Step 0\n");

	/* Do not handle fragment/non-TCP packets */
       if (iphdr == (struct ip *)NULL || iphdr->ip_p != IPPROTO_TCP || IS_FRAGMENT(iphdr->ip_off))
		return (_rtl8651_tblDrvAclRule_t *)NULL;

	ip_s = PKTGET_UINT32_UNALIGNED(&iphdr->ip_src);
	ip_d = PKTGET_UINT32_UNALIGNED(&iphdr->ip_dst);

//	printk("IP ID: %-#4.4X\n", iphdr->ip_id);

	tcphdr = (struct tcphdr *)((uint8 *)iphdr + ((iphdr->ip_vhl  & 0x0f)<<2));
	 
//	printk("Step 1\n");

/*
	if (tcphdr->th_flags & TH_SYN)
		printk("SYN\n");
	if (tcphdr->th_flags & TH_RST)
		printk("RST\n");
*/

	aclHead = (struct _aclStruct *)&DrvTbl.netIface->mnqHead;

	CTAILQ_FOREACH(rule, aclHead, nextRule) {
		int ruleno = 0;

		/* Check if the rule needs to be marked */
		if (DrvTbl.rateLimit[rule->rlIndex].mark != 1)
			break;

		ruleno++;

//		printk("Rule %d\n", ruleno);
	
		 if (((tcphdr->th_flags & rule->tcpFlagMask_) != rule->tcpFlag_) ||
			 !(ntohs(tcphdr->th_sport)>=rule->tcpSrcPortLB_ && ntohs(tcphdr->th_sport) <= rule->tcpSrcPortUB_) ||
			 !(ntohs(tcphdr->th_dport)>=rule->tcpDstPortLB_ && ntohs(tcphdr->th_dport) <= rule->tcpDstPortUB_) )
				break;

//		printk("Step 3\n");

		if (((iphdr->ip_tos & rule->tosMask_) == rule->tos_) && (rule->dsid==dsid)) {
			if (rule->isIPRange_ == 1) {	
				/* If both start and end are zero,  we consider it is match*/
				if ((rule->srcIpAddrStart_==0 && rule->srcIpAddrEnd_==0) || 
				    (rule->srcIpAddrStart_<=ip_s && rule->srcIpAddrEnd_>=ip_s)) { //source match
					if ((rule->dstIpAddrStart_==0 && rule->dstIpAddrEnd_==0) ||
					    (rule->dstIpAddrStart_<=ip_d && rule->dstIpAddrEnd_>=ip_d)) //destination match
					    	return (_rtl8651_tblDrvAclRule_t *)rule;
				}
			}
			else {
				if ((( ip_s & rule->srcIpAddrMask_) == rule->srcIpAddr_) && ((ip_d & rule->dstIpAddrMask_) == rule->dstIpAddr_) )
					return (_rtl8651_tblDrvAclRule_t *)rule;
			}
		}	
	}

//	printk("Step 4\n");
	
	return (_rtl8651_tblDrvAclRule_t *)NULL;
}
#endif

/*=======================================
  * Ratelimit counter  APIs
  *=======================================*/
#define RTL865X_RATE_LIMIT_COUNTER_APIs

#undef RATE_LIMIT_COUNTER_DEBUG

#define RATELIMIT_COUNTER_TABLE_INDEX(entry)		(entry - &rateLimitCounterRegisterTable[0])
#define RATELIMIT_COUNTER_MAX_TOKEN				0xffffff	/* for counter init/refill */
#define RATELIMIT_COUNTER_MIN_TOKEN				0xfffff		/* for automatically remove */

/*
	Add the rate limit counter entry for specific netif_t and counterType.
	The new rate limit counter's token would be initiated to "RATELIMIT_COUNTER_MAX_TOKEN"
*/
int32 _rtl8651_addRateLimitCounter(rtl8651_tblDrv_networkIntfTable_t *netif_t, uint8 counterType)
{
	rtl8651_tblDrv_rateLimitCounter_t *freeEntry=NULL;
	uint32 index, GroupId, input;
	int32 retval;

#ifdef RATE_LIMIT_COUNTER_DEBUG
	printk("[Note] Add Rate Limit Counter for interface (%s) type (%d)\n", netif_t->ifName, counterType);
#endif


	for ( index=0;index<RATELIMIT_COUNTER_RESV_SIZE;index++ )
	{
		if ( !rateLimitCounterRegisterTable[index].valid )
		{
			if ( freeEntry==NULL )
				freeEntry = &rateLimitCounterRegisterTable[index];
			continue;
		}
		if ( rateLimitCounterRegisterTable[index].netif_t==netif_t &&
		     rateLimitCounterRegisterTable[index].counterType==counterType )
		{
			return SUCCESS;	/* duplicate counterType added by same netif_t */
		}
	}
	if ( freeEntry==NULL )
		return TBLDRV_ENOFREEBUFFER;

	input = GroupId = RATELIMIT_COUNTER_TABLE_INDEX(freeEntry)+RATELIMIT_COUNTER_RESV_BASE;	/* unique Group ID of each rate limit counter entry */

#ifdef RATE_LIMIT_COUNTER_DEBUG
	printk("[Note] Add Rate Limit Group for Group id (%d) interface (%s) type (%d)\n", GroupId, netif_t->ifName, counterType);
#endif

	if ( (retval =_rtl8651_addRateLimitGroup(&input, netif_t->ifName, RATELIMIT_COUNTER_MAX_TOKEN, 0, FALSE, 0, 0 /* queue type == 0, don't updated in _rtl8651_mnQueueTimer() */))
		== SUCCESS )
	{
#ifdef RATE_LIMIT_COUNTER_DEBUG
		printk("\t\t==> SUCCESS, rateLimitIndex (%d)\n", input);
#endif

		assert(DrvTbl.rateLimit[input].groupId == GroupId);
		freeEntry->valid = 1;
		freeEntry->netif_t = netif_t;
		freeEntry->rateLimitIndex= input;	/* addRateLimitGroup would return the index of rate limit table */
		freeEntry->counterType = counterType;

		/*reset rate limit table to Actually reset rate limit entry content */
		retval = _rtl8651_getRateLimitCounter(netif_t, counterType, TRUE);

		return SUCCESS;
	} 

#ifdef RATE_LIMIT_COUNTER_DEBUG
	printk("\t\t==> Not SUCCESS (%d)\n", retval);
#endif

	return retval;
}

/*
	Add ACL Rule for a rate limit counter
*/
int32 _rtl8651_addRateLimitCounterRule(rtl8651_tblDrvAclRule_t *rule, rtl8651_tblDrv_networkIntfTable_t *netif_t, uint8 counterType)
{
	rtl8651_tblDrv_rateLimitCounter_t *entry = &rateLimitCounterRegisterTable[0];
	uint32 index;

	rule->isEgressRateLimitRule_ = 1;	/* We treat rate limit counter as one kind of egress rate limit rule. */

	for ( index=0;index<RATELIMIT_COUNTER_RESV_SIZE;index++,entry++ )
	{
		if ( entry->valid && entry->netif_t==netif_t && entry->counterType==counterType )
		{
#ifdef RATE_LIMIT_COUNTER_DEBUG
			printk("[Note] Add Rate Limit Counter ACL for interface (%s) type (%d)\n", netif_t->ifName, counterType);
#endif
			return _rtl8651_addRateLimitRule(rule, index+RATELIMIT_COUNTER_RESV_BASE, TRUE);
		}
	}
	return TBLDRV_EENTRYNOTFOUND;
}

/*
	Delete ACL Rule for a rate limit counter
*/
int32 _rtl8651_delRateLimitCounterRule(rtl8651_tblDrvAclRule_t *rule, rtl8651_tblDrv_networkIntfTable_t *netif_t, uint8 counterType)
{
	rtl8651_tblDrv_rateLimitCounter_t *entry = &rateLimitCounterRegisterTable[0];
	uint32 index;
	int32 retval;

	rule->isEgressRateLimitRule_ = 1;       /* We treat rate limit counter as one kind of egress rate limit rule. */

	for ( index=0;index<RATELIMIT_COUNTER_RESV_SIZE;index++,entry++ )
	{
		if ( entry->valid && entry->netif_t==netif_t && entry->counterType==counterType )
		{
			#ifdef RATE_LIMIT_COUNTER_DEBUG
				printk("[Note] Delete Rate Limit Counter ACL for (%s) type (%d)\n", netif_t->ifName, counterType);
			#endif

			retval = _rtl8651_delRateLimitRule(rule, index+RATELIMIT_COUNTER_RESV_BASE);

			assert(retval == SUCCESS);
			#ifdef RATE_LIMIT_COUNTER_DEBUG
			printk("\t\t==> return value (%d)\n", retval);
			#endif

			if (DrvTbl.rateLimit[entry->rateLimitIndex].ref_count == 0)
			{
				#ifdef RATE_LIMIT_COUNTER_DEBUG
					printk("[Note] Reference count == 0, Delete Rate Limit Counter for (%s) type (%d)\n", netif_t->ifName, counterType);
				#endif

				retval = _rtl8651_delRateLimitGroup(index+RATELIMIT_COUNTER_RESV_BASE);

				assert(retval == SUCCESS);
				#ifdef RATE_LIMIT_COUNTER_DEBUG
				printk("\t\t==> return value (%d)\n", retval);
				#endif

				/* invalidate this entry */
				memset(entry, 0, sizeof(rtl8651_tblDrv_rateLimitCounter_t));
			}
			return retval;
		}
	}
	return TBLDRV_EENTRYNOTFOUND;
}

/*
	Remove the Rate Limit Counter entry for specific network interface and
	counter type.
*/
int32 _rtl8651_delRateLimitCounter(rtl8651_tblDrv_networkIntfTable_t *netif_t, uint8 counterType)
{
	rtl8651_tblDrv_rateLimitCounter_t *entry = &rateLimitCounterRegisterTable[0];
	uint32 index;
	int32 retval = TBLDRV_EENTRYNOTFOUND;

#ifdef RATE_LIMIT_COUNTER_DEBUG
	printk("[Note] Delete Rate Limit Counter for (%s) type (%d)\n", netif_t->ifName, counterType);
#endif

	for ( index=0;index<RATELIMIT_COUNTER_RESV_SIZE;index++,entry++ )
	{
		if ( entry->valid && entry->netif_t==netif_t && entry->counterType==counterType )
		{
			_rtl8651_tblDrvAclRule_t *acl_t, *next_acl_t;

			/* remove the corresponding ACL rules */
			/* 
				Currently, rate limit counter rule is treated as egress rate limit rule.
				Therefore, we only traverse outRateLimitHead list.
			*/
			acl_t = CTAILQ_FIRST(&entry->netif_t->outRateLimitHead);
			while(acl_t) {

				next_acl_t = CTAILQ_NEXT(acl_t, nextRule);

				if (acl_t->isRateLimitCounter_ &&
				     acl_t->rlIndex == entry->rateLimitIndex) {

					#ifdef RATE_LIMIT_COUNTER_DEBUG
					printk("[Note] Remove Rate Limit Rule (%p) for interface (%s) type [%d]\n", acl_t, netif_t->ifName, counterType);
					#endif
					retval = _rtl8651_delRateLimitRule((rtl8651_tblDrvAclRule_t *)acl_t, index+RATELIMIT_COUNTER_RESV_BASE);
					assert(retval == SUCCESS);
					#ifdef RATE_LIMIT_COUNTER_DEBUG
					printk("\t\t==> return value (%d)\n", retval);
					#endif
				}

				acl_t = next_acl_t;
			}

			#ifdef RATE_LIMIT_COUNTER_DEBUG
			printk("[Note] Remove Rate Limit Group (%s) type [%d]\n", netif_t->ifName, counterType);
			#endif

			retval = _rtl8651_delRateLimitGroup(index+RATELIMIT_COUNTER_RESV_BASE);
			assert(retval == SUCCESS);	
			#ifdef RATE_LIMIT_COUNTER_DEBUG
			printk("\t\t==> return value (%d)\n", retval);
			#endif

			/* invalidate this entry */
			memset(entry, 0, sizeof(rtl8651_tblDrv_rateLimitCounter_t));
			
			break;
		}
	}
	return retval;
}

/*
	Get the token count for specific network interface and counter type.
	if refill==1, we refill the token count for this entry in ASIC.
*/
int32 _rtl8651_getRateLimitCounter(rtl8651_tblDrv_networkIntfTable_t *netif_t, uint8 counterType, uint8 refill)
{
	rtl8651_tblDrv_rateLimitCounter_t *entry = &rateLimitCounterRegisterTable[0];
	rtl865x_tblAsicDrv_rateLimitParam_t asic_rl;
	uint32 index;
	uint32 counter = 0;

	for ( index=0;index<RATELIMIT_COUNTER_RESV_SIZE;index++,entry++ )
	{
		if ( entry->valid && entry->netif_t==netif_t && entry->counterType==counterType )
			break;
	}
	if ( index == RATELIMIT_COUNTER_RESV_SIZE )
		goto fail;

	/* get the ASIC counter of this entry */
	if (rtl8651_getAsicRateLimitTable(entry->rateLimitIndex, &asic_rl)==SUCCESS)
	{
		counter = RATELIMIT_COUNTER_MAX_TOKEN - asic_rl.token;
#ifdef RATE_LIMIT_COUNTER_DEBUG
		printk("[Note] Get Rate Limit Counter for (%s) type (%d) idx (%d) result (%d)\n", netif_t->ifName, counterType, entry->rateLimitIndex, counter);
#endif

		if ( refill )
		{
			asic_rl.token = RATELIMIT_COUNTER_MAX_TOKEN;
			asic_rl.maxToken = RATELIMIT_COUNTER_MAX_TOKEN;
			asic_rl.refill_number = 0;

#ifdef RATE_LIMIT_COUNTER_DEBUG
			printk("[Note] Refill Rate Limit Counter idx[%d] --\n\ttoken (%u)\n\tmaxToken (%u)\n\trefill_number (%u)\n\tremainUnit (%u)\n\tintervalUnit (%u)\n",
			entry->rateLimitIndex,
			asic_rl.token,
			asic_rl.maxToken,
			asic_rl.refill_number,
			asic_rl.t_remainUnit,
			asic_rl.t_intervalUnit);
#endif

			rtl8651_setAsicRateLimitTable(entry->rateLimitIndex, &asic_rl);
		}
	}
fail:
	return counter;
}

/*
		In order to keep the correctness of packet forwarding,
		The Rate Limit Counter entry would be removed automatically if we find the remaining token
		count of this entry is less than the low threshold.
*/
void _rtl8651_rateLimitCounterTimeUpdate(uint32 secpassed)
{
	rtl8651_tblDrv_rateLimitCounter_t *entry = &rateLimitCounterRegisterTable[0];
	uint32 index;

	for ( index=0;index<RATELIMIT_COUNTER_RESV_SIZE;index++,entry++ )
	{
		if ( entry->valid )
		{
			if ( !entry->netif_t->valid ) {
				assert(0);
				goto remove;
			}
			if ( _rtl8651_getRateLimitCounter(entry->netif_t, entry->counterType, 0) >=
			     (RATELIMIT_COUNTER_MAX_TOKEN - RATELIMIT_COUNTER_MIN_TOKEN) )
			{	/* remove this entry automatically*/
				#ifdef RATE_LIMIT_COUNTER_DEBUG
				rtlglue_printf("[Note] Remove Rate Limit Counter of (%d) type for (%s) interface\n", entry->counterType, entry->netif_t->ifName);
				#endif
remove:				
				_rtl8651_delRateLimitCounter(entry->netif_t, entry->counterType);
			}
		}
	}
}

void _rtl8651_flushRateLimitCounter(void)
{
	rtl8651_tblDrv_rateLimitCounter_t *entry = &rateLimitCounterRegisterTable[0];
	uint32 index;

	for ( index=0;index<RATELIMIT_COUNTER_RESV_SIZE;index++,entry++ )
	{
		if ( entry->valid )
		{
			_rtl8651_delRateLimitCounter(entry->netif_t, entry->counterType);
		}
	}
}

/*=======================================
  * NextHop Table  APIs
  *=======================================*/
#define RTL865X_NEXT_HOP_APIs


static uint32 _l2Reserved_;

static int32 _rtl8651_init_nexthopEntry(uint32 entryIndex)
{
	ether_addr_t mac = { {0x00, 0x00, 0x0a, 0x00, 0x00, 0x0f} };
	rtl865x_tblAsicDrv_nextHopParam_t asic;
	rtl865x_tblAsicDrv_l2Param_t l2_asic;
	int32 retval;

	/*
	  * Reserve a next hop entry and a l2 entry.
	  * This reserved entry will be referenced by those entries which have no full
	  * information, (nexthop, MAC) pair.
	  */

	if (entryIndex >= RTL8651_NEXTHOPTBL_SIZE)
		return FAILED;

	_l2Reserved_ = rtl8651_filterDbIndex(&mac);
	memset((void *)&l2_asic, 0, sizeof(rtl865x_tblAsicDrv_l2Param_t));
	memcpy((void *)&l2_asic.macAddr, (void *)&mac, 6);
	l2_asic.cpu			= TRUE;
	l2_asic.isStatic 	= TRUE;
	l2_asic.ageSec		= 500;
	if ((retval = rtl8651_setAsicL2Table(_l2Reserved_, 0, &l2_asic)) != SUCCESS)
		return retval;

	memset((void *)&asic, 0, sizeof(rtl865x_tblAsicDrv_nextHopParam_t));
	asic.nextHopColumn 	= 0;
	asic.nextHopRow	 	= _l2Reserved_;
	retval = rtl8651_setAsicNextHopTable(entryIndex, &asic);
	return retval;
}



static void 
_rtl8651_updateNextHop_NHIDX(rtl8651_tblDrv_arpEntry_t *arp_t,  rtl8651_tblDrv_pppTable_t *session_t)
{
	rtl8651_tblDrv_nextHopEntry_t *entry_t;

	/*
	  * Note that the both arp_t and pppoe_t can not be non-NULL. If the arp_t != NULL,
	  * we use it to update ethernet type nexthop; otherwise use pppoe_t to update
	  * PPPoE nexthop.
	  */

	assert(!(arp_t && session_t));
	assert(arp_t || session_t);

	/*
	  * Assumption: 
	  * If the nexthop entry's nexthop is the specified ARP, it implies remove the entry's 
	  * nexthop. Contrarily if the nexthop entry's nexthop is NULL, implies add the ARP
	  * to the entry.
	  */
	if (arp_t) {
		SLIST_FOREACH(entry_t, &DrvTbl.inuseList.nexthop, next) {
			if (entry_t->nextHopType != RTL8651_LL_VLAN)
				continue;
			if (!entry_t->nexthop_t && entry_t->nexthop == arp_t->ipAddr) {
				/* assign the arp_t to the nexthop entry */
				entry_t->nexthop_t = (void *)arp_t;
				_rtl8651_updateNextHopEntry(0, entry_t);
			} else  if (entry_t->nexthop_t && entry_t->nexthop == arp_t->ipAddr) {
				entry_t->nexthop_t = (void *)NULL;
				_rtl8651_updateNextHopEntry(0, entry_t);
			}
		}
	}
	else {
		/* Traversal all PPPoE type entry in the nexthop table, and try to update its L2 info. */
		SLIST_FOREACH(entry_t, &DrvTbl.inuseList.nexthop, next) {
			if (entry_t->nexthop_t == (void *)session_t)
				_rtl8651_updateNextHopEntry(0, entry_t);
		}
	}
}



static void 
_rtl8651_updateNextHopEntry(ipaddr_t nexthop, rtl8651_tblDrv_nextHopEntry_t *entry_t)
{
	rtl865x_tblAsicDrv_nextHopParam_t asic;

	/*
	  * If nexthop != 0, we use it to find the entry in the nexthop table, otherwise, we use
	  * entry_t as the wanted entry to update to ASIC.
	  */

	assert(nexthop || entry_t);

	if (nexthop != 0) {
		SLIST_FOREACH(entry_t, &DrvTbl.inuseList.nexthop, next)
			if (entry_t->nexthop == nexthop)
				break;
	}
	
	bzero(&asic, sizeof(rtl865x_tblAsicDrv_nextHopParam_t));
	if (entry_t->nextHopType == RTL8651_LL_VLAN ) {
		rtl8651_tblDrv_arpEntry_t *arp_t = (rtl8651_tblDrv_arpEntry_t *)entry_t->nexthop_t;
		asic.nextHopRow		= (arp_t)? rtl8651_filterDbIndex(&arp_t->macInfo->macAddr):  _l2Reserved_;
		asic.nextHopColumn	= (arp_t)? arp_t->macInfo->asicPos: 0;
		asic.isPppoe			= FALSE;
		asic.pppoeIdx			= 0;
	}
	else {
		/*
		  * NOTE:
		  * How to handle if property set but no NAPT specified????
		  */
		rtl8651_tblDrv_pppTable_t *pppoe_t = (rtl8651_tblDrv_pppTable_t *)entry_t->nexthop_t;
		assert(pppoe_t);
		asic.nextHopRow			= (pppoe_t->initiated)? rtl8651_filterDbIndex(&pppoe_t->macInfo->macAddr): _l2Reserved_;
		asic.nextHopColumn		= (pppoe_t->initiated)? pppoe_t->macInfo->asicPos: 0;
		asic.isPppoe 			= (pppoe_t->type == RTL8651_PPPOE)? TRUE: FALSE;
		asic.pppoeIdx			= (pppoe_t->type == RTL8651_PPPOE)? (PPPOE_TABLE_INDEX(pppoe_t)): 0;
	}
	

//	if (entry_t->nextHopType == RTL8651_LL_PPTP || entry_t->nextHopType == RTL8651_LL_L2TP)
//		asic.dvid = rtl8651_vlanTableIndex(tunnel.loopBackVid);
//	else asic.dvid = rtl8651_vlanTableIndex(entry_t->dvid);

	asic.dvid 			= rtl8651_vlanTableIndex(entry_t->dvid);
	asic.extIntIpIdx	= entry_t->ip_t? IP_TABLE_INDEX(entry_t->ip_t): 0;
	rtl8651_setAsicNextHopTable(entry_t->entryIndex,  &asic);
}






static void  _rtl8651_arrangeNextHopTable(ipaddr_t nexthop, rtl8651_tblDrv_filterDbTableEntry_t *macInfo)
{
	rtl8651_tblDrv_nextHopEntry_t *entry_t=SLIST_FIRST(&DrvTbl.inuseList.nexthop);
	rtl865x_tblAsicDrv_nextHopParam_t nxthop;
	rtl8651_tblDrv_ipEntry_t *ipEntry_t=DrvTbl.iptbl;
	uint32 entry, flag=0;


	/*
	  * If the nexthop != 0, then we only update those entries whose nexthop is
	  * the specified nexthop IP address to specified L2 info. Note that, under
	  * previous condition, if the macInfo == NULL, then we update the nexthop
	  * entry with its L2 info to the RESERVED entry.
	  */
	bzero(&nxthop, sizeof(rtl865x_tblAsicDrv_nextHopParam_t));
	if (nexthop != 0) {
		SLIST_FOREACH(entry_t, &DrvTbl.inuseList.nexthop, next) {
			if (entry_t->nexthop != nexthop)
				break;
			rtl8651_getAsicNextHopTable(entry_t->entryIndex, &nxthop);
			nxthop.nextHopRow	= (!macInfo)? _l2Reserved_: rtl8651_filterDbIndex(&macInfo->macAddr);
			nxthop.nextHopColumn	= (!macInfo)? 0: macInfo->asicPos;
			nxthop.extIntIpIdx		= (entry_t->ip_t)? IP_TABLE_INDEX(entry_t->ip_t): 0;
			rtl8651_setAsicNextHopTable(entry_t->entryIndex, &nxthop);
		}
		return;
	}


	/* Only for testing platform: make sure entries was successfully removed */
#ifdef RTL865X_TEST
	_rtl8651_delAsicEntry(TYPE_NEXT_HOP_TABLE, 1, RTL8651_NEXTHOPTBL_SIZE-1);
#endif


	/*
	  * Rearrange the nexthop table.
	  * Once this table is rearranged, the L3 table, IP table, ACL table
	  * and Napt tcp/udp table should be updated.
	  */
	  
	
	for(entry=1; entry<RTL8651_NEXTHOPTBL_SIZE&&entry_t; entry++) {
		/*
		  * If 'nexthop_t' is NULL, it means currently no L2 entry was found. The nexthop entry's nexthop Index
		  * should point to the default l2 entry we reserved before.
		  * If 'entry_t->ip_t' is NULL, it means no NAPT IP address was specified. 
		  */		
		if (entry_t->rt_flag && (flag==0) && (entry&0x01)) {
			flag = 1;
			continue;
		}
		if (!entry_t->rt_flag)
			flag = 0;
		else flag = 1;
		entry_t->entryIndex = entry; /* because the first one is reserved for unresolved nexthop */ 
		_rtl8651_updateNextHopEntry(0, entry_t);
		entry_t = SLIST_NEXT(entry_t, next);
	}

	{ /* Update L3 Table */
		rtl8651_tblDrv_routeTable_t *rt_t = DrvTbl.route;
		for(entry=0; entry<rtl8651_tblDrvPara.routingTableSize; entry++, rt_t++) {
			if (rt_t->valid && rt_t->process == RT_NEXTHOP)
				_rtl8651_updateL3Entry(rt_t);
		}
	}
	

	/* Update IP Table */
	for(entry=0; entry<RTL8651_IPTABLE_SIZE; entry++, ipEntry_t++)
		if (ipEntry_t->valid == 1)
			_rtl8651_updateIPTableEntry(ipEntry_t);

	/* Update ACL: Policy route */
	_rtl8651_arrangeAllACL();

	/* Update NAPT Entry */
	
	
}





static int32 _rtl8651_addNextHopEntry(int8 attr, void *ptr_t, rtl8651_tblDrv_networkIntfTable_t *netif_t, ipaddr_t nexthop, ipaddr_t naptip, rtl8651_tblDrv_pppTable_t *session_t)
{
	rtl8651_tblDrv_nextHopEntry_t *entry_t, *last_t=NULL;
	rtl8651_tblDrv_nextHopEntry_t **nht_tt=NULL;
	rtl8651_tblDrv_routeTable_t *rt_t=NULL;
	rtl8651_tblDrv_ipEntry_t *ip_t;
	int32 entry=0;//, retval;


	/*
	  * NOTE:
	  * (1) ETHERNET type network interface:
	  *	  If nexthop is not specfied, use default route's nexthop. If no default route's nexthop, use RESERVED entry.
	  *	  a) If naptip != 0 (naptip is specified), it means the entry is added for:
	  *		I. L3 entry's nexthop in multiple napt IP environment
	  *	       II. Naptt hash 1 static entry
	  *	  b) If naptip == 0 (naptip is un-specified), it means that the entry is added for:
	  *	         I. Local Server
	  *		II. NAT
	  *	       III. Server Port
	  *
	  * (2) PPPoE/PPTP/L2TP type network interface:
	  *	  the "nexthop" and "naptip" are meaningless. The "session_t" will explicitly specify which
	  *	  PPPoE session (PPTP/L2TP session) to be used.
	  */
	 
	assert(ptr_t && netif_t);
	if (netif_t->linkLayerType != RTL8651_LL_VLAN && netif_t->linkLayerType != RTL8651_LL_LIPUNNUMBERED) {
		if (!session_t || session_t->netif_t != netif_t)
			return FAILED;
	}
	
	
	switch(attr) {
	case NEXTHOP_NAPT_H2:
		/* 
		  * (1) If the nexthop == 0, the nexthop is the default route's nexthop. If the default route's
		  * 	nexthop has more than one nexthop in the nexthop table, we always select the first
		  * 	one as the default nexthop of NAT. If no default route's nexthop exists, the NAT's nexthop
		  *	uses a RESERVED nexthop entry temporarily.
		  * (2) If a duplicate entry was found, increase its reference count; otherwise
		  * 	add a new one.
		  */
		nht_tt = &(((rtl8651_tblDrv_ipEntry_t *)ptr_t)->nextHop_t);
		if (netif_t->linkLayerType == RTL8651_LL_VLAN ) {
			/* for ethernet-based network interface */
			if (nexthop == 0) {
				/* Try to use default route's nexthop */
				if (DrvTbl.route[rtl8651_tblDrvPara.routingTableSize-1].valid == 0) {
					((rtl8651_tblDrv_ipEntry_t *)ptr_t)->def_flag = 1;
					*nht_tt = NULL;
					return SUCCESS;
				}
				nexthop = DrvTbl.route[rtl8651_tblDrvPara.routingTableSize-1].nextHop;
				((rtl8651_tblDrv_ipEntry_t *)ptr_t)->def_flag = 1;
			}
			SLIST_FOREACH(entry_t, &DrvTbl.inuseList.nexthop, next) {
				if (entry_t->rt_flag == 1)
					continue;
				if (entry_t->nextHopType==RTL8651_LL_VLAN && entry_t->nexthop == nexthop) {
					entry_t->refcount ++;
					*nht_tt = entry_t;
					_rtl8651_updateIPTableEntry(ptr_t);
					return SUCCESS;
				}
			}
		}
		else {
			/* for session-based network interface */
			SLIST_FOREACH(entry_t, &DrvTbl.inuseList.nexthop, next) {
				if (entry_t->rt_flag == 1)
					continue;
				if ( entry_t->nextHopType!=RTL8651_LL_VLAN && entry_t->nexthop_t == session_t) {
					entry_t->refcount ++;
					*nht_tt = entry_t;
					_rtl8651_updateIPTableEntry(ptr_t);
					return SUCCESS;
				}
			}
		}
		break; /* Allocate new one */
	

	case NEXTHOP_L3:
		/* Currently, multiple nexthop is not support */
		rt_t = (rtl8651_tblDrv_routeTable_t *)ptr_t;
//		if (rt_t->rt_nhstart_t)
//			return FAILED;
		/* All entries referenced by l3 entry should be added in the tail */
		SLIST_FOREACH(entry_t, &DrvTbl.freeList.nexthop, next) 
			entry++;  
		if (entry < 2)
			return FAILED; /* Out of free entry */
		SLIST_FOREACH(entry_t, &DrvTbl.inuseList.nexthop, next)
			last_t = entry_t;
		break; /* Allocate new one */

	case NEXTHOP_POLICY:	
		nht_tt = &(((_rtl8651_tblDrvPolicy_t *)ptr_t)->nexthop_t);
		/* for ethernet-based network interface */
		SLIST_FOREACH(entry_t, &DrvTbl.inuseList.nexthop, next) {
			if (entry_t->rt_flag == 1 || entry_t->nextHopType != netif_t->linkLayerType)
				continue;
			if (netif_t->linkLayerType == RTL8651_LL_VLAN) {
				if (entry_t->nexthop == nexthop && entry_t->ip_t && entry_t->ip_t->extip == naptip) {
					entry_t->refcount ++; /* high priority: all match */
					*nht_tt = entry_t;
					return SUCCESS;
				}
			}
			else {
				assert(entry_t->nexthop_t);
				if (entry_t->nexthop_t == session_t)  {
					entry_t->refcount ++;
					*nht_tt = entry_t;
					return SUCCESS;
				}
			}
		}
		SLIST_FOREACH(entry_t, &DrvTbl.inuseList.nexthop, next) {
			if (entry_t->rt_flag == 1 ||  entry_t->nextHopType != netif_t->linkLayerType)
				continue;
			if (netif_t->linkLayerType == RTL8651_LL_VLAN) {
				if (entry_t->nexthop == nexthop && !entry_t->ip_t) {
					entry_t->ip_t = _rtl8651_getIPTableEntry(naptip, 0, IP_ATTR_NAPT);
					assert(entry_t->ip_t);
					entry_t->refcount ++; /* low priority */
					*nht_tt = entry_t;
					return SUCCESS;
				}
			}
		}
		break; /* Allocate new one */

	case NEXTHOP_NAPT_H1:
		return FAILED;
		
	default: assert(0);
	
	}


//add_Entry_:

	/* Allocate an empty entry for new one */
	if ((entry_t=SLIST_FIRST(&DrvTbl.freeList.nexthop)) == NULL) 
		return FAILED;

	SLIST_REMOVE_HEAD(&DrvTbl.freeList.nexthop, next);	
	entry_t->valid			= 1;
	entry_t->dvid 			= netif_t->vid;
	entry_t->nextHopType	= netif_t->linkLayerType;
	entry_t->refcount		= 1;
	entry_t->rt_flag		= (attr == NEXTHOP_L3) ? 1: 0;
	if (netif_t->linkLayerType == RTL8651_LL_VLAN) {
		entry_t->nexthop_t	= (void *)_rtl8651_getArpEntry(netif_t, nexthop, FALSE);
		entry_t->nexthop	= nexthop;
	} else {
		entry_t->nexthop_t	= (void *)session_t;
		entry_t->nexthop	= (session_t->session_t->ip_t)? session_t->session_t->ip_t->ipAddr: 0;
		if (attr < NEXTHOP_L3)
			naptip = 0;
		else 	naptip = entry_t->nexthop;
		/* if PPTP/L2TP, use LoopBack VID */
		if (	(netif_t->linkLayerType == RTL8651_LL_PPTP) ||
			(netif_t->linkLayerType == RTL8651_LL_L2TP))
			entry_t->dvid = tunnel.loopBackVid;
	}
	entry_t->ip_t = (naptip!=0&&attr!=NEXTHOP_NAPT_H2)? _rtl8651_getIPTableEntry(naptip, 0, IP_ATTR_NAPT): NULL;

	if (attr != NEXTHOP_L3) {
		/* entries referenced by non l3 entry should be added in the head */
		SLIST_INSERT_HEAD(&DrvTbl.inuseList.nexthop, entry_t, next);
		*nht_tt = entry_t;
	}
	else {
		/* entries referenced by l3 entry should be added in the tail */
		rt_t->rt_nhstart_t = entry_t; entry = 0;
		if (netif_t->linkLayerType == RTL8651_LL_VLAN) {
			if (naptip == 0) {
				if (!def_napt_t) {
					ip_t = NULL;
				} else ip_t = def_napt_t; 
			} else ip_t = entry_t->ip_t;
		}
		else ip_t = entry_t->ip_t;
		
		do {
			entry ++;
			if (!last_t)
				SLIST_INSERT_HEAD(&DrvTbl.inuseList.nexthop, entry_t, next);
			else 	SLIST_INSERT_AFTER(last_t, entry_t, next);
			entry_t->ip_t = ip_t;
			if (entry >= 2) break;
			last_t = entry_t;
			entry_t = SLIST_FIRST(&DrvTbl.freeList.nexthop);
			SLIST_REMOVE_HEAD(&DrvTbl.freeList.nexthop, next);
			memcpy(entry_t, last_t, sizeof(rtl8651_tblDrv_nextHopEntry_t));
		} while (1);
		rt_t->rt_nhend_t = entry_t;
	}
	
	_rtl8651_arrangeNextHopTable(0, NULL);
	return SUCCESS;
}



static int32 _rtl8651_delNextHopEntry(int8 attr, void *ptr_t)
{
	rtl8651_tblDrv_nextHopEntry_t *entry_t=NULL, *del_t, **p_t=NULL;
	rtl8651_tblDrv_routeTable_t *rt_t;
//	rtl8651_tblDrv_policyRoute_t *policy_t;
	rtl8651_tblDrv_ipEntry_t *ip_t;
	uint32 entry;

	assert(ptr_t);

	switch(attr) 
	{
	case NEXTHOP_NAPT_H2:
		ip_t = (rtl8651_tblDrv_ipEntry_t *)ptr_t;
		if ((entry_t=ip_t->nextHop_t) == NULL)
			return SUCCESS;
		if ((--entry_t->refcount) > 0) {
			ip_t->nextHop_t = NULL;
			_rtl8651_updateIPTableEntry(ptr_t);
			return SUCCESS;
		}
		p_t = &ip_t->nextHop_t ;
		break;

	case NEXTHOP_L3:
		rt_t = (rtl8651_tblDrv_routeTable_t *)ptr_t;
		del_t = rt_t->rt_nhstart_t;	
		entry = rt_t->rt_nhend_t->entryIndex - rt_t->rt_nhstart_t->entryIndex + 1;
		while (entry> 0 ) {
			entry_t = SLIST_NEXT(del_t, next);
			SLIST_REMOVE(&DrvTbl.inuseList.nexthop, del_t, rtl8651_tblDrv_nextHopEntry_s, next);
			SLIST_INSERT_HEAD(&DrvTbl.freeList.nexthop, del_t, next);
			del_t = entry_t; 
			entry--;
		}
		rt_t->rt_nhstart_t = rt_t->rt_nhend_t = NULL;
		_rtl8651_arrangeNextHopTable(0, NULL);
		return SUCCESS;

#if 0
	case NEXTHOP_POLICY:
		policy_t = (rtl8651_tblDrv_policyRoute_t *)ptr_t;
		entry_t = policy_t->nexthop_t;
		if ((--entry_t->refcount) > 0) {
			policy_t->nexthop_t = NULL;
			policy_t->naptip = 0;
			return SUCCESS;
		}
		p_t = &policy_t->nexthop_t;
		break;
#endif

	case NEXTHOP_NAPT_H1:
		return FAILED;
	
	default: assert(0);

	}

	SLIST_REMOVE(&DrvTbl.inuseList.nexthop, entry_t, rtl8651_tblDrv_nextHopEntry_s, next);
	SLIST_INSERT_HEAD(&DrvTbl.freeList.nexthop, entry_t, next);
	*p_t = NULL;
	
	_rtl8651_arrangeNextHopTable(0, NULL);	
	return SUCCESS;
}



int32 _rtl8651_addHash2ToNexthop(ipaddr_t nexthop, ipaddr_t naptip)
{
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_ipEntry_t *ipEntry_t;

	if ((netif_t = _rtl8651_getIpInfo(naptip, NULL, NULL, NULL)) == NULL)
		return FAILED;
	if (_rtl8651_isNaptMapping(naptip) == FALSE)
		return FAILED;
		
	if (netif_t->linkLayerType != RTL8651_LL_VLAN) 
		nexthop = 0;
	if ((ipEntry_t=_rtl8651_addIpEntry(naptip, nexthop, IP_ATTR_NAPT, netif_t)) == NULL)
		return FAILED;

	return (IP_TABLE_INDEX(ipEntry_t));
}


int32 _rtl8651_delHash2FromNexthop(ipaddr_t nexthop, ipaddr_t naptip)
{
	//rtl8651_tblDrv_networkIntfTable_t *netif_t;
	int32 retval;

	if (_rtl8651_isNaptMapping(naptip) == FALSE)
		return FAILED;
	retval = _rtl8651_delIpEntry(IP_ATTR_NAPT, naptip, nexthop);

	return retval;
}


int32 _rtl8651_findHash2ToNexthop(ipaddr_t nexthop, ipaddr_t naptip)
{
	rtl8651_tblDrv_ipEntry_t *ipEntry_t=DrvTbl.iptbl;
	struct rtl8651_tblDrv_nextHopEntry_s *nexthop_t;
	int32 entry;

	if (nexthop == 0) {
		if ( def_napt_t == NULL )
			return FAILED;
		nexthop = def_napt_t->nextHop_t->nexthop;
		if (nexthop == 0)
			return FAILED;
	}
	for(entry=0; entry<RTL8651_IPTABLE_SIZE; entry++, ipEntry_t++) {
		if (ipEntry_t->valid == 0 || ipEntry_t->extip!=naptip)
			continue;
		nexthop_t = ipEntry_t->nextHop_t;
		if (nexthop_t == NULL)
			continue;
		if (nexthop_t->nexthop == nexthop)
			return IP_TABLE_INDEX(ipEntry_t);
	}
	return FAILED;
}


#if 0

static int32 _rtl8651_addHash2_nexthop(ipaddr_t nexthop, ipaddr_t naptip, uint32 *ipidx)
{
	rtl8651_tblDrv_ipEntry_t *ipEntry_t = DrvTbl.iptbl;
	rtl8651_tblDrv_networkIntfTable_t *netif_t;
	rtl8651_tblDrv_pppoeSession_t *session_t;
	rtl8651_tblDrv_routeTable_t *rt_t;
	int32 entry, retval;

	/*
	  * Assumption: 
	  * If the nextohp is not specified, use the default route's nexthop as the naptip's
	  * nexthop. If no default route is present, point to a NULL but the ASIC should 
	  * point to a RESERVED nexthop entry.
	  * Note:
	  * If the network interface is a PPPoE interface, the specified naptip will implicitly imply which 
	  * nexthop to be used. Hence in such situation, if nexthop is specified, ignore it!!!
	  * Note XXXX:
	  * This function doesn't support on demand session.
	  */

	assert(naptip && ipidx);

	/* Make sure the naptip is a NAPT IP address, and then get its netwrok interface */
	if (_rtl8651_getIPTableEntry(naptip, 0, IP_ATTR_NAPT) == NULL)
		return FAILED;
	if ((netif_t=_rtl8651_getIpInfo(naptip, NULL, NULL, NULL)) == NULL)
		return FAILED;
	
	if (netif_t->linkLayerType != RTL8651_LL_VLAN) {
		/* 
		  * If the network interface is PPPoE/PPTP/L2TP type, always use specified NAPT IP address 
		  * as nexthop. And find its corresponding sessoin.
		  */
		nexthop = naptip;
		SLIST_FOREACH(session_t, &netif_t->sessionHead, nextSession) {
			if (session_t->ip_t && session_t->ip_t->ipAddr == naptip)
				break;
		}
		if (!session_t) /* no session was found */
			return FAILED;
	}
	else {
		/* 
		  * If the specified nexhtop is the default route's nexthop, force it select 
		  * section I. Otherwise, the Section II will be selected.
		  */
		rt_t = &DrvTbl.route[rtl8651_tblDrvPara.routingTableSize-1];
		if (nexthop==rt_t->rt_nhstart_t->nexthop)
			nexthop = 0;
	}


	/* 
	  * Add or Locate the IP Entry:
	  * For those nexthops which are in the same submet as WAN, we need to add another NAPT entry
	  * to IP Table.
	  */
	for(entry=0; entry<RTL8651_IPTABLE_SIZE; entry++, ipEntry_t++)
		if (ipEntry_t->type==IP_ATTR_NAPT && ipEntry_t->extip==naptip)
			break;
	if (entry == RTL8651_IPTABLE_SIZE)
		return FAILED;





	/* 
	  * Section I: 
	  * If no nexthop is specified, use default route's nexthop. If no default route is present,
	  * use RESERVED nexthop entry.
	  */
	if (nexthop == 0) {
		rt_t = &DrvTbl.route[rtl8651_tblDrvPara.routingTableSize-1];
		netif_t = def_netif_t;
		if (!ipEntry_t->nextHop_t) {
			/* If no default route is present, use RESERVED entry */
			if (IS_ENTRY_INVALID(rt_t))
				ipEntry_t->nextHop_t = NULL;
			else 	if (_rtl8651_addNextHopEntry(NEXTHOP_NAPT_H2, ipEntry_t, netif_t, rt_t->rt_nhstart_t->nexthop, 0, NULL) == FAILED)
			/* This entry is not referenced by NAPT hash 2 */
				return FAILED;			
			ipEntry_t->ref_cnt = 0;
		}
		/* Entry is referenced by NAPT hash 2, but not use default route' nexthop */
		else if (!ipEntry_t->def_flag) 
			return FAILED;
		*ipidx = entry;
		ipEntry_t->ref_cnt ++;	
		ipEntry_t->def_flag = 1;
		return SUCCESS;
	}

	/*
	  * Section II:
	  * The nexthop is specified and is not default route's nexthop.
	  */
	if (ipEntry_t->nextHop_t) {
		assert(ipEntry_t->ref_cnt > 0);
		if (nexthop != ipEntry_t->nextHop_t->nexthop)
			return FAILED; /* conflict */
		*ipidx = entry;
		ipEntry_t->def_flag = 0;
		ipEntry_t->ref_cnt ++;
		return SUCCESS;
	}

	
	if (netif_t->linkLayerType == RTL8651_LL_VLAN)
		retval = _rtl8651_addNextHopEntry(NEXTHOP_NAPT_H2, ipEntry_t, netif_t,  nexthop, 0, NULL);
	else retval = _rtl8651_addNextHopEntry(NEXTHOP_NAPT_H2, ipEntry_t, netif_t, 0, 0, session_t);
	if (retval == FAILED)
		return FAILED;
	ipEntry_t->def_flag = 0;
	ipEntry_t->ref_cnt = 1;
	*ipidx = entry;
	return SUCCESS;
}
#endif








/*=======================================
  * System Relative APIs
  *=======================================*/
#define RTL865X_SYSTEM_RELATIVE_APIs

#define LINK_UP(v)					( v&(1<<4) )
/*
@func void | rtl8651_timeUpdate | Timer for Table Drvier.
@parm uint32 | secPassed | How many seconds passed.
@comm
If table driver is used, this API should be called periodically. It will update some tables in the Table Driver.
*/
void _rtl8651_timeUpdate(uint32 secPassed) 
{
	static uint32 secCounter=0;
	
	_rtl8651_timeUpdateNaptIcmpTable(secPassed);	/* Update NAPT ICMP aging time */
	_rtl8651_softwareNaptAging(secPassed);			/* Update NAPT TCP/UDP flow aging time */
	_rtl8651_timeUpdateArp(secPassed);				/* Update ARP entry aging time */
	_rtl8651_timeUpdateSessionHangUp(secPassed);	/* Update Sessopn hang-up aging time */
	_rtl8651_timeUpdateMulticast(secPassed);				/* Update Multicast aging time */
	//_rtl8651_mnQueueTimer();
	_rtl8651_fwdEngineTimeUpdate(secPassed);		/* UPNP/DoS */

	_rtl8651_dynamicPolicyRouteTimeupdate(secPassed);	/* dynamic policy route */

	_rtl8651_ipClassTimeUpdate();
	_rtl8651_naptTimerAdaptiveAdjust(1, 10, secPassed);
	_sysUpSeconds+=secPassed;

	_rtl8651_rateLimitCounterTimeUpdate(secPassed);	/* Rate Limit Counter Time Update */

	_rtl8651_run_timer_list();
	_rtl8651_dosProc_blockSip_discharge(_sysUpSeconds);	/* Update source ip blocking list */

	_rtl8651_dns_timeUpdate(secPassed);

	/* For RTL865xB cut-c & before that. */
//	if (RtkHomeGatewayChipRevisionID <= 0x02)
	{
		/* Patch for Link Change Lost */
		if (++secCounter == 5) {
			uint32 port, value;

			for(port = 0; port < (RTL8651_PORT_NUMBER - 1); port++) {
				value = (port==4) ? READ_MEM32(PSIR1) : (READ_MEM32(PSIR0)>> (port * 8));
				if (LINK_UP(value) && (_rtl8651_portStatusPatch(port) == RTL8651_LINKPATCH_DRIVERDOWN)) 
				{
//					rtlglue_printf("%s():%d  port %d link up!\n", __FUNCTION__, __LINE__, port );
					rtl8651_setAsicEthernetLinkStatus(port, TRUE);
				}
				else if ((!LINK_UP(value)) && (_rtl8651_portStatusPatch(port) == RTL8651_LINKPATCH_DRIVERUP))
				{
//					rtlglue_printf("%s():%d port %d link down!\n", __FUNCTION__, __LINE__, port);
					rtl8651_setAsicEthernetLinkStatus(port, FALSE);
				}
			}
			secCounter = 0;
		}
	}
}



/*
@func int32		| _rtl8651_realtimeSchedule	| To provide realtime scheduling
@parm void	|	| No parameter is needed.
@rvalue void	| No return value
@comm 
To provide fast network processing, ROME driver is working in the ISR mode.
Thus, all the instrrupts, such as timer and PCM, are masked in this mode.
It means we will lose the interrupt if the network traffic is very busy.
To solve this problem, we need to check timer value before processing packet.

The procedure in this function is listed as below:
  1. check PCM interrupt
  2. compute passed time since last time executed this function
  3. check msec-routine list
  4. check second-routine list, said _rtl8651_timeUpdate().
*/
uint32 tuRemind;   /* unit is the same as TC0DATA, say "timer unit". */
uint32 accJiffies; /* accumulated jiffies, unit is the same as jiffies, 1/HZ. */
uint32 timer1Sec;  /* sysUpTime of Timer1 */
uint32 prevTC1CNT; /* Previous value of Timer 1 */

void _rtl8651_realtimeSchedule( void )
{
    uint32 jifPassed;
    uint32 secPassed;
	uint32 currTC1CNT;
	uint32 systemHZ;
	uint32 tc0data;

	systemHZ = rtl865x_getHZ();
	tc0data = READ_MEM32(TC0DATA)>>TCD_OFFSET;

#ifdef CONFIG_RTL865X_VOIP
	/* check PCM interrupt */
	voip_pollPcmInt();
#endif
#ifdef CONFIG_RTL865X_PCM
	/* check SLIC interrupt */
	_slic_PollInt();
#endif

	/* compute passed time since last time executed this function */
	currTC1CNT = READ_MEM32(TC1CNT);
	if ( prevTC1CNT >= currTC1CNT )
	{
		/* No wrap happened. */
		tuRemind += (prevTC1CNT-currTC1CNT)>>TCD_OFFSET; /* how many units are passed since last check? */
	}
	else
	{
		/* Timer1 wrapped!! */
		tuRemind += (prevTC1CNT+(0xffffff00-currTC1CNT)+(0x1<<TCD_OFFSET))>>TCD_OFFSET; /* how many units are passed since last check? */
	}
	prevTC1CNT = currTC1CNT; /* keep TC1CNT value for next time check */

	/* If tc0data is zero, it means 'time is frozen.' */
	if ( tc0data == 0 )
	{
		jifPassed = 0;
	}
	else
	{
		jifPassed = tuRemind / tc0data;
		tuRemind = tuRemind % tc0data;
	}
 
#ifdef CONFIG_RTL865X_VOIP
	/* check msec-routine list */
	if ( jifPassed > 0 ) voip_ms_routine( jifPassed * (1000/systemHZ) );
#endif

	/* check second-routine list, said _rtl8651_timeUpdate() */
    secPassed = ((accJiffies+jifPassed)/systemHZ)-(accJiffies/systemHZ);/* Count up only when acroosing HZ. */
    if ( secPassed > 0 ) _rtl8651_timeUpdate(secPassed);
    
    timer1Sec += secPassed;
    accJiffies += jifPassed;
}


void rtl8651_realtimeSchedule( void )
{
	rtlglue_drvMutexLock();//Lock resource
	_rtl8651_realtimeSchedule();
	rtlglue_drvMutexUnlock();//Unlock resource	
}

/*
 * backward compatible API.
 * use _rtl8651_realtimeSchedule() if possible.
 */
void rtl8651_timeUpdate(uint32 dummy)
{
	_rtl8651_realtimeSchedule();
}


int32 rtl8651_layer3_alloc(void){
	int32 index;

	TBL_MEM_ALLOC(AsicTableUsage.arp, rtl8651_tblDrv_arpAsicArrangementEntry_t, RTL8651_ARPARRANGE_NUM);
	TBL_MEM_ALLOC(DrvTbl.route, rtl8651_tblDrv_routeTable_t, rtl8651_tblDrvPara.routingTableSize);	
	TBL_MEM_ALLOC(DrvTbl.netIface, rtl8651_tblDrv_networkIntfTable_t, rtl8651_tblDrvPara.networkIntfTableSize);
	TBL_MEM_ALLOC(DrvTbl.pppoe, rtl8651_tblDrv_pppTable_t, (rtl8651_tblDrvPara.pptpSize+rtl8651_tblDrvPara.l2tpSize+RTL8651_PPPOETBL_SIZE));
	TBL_MEM_ALLOC(DrvTbl.iptbl, rtl8651_tblDrv_ipEntry_t, RTL8651_IPTABLE_SIZE);
	TBL_MEM_ALLOC(DrvTbl.rateLimit, rtl8651_tblDrv_rateLimit_t, RTL8651_RATELIMITTBL_SIZE);

	
	
	////TBL_MEM_ALLOC(DrvTbl.iptbl, rtl8651_tblDrv_ipEntry_t, RTL8651_IPTABLE_SIZE);

	/*==========================================================
	  * Allocate memory for MAC Access Control Table
	  *==========================================================*/
	{
		rtl8651_tblDrv_macACEntry_t *macCtrlTable;
		CTAILQ_INIT(&DrvTbl.freeList.freeMacAccessControl);
		if (rtl8651_tblDrvPara.macAccessControllSize > 0) {
			TBL_MEM_ALLOC(macCtrlTable, rtl8651_tblDrv_macACEntry_t, rtl8651_tblDrvPara.macAccessControllSize);
			for(index=0; index<rtl8651_tblDrvPara.macAccessControllSize; index++)
				CTAILQ_INSERT_HEAD(&DrvTbl.freeList.freeMacAccessControl, &macCtrlTable[index], nextMAC);
		}
		TBL_MEM_ALLOC(DrvTbl.macACInfo, rtl8651_tblDrv_macContrl_t, 1);
	}


	/*==========================================================
	  * Allocate memory for Multicast table
	  *==========================================================*/
	{
		rtl8651_tblDrv_mCast_t *multiCast_t;
		TBL_MEM_ALLOC(multiCast_t, rtl8651_tblDrv_mCast_t ,rtl8651_tblDrvPara.mCastTblSize);
		TAILQ_INIT(&DrvTbl.freeList.freeMultiCast);
		for(index=0; index<rtl8651_tblDrvPara.mCastTblSize; index++)
			TAILQ_INSERT_HEAD(&DrvTbl.freeList.freeMultiCast, &multiCast_t[index], nextMCast);

		TBL_MEM_ALLOC(multiCast_t, rtl8651_tblDrv_mCast_t, RTL8651_MULTICASTTBL_SIZE);
		DrvTbl.inuseList.mCastTbl = (void *)multiCast_t;
	}

	/*==========================================================
	  * Allocate memory for Session-Based connection
	  *==========================================================*/
	{
		rtl8651_tblDrvSession_t *session_t;
		TBL_MEM_ALLOC(session_t, rtl8651_tblDrvSession_t, rtl8651_tblDrvPara.dialSessionNumber);
		TAILQ_INIT(&DrvTbl.freeList.freeSession);
		for(index=0; index<rtl8651_tblDrvPara.dialSessionNumber; index++)
			TAILQ_INSERT_HEAD(&DrvTbl.freeList.freeSession, &session_t[index], nextSession);
	}
	/*==========================================================
	  * Allocate memory for Static Policy route and Demand route
	  *==========================================================*/
	{
		_rtl8651_tblDrvPolicy_t *policy_t;
		TBL_MEM_ALLOC(policy_t, _rtl8651_tblDrvPolicy_t, rtl8651_tblDrvPara.policyRules);
		TAILQ_INIT(&DrvTbl.freeList.freePolicy);
		for(index=0; index<rtl8651_tblDrvPara.policyRules; index++)
			TAILQ_INSERT_HEAD(&DrvTbl.freeList.freePolicy, &policy_t[index], nextPolicy_t);
	}
	/*==========================================================
	  * Allocate memory for Dynamic Policy route and Demand route
	  *==========================================================*/
	{

		TAILQ_HEAD(tailqHead, _rtl8651_tblDrvPolicy_s);
		TAILQ_HEAD(hashHead, _rtl8651_tblDrvPolicy_s);
		struct hashHead *tempPtr;
		
		TBL_MEM_ALLOC(tempPtr, struct hashHead , rtl8651_tblDrvPara.dynamicPolicy); //warning: using "void" replace "struct tailqHead".
		DrvTbl.dynamicPolicy.hashTable = (void *)tempPtr;			
		TBL_MEM_ALLOC(DrvTbl.dynamicPolicy.policy_t, _rtl8651_tblDrvPolicy_t, rtl8651_tblDrvPara.dynamicPolicy);
		
				
	}
	/*==========================================================
	  * Allocate memory for ACL entry
	  *==========================================================*/
	{
		int32 i;
		_rtl8651_tblDrvAclRule_t *aclRule;
		TBL_MEM_ALLOC(aclRule,_rtl8651_tblDrvAclRule_t, rtl8651_tblDrvPara.aclSize);
		CTAILQ_INIT(&DrvTbl.freeList.aclRule);
		for(i=0; i<rtl8651_tblDrvPara.aclSize; i++)
			CTAILQ_INSERT_HEAD(&DrvTbl.freeList.aclRule, &aclRule[i], nextRule);
	}
	/*==========================================================
	  * Allocate memory for Network Interface
	  *==========================================================*/
	{
		rtl8651_tblDrv_ipIntfEntry_t * tempIpIntf;
		TBL_MEM_ALLOC(tempIpIntf, rtl8651_tblDrv_ipIntfEntry_t, rtl8651_tblDrvPara.ipIntfEntryNumber);
		SLIST_INIT(&DrvTbl.freeList.ipIface);
		for(index=0; index<rtl8651_tblDrvPara.ipIntfEntryNumber; index++){
			SLIST_INIT(&tempIpIntf[index].ipAddr);
			SLIST_INIT(&tempIpIntf[index].localServer);
			SLIST_INIT(&tempIpIntf[index].nextArp);
			SLIST_INSERT_HEAD(&DrvTbl.freeList.ipIface, &tempIpIntf[index], nextIp);
		}
	}

	/*==========================================================
	  * Allocate memory for IP address list
	  *==========================================================*/
	{
		rtl8651_tblDrv_ipIntfIpAddrEntry_t * tempIpAddr;
		TBL_MEM_ALLOC(tempIpAddr, rtl8651_tblDrv_ipIntfIpAddrEntry_t, rtl8651_tblDrvPara.ipIntfIpAddrNumber);
		SLIST_INIT(&DrvTbl.freeList.ipAddr);
		for(index=0; index<rtl8651_tblDrvPara.ipIntfIpAddrNumber; index++)
			SLIST_INSERT_HEAD(&DrvTbl.freeList.ipAddr, &tempIpAddr[index], nextIpAddr);
	}
	/*==========================================================
	  * Allocate memory for ARP entry
	  *==========================================================*/
	{
		rtl8651_tblDrv_arpEntry_t * tempArp;
		TBL_MEM_ALLOC(tempArp, rtl8651_tblDrv_arpEntry_t, rtl8651_tblDrvPara.arpTableSize);
		CSLIST_INIT(&DrvTbl.freeList.arp);
		for(index=0; index<rtl8651_tblDrvPara.arpTableSize; index++)
			CSLIST_INSERT_HEAD(&DrvTbl.freeList.arp, &tempArp[index], next);
	}
	

	/*==========================================================
	  * Allocate memory for Protocol Based NAT
	  *==========================================================*/
	{
		rtl8651_tblDrv_protocolBasedNat_t *pbNat;
		TBL_MEM_ALLOC(pbNat, rtl8651_tblDrv_protocolBasedNat_t, rtl8651_tblDrvPara.protocolBasedNatSize);
		SLIST_INIT(&DrvTbl.inuseList.protocolBasedNat);
		SLIST_INIT(&DrvTbl.freeList.protocolBasedNat);
		for(index=0;index<rtl8651_tblDrvPara.protocolBasedNatSize;index++)
		{
			SLIST_INSERT_HEAD(&DrvTbl.freeList.protocolBasedNat, &pbNat[index], next);
		}
	}

	/*==========================================================
	  * Allocate memory for Next Hop Table
	  *==========================================================*/
	{
		rtl8651_tblDrv_nextHopEntry_t *tmp_t;
		SLIST_INIT(&DrvTbl.freeList.nexthop);
		TBL_MEM_ALLOC(tmp_t, rtl8651_tblDrv_nextHopEntry_t, RTL8651_NEXTHOPTBL_SIZE);
		for(index=0; index<RTL8651_NEXTHOPTBL_SIZE; index++)
			SLIST_INSERT_HEAD(&DrvTbl.freeList.nexthop, &tmp_t[index], next);
	}

	/*==========================================================
	  * Allocate memory for IP unnumbered Table
	  *==========================================================*/
	{
		rtl8651_tblDrv_ipUnnumbered_t *tmp_t;
		SLIST_INIT(&DrvTbl.freeList.freeIpUnnumber);
		SLIST_INIT(&DrvTbl.inuseList.ipUnnumber);
		TBL_MEM_ALLOC(tmp_t, rtl8651_tblDrv_ipUnnumbered_t, rtl8651_tblDrvPara.ipUnnumberSize);
		for(index=0; index<rtl8651_tblDrvPara.ipUnnumberSize; index++)
			SLIST_INSERT_HEAD(&DrvTbl.freeList.freeIpUnnumber, &tmp_t[index], nextIpUnnumberEntry);
	}

	
#ifdef CONFIG_RTL865X_IPSEC
#define	SPIUNIT (2 /* {ESP+IPIP} */ *2 /* bi-direction */ *2 /*  for_exchange_key */ * 4 /*reliability factor*/)
#define	SPIGRPUNIT (SPIUNIT / 2 /* don't distiguish ESP/IPIP */)
#define	EROUTEUNIT (SPIGRPUNIT)
	_rtl8651_allocIpsec(	rtl8651_tblDrvPara.ipSecTunnelNumber * SPIUNIT,
						rtl8651_tblDrvPara.ipSecTunnelNumber * SPIGRPUNIT,
						rtl8651_tblDrvPara.ipSecTunnelNumber * EROUTEUNIT);
#endif	

	return SUCCESS;
	

	#if 0 //Legacy??? Don't enable until we return to 8651B development...
	{//Allocate Software Policy Route entry
		rtl8651_tblDrv_policyRoute_t *tmp_t;
		TBL_MEM_ALLOC(tmp_t, rtl8651_tblDrv_policyRoute_t, rtl8651_tblDrvPara.policyRules);
		SLIST_INIT(&DrvTbl.freeList.policy);
		for(index=0; index<rtl8651_tblDrvPara.policyRules; index++)
			SLIST_INSERT_HEAD(&DrvTbl.freeList.policy, &tmp_t[index], next);
	}
	#endif
	return SUCCESS;
}


int32 rtl8651_layer3_collect(void) {
	uint32 entry, index;


	/*==========================================================
	  * Collect MAC Control Access Table
	  *==========================================================*/
	{
		rtl8651_tblDrv_macACEntry_t *macCtrl;
		while ((macCtrl=CTAILQ_FIRST(&DrvTbl.macACInfo->macAccessControl))) {
			CTAILQ_REMOVE(&DrvTbl.macACInfo->macAccessControl, macCtrl, nextMAC);
			CTAILQ_INSERT_HEAD(&DrvTbl.freeList.freeMacAccessControl, macCtrl, nextMAC);
		}
		assert(CTAILQ_TOTAL(&DrvTbl.freeList.freeMacAccessControl) == rtl8651_tblDrvPara.macAccessControllSize);
	}
	 

	/*==========================================================
	  * Collect Multicast entry
	  *==========================================================*/
	{
		rtl8651_tblDrv_mCast_t *mCast_t;
		for(entry=0; entry<rtl8651_tblDrvPara.mCastTblSize; entry++) {
			while ((mCast_t=TAILQ_FIRST(&DrvTbl.inuseList.mCastTbl[entry]))) {
				TAILQ_REMOVE(&DrvTbl.inuseList.mCastTbl[entry], mCast_t, nextMCast);
				TAILQ_INSERT_HEAD(&DrvTbl.freeList.freeMultiCast, mCast_t, nextMCast);
			}
		}
		entry = 0;
		TAILQ_FOREACH(mCast_t, &DrvTbl.freeList.freeMultiCast, nextMCast)
			entry ++;
		assert(entry == rtl8651_tblDrvPara.mCastTblSize);
	}

	  
	/*==========================================================
	  * Collect Static Policy route and Demand route
	  *==========================================================*/
	{
		rtl8651_tblDrvSession_t *session_t;
		rtl8651_tblDrv_networkIntfTable_t *netif_t=DrvTbl.netIface;
		for(entry=0; entry<rtl8651_tblDrvPara.networkIntfTableSize; entry++, netif_t++) {
			if (netif_t->valid == 0)
				continue;
			while((session_t=CTAILQ_FIRST(&netif_t->sessionHead))) {
				CTAILQ_REMOVE(&netif_t->sessionHead, session_t, nextSession);
				TAILQ_INSERT_HEAD(&DrvTbl.freeList.freeSession, session_t, nextSession);
			}
		}
		entry = 0;
		TAILQ_FOREACH(session_t, &DrvTbl.freeList.freeSession, nextSession)
			entry ++;
		assert(entry == rtl8651_tblDrvPara.dialSessionNumber);
	}
	/*==========================================================
	  * Collect Static Policy route and Demand route
	  *==========================================================*/
	{
		_rtl8651_tblDrvPolicy_t *policy_t;
		rtl8651_tblDrv_networkIntfTable_t *netif_t=DrvTbl.netIface;
		while((policy_t=TAILQ_FIRST(&DrvTbl.inuseList.policyRoute))) {
			TAILQ_REMOVE(&DrvTbl.inuseList.policyRoute, policy_t, nextPolicy_t);
			TAILQ_INSERT_HEAD(&DrvTbl.freeList.freePolicy, policy_t, nextPolicy_t);
		}
		for(entry=0; entry< rtl8651_tblDrvPara.networkIntfTableSize; entry++, netif_t++) {
			if (netif_t->valid == 0)
				continue;
			while((policy_t=CTAILQ_FIRST(&netif_t->policyQosHead))) {
				CTAILQ_REMOVE(&netif_t->policyQosHead, policy_t, nextPolicy_t);
				TAILQ_INSERT_HEAD(&DrvTbl.freeList.freePolicy, policy_t, nextPolicy_t);
			}
		}
		entry = 0;
		TAILQ_FOREACH(policy_t, &DrvTbl.freeList.freePolicy, nextPolicy_t)
			entry ++;
		assert(entry == rtl8651_tblDrvPara.policyRules);
	}
	
	{//Collect ACL rule
		_rtl8651_tblDrvAclRule_t *rule;
		rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
		int32 i;
		for(i=0,netIfPtr=&DrvTbl.netIface[0]; i< rtl8651_tblDrvPara.networkIntfTableSize; i++,netIfPtr++)
			if (netIfPtr->valid == 1) {
				/* Ingress rule */
				while ((rule=CTAILQ_FIRST(&netIfPtr->inAclHead))) {
					CTAILQ_REMOVE(&netIfPtr->inAclHead, rule, nextRule);
					CTAILQ_INSERT_HEAD(&DrvTbl.freeList.aclRule, rule, nextRule);
				}
				/* Egress rule */
				while ((rule=CTAILQ_FIRST(&netIfPtr->outAclHead))) {
					CTAILQ_REMOVE(&netIfPtr->outAclHead, rule, nextRule);
					CTAILQ_INSERT_HEAD(&DrvTbl.freeList.aclRule, rule, nextRule);
				}
				/* Ingress Rate Limit */
				while ((rule=CTAILQ_FIRST(&netIfPtr->inRateLimitHead))) {
					CTAILQ_REMOVE(&netIfPtr->inRateLimitHead, rule, nextRule);
					CTAILQ_INSERT_HEAD(&DrvTbl.freeList.aclRule, rule, nextRule);
				}
				/* Egress Rate Limit */
				while ((rule = CTAILQ_FIRST(&netIfPtr->outRateLimitHead))) {
					CTAILQ_REMOVE(&netIfPtr->outRateLimitHead, rule, nextRule);
					CTAILQ_INSERT_HEAD(&DrvTbl.freeList.aclRule, rule, nextRule);
				}

				/* MN Queue */
				while ((rule=CTAILQ_FIRST(&netIfPtr->mnqHead))) {
					CTAILQ_REMOVE(&netIfPtr->mnqHead, rule, nextRule);
					CTAILQ_INSERT_HEAD(&DrvTbl.freeList.aclRule, rule, nextRule);
				}				
			}
		assert(CTAILQ_TOTAL(&DrvTbl.freeList.aclRule) == rtl8651_tblDrvPara.aclSize);
	}

	{//Collect DrvTbl.freeList.ipAddr, DrvTbl.freeList.arp and then DrvTbl.freeList.ipIface, DrvTbl.freeList.ipAcl
		rtl8651_tblDrv_ipIntfEntry_t * ipIntfPtr;
		rtl8651_tblDrv_ipIntfIpAddrEntry_t * ipIntfIpPtr;
		rtl8651_tblDrv_arpEntry_t * tempArpPtr;
		rtl8651_tblDrv_networkIntfTable_t *netIfPtr;

		
		for(index=0,netIfPtr=&DrvTbl.netIface[0]; index< rtl8651_tblDrvPara.networkIntfTableSize; index++,netIfPtr++) {
			if(netIfPtr->valid == 1) {//Interface valid, network interface is meaningful
				while (SLIST_FIRST(&(netIfPtr->IpHead))) {
					/* Collect IP interface entry */
					ipIntfPtr = SLIST_FIRST(&(netIfPtr->IpHead));
					SLIST_REMOVE_HEAD(&(netIfPtr->IpHead), nextIp);
					SLIST_INSERT_HEAD(&DrvTbl.freeList.ipIface, ipIntfPtr, nextIp);	
					/* Collect IP address entry */
					while(SLIST_FIRST(&ipIntfPtr->ipAddr)) {
						ipIntfIpPtr = SLIST_FIRST(&ipIntfPtr->ipAddr);
						SLIST_REMOVE_HEAD(&(ipIntfPtr->ipAddr), nextIpAddr);
						SLIST_INSERT_HEAD(&DrvTbl.freeList.ipAddr, ipIntfIpPtr, nextIpAddr);
					}
					/* Collect Local Server entry */
					while(SLIST_FIRST(&(ipIntfPtr->localServer))) {
						ipIntfIpPtr = SLIST_FIRST(&(ipIntfPtr->localServer));
						SLIST_REMOVE_HEAD(&(ipIntfPtr->localServer), nextIpAddr);
						SLIST_INSERT_HEAD(&DrvTbl.freeList.ipAddr, ipIntfIpPtr, nextIpAddr);
					}
					/* Collect ARP entry */
					while((tempArpPtr=SLIST_FIRST(&ipIntfPtr->nextArp))) {
						SLIST_REMOVE_HEAD(&ipIntfPtr->nextArp, next);
						CSLIST_INSERT_HEAD(&DrvTbl.freeList.arp, tempArpPtr, next);
					}
				}
			}
		}
		entry = 0;
		SLIST_FOREACH(ipIntfPtr, &DrvTbl.freeList.ipIface, nextIp) 
			entry ++;
		SLIST_FOREACH(ipIntfIpPtr, &DrvTbl.freeList.ipAddr, nextIpAddr) 
			entry ++;
		assert(entry == (rtl8651_tblDrvPara.ipIntfEntryNumber+rtl8651_tblDrvPara.ipIntfIpAddrNumber));
		assert(CSLIST_TOTAL(&DrvTbl.freeList.arp) == rtl8651_tblDrvPara.arpTableSize);
	}

	
	/*==========================================================
	  * Collect for Protocol Based NAT
	  *==========================================================*/
	{
		rtl8651_tblDrv_protocolBasedNat_t *pbNat;

		while( (pbNat = SLIST_FIRST(&DrvTbl.inuseList.protocolBasedNat))!=NULL )
		{
			SLIST_REMOVE_HEAD(&DrvTbl.inuseList.protocolBasedNat, next);
			SLIST_INSERT_HEAD(&DrvTbl.freeList.protocolBasedNat, pbNat, next);
		}
		entry = 0;
		SLIST_FOREACH(pbNat, &DrvTbl.freeList.protocolBasedNat, next) 
			entry++;
		assert(entry == rtl8651_tblDrvPara.protocolBasedNatSize);
	}

	/*==========================================================
	  * Collect Next Hop Table entry
	  *==========================================================*/
	{
		rtl8651_tblDrv_nextHopEntry_t *tmp_t;
		while((tmp_t=SLIST_FIRST(&DrvTbl.inuseList.nexthop))) {
			SLIST_REMOVE_HEAD(&DrvTbl.inuseList.nexthop, next);
			SLIST_INSERT_HEAD(&DrvTbl.freeList.nexthop, tmp_t, next);
		}
		entry = 0;
		SLIST_FOREACH(tmp_t, &DrvTbl.freeList.nexthop, next)
			entry ++;
		assert(entry == RTL8651_NEXTHOPTBL_SIZE);
	}

	/*==========================================================
	  * Collect for IP unnumbered table
	  *==========================================================*/
	{
		rtl8651_tblDrv_ipUnnumbered_t *ipUnnumber;

		while( (ipUnnumber = SLIST_FIRST(&DrvTbl.inuseList.ipUnnumber))!=NULL )
		{
			SLIST_REMOVE_HEAD(&DrvTbl.inuseList.ipUnnumber, nextIpUnnumberEntry);
			SLIST_INSERT_HEAD(&DrvTbl.freeList.freeIpUnnumber, ipUnnumber, nextIpUnnumberEntry);
		}
		entry = 0;
		SLIST_FOREACH(ipUnnumber, &DrvTbl.freeList.freeIpUnnumber, nextIpUnnumberEntry) 
			entry++;
		assert(entry == rtl8651_tblDrvPara.ipUnnumberSize);
	}

#if 0 //Legacy??? Don't enable until we return to 8651B development...
	{//Collect Software Policy route entry
		rtl8651_tblDrv_policyRoute_t *tmp_t;
		while((tmp_t=SLIST_FIRST(&DrvTbl.inuseList.policy))) {
			SLIST_REMOVE_HEAD(&DrvTbl.inuseList.policy, next);
			SLIST_INSERT_HEAD(&DrvTbl.freeList.policy, tmp_t, next);
		}
		while((tmp_t=SLIST_FIRST(&DrvTbl.inuseList.d_policy))) {
			SLIST_REMOVE_HEAD(&DrvTbl.inuseList.d_policy, next);
			SLIST_INSERT_HEAD(&DrvTbl.freeList.policy, tmp_t, next);
		}
		entry = 0;
		SLIST_FOREACH(tmp_t, &DrvTbl.freeList.policy, next)
			entry++;
		assert(entry == rtl8651_tblDrvPara.policyRules);
	}
#endif	
	return SUCCESS;	
}


//Initialization all Layer3 tables and linked lists. For software reset purpose.
int32 rtl8651_layer3_init(void)
{
	uint32	entry=0,i, netIdx;
	rtl8651_tblDrv_networkIntfTable_t *netIfPtr;
	/////////////Initialize L3 static variables.//////////////////

	pureSoftFwd = FALSE; //If this function is called. Default turned on ASIC L3/4 acceleration
	for(i=0; i< RTL8651_IPTABLE_SIZE; i++)
		_gidxRegister[i]=-1;
	//Initialize software based policy route
	for (i=0;i<DYNAMICPOLICYSIZE-1;i++){
		rtDynamicPolicy[i].timenext=(_rtl8651_tblDrvPolicyRoute_t *)&rtDynamicPolicy[i+1];
		rtDynamicPolicy[i+1].timeprev=(_rtl8651_tblDrvPolicyRoute_t *)&rtDynamicPolicy[i];
	}
	rtDynamicPolicy[0].timeprev=(_rtl8651_tblDrvPolicyRoute_t *)&rtDynamicPolicy[DYNAMICPOLICYSIZE-1];
	rtDynamicPolicy[DYNAMICPOLICYSIZE-1].timenext=(_rtl8651_tblDrvPolicyRoute_t *)&rtDynamicPolicy[0];
	rtDynamicPolicyHeadPtr=(_rtl8651_tblDrvPolicyRoute_t *)&rtDynamicPolicy[0];
	lanSideIpUnnumberedWan = NULL;
	lanSideIpUnnumberedLan = NULL;
	rtl8651_ipUnnumberGlobalCtrlBlock.ip_unnumber_ACL_budget = IP_UNNUMBER_MAX_ASIC_ACL_PATCH;
	rtl8651_ipUnnumberGlobalCtrlBlock.ip_unnumber_LS_budget = IP_UNNUMBER_MAX_ASIC_IP_ENTRY;
	naptIpCount = localServerCount = natIpCount = 0;
	aclMode = ACL_MODE_NONE; 
	rtEntry = 0;
	arpAgingTime = 30;//300;
	// reset _rtl8651_multiple_dsid
	_rtl8651_multiple_dsid = 0;
	_rtl8651_pppoe_count = 0;



	/////////////Initialize L3 allocated arrays. //////////////////

	memset((int8 *)DrvTbl.iptbl, 0, sizeof(rtl8651_tblDrv_ipEntry_t)*RTL8651_IPTABLE_SIZE);
	//Initial Arp Arrangement table
	memset((int8 *)AsicTableUsage.arp, 0, RTL8651_ARPARRANGE_NUM*sizeof(rtl8651_tblDrv_arpAsicArrangementEntry_t));
	//Initial IP interface
	memset((int8 *)DrvTbl.netIface, 0,  rtl8651_tblDrvPara.networkIntfTableSize*sizeof(rtl8651_tblDrv_networkIntfTable_t));	
	for(netIdx=0,netIfPtr=DrvTbl.netIface; netIdx< rtl8651_tblDrvPara.networkIntfTableSize; netIdx++, netIfPtr++){
		SLIST_INIT(&(netIfPtr->IpHead));
		SLIST_INIT(&(netIfPtr->nextNat));
		SLIST_INIT(&(netIfPtr->nextNaptServerPort));
		CTAILQ_INIT(&netIfPtr->inAclHead);
		CTAILQ_INIT(&netIfPtr->outAclHead);
	}
	//Initial routing table
	memset((int8 *)DrvTbl.route, 0, rtl8651_tblDrvPara.routingTableSize*sizeof(rtl8651_tblDrv_routeTable_t));

	/* IP Class */
	ipClassCount = 0;
	cur_rt_t = &DrvTbl.route[RTL8651_ROUTINGTBL_SIZE-2];
	memset(ipClassABMonitorTable, 0,sizeof(ipClassABMonitorTable_t)*IP_CLASS_MAX_ENTRY);
	_rtl8651_dsid = 0;

	/* Init MAC Accession Control */
	DrvTbl.macACInfo->def_permit 	= TRUE;
	DrvTbl.macACInfo->enable	 	= FALSE;
	CTAILQ_INIT(&DrvTbl.macACInfo->macAccessControl);
	
	/* Reserved ACL rules */
	RTL8651_SET_DEFAULT_ACL_DROPALL(RTL8651_ACLTBL_DROP_ALL);
	RTL8651_SET_DEFAULT_ACL_TOCPU(RTL8651_ACLTBL_ALL_TO_CPU);
	RTL8651_SET_DEFAULT_ACL_PERMITALL(RTL8651_ACLTBL_PERMIT_ALL);

	/* Install L2 entry changed notify funciton between L2 and  L3/4 driver */	
	rtl8651_installPortStatusChangeNotifier(rtl8651_updatePortStatus);
	rtl8651_installFDBEntryChangeNotifier(_rtl8651_syncL2Entrychange);

	/* Init Rate Limit Counter */
	memset(rateLimitCounterRegisterTable, 0, sizeof(rtl8651_tblDrv_rateLimitCounter_t)*RATELIMIT_COUNTER_RESV_SIZE);

	bzero(&tunnel, sizeof(rtl8651_tblDrv_miiTunneling_t));

#if 0 //Legacy?? don't turn these on until we go back to 8651B development.
	{//Initial IP table
		memset((int8 *)DrvTbl.iptbl, 0, RTL8651_IPTABLE_SIZE*sizeof(rtl8651_tblDrv_ipEntry_t));
		for(entry=0; entry<RTL8651_IPTABLE_SIZE; entry++)
			if (ipEntryMask & (1<<entry))
				DrvTbl.iptbl[entry].v = 1;
			else DrvTbl.iptbl[entry].v = 0;
	}
	{ //Initial Next Hop Table
		memset((int8 *)TBL_nexthop, 0, RTL8651_NEXTHOPTBL_SIZE*sizeof(rtl8651_tblDrv_nextHopEntry_t));	
		SLIST_INIT(&DrvTbl.inuseList.nexthop);
		
	}
	memset(&policyEntry, 0, sizeof(rtl8651_tblDrv_routeTable_t));
	//Initial Software Policy Route entry
	SLIST_INIT(&DrvTbl.inuseList.policy);
	SLIST_INIT(&DrvTbl.inuseList.d_policy);
	policyEntry.valid = policyEntry.v = 1;
	policyEntry.process = RT_POLICY;
	def_netif_t = NULL;
#endif	

	/* Init multicast table */
	for(entry=0; entry<rtl8651_tblDrvPara.mCastTblSize; entry++)
		TAILQ_INIT(&DrvTbl.inuseList.mCastTbl[entry]);
	
	/* Init PPTP , L2TP and PPPoE Table */
	bzero(DrvTbl.pppoe, (sizeof(rtl8651_tblDrv_pppTable_t)*(rtl8651_tblDrvPara.pptpSize+rtl8651_tblDrvPara.l2tpSize+RTL8651_PPPOETBL_SIZE)));
	DrvTbl.pptp = &DrvTbl.pppoe[RTL8651_PPPOETBL_SIZE];
	DrvTbl.l2tp = &DrvTbl.pppoe[RTL8651_PPPOETBL_SIZE + rtl8651_tblDrvPara.pptpSize];
	
	/*======================================================
	  * Initial Dynamic policy route
	  *======================================================*/
	{

		uint32 entry;
		bzero(DrvTbl.dynamicPolicy.hashTable, sizeof(_rtl8651_tblDrvPolicy_t *)*rtl8651_tblDrvPara.dynamicPolicy);
		bzero(DrvTbl.dynamicPolicy.policy_t, sizeof(_rtl8651_tblDrvPolicy_t)*rtl8651_tblDrvPara.dynamicPolicy);
		CIRCLEQ_INIT(&DrvTbl.dynamicPolicy.dyHead);
		for(entry=0; entry<rtl8651_tblDrvPara.dynamicPolicy; entry++) 
			CIRCLEQ_INSERT_HEAD(&DrvTbl.dynamicPolicy.dyHead, &DrvTbl.dynamicPolicy.policy_t[entry], nextTime_t);
	}

	/*=============================
	  * Initial Policy route and Demand route
	  *=============================*/
	TAILQ_INIT(&DrvTbl.inuseList.policyRoute);
	//TAILQ_INIT(&DrvTbl.inuseList.policyQos);
	{ /* Initial Policy route */
	
		bzero(rtStaticPolicy, sizeof(_rtl8651_tblDrvPolicyRoute_t)*STATICPOLICYSIZE);
		bzero(rtDemandRouteTable, sizeof(rtl8651_tblDrvDemandRoute_t) * STATICPOLICYSIZE);
		bzero(rtDynamicPolicy, sizeof(_rtl8651_tblDrvPolicyRoute_t)*DYNAMICPOLICYSIZE);
		bzero(rtHashTable, sizeof(_rtl8651_tblDrvPolicyRoute_t *)*DYNAMICPOLICYSIZE);
		for (i=0;i<DYNAMICPOLICYSIZE-1;i++)
		{
			rtDynamicPolicy[i].timenext=(_rtl8651_tblDrvPolicyRoute_t *)&rtDynamicPolicy[i+1];
			rtDynamicPolicy[i+1].timeprev=(_rtl8651_tblDrvPolicyRoute_t *)&rtDynamicPolicy[i];
		}
		rtDynamicPolicy[0].timeprev=(_rtl8651_tblDrvPolicyRoute_t *)&rtDynamicPolicy[DYNAMICPOLICYSIZE-1];
		rtDynamicPolicy[DYNAMICPOLICYSIZE-1].timenext=(_rtl8651_tblDrvPolicyRoute_t *)&rtDynamicPolicy[0];
		rtDynamicPolicyHeadPtr=(_rtl8651_tblDrvPolicyRoute_t *)&rtDynamicPolicy[0];
	}

	/*===========================================
	  * Initial Rate Limit Table
	  *===========================================*/
	{
		rtl8651_tblDrv_rateLimit_t *rl_t = DrvTbl.rateLimit;
		for(entry=0; entry<RTL8651_RATELIMITTBL_SIZE; entry++, rl_t++)
		{
			rl_t->groupId = 0;
			rl_t->queueType = 0;
		}
	}


	/*=============================
	  * Initial VLAN for Passthru
	  *=============================*/
	rtl8651_gVlanIDforPassthru = RTL8651_PASSTHRU_VLANID_IS_NOT_SET;   /* the VLAN created for passthru */
	rtl8651_gRefCountofVlanforPassthru = 0; /* reference count of this VLAN */

	/*=============================
	  * Initial PPPoE Passthru
	  *=============================*/
	rtl8651_l2protoPassThrough &= ~PASSTHRU_PPPOE;
	rtl8651_drvDropUnknownPppoePADT = FALSE;
	memset(&rtl8651_knownPppoePassthruEthHdr,0,sizeof(struct ether_header));
#if 1//CONFIG_RTL865XB_EXP_PPPOE_PASSTHRU
	// clear PPPoE Passthru VLAN index
	rtl8651_gPPPoEExternalVlanIndex = RTL8651_VLAN_NUMBER;
	memset(&rtl8651_gPPPoEExternalGatewayMAC,0,sizeof(rtl8651_gPPPoEExternalGatewayMAC));
	rtl8651_gPPPoEPassthruVlanID = RTL8651_PPPOEPASSTHRU_VLANID_IS_NOT_SET;
#else//CONFIG_RTL865XB
#endif//CONFIG_RTL865XB
	// force to insert 0x8863 trapping rule
	_rtl8651_arrangeAllACL();

	/*=============================
	  * Initial IPv6 Passthru
	  *=============================*/
#if 1//CONFIG_RTL865XB_EXP_IPV6_PASSTHRU
	rtl8651_l2protoPassThrough &= ~PASSTHRU_IPV6;
	// clear IPv6 Passthru VLAN index
	rtl8651_gIPv6ExternalVlanIndex = RTL8651_VLAN_NUMBER;
	memset(&rtl8651_gIPv6ExternalGatewayMAC, 0, sizeof(rtl8651_gIPv6ExternalGatewayMAC));
	rtl8651_gIPv6PassthruVlanID = RTL8651_IPV6PASSTHRU_VLANID_IS_NOT_SET;
	rtl8651_pbv_ipv6passthru = RTL8651_IPV6PASSTHRU_PBVID_IS_NOT_SET;
#endif//CONFIG_RTL865XB

#if 1//CONFIG_RTL865XB_EXP_IPX_PASSTHRU
	rtl8651_l2protoPassThrough &= ~PASSTHRU_IPX;
	// clear IPX Passthru VLAN index
	rtl8651_gIPXExternalVlanIndex = RTL8651_VLAN_NUMBER;
	memset(&rtl8651_gIPXExternalGatewayMAC, 0, sizeof(rtl8651_gIPXExternalGatewayMAC));
	rtl8651_gIPXPassthruVlanID = RTL8651_IPXPASSTHRU_VLANID_IS_NOT_SET;
	rtl8651_pbv_ipxpassthru = RTL8651_PBV_RULE_IPX;
#endif//CONFIG_RTL865XB

#if 1//CONFIG_RTL865XB_EXP_NETBIOS_PASSTHRU
	rtl8651_l2protoPassThrough &= ~PASSTHRU_NETBIOS;
	// clear Netbios Passthru VLAN index
	rtl8651_gNetbiosExternalVlanIndex = RTL8651_VLAN_NUMBER;
	memset(&rtl8651_gNetbiosExternalGatewayMAC, 0, sizeof(rtl8651_gNetbiosExternalGatewayMAC));
	rtl8651_gNetbiosPassthruVlanID = RTL8651_NETBIOSPASSTHRU_VLANID_IS_NOT_SET;
	rtl8651_pbv_netbiospassthru = RTL8651_PBV_RULE_NETBIOS;
#endif//CONFIG_RTL865XB

	/*=============================
	  * Initial Protocol-Based NAT
	  *=============================*/
	rtl8651_drvProtocolBasedNatEnable = FALSE;

	p_mtuHandler = NULL;
	
	/* Set default Routing entry */
	_rtl8651_updateDefaultRoute();

	/* Initial nexthop table */
	_rtl8651_init_nexthopEntry(0);

#ifdef CONFIG_RTL865X_IPSEC
	_rtl8651_initIpsec();
#endif	

	/* init dial-up-related array */
	memset( rtl8651_sessionHangUp, 0, sizeof( rtl8651_sessionHangUp ) );
	memset( &rtl8651AsicAge, 0, sizeof( rtl8651AsicAge ) );
	memset( &rtl8651GlobalControl, 0, sizeof( rtl8651GlobalControl ) );

	return SUCCESS;
}



int32  rtl8651_layer4_alloc(void){
	int32 i;

	////TBL_MEM_ALLOC(TBL_protoTrap, rtl8651_tblDrv_protoTrap_t, RTL8651_PROTOCOLTRAP_SIZE);
	TBL_MEM_ALLOC(AsicTableUsage.serverPort, rtl8651_tblDrv_serverPortAsicArrangementEntry_t, RTL8651_SERVERPORTTBL_SIZE);	
	TBL_MEM_ALLOC(AsicTableUsage.alg, rtl8651_tblDrv_algAsicArrangementEntry_t, RTL8651_ALGTBL_SIZE);
	TBL_MEM_ALLOC(DrvTbl.tcpudp, rtl8651DrvNaptLine_t, RTL8651_TCPUDPTBL_SIZE);
	TBL_MEM_ALLOC(DrvTbl.icmp, rtl8651_tblDrv_naptIcmpFlowEntry_t*, RTL8651_ICMPTBL_SIZE);
	TBL_MEM_ALLOC(rtl8651NaptIcmpBackwardCollisionCountTable,uint16,RTL8651_ICMPTBL_SIZE);
	TBL_MEM_ALLOC(rtl8651NaptIcmpFlowNumberTable,uint16,RTL8651_ICMPTBL_SIZE);
	TBL_MEM_ALLOC(rtl8651NaptIcmpFlowNonAsicNumberTable,uint16,RTL8651_ICMPTBL_SIZE);

	TBL_MEM_ALLOC(rtl8651DmzHost,rtl8651DmzHostStruct_t ,rtl8651_tblDrvPara.dmzHostsNum);
	TBL_MEM_ALLOC(_rtl8651_udpSizeThreshValue,uint16,rtl8651_tblDrvPara.dialSessionNumber);
	TBL_MEM_ALLOC(rtl8651_wanStatus,int,rtl8651_tblDrvPara.dialSessionNumber);

	/*==========================================================
	  * Allocate memory for napt server port list
	  *==========================================================*/
	{
		rtl8651_tblDrv_naptServerPortEntry_t * tempNaptServerPortPtr;
		TBL_MEM_ALLOC(tempNaptServerPortPtr, rtl8651_tblDrv_naptServerPortEntry_t, rtl8651_tblDrvPara.naptServerPortEntryNumber);
		SLIST_INIT(&DrvTbl.freeList.serverPort);
		for(i=0; i<rtl8651_tblDrvPara.naptServerPortEntryNumber; i++)
			SLIST_INSERT_HEAD(&DrvTbl.freeList.serverPort, &tempNaptServerPortPtr[i], nextServerPort);
	}
	/*==========================================================
	  * Allocate memory for NAT list
	  *==========================================================*/
	{
		rtl8651_tblDrv_natEntry_t * tempNatMappintPtr;
		TBL_MEM_ALLOC(tempNatMappintPtr, rtl8651_tblDrv_natEntry_t, rtl8651_tblDrvPara.natSize);
		SLIST_INIT(&DrvTbl.freeList.natMapping);
		for(i=0; i<rtl8651_tblDrvPara.natSize; i++)
			SLIST_INSERT_HEAD(&DrvTbl.freeList.natMapping, &tempNatMappintPtr[i], nextNatEntry);
	}
	/*==========================================================
	  * Allocate memory for ALG entry
	  *==========================================================*/
	{
		rtl8651_tblDrv_algEntry_t * tempAlgPtr;
		TBL_MEM_ALLOC(tempAlgPtr, rtl8651_tblDrv_algEntry_t, rtl8651_tblDrvPara.algSize);
		SLIST_INIT(&DrvTbl.freeList.alg);
		for(i=0; i<rtl8651_tblDrvPara.algSize; i++)
			SLIST_INSERT_HEAD(&DrvTbl.freeList.alg, &tempAlgPtr[i], nextAlgEntry);
	}
	/*==========================================================
	  * Allocate memory for napt tcp/udp flow entry list
	  *==========================================================*/
	{
		rtl8651_tblDrv_naptTcpUdpFlowEntry_t  *_naptFlowPool;
		int wanted, allocated;
		
		wanted = rtl8651_tblDrvPara.naptTcpUdpFlowSize;
		allocated = 0;
		CTAILQ_INIT(&DrvTbl.freeList.tcpudp);

		while( allocated < wanted )
		{
			int allocMore;
			allocMore = wanted - allocated;
#ifdef __uClinux__
#else
			// In MMU, the maximal memory size to be allocated is 128K.
			// Allocating 512 connections is under this limit.
			if ( allocMore > 512 ) allocMore = 512;
#endif
			
			TBL_MEM_ALLOC(_naptFlowPool, rtl8651_tblDrv_naptTcpUdpFlowEntry_t, allocMore);
			memset( _naptFlowPool, 0,sizeof(rtl8651_tblDrv_naptTcpUdpFlowEntry_t)*allocMore);		
			//All entries are in free list.
			for(i=0; i<allocMore; i++) 
				CTAILQ_INSERT_HEAD(&DrvTbl.freeList.tcpudp, &_naptFlowPool[i], next);

			allocated += allocMore;
		}
	}
	/*==========================================================
	  * Allocate memory for napt icmp flow entry list
	  *==========================================================*/
	{
		rtl8651_tblDrv_naptIcmpFlowEntry_t * tempNaptIcmpFlowPtr;
		TBL_MEM_ALLOC(tempNaptIcmpFlowPtr, rtl8651_tblDrv_naptIcmpFlowEntry_t, 1);
		DrvTbl.freeList.icmp = tempNaptIcmpFlowPtr;
		for(i=1; i<rtl8651_tblDrvPara.naptIcmpFlowSize; i++) {
			tempNaptIcmpFlowPtr->next = (rtl8651_tblDrv_naptIcmpFlowEntry_t *) rtlglue_malloc(sizeof(rtl8651_tblDrv_naptIcmpFlowEntry_t));
			assert(tempNaptIcmpFlowPtr->next);
			tempNaptIcmpFlowPtr = tempNaptIcmpFlowPtr->next;
		}
		tempNaptIcmpFlowPtr->next = (rtl8651_tblDrv_naptIcmpFlowEntry_t *)NULL;
	}
	/*=========================================================
	  * Url Filtering Database Initialization
	  *=========================================================*/
	#define URL_DB_SIZE	(rtl8651_tblDrvPara.urlfilterDBSize*rtl8651_tblDrvPara.dialSessionNumber)
	{
		uint8 httpHdrPattern[5] = "\r\n\r\n";
		uint8 httpHostpattern[9] = "\r\nHost: ";
		uint8 httpGetPattern[6] = "GET \\";
		uint8 httpGet1Pattern[5] = "GET ";
		uint8 httphttpPattern[8] = "http://";
		uint8 httpPathGetPattern[5] = "GET /";
		uint16 *_temp2, *_temp4;
		int8 *_temp, *_temp3;
		int32 index;
		

		_urlFilter.urlFilterCount = 0;
		mBuf_getBMjmpTable(httpHdrPattern, _urlFilter.urlHdrJmpTbl,  4, TRUE);
		mBuf_getBMjmpTable(httpHostpattern, _urlFilter.urlHostJmpTbl,  8, TRUE);
		mBuf_getBMjmpTable(httpGetPattern, _urlFilter.urlGetJmpTbl,  5, TRUE);
		mBuf_getBMjmpTable(httpGet1Pattern, _urlFilter.urlGet1JmpTbl, 4, TRUE);
		mBuf_getBMjmpTable(httphttpPattern, _urlFilter.urlhttpJmpTbl, 7, FALSE);
		mBuf_getBMjmpTable(httpPathGetPattern, _urlFilter.pathGetJmpTbl,  5, TRUE);

		_urlFilter.urlFilterDB 		= (int8 **)rtlglue_malloc(URL_DB_SIZE*sizeof(int8 *));
		if (_urlFilter.urlFilterDB == NULL)
			RTL_BUG("urlFilterDB memory allocation failed.");
		_urlFilter.pathFilterDB		= (int8 **)rtlglue_malloc(URL_DB_SIZE*sizeof(int8 *));		/* for path filtering spec. */
		if (_urlFilter.pathFilterDB == NULL)
			RTL_BUG("pathFilterDB memory allocation failed.");
		_urlFilter.urlFilterJmpTbl	= (uint16 **)rtlglue_malloc(URL_DB_SIZE*sizeof(uint16 *));
		if (_urlFilter.urlFilterJmpTbl == NULL)
			RTL_BUG("urlFilterJmpTbl memory allocation failed.");
		_urlFilter.pathFilterJmpTbl	= (uint16 **)rtlglue_malloc(URL_DB_SIZE*sizeof(uint16 *));	/* for path filtering spec. */
		if (_urlFilter.pathFilterJmpTbl == NULL)
			RTL_BUG("pathFilterJmpTbl memory allocation failed.");
		_urlFilter.urlFilterStrLen	= (int32 *)rtlglue_malloc(URL_DB_SIZE*sizeof(int32));
		if (_urlFilter.urlFilterStrLen == NULL)
			RTL_BUG("urlFilterStrLen memory allocation failed.");
		_urlFilter.pathFilterStrLen	= (int32 *)rtlglue_malloc(URL_DB_SIZE*sizeof(int32));		/* for path filtering spec. */
		if (_urlFilter.pathFilterStrLen == NULL)
			RTL_BUG("pathFilterStrLen memory allocation failed.");
		_urlFilter.urlExactlyMatch	= (uint8 *)rtlglue_malloc(URL_DB_SIZE*sizeof(uint8));
		if (_urlFilter.urlExactlyMatch == NULL)
			RTL_BUG("urlExactlyMatch memory allocation failed.");
		_urlFilter.pathExactlyMatch	= (uint8 *)rtlglue_malloc(URL_DB_SIZE*sizeof(uint8));		/* for path filtering spec. */
		if (_urlFilter.pathExactlyMatch == NULL)
			RTL_BUG("pathExactlyMatch memory allocation failed.");
		_urlFilter.sessionId		= (uint32 *)rtlglue_malloc(URL_DB_SIZE*sizeof(uint32));
		if (_urlFilter.sessionId == NULL)
			RTL_BUG("sessionId memory allocation failed.");
#ifdef RTL865XB_URLFILTER_UNKNOWNURLTYPE_SUPPORT
		_urlFilter.ruleType		= (uint8 *)rtlglue_malloc(URL_DB_SIZE*sizeof(uint8));
		if (_urlFilter.ruleType == NULL)
			RTL_BUG("ruleType memory allocation failed.");
#endif

		assert(_urlFilter.urlFilterDB && _urlFilter.urlFilterJmpTbl && _urlFilter.urlFilterStrLen && _urlFilter.urlExactlyMatch && 
			_urlFilter.sessionId && _urlFilter.pathFilterDB && _urlFilter.pathFilterJmpTbl && _urlFilter.pathFilterStrLen && _urlFilter.pathExactlyMatch);
		
		_temp 	= (int8 *) rtlglue_malloc(URL_DB_SIZE*rtl8651_tblDrvPara.urlfilterStringLen);
		if (_temp == NULL)
			RTL_BUG("_temp memory allocation failed.");
		_temp2	=(uint16 *) rtlglue_malloc(URL_DB_SIZE*256*sizeof(uint16));
		if (_temp2 == NULL)
			RTL_BUG("_temp2 memory allocation failed.");
		_temp3 	= (int8 *) rtlglue_malloc(URL_DB_SIZE*rtl8651_tblDrvPara.pathfilterStringLen);
		if (_temp3 == NULL)
			RTL_BUG("_temp3 memory allocation failed.");
		_temp4	=(uint16 *) rtlglue_malloc(URL_DB_SIZE*256*sizeof(uint16));
		if (_temp4 == NULL)
			RTL_BUG("_temp4 memory allocation failed.");
		assert(_temp && _temp2);

		memset( _temp, 0, URL_DB_SIZE*rtl8651_tblDrvPara.urlfilterStringLen);
		memset( _temp2, 0, (URL_DB_SIZE*256*sizeof(uint16)));
		memset( _temp3, 0, URL_DB_SIZE*rtl8651_tblDrvPara.pathfilterStringLen);
		memset( _temp4, 0, (URL_DB_SIZE*256*sizeof(uint16)));
		for(index=0; index<URL_DB_SIZE; index++) {
			_urlFilter.urlFilterDB[index] 		= &_temp[index*rtl8651_tblDrvPara.urlfilterStringLen];
			_urlFilter.urlFilterJmpTbl[index]		= &_temp2[index*256];
			_urlFilter.urlFilterStrLen[index]		= 0;
			_urlFilter.urlExactlyMatch[index]		= 0;
			_urlFilter.pathFilterDB[index] 		= &_temp3[index*rtl8651_tblDrvPara.pathfilterStringLen];
			_urlFilter.pathFilterJmpTbl[index]	= &_temp4[index*256];			
			_urlFilter.pathFilterStrLen[index]		= 0;
			_urlFilter.pathExactlyMatch[index]	= 0;
			_urlFilter.sessionId[index]			= 0;
#ifdef RTL865XB_URLFILTER_UNKNOWNURLTYPE_SUPPORT
			_urlFilter.ruleType[index]			= 0;
#endif
			_urlFilter.callBackFunc				= NULL;
			SLIST_INIT(&(_urlFilter.pktFilter[index]));
		}
	}
	#undef URL_DB_SIZE
	/*==========================================================
	  * Allocate memory for URL Packet filtering
	  *==========================================================*/
	{
		urlPktFiltering_t *tmp_t;
		SLIST_INIT(&DrvTbl.freeList.freeUrlPktFilter);
		SLIST_INIT(&DrvTbl.inuseList.urlPktFilter);
		TBL_MEM_ALLOC(tmp_t, urlPktFiltering_t, rtl8651_tblDrvPara.urlPktfilterDBSize);
		for(i=0; i<rtl8651_tblDrvPara.urlPktfilterDBSize; i++)
			SLIST_INSERT_HEAD(&DrvTbl.freeList.freeUrlPktFilter, &tmp_t[i], next);
	}




	return SUCCESS;
}

int32 rtl8651_layer4_collect(void) {
	int32 i;


	/*==============================================================
	  * Collect NAPT Server Port List
	  *==============================================================*/
	{
		rtl8651_tblDrv_networkIntfTable_t *netif_t;
		rtl8651_tblDrv_naptServerPortEntry_t *delsp_t;
		uint32 entry = 0;

		for(i=0, netif_t=DrvTbl.netIface; i<rtl8651_tblDrvPara.networkIntfTableSize;i++, netif_t++) {
			if (netif_t->valid == 0)
				continue;
			while ((delsp_t=SLIST_FIRST(&netif_t->nextNaptServerPort))) {
				SLIST_REMOVE_HEAD(&netif_t->nextNaptServerPort, nextServerPort);
				SLIST_INSERT_HEAD(&DrvTbl.freeList.serverPort, delsp_t, nextServerPort);
			}
		}
		SLIST_FOREACH(delsp_t, &DrvTbl.freeList.serverPort, nextServerPort)
			entry ++;
		assert(entry == rtl8651_tblDrvPara.naptServerPortEntryNumber);
	}
	
	{//Collect data structures attached on NAT 
		rtl8651_tblDrv_natEntry_t * tempNatPtr;
		rtl8651_tblDrv_networkIntfTable_t *netIfPtr;

		for(i=0,netIfPtr=&DrvTbl.netIface[0];i<rtl8651_tblDrvPara.networkIntfTableSize;i++,netIfPtr++) 
			if (netIfPtr->valid == 1) {
				/* Collect NAT */
				SLIST_FOREACH(tempNatPtr, &(netIfPtr->nextNat), nextNatEntry) 
					if (tempNatPtr->nextNatEntry.sle_next == NULL)
						break;
				if (tempNatPtr != NULL) {
					tempNatPtr->nextNatEntry.sle_next = DrvTbl.freeList.natMapping.slh_first;
					DrvTbl.freeList.natMapping.slh_first = netIfPtr->nextNat.slh_first;
				}
				SLIST_INIT(&(netIfPtr->nextNat));
			}
		i=0;
		SLIST_FOREACH(tempNatPtr, &DrvTbl.freeList.natMapping, nextNatEntry) 
			i++;
		assert(i==rtl8651_tblDrvPara.natSize);
	}
	{ // Collect NAPT ICMP
		rtl8651_tblDrv_naptIcmpFlowEntry_t * tempNaptIcmpFlowPtr;
		rtl8651_flushAllNaptConnections();
		assert(CTAILQ_TOTAL(&DrvTbl.freeList.tcpudp)==rtl8651_tblDrvPara.naptTcpUdpFlowSize);
		i=0;
		tempNaptIcmpFlowPtr = DrvTbl.freeList.icmp;
		while(tempNaptIcmpFlowPtr) {
			i++;
			tempNaptIcmpFlowPtr = tempNaptIcmpFlowPtr->next;
		}
		assert(i==rtl8651_tblDrvPara.naptIcmpFlowSize);
	}
	{ // Collect ALG entry
		rtl8651_tblDrv_algEntry_t * tempAlgPtr;
		SLIST_FOREACH(tempAlgPtr, &DrvTbl.inuseList.alg, nextAlgEntry) 
			if (tempAlgPtr->nextAlgEntry.sle_next == NULL)
				break;
		if (tempAlgPtr) {
			tempAlgPtr->nextAlgEntry.sle_next = DrvTbl.freeList.alg.slh_first;
			DrvTbl.freeList.alg.slh_first = DrvTbl.inuseList.alg.slh_first;
			DrvTbl.inuseList.alg.slh_first = NULL;
		}
		i=0;
		SLIST_FOREACH(tempAlgPtr, &DrvTbl.freeList.alg, nextAlgEntry) 
			i++;
		assert(i==rtl8651_tblDrvPara.algSize);
	}
	/*==========================================================
	  * Collect for URL Packet filtering
	  *==========================================================*/
	{
		urlPktFiltering_t *pktFilter;

		while( (pktFilter = SLIST_FIRST(&DrvTbl.inuseList.urlPktFilter))!=NULL )
		{
			SLIST_REMOVE_HEAD(&DrvTbl.inuseList.urlPktFilter, next);
			SLIST_INSERT_HEAD(&DrvTbl.freeList.freeUrlPktFilter, pktFilter, next);
		}
		i = 0;
		SLIST_FOREACH(pktFilter, &DrvTbl.freeList.freeUrlPktFilter, next) 
			i++;
		assert(i == rtl8651_tblDrvPara.urlPktfilterDBSize);
	}

#ifdef CONFIG_RTL865X_VOIP
	/* VoIP Supoorting */
	voip_collect();
#endif


	return SUCCESS;
}


//Initialization all Layer4 tables and linked lists. For software reset purpose.
int32  rtl8651_layer4_init(void){
	int32 i,j;

	/////////////Initialize L4 static variables.//////////////////
	_enableAdaptiveNAPTfastTimeout=0;
	_enableSmartNaptSwappingByLoad=1;
	_smartNaptSwappingThresholdTcp = 100;// 100 -- about 1.2Mbps
	_smartNaptSwappingThresholdUdp = 5;// 5 -- about 60Kbps
	_smartNaptSwappingThresholdCdrouter = 5; // It is many enough for SYN/SYN-ACK/ACK/PUSH... */
	_adaptiveTimeoutThreshold=100; 
	rtl8651ExistedNaptIcmpFlows = 0;
	multiNxtHop = NULL;
	softNaptIp = 0;
	mnqueue_ctrl.wanBandwidth = 0;
	mnQ_enable = FALSE;
	enable4WayHash = FALSE;
	def_napt_t = NULL;

	/////////////Initialize L4 allocated arrays. //////////////////

	/* Initialize Protocol Trap Table */
	////memset((int8 *)TBL_protoTrap, 0, sizeof(rtl8651_tblDrv_protoTrap_t)*RTL8651_PROTOCOLTRAP_SIZE);

	memset(DrvTbl.tcpudp, 0, RTL8651_TCPUDPTBL_SIZE * sizeof(rtl8651DrvNaptLine_t));
	memset((char*)rtl8651_wanStatus, 0,rtl8651_tblDrvPara.dialSessionNumber*sizeof(int));
	for(i=0; i < RTL8651_TCPUDPTBL_SIZE; i++)
		for(j=NAPT_ASIC_CACHED_OUT; j < NAPT_TOTAL_LISTS; j++)
			SLIST_INIT(&DrvTbl.tcpudp[i].list[j]);

	for(i=0; i<RTL8651_SERVERPORTTBL_SIZE; i++)
		AsicTableUsage.serverPort[i].valid = 0;

	//Initial napt icmp flow table
	for(i=0; i<RTL8651_ICMPTBL_SIZE; i++) {
		DrvTbl.icmp[i] = (rtl8651_tblDrv_naptIcmpFlowEntry_t *)NULL;
		rtl8651NaptIcmpBackwardCollisionCountTable[i] = 0;
		rtl8651NaptIcmpFlowNumberTable[i] = 0;
		rtl8651NaptIcmpFlowNonAsicNumberTable[i] = 0;
	}

	// ALG
	for(i=0; i<RTL8651_ALGTBL_SIZE; i++)
		AsicTableUsage.alg[i].valid = 0;

	/* Initial napt timeout value */
	rtl8651_setNaptIcmpTimeout(60);
	rtl8651_setNaptIcmpFastTimeout(2);
	rtl8651_setNaptUdpTimeout(60*15);	
	rtl8651_setNaptTcpLongTimeout(24 * 60 * 60);
	rtl8651_setNaptTcpMediumTimeout(600);
	rtl8651_setNaptTcpFastTimeout(20);
	rtl8651_setNaptTcpFinTimeout(20);

	rtl8651_getAsicNaptAutoAddDelete(&naptAutoLearn, NULL);

	SLIST_INIT(&DrvTbl.inuseList.alg);
	CTAILQ_INIT(&DrvTbl.inuseList.tcpudp);
	memset(rtl8651DmzHost, 0, sizeof(rtl8651DmzHostStruct_t)*rtl8651_tblDrvPara.dmzHostsNum);		

	//should be in fwd eng's init function...
	for (i=0;i<rtl8651_tblDrvPara.dialSessionNumber;i++)
		_rtl8651_udpSizeThreshValue[i]=0xffff;

#if 0 //Legacy?? don't turn it on until we return to 8651B development.
	def_napt_t = NULL;
	/* Initialize Server Port Table */
	{
		TBL_MEM_ALLOC( DrvTbl.serverport, rtl8651_tblDrv_naptServerPortEntry_t, RTL8651_SERVERPORTTBL_SIZE );
		memset(DrvTbl.serverport, 0, sizeof(rtl8651_tblDrv_naptServerPortEntry_t)*RTL8651_SERVERPORTTBL_SIZE);
		for(i=0; i<RTL8651_SERVERPORTTBL_SIZE; i++)
			if (spEntryMask & (1<<i))
				DrvTbl.serverport[i].usable = 1;
			else DrvTbl.serverport[i].usable = 0;
	}
#endif

#ifdef CACHE_FIND_NAPT_RESULT
	INVALID_FIND_CACHE();
#endif/*CACHE_FIND_NAPT_RESULT*/

#ifdef CACHE_DOS_COUNT_NAPT_RESULT
	INVALID_DOS_COUNT_CACHE();
#endif

#ifdef FLUSH_NAPT_OLDEST_CONNECTION2
	candidateToBeFreed = NULL;
#endif/*FLUSH_NAPT_OLDEST_CONNECTION2*/



	/* Initialize URL filter */
	{
		#define URL_DB_SIZE	(rtl8651_tblDrvPara.urlfilterDBSize*rtl8651_tblDrvPara.dialSessionNumber)

		int32 index;

		for(index=0; index<URL_DB_SIZE; index++) {
			memset(_urlFilter.urlFilterDB[index], 0, rtl8651_tblDrvPara.urlfilterStringLen);
			memset(_urlFilter.urlFilterJmpTbl[index], 0, 256 * sizeof(uint16));
			_urlFilter.urlFilterStrLen[index]		= 0;
			_urlFilter.urlExactlyMatch[index]		= 0;
			memset(_urlFilter.pathFilterDB[index], 0, rtl8651_tblDrvPara.pathfilterStringLen);
			memset(_urlFilter.pathFilterJmpTbl[index], 0, 256 * sizeof(uint16));
			_urlFilter.pathFilterStrLen[index]		= 0;
			_urlFilter.pathExactlyMatch[index]	= 0;
			_urlFilter.sessionId[index]			= 0;
#ifdef RTL865XB_URLFILTER_UNKNOWNURLTYPE_SUPPORT
			_urlFilter.ruleType[index]			= 0;
#endif
			_urlFilter.callBackFunc				= NULL;
			SLIST_INIT(&(_urlFilter.pktFilter[index]));
		}
		#undef URL_DB_SIZE
	}

#ifdef CONFIG_RTL865X_VOIP
	/* VoIP Supoorting */
	voip_init( 4 /*channel*/, 4*4 /*media*/ );
#endif

	return SUCCESS;
	
}


int32 rtl8651_log_alloc(void){
	/* allocate logging variables */
	TBL_MEM_ALLOC(_loggingModule,uint32,rtl8651_tblDrvPara.dialSessionNumber);
	/* allocate acl dsid control variables */
#ifdef RTL8651_DSID_SUPPORT
	TBL_MEM_ALLOC(_rtl8651_acl_drop_nolog,uint8,rtl8651_tblDrvPara.dialSessionNumber);
#endif
	return SUCCESS;
}

int32 rtl8651_log_init(void){
	int32 index;

	/* init call-back function */
	_pUserLoggingFunc = NULL;

	/* clear all switches */
	for(index=0;index<rtl8651_tblDrvPara.dialSessionNumber;index++)
		_loggingModule[index]=0;
#ifdef RTL8651_DSID_SUPPORT
	for(index=0;index<rtl8651_tblDrvPara.dialSessionNumber;index++)
		_rtl8651_acl_drop_nolog[index]=0;
#endif
	return SUCCESS;
}



//for testing only.
int32 rtl8651_tblDrvReinit(void) {
#ifdef RTL865X_TEST
	int32 port;
#endif

	int32 retval;

	DRV_INIT_CHECK(rtl8651_initAsic());
	DRV_INIT_CHECK(rtl8651_layer2_collect());
	DRV_INIT_CHECK(rtl8651_layer4_collect());
	DRV_INIT_CHECK(rtl8651_layer3_collect());


	//_rtl8651_tblDrvListCollect();
	//Initial array structure
	DRV_INIT_CHECK(rtl8651_layer2_init());
	rtl8651_installPortStatusChangeNotifier(rtl8651_updatePortStatus);
	/* Install L2 entry changed notify funciton between L2 and  L3/4 driver */	
	rtl8651_installFDBEntryChangeNotifier(_rtl8651_syncL2Entrychange);
	DRV_INIT_CHECK(rtl8651_layer3_init());
	DRV_INIT_CHECK(rtl8651_layer4_init());
	DRV_INIT_CHECK(rtl8651_log_init());
	////_rtl8651_tblDrvArrayInit();

	_rtl8651_ReinitUrlFilterTrustedUserDb();


	//Common initialization for testing
	retval = rtl8651_addSpanningTreeInstance(0); 	assert(retval == SUCCESS);
	retval = rtl8651_addFilterDatabase(0);			assert(retval == SUCCESS);
#ifdef RTL865X_TEST
	_rtl8651_addVlan(1);
	for(port=0; port<RTL8651_PORT_NUMBER+rtl8651_totalExtPortNum; port++) {
		retval = rtl8651_addVlanPortMember(1, port);	assert(retval == SUCCESS);
		retval = rtl8651_setPvid(port, 1);				assert(retval == SUCCESS);
	}
#endif

	// re-init timer
	_sysUpSeconds = 0;


#ifdef CACHE_FIND_NAPT_RESULT
	INVALID_FIND_CACHE();
#endif/*CACHE_FIND_NAPT_RESULT*/

#ifdef CACHE_DOS_COUNT_NAPT_RESULT
	INVALID_DOS_COUNT_CACHE();
#endif

#ifdef FLUSH_NAPT_OLDEST_CONNECTION2
	candidateToBeFreed = NULL;
#endif/*FLUSH_NAPT_OLDEST_CONNECTION2*/

	WRITE_MEM32( TCCNR, READ_MEM32(TCCNR) & ~TC1EN );/* Disable timer1 */
	WRITE_MEM32( TC1DATA, 0xffffff00);	
	prevTC1CNT = READ_MEM32( TC1DATA );
	#ifdef RTL865X_TEST
	WRITE_MEM32(TC0DATA, (((0x100000000/4096)/100)+1) << TCD_OFFSET );
	WRITE_MEM32(TC1CNT, 0xffffff00);	
	#endif/*RTL865X_TEST*/
	WRITE_MEM32( TCCNR, ( READ_MEM32(TCCNR) | TC1EN ) | TC1MODE_TIMER );/* Enable timer1 - timer mode */
	WRITE_MEM32( TCIR, READ_MEM32(TCIR) & ~TC1IE ); /* Disable timer1 interrupt */

	return SUCCESS;
}

//Almost same as rtl8651_tblDrvReinit, clear all settings and return to code default value.
int32 rtl8651_tblDrvReset(void) {
	
	
	//Initial ASIC table
	rtl8651_initAsic();
	//Collect all all link list entries

	DRV_INIT_CHECK(rtl8651_layer2_collect());
	DRV_INIT_CHECK(rtl8651_layer3_collect());
	DRV_INIT_CHECK(rtl8651_layer4_collect());
	//_rtl8651_tblDrvListCollect();
	//Initial array structure
	DRV_INIT_CHECK(rtl8651_layer2_init());
	rtl8651_installPortStatusChangeNotifier(rtl8651_updatePortStatus);
	/* Install L2 entry changed notify funciton between L2 and  L3/4 driver */	
	rtl8651_installFDBEntryChangeNotifier(_rtl8651_syncL2Entrychange);
	DRV_INIT_CHECK(rtl8651_layer3_init());
	DRV_INIT_CHECK(rtl8651_layer4_init());
	DRV_INIT_CHECK(rtl8651_log_init());
	////_rtl8651_tblDrvArrayInit();
	return SUCCESS;
}



/*
@func int32 | rtl8651_tblDrvInit | Initiate Table Driver.
@parm rtl8651_tblDrvInitPara_t * | configPara | Initial parameters setting.
@rvalue SUCCESS | Table Driver Initialization was success.
@rvalue FAILED | Table DRiver Initialization failed.
@comm
This API should be called to initiate Table Driver when in the RTL8651 initialization state.
*/
int32 rtl8651_tblDrvInit(rtl8651_tblDrvInitPara_t * configPara) {
	DRV_INIT_CHECK(rtl8651_initDrvParam(configPara));
	//Initial ASIC table
	DRV_INIT_CHECK(rtl8651_initAsic());
	DRV_INIT_CHECK(rtl8651_layer2_alloc());
	DRV_INIT_CHECK(rtl8651_layer3_alloc());
	DRV_INIT_CHECK(rtl8651_layer4_alloc());
	DRV_INIT_CHECK(rtl8651_log_alloc());
	//Now initialize all dynamically allocated linked lists and all arrays.
	DRV_INIT_CHECK(rtl8651_layer2_init());
	rtl8651_installPortStatusChangeNotifier(rtl8651_updatePortStatus);
	/* Install L2 entry changed notify funciton between L2 and  L3/4 driver */	
	rtl8651_installFDBEntryChangeNotifier(_rtl8651_syncL2Entrychange);
	DRV_INIT_CHECK(rtl8651_layer3_init());
	DRV_INIT_CHECK(rtl8651_layer4_init());
	DRV_INIT_CHECK(rtl8651_log_init());
	_rtl8651_initURLfilter();	

       _rtl8651_initUrlFilterTrustedUserDb();	
	   
//	rtl8651_installMTUCallBackFunction(myTesting);
	WRITE_MEM32( TCCNR, READ_MEM32(TCCNR) & ~TC1EN );/* Disable timer1 */
	WRITE_MEM32( TC1DATA, 0xffffff00);	
	prevTC1CNT = READ_MEM32( TC1DATA );
	#ifdef RTL865X_TEST
		WRITE_MEM32(TC0DATA, (((0x100000000/4096)/100)+1) << TCD_OFFSET );
		WRITE_MEM32(TC1CNT, 0xffffff00);	
	#endif/*RTL865X_TEST*/
	WRITE_MEM32( TCCNR, ( READ_MEM32(TCCNR) | TC1EN ) | TC1MODE_TIMER );/* Enable timer1 - timer mode */
	WRITE_MEM32( TCIR, READ_MEM32(TCIR) & ~TC1IE ); /* Disable timer1 interrupt */

	return SUCCESS;
}



#ifdef RTL865X_TEST
/* for test code only */
uint32 getmstime()
{
	return time(NULL)/1000;
}
#endif//RTL865X_TEST



/*
@func int32 | _rtl8651_createVlanForPassthru | This function is used to create a VLAN including all ports. It is useful for PPPoE passthru and IPv6 passthru to share one VLAN.
@parm ether_addr_t | gmac | External MAC address (input)
@parm uint16* | pvid | pointer to store new allocated VLAN ID (output)
@rvalue int32 | SUCCESS
@comm
*/
static int32 _rtl8651_createVlanForPassthru( ether_addr_t *gmac, uint16 *pvid )
{
	rtl8651_tblDrv_vlanTable_t *vlanPtr;
	uint16 vid;
	int i;
	uint32 retval;

	assert( pvid );
	/*rtlglue_printf( "_rtl8651_createVlanForPassthru(): rtl8651_gVlanIDforPassthru:%d\n", rtl8651_gVlanIDforPassthru );*/

	if ( rtl8651_gVlanIDforPassthru == RTL8651_PASSTHRU_VLANID_IS_NOT_SET )
	{
		
		// find out empty VLAN
		for( i = 0; i < RTL8651_VLAN_NUMBER; i++ )
		{
			rtl865x_tblAsicDrv_vlanParam_t try;
			retval = rtl8651_getAsicVlan( i, &try );
			if ( try.valid == FALSE )
				break; // found!!
		}
		if ( i == RTL8651_VLAN_NUMBER )
		{
			rtlglue_printf("No more availiable VLAN entry!\n");
			return TBLDRV_ENOVLAN;
		}

		vid = i+8; /* VLAN ID: 8,9,10... */
		/* rtlglue_printf("_rtl8651_createVlanForPassthru(vid:%d)\n",vid); */
		*pvid = vid;
		rtl8651_gVlanIDforPassthru = vid;

		retval = _rtl8651_addVlan( vid );
		/* if ( retval != SUCCESS ) printk("_rtl8651_addVlan() returns %d.\n",retval); */
		assert( retval == SUCCESS );

		vlanPtr = &DrvTbl.vlan[rtl8651_vlanTableIndex(vid)];
		vlanPtr->broadcastToCpu = FALSE;
		vlanPtr->inAclEnd = RTL8651_ACLTBL_PERMIT_ALL;
		vlanPtr->inAclStart = RTL8651_ACLTBL_PERMIT_ALL;
		vlanPtr->internal = FALSE;
		memcpy( &vlanPtr->macAddr, gmac, 6 );
		vlanPtr->macAddrNumber = 1;
		vlanPtr->memberPortMask = 0x1f|0x1e0;
		vlanPtr->outAclEnd = RTL8651_ACLTBL_PERMIT_ALL;
		vlanPtr->outAclStart = RTL8651_ACLTBL_PERMIT_ALL;
		vlanPtr->promiscuous = FALSE;
		vlanPtr->untagPortMask = 0x1f|0x1e0;
		vlanPtr->valid = TRUE;
		vlanPtr->softRoute = FALSE; /* to set 'enable routing' = 1 */
		vlanPtr->ipAttached = TRUE; /* to set 'enable routing' = 1 */

		/* rearrange multicast port status */
		_rtl8651_arrangeMulticastPortStatus();

		retval = RTL8651_SETASICVLAN( vlanPtr );
	}
	else
	{
		/* VLAN has created */
		*pvid = rtl8651_gVlanIDforPassthru;

		retval = SUCCESS;
	}

	// increase reference count
	if ( retval == SUCCESS ) rtl8651_gRefCountofVlanforPassthru++;

	assert( rtl8651_gRefCountofVlanforPassthru >= 0 );
	/*rtlglue_printf("[%s] rtl8651_gRefCountofVlanforPassthru=%d\n",__FUNCTION__,rtl8651_gRefCountofVlanforPassthru);*/
	return retval;
}


/*
@func int32 | _rtl8651_destroyVlanForPassthru | This function is used to destroy the VLAN allocated for PPPoE passthru and IPv6 passthru.
@parm ether_addr_t | gmac | External MAC address (input)
@parm uint16 | vid | pointer to store new allocated VLAN ID (input)
@rvalue int32 | SUCCESS
@comm
*/
static int32 _rtl8651_destroyVlanForPassthru( ether_addr_t gmac, uint16 vid )
{
	int32 ret;

	// unreference the gateway mac
	ret = _rtl8651_assignVlanMacAddress( vid, &gmac, 0 );
	assert( ret == SUCCESS );

	// decrease reference count
	rtl8651_gRefCountofVlanforPassthru--;
	assert( rtl8651_gRefCountofVlanforPassthru >= 0 );
	/*rtlglue_printf("[%s] rtl8651_gRefCountofVlanforPassthru=%d\n",__FUNCTION__,rtl8651_gRefCountofVlanforPassthru);*/

	if ( rtl8651_gRefCountofVlanforPassthru == 0 )
	{
		ret = _rtl8651_delVlan( vid );
		rtl8651_gVlanIDforPassthru = RTL8651_PASSTHRU_VLANID_IS_NOT_SET;
	}
	else
	{
		// someone is still referencing this VLAN.
		ret = SUCCESS;
	}
	//if ( ret != SUCCESS ) rtlglue_printf("_rtl8651_delVlan(VlanID:%d) returns %d.\n", rtl8651_gPPPoEPassthruVlanID, ret );
	return ret;
}


/*
@func int32 | rtl8651_transformPasstheuVlanId | This function is used to transform passthru VLAN ID.
#parm uint32 | oldVlanIdx | The original VLAN index.
@parm uint32* | newVlanIdx | The address of vlan ID to store new VLAN index.
@rvalue int32 | SUCCESS -- transformed
              | FAILED -- not transform
@comm
This is the combination of rtl8651_getPppoePassthruVid(), rtl8651_getIpv6PassthruVid(),
rtl8651_getNetbiosPassthruVid(), and rtl8651_getIpxPassthruVid().
*/
int32 rtl8651_transformPassthruVlanId( uint32 oldVlanIdx, uint32* newVlanIdx )
{
	int32 retval = SUCCESS;
	//rtlglue_printf("%s():%d\n", __FUNCTION__, __LINE__);
	rtlglue_drvMutexLock();
	
	if ( ( rtl8651_l2protoPassThrough & PASSTHRU_PPPOE ) &&
	     oldVlanIdx == rtl8651_vlanTableIndex(rtl8651_gPPPoEPassthruVlanID) )
	{
		*newVlanIdx = rtl8651_gPPPoEExternalVlanIndex;
	}
	else if ( ( rtl8651_l2protoPassThrough & PASSTHRU_IPV6 ) &&
	          oldVlanIdx == rtl8651_vlanTableIndex(rtl8651_gIPv6PassthruVlanID) )
	{
		*newVlanIdx = rtl8651_gIPv6ExternalVlanIndex;
	}
	else if ( ( rtl8651_l2protoPassThrough & PASSTHRU_IPX ) &&
	          oldVlanIdx == rtl8651_vlanTableIndex(rtl8651_gIPXPassthruVlanID) )
	{
		*newVlanIdx = rtl8651_gIPXExternalVlanIndex;
	}
	else if ( ( rtl8651_l2protoPassThrough & PASSTHRU_NETBIOS ) &&
	          oldVlanIdx == rtl8651_vlanTableIndex(rtl8651_gNetbiosPassthruVlanID) )
	{
		*newVlanIdx = rtl8651_gNetbiosExternalVlanIndex;
	}
	else
		retval = FAILED;

	rtlglue_drvMutexUnlock();
	
	return retval;
}

/*
@func int32 | rtl8651_EnablePppoePassthru | This function is used to enable/disable PPPoE Passthru.
@parm uint8 | enable | TRUE for enable, FALSE for disable
@rvalue int8 | previous enable states
@comm
*/
int8 rtl8651_EnablePppoePassthru( int8 enable )
{
	int8 prevValue;
	int32 retval;

	//rtlglue_printf("%s:%d  %s\n", __FILE__, __LINE__, enable?"TRUE":"FALSE");
	rtlglue_drvMutexLock();
	
	prevValue = rtl8651_l2protoPassThrough & PASSTHRU_PPPOE ? TRUE : FALSE;

	if ( enable )
		rtl8651_l2protoPassThrough |= PASSTHRU_PPPOE;
	else
		rtl8651_l2protoPassThrough &= ~PASSTHRU_PPPOE;
#if 1//CONFIG_RTL865XB_EXP_PPPOE_PASSTHRU

	// create or destroy VLAN and protocol-based VLAN ID
	if ( prevValue == TRUE && enable == FALSE )
	{
		// destory 
		retval = _rtl8651_destroyProtocolBasedVLANforPPPoEPassthru();
	}
	else if ( prevValue == FALSE && enable == TRUE )
	{
		// create
		retval = _rtl8651_createProtocolBasedVLANforPPPoEPassthru();
	}
	
#else//CONFIG_RTL865XB
#endif//CONFIG_RTL865XB

	// re-arrange ACL
	_rtl8651_arrangeAllACL();
	

	rtlglue_drvMutexUnlock();
	
	return prevValue;
}


/*
@func int32 | rtl8651_getPppoePassthruVid | This function returns the VLAN id of PPPoE Passthru.
@rvalue int32 | -1 if PPPoE Passthru is disabled, otherwise 0-4095 means VLAN id.
@comm
*/
int32 rtl8651_getPppoePassthruVid()
{
	int32 retval;
	//rtlglue_printf("%s:%d  %s\n", __FILE__, __LINE__, enable?"TRUE":"FALSE");
	rtlglue_drvMutexLock();
	
	if ( rtl8651_l2protoPassThrough & PASSTHRU_PPPOE )
		retval = rtl8651_gPPPoEPassthruVlanID;
	else
		retval = FAILED;

	rtlglue_drvMutexUnlock();
	
	return retval;
}


/*
@func int32 | rtl8651_enableDropUnknownPppoePADT | This function is used to enable/disable PPPoE Passthru.
@parm uint8 | enable | TRUE for enable, FALSE for disable
@rvalue int8 | previous enable states
@comm
If enabled, a PPPoE PADT from WAN with unknown destination MAC address will be dropped.
If disbled, all unknown PPPoE packets will be broadcasted.
*/
int8 rtl8651_enableDropUnknownPppoePADT( int8 enable )
{
	int8 prevValue;
	int i;

	/*
	 * This function is for CDRouter.
	 */

	//rtlglue_printf("%s:%d  %s\n", __FILE__, __LINE__, enable?"TRUE":"FALSE");
	rtlglue_drvMutexLock();
	
	prevValue = rtl8651_drvDropUnknownPppoePADT;

	rtl8651_drvDropUnknownPppoePADT = enable;
	
	// re-arrange ACL
	_rtl8651_arrangeAllACL();

	/*
	 *  rtl8651_enableDropUnknownPppoePADT() may be called after rtl8651_EnablePppoePassthru().
	 *  
	 *  Therefore, we must update Protocal-Based VLAN index.
	 */
	for( i = 0; i < RTL8651_PORT_NUMBER+rtl8651_totalExtPortNum; i++ )
	{
		int8 linkup;
		
		rtl8651_getEthernetPortLinkStatus( i, &linkup, NULL, NULL, NULL );

		if ( linkup )
		{
			rtl8651_setEthernetPortLinkStatus( i, linkup );
		}
	}

	rtlglue_drvMutexUnlock();
	
	return prevValue;
}


#if 1//CONFIG_RTL865XB_EXP_PPPOE_PASSTHRU


int32 _rtl8651_setExternalInterfaceforPPPoEPassthru( ether_addr_t* gmac, uint16 ExtVid )
{

	// External PPPoE interface is being configured.
	rtl8651_gPPPoEExternalVlanIndex = rtl8651_vlanTableIndex( ExtVid );
	memcpy( &rtl8651_gPPPoEExternalGatewayMAC, gmac, sizeof(ether_addr_t) );

	return SUCCESS;
}

	
int32 rtl8651_getExternalInterfaceforPPPoEPassthru()
{
	if ( rtl8651_l2protoPassThrough & PASSTHRU_PPPOE )
		return rtl8651_gPPPoEExternalVlanIndex;
	else
		return SUCCESS;
}

	
int32 _rtl8651_createProtocolBasedVLANforPPPoEPassthru(void)
{
	int i;
	uint32 retval;
	rtl8651_tblDrv_vlanTable_t *vlan;
	
	/*rtlglue_printf("_rtl8651_createProtocolBasedVLANforPPPoEPassthru()...\n");*/
	//rtlglue_printf("ExtVlan:%d  PassthruVlan:%d\n", rtl8651_gPPPoEExternalVlanIndex, rtl8651_gPPPoEPassthruVlanID );

	if ( rtl8651_gPPPoEExternalVlanIndex == RTL8651_VLAN_NUMBER )
	{
		rtlglue_printf( "PPPoE Passthru: External Interface is unknown !!!\n" );
		return FAILED;
	}
		
	if ( rtl8651_gPPPoEPassthruVlanID != RTL8651_PPPOEPASSTHRU_VLANID_IS_NOT_SET )
	{
		rtlglue_printf( "PPPoE Passthru already enabled, rtl8651_gPPPoEPassthruVlanID=%d\n", rtl8651_gPPPoEPassthruVlanID );
		return SUCCESS;
	}

	retval = _rtl8651_createVlanForPassthru( &rtl8651_gPPPoEExternalGatewayMAC, &rtl8651_gPPPoEPassthruVlanID );
	assert( retval == SUCCESS );

	/* point to WAN's ACL */
	vlan = &DrvTbl.vlan[rtl8651_vlanTableIndex(rtl8651_gPPPoEPassthruVlanID)];
	vlan->inAclStart = DrvTbl.vlan[rtl8651_gPPPoEExternalVlanIndex].inAclStart;
	vlan->inAclEnd = DrvTbl.vlan[rtl8651_gPPPoEExternalVlanIndex].inAclEnd;
	vlan->outAclStart = DrvTbl.vlan[rtl8651_gPPPoEExternalVlanIndex].outAclStart;
	vlan->outAclEnd = DrvTbl.vlan[rtl8651_gPPPoEExternalVlanIndex].outAclEnd;
	retval = RTL8651_SETASICVLAN( vlan );
	assert( retval == SUCCESS );
	
	//
	//  _rtl8651_createProtocolBasedVLANforPPPoEPassthru() may have NOT be executed
	//    when the first time link status change.
	//    Thus, the protocol-based VLAN index is pointed to 0, which is WRONG.
	//  To solve this problem, we must update the VLAN index 
	//    when rtl8651_gPPPoEPassthruVlanID is updated.
	//
	for( i = 0; i < RTL8651_PORT_NUMBER+rtl8651_totalExtPortNum; i++ )
	{
		int8 linkup;
		
		rtl8651_getEthernetPortLinkStatus( i, &linkup, NULL, NULL, NULL );

		if ( linkup )
		{
			rtl8651_setEthernetPortLinkStatus( i, linkup );
		}
	}

	return SUCCESS;
}


int32 _rtl8651_destroyProtocolBasedVLANforPPPoEPassthru(void)
{
	int32 ret, i;
		
	if ( rtl8651_gPPPoEExternalVlanIndex == RTL8651_VLAN_NUMBER )
	{
		rtlglue_printf( "PPPoE Passthru: External Interface is unknown !!!\n" );
		return FAILED;
	}
		
	if( rtl8651_gPPPoEPassthruVlanID == RTL8651_PPPOEPASSTHRU_VLANID_IS_NOT_SET )
	{
		rtlglue_printf("rtl8651_gPPPoEExternalVlanIndex is NOT set, but _rtl8651_destroyProtocolBasedVLANforPPPoEPassthru() is called.\n");
		return SUCCESS;
	}

	//
	// destroy VLAN
	//
	ret = _rtl8651_destroyVlanForPassthru( rtl8651_gPPPoEExternalGatewayMAC, rtl8651_gPPPoEPassthruVlanID );
	assert( ret == SUCCESS );
	
	//
	//	If the link is up, we must clear the protocol based VLAN ID
	//
	for( i = 0; i < RTL8651_PORT_NUMBER+rtl8651_totalExtPortNum; i++ )
	{
		int8 linkup;
		
		rtl8651_getEthernetPortLinkStatus( i, &linkup, NULL, NULL, NULL );

		if ( linkup )
		{
			uint8 valid, vlanIdx;

			/*  For CDRouter.
			 *  Clear PPPoE Ctrl packet from Protocol-based VLAN
			 *   when we disabled "drop unknown PPPoE packet" function. */
			if ( 1 || rtl8651_drvDropUnknownPppoePADT == FALSE )
			{
				// verify
				rtl8651_getProtocolBasedVLAN( RTL8651_PBV_RULE_PPPOE_CONTROL, i, &valid, &vlanIdx );
				assert( valid && vlanIdx == rtl8651_vlanTableIndex( rtl8651_gPPPoEPassthruVlanID ) );

				// clean it
				rtl8651_setProtocolBasedVLAN( RTL8651_PBV_RULE_PPPOE_CONTROL, i, 0, 0 );
			}
			
			// verify
			rtl8651_getProtocolBasedVLAN( RTL8651_PBV_RULE_PPPOE_SESSION, i, &valid, &vlanIdx );
			assert( valid && vlanIdx == rtl8651_vlanTableIndex( rtl8651_gPPPoEPassthruVlanID ) );

			// clean it
			rtl8651_setProtocolBasedVLAN( RTL8651_PBV_RULE_PPPOE_SESSION, i, 0, 0 );
		}
	}

	rtl8651_gPPPoEPassthruVlanID = RTL8651_PPPOEPASSTHRU_VLANID_IS_NOT_SET;
	
	return SUCCESS;
}


#else//CONFIG_RTL865XB

uint32 _rtl8651_acl_PPPoE_Passthru( uint32 aclStart, uint32 aclEnd, rtl8651_tblDrv_networkIntfTable_t *netIfPtr )
{
	_rtl8651_tblDrvAclRule_t rule;
	int32 retval;
	uint32 count=0;
	rtl8651_tblDrv_filterDbTableEntry_t *macInfo;

	/* printk("\n\n***** _rtl8651_acl_PPPoE_Passthru(aclStart:%d,aclEnd:%d)\n\n",aclStart,aclEnd); */
	if ( aclStart > aclEnd || aclEnd>=RTL8651_ACLTBL_SIZE )	
		return count;

	// add ACL to trap 0x8863 in every VLAN
	bzero(&rule, sizeof(_rtl8651_tblDrvAclRule_t));
	rule.ruleType_              = RTL8651_ACL_MAC;
	rule.actionType_            = RTL8651_ACL_CPU;
	rule.un_ty.MAC._typeLen     = 0x8863;
	rule.un_ty.MAC._typeLenMask = 0xffff;

	retval = rtl8651_setAsicAclRule(aclStart, &rule);
	if (retval == SUCCESS)
	{
		aclStart++;
		count++;
	}

	if ( aclStart > aclEnd || aclEnd>=RTL8651_ACLTBL_SIZE )	
		return count;

	// add ACL to trap 0x8864 in every VLAN
	bzero(&rule, sizeof(_rtl8651_tblDrvAclRule_t));
	rule.ruleType_              = RTL8651_ACL_MAC;
	rule.actionType_            = RTL8651_ACL_CPU;
	rule.un_ty.MAC._typeLen     = 0x8864;
	rule.un_ty.MAC._typeLenMask = 0xffff;

	retval = rtl8651_setAsicAclRule(aclStart, &rule);
	if (retval == SUCCESS)
	{
		aclStart++;
		count++;
	}

	return count;
}
#endif//CONFIG_RTL865XB



/*
@func int32 | rtl8651_EnableIpv6Passthru | This function is used to enable/disable PPPoE Passthru.
@parm int8 | enable | TRUE for enable, FALSE for disable
@rvalue int8 | previous enable states
@comm
*/
int8 rtl8651_EnableIpv6Passthru( int8 enable )
{
	int8 prevValue;
	int32 retval;

	//rtlglue_printf("%s:%d  %s\n", __FILE__, __LINE__, enable?"TRUE":"FALSE");
	rtlglue_drvMutexLock();
	
	prevValue = rtl8651_l2protoPassThrough&PASSTHRU_IPV6?TRUE:FALSE;

	if ( enable )
		rtl8651_l2protoPassThrough |= PASSTHRU_IPV6;
	else
		rtl8651_l2protoPassThrough &= ~PASSTHRU_IPV6;
#if 1//CONFIG_RTL865XB_EXP_IPV6_PASSTHRU

	// create or destroy VLAN and protocol-based VLAN ID
	if ( prevValue == TRUE && enable == FALSE )
	{
		// destory 
		retval = _rtl8651_destroyProtocolBasedVLANforIPv6Passthru();
	}
	else if ( prevValue == FALSE && enable == TRUE )
	{
		// create
		retval = _rtl8651_createProtocolBasedVLANforIPv6Passthru();
	}
	
#else//CONFIG_RTL865XB

	// re-arrange ACL
	_rtl8651_arrangeAllACL();
	
#endif//CONFIG_RTL865XB

	rtlglue_drvMutexUnlock();
	
	return prevValue;
}


/*
@func int32 | rtl8651_getIpv6PassthruVid | This function returns the VLAN id of IPv6 Passthru.
@rvalue int32 | -1 if IPv6 Passthru is disabled, otherwise 0-4095 means VLAN id.
@comm
*/
int32 rtl8651_getIpv6PassthruVid()
{
	int32 retval;
	//rtlglue_printf("%s:%d  %s\n", __FILE__, __LINE__, enable?"TRUE":"FALSE");
	rtlglue_drvMutexLock();
	
	if ( rtl8651_l2protoPassThrough & PASSTHRU_IPV6 )
		retval = rtl8651_gIPv6PassthruVlanID;
	else
		retval = FAILED;

	rtlglue_drvMutexUnlock();
	
	return retval;
}


#if 1//CONFIG_RTL865XB_EXP_IPV6_PASSTHRU


int32 _rtl8651_setExternalInterfaceforIPv6Passthru( ether_addr_t* gmac, uint16 ExtVid )
{

	// External IPv6 interface is being configured.
	rtl8651_gIPv6ExternalVlanIndex = rtl8651_vlanTableIndex( ExtVid );
	memcpy( &rtl8651_gIPv6ExternalGatewayMAC, gmac, sizeof(ether_addr_t) );

	return SUCCESS;
}

	
int32 rtl8651_getExternalInterfaceforIPv6Passthru()
{
	if ( rtl8651_l2protoPassThrough & PASSTHRU_IPV6 )
		return  rtl8651_gIPv6ExternalVlanIndex;
	else
		return FAILED;
}

	
int32 _rtl8651_createProtocolBasedVLANforIPv6Passthru(void)
{
	int i;
	uint32 retval;
	uint8 ProtocolType;
	uint16 ProtocolValue;
	
	//rtlglue_printf("_rtl8651_createProtocolBasedVLANforIPv6Passthru()...\n");

	if( rtl8651_gIPv6ExternalVlanIndex == RTL8651_VLAN_NUMBER )
		return FAILED; // external interface is not set
		
	if ( rtl8651_gIPv6PassthruVlanID != RTL8651_IPV6PASSTHRU_VLANID_IS_NOT_SET )
		return SUCCESS; // already added

	retval = _rtl8651_createVlanForPassthru( &rtl8651_gIPv6ExternalGatewayMAC, &rtl8651_gIPv6PassthruVlanID );
	assert( retval == SUCCESS );

	//
	//  Configure Protocol-Based VLAN Control Register
	//
	for( rtl8651_pbv_ipv6passthru = RTL8651_PBV_RULE_USR1;
	     rtl8651_pbv_ipv6passthru <= RTL8651_PBV_RULE_USR2;
	     rtl8651_pbv_ipv6passthru++ )
	{
		// find empty protocol-based VLAN control register
		rtl8651_queryProtocolBasedVLAN( rtl8651_pbv_ipv6passthru, &ProtocolType, &ProtocolValue );
		if ( ProtocolType == 0x0 && ProtocolValue == 0x0000 )
			break;
	}
	assert( rtl8651_pbv_ipv6passthru <= RTL8651_PBV_RULE_USR2 );

	// EthernetII, IPv6(0x86DD)
	rtl8651_defineProtocolBasedVLAN( rtl8651_pbv_ipv6passthru, 0x0, 0x86DD );

	//
	//  _rtl8651_createProtocolBasedVLANforIPv6Passthru() may have NOT be executed
	//    when the first time link status change.
	//    Thus, the protocol-based VLAN index is pointed to 0, which is WRONG.
	//  To solve this problem, we must update the VLAN index 
	//    when rtl8651_gPPPoEPassthruVlanID is updated.
	//
	for( i = 0; i < RTL8651_PORT_NUMBER+rtl8651_totalExtPortNum; i++ )
	{
		int8 linkup;
		
		rtl8651_getEthernetPortLinkStatus( i, &linkup, NULL, NULL, NULL );

		if ( linkup )
		{
			rtl8651_setEthernetPortLinkStatus( i, linkup );
		}
	}

	return SUCCESS;
}


int32 _rtl8651_destroyProtocolBasedVLANforIPv6Passthru(void)
{
	int32 ret, i;

	if ( rtl8651_gIPv6ExternalVlanIndex == RTL8651_VLAN_NUMBER )
		return FAILED; // external interface is not set
		
	if( rtl8651_gIPv6PassthruVlanID == RTL8651_IPV6PASSTHRU_VLANID_IS_NOT_SET )
	{
		//rtlglue_printf("rtl8651_gIPv6ExternalVlanIndex is NOT set, but _rtl8651_destroyProtocolBasedVLANforIPv6Passthru() is called.\n");
		return SUCCESS;
	}

	//
	// destroy VLAN
	//
	ret = _rtl8651_destroyVlanForPassthru( rtl8651_gIPv6ExternalGatewayMAC, rtl8651_gIPv6PassthruVlanID );
	assert( ret == SUCCESS );
	
	//
	//  Clear Protocol-Based VLAN Control Register
	//
	WRITE_MEM32( PBVCR+(rtl8651_pbv_ipv6passthru-5)*4, 0x0 << PBVCR_PROTO_TYPE_OFFSET | 0x0000 << PBVCR_PROTO_VALUE_OFFSET );

	//
	//	If the link is up, we must clear the protocol based VLAN ID
	//
	for( i = 0; i < RTL8651_PORT_NUMBER+rtl8651_totalExtPortNum; i++ )
	{
		int8 linkup;
		
		rtl8651_getEthernetPortLinkStatus( i, &linkup, NULL, NULL, NULL );

		if ( linkup )
		{
			uint8 valid, vlanIdx;

			// verify
			rtl8651_getProtocolBasedVLAN( rtl8651_pbv_ipv6passthru, i, &valid, &vlanIdx );
			assert( valid && vlanIdx == rtl8651_vlanTableIndex( rtl8651_gIPv6PassthruVlanID ) );

			// clean it
			rtl8651_setProtocolBasedVLAN( rtl8651_pbv_ipv6passthru, i, 0, 0 );
		}
	}

	rtl8651_gIPv6PassthruVlanID = RTL8651_IPV6PASSTHRU_VLANID_IS_NOT_SET;
	rtl8651_pbv_ipv6passthru = RTL8651_IPV6PASSTHRU_PBVID_IS_NOT_SET;
	
	return SUCCESS;
}


#else//CONFIG_RTL865XB

uint32 _rtl8651_acl_IPv6_Passthru( uint32 aclStart, uint32 aclEnd, rtl8651_tblDrv_networkIntfTable_t *netIfPtr )
{
	_rtl8651_tblDrvAclRule_t rule;
	int32 retval;
	uint32 count=0;

	//printk("\n\n***** _rtl8651_acl_PPPoE_Passthru(aclStart:%d,aclEnd:%d)\n\n",aclStart,aclEnd);

	if ( aclStart > aclEnd || aclEnd>=RTL8651_ACLTBL_SIZE )	
		return count;

	// add ACL to trap 0x86DD in internal VLAN
	bzero(&rule, sizeof(_rtl8651_tblDrvAclRule_t));
	rule.ruleType_              = RTL8651_ACL_MAC;
	rule.actionType_            = RTL8651_ACL_CPU;
	rule.un_ty.MAC._typeLen     = 0x86DD;
	rule.un_ty.MAC._typeLenMask = 0xffff;

	retval = rtl8651_setAsicAclRule(aclStart, &rule);
	if (retval == SUCCESS)
	{
		aclStart++;
		count++;
	}

	return count;
}


#endif//CONFIG_RTL865XB


/*
@func int32 | rtl8651_EnableIpxPassthru | This function is used to enable/disable IPX Passthru.
@parm int8 | enable | TRUE for enable, FALSE for disable
@rvalue int8 | previous enable states
@comm
*/
int8 rtl8651_EnableIpxPassthru( int8 enable )
{
	int8 prevValue;
	int32 retval;

	rtlglue_drvMutexLock();
	
	prevValue = rtl8651_l2protoPassThrough & PASSTHRU_IPX?TRUE:FALSE;

	if ( enable )
		rtl8651_l2protoPassThrough |= PASSTHRU_IPX;
	else
		rtl8651_l2protoPassThrough &= ~PASSTHRU_IPX;
	
#if 1//CONFIG_RTL865XB_EXP_IPX_PASSTHRU

	// create or destroy VLAN and protocol-based VLAN ID
	if ( prevValue == TRUE && enable == FALSE )
	{
		// destory 
		retval = _rtl8651_destroyProtocolBasedVLANforIPXPassthru();
	}
	else if ( prevValue == FALSE && enable == TRUE )
	{
		// create
		retval = _rtl8651_createProtocolBasedVLANforIPXPassthru();
	}
	
#endif//CONFIG_RTL865XB

	rtlglue_drvMutexUnlock();
	
	return prevValue;
}


/*
@func int32 | rtl8651_getIpxPassthruVid | This function returns the VLAN id of IPX Passthru.
@rvalue int32 | -1 if IPX Passthru is disabled, otherwise 0-4095 means VLAN id.
@comm
*/
int32 rtl8651_getIpxPassthruVid()
{
	int32 retval;
	//rtlglue_printf("%s:%d  %s\n", __FILE__, __LINE__, enable?"TRUE":"FALSE");
	rtlglue_drvMutexLock();
	
	if ( rtl8651_l2protoPassThrough & PASSTHRU_IPX )
		retval = rtl8651_gIPXPassthruVlanID;
	else
		retval = FAILED;

	rtlglue_drvMutexUnlock();
	
	return retval;
}


#if 1//CONFIG_RTL865XB_EXP_IPX_PASSTHRU


int32 _rtl8651_setExternalInterfaceforIPXPassthru( ether_addr_t* gmac, uint16 ExtVid )
{

	// External IPX interface is being configured.
	rtl8651_gIPXExternalVlanIndex = rtl8651_vlanTableIndex( ExtVid );
	memcpy( &rtl8651_gIPXExternalGatewayMAC, gmac, sizeof(ether_addr_t) );

	return SUCCESS;
}

	
int32 rtl8651_getExternalInterfaceforIPXPassthru()
{
	if ( rtl8651_l2protoPassThrough & PASSTHRU_IPX )
		return  rtl8651_gIPXExternalVlanIndex;
	else
		return FAILED;
}

	
int32 _rtl8651_createProtocolBasedVLANforIPXPassthru(void)
{
	int i;
	uint32 retval;
	
	/*rtlglue_printf("_rtl8651_createProtocolBasedVLANforIPXPassthru()... ExtVlan=%d PassthruVlan=%d \n",rtl8651_gIPXExternalVlanIndex,rtl8651_gIPXPassthruVlanID);*/

	if( rtl8651_gIPXExternalVlanIndex == RTL8651_VLAN_NUMBER )
		return FAILED; // external interface is not set
		
	if ( rtl8651_gIPXPassthruVlanID != RTL8651_IPXPASSTHRU_VLANID_IS_NOT_SET )
		return SUCCESS; // already added

	retval = _rtl8651_createVlanForPassthru( &rtl8651_gIPXExternalGatewayMAC, &rtl8651_gIPXPassthruVlanID );
	assert( retval == SUCCESS );

	//
	//  _rtl8651_createProtocolBasedVLANforIPXPassthru() may have NOT be executed
	//    when the first time link status change.
	//    Thus, the protocol-based VLAN index is pointed to 0, which is WRONG.
	//  To solve this problem, we must update the VLAN index 
	//    when rtl8651_gIPXPassthruVlanID is updated.
	//
	for( i = 0; i < RTL8651_PORT_NUMBER+rtl8651_totalExtPortNum; i++ )
	{
		int8 linkup;
		
		rtl8651_getEthernetPortLinkStatus( i, &linkup, NULL, NULL, NULL );

		if ( linkup )
		{
			rtl8651_setEthernetPortLinkStatus( i, linkup );
		}
	}

	return SUCCESS;
}


int32 _rtl8651_destroyProtocolBasedVLANforIPXPassthru(void)
{
	int32 ret, i;

	if ( rtl8651_gIPXExternalVlanIndex == RTL8651_VLAN_NUMBER )
		return FAILED; // external interface is not set
		
	if( rtl8651_gIPXPassthruVlanID == RTL8651_IPXPASSTHRU_VLANID_IS_NOT_SET )
	{
		//rtlglue_printf("rtl8651_gIPXExternalVlanIndex is NOT set, but _rtl8651_destroyProtocolBasedVLANforIPXPassthru() is called.\n");
		return SUCCESS;
	}

	//
	// destroy VLAN
	//
	ret = _rtl8651_destroyVlanForPassthru( rtl8651_gIPXExternalGatewayMAC, rtl8651_gIPXPassthruVlanID );
	assert( ret == SUCCESS );
	
	//
	//	If the link is up, we must clear the protocol based VLAN ID
	//
	for( i = 0; i < RTL8651_PORT_NUMBER+rtl8651_totalExtPortNum; i++ )
	{
		int8 linkup;
		
		rtl8651_getEthernetPortLinkStatus( i, &linkup, NULL, NULL, NULL );

		if ( linkup )
		{
			uint8 valid, vlanIdx;

			// verify
			rtl8651_getProtocolBasedVLAN( rtl8651_pbv_ipxpassthru, i, &valid, &vlanIdx );
			assert( valid && vlanIdx == rtl8651_vlanTableIndex( rtl8651_gIPXPassthruVlanID ) );

			// clean it
			rtl8651_setProtocolBasedVLAN( rtl8651_pbv_ipxpassthru, i, 0, 0 );
		}
	}

	rtl8651_gIPXPassthruVlanID = RTL8651_IPXPASSTHRU_VLANID_IS_NOT_SET;
	
	return SUCCESS;
}


#else//CONFIG_RTL865XB
#endif//CONFIG_RTL865XB


/*
@func int32 | rtl8651_EnableNetbiosPassthru | This function is used to enable/disable Netbios Passthru.
@parm int8 | enable | TRUE for enable, FALSE for disable
@rvalue int8 | previous enable states
@comm
*/
int8 rtl8651_EnableNetbiosPassthru( int8 enable )
{
	int8 prevValue;
	int32 retval;

	rtlglue_drvMutexLock();
	
	prevValue = rtl8651_l2protoPassThrough&PASSTHRU_NETBIOS?TRUE:FALSE;

	if ( enable )
		rtl8651_l2protoPassThrough |= PASSTHRU_NETBIOS;
	else
		rtl8651_l2protoPassThrough &= ~PASSTHRU_NETBIOS;
	
#if 1//CONFIG_RTL865XB_EXP_NETBIOS_PASSTHRU

	// create or destroy VLAN and protocol-based VLAN ID
	if ( prevValue == TRUE && enable == FALSE )
	{
		// destory 
		retval = _rtl8651_destroyProtocolBasedVLANforNetbiosPassthru();
	}
	else if ( prevValue == FALSE && enable == TRUE )
	{
		// create
		retval = _rtl8651_createProtocolBasedVLANforNetbiosPassthru();
	}
	
#endif//CONFIG_RTL865XB

	rtlglue_drvMutexUnlock();
	
	return prevValue;
}


/*
@func int32 | rtl8651_getNetbiosPassthruVid | This function returns the VLAN id of Netbios Passthru.
@rvalue int32 | -1 if Netbios Passthru is disabled, otherwise 0-4095 means VLAN id.
@comm
*/
int32 rtl8651_getNetbiosPassthruVid()
{
	int32 retval;
	//rtlglue_printf("%s:%d  %s\n", __FILE__, __LINE__, enable?"TRUE":"FALSE");
	rtlglue_drvMutexLock();
	
	if ( rtl8651_l2protoPassThrough & PASSTHRU_NETBIOS )
		retval = rtl8651_gNetbiosPassthruVlanID;
	else
		retval = FAILED;

	rtlglue_drvMutexUnlock();
	
	return retval;
}


#if 1//CONFIG_RTL865XB_EXP_NETBIOS_PASSTHRU


int32 _rtl8651_setExternalInterfaceforNetbiosPassthru( ether_addr_t* gmac, uint16 ExtVid )
{

	// External Netbios interface is being configured.
	rtl8651_gNetbiosExternalVlanIndex = rtl8651_vlanTableIndex( ExtVid );
	memcpy( &rtl8651_gNetbiosExternalGatewayMAC, gmac, sizeof(ether_addr_t) );

	return SUCCESS;
}

	
int32 rtl8651_getExternalInterfaceforNetbiosPassthru()
{
	if ( rtl8651_l2protoPassThrough & PASSTHRU_NETBIOS )
		return  rtl8651_gNetbiosExternalVlanIndex;
	else
		return FAILED;
}

	
int32 _rtl8651_createProtocolBasedVLANforNetbiosPassthru(void)
{
	int i;
	uint32 retval;
	
	//rtlglue_printf("_rtl8651_createProtocolBasedVLANforNetbiosPassthru()...\n");

	if( rtl8651_gNetbiosExternalVlanIndex == RTL8651_VLAN_NUMBER )
		return FAILED; // external interface is not set
		
	if ( rtl8651_gNetbiosPassthruVlanID != RTL8651_NETBIOSPASSTHRU_VLANID_IS_NOT_SET )
		return SUCCESS; // already added

	retval = _rtl8651_createVlanForPassthru( &rtl8651_gNetbiosExternalGatewayMAC, &rtl8651_gNetbiosPassthruVlanID );
	assert( retval == SUCCESS );

	//
	//  _rtl8651_createProtocolBasedVLANforNetbiosPassthru() may have NOT be executed
	//    when the first time link status change.
	//    Thus, the protocol-based VLAN index is pointed to 0, which is WRONG.
	//  To solve this problem, we must update the VLAN index 
	//    when rtl8651_gNetbiosPassthruVlanID is updated.
	//
	for( i = 0; i < RTL8651_PORT_NUMBER+rtl8651_totalExtPortNum; i++ )
	{
		int8 linkup;
		
		rtl8651_getEthernetPortLinkStatus( i, &linkup, NULL, NULL, NULL );

		if ( linkup )
		{
			rtl8651_setEthernetPortLinkStatus( i, linkup );
		}
	}

	return SUCCESS;
}


int32 _rtl8651_destroyProtocolBasedVLANforNetbiosPassthru(void)
{
	int32 ret, i;

	if ( rtl8651_gNetbiosExternalVlanIndex == RTL8651_VLAN_NUMBER )
		return FAILED; // external interface is not set
		
	if( rtl8651_gNetbiosPassthruVlanID == RTL8651_NETBIOSPASSTHRU_VLANID_IS_NOT_SET )
	{
		//rtlglue_printf("rtl8651_gNetbiosExternalVlanIndex is NOT set, but _rtl8651_destroyProtocolBasedVLANforINetbiosXPassthru() is called.\n");
		return SUCCESS;
	}

	//
	// destroy VLAN
	//
	ret = _rtl8651_destroyVlanForPassthru( rtl8651_gNetbiosExternalGatewayMAC, rtl8651_gNetbiosPassthruVlanID );
	assert( ret == SUCCESS );
	
	//
	//	If the link is up, we must clear the protocol based VLAN ID
	//
	for( i = 0; i < RTL8651_PORT_NUMBER+rtl8651_totalExtPortNum; i++ )
	{
		int8 linkup;
		
		rtl8651_getEthernetPortLinkStatus( i, &linkup, NULL, NULL, NULL );

		if ( linkup )
		{
			uint8 valid, vlanIdx;

			// verify
			rtl8651_getProtocolBasedVLAN( rtl8651_pbv_netbiospassthru, i, &valid, &vlanIdx );
			assert( valid && vlanIdx == rtl8651_vlanTableIndex( rtl8651_gNetbiosPassthruVlanID ) );

			// clean it
			rtl8651_setProtocolBasedVLAN( rtl8651_pbv_netbiospassthru, i, 0, 0 );
		}
	}

	rtl8651_gNetbiosPassthruVlanID = RTL8651_NETBIOSPASSTHRU_VLANID_IS_NOT_SET;
	
	return SUCCESS;
}


#else//CONFIG_RTL865XB
#endif//CONFIG_RTL865XB



/*
@func uint8 | rtl8651_EnableProtocolBasedNAT | Enable or disable protocol based NAT
@parm int32 | enable | TRUE to enable. FALSE to disable.
@rvalue int32 | status of original setting
@comm
*/
uint8 rtl8651_EnableProtocolBasedNAT( uint8 enable )
{
	uint8 prevValue;

	rtlglue_drvMutexLock();
	prevValue = rtl8651_drvProtocolBasedNatEnable;
	rtl8651_drvProtocolBasedNatEnable = enable;
	rtlglue_drvMutexUnlock();

	return prevValue;
}


/*
@func int32 | rtl8651_addProtocolBasedNAT | add entry for protocol based NAT
@parm uint8 | protocol | IP protocol
@parm ipaddr_t | ExtIp | External IP
@parm ipaddr_t | IntIp | Internal IP
@rvalue SUCCESS |
@comm
*/
int32 _rtl8651_addProtocolBasedNAT( uint8 protocol, ipaddr_t ExtIp, ipaddr_t IntIp )
{
	rtl8651_tblDrv_protocolBasedNat_t *pbNat;

	/* The following protocol is NOT supported */
	switch( protocol )
	{
		case IPPROTO_ICMP:
		case IPPROTO_IGMP:
		case IPPROTO_TCP:
		case IPPROTO_UDP:
		case IPPROTO_GRE:
			return TBLDRV_EPBNAT_PROTOCOL_NOT_SUPPORTED;
	}
	
	// check if duplicate
	for( pbNat = SLIST_FIRST(&DrvTbl.inuseList.protocolBasedNat);
	     pbNat != NULL;
	     pbNat = SLIST_NEXT(pbNat,next) )
	{
		if ( pbNat->protocol == protocol &&
		     pbNat->ExtIp == ExtIp  )
		{
			// found
			break;
		}
	}

	if ( pbNat != NULL )
	{
		return TBLDRV_EPBNAT_ENTRY_EXIST;
	}
	else
	{
		// client/server MAC pair is not found, allocate new one.
		pbNat = SLIST_FIRST(&DrvTbl.freeList.protocolBasedNat);

		/* no free session available */
		if ( pbNat == NULL )
			return TBLDRV_EPBNAT_NO_FREE_ENTRY;

		pbNat->protocol = protocol;
		pbNat->ExtIp = ExtIp;
		pbNat->IntIp = IntIp;

		// link to inuse list
		SLIST_REMOVE_HEAD(&DrvTbl.freeList.protocolBasedNat, next);
		SLIST_INSERT_HEAD(&DrvTbl.inuseList.protocolBasedNat, pbNat, next);
		
		//printk("rtl8651_addPPPoEPassthru(): insert new\n");
	}

	return SUCCESS;
}


int32 rtl8651_addProtocolBasedNAT( uint8 protocol, ipaddr_t ExtIp, ipaddr_t IntIp )
{
	int32 retval;

	rtlglue_drvMutexLock();
	retval = _rtl8651_addProtocolBasedNAT( protocol, ExtIp, IntIp );
	rtlglue_drvMutexUnlock();

	return retval;
}


/*
@func int32 | rtl8651_delProtocolBasedNAT | delete entry for protocol based NAT
@parm uint8 | protocol | IP protocol
@parm ipaddr_t | ExtIp | External IP
@parm ipaddr_t | IntIp | Internal IP
@rvalue SUCCESS |
@comm
*/
int32 _rtl8651_delProtocolBasedNAT( uint8 protocol, ipaddr_t ExtIp, ipaddr_t IntIp )
{
	rtl8651_tblDrv_protocolBasedNat_t *pbNat;
	
	// check if duplicate
	for( pbNat = SLIST_FIRST(&DrvTbl.inuseList.protocolBasedNat);
	     pbNat != NULL;
	     pbNat = SLIST_NEXT(pbNat,next) )
	{
		if ( pbNat->protocol == protocol &&
		     pbNat->ExtIp == ExtIp &&
		     pbNat->IntIp == IntIp )
		{
			// found
			break;
		}
	}

	if ( pbNat == NULL )
	{
		return TBLDRV_EPBNAT_ENTRY_NOT_FOUND;
	}

	memset( pbNat, 0, offsetof(struct rtl8651_tblDrv_protocolBasedNat_s,next) );
	
	// link to free list
	SLIST_REMOVE( &DrvTbl.inuseList.protocolBasedNat, pbNat, rtl8651_tblDrv_protocolBasedNat_s, next );
	SLIST_INSERT_HEAD( &DrvTbl.freeList.protocolBasedNat, pbNat, next );

	return SUCCESS;
}


int32 rtl8651_delProtocolBasedNAT( uint8 protocol, ipaddr_t ExtIp, ipaddr_t IntIp )
{
	int32 retval;

	rtlglue_drvMutexLock();
	retval = _rtl8651_delProtocolBasedNAT( protocol, ExtIp, IntIp );
	rtlglue_drvMutexUnlock();

	return retval;
}


/*
@func int32 | rtl8651_getInboundProtocolBasedNAT | get entry for protocol based NAT
@parm uint8 | protocol | IP protocol
@parm ipaddr_t | ExtIp | External IP
@parm ipaddr_t* | IntIp | Pointer to internal IP
@rvalue SUCCESS |
@comm
*/
int32 _rtl8651_getInboundProtocolBasedNAT( uint8 protocol, ipaddr_t ExtIp, ipaddr_t *IntIp )
{
	rtl8651_tblDrv_protocolBasedNat_t *pbNat;
	
	// check if duplicate
	for( pbNat = SLIST_FIRST(&DrvTbl.inuseList.protocolBasedNat);
	     pbNat != NULL;
	     pbNat = SLIST_NEXT(pbNat,next) )
	{
		if ( pbNat->protocol == protocol &&
		     pbNat->ExtIp == ExtIp )
		{
			// found
			break;
		}
	}

	if ( pbNat == NULL )
	{
		return TBLDRV_EPBNAT_ENTRY_NOT_FOUND;
	}

	if ( IntIp ) *IntIp = pbNat->IntIp;
	
	return SUCCESS;
}


/*
@func int32 | rtl8651_getOutboundProtocolBasedNAT | get entry for protocol based NAT
@parm uint8 | protocol | IP protocol
@parm ipaddr_t* | ExtIp | Pointer to external IP
@parm ipaddr_t | IntIp | Internal IP
@rvalue SUCCESS |
@comm
*/
int32 _rtl8651_getOutboundProtocolBasedNAT( uint8 protocol, ipaddr_t *ExtIp, ipaddr_t IntIp )
{
	rtl8651_tblDrv_protocolBasedNat_t *pbNat;
	
	// check if duplicate
	for( pbNat = SLIST_FIRST(&DrvTbl.inuseList.protocolBasedNat);
	     pbNat != NULL;
	     pbNat = SLIST_NEXT(pbNat,next) )
	{
		if ( pbNat->protocol == protocol &&
		     pbNat->IntIp == IntIp )
		{
			// found
			break;
		}
	}

	if ( pbNat == NULL )
	{
		return TBLDRV_EPBNAT_ENTRY_NOT_FOUND;
	}

	if ( ExtIp ) *ExtIp = pbNat->ExtIp;
	
	return SUCCESS;
}





/*=====================================================
  * Configure Auto Cross-Over/Straight-through detection
  *=====================================================*/
#define RTL865X_MDI_MDIX_DETECTION_APIs


/*
@func int32 | rtl8651_autoCrossOverDetect | Auto Cross-Over/Straight-Through detection.
@parm uint32 | isEnable | TRUE: Auto detection, FALSE: Non auto detection.
@rvalue SUCCESS | Successful configure.
@rvalue FAILED | Fail to configure.
@comm
This API sets all port to have auto cross-over/straight-through detection.
*/
int32 rtl8651_autoCrossOverDetect(uint32 isEnable)
{	
	uint32 port;
	int32 retval;
	for(port=0; port<RTL8651_PORT_NUMBER-1; port++) {
		retval = rtl8651_autoMdiMdix(port, isEnable);
		if (retval != SUCCESS)
			return FAILED;
	}
	return SUCCESS;
}

/*
@func int32 | rtl8651_portAutoCrossOverDetect | Auto Cross-Over/Straight-Through detection.
@parm uint32 | isEnable | TRUE: Auto detection, FALSE: Non auto detection.
@rvalue SUCCESS | Successful configure.
@rvalue FAILED | Fail to configure.
@comm
This API sets a specified port to have auto cross-over/straight-through detection.
*/
int32 rtl8651_portAutoCrossOverDetect(uint32 port, uint32 isEnable)
{
	return rtl8651_autoMdiMdix(port, isEnable);
}


int32 rtl8651_setMdiMdix(uint32 isMdi)
{
	uint32 port;
	int32 retval;
	for(port=0; port<RTL8651_PORT_NUMBER-1; port++) {
		retval = rtl8651_selectMdiMdix(port, isMdi);
		if (retval != SUCCESS)
			return FAILED;
	}
	return SUCCESS;
}


int32 rtl8651_setPortMdiMdix(uint32 port, uint32 isMdi)
{
	return rtl8651_selectMdiMdix(port, isMdi);
}


#ifdef RTL865X_TEST
/*=====================================================
  * For Test Code only
  *====================================================*/
int32 rtl8651_getDrvMutex(void)
{
	return rtlglue_getDrvMutex();
}
int32 rtl8651_reinitDrvMutex()
{
	return rtlglue_reinitDrvMutex();
}
int32 rtl8651_getMbufMutex(void)
{
	return rtlglue_getMbufMutex();
}
int32 rtl8651_reinitMbufMutex()
{
	return rtlglue_reinitMbufMutex();
}
#endif /* RTL865X_TEST */


#ifdef RTL865XB_ACLDB

rtl8651_ACLDB_t acldb[RTL8651_ACLTBL_SIZE];

/*
 *  Called by _rtl8651_arrangeAllACL().
 */
int32 _rtl8651_initACLDB(void)
{
	int i;

	for( i = 0; i < RTL8651_ACLTBL_SIZE; i++ )
	{
		acldb[i].func = NULL;
		acldb[i].userDefined = NULL;
	}

	return SUCCESS;
}

/*
 *  Called by ACL-related function, ex: _rtl8651_tryToAddAclRuleForIpsec()
 */
int32 _rtl8651_registerACLDB( int32 ruleNo, 
                             acldb_callback_t func,
                             void *userDefined )
{
//	printk("***  registerACLDB --- %d %p %p\n", ruleNo, func, userDefined);

	acldb[ruleNo].func = func;
	acldb[ruleNo].userDefined = userDefined;

	return SUCCESS;
}

/*
 *  Called by FwdEng (fast path if reason bits carry ACL rule)
 */
inline int32 _rtl8651_jumptoACLDB( int32 ruleNo, struct rtl_pktHdr *pkthdr, struct ip *pip )
{
	int32 retval = FAILED;

//	printk("jumptoAclDB --- %d %p\n", ruleNo, acldb[ruleNo].func);

	if ( acldb[ruleNo].func )
	{
		acldb[ruleNo].count++;
		retval = (*acldb[ruleNo].func)( ruleNo, pkthdr, pip, acldb[ruleNo].userDefined );
	}

	return retval;
}

#endif/*RTL865XB_ACLDB*/

#ifdef DROP_NEW_CONNECTION_WHEN_NAT_FULL
int32 rtl8651_configDropingNewConnWhenNatFull(int32 enable)
{
	if(enable==TRUE)
	{
		rtl8651_dropNewConnWhenNatFull=TRUE;
	}
	else
	{
		rtl8651_dropNewConnWhenNatFull=FALSE;
	}
	return SUCCESS;
}
#endif


